//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cocalc.h
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/6/96     M.Perry    derived from Dave's code.
//  2.0    09/17/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__cocalc_h)
#define __cocalc_h

#include "cstcalc.h"

#if defined(BJ_COMPILE)
class TStatCalcBJ;
#endif
#if defined(CRAPS_COMPILE)
class TStatCalcCraps;
#endif
#if defined(KENO_COMPILE)
class TStatCalcKeno;
#endif
#if defined(ROULETTE_COMPILE)
class TStatCalcRoulette;
#endif
#if defined(SLOTS_COMPILE)
class TStatCalcSlots;
#endif
#if defined(VP_COMPILE)
class TStatCalcVP;
#endif
//*******************************************************************
class TStatCalcOverall:public TCasinoStatCalc
{
public:
  enum
  {
    oTotalCashBet = 1,
    oAvgCashBet,
    oStartingBankroll,
    oEndingBankroll,
    oTotalHoursPlayed,
    oTotalCashWL,
    oAvgWLPerHour,
    oCumCashWL,
    oPeakCashWinLevel,
    oLowCashLossLevel,
    oblank1,
    oSESSIONS,
    oSESSIONSRECORDED,
    #if defined(BJ_COMPILE)
    oBJSessions,
    #endif
    #if defined(CRAPS_COMPILE)
    oCrapsSessions,
    #endif
    #if defined(KENO_COMPILE)
    oKenoSessions,
    #endif
    #if defined(ROULETTE_COMPILE)
    oRouletteSessions,
    #endif
    #if defined(SLOTS_COMPILE)
    oSlotsSessions,
    #endif
    #if defined(VP_COMPILE)
    oVPSessions,
    #endif
    oblank2,
    oTOTALCASHBET,
    #if defined(BJ_COMPILE)
    oBJCash,
    #endif
    #if defined(CRAPS_COMPILE)
    oCrapsCash,
    #endif
    #if defined(KENO_COMPILE)
    oKenoCash,
    #endif
    #if defined(ROULETTE_COMPILE)
    oRouletteCash,
    #endif
    #if defined(SLOTS_COMPILE)
    oSlotsCash,
    #endif
    #if defined(VP_COMPILE)
    oVPCash,
    #endif
    oblank3,
    oTOTALHOURSPLAYED,
    #if defined(BJ_COMPILE)
    oBJHoursPlayed,
    #endif
    #if defined(CRAPS_COMPILE)
    oCrapsHoursPlayed,
    #endif
    #if defined(KENO_COMPILE)
    oKenoHoursPlayed,
    #endif
    #if defined(ROULETTE_COMPILE)
    oRouletteHoursPlayed,
    #endif
    #if defined(SLOTS_COMPILE)
    oSlotsHoursPlayed,
    #endif
    #if defined(VP_COMPILE)
    oVPHoursPlayed,
    #endif
    oblank4,
    oTOTALCASHWL,
    #if defined(BJ_COMPILE)
    oBJCashWL,
    #endif
    #if defined(CRAPS_COMPILE)
    oCrapsCashWL,
    #endif
    #if defined(KENO_COMPILE)
    oKenoCashWL,
    #endif
    #if defined(ROULETTE_COMPILE)
    oRouletteCashWL,
    #endif
    #if defined(SLOTS_COMPILE)
    oSlotsCashWL,
    #endif
    #if defined(VP_COMPILE)
    oVPCashWL,
    #endif
    oblank5,
    oRETURNPERC,
    #if defined(BJ_COMPILE)
    oBJPerc,
    #endif
    #if defined(CRAPS_COMPILE)
    oCrapsPerc,
    #endif
    #if defined(KENO_COMPILE)
    oKenoPerc,
    #endif
    #if defined(ROULETTE_COMPILE)
    oRoulettePerc,
    #endif
    #if defined(SLOTS_COMPILE)
    oSlotsPerc,
    #endif
    #if defined(VP_COMPILE)
    oVPPerc,
    #endif
    oblank6,
    oWLPERHOUR,
    #if defined(BJ_COMPILE)
    oBJCashWLPerHour,
    #endif
    #if defined(CRAPS_COMPILE)
    oCrapsWLPerHour,
    #endif
    #if defined(KENO_COMPILE)
    oKenoWLPerHour,
    #endif
    #if defined(ROULETTE_COMPILE)
    oRouletteWLPerHour,
    #endif
    #if defined(SLOTS_COMPILE)
    oSlotsWLPerHour,
    #endif
    #if defined(VP_COMPILE)
    oVPWLPerHour,
    #endif
    oNumRows
  };
private:
#if defined(BJ_COMPILE)
  TStatCalcBJ *PBJStats;
#endif
#if defined(CRAPS_COMPILE)
  TStatCalcCraps *PCrapsStats;
#endif
#if defined(KENO_COMPILE)
  TStatCalcKeno *PKenoStats;
#endif
#if defined(ROULETTE_COMPILE)
  TStatCalcRoulette *PRouletteStats;
#endif
#if defined(SLOTS_COMPILE)
  TStatCalcSlots *PSlotsStats;
#endif
#if defined(VP_COMPILE)
  TStatCalcVP *PVPStats;
#endif

protected:
  int16 LabelResIds[oNumRows];
  string LabelStrs[oNumRows];

public:
  string GetLabelStr(int16 row_ndx);
  TStatCalcOverall(TCasinoStatDlg *pstatdlg);
  virtual ~TStatCalcOverall();
  virtual void LocalFillGrid();
  virtual void LocalCalculate();
  virtual void LocalMakeAllStrings();
  virtual void GetSessionPtrs();

protected:
  virtual void Make_TotalCashBetsStrs();
  virtual void Calc_TotalCashBets();

  virtual void Make_TotalNumBetsStrs();
  virtual void Calc_TotalNumBets();

  virtual void Make_AvgCashBetsStrs();
  virtual void Calc_AvgCashBets();

  virtual void Make_StartingBankrollStrs();
  virtual void Calc_StartingBankroll();

  virtual void Make_EndingBankrollStrs();
  virtual void Calc_EndingBankroll();

  virtual void Make_HoursPlayedStrs();
  virtual void Calc_HoursPlayed();

  virtual void Make_TotalCashWLStrs();
  virtual void Calc_TotalCashWL();

  virtual void Make_AvgWLPerHourStrs();
  virtual void Calc_AvgWLPerHour();

  virtual void Make_PeakCashWinLevelStrs();
  virtual void Calc_PeakCashWinLevel();

  virtual void Make_LowCashLostLevelStrs();
  virtual void Calc_LowCashLostLevel();

#if defined(BJ_COMPILE)
  void Calc_NumBJSessions();
  void Make_NumBJSessionsStr();
  string NumBJSessionsStr;
  long NumBJSessions;
#endif

#if defined(CRAPS_COMPILE)
  void Calc_NumCrapsSessions();
  void Make_NumCrapsSessionsStr();
  string NumCrapsSessionsStr;
  long NumCrapsSessions;
#endif

#if defined(KENO_COMPILE)
  void Calc_NumKenoSessions();
  void Make_NumKenoSessionsStr();
  string NumKenoSessionsStr;
  long NumKenoSessions;
#endif

#if defined(ROULETTE_COMPILE)
  void Calc_NumRouletteSessions();
  void Make_NumRouletteSessionsStr();
  string NumRouletteSessionsStr;
  long NumRouletteSessions;
#endif

#if defined(SLOTS_COMPILE)
  void Calc_NumSlotsSessions();
  void Make_NumSlotsSessionsStr();
  string NumSlotsSessionsStr;
  long NumSlotsSessions;
#endif

#if defined(VP_COMPILE)
  void Calc_NumVPSessions();
  void Make_NumVPSessionsStr();
  string NumVPSessionsStr;
  long NumVPSessions;
#endif

  void Calc_NumAllSessions();
  void Make_NumAllSessionsStr();
  string NumAllSessionsStr;
  long NumAllSessions;

#if defined(BJ_COMPILE)
  void Calc_BJTotalCashBets();
  void Make_BJTotalCashBetsStr();
  string BJTotalCashBetsStr;
  long BJTotalCashBets;
#endif

#if defined(CRAPS_COMPILE)
  void Calc_CrapsTotalCashBets();
  void Make_CrapsTotalCashBetsStr();
  string CrapsTotalCashBetsStr;
  long CrapsTotalCashBets;
#endif

#if defined(KENO_COMPILE)
  void Calc_KenoTotalCashBets();
  void Make_KenoTotalCashBetsStr();
  string KenoTotalCashBetsStr;
  long KenoTotalCashBets;
#endif

#if defined(ROULETTE_COMPILE)
  void Calc_RouletteTotalCashBets();
  void Make_RouletteTotalCashBetsStr();
  string RouletteTotalCashBetsStr;
  long RouletteTotalCashBets;
#endif

#if defined(SLOTS_COMPILE)
  void Calc_SlotsTotalCashBets();
  void Make_SlotsTotalCashBetsStr();
  string SlotsTotalCashBetsStr;
  long SlotsTotalCashBets;
#endif

#if defined(VP_COMPILE)
  void Calc_VPTotalCashBets();
  void Make_VPTotalCashBetsStr();
  string VPTotalCashBetsStr;
  long VPTotalCashBets;
#endif

#if defined(BJ_COMPILE)
  void Calc_BJHoursPlayed();
  void Make_BJHoursPlayedStr();
  string BJHoursPlayedStr;
  DWORD BJSecondsPlayed;
#endif

#if defined(CRAPS_COMPILE)
  void Calc_CrapsHoursPlayed();
  void Make_CrapsHoursPlayedStr();
  string CrapsHoursPlayedStr;
  DWORD CrapsSecondsPlayed;
#endif

#if defined(KENO_COMPILE)
  void Calc_KenoHoursPlayed();
  void Make_KenoHoursPlayedStr();
  string KenoHoursPlayedStr;
  DWORD KenoSecondsPlayed;
#endif

#if defined(ROULETTE_COMPILE)
  void Calc_RouletteHoursPlayed();
  void Make_RouletteHoursPlayedStr();
  string RouletteHoursPlayedStr;
  DWORD RouletteSecondsPlayed;
#endif

#if defined(SLOTS_COMPILE)
  void Calc_SlotsHoursPlayed();
  void Make_SlotsHoursPlayedStr();
  string SlotsHoursPlayedStr;
  DWORD SlotsSecondsPlayed;
#endif

#if defined(VP_COMPILE)
  void Calc_VPHoursPlayed();
  void Make_VPHoursPlayedStr();
  string VPHoursPlayedStr;
  DWORD VPSecondsPlayed;
#endif

#if defined(BJ_COMPILE)
  void Calc_BJCashWL();
  void Make_BJCashWLStr();
  string BJCashWLStr;
  long BJCashWL;
#endif

#if defined(CRAPS_COMPILE)
  void Calc_CrapsCashWL();
  void Make_CrapsCashWLStr();
  string CrapsCashWLStr;
  long CrapsCashWL;
#endif

#if defined(KENO_COMPILE)
  void Calc_KenoCashWL();
  void Make_KenoCashWLStr();
  string KenoCashWLStr;
  long KenoCashWL;
#endif

#if defined(ROULETTE_COMPILE)
  void Calc_RouletteCashWL();
  void Make_RouletteCashWLStr();
  string RouletteCashWLStr;
  long RouletteCashWL;
#endif

#if defined(SLOTS_COMPILE)
  void Calc_SlotsCashWL();
  void Make_SlotsCashWLStr();
  string SlotsCashWLStr;
  long SlotsCashWL;
#endif

#if defined(VP_COMPILE)
  void Calc_VPCashWL();
  void Make_VPCashWLStr();
  string VPCashWLStr;
  long VPCashWL;
#endif

#if defined(BJ_COMPILE)
  void Calc_BJPercRet();
  void Make_BJPercRetStr();
  string BJPercRetStr;
  float BJPercRet;
#endif

#if defined(CRAPS_COMPILE)
  void Calc_CrapsPercRet();
  void Make_CrapsPercRetStr();
  string CrapsPercRetStr;
  float CrapsPercRet;
#endif

#if defined(KENO_COMPILE)
  void Calc_KenoPercRet();
  void Make_KenoPercRetStr();
  string KenoPercRetStr;
  float KenoPercRet;
#endif

#if defined(ROULETTE_COMPILE)
  void Calc_RoulettePercRet();
  void Make_RoulettePercRetStr();
  string RoulettePercRetStr;
  float RoulettePercRet;
#endif

#if defined(SLOTS_COMPILE)
  void Calc_SlotsPercRet();
  void Make_SlotsPercRetStr();
  string SlotsPercRetStr;
  float SlotsPercRet;
#endif

#if defined(VP_COMPILE)
  void Calc_VPPercRet();
  void Make_VPPercRetStr();
  string VPPercRetStr;
  float VPPercRet;
#endif

#if defined(BJ_COMPILE)
  void Calc_BJCashWLPerHour();
  void Make_BJCashWLPerHourStr();
  string BJCashWLPerHourStr;
  float BJCashWLPerHour;
#endif

#if defined(CRAPS_COMPILE)
  void Calc_CrapsCashWLPerHour();
  void Make_CrapsCashWLPerHourStr();
  string CrapsCashWLPerHourStr;
  float CrapsCashWLPerHour;
#endif

#if defined(KENO_COMPILE)
  void Calc_KenoCashWLPerHour();
  void Make_KenoCashWLPerHourStr();
  string KenoCashWLPerHourStr;
  float KenoCashWLPerHour;
#endif

#if defined(ROULETTE_COMPILE)
  void Calc_RouletteCashWLPerHour();
  void Make_RouletteCashWLPerHourStr();
  string RouletteCashWLPerHourStr;
  float RouletteCashWLPerHour;
#endif

#if defined(SLOTS_COMPILE)
  void Calc_SlotsCashWLPerHour();
  void Make_SlotsCashWLPerHourStr();
  string SlotsCashWLPerHourStr;
  float SlotsCashWLPerHour;
#endif

#if defined(VP_COMPILE)
  void Calc_VPCashWLPerHour();
  void Make_VPCashWLPerHourStr();
  string VPCashWLPerHourStr;
  float VPCashWLPerHour;
#endif
};
//*******************************************************************
#endif