//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cplaydat.h
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     10/12/95   M.Perry     original baseline
//  1.1     5/29/96    A.Krist     Changed craps var's and added slots and keno
//  1.3     5/19/97    M.Perry     added new define's
//  2.0    08/31/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__cplaydat_h)
#define __cplaydat_h

#include "oplaydat.h"
#include "cgametyp.h"

#if defined(VP_COMPILE)
  #include "cvpdata.h"
#endif

#if defined(SLOTS_COMPILE)
  #include "csdata.h"
#endif

#if defined(BJ_COMPILE)
  #include "cbjdata.h"
#endif

#if defined(CRAPS_COMPILE)
  #include "ccdata.h"
#endif

#if defined(ROULETTE_COMPILE)
  #include "crdata.h"
#endif

#if defined(KENO_COMPILE)
  #include "ckdata.h"
#endif
//*******************************************************************
struct TCasinoPlayerData:public TPlayerData
{
  enum {NUMCOLUMNS = 6};

  TCasGameType GameType;
  bool CoachOn;

  void AddCash(TMoney &money)
  {
    Cash += money;
    #if defined(ROUNDCASHTO5CENT)
    Cash.RoundTo5Cent();
    #endif
  }
  void SubCash(TMoney &money)
  {
    Cash -= money;
    if (Cash < 0.0)
    {
      Cash = 0.0;
    }
    #if defined(ROUNDCASHTO5CENT)
    else
    {
      Cash.RoundTo5Cent();
    }
    #endif
  }

  #if defined(PLAYERDATA_SALARY)
  uint32 CreationSec;
  uint32 LastPayedSec;
  void SetCreationSec() {CreationSec = TTime().Seconds();}
  TMoney Salary;
  TMoney GetSalary() {return Salary;}
  void SetSalary(TMoney &salary) {Salary = salary;}
  #endif

  #if defined(PLAYERDATA_CREDIT)
  TMoney AvailNow;
  TMoney CreditLine;
  TMoney CashLimit;
  TMoney MaxDebt;
  TMoney MinDebt;
  TMoney Debt;
  void CalcAvailNow();
  void SetMaxMinDebt();
  void SetMinDebt() {if (Debt < MinDebt) {MinDebt = Debt;}}
  void SetCashLimit();
  void CalcCreditLine();
  void CalcDebtData()
  {
    SetCashLimit();
    CalcAvailNow();
    CalcCreditLine();
  }
  bool CreditCheck()
  {
    CalcDebtData();
    if (AvailNow > 0.0) {return true;}
    return false;
  }
  #endif

  int16 Columns[NUMCOLUMNS];

  #if defined(VP_COMPILE)
  TCasinoVPData VPData;
  #endif

  #if defined(BJ_COMPILE)
  TCasinoBJData BJData;
  #endif

  #if defined(ROULETTE_COMPILE)
  TCasinoRData  RData;
  #endif

  #if defined(CRAPS_COMPILE)
  TCasinoCData  CData;
  #endif

  #if defined(SLOTS_COMPILE)
  TCasinoSData  SData;
  #endif

  #if defined(KENO_COMPILE)
  TCasinoKData  KData;
  #endif

  // proc's
  void ClearColumns();
  void ClearData();
  int16 GetColumn(int16 colnum);
  void SetColumn(int16 colnum,int16 value);
  void ToggleCoach() {CoachOn = !CoachOn;}
  TCasinoPlayerData();
  #if defined(BJ_COMPILE)
  void StartBJSession(int16 subgametype,TMoney denom);
  void EndBJSession();
  #endif
  #if defined(ROULETTE_COMPILE)
  void StartRSession(int16 subgametype,TMoney denom);
  void EndRSession();
  #endif
  #if defined(CRAPS_COMPILE)
  void StartCSession(int16 subgametype,TMoney denom);
  void EndCSession();
  #endif
  #if defined(VP_COMPILE)
  void StartVPSession(int16 subgametype,TMoney denom);
  void EndVPSession();
  #endif
  #if defined(SLOTS_COMPILE)
  void StartSSession(int16 subgametype,TMoney denom);
  void EndSSession();
  #endif
  #if defined(KENO_COMPILE)
  void StartKSession(int16 subgametype,TMoney denom);
  void EndKSession();
  bool IsActiveKenoTicket();
  #endif
};
//*******************************************************************
#endif

