//*******************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//*******************************************************************
//
//       File Name:  bjdeck.cpp
//    Project Name:  Ogle
// Original Author:  Adam K. Krist
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- ---------  ----------  ------------------------------
//  1.0     06/01/95   A. Krist    Original baseline
//  2.0     09/23/97   M.Perry     32-bit conversion.
//*******************************************************************
#include <owl\dialog.h>
#include "owl\inputdia.h"
#include "bjdeck.h"
#include <stdlib.h>
#include "casbjmw.h"
#include "cbjglord.h"
#include "cbjplay.h"

#define  NUMBEROFPLAYERS 3
#define  HUMAN 3
//*******************************************************************
TBJDeck::TBJDeck(int16 numofdecks)
{
  switch (numofdecks)
  {
    case 2: NumCards = DECK2;break;
    case 4: NumCards = DECK4;break;
    case 6: NumCards = DECK6;break;
    default: NumCards = DECK;
  }
  deck = new TCard[NumCards];
  top  = 0;
}
//*******************************************************************
TBJDeck::~TBJDeck()
{
  delete [] deck;
  deck = NULL;
  top  = 0;
  NumCards = 0;
}
//*******************************************************************
int16 TBJDeck::InitDeck(int16 numofcards)
// bj glord was gpf'ing because deck wasn't big enough.
// this could all be made much cleaner (deck,cards,ctor,initdeck,changedecks,etc) ...
{
  if (numofcards != NumCards) // sanity check.
  {
    delete [] deck;
    deck = NULL;
    NumCards = numofcards;
    deck = new TCard[NumCards];
  }

  for(int16 k=0;k<numofcards/52;k++)
  {
	 for(int16 i=0; i < 4 ;i++)
	 {
		for(int16 j=0; j < 13 ;j++)
		{
		  deck[(i*13)+j+(k*52)].name = (TCardName)(j+1);
		  deck[(i*13)+j+(k*52)].suit = (TSuit)i;
		}
	 }
  }
  return 1;
}
//*******************************************************************
int16 TBJDeck::Swap(int16 x,int16 y)
{
  TCard  tmp;

  tmp     = deck[x];
  deck[x] = deck[y];
  deck[y] = tmp;
  return 1;
}//*******************************************************************
int16 TBJDeck::Shuffler(int16 decks)
{
  int16 count,x,y;
  count = 1000;

  for(int16 i=0; i < count; i++)
  {
	 x = int16(random(decks));
	 y = int16(random(decks));
	 Swap(x,y);
  }
  return 1;
}//*******************************************************************
int16 TBJDeck::ChangeDecks(int16 newnumofdecks)
{
  delete [] deck;
  NumCards = newnumofdecks;
  deck = new TCard[newnumofdecks];
  top  = 0;
  InitDeck(newnumofdecks);
  return 1;
}
//*******************************************************************
int16 TBJDeck::Busted(THand * h)
{
  int16 total=0;
  int16 ace=0;
  for(int16 i=0; i < h->inhand ;i++)
  {
	 if(h->cards[i].name == AceHi)
	 {
		total++;
		ace = 1;
	 }
	 else
	 {
		if(h->cards[i].name < Jack)
		{
		  total += int16(h->cards[i].name + 1);
		}
		else
		{
		  if(h->cards[i].name > Ten  && h->cards[i].name < AceHi)
		  {
			 total += int16(10);
		  }
		}
	 }
  }
  h->lohand = total;
  if(ace && total < 12)
  {
	 total += int16(10);
  }
  h->highhand = total;
  if(total > 21)
  {
	 return 1;
  }
  else
  {
	 return 0;
  }
}
//*******************************************************************
int16 TBJDeck::BlackJack(TBJHand * h)
{
  if(h->cards[0].name == AceHi || h->cards[1].name == AceHi)
  {
	 if((h->cards[0].name >= Ten && h->cards[0].name < AceHi) ||
		 (h->cards[1].name >= Ten && h->cards[1].name < AceHi))
	 {
		h->bj = 1;
		return 1; // 1=BlackJack,0=No blackjack
	 }
  }
  return 0;
}
//*******************************************************************
int16 TBJDeck::DontSplitDraw(THand * h,TBlackJackModeWin *pmwin,int16 totalcards)
{
  if(h->cards[h->inhand-1].name > Nine && h->cards[h->inhand-1].name < AceHi)
  {
	 if(deck[top].name > Nine && deck[top].name < AceHi)
	 {
		for(int16 i= int16(top+1);i<totalcards;i++)
		{
		  if(deck[i].name < Nine || deck[i].name == AceHi)
		  {
			 Swap(top,i);
			 Draw(h,pmwin);
			 return 1;
		  }
		}
	 }
	 Draw(h,pmwin);
	 return 1;
  }
  if(h->cards[h->inhand-1].name == deck[top].name)
  {
	 for(int16 i= int16(top+1);i<totalcards;i++)
	 {
		if(h->cards[h->inhand-1].name != deck[i].name)
		{
		  Swap(top,i);
		  Draw(h,pmwin);
		  return 1;
		}
	 }
  }
  Draw(h,pmwin);
  return 1;
}
//*******************************************************************
int16 TBJDeck::Draw(THand * h,TBlackJackModeWin *pmwin)
{
  #ifdef DEBUGCARD
  pmwin->SetUpdatePause();
  char cardnum[5] = "";
  TInputDialog getcard(pmwin,"","card number",cardnum,5);
  getcard.Execute();
  pmwin->ClearUpdatePause();
  h->cards[h->inhand].suit = atoi(cardnum)/14;
  h->cards[h->inhand].name = atoi(cardnum)%14;

  pmwin->GetBJLord()->Count(&h->cards[h->inhand]);
  #else
  int16 temp=0;
  if(h->inhand == 4)
  {
	 for(int16 i=0;i<4;i++)
	 {
		if(h->cards[i].name == AceHi)
		{
		  temp++;
		}
		else
		{
		  if(h->cards[i].name > Nine)
		  {
			 temp += int16(10);
		  }
		  else
		  {
			 temp += int16(h->cards[i].name + 1);
		  }
		}
	 }
	 for(int16 i=top;i<pmwin->GetBJLord()->numofdecks;i++)
	 {
		if(deck[i].name == AceHi) {;}
		else
		{
		  if(deck[i].name + 1 + temp >= 12)
		  {
			 Swap(top,i);
			 break;
		  }
		}
	 }
  }
  if(h->inhand == 5)
  {
	 for(int16 i=0;i<5;i++)
	 {
		if(h->cards[i].name == AceHi)
		{
		  temp++;
		}
		else
		{
		  if(h->cards[i].name > Nine)
		  {
			 temp += int16(10);
		  }
		  else
		  {
			 temp += int16(h->cards[i].name + 1);
		  }
		}
	 }
	 for(int16 i=top;i<pmwin->GetBJLord()->numofdecks;i++)
	 {
		if(deck[i].name + 1 + temp >= 22)
		{
		  Swap(top,i);
		}
	 }
	 if(deck[top].name + 1 + temp < 22)
	 {
		for(int16 i=0;i<top;i++)
		{
		  if(deck[i].name + 1 + temp >= 22)
		  {
			 Swap(top,i);
			 break;
		  }
		}
	 }
  }

  pmwin->GetBJLord()->Count(&deck[top]);
  h->cards[h->inhand] = deck[top++];

  #endif
  h->cards[h->inhand].updown = Up;
  h->inhand++;
  return 1;
}
//*******************************************************************
