//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cbjcalc.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/6/96     M.Perry     derived from Dave's code.
//  2.0    09/17/97    M.Perry     32-bit conversion.
//********************************************************************
#include "cbjcalc.h"
#include "cstatdlg.h"
#include <stdio.h>
#include <classlib\time.h>
//*******************************************************************
TStatCalcBJ::TStatCalcBJ(TCasinoStatDlg *pstatdlg)
:TCasinoStatCalc(pstatdlg)
{
}
//*******************************************************************
TStatCalcBJ::~TStatCalcBJ()
{
}
//*******************************************************************
void TStatCalcBJ::GetSessionPtrs()
{
  if (PPlayerData == NULL) {return;}
  PData = &(PPlayerData->BJData);
  LastSessionNum = -1;
  for (int sessioniterator = GetBJData()->GetFirstValidSession();
       sessioniterator != -1;
       sessioniterator = GetBJData()->GetNextValidSession())
  {
    LastSessionNum++;
    PSessions[LastSessionNum] = &(GetBJData()->Sessions[sessioniterator]);
  }
}
//*******************************************************************
bool TStatCalcBJ::GetValues(int statindex,float *values,int maxvalues)
{
  int num = GetMaxValueIndex(maxvalues);

  switch (statindex)
  {
    case bjTotalNumBets         :return GetTotalNumBets(values,num);
    case bjTotalCashBets        :return GetTotalCashBets(values,num);
    case bjAvgCashBets          :return GetAvgCashBets(values,num);
    case bjTotalCashWL          :return GetTotalCashWL(values,num);
    case bjCumCashWL            :return GetTotalCashWL(values,num);
    case bjHandsWon             :return GetWins(values,num);
    case bjHandsLost            :return GetLosses(values,num);
    case bjPercHandsWon         :return GetPercWins(values,num);
    case bjLongestWinStreak     :return GetLongestWinStreaks(values,num);
    case bjLongestLosingStreak  :return GetLongestLosingStreak(values,num);
    case bjLargestCashWin       :return GetLargestCashWin(values,num);
    case bjLargestCashLost      :return GetLargestCashLost(values,num);
    case bjPeakCashWinLevel     :return GetPeakWinLevel(values,num);
    case bjLowCashLossLevel     :return GetLowCashLostLevel(values,num);
    case bjNumBJ                :return GetNumBJ(values,num);
    case bjNumDD                :return GetNumDD(values,num);
    case bjNumIncPlays          :return GetNumIncPlays(values,num);
    case bjPercAcc              :return GetPercAccuracy(values,num);
    case bjHandsPlus            :return GetHandsPlus(values,num);
    case bjCashWLPlus           :return GetCashWLPlus(values,num);
    case bjHandsMinus           :return GetHandsMinus(values,num);
    case bjCashWLMinus          :return GetCashWLMinus(values,num);
    default                     :return false;
  };
//  return false;
}
//*******************************************************************
bool TStatCalcBJ::GetNumBJ(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetBJSession(i)->BJWins;
  }
  return true;
}
//*******************************************************************
bool TStatCalcBJ::GetNumDD(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetBJSession(i)->DDowns;
  }
  return true;
}
//*******************************************************************
bool TStatCalcBJ::GetNumIncPlays(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetBJSession(i)->IncorrectPlays;
  }
  return true;
}
//*******************************************************************
bool TStatCalcBJ::GetPercAccuracy(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = PercPlayingAccuracy[i];
  }
  return true;
}
//*******************************************************************
bool TStatCalcBJ::GetHandsPlus(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetBJSession(i)->HandsPlayedCGT1;
  }
  return true;
}
//*******************************************************************
bool TStatCalcBJ::GetCashWLPlus(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = (double)GetBJSession(i)->TotalBetsCashCGT1;
  }
  return true;                   
}
//*******************************************************************
bool TStatCalcBJ::GetHandsMinus(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = 0.0;
  }
  return true;
}
//*******************************************************************
bool TStatCalcBJ::GetCashWLMinus(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = 0.0;
  }
  return true;
}
//*******************************************************************
#pragma argsused
string TStatCalcBJ::MakeSubGameTypeStr(int subgametype,TMoney gamedenom)
{
  string s = "Unk bj";
  switch (subgametype)
  {
    case cbj1Deck       : s = "1 Deck";break;
    case cbj2Deck       : s = "2 Deck";break;
    case cbj4Deck       : s = "4 Deck";break;
    case cbj6Deck       : s = "6 Deck";break;
  }
  return s;
}
//*******************************************************************
void TStatCalcBJ::Make_CumulativeCashWLStrs()
{
  // use TotalCasWL
}
//*******************************************************************
void TStatCalcBJ::Calc_CumulativeCashWL()
{
  // same as TotalCashWL
}
//*******************************************************************
void TStatCalcBJ::Make_HandsPlayedStrs()
{
  // use TotalNumBets
}
//*******************************************************************
void TStatCalcBJ::Calc_HandsPlayed()
{
  // same as TotalNumBets
}
//*******************************************************************
void TStatCalcBJ::Make_HandsPushedStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HandsPushedStr[i] = MakeString(GetBJSession(i)->Pushes);
  }
  Total_HandsPushedStr = MakeString(Total_HandsPushed);
}
//*******************************************************************
void TStatCalcBJ::Calc_HandsPushed()
{
  Total_HandsPushed = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_HandsPushed += GetBJSession(i)->Pushes;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_PercHandsPushedStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercHandsPushedStr[i] = MakeString(PercHandsPushed[i]);
  }
  Total_PercHandsPushedStr = MakeString(Total_PercHandsPushed);
}
//*******************************************************************
void TStatCalcBJ::Calc_PercHandsPushed()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercHandsPushed[i] = GetPerc(GetBJSession(i)->Pushes,TotalNumBets[i]);
  }
  Total_PercHandsPushed = GetPerc(Total_HandsPushed,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_InsuranceBetsWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    InsuranceBetsWonStr[i] = MakeString(GetBJSession(i)->InsureWon);
  }
  Total_InsuranceBetsWonStr = MakeString(Total_InsuranceBetsWon);
}
//*******************************************************************
void TStatCalcBJ::Calc_InsuranceBetsWon()
{
  Total_InsuranceBetsWon = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_InsuranceBetsWon += GetBJSession(i)->InsureWon;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_InsuranceBetsLostStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    InsuranceBetsLostStr[i] = MakeString(GetBJSession(i)->InsureLost);
  }
  Total_InsuranceBetsLostStr = MakeString(Total_InsuranceBetsLost);
}
//*******************************************************************
void TStatCalcBJ::Calc_InsuranceBetsLost()
{
  Total_InsuranceBetsLost = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_InsuranceBetsLost += GetBJSession(i)->InsureLost;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_NumBJStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    NumBJStr[i] = MakeString(GetBJSession(i)->BJWins);
  }
  Total_NumBJStr = MakeString(Total_NumBJ);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumBJ()
{
  Total_NumBJ = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumBJ += GetBJSession(i)->BJWins;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_NumDDStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    NumDDStr[i] = MakeString(GetBJSession(i)->DDowns);
  }
  Total_NumDDStr = MakeString(Total_NumDD);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumDD()
{
  Total_NumDD = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumDD += GetBJSession(i)->DDowns;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_NumSplitsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    NumSplitsStr[i] = MakeString(GetBJSession(i)->Splits);
  }
  Total_NumSplitsStr = MakeString(Total_NumSplits);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumSplits()
{
  Total_NumSplits = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumSplits += GetBJSession(i)->Splits;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_PercBJStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercBJStr[i] = MakeString(PercBJ[i]);
  }
  Total_PercBJStr = MakeString(Total_PercBJ);
}
//*******************************************************************
void TStatCalcBJ::Calc_PercBJ()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercBJ[i] = GetPerc(GetBJSession(i)->BJWins,TotalNumBets[i]);
  }
  Total_PercBJ = GetPerc(Total_NumBJ,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_PercDDStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercDDStr[i] = MakeString(PercDD[i]);
  }
  Total_PercDDStr = MakeString(Total_PercDD);
}
//*******************************************************************
void TStatCalcBJ::Calc_PercDD()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercDD[i] = GetPerc(GetBJSession(i)->DDowns,TotalNumBets[i]);
  }
  Total_PercDD = GetPerc(Total_NumDD,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_PercSplitsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercSplitsStr[i] = MakeString(PercSplits[i]);
  }
  Total_PercSplitsStr = MakeString(Total_PercSplits);
}
//*******************************************************************
void TStatCalcBJ::Calc_PercSplits()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
	 PercSplits[i] = GetPerc(GetBJSession(i)->Splits,TotalNumBets[i]-GetBJSession(i)->Splits);
  }
  Total_PercSplits = GetPerc(Total_NumSplits,Total_TotalNumBets-Total_NumSplits);
}
//*******************************************************************
void TStatCalcBJ::Make_CashWLDDStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    CashWLDDStr[i] = MakeString(GetBJSession(i)->DDWinnings);
  }
  Total_CashWLDDStr = MakeString(Total_CashWLDD);
}
//*******************************************************************
void TStatCalcBJ::Calc_CashWLDD()
{
  Total_CashWLDD = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
	 Total_CashWLDD += GetBJSession(i)->DDWinnings;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_CashWLSplitsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    CashWLSplitsStr[i] = MakeString(GetBJSession(i)->SplitsWL);
  }
  Total_CashWLSplitsStr = MakeString(Total_CashWLSplits);
}
//*******************************************************************
void TStatCalcBJ::Calc_CashWLSplits()
{
  Total_CashWLSplits = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_CashWLSplits += GetBJSession(i)->SplitsWL;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_PercBustsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercBustsStr[i] = MakeString(PercBusts[i]);
  }
  Total_PercBustsStr = MakeString(Total_PercBusts);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumBusts()
{
  Total_NumBusts = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumBusts += GetBJSession(i)->Busts;
  }
}
//*******************************************************************
void TStatCalcBJ::Calc_PercBusts()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercBusts[i] = GetPerc(GetBJSession(i)->Busts,TotalNumBets[i]);
  }
  Total_PercBusts = GetPerc(Total_NumBusts,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_PercStand21Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand21Str[i] = MakeString(PercStand21[i]);
  }
  Total_PercStand21Str = MakeString(Total_PercStand21);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumStand21()
{
  Total_NumStand21 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumStand21 += GetBJSession(i)->Player21;
  }
}
//*******************************************************************
void TStatCalcBJ::Calc_PercStand21()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand21[i] = GetPerc(GetBJSession(i)->Player21,TotalNumBets[i]);
  }
  Total_PercStand21 = GetPerc(Total_NumStand21,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_PercStand20Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand20Str[i] = MakeString(PercStand20[i]);
  }
  Total_PercStand20Str = MakeString(Total_PercStand20);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumStand20()
{
  Total_NumStand20 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumStand20 += GetBJSession(i)->Player20;
  }
}
//*******************************************************************
void TStatCalcBJ::Calc_PercStand20()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand20[i] = GetPerc(GetBJSession(i)->Player20,TotalNumBets[i]);
  }
  Total_PercStand20 = GetPerc(Total_NumStand20,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_PercStand19Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand19Str[i] = MakeString(PercStand19[i]);
  }
  Total_PercStand19Str = MakeString(Total_PercStand19);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumStand19()
{
  Total_NumStand19 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumStand19 += GetBJSession(i)->Player19;
  }
}
//*******************************************************************
void TStatCalcBJ::Calc_PercStand19()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand19[i] = GetPerc(GetBJSession(i)->Player19,TotalNumBets[i]);
  }
  Total_PercStand19 = GetPerc(Total_NumStand19,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_PercStand18Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand18Str[i] = MakeString(PercStand18[i]);
  }
  Total_PercStand18Str = MakeString(Total_PercStand18);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumStand18()
{
  Total_NumStand18 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumStand18 += GetBJSession(i)->Player18;
  }
}
//*******************************************************************
void TStatCalcBJ::Calc_PercStand18()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand18[i] = GetPerc(GetBJSession(i)->Player18,TotalNumBets[i]);
  }
  Total_PercStand18 = GetPerc(Total_NumStand18,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_PercStand17Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand17Str[i] = MakeString(PercStand17[i]);
  }
  Total_PercStand17Str = MakeString(Total_PercStand17);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumStand17()
{
  Total_NumStand17 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumStand17 += GetBJSession(i)->Player17;
  }
}
//*******************************************************************
void TStatCalcBJ::Calc_PercStand17()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStand17[i] = GetPerc(GetBJSession(i)->Player17,TotalNumBets[i]);
  }
  Total_PercStand17 = GetPerc(Total_NumStand17,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_PercStandLT17Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStandLT17Str[i] = MakeString(PercStandLT17[i]);
  }
  Total_PercStandLT17Str = MakeString(Total_PercStandLT17);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumStandLT17()
{
  Total_NumStandLT17 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumStandLT17 += GetBJSession(i)->Player16Less;
  }
}
//*******************************************************************
void TStatCalcBJ::Calc_PercStandLT17()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStandLT17[i] = GetPerc(GetBJSession(i)->Player16Less,TotalNumBets[i]);
  }
  Total_PercStandLT17 = GetPerc(Total_NumStandLT17,Total_TotalNumBets);
}
//*******************************************************************
void TStatCalcBJ::Make_NumBadStrategyPlaysStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    NumBadStrategyPlaysStr[i] = MakeString(GetBJSession(i)->IncorrectPlays);
  }
  Total_NumBadStrategyPlaysStr = MakeString(Total_NumBadStrategyPlays);
}
//*******************************************************************
void TStatCalcBJ::Calc_NumBadStrategyPlays()
{
  Total_NumBadStrategyPlays = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumBadStrategyPlays += GetBJSession(i)->IncorrectPlays;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_PercPlayingAccuracyStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercPlayingAccuracyStr[i] = MakeString(PercPlayingAccuracy[i]);
  }
  Total_PercPlayingAccuracyStr = MakeString(Total_PercPlayingAccuracy);
}
//*******************************************************************
void TStatCalcBJ::Calc_PercPlayingAccuracy()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
	 if(TotalNumBets[i] < GetBJSession(i)->IncorrectPlays)
	 {
		PercPlayingAccuracy[i] = GetPerc(0.0,TotalNumBets[i]);
	 }
	 else
	 {
		PercPlayingAccuracy[i] = GetPerc(TotalNumBets[i] - GetBJSession(i)->IncorrectPlays,TotalNumBets[i]);
	 }
  }
  if(Total_NumBadStrategyPlays > Total_TotalNumBets)
  {
	 Total_PercPlayingAccuracy = GetPerc(0.0,Total_TotalNumBets);
  }
  else
  {
	 Total_PercPlayingAccuracy = GetPerc(Total_TotalNumBets-Total_NumBadStrategyPlays,Total_TotalNumBets);
  }
}
//*******************************************************************
void TStatCalcBJ::Make_HandsPlusCountStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HandsPlusCountStr[i] = MakeString(GetBJSession(i)->HandsPlayedCGT1);
  }
  Total_HandsPlusCountStr = MakeString(Total_HandsPlusCount);
}
//*******************************************************************
void TStatCalcBJ::Calc_HandsPlusCount()
{
  Total_HandsPlusCount = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_HandsPlusCount += GetBJSession(i)->HandsPlayedCGT1;
  }
}
//*******************************************************************
void TStatCalcBJ::Make_AvgBetPlusCountStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    AvgBetPlusCountStr[i] = MakeString(AvgBetPlusCount[i]);
  }
  Total_AvgBetPlusCountStr = MakeString(Total_AvgBetPlusCount);
}
//*******************************************************************
void TStatCalcBJ::Calc_AvgBetPlusCount()
{
  Total_TotalBetsCashPlusCount = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
	 AvgBetPlusCount[i] = GetAvg(GetBJSession(i)->TotalBetsCashCGT1,GetBJSession(i)->HandsPlayedCGT1);
	 Total_TotalBetsCashPlusCount += GetBJSession(i)->TotalBetsCashCGT1;
  }
  Total_AvgBetPlusCount = GetAvg(Total_TotalBetsCashPlusCount,Total_HandsPlusCount);
}
//*******************************************************************
void TStatCalcBJ::Make_CashWLPlusCountStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
	 CashWLPlusCountStr[i] = MakeString(GetBJSession(i)->TotalCashWLCGT1); // Adam
  }
  Total_CashWLPlusCountStr = MakeString(Total_CashWLPlusCount); // Adam
}
//*******************************************************************
void TStatCalcBJ::Calc_CashWLPlusCount()
{
  Total_CashWLPlusCount = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
	 Total_CashWLPlusCount += GetBJSession(i)->TotalCashWLCGT1; // Adam
  }
}
//*******************************************************************
void TStatCalcBJ::Make_HandsZMinusCountStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HandsZMinusCountStr[i] = MakeString(HandsZMinusCount[i]);
  }
  Total_HandsZMinusCountStr = MakeString(Total_HandsZMinusCount);
}
//*******************************************************************
void TStatCalcBJ::Calc_HandsZMinusCount()
{
  Total_HandsZMinusCount = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
	 HandsZMinusCount[i] = GetBJSession(i)->HandsPlayedCLT2;// - GetBJSession(i)->WinsCGT1;
	 Total_HandsZMinusCount += HandsZMinusCount[i];
  }
}
//*******************************************************************
void TStatCalcBJ::Make_AvgBetZMinusCountStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
	 AvgBetZMinusCountStr[i] = MakeString(AvgBetZMinusCount[i]);//(float)0.0);
  }
  Total_AvgBetZMinusCountStr = MakeString(Total_AvgBetZMinusCount);
}
//*******************************************************************
void TStatCalcBJ::Calc_AvgBetZMinusCount()
{
  TMoney Total_TotalBetsCashZMinusCount = 0;
  for (int i =0;i<=LastSessionNum;i++)
  {
	 AvgBetZMinusCount[i] = GetAvg(GetBJSession(i)->TotalBetsCashCLT2,GetBJSession(i)->HandsPlayedCLT2);//0.0;
	 Total_TotalBetsCashZMinusCount += GetBJSession(i)->TotalBetsCashCLT2;
  }
  Total_AvgBetZMinusCount = GetAvg(Total_TotalBetsCashZMinusCount,Total_HandsZMinusCount);//0.0;
}
//*******************************************************************
void TStatCalcBJ::Make_CashWLZMinusCountStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    CashWLZMinusCountStr[i] = MakeString(CashWLZMinusCount[i]);
  }
  Total_CashWLZMinusCountStr = MakeString(Total_CashWLZMinusCount);
}
//*******************************************************************
void TStatCalcBJ::Calc_CashWLZMinusCount()
{
  Total_CashWLZMinusCount = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
	 CashWLZMinusCount[i] = GetBJSession(i)->TotalCashWLCLT1;//0.0;
	 Total_CashWLZMinusCount += GetBJSession(i)->TotalCashWLCLT1;
  }
  //Total_CashWLZMinusCount = 0.0;
}
//*******************************************************************
void TStatCalcBJ::LocalCalculate()
{
  TCasinoStatCalc::LocalCalculate();

  if (PPlayerData == NULL) {return;}

  Calc_CumulativeCashWL();
  Calc_HandsPlayed();
  Calc_HandsPushed();
  Calc_PercHandsPushed();
  Calc_InsuranceBetsWon();
  Calc_InsuranceBetsLost();
  Calc_NumBJ();
  Calc_NumDD();
  Calc_NumSplits();
  Calc_PercBJ();
  Calc_PercDD();
  Calc_PercSplits();
  Calc_CashWLDD();
  Calc_CashWLSplits();
  Calc_NumBusts();
  Calc_PercBusts();
  Calc_NumStand21();
  Calc_PercStand21();
  Calc_NumStand20();
  Calc_PercStand20();
  Calc_NumStand19();
  Calc_PercStand19();
  Calc_NumStand18();
  Calc_PercStand18();
  Calc_NumStand17();
  Calc_PercStand17();
  Calc_NumStandLT17();
  Calc_PercStandLT17();
  Calc_NumBadStrategyPlays();
  Calc_PercPlayingAccuracy();
  Calc_HandsPlusCount();
  Calc_AvgBetPlusCount();
  Calc_CashWLPlusCount();
  Calc_HandsZMinusCount();
  Calc_AvgBetZMinusCount();
  Calc_CashWLZMinusCount();
}
//*******************************************************************
void TStatCalcBJ::LocalMakeAllStrings()
{
  TCasinoStatCalc::LocalMakeAllStrings();

  if (PPlayerData == NULL) {return;}

  Make_CumulativeCashWLStrs();
  Make_HandsPlayedStrs();
  Make_HandsPushedStrs();
  Make_PercHandsPushedStrs();
  Make_InsuranceBetsWonStrs();
  Make_InsuranceBetsLostStrs();
  Make_NumBJStrs();
  Make_NumDDStrs();
  Make_NumSplitsStrs();
  Make_PercBJStrs();
  Make_PercDDStrs();
  Make_PercSplitsStrs();
  Make_CashWLDDStrs();
  Make_CashWLSplitsStrs();
  Make_PercBustsStrs();
  Make_PercStand21Strs();
  Make_PercStand20Strs();
  Make_PercStand19Strs();
  Make_PercStand18Strs();
  Make_PercStand17Strs();
  Make_PercStandLT17Strs();
  Make_NumBadStrategyPlaysStrs();
  Make_PercPlayingAccuracyStrs();
  Make_HandsPlusCountStrs();
  Make_AvgBetPlusCountStrs();
  Make_CashWLPlusCountStrs();
  Make_HandsZMinusCountStrs();
  Make_AvgBetZMinusCountStrs();
  Make_CashWLZMinusCountStrs();
}
//*******************************************************************
void TStatCalcBJ::LocalFillGrid()
{
  TCasinoStatCalc::LocalFillGrid();

  if (PPlayerData == NULL) {return;}

  for(int col = 0;col <= LastSessionNum;col++)
  {
    int c = col + 1;
    PStatDlg->cell[1][c] = DateStr[col];
    PStatDlg->cell[2][c] = HoursPlayedStr[col];
    PStatDlg->cell[3][c] = TypeGamePlayedStr[col];
    PStatDlg->cell[6][c] = TotalNumBetsStr[col];
    PStatDlg->cell[7][c] = TotalCashBetsStr[col];
    PStatDlg->cell[8][c] = AvgCashBetsStr[col];
    PStatDlg->cell[9][c] = StartingBankrollStr[col];
    PStatDlg->cell[10][c] = EndingBankrollStr[col];
    PStatDlg->cell[11][c] = TotalCashWLStr[col];
    PStatDlg->cell[12][c] = ReturnPercStr[col];
    PStatDlg->cell[13][c] = AvgWLPerHourStr[col];
    PStatDlg->cell[14][c] = TotalCashWLStr[col];
    PStatDlg->cell[17][c] = TotalNumBetsStr[col];
    PStatDlg->cell[18][c] = HandsWonStr[col];
    PStatDlg->cell[19][c] = HandsLostStr[col];
    PStatDlg->cell[20][c] = HandsPushedStr[col];
    PStatDlg->cell[21][c] = PercHandsWonStr[col];
    PStatDlg->cell[22][c] = PercHandsLostStr[col];
    PStatDlg->cell[23][c] = PercHandsPushedStr[col];
    PStatDlg->cell[24][c] = LongestWinStreakStr[col];
    PStatDlg->cell[25][c] = LongestLosingStreakStr[col];
    PStatDlg->cell[26][c] = LargestCashWinStr[col];
    PStatDlg->cell[27][c] = LargestCashLostStr[col];
    PStatDlg->cell[28][c] = PeakCashWinLevelStr[col];
    PStatDlg->cell[29][c] = LowCashLostLevelStr[col];
    PStatDlg->cell[31][c] = InsuranceBetsWonStr[col];
    PStatDlg->cell[32][c] = InsuranceBetsLostStr[col];
    PStatDlg->cell[33][c] = NumBJStr[col];
    PStatDlg->cell[34][c] = NumDDStr[col];
    PStatDlg->cell[35][c] = NumSplitsStr[col];
    PStatDlg->cell[36][c] = PercBJStr[col];
    PStatDlg->cell[37][c] = PercDDStr[col];
    PStatDlg->cell[38][c] = PercSplitsStr[col];
    PStatDlg->cell[39][c] = CashWLDDStr[col];
    PStatDlg->cell[40][c] = CashWLSplitsStr[col];
    PStatDlg->cell[42][c] = PercBustsStr[col];
    PStatDlg->cell[43][c] = PercStand21Str[col];
    PStatDlg->cell[44][c] = PercStand20Str[col];
    PStatDlg->cell[45][c] = PercStand19Str[col];
    PStatDlg->cell[46][c] = PercStand18Str[col];
    PStatDlg->cell[47][c] = PercStand17Str[col];
    PStatDlg->cell[48][c] = PercStandLT17Str[col];
    PStatDlg->cell[51][c] = NumBadStrategyPlaysStr[col];
    PStatDlg->cell[52][c] = PercPlayingAccuracyStr[col];
    PStatDlg->cell[54][c] = HandsPlusCountStr[col];
    PStatDlg->cell[55][c] = AvgBetPlusCountStr[col];
    PStatDlg->cell[56][c] = CashWLPlusCountStr[col];
    PStatDlg->cell[57][c] = HandsZMinusCountStr[col];
    PStatDlg->cell[58][c] = AvgBetZMinusCountStr[col];
    PStatDlg->cell[59][c] = CashWLZMinusCountStr[col];
  }
  PStatDlg->totalcol[1] = Total_DateStr;
  PStatDlg->totalcol[2] = Total_HoursPlayedStr;
  PStatDlg->totalcol[3] = Total_TypeGamePlayedStr;
  PStatDlg->totalcol[6] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[7] = Total_TotalCashBetsStr;
  PStatDlg->totalcol[8] = Total_AvgCashBetsStr;
  PStatDlg->totalcol[9] = Total_StartingBankrollStr;
  PStatDlg->totalcol[10] = Total_EndingBankrollStr;
  PStatDlg->totalcol[11] = Total_TotalCashWLStr;
  PStatDlg->totalcol[12] = Total_ReturnPercStr;
  PStatDlg->totalcol[13] = Total_AvgWLPerHourStr;
  PStatDlg->totalcol[14] = Total_TotalCashWLStr;
  PStatDlg->totalcol[17] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[18] = Total_HandsWonStr;
  PStatDlg->totalcol[19] = Total_HandsLostStr;
  PStatDlg->totalcol[20] = Total_HandsPushedStr;
  PStatDlg->totalcol[21] = Total_PercHandsWonStr;
  PStatDlg->totalcol[22] = Total_PercHandsLostStr;
  PStatDlg->totalcol[23] = Total_PercHandsPushedStr;
  PStatDlg->totalcol[24] = Total_LongestWinStreakStr;
  PStatDlg->totalcol[25] = Total_LongestLosingStreakStr;
  PStatDlg->totalcol[26] = Total_LargestCashWinStr;
  PStatDlg->totalcol[27] = Total_LargestCashLostStr;
  PStatDlg->totalcol[28] = Total_PeakCashWinLevelStr;
  PStatDlg->totalcol[29] = Total_LowCashLostLevelStr;
  PStatDlg->totalcol[31] = Total_InsuranceBetsWonStr;
  PStatDlg->totalcol[32] = Total_InsuranceBetsLostStr;
  PStatDlg->totalcol[33] = Total_NumBJStr;
  PStatDlg->totalcol[34] = Total_NumDDStr;
  PStatDlg->totalcol[35] = Total_NumSplitsStr;
  PStatDlg->totalcol[36] = Total_PercBJStr;
  PStatDlg->totalcol[37] = Total_PercDDStr;
  PStatDlg->totalcol[38] = Total_PercSplitsStr;
  PStatDlg->totalcol[39] = Total_CashWLDDStr;
  PStatDlg->totalcol[40] = Total_CashWLSplitsStr;
  PStatDlg->totalcol[42] = Total_PercBustsStr;
  PStatDlg->totalcol[43] = Total_PercStand21Str;
  PStatDlg->totalcol[44] = Total_PercStand20Str;
  PStatDlg->totalcol[45] = Total_PercStand19Str;
  PStatDlg->totalcol[46] = Total_PercStand18Str;
  PStatDlg->totalcol[47] = Total_PercStand17Str;
  PStatDlg->totalcol[48] = Total_PercStandLT17Str;
  PStatDlg->totalcol[51] = Total_NumBadStrategyPlaysStr;
  PStatDlg->totalcol[52] = Total_PercPlayingAccuracyStr;
  PStatDlg->totalcol[54] = Total_HandsPlusCountStr;
  PStatDlg->totalcol[55] = Total_AvgBetPlusCountStr;
  PStatDlg->totalcol[56] = Total_CashWLPlusCountStr;
  PStatDlg->totalcol[57] = Total_HandsZMinusCountStr;
  PStatDlg->totalcol[58] = Total_AvgBetZMinusCountStr;
  PStatDlg->totalcol[59] = Total_CashWLZMinusCountStr;
}
//*******************************************************************
//*******************************************************************
TBJGraph::TBJGraph(TCasinoStatDlg* pstatdlg,TStatCalcBJ *pstatcalc)
:TCasinoGraphDlg(pstatdlg,pstatcalc)
{
  strName = "Blackjack";
  BaseStringID = BlackjackLabel;

  StatTT[0] = TStatCalcBJ::bjTotalNumBets;
  StatTT[1] = TStatCalcBJ::bjTotalCashBets;
  StatTT[2] = TStatCalcBJ::bjAvgCashBets;
  StatTT[3] = TStatCalcBJ::bjTotalCashWL;
  StatTT[4] = TStatCalcBJ::bjCumCashWL;
  StatTT[5] = TStatCalcBJ::bjHandsWon;
  StatTT[6] = TStatCalcBJ::bjHandsLost;
  StatTT[7] = TStatCalcBJ::bjPercHandsWon;
  StatTT[8] = TStatCalcBJ::bjLongestWinStreak;
  StatTT[9] = TStatCalcBJ::bjLongestLosingStreak;
  StatTT[10] = TStatCalcBJ::bjLargestCashWin;
  StatTT[11] = TStatCalcBJ::bjLargestCashLost;
  StatTT[12] = TStatCalcBJ::bjPeakCashWinLevel;
  StatTT[13] = TStatCalcBJ::bjLowCashLossLevel;
  StatTT[14] = TStatCalcBJ::bjNumBJ;
  StatTT[15] = TStatCalcBJ::bjNumDD;
  StatTT[16] = TStatCalcBJ::bjNumIncPlays;
  StatTT[17] = TStatCalcBJ::bjPercAcc;
  StatTT[18] = TStatCalcBJ::bjHandsPlus;
  StatTT[19] = TStatCalcBJ::bjCashWLPlus;
  StatTT[20] = TStatCalcBJ::bjHandsMinus;
  StatTT[21] = TStatCalcBJ::bjCashWLMinus;

  LastStatIndex = 21;
}
//*******************************************************************
TBJGraph::~TBJGraph()
{
}
//*******************************************************************
