//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cgraph.cpp                                
//    Project Name:  Casino
// Original Author:  Dave Anderson
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/9/96     M.Perry    modified Dave's code.
//********************************************************************
#include "cgraph.h"
#include <owl/dc.h>
#include "cstatdlg.h"
#include "cstcalc.h"
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TCasinoGraphDlg,TDialog)
	EV_WM_PAINT,
	EV_COMMAND(IDC_OK,CmOK),
	EV_CBN_SELCHANGE(IDC_COMBOBOX1,EvCbnSelchange),
END_RESPONSE_TABLE;
//*******************************************************************
void TCasinoGraphDlg::EvCbnSelchange()
{
  int statindex = PStats_CB->GetSelIndex();
  if (!CheckStatIndex(statindex)) {return;}
  StatIndex = statindex;
  GetData();
  SetTitle();
  Invalidate();
}
//*******************************************************************
TCasinoGraphDlg::TCasinoGraphDlg(TCasinoStatDlg* pstatdlg,TCasinoStatCalc *pstatcalc,TResId resIdDialog)
:TDialog(pstatdlg,resIdDialog),
  PStatCalc(pstatcalc),
  m_numYDivisions(16),
  StatIndex(1),
  LastStatIndex(-1),
  BaseStringID(0),
  UseStatLabelStr(false)
{
  PStatCalc->Calculate();
  strSlash = " - ";
  PStatDlg =  pstatdlg;
  m_xRes = ::GetSystemMetrics(SM_CXSCREEN);
  m_yRes = ::GetSystemMetrics(SM_CYSCREEN);
  PStats_CB = new TComboBox(this,IDC_COMBOBOX1);

  for (int i=0;i<MAX_GRAPH_SESSIONS;i++)
  {
    GraphValues[i] = 0.0;
  }

  for (int16 i = 0;i<MAX_STAT_CB_ENTRIES;i++)
  {
    StatLabelStr[i] = "";
  }
}
//*******************************************************************
TCasinoGraphDlg::~TCasinoGraphDlg()
{
  delete PStats_CB;
}
//*******************************************************************
void TCasinoGraphDlg::CmOK()
{
  Destroy();
}
//*******************************************************************
void TCasinoGraphDlg::SetupWindow()
{
  TDialog::SetupWindow();
  LoadLB();
  SetTitle();

  TRect rec = GetWindowRect();
  m_dialogWidth  = rec.right-rec.left;
  m_dialogHeight = rec.bottom - rec.top;
  int xRes=   ::GetSystemMetrics(SM_CXSCREEN);
  int yRes=   ::GetSystemMetrics(SM_CYSCREEN);

  MoveWindow((xRes - m_dialogWidth)/2,(yRes - m_dialogHeight)/2, m_dialogWidth,m_dialogHeight);
  SetBoxSize();
  SetAxesPositions();
  m_numYDivisions = EstablishNumYDivisions();
  EstablishXTickDivisions();

  GetData();
}
//*******************************************************************
void TCasinoGraphDlg::EvPaint()
{
  TPaintDC dc(*this);
  TRect rec = GetClientRect();
  randomize();
  m_RED = short(rand()%255);
  m_GREEN = short(rand()%255);
  m_BLUE = short(rand()%255);

  //mrp - v2.4 - 09/16/99 - add conditional compile's around bg colors.
  #if defined(ENB_STATS_BGCOLOR)
  dc.FillRect(rec,TBrush(TColor(RGB(m_RED,m_GREEN,m_BLUE))));
  #endif

  dc.Rectangle(m_box_X1,m_box_Y1,m_box_X2,m_box_Y2);
  if (!DrawXTicks(dc)){CmOK();};
  DrawXLabel(dc);
  DrawResults(dc);
  DrawXAxis(dc);
  DrawYAxis(dc);
}
//*******************************************************************
void TCasinoGraphDlg::DrawXAxis(TPaintDC& dc)
{
  dc.MoveTo(m_Origin_X,m_nZeroY);
  dc.LineTo(m_XTick[MAX_GRAPH_SESSIONS],m_nZeroY);
}
//*******************************************************************
void TCasinoGraphDlg::DrawYAxis(TPaintDC& dc)
{
  dc.MoveTo(m_Origin_X,m_Origin_Y);
  dc.LineTo(m_topofYAxis_X1,m_topTickOfY);
}
//*******************************************************************
void TCasinoGraphDlg::FindMinMaxAndRange()
{
  m_nMin =  100000000l;
  m_nMax = -100000000l;
  for (int i=0;i<MAX_GRAPH_SESSIONS;i++)
  {
    if (GraphValues[i] > m_nMax) {m_nMax = GraphValues[i];}
    if (GraphValues[i] < m_nMin) {m_nMin = GraphValues[i];}
  }
  if ((m_nMin > 0) && (m_nMax > 0))  {m_nMin =0;}
  if ((m_nMin < 0) && (m_nMax < 0))  {m_nMax =0;}
  m_nRange = m_nMax - m_nMin;
}
//*******************************************************************
void TCasinoGraphDlg::DrawResults(TPaintDC& dc)
{
  TRect rc;
  DrawYTicksAndLabels(dc);
  float nScale = (m_Origin_Y - m_topofYAxis_Y1) / m_numYDivisions;
  for (int i=0;i<MAX_GRAPH_SESSIONS;i++)
  {
    rc.left = m_XTick[i];
    rc.right = m_XTick[i + 1];

    if (GraphValues[i] > 16000000l) {GraphValues[i] = 16000000l;}

    rc.top = m_nZeroY - GraphValues[i] / m_fScale * nScale;
    rc.bottom =  m_nZeroY;
    rc.Normalize();
    if (GraphValues[i] >= 0)
	 {         //green
      TColor positive(7,103,0);//need to use paletteentry index to prevent dithering
      TBrush br1(positive);
      dc.FillRect(rc,br1);
    }
    else
    {         //red
      TColor negative(179,0,0);
      TBrush br1(negative);
      dc.FillRect(rc,br1);
    }
  }
}
//*******************************************************************
void TCasinoGraphDlg::DrawXLabel(TPaintDC& dc)
{
  TSize sizeString = dc.GetTextExtent("Session Number",15);
  dc.SetBkMode(TRANSPARENT);
  dc.TextOut(m_box_X1+(m_box_X2-m_box_X1)/2-sizeString.cx/2,m_box_Y2+5,"Session Number");
}
//*******************************************************************
void TCasinoGraphDlg::SetBoxSize()
{
  int nCapHt = ::GetSystemMetrics(SM_CYCAPTION);
  m_box_X1 = 15;                                       //left
  m_box_Y1 = 40;                                       //top
  m_box_X2 = m_dialogWidth - m_box_X1;                 //right
  m_box_Y2 = m_dialogHeight - m_box_X1 - nCapHt- 20;   //bottom
}
//*******************************************************************
void TCasinoGraphDlg::SetAxesPositions()
{
  m_topofYAxis_X1   = m_box_X1 + 65;
  m_topofYAxis_Y1   = m_box_Y1 + 5;
  m_Origin_X        = m_box_X1 + 65;
  m_Origin_Y        = m_box_Y2 - 25;
  m_EndofXAxis_X    = m_box_X2 - 10;
  m_EndofXAxis_Y    = m_box_Y2 - 25;
}
//*******************************************************************
void TCasinoGraphDlg::DrawYTicksAndLabels(TPaintDC& dc)
{  //called by DrawResults()
  EstablishYTickDivisions(dc);
  DrawYLabels(dc);
}
//*******************************************************************
int TCasinoGraphDlg::EstablishNumYDivisions()
{
  TPaintDC dc(*this);
  TEXTMETRIC tmMetrics;
  dc.GetTextMetrics(tmMetrics);
  int YAxisExtent = m_Origin_Y - m_topofYAxis_Y1;
  int result = (YAxisExtent/(tmMetrics.tmHeight + tmMetrics.tmExternalLeading));
  // mike added below.
  if (result >= NUM_Y_TICKS) {result = NUM_Y_TICKS - 1;}
  return result;
}
//*******************************************************************
void TCasinoGraphDlg::EstablishYTickDivisions(TPaintDC& dc)
{
  FindMinMaxAndRange();
  TEXTMETRIC tmMetrics;
  dc.GetTextMetrics(tmMetrics);
  float nDivision;
  float fScale = (m_Origin_Y - m_topofYAxis_Y1) / m_numYDivisions;

  for (int j=0;j<m_numYDivisions;j++)
  {
    nDivision = j* fScale;
    m_YTick[j] = m_Origin_Y  - nDivision -  tmMetrics.tmHeight/2;
    //draw tick
    dc.MoveTo(m_Origin_X-3,m_Origin_Y  - nDivision);
    dc.LineTo(m_Origin_X+3,m_Origin_Y  - nDivision);
  }
  m_topTickOfY = m_Origin_Y - nDivision;
}
//*******************************************************************
void TCasinoGraphDlg::DrawYLabels(TPaintDC& dc)
{
  TEXTMETRIC tmMetrics;
  dc.GetTextMetrics(tmMetrics);
  int nCharHt = tmMetrics.tmHeight;
  int nCharWidth = tmMetrics.tmAveCharWidth;
  m_fScale = FindYScale();

  long int first = 1;
  if (m_nMin < 0)
  {
    long int r = m_nMin / m_fScale;
    first = r-1;
  }
  if (m_nMin >= 0)
  {
    first = 0;
  }

  long int start_at = m_fScale * first;
  long int jj;
  for (int j=0;j<m_numYDivisions;j++)
  {
    char buf[33];
    jj = start_at + j * m_fScale;
    if (jj==0) {m_nZeroY = m_YTick[j] + nCharHt/2;}
    ltoa(jj,buf,10);
    dc.TextOut(m_Origin_X-((1+strlen(buf)) * nCharWidth),m_YTick[j],buf);
  }
}
//*******************************************************************
void TCasinoGraphDlg::EstablishXTickDivisions()
{
  int nDivision = (m_EndofXAxis_X - m_Origin_X) / (MAX_GRAPH_SESSIONS);
  for (int i=0;i < NUM_X_TICKS;i++)
  {
    m_XTick[i] = m_Origin_X + (nDivision * i);
  }
}
//*******************************************************************
BOOL TCasinoGraphDlg::DoTheXCharsFit(TPaintDC& dc)
{
  TEXTMETRIC tmMetrics;
  dc.GetTextMetrics(tmMetrics);
  int nCharWidth = tmMetrics.tmAveCharWidth;
  //will have
  // 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25
  //which is 41 chars
  int nXAxisLength = m_XTick[MAX_GRAPH_SESSIONS] - m_Origin_X;
  if (41 * nCharWidth < nXAxisLength) {return TRUE;}
  return FALSE;
}
//*******************************************************************
BOOL TCasinoGraphDlg::DrawXTicks(TPaintDC& dc)
{
  if (!DoTheXCharsFit(dc))
  {
    ::MessageBox(this->HWindow,"Inadequate resolution","Graph not available",MB_OK);
    return FALSE;
  }
  char temp[5];
  int buffer[2];
  dc.GetCharWidth((uint)"0",(uint)"1",buffer);
  for (int i=0;i<MAX_GRAPH_SESSIONS;i++)
  {
    dc.MoveTo(m_XTick[i + 1],m_Origin_Y - 3);
    dc.LineTo(m_XTick[i + 1],m_Origin_Y + 3);
    itoa(i + 1,temp,10);
    if ((i + 1) < 10)
    {
      if ((i + 1) % 2)
      {
        dc.TextOut(m_XTick[i] + (m_XTick[i + 1] - m_XTick[i])/2 - buffer[0] / 2,
                   m_Origin_Y + 2,temp);
      }
      else
      {
        dc.TextOut(m_XTick[i] + (m_XTick[i + 1] - m_XTick[i])/2 - buffer[0] / 2,
                   m_Origin_Y + 4,temp);
      }
    }
    else
    {
      if ((i + 1) % 2)
      {
        dc.TextOut(m_XTick[i] + (m_XTick[i + 1] - m_XTick[i])/2 - buffer[0],
                   m_Origin_Y + 2,temp);
      }
      else
      {
        dc.TextOut(m_XTick[i] + (m_XTick[i + 1] - m_XTick[i])/2 - buffer[0],
                   m_Origin_Y + 4,temp);
      }
    }
  }
  return true;
}
//*******************************************************************
float TCasinoGraphDlg::FindYScale()
{
  float fNumberPerTick = m_nRange / m_numYDivisions;
  long int roundedNumberPerTick = (long int)(fNumberPerTick + 0.5);
  float floatedRoundedNumberPerTick = roundedNumberPerTick;

  if (floatedRoundedNumberPerTick < 1.0)             {return 1.0;}
  else if (floatedRoundedNumberPerTick < 2.0)        {return 2.0;}
  else if (floatedRoundedNumberPerTick < 5.0)        {return 5.0;}
  else if (floatedRoundedNumberPerTick < 10.0)       {return 10.0;}
  else if (floatedRoundedNumberPerTick < 20.0)       {return 20.0;}
  else if (floatedRoundedNumberPerTick < 50.0)       {return 50.0;}
  else if (floatedRoundedNumberPerTick < 100.0)      {return 100.0;}
  else if (floatedRoundedNumberPerTick < 200.0)      {return 200.0;}
  else if (floatedRoundedNumberPerTick < 250.0)      {return 250.0;}
  else if (floatedRoundedNumberPerTick < 500.0)      {return 500.0;}
  else if (floatedRoundedNumberPerTick < 1000.0)     {return 1000.0;}
  else if (floatedRoundedNumberPerTick < 2000.0)     {return 2000.0;}
  else if (floatedRoundedNumberPerTick < 5000.0)     {return 5000.0;}
  else if (floatedRoundedNumberPerTick < 10000.0)    {return 10000.0;}
  else if (floatedRoundedNumberPerTick < 20000.0)    {return 20000.0;}
  else if (floatedRoundedNumberPerTick < 50000.0)    {return 50000.0;}
  else if (floatedRoundedNumberPerTick < 100000.0)   {return 100000.0;}
  else if (floatedRoundedNumberPerTick < 200000.0)   {return 200000.0;}
  else if (floatedRoundedNumberPerTick < 500000.0)   {return 500000.0;}
  else if (floatedRoundedNumberPerTick < 1000000.0)  {return 1000000.0;}
  return 1000000.0;
}
//*******************************************************************
void TCasinoGraphDlg::LoadLB()
{
  for (int i=0;i<=LastStatIndex;i++)
  {
    string s;
    if (UseStatLabelStr)
    {
      s = StatLabelStr[i];
    }
    else
    {
      s = string(*GetModule(),GetStatStrID(StatTT[i]));
    }
    PStats_CB->AddString(s.c_str());
  }
  PStats_CB->SetSelIndex(StatIndex);
}
//*******************************************************************
void TCasinoGraphDlg::GetData()
{
  if (PStatCalc == NULL) {return;}
  if (!CheckStatIndex(StatIndex)) {return;}

  for (int i=0;i<MAX_GRAPH_SESSIONS;i++)
  {
    GraphValues[i] = 0.0;
  }

  if (!PStatCalc->GetValues(StatTT[StatIndex],GraphValues,MAX_GRAPH_SESSIONS))
  {
    MessageBox("Unknown error receiving data from statcalc","Error - Graph",MB_OK);
  }
}
//*******************************************************************
string TCasinoGraphDlg::GetTitle()
{
  string s;
  if (UseStatLabelStr)
  {
    s = StatLabelStr[StatIndex];
  }
  else
  {
    s = strName + strSlash + string(*GetModule(),GetStatStrID(StatTT[StatIndex]));
  }
  return s;
}
//*******************************************************************
void TCasinoGraphDlg::SetTitle()
{
  SetWindowText(GetTitle().c_str());
}
//*******************************************************************

