//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  crcalc.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/6/96     M.Perry    derived from Dave's code
//  2.0    09/17/97    M.Perry    32-bit conversion.
//********************************************************************
#include "crcalc.h"
#include "cstatdlg.h"
#include <stdio.h>
#include <classlib\time.h>
//*******************************************************************
TStatCalcRoulette::TStatCalcRoulette(TCasinoStatDlg *pstatdlg)
:TCasinoStatCalc(pstatdlg)
{
}
//*******************************************************************
TStatCalcRoulette::~TStatCalcRoulette()
{
}
//*******************************************************************
void TStatCalcRoulette::GetSessionPtrs()
{
  if (PPlayerData == NULL) {return;}
  PData = &(PPlayerData->RData);

  LastSessionNum = -1;
  for (int sessioniterator = GetRData()->GetFirstValidSession();
       sessioniterator != -1;
       sessioniterator = GetRData()->GetNextValidSession())
  {
    LastSessionNum++;
    PSessions[LastSessionNum] = &(GetRData()->Sessions[sessioniterator]);
  }
}
//*******************************************************************
bool TStatCalcRoulette::GetValues(int statindex,float *values,int maxvalues)
{
  int num = GetMaxValueIndex(maxvalues);
  switch (statindex)
  {
    case rTotalNumBets      :return GetTotalNumBets(values,num);
    case rTotalCashBets     :return GetTotalCashBets(values,num);
    case rAvgCashBets       :return GetAvgCashBets(values,num);
    case rTotalCashWL       :return GetTotalCashWL(values,num);
    case rCumCashWL         :return GetTotalCashWL(values,num);
    case rBetsWon           :return GetWins(values,num);
    case rBetsLost          :return GetLosses(values,num);
    case rLargestCashWin    :return GetLargestCashWin(values,num);
    case rLargestCashLoss   :return GetLargestCashLost(values,num);
    case rLongestWinStreak  :return GetLongestWinStreaks(values,num);
    case rRedStreak         :return GetRedStreaks(values,num);
    case rBlackStreak       :return GetBlackStreaks(values,num);
    case rOddStreak         :return GetOddStreaks(values,num);
    case rEvenStreak        :return GetEvenStreaks(values,num);
    case rHighStreak        :return GetHighStreaks(values,num);
    case rLowStreak         :return GetLowStreaks(values,num);
    default                 :return false;
  };
//  return false;
}
//*******************************************************************
bool TStatCalcRoulette::GetRedStreaks(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetRSession(i)->RedStreak;
  }
  return true;
}
//*******************************************************************
bool TStatCalcRoulette::GetBlackStreaks(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetRSession(i)->BlackStreak;
  }
  return true;
}
//*******************************************************************
bool TStatCalcRoulette::GetOddStreaks(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetRSession(i)->OddStreak;
  }
  return true;
}
//*******************************************************************
bool TStatCalcRoulette::GetEvenStreaks(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetRSession(i)->EvenStreak;
  }
  return true;
}
//*******************************************************************
bool TStatCalcRoulette::GetHighStreaks(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetRSession(i)->HighStreak;
  }
  return true;
}
//*******************************************************************
bool TStatCalcRoulette::GetLowStreaks(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetRSession(i)->LowStreak;
  }
  return true;
}
//*******************************************************************
#pragma argsused
string TStatCalcRoulette::MakeSubGameTypeStr(int subgametype,TMoney gamedenom)
{
  string s = "Roulette";
  return s;
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalNumBets()
{
  Total_TotalNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalNumBets[i] = GetRSession(i)->SingleNumBets +
                      GetRSession(i)->TwoNumBets +
                      GetRSession(i)->ThreeNumBets +
                      GetRSession(i)->FourNumBets +
                      GetRSession(i)->FiveNumBets +
                      GetRSession(i)->SixNumBets +
                      GetRSession(i)->ColumnBets +
                      GetRSession(i)->DozenBets +
                      GetRSession(i)->RedBets +
                      GetRSession(i)->BlackBets +
                      GetRSession(i)->OddBets +
                      GetRSession(i)->EvenBets +
                      GetRSession(i)->HighBets +
			             GetRSession(i)->LowBets;
    Total_TotalNumBets += TotalNumBets[i];
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_CumulativeCashWLStrs()
{
  // use TotalCasWL
}
//*******************************************************************
void TStatCalcRoulette::Calc_CumulativeCashWL()
{
  // same as TotalCashWL
}
//*******************************************************************
void TStatCalcRoulette::Make_SingleNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    SingleNumBetsStr[i] = MakeString(GetRSession(i)->SingleNumBets);
  }
  Total_SingleNumBetsStr = MakeString(Total_SingleNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_SingleNumBets()
{
  Total_SingleNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_SingleNumBets += GetRSession(i)->SingleNumBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TwoNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoNumBetsStr[i] = MakeString(GetRSession(i)->TwoNumBets);
  }
  Total_TwoNumBetsStr = MakeString(Total_TwoNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TwoNumBets()
{
  Total_TwoNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoNumBets += GetRSession(i)->TwoNumBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_ThreeNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeNumBetsStr[i] = MakeString(GetRSession(i)->ThreeNumBets);
  }
  Total_ThreeNumBetsStr = MakeString(Total_ThreeNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_ThreeNumBets()
{
  Total_ThreeNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeNumBets += GetRSession(i)->ThreeNumBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FourNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourNumBetsStr[i] = MakeString(GetRSession(i)->FourNumBets);
  }
  Total_FourNumBetsStr = MakeString(Total_FourNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FourNumBets()
{
  Total_FourNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourNumBets += GetRSession(i)->FourNumBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FiveNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveNumBetsStr[i] = MakeString(GetRSession(i)->FiveNumBets);
  }
  Total_FiveNumBetsStr = MakeString(Total_FiveNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FiveNumBets()
{
  Total_FiveNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FiveNumBets += GetRSession(i)->FiveNumBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_SixNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    SixNumBetsStr[i] = MakeString(GetRSession(i)->SixNumBets);
  }
  Total_SixNumBetsStr = MakeString(Total_SixNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_SixNumBets()
{
  Total_SixNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_SixNumBets += GetRSession(i)->SixNumBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_ColumnBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ColumnBetsStr[i] = MakeString(GetRSession(i)->ColumnBets);
  }
  Total_ColumnBetsStr = MakeString(Total_ColumnBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_ColumnBets()
{
  Total_ColumnBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ColumnBets += GetRSession(i)->ColumnBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_DozenBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    DozenBetsStr[i] = MakeString(GetRSession(i)->DozenBets);
  }
  Total_DozenBetsStr = MakeString(Total_DozenBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_DozenBets()
{
  Total_DozenBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_DozenBets += GetRSession(i)->DozenBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_RedBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RedBetsStr[i] = MakeString(GetRSession(i)->RedBets);
  }
  Total_RedBetsStr = MakeString(Total_RedBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_RedBets()
{
  Total_RedBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RedBets += GetRSession(i)->RedBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_BlackBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    BlackBetsStr[i] = MakeString(GetRSession(i)->BlackBets);
  }
  Total_BlackBetsStr = MakeString(Total_BlackBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_BlackBets()
{
  Total_BlackBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_BlackBets += GetRSession(i)->BlackBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_OddBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    OddBetsStr[i] = MakeString(GetRSession(i)->OddBets);
  }
  Total_OddBetsStr = MakeString(Total_OddBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_OddBets()
{
  Total_OddBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_OddBets += GetRSession(i)->OddBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_EvenBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    EvenBetsStr[i] = MakeString(GetRSession(i)->EvenBets);
  }
  Total_EvenBetsStr = MakeString(Total_EvenBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_EvenBets()
{
  Total_EvenBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_EvenBets += GetRSession(i)->EvenBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_HighBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HighBetsStr[i] = MakeString(GetRSession(i)->HighBets);
  }
  Total_HighBetsStr = MakeString(Total_HighBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_HighBets()
{
  Total_HighBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_HighBets += GetRSession(i)->HighBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_LowBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LowBetsStr[i] = MakeString(GetRSession(i)->LowBets);
  }
  Total_LowBetsStr = MakeString(Total_LowBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_LowBets()
{
  Total_LowBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_LowBets += GetRSession(i)->LowBets;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_EvenMoneyBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    EvenMoneyBetsStr[i] = MakeString(EvenMoneyBets[i]);
  }
  Total_EvenMoneyBetsStr = MakeString(Total_EvenMoneyBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_EvenMoneyBets()
{
  Total_EvenMoneyBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    EvenMoneyBets[i] = GetRSession(i)->RedBets +
                       GetRSession(i)->BlackBets +
                       GetRSession(i)->OddBets +
                       GetRSession(i)->EvenBets +
                       GetRSession(i)->HighBets +
                       GetRSession(i)->LowBets;
    Total_EvenMoneyBets += EvenMoneyBets[i];
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLSingleNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLSingleNumBetsStr[i] = MakeString(GetRSession(i)->SingleNumBetsTotalCashWL);
  }
  Total_TotalCashWLSingleNumBetsStr = MakeString(Total_TotalCashWLSingleNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLSingleNumBets()
{
  Total_TotalCashWLSingleNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLSingleNumBets += (long)GetRSession(i)->SingleNumBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLTwoNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLTwoNumBetsStr[i] = MakeString(GetRSession(i)->TwoNumBetsTotalCashWL);
  }
  Total_TotalCashWLTwoNumBetsStr = MakeString(Total_TotalCashWLTwoNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLTwoNumBets()
{
  Total_TotalCashWLTwoNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLTwoNumBets += (long)GetRSession(i)->TwoNumBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLThreeNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLThreeNumBetsStr[i] = MakeString(GetRSession(i)->ThreeNumBetsTotalCashWL);
  }
  Total_TotalCashWLThreeNumBetsStr = MakeString(Total_TotalCashWLThreeNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLThreeNumBets()
{
  Total_TotalCashWLThreeNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLThreeNumBets += (long)GetRSession(i)->ThreeNumBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLFourNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLFourNumBetsStr[i] = MakeString(GetRSession(i)->FourNumBetsTotalCashWL);
  }
  Total_TotalCashWLFourNumBetsStr = MakeString(Total_TotalCashWLFourNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLFourNumBets()
{
  Total_TotalCashWLFourNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLFourNumBets += (long)GetRSession(i)->FourNumBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLFiveNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLFiveNumBetsStr[i] = MakeString(GetRSession(i)->FiveNumBetsTotalCashWL);
  }
  Total_TotalCashWLFiveNumBetsStr = MakeString(Total_TotalCashWLFiveNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLFiveNumBets()
{
  Total_TotalCashWLFiveNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLFiveNumBets += (long)GetRSession(i)->FiveNumBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLSixNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLSixNumBetsStr[i] = MakeString(GetRSession(i)->SixNumBetsTotalCashWL);
  }
  Total_TotalCashWLSixNumBetsStr = MakeString(Total_TotalCashWLSixNumBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLSixNumBets()
{
  Total_TotalCashWLSixNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLSixNumBets += (long)GetRSession(i)->SixNumBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLColumnBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLColumnBetsStr[i] = MakeString(GetRSession(i)->ColumnBetsTotalCashWL);
  }
  Total_TotalCashWLColumnBetsStr = MakeString(Total_TotalCashWLColumnBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLColumnBets()
{
  Total_TotalCashWLColumnBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLColumnBets += (long)GetRSession(i)->ColumnBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLDozenBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLDozenBetsStr[i] = MakeString(GetRSession(i)->DozenBetsTotalCashWL);
  }
  Total_TotalCashWLDozenBetsStr = MakeString(Total_TotalCashWLDozenBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLDozenBets()
{
  Total_TotalCashWLDozenBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLDozenBets += (long)GetRSession(i)->DozenBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLRedBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLRedBetsStr[i] = MakeString(GetRSession(i)->RedBetsTotalCashWL);
  }
  Total_TotalCashWLRedBetsStr = MakeString(Total_TotalCashWLRedBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLRedBets()
{
  Total_TotalCashWLRedBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLRedBets += (long)GetRSession(i)->RedBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLBlackBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLBlackBetsStr[i] = MakeString(GetRSession(i)->BlackBetsTotalCashWL);
  }
  Total_TotalCashWLBlackBetsStr = MakeString(Total_TotalCashWLBlackBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLBlackBets()
{
  Total_TotalCashWLBlackBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLBlackBets += (long)GetRSession(i)->BlackBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLOddBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLOddBetsStr[i] = MakeString(GetRSession(i)->OddBetsTotalCashWL);
  }
  Total_TotalCashWLOddBetsStr = MakeString(Total_TotalCashWLOddBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLOddBets()
{
  Total_TotalCashWLOddBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLOddBets += (long)GetRSession(i)->OddBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLEvenBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLEvenBetsStr[i] = MakeString(GetRSession(i)->EvenBetsTotalCashWL);
  }
  Total_TotalCashWLEvenBetsStr = MakeString(Total_TotalCashWLEvenBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLEvenBets()
{
  Total_TotalCashWLEvenBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLEvenBets += (long)GetRSession(i)->EvenBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLHighBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLHighBetsStr[i] = MakeString(GetRSession(i)->HighBetsTotalCashWL);
  }
  Total_TotalCashWLHighBetsStr = MakeString(Total_TotalCashWLHighBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLHighBets()
{
  Total_TotalCashWLHighBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLHighBets += (long)GetRSession(i)->HighBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLLowBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLLowBetsStr[i] = MakeString(GetRSession(i)->LowBetsTotalCashWL);
  }
  Total_TotalCashWLLowBetsStr = MakeString(Total_TotalCashWLLowBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLLowBets()
{
  Total_TotalCashWLLowBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWLLowBets += (long)GetRSession(i)->LowBetsTotalCashWL;
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TotalCashWLEvenMoneyBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLEvenMoneyBetsStr[i] = MakeString(TotalCashWLEvenMoneyBets[i]);
  }
  Total_TotalCashWLEvenMoneyBetsStr = MakeString(Total_TotalCashWLEvenMoneyBets);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TotalCashWLEvenMoneyBets()
{
  Total_TotalCashWLEvenMoneyBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWLEvenMoneyBets[i] = GetRSession(i)->RedBetsTotalCashWL +
                                  GetRSession(i)->BlackBetsTotalCashWL +
                                  GetRSession(i)->OddBetsTotalCashWL +
                                  GetRSession(i)->EvenBetsTotalCashWL +
                                  GetRSession(i)->HighBetsTotalCashWL +
                                  GetRSession(i)->LowBetsTotalCashWL;
    Total_TotalCashWLEvenMoneyBets += TotalCashWLEvenMoneyBets[i];
  }
}
//*******************************************************************
void TStatCalcRoulette::Calc_MostFreqNums()
{
  for (int nums = 0;nums < MAX_ROULETTE_SPOTS;nums++)
  {
    TotalMostFreqNums[nums] = 0;
  }

  for (int i=0;i<=LastSessionNum;i++)
  {
    for (int j=0;j<MAX_ROULETTE_SPOTS;j++)
    {
      TotalMostFreqNums[j] += GetRSession(i)->MostFrequentNums[j];
    }
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_MostFreqNum1Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum1Str[i] = MakeString(MostFreqNum1Index[i]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNum1Str = NAStr;
  }
  else
  {
    Total_MostFreqNum1Str = MakeString(Total_MostFreqNum1Index);
  }
}
//*******************************************************************
void TStatCalcRoulette::Calc_MostFreqNum1()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum1Index[i] = 0;
    for (int j=0;j<MAX_ROULETTE_SPOTS;j++)
    {
      if (GetRSession(i)->MostFrequentNums[j] >
          GetRSession(i)->MostFrequentNums[MostFreqNum1Index[i]])
      {
        MostFreqNum1Index[i] = j;
      }
    }
  }
  Total_MostFreqNum1Index = 0;
  for (int nums = 0;nums < MAX_ROULETTE_SPOTS;nums++)
  {
    if (TotalMostFreqNums[nums] > TotalMostFreqNums[Total_MostFreqNum1Index])
    {
      Total_MostFreqNum1Index = nums;
    }
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_MostFreqNum2Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum2Str[i] = MakeString(MostFreqNum2Index[i]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNum2Str = NAStr;
  }
  else
  {
    Total_MostFreqNum2Str = MakeString(Total_MostFreqNum2Index);
  }
}
//*******************************************************************
void TStatCalcRoulette::Calc_MostFreqNum2()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum2Index[i] = 0;
    for (int j=0;j<MAX_ROULETTE_SPOTS;j++)
    {
      if (j != MostFreqNum1Index[i])
      {
        if (GetRSession(i)->MostFrequentNums[j] >
            GetRSession(i)->MostFrequentNums[MostFreqNum2Index[i]])
        {
          MostFreqNum2Index[i] = j;
        }
      }
    }
  }
  Total_MostFreqNum2Index = 0;
  for (int nums = 0;nums < MAX_ROULETTE_SPOTS;nums++)
  {
    if (nums != Total_MostFreqNum1Index)
    {
      if (TotalMostFreqNums[nums] > TotalMostFreqNums[Total_MostFreqNum2Index])
      {
        Total_MostFreqNum2Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_MostFreqNum3Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum3Str[i] = MakeString(MostFreqNum3Index[i]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNum3Str = NAStr;
  }
  else
  {
    Total_MostFreqNum3Str = MakeString(Total_MostFreqNum3Index);
  }
}
//*******************************************************************
void TStatCalcRoulette::Calc_MostFreqNum3()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum3Index[i] = 0;
    for (int j=0;j<MAX_ROULETTE_SPOTS;j++)
    {
      if (j != MostFreqNum1Index[i] && j != MostFreqNum2Index[i])
      {
        if (GetRSession(i)->MostFrequentNums[j] >
            GetRSession(i)->MostFrequentNums[MostFreqNum3Index[i]])
        {
          MostFreqNum3Index[i] = j;
        }
      }
    }
  }
  Total_MostFreqNum3Index = 0;
  for (int nums = 0;nums < MAX_ROULETTE_SPOTS;nums++)
  {
    if (nums != Total_MostFreqNum1Index && nums != Total_MostFreqNum2Index)
    {
      if (TotalMostFreqNums[nums] > TotalMostFreqNums[Total_MostFreqNum3Index])
      {
        Total_MostFreqNum3Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_MostFreqNumOcc1Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNumOcc1Str[i] = MakeString(GetRSession(i)->MostFrequentNums[MostFreqNum1Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNumOcc1Str = NAStr;
  }
  else
  {
    Total_MostFreqNumOcc1Str = MakeString(TotalMostFreqNums[Total_MostFreqNum1Index]);
  }
}
//*******************************************************************
void TStatCalcRoulette::Calc_MostFreqNumOcc1()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcRoulette::Make_MostFreqNumOcc2Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNumOcc2Str[i] = MakeString(GetRSession(i)->MostFrequentNums[MostFreqNum2Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNumOcc2Str = NAStr;
  }
  else
  {
    Total_MostFreqNumOcc2Str = MakeString(TotalMostFreqNums[Total_MostFreqNum2Index]);
  }
}
//*******************************************************************
void TStatCalcRoulette::Calc_MostFreqNumOcc2()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcRoulette::Make_MostFreqNumOcc3Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNumOcc3Str[i] = MakeString(GetRSession(i)->MostFrequentNums[MostFreqNum3Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNumOcc3Str = NAStr;
  }
  else
  {
    Total_MostFreqNumOcc3Str = MakeString(TotalMostFreqNums[Total_MostFreqNum3Index]);
  }
}
//*******************************************************************
void TStatCalcRoulette::Calc_MostFreqNumOcc3()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcRoulette::Make_OneRedInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    OneRedInRowStr[i] = MakeString(GetRSession(i)->OneRedInRow);
  }
  Total_OneRedInRowStr = MakeString(Total_OneRedInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_OneRedInRow()
{
  Total_OneRedInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_OneRedInRow = (int)max(GetRSession(i)->OneRedInRow,Total_OneRedInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TwoRedInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoRedInRowStr[i] = MakeString(GetRSession(i)->TwoRedInRow);
  }
  Total_TwoRedInRowStr = MakeString(Total_TwoRedInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TwoRedInRow()
{
  Total_TwoRedInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoRedInRow = (int)max(GetRSession(i)->TwoRedInRow,Total_TwoRedInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_ThreeRedInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeRedInRowStr[i] = MakeString(GetRSession(i)->ThreeRedInRow);
  }
  Total_ThreeRedInRowStr = MakeString(Total_ThreeRedInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_ThreeRedInRow()
{
  Total_ThreeRedInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeRedInRow = (int)max(GetRSession(i)->ThreeRedInRow,Total_ThreeRedInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FourRedInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourRedInRowStr[i] = MakeString(GetRSession(i)->FourRedInRow);
  }
  Total_FourRedInRowStr = MakeString(Total_FourRedInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FourRedInRow()
{
  Total_FourRedInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourRedInRow = (int)max(GetRSession(i)->FourRedInRow,Total_FourRedInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FivePlusRedInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FivePlusRedInRowStr[i] = MakeString(GetRSession(i)->FivePlusRedInRow);
  }
  Total_FivePlusRedInRowStr = MakeString(Total_FivePlusRedInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FivePlusRedInRow()
{
  Total_FivePlusRedInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FivePlusRedInRow = (int)max(GetRSession(i)->FivePlusRedInRow,Total_FivePlusRedInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_RedStreakStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RedStreakStr[i] = MakeString(GetRSession(i)->RedStreak);
  }
  Total_RedStreakStr = MakeString(Total_RedStreak);
}
//*******************************************************************
void TStatCalcRoulette::Calc_RedStreak()
{
  Total_RedStreak = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RedStreak = (int)max(GetRSession(i)->RedStreak,Total_RedStreak);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_OneBlackInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    OneBlackInRowStr[i] = MakeString(GetRSession(i)->OneBlackInRow);
  }
  Total_OneBlackInRowStr = MakeString(Total_OneBlackInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_OneBlackInRow()
{
  Total_OneBlackInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_OneBlackInRow = (int)max(GetRSession(i)->OneBlackInRow,Total_OneBlackInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TwoBlackInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoBlackInRowStr[i] = MakeString(GetRSession(i)->TwoBlackInRow);
  }
  Total_TwoBlackInRowStr = MakeString(Total_TwoBlackInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TwoBlackInRow()
{
  Total_TwoBlackInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoBlackInRow = (int)max(GetRSession(i)->TwoBlackInRow,Total_TwoBlackInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_ThreeBlackInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeBlackInRowStr[i] = MakeString(GetRSession(i)->ThreeBlackInRow);
  }
  Total_ThreeBlackInRowStr = MakeString(Total_ThreeBlackInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_ThreeBlackInRow()
{
  Total_ThreeBlackInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeBlackInRow = (int)max(GetRSession(i)->ThreeBlackInRow,Total_ThreeBlackInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FourBlackInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourBlackInRowStr[i] = MakeString(GetRSession(i)->FourBlackInRow);
  }
  Total_FourBlackInRowStr = MakeString(Total_FourBlackInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FourBlackInRow()
{
  Total_FourBlackInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourBlackInRow = (int)max(GetRSession(i)->FourBlackInRow,Total_FourBlackInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FivePlusBlackInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FivePlusBlackInRowStr[i] = MakeString(GetRSession(i)->FivePlusBlackInRow);
  }
  Total_FivePlusBlackInRowStr = MakeString(Total_FivePlusBlackInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FivePlusBlackInRow()
{
  Total_FivePlusBlackInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FivePlusBlackInRow = (int)max(GetRSession(i)->FivePlusBlackInRow,Total_FivePlusBlackInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_BlackStreakStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    BlackStreakStr[i] = MakeString(GetRSession(i)->BlackStreak);
  }
  Total_BlackStreakStr = MakeString(Total_BlackStreak);
}
//*******************************************************************
void TStatCalcRoulette::Calc_BlackStreak()
{
  Total_BlackStreak = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_BlackStreak = (int)max(GetRSession(i)->BlackStreak,Total_BlackStreak);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_OneOddInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    OneOddInRowStr[i] = MakeString(GetRSession(i)->OneOddInRow);
  }
  Total_OneOddInRowStr = MakeString(Total_OneOddInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_OneOddInRow()
{
  Total_OneOddInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_OneOddInRow = (int)max(GetRSession(i)->OneOddInRow,Total_OneOddInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TwoOddInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoOddInRowStr[i] = MakeString(GetRSession(i)->TwoOddInRow);
  }
  Total_TwoOddInRowStr = MakeString(Total_TwoOddInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TwoOddInRow()
{
  Total_TwoOddInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoOddInRow = (int)max(GetRSession(i)->TwoOddInRow,Total_TwoOddInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_ThreeOddInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeOddInRowStr[i] = MakeString(GetRSession(i)->ThreeOddInRow);
  }
  Total_ThreeOddInRowStr = MakeString(Total_ThreeOddInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_ThreeOddInRow()
{
  Total_ThreeOddInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeOddInRow = (int)max(GetRSession(i)->ThreeOddInRow,Total_ThreeOddInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FourOddInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOddInRowStr[i] = MakeString(GetRSession(i)->FourOddInRow);
  }
  Total_FourOddInRowStr = MakeString(Total_FourOddInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FourOddInRow()
{
  Total_FourOddInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOddInRow = (int)max(GetRSession(i)->FourOddInRow,Total_FourOddInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FivePlusOddInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FivePlusOddInRowStr[i] = MakeString(GetRSession(i)->FivePlusOddInRow);
  }
  Total_FivePlusOddInRowStr = MakeString(Total_FivePlusOddInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FivePlusOddInRow()
{
  Total_FivePlusOddInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FivePlusOddInRow = (int)max(GetRSession(i)->FivePlusOddInRow,Total_FivePlusOddInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_OddStreakStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    OddStreakStr[i] = MakeString(GetRSession(i)->OddStreak);
  }
  Total_OddStreakStr = MakeString(Total_OddStreak);
}
//*******************************************************************
void TStatCalcRoulette::Calc_OddStreak()
{
  Total_OddStreak = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_OddStreak = (int)max(GetRSession(i)->OddStreak,Total_OddStreak);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_OneEvenInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    OneEvenInRowStr[i] = MakeString(GetRSession(i)->OneEvenInRow);
  }
  Total_OneEvenInRowStr = MakeString(Total_OneEvenInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_OneEvenInRow()
{
  Total_OneEvenInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_OneEvenInRow = (int)max(GetRSession(i)->OneEvenInRow,Total_OneEvenInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TwoEvenInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoEvenInRowStr[i] = MakeString(GetRSession(i)->TwoEvenInRow);
  }
  Total_TwoEvenInRowStr = MakeString(Total_TwoEvenInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TwoEvenInRow()
{
  Total_TwoEvenInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoEvenInRow = (int)max(GetRSession(i)->TwoEvenInRow,Total_TwoEvenInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_ThreeEvenInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeEvenInRowStr[i] = MakeString(GetRSession(i)->ThreeEvenInRow);
  }
  Total_ThreeEvenInRowStr = MakeString(Total_ThreeEvenInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_ThreeEvenInRow()
{
  Total_ThreeEvenInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeEvenInRow = (int)max(GetRSession(i)->ThreeEvenInRow,Total_ThreeEvenInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FourEvenInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourEvenInRowStr[i] = MakeString(GetRSession(i)->FourEvenInRow);
  }
  Total_FourEvenInRowStr = MakeString(Total_FourEvenInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FourEvenInRow()
{
  Total_FourEvenInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourEvenInRow = (int)max(GetRSession(i)->FourEvenInRow,Total_FourEvenInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FivePlusEvenInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FivePlusEvenInRowStr[i] = MakeString(GetRSession(i)->FivePlusEvenInRow);
  }
  Total_FivePlusEvenInRowStr = MakeString(Total_FivePlusEvenInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FivePlusEvenInRow()
{
  Total_FivePlusEvenInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FivePlusEvenInRow = (int)max(GetRSession(i)->FivePlusEvenInRow,Total_FivePlusEvenInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_EvenStreakStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    EvenStreakStr[i] = MakeString(GetRSession(i)->EvenStreak);
  }
  Total_EvenStreakStr = MakeString(Total_EvenStreak);
}
//*******************************************************************
void TStatCalcRoulette::Calc_EvenStreak()
{
  Total_EvenStreak = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_EvenStreak = (int)max(GetRSession(i)->EvenStreak,Total_EvenStreak);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_OneHighInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    OneHighInRowStr[i] = MakeString(GetRSession(i)->OneHighInRow);
  }
  Total_OneHighInRowStr = MakeString(Total_OneHighInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_OneHighInRow()
{
  Total_OneHighInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_OneHighInRow = (int)max(GetRSession(i)->OneHighInRow,Total_OneHighInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TwoHighInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoHighInRowStr[i] = MakeString(GetRSession(i)->TwoHighInRow);
  }
  Total_TwoHighInRowStr = MakeString(Total_TwoHighInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TwoHighInRow()
{
  Total_TwoHighInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoHighInRow = (int)max(GetRSession(i)->TwoHighInRow,Total_TwoHighInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_ThreeHighInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeHighInRowStr[i] = MakeString(GetRSession(i)->ThreeHighInRow);
  }
  Total_ThreeHighInRowStr = MakeString(Total_ThreeHighInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_ThreeHighInRow()
{
  Total_ThreeHighInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeHighInRow = (int)max(GetRSession(i)->ThreeHighInRow,Total_ThreeHighInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FourHighInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourHighInRowStr[i] = MakeString(GetRSession(i)->FourHighInRow);
  }
  Total_FourHighInRowStr = MakeString(Total_FourHighInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FourHighInRow()
{
  Total_FourHighInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourHighInRow = (int)max(GetRSession(i)->FourHighInRow,Total_FourHighInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FivePlusHighInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FivePlusHighInRowStr[i] = MakeString(GetRSession(i)->FivePlusHighInRow);
  }
  Total_FivePlusHighInRowStr = MakeString(Total_FivePlusHighInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FivePlusHighInRow()
{
  Total_FivePlusHighInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FivePlusHighInRow = (int)max(GetRSession(i)->FivePlusHighInRow,Total_FivePlusHighInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_HighStreakStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HighStreakStr[i] = MakeString(GetRSession(i)->HighStreak);
  }
  Total_HighStreakStr = MakeString(Total_HighStreak);
}
//*******************************************************************
void TStatCalcRoulette::Calc_HighStreak()
{
  Total_HighStreak = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_HighStreak = (int)max(GetRSession(i)->HighStreak,Total_HighStreak);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_OneLowInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    OneLowInRowStr[i] = MakeString(GetRSession(i)->OneLowInRow);
  }
  Total_OneLowInRowStr = MakeString(Total_OneLowInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_OneLowInRow()
{
  Total_OneLowInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_OneLowInRow = (int)max(GetRSession(i)->OneLowInRow,Total_OneLowInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_TwoLowInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoLowInRowStr[i] = MakeString(GetRSession(i)->TwoLowInRow);
  }
  Total_TwoLowInRowStr = MakeString(Total_TwoLowInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_TwoLowInRow()
{
  Total_TwoLowInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoLowInRow = (int)max(GetRSession(i)->TwoLowInRow,Total_TwoLowInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_ThreeLowInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeLowInRowStr[i] = MakeString(GetRSession(i)->ThreeLowInRow);
  }
  Total_ThreeLowInRowStr = MakeString(Total_ThreeLowInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_ThreeLowInRow()
{
  Total_ThreeLowInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeLowInRow = (int)max(GetRSession(i)->ThreeLowInRow,Total_ThreeLowInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FourLowInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourLowInRowStr[i] = MakeString(GetRSession(i)->FourLowInRow);
  }
  Total_FourLowInRowStr = MakeString(Total_FourLowInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FourLowInRow()
{
  Total_FourLowInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourLowInRow = (int)max(GetRSession(i)->FourLowInRow,Total_FourLowInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_FivePlusLowInRowStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FivePlusLowInRowStr[i] = MakeString(GetRSession(i)->FivePlusLowInRow);
  }
  Total_FivePlusLowInRowStr = MakeString(Total_FivePlusLowInRow);
}
//*******************************************************************
void TStatCalcRoulette::Calc_FivePlusLowInRow()
{
  Total_FivePlusLowInRow = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FivePlusLowInRow = (int)max(GetRSession(i)->FivePlusLowInRow,Total_FivePlusLowInRow);
  }
}
//*******************************************************************
void TStatCalcRoulette::Make_LowStreakStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LowStreakStr[i] = MakeString(GetRSession(i)->LowStreak);
  }
  Total_LowStreakStr = MakeString(Total_LowStreak);
}
//*******************************************************************
void TStatCalcRoulette::Calc_LowStreak()
{
  Total_LowStreak = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_LowStreak = (int)max(GetRSession(i)->LowStreak,Total_LowStreak);
  }
}
//*******************************************************************
void TStatCalcRoulette::LocalCalculate()
{
  TCasinoStatCalc::LocalCalculate();

  if (PPlayerData == NULL) {return;}
  Calc_CumulativeCashWL();
  Calc_SingleNumBets();
  Calc_TwoNumBets();
  Calc_ThreeNumBets();
  Calc_FourNumBets();
  Calc_FiveNumBets();
  Calc_SixNumBets();
  Calc_ColumnBets();
  Calc_DozenBets();
  Calc_RedBets();
  Calc_BlackBets();
  Calc_OddBets();
  Calc_EvenBets();
  Calc_HighBets();
  Calc_LowBets();
  Calc_EvenMoneyBets();
  Calc_TotalCashWLSingleNumBets();
  Calc_TotalCashWLTwoNumBets();
  Calc_TotalCashWLThreeNumBets();
  Calc_TotalCashWLFourNumBets();
  Calc_TotalCashWLFiveNumBets();
  Calc_TotalCashWLSixNumBets();
  Calc_TotalCashWLColumnBets();
  Calc_TotalCashWLDozenBets();
  Calc_TotalCashWLRedBets();
  Calc_TotalCashWLBlackBets();
  Calc_TotalCashWLOddBets();
  Calc_TotalCashWLEvenBets();
  Calc_TotalCashWLHighBets();
  Calc_TotalCashWLLowBets();
  Calc_TotalCashWLEvenMoneyBets();
  Calc_MostFreqNums();
  Calc_MostFreqNum1();
  Calc_MostFreqNum2();
  Calc_MostFreqNum3();
  Calc_MostFreqNumOcc1();
  Calc_MostFreqNumOcc2();
  Calc_MostFreqNumOcc3();
  Calc_OneRedInRow();
  Calc_TwoRedInRow();
  Calc_ThreeRedInRow();
  Calc_FourRedInRow();
  Calc_FivePlusRedInRow();
  Calc_RedStreak();
  Calc_OneBlackInRow();
  Calc_TwoBlackInRow();
  Calc_ThreeBlackInRow();
  Calc_FourBlackInRow();
  Calc_FivePlusBlackInRow();
  Calc_BlackStreak();
  Calc_OneOddInRow();
  Calc_TwoOddInRow();
  Calc_ThreeOddInRow();
  Calc_FourOddInRow();
  Calc_FivePlusOddInRow();
  Calc_OddStreak();
  Calc_OneEvenInRow();
  Calc_TwoEvenInRow();
  Calc_ThreeEvenInRow();
  Calc_FourEvenInRow();
  Calc_FivePlusEvenInRow();
  Calc_EvenStreak();
  Calc_OneHighInRow();
  Calc_TwoHighInRow();
  Calc_ThreeHighInRow();
  Calc_FourHighInRow();
  Calc_FivePlusHighInRow();
  Calc_HighStreak();
  Calc_OneLowInRow();
  Calc_TwoLowInRow();
  Calc_ThreeLowInRow();
  Calc_FourLowInRow();
  Calc_FivePlusLowInRow();
  Calc_LowStreak();
}
//*******************************************************************
void TStatCalcRoulette::LocalMakeAllStrings()
{
  TCasinoStatCalc::LocalMakeAllStrings();

  if (PPlayerData == NULL) {return;}

  Make_CumulativeCashWLStrs();
  Make_SingleNumBetsStrs();
  Make_TwoNumBetsStrs();
  Make_ThreeNumBetsStrs();
  Make_FourNumBetsStrs();
  Make_FiveNumBetsStrs();
  Make_SixNumBetsStrs();
  Make_ColumnBetsStrs();
  Make_DozenBetsStrs();
  Make_RedBetsStrs();
  Make_BlackBetsStrs();
  Make_OddBetsStrs();
  Make_EvenBetsStrs();
  Make_HighBetsStrs();
  Make_LowBetsStrs();
  Make_EvenMoneyBetsStrs();
  Make_TotalCashWLSingleNumBetsStrs();
  Make_TotalCashWLTwoNumBetsStrs();
  Make_TotalCashWLThreeNumBetsStrs();
  Make_TotalCashWLFourNumBetsStrs();
  Make_TotalCashWLFiveNumBetsStrs();
  Make_TotalCashWLSixNumBetsStrs();
  Make_TotalCashWLColumnBetsStrs();
  Make_TotalCashWLDozenBetsStrs();
  Make_TotalCashWLRedBetsStrs();
  Make_TotalCashWLBlackBetsStrs();
  Make_TotalCashWLOddBetsStrs();
  Make_TotalCashWLEvenBetsStrs();
  Make_TotalCashWLHighBetsStrs();
  Make_TotalCashWLLowBetsStrs();
  Make_TotalCashWLEvenMoneyBetsStrs();
  Make_LongestWinStreakStrs();
  Make_LongestLosingStreakStrs();
  Make_MostFreqNum1Strs();
  Make_MostFreqNum2Strs();
  Make_MostFreqNum3Strs();
  Make_MostFreqNumOcc1Strs();
  Make_MostFreqNumOcc2Strs();
  Make_MostFreqNumOcc3Strs();
  Make_OneRedInRowStrs();
  Make_TwoRedInRowStrs();
  Make_ThreeRedInRowStrs();
  Make_FourRedInRowStrs();
  Make_FivePlusRedInRowStrs();
  Make_RedStreakStrs();
  Make_OneBlackInRowStrs();
  Make_TwoBlackInRowStrs();
  Make_ThreeBlackInRowStrs();
  Make_FourBlackInRowStrs();
  Make_FivePlusBlackInRowStrs();
  Make_BlackStreakStrs();
  Make_OneOddInRowStrs();
  Make_TwoOddInRowStrs();
  Make_ThreeOddInRowStrs();
  Make_FourOddInRowStrs();
  Make_FivePlusOddInRowStrs();
  Make_OddStreakStrs();
  Make_OneEvenInRowStrs();
  Make_TwoEvenInRowStrs();
  Make_ThreeEvenInRowStrs();
  Make_FourEvenInRowStrs();
  Make_FivePlusEvenInRowStrs();
  Make_EvenStreakStrs();
  Make_OneHighInRowStrs();
  Make_TwoHighInRowStrs();
  Make_ThreeHighInRowStrs();
  Make_FourHighInRowStrs();
  Make_FivePlusHighInRowStrs();
  Make_HighStreakStrs();
  Make_OneLowInRowStrs();
  Make_TwoLowInRowStrs();
  Make_ThreeLowInRowStrs();
  Make_FourLowInRowStrs();
  Make_FivePlusLowInRowStrs();
  Make_LowStreakStrs();
}
//*******************************************************************
void TStatCalcRoulette::LocalFillGrid()
{
  TCasinoStatCalc::LocalFillGrid();

  if (PPlayerData == NULL) {return;}

  for(int col = 0;col <= LastSessionNum;col++)
  {
    int c = col + 1;
    PStatDlg->cell[1][c] = DateStr[col];
    PStatDlg->cell[2][c] = HoursPlayedStr[col];
    PStatDlg->cell[3][c] = TypeGamePlayedStr[col];
    PStatDlg->cell[6][c] = TotalNumBetsStr[col];
    PStatDlg->cell[7][c] = TotalCashBetsStr[col];
    PStatDlg->cell[8][c] = AvgCashBetsStr[col];
    PStatDlg->cell[9][c] = StartingBankrollStr[col];
    PStatDlg->cell[10][c] = EndingBankrollStr[col];
    PStatDlg->cell[11][c] = TotalCashWLStr[col];
    PStatDlg->cell[12][c] = ReturnPercStr[col];
    PStatDlg->cell[13][c] = AvgWLPerHourStr[col];
	 PStatDlg->cell[14][c] = TotalCashWLStr[col];
    PStatDlg->cell[17][c] = TotalNumBetsStr[col];
    PStatDlg->cell[18][c] = HandsWonStr[col];
    PStatDlg->cell[19][c] = HandsLostStr[col];
    PStatDlg->cell[20][c] = PercHandsWonStr[col];
    PStatDlg->cell[21][c] = PercHandsLostStr[col];
    PStatDlg->cell[22][c] = LargestCashWinStr[col];
    PStatDlg->cell[23][c] = LargestCashLostStr[col];
    PStatDlg->cell[24][c] = SingleNumBetsStr[col];
    PStatDlg->cell[25][c] = TwoNumBetsStr[col];
    PStatDlg->cell[26][c] = ThreeNumBetsStr[col];
    PStatDlg->cell[27][c] = FourNumBetsStr[col];
    PStatDlg->cell[28][c] = FiveNumBetsStr[col];
    PStatDlg->cell[29][c] = SixNumBetsStr[col];
    PStatDlg->cell[30][c] = ColumnBetsStr[col];
    PStatDlg->cell[31][c] = DozenBetsStr[col];
    PStatDlg->cell[32][c] = EvenMoneyBetsStr[col];
    PStatDlg->cell[33][c] = RedBetsStr[col];
    PStatDlg->cell[34][c] = BlackBetsStr[col];
    PStatDlg->cell[35][c] = OddBetsStr[col];
    PStatDlg->cell[36][c] = EvenBetsStr[col];
    PStatDlg->cell[37][c] = HighBetsStr[col];
    PStatDlg->cell[38][c] = LowBetsStr[col];
    PStatDlg->cell[41][c] = TotalCashWLSingleNumBetsStr[col];
    PStatDlg->cell[42][c] = TotalCashWLTwoNumBetsStr[col];
    PStatDlg->cell[43][c] = TotalCashWLThreeNumBetsStr[col];
    PStatDlg->cell[44][c] = TotalCashWLFourNumBetsStr[col];
    PStatDlg->cell[45][c] = TotalCashWLFiveNumBetsStr[col];
    PStatDlg->cell[46][c] = TotalCashWLSixNumBetsStr[col];
	 PStatDlg->cell[47][c] = TotalCashWLColumnBetsStr[col];
    PStatDlg->cell[48][c] = TotalCashWLDozenBetsStr[col];
    PStatDlg->cell[49][c] = TotalCashWLEvenMoneyBetsStr[col];
    PStatDlg->cell[50][c] = TotalCashWLRedBetsStr[col];
    PStatDlg->cell[51][c] = TotalCashWLBlackBetsStr[col];
    PStatDlg->cell[52][c] = TotalCashWLOddBetsStr[col];
    PStatDlg->cell[53][c] = TotalCashWLEvenBetsStr[col];
    PStatDlg->cell[54][c] = TotalCashWLHighBetsStr[col];
    PStatDlg->cell[55][c] = TotalCashWLLowBetsStr[col];
    PStatDlg->cell[58][c] = LongestWinStreakStr[col];
    PStatDlg->cell[59][c] = LongestLosingStreakStr[col];
    PStatDlg->cell[60][c] = MostFreqNum1Str[col];
    PStatDlg->cell[61][c] = MostFreqNumOcc1Str[col];
    PStatDlg->cell[62][c] = MostFreqNum2Str[col];
    PStatDlg->cell[63][c] = MostFreqNumOcc2Str[col];
    PStatDlg->cell[64][c] = MostFreqNum3Str[col];
    PStatDlg->cell[65][c] = MostFreqNumOcc3Str[col];
    PStatDlg->cell[67][c] = OneRedInRowStr[col];
    PStatDlg->cell[68][c] = TwoRedInRowStr[col];
    PStatDlg->cell[69][c] = ThreeRedInRowStr[col];
    PStatDlg->cell[70][c] = FourRedInRowStr[col];
    PStatDlg->cell[71][c] = FivePlusRedInRowStr[col];
    PStatDlg->cell[72][c] = RedStreakStr[col];
    PStatDlg->cell[74][c] = OneBlackInRowStr[col];
    PStatDlg->cell[75][c] = TwoBlackInRowStr[col];
    PStatDlg->cell[76][c] = ThreeBlackInRowStr[col];
    PStatDlg->cell[77][c] = FourBlackInRowStr[col];
    PStatDlg->cell[78][c] = FivePlusBlackInRowStr[col];
    PStatDlg->cell[79][c] = BlackStreakStr[col];
	 PStatDlg->cell[81][c] = OneOddInRowStr[col];
    PStatDlg->cell[82][c] = TwoOddInRowStr[col];
    PStatDlg->cell[83][c] = ThreeOddInRowStr[col];
    PStatDlg->cell[84][c] = FourOddInRowStr[col];
    PStatDlg->cell[85][c] = FivePlusOddInRowStr[col];
    PStatDlg->cell[86][c] = OddStreakStr[col];
    PStatDlg->cell[88][c] = OneEvenInRowStr[col];
    PStatDlg->cell[89][c] = TwoEvenInRowStr[col];
    PStatDlg->cell[90][c] = ThreeEvenInRowStr[col];
    PStatDlg->cell[91][c] = FourEvenInRowStr[col];
    PStatDlg->cell[92][c] = FivePlusEvenInRowStr[col];
    PStatDlg->cell[93][c] = EvenStreakStr[col];
    PStatDlg->cell[95][c] = OneHighInRowStr[col];
    PStatDlg->cell[96][c] = TwoHighInRowStr[col];
    PStatDlg->cell[97][c] = ThreeHighInRowStr[col];
    PStatDlg->cell[98][c] = FourHighInRowStr[col];
    PStatDlg->cell[99][c] = FivePlusHighInRowStr[col];
    PStatDlg->cell[100][c] = HighStreakStr[col];
    PStatDlg->cell[102][c] = OneLowInRowStr[col];
    PStatDlg->cell[103][c] = TwoLowInRowStr[col];
    PStatDlg->cell[104][c] = ThreeLowInRowStr[col];
    PStatDlg->cell[105][c] = FourLowInRowStr[col];
    PStatDlg->cell[106][c] = FivePlusLowInRowStr[col];
    PStatDlg->cell[107][c] = LowStreakStr[col];
  }

  PStatDlg->totalcol[1] = Total_DateStr;
  PStatDlg->totalcol[2] = Total_HoursPlayedStr;
  PStatDlg->totalcol[3] = Total_TypeGamePlayedStr;
  PStatDlg->totalcol[6] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[7] = Total_TotalCashBetsStr;
  PStatDlg->totalcol[8] = Total_AvgCashBetsStr;
  PStatDlg->totalcol[9] = Total_StartingBankrollStr;
  PStatDlg->totalcol[10] = Total_EndingBankrollStr;
  PStatDlg->totalcol[11] = Total_TotalCashWLStr;
  PStatDlg->totalcol[12] = Total_ReturnPercStr;
  PStatDlg->totalcol[13] = Total_AvgWLPerHourStr;
  PStatDlg->totalcol[14] = Total_TotalCashWLStr;
  PStatDlg->totalcol[17] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[18] = Total_HandsWonStr;
  PStatDlg->totalcol[19] = Total_HandsLostStr;
  PStatDlg->totalcol[20] = Total_PercHandsWonStr;
  PStatDlg->totalcol[21] = Total_PercHandsLostStr;
  PStatDlg->totalcol[22] = Total_LargestCashWinStr;
  PStatDlg->totalcol[23] = Total_LargestCashLostStr;
  PStatDlg->totalcol[24] = Total_SingleNumBetsStr;
  PStatDlg->totalcol[25] = Total_TwoNumBetsStr;
  PStatDlg->totalcol[26] = Total_ThreeNumBetsStr;
  PStatDlg->totalcol[27] = Total_FourNumBetsStr;
  PStatDlg->totalcol[28] = Total_FiveNumBetsStr;
  PStatDlg->totalcol[29] = Total_SixNumBetsStr;
  PStatDlg->totalcol[30] = Total_ColumnBetsStr;
  PStatDlg->totalcol[31] = Total_DozenBetsStr;
  PStatDlg->totalcol[32] = Total_EvenMoneyBetsStr;
  PStatDlg->totalcol[33] = Total_RedBetsStr;
  PStatDlg->totalcol[34] = Total_BlackBetsStr;
  PStatDlg->totalcol[35] = Total_OddBetsStr;
  PStatDlg->totalcol[36] = Total_EvenBetsStr;
  PStatDlg->totalcol[37] = Total_HighBetsStr;
  PStatDlg->totalcol[38] = Total_LowBetsStr;
  PStatDlg->totalcol[41] = Total_TotalCashWLSingleNumBetsStr;
  PStatDlg->totalcol[42] = Total_TotalCashWLTwoNumBetsStr;
  PStatDlg->totalcol[43] = Total_TotalCashWLThreeNumBetsStr;
  PStatDlg->totalcol[44] = Total_TotalCashWLFourNumBetsStr;
  PStatDlg->totalcol[45] = Total_TotalCashWLFiveNumBetsStr;
  PStatDlg->totalcol[46] = Total_TotalCashWLSixNumBetsStr;
  PStatDlg->totalcol[47] = Total_TotalCashWLColumnBetsStr;
  PStatDlg->totalcol[48] = Total_TotalCashWLDozenBetsStr;
  PStatDlg->totalcol[49] = Total_TotalCashWLEvenMoneyBetsStr;
  PStatDlg->totalcol[50] = Total_TotalCashWLRedBetsStr;
  PStatDlg->totalcol[51] = Total_TotalCashWLBlackBetsStr;
  PStatDlg->totalcol[52] = Total_TotalCashWLOddBetsStr;
  PStatDlg->totalcol[53] = Total_TotalCashWLEvenBetsStr;
  PStatDlg->totalcol[54] = Total_TotalCashWLHighBetsStr;
  PStatDlg->totalcol[55] = Total_TotalCashWLLowBetsStr;
  PStatDlg->totalcol[58] = Total_LongestWinStreakStr;
  PStatDlg->totalcol[59] = Total_LongestLosingStreakStr;
  PStatDlg->totalcol[60] = Total_MostFreqNum1Str;
  PStatDlg->totalcol[61] = Total_MostFreqNumOcc1Str;
  PStatDlg->totalcol[62] = Total_MostFreqNum2Str;
  PStatDlg->totalcol[63] = Total_MostFreqNumOcc2Str;
  PStatDlg->totalcol[64] = Total_MostFreqNum3Str;
  PStatDlg->totalcol[65] = Total_MostFreqNumOcc3Str;
  PStatDlg->totalcol[67] = Total_OneRedInRowStr;
  PStatDlg->totalcol[68] = Total_TwoRedInRowStr;
  PStatDlg->totalcol[69] = Total_ThreeRedInRowStr;
  PStatDlg->totalcol[70] = Total_FourRedInRowStr;
  PStatDlg->totalcol[71] = Total_FivePlusRedInRowStr;
  PStatDlg->totalcol[72] = Total_RedStreakStr;
  PStatDlg->totalcol[74] = Total_OneBlackInRowStr;
  PStatDlg->totalcol[75] = Total_TwoBlackInRowStr;
  PStatDlg->totalcol[76] = Total_ThreeBlackInRowStr;
  PStatDlg->totalcol[77] = Total_FourBlackInRowStr;
  PStatDlg->totalcol[78] = Total_FivePlusBlackInRowStr;
  PStatDlg->totalcol[79] = Total_BlackStreakStr;
  PStatDlg->totalcol[81] = Total_OneOddInRowStr;
  PStatDlg->totalcol[82] = Total_TwoOddInRowStr;
  PStatDlg->totalcol[83] = Total_ThreeOddInRowStr;
  PStatDlg->totalcol[84] = Total_FourOddInRowStr;
  PStatDlg->totalcol[85] = Total_FivePlusOddInRowStr;
  PStatDlg->totalcol[86] = Total_OddStreakStr;
  PStatDlg->totalcol[88] = Total_OneEvenInRowStr;
  PStatDlg->totalcol[89] = Total_TwoEvenInRowStr;
  PStatDlg->totalcol[90] = Total_ThreeEvenInRowStr;
  PStatDlg->totalcol[91] = Total_FourEvenInRowStr;
  PStatDlg->totalcol[92] = Total_FivePlusEvenInRowStr;
  PStatDlg->totalcol[93] = Total_EvenStreakStr;
  PStatDlg->totalcol[95] = Total_OneHighInRowStr;
  PStatDlg->totalcol[96] = Total_TwoHighInRowStr;
  PStatDlg->totalcol[97] = Total_ThreeHighInRowStr;
  PStatDlg->totalcol[98] = Total_FourHighInRowStr;
  PStatDlg->totalcol[99] = Total_FivePlusHighInRowStr;
  PStatDlg->totalcol[100] = Total_HighStreakStr;
  PStatDlg->totalcol[102] = Total_OneLowInRowStr;
  PStatDlg->totalcol[103] = Total_TwoLowInRowStr;
  PStatDlg->totalcol[104] = Total_ThreeLowInRowStr;
  PStatDlg->totalcol[105] = Total_FourLowInRowStr;
  PStatDlg->totalcol[106] = Total_FivePlusLowInRowStr;
  PStatDlg->totalcol[107] = Total_LowStreakStr;
}
//*******************************************************************
//*******************************************************************
TRouletteGraph::TRouletteGraph(TCasinoStatDlg* pstatdlg,TStatCalcRoulette *pstatcalc)
:TCasinoGraphDlg(pstatdlg,pstatcalc)
{
  strName = "Roulette";
  BaseStringID = RouletteLabel;

  StatTT[0] = TStatCalcRoulette::rTotalNumBets;
  StatTT[1] = TStatCalcRoulette::rTotalCashBets;
  StatTT[2] = TStatCalcRoulette::rAvgCashBets;
  StatTT[3] = TStatCalcRoulette::rTotalCashWL;
  StatTT[4] = TStatCalcRoulette::rCumCashWL;
  StatTT[5] = TStatCalcRoulette::rBetsWon;
  StatTT[6] = TStatCalcRoulette::rBetsLost;
  StatTT[7] = TStatCalcRoulette::rLargestCashWin;
  StatTT[8] = TStatCalcRoulette::rLargestCashLoss;
  StatTT[9] = TStatCalcRoulette::rLongestWinStreak;
  StatTT[10] = TStatCalcRoulette::rRedStreak;
  StatTT[11] = TStatCalcRoulette::rBlackStreak;
  StatTT[12] = TStatCalcRoulette::rOddStreak;
  StatTT[13] = TStatCalcRoulette::rEvenStreak;
  StatTT[14] = TStatCalcRoulette::rHighStreak;
  StatTT[15] = TStatCalcRoulette::rLowStreak;

  LastStatIndex = 15;
}
//*******************************************************************
TRouletteGraph::~TRouletteGraph()
{
}
//*******************************************************************
