//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cscalc.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/6/96     M.Perry     derived from Dave's code
//  2.0     9/16/97    M.Perry     32-bit conversion.
//********************************************************************
#include "cscalc.h"
#include "cstatdlg.h"
#include <stdio.h>
#include <classlib\time.h>

//*******************************************************************
TStatCalcSlots::TStatCalcSlots(TCasinoStatDlg *pstatdlg)
:TCasinoStatCalc(pstatdlg)
{
}
//*******************************************************************
TStatCalcSlots::~TStatCalcSlots()
{
}
//*******************************************************************
void TStatCalcSlots::GetSessionPtrs()
{
  if (PPlayerData == NULL) {return;}
  PData = &(PPlayerData->SData);

  LastSessionNum = -1;
  for (int sessioniterator = GetSData()->GetFirstValidSession();
       sessioniterator != -1;
       sessioniterator = GetSData()->GetNextValidSession())
  {
    LastSessionNum++;
    PSessions[LastSessionNum] = &(GetSData()->Sessions[sessioniterator]);
  }
}
//*******************************************************************
bool TStatCalcSlots::GetValues(int statindex,float *values,int maxvalues)
{
  int num = GetMaxValueIndex(maxvalues);
  switch (statindex)
  {
    case sTotalNumBets        :return GetTotalNumBets(values,num);
    case sTotalCashBets       :return GetTotalCashBets(values,num);
    case sAvgCashBets         :return GetAvgCashBets(values,num);
    case sTotalCashWL         :return GetTotalCashWL(values,num);
    case sCumCashWL           :return GetTotalCashWL(values,num);
    case sSpinsWon            :return GetWins(values,num);
    case sSpinsLost           :return GetLosses(values,num);
    case sPercWon             :return GetPercWins(values,num);
    case sPercLost            :return GetPercLosses(values,num);
    case sLongestWinStreak    :return GetLongestWinStreaks(values,num);
    case sLongestLosingStreak :return GetLongestLosingStreak(values,num);
    case sLargestCashWin      :return GetLargestCashWin(values,num);
    case sPeakCashWinLevel    :return GetPeakWinLevel(values,num);
    case sLowCashLossLevel    :return GetLowCashLostLevel(values,num);
    case sLowPayoffs          :return GetLowPayoffs(values,num);
    case sHighPayoffs         :return GetHighPayoffs(values,num);
    default                   :return false;
  };
//  return false;
}
//*******************************************************************
bool TStatCalcSlots::GetLowPayoffs(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetSSession(i)->nLowPayoffs;
  }
  return true;
}
//*******************************************************************
bool TStatCalcSlots::GetHighPayoffs(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetSSession(i)->nHiPayoffs;
  }
  return true;
}
//*******************************************************************
string TStatCalcSlots::MakeSubGameTypeStr(int subgametype,TMoney gamedenom)
{
  string s = "Unk ";
  switch (subgametype)
  {
    case cslAlladinsLamp     : s = "AL ";break;
    case cslParadiseDreaming : s = "PD ";break;
    case cslCherrySlam       : s = "CS ";break;
    case cslJungleJingle     : s = "JJ ";break;
    case cslLuckyLeprechaun  : s = "LL ";break;
    case cslTotalEclipse     : s = "TE ";break;
    case cslJokersWild       : s = "JW ";break;
    case cslPiratesPlunder   : s = "PP ";break;
    case cslSpiritOf76       : s = "S76 ";break;
    case cslRockRoll         : s = "RRL ";break;
    case cslPoolShark        : s = "PS ";break;
    case cslGoldRush         : s = "GR ";break;
    case cslTreasuresAtlantis: s = "TA ";break;
    case cslZodiacAttackI    : s = "ZAI ";break;
    case cslZodiacAttackII   : s = "ZAII ";break;
    case cslZodiacAttackIII  : s = "ZAIII ";break;
    case cslFruitLoot        : s = "FL ";break;
    case cslRagsRiches       : s = "RR ";break;
    case cslSilverSafari     : s = "SS ";break;
    case cslTexasTwister     : s = "TT ";break;
    case cslThunderAlley     : s = "ThA ";break;
    case cslTreasureHaunt    : s = "TH ";break;
    case cslTycoonsTrust     : s = "TTr ";break;
    case cslWinnersCircle    : s = "WC ";break;
    case cslCashQuest        : s = "CQ ";break;
    case cslBirthdayBash     : s = "BB ";break;
    case cslSouthernCross    : s = "SC ";break;
    case cslMonkeyMadness    : s = "MM ";break;
    case cslAlienInvasion    : s = "AI ";break;
    case cslSuperJackpot     : s = "SJ ";break;
    case cslRoadRally        : s = "Ral";break;
    case cslHotReels         : s = "HR ";break;
    case cslSwampMusic       : s = "SM ";break;
    case cslLightningStrikes : s = "LS ";break;
    case cslQuestGold        : s = "QG ";break;
    case cslShootMoon        : s = "SM ";break;
  }
//  s += MakeDenomStr(gamedenom);
  s += gamedenom.GetStr(true,false);
  return s;
}                                        
//*******************************************************************
void TStatCalcSlots::Make_CumulativeCashWLStrs()
{
  // use total cash WL
}
//*******************************************************************
void TStatCalcSlots::Calc_CumulativeCashWL()
{
  // use totalcashWL
}
//*******************************************************************
void TStatCalcSlots::Make_LowPayoffsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LowPayoffsStr[i] = MakeString(GetSSession(i)->nLowPayoffs);
  }
  Total_LowPayoffsStr = MakeString(Total_LowPayoffs);
}
//*******************************************************************
void TStatCalcSlots::Calc_LowPayoffs()
{
  Total_LowPayoffs = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_LowPayoffs += GetSSession(i)->nLowPayoffs;
  }
}
//*******************************************************************
void TStatCalcSlots::Make_HighPayoffsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HighPayoffsStr[i] = MakeString(GetSSession(i)->nHiPayoffs);
  }
  Total_HighPayoffsStr = MakeString(Total_HighPayoffs);
}
//*******************************************************************
void TStatCalcSlots::Calc_HighPayoffs()
{
  Total_HighPayoffs = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_HighPayoffs += GetSSession(i)->nHiPayoffs;
  }
}
//*******************************************************************
void TStatCalcSlots::LocalCalculate()
{
  TCasinoStatCalc::LocalCalculate();

  if (PPlayerData == NULL) {return;}
  Calc_CumulativeCashWL();
  Calc_LowPayoffs();
  Calc_HighPayoffs();
}
//*******************************************************************
void TStatCalcSlots::LocalMakeAllStrings()
{
  TCasinoStatCalc::LocalMakeAllStrings();

  if (PPlayerData == NULL) {return;}

//  Make_DateStrs();
//  Make_HoursPlayedStrs();
//  Make_TypeGamePlayedStrs();
//  Make_TotalNumBetsStrs();
//  Make_TotalCashBetsStrs();
//  Make_AvgCashBetsStrs();
//  Make_StartingBankrollStrs();
//  Make_EndingBankrollStrs();
//  Make_TotalCashWLStrs();
//  Make_ReturnPercStrs();
//  Make_AvgWLPerHourStrs();
  Make_CumulativeCashWLStrs();
//  Make_HandsWonStrs();
//  Make_HandsLostStrs();
//  Make_PercHandsWonStrs();
//  Make_PercHandsLostStrs();
//  Make_LongestWinStreakStrs();
//  Make_LongestLosingStreakStrs();
//  Make_LargestCashWinStrs();
//  Make_PeakCashWinLevelStrs();
//  Make_LowCashLostLevelStrs();
  Make_LowPayoffsStrs();
  Make_HighPayoffsStrs();
}
//*******************************************************************
void TStatCalcSlots::LocalFillGrid()
{
  TCasinoStatCalc::LocalFillGrid();

  if (PPlayerData == NULL) {return;}

  for(int col = 0;col <= LastSessionNum;col++)
  {
    int c = col + 1;
    PStatDlg->cell[1][c] = DateStr[col];
    PStatDlg->cell[2][c] = HoursPlayedStr[col];
    PStatDlg->cell[3][c] = TypeGamePlayedStr[col];
    PStatDlg->cell[6][c] = TotalNumBetsStr[col];
    PStatDlg->cell[7][c] = TotalCashBetsStr[col];
    PStatDlg->cell[8][c] = AvgCashBetsStr[col];
    PStatDlg->cell[9][c] = StartingBankrollStr[col];
    PStatDlg->cell[10][c] = EndingBankrollStr[col];
    PStatDlg->cell[11][c] = TotalCashWLStr[col];
    PStatDlg->cell[12][c] = ReturnPercStr[col];
    PStatDlg->cell[13][c] = AvgWLPerHourStr[col];
    PStatDlg->cell[14][c] = TotalCashWLStr[col];
    PStatDlg->cell[17][c] = TotalNumBetsStr[col];
    PStatDlg->cell[18][c] = HandsWonStr[col];
    PStatDlg->cell[19][c] = HandsLostStr[col];
    PStatDlg->cell[20][c] = PercHandsWonStr[col];
    PStatDlg->cell[21][c] = PercHandsLostStr[col];
    PStatDlg->cell[22][c] = LongestWinStreakStr[col];
    PStatDlg->cell[23][c] = LongestLosingStreakStr[col];
    PStatDlg->cell[24][c] = LargestCashWinStr[col];
    PStatDlg->cell[25][c] = PeakCashWinLevelStr[col];
    PStatDlg->cell[26][c] = LowCashLostLevelStr[col];
    PStatDlg->cell[27][c] = LowPayoffsStr[col];
    PStatDlg->cell[28][c] = HighPayoffsStr[col];
  }

  PStatDlg->totalcol[1] = Total_DateStr;
  PStatDlg->totalcol[2] = Total_HoursPlayedStr;
  PStatDlg->totalcol[3] = Total_TypeGamePlayedStr;
  PStatDlg->totalcol[6] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[7] = Total_TotalCashBetsStr;
  PStatDlg->totalcol[8] = Total_AvgCashBetsStr;
  PStatDlg->totalcol[9] = Total_StartingBankrollStr;
  PStatDlg->totalcol[10] = Total_EndingBankrollStr;
  PStatDlg->totalcol[11] = Total_TotalCashWLStr;
  PStatDlg->totalcol[12] = Total_ReturnPercStr;
  PStatDlg->totalcol[13] = Total_AvgWLPerHourStr;
  PStatDlg->totalcol[14] = Total_TotalCashWLStr;
  PStatDlg->totalcol[17] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[18] = Total_HandsWonStr;
  PStatDlg->totalcol[19] = Total_HandsLostStr;
  PStatDlg->totalcol[20] = Total_PercHandsWonStr;
  PStatDlg->totalcol[21] = Total_PercHandsLostStr;
  PStatDlg->totalcol[22] = Total_LongestWinStreakStr;
  PStatDlg->totalcol[23] = Total_LongestLosingStreakStr;
  PStatDlg->totalcol[24] = Total_LargestCashWinStr;
  PStatDlg->totalcol[25] = Total_PeakCashWinLevelStr;
  PStatDlg->totalcol[26] = Total_LowCashLostLevelStr;
  PStatDlg->totalcol[27] = Total_LowPayoffsStr;
  PStatDlg->totalcol[28] = Total_HighPayoffsStr;
}
//*******************************************************************
//*******************************************************************
TSlotsGraph::TSlotsGraph(TCasinoStatDlg* pstatdlg,TStatCalcSlots *pstatcalc)
:TCasinoGraphDlg(pstatdlg,pstatcalc)
{
  strName = "Slots";
  BaseStringID = SlotsLabel;

  StatTT[0] = TStatCalcSlots::sTotalNumBets;
  StatTT[1] = TStatCalcSlots::sTotalCashBets;
  StatTT[2] = TStatCalcSlots::sAvgCashBets;
  StatTT[3] = TStatCalcSlots::sTotalCashWL;
  StatTT[4] = TStatCalcSlots::sCumCashWL;
  StatTT[5] = TStatCalcSlots::sSpinsWon;
  StatTT[6] = TStatCalcSlots::sSpinsLost;
  StatTT[7] = TStatCalcSlots::sPercWon;
  StatTT[8] = TStatCalcSlots::sPercLost;
  StatTT[9] = TStatCalcSlots::sLongestWinStreak;
  StatTT[10] = TStatCalcSlots::sLongestLosingStreak;
  StatTT[11] = TStatCalcSlots::sLargestCashWin;
  StatTT[12] = TStatCalcSlots::sPeakCashWinLevel;
  StatTT[13] = TStatCalcSlots::sLowCashLossLevel;
  StatTT[14] = TStatCalcSlots::sLowPayoffs;
  StatTT[15] = TStatCalcSlots::sHighPayoffs;

  LastStatIndex = 15;
}
//*******************************************************************
TSlotsGraph::~TSlotsGraph()
{
}
//*******************************************************************
