//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cvpcalc.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/6/96     M.Perry    derived from Dave's code
//  2.0    09/17/97    M.Perry    32-bit conversion.
//********************************************************************
#include "cvpcalc.h"
#include "cstatdlg.h"
#include <stdio.h>
#include <classlib\time.h>
#include <classlib\date.h>
#include "oapp.h"
//*******************************************************************
string TStatCalcVP::GetLabelStr(int16 row_ndx)
{
  string s = TAppString::strEmpty;
  if (row_ndx >= 0 && row_ndx < vpNumRows)
  {
    s = LabelStrs[row_ndx];
  }
  return s;
}
//*******************************************************************
TStatCalcVP::TStatCalcVP(TCasinoStatDlg *pstatdlg)
:TCasinoStatCalc(pstatdlg)
{
  // setup Label resource ids & then go get strings.

  for (int16 i=0;i<vpNumRows;i++)
  {
    LabelResIds[i] = 0;
    LabelStrs[i] = TAppString::strEmpty;
  }

  //row ndx enum name to label string res id
  LabelResIds[vpWinLossStats] = VideoPokerLabel + 1;
  LabelResIds[vpGameStats] = VideoPokerLabel + 2;
  LabelResIds[vpNumOccurrences] = VideoPokerLabel + 3;
  LabelResIds[vpTotalCashWins] = VideoPokerLabel + 4;
  LabelResIds[vpPercTotalCashWon] = VideoPokerLabel + 5;
  LabelResIds[vpAdvancedStats] = VideoPokerLabel + 6;
  LabelResIds[vpDate] = VideoPokerLabel + 7;
  LabelResIds[vpHoursPlayed] = VideoPokerLabel + 8;
  LabelResIds[vpGameType] = VideoPokerLabel + 9;
  LabelResIds[vpTotalNumBets] = VideoPokerLabel + 10;
  LabelResIds[vpTotalCashBets] = VideoPokerLabel + 11;
  LabelResIds[vpAvgCashBet] = VideoPokerLabel + 12;
  LabelResIds[vpStartingBankroll] = VideoPokerLabel + 13;
  LabelResIds[vpEndingBankroll] = VideoPokerLabel + 14;
  LabelResIds[vpTotalCashWL] = VideoPokerLabel + 15;
  LabelResIds[vpRetPerc] = VideoPokerLabel + 16;
  LabelResIds[vpAvgWLPerHour] = VideoPokerLabel + 17;
  LabelResIds[vpCumCashWL] = VideoPokerLabel + 18;
  LabelResIds[vpHandsPlayed] = VideoPokerLabel + 19;
  LabelResIds[vpHandsWon] = VideoPokerLabel + 20;
  LabelResIds[vpHandsLost] = VideoPokerLabel + 21;
  LabelResIds[vpPercHandsWon] = VideoPokerLabel + 22;
  LabelResIds[vpPercHandsLost] = VideoPokerLabel + 23;
  LabelResIds[vpLongestWinStreak] = VideoPokerLabel + 24;
  LabelResIds[vpLongestLosingStreak] = VideoPokerLabel + 25;
  LabelResIds[vpLargestCashWin] = VideoPokerLabel + 26;
  LabelResIds[vpPeakCashWinLevel] = VideoPokerLabel + 27;
  LabelResIds[vpLowCashLossLevel] = VideoPokerLabel + 28;
  LabelResIds[vpGarbageHands] = VideoPokerLabel + 29;
  LabelResIds[vpHighPairs] = VideoPokerLabel + 32;
  LabelResIds[vpTwoPairs] = VideoPokerLabel + 33;
  LabelResIds[vpThreeofaKinds] = VideoPokerLabel + 34;
  LabelResIds[vpStraights] = VideoPokerLabel + 35;
  LabelResIds[vpFlushes] = VideoPokerLabel + 36;
  LabelResIds[vpFullHouses] = VideoPokerLabel + 37;
  LabelResIds[vpFourofaKinds] = VideoPokerLabel + 38;
  LabelResIds[vpStraightFlushes] = VideoPokerLabel + 39;
  LabelResIds[vpRoyalFlushesNatural] = VideoPokerLabel + 40;
  LabelResIds[vpRoyalFlushesWild] = VideoPokerLabel + 41;
  LabelResIds[vpFiveofaKinds] = VideoPokerLabel + 43;
  LabelResIds[vpFourDeuces] = VideoPokerLabel + 42;
  LabelResIds[vpTotalCashBets2] = VideoPokerLabel + 11;
  LabelResIds[vpTotalCashWon] = VideoPokerLabel + 31;
  LabelResIds[vpHighPairsCash] = VideoPokerLabel + 32;
  LabelResIds[vpTwoPairsCash] = VideoPokerLabel + 33;
  LabelResIds[vpThreeofaKindsCash] = VideoPokerLabel + 34;
  LabelResIds[vpStraightsCash] = VideoPokerLabel + 35;
  LabelResIds[vpFlushesCash] = VideoPokerLabel + 36;
  LabelResIds[vpFullHousesCash] = VideoPokerLabel + 37;
  LabelResIds[vpFourofaKindsCash] = VideoPokerLabel + 38;
  LabelResIds[vpStraightFlushesCash] = VideoPokerLabel + 39;
  LabelResIds[vpRoyalFlushesNaturalCash] = VideoPokerLabel + 40;
  LabelResIds[vpRoyalFlushesWildCash] = VideoPokerLabel + 41;
  LabelResIds[vpFourDeucesCash] = VideoPokerLabel + 42;
  LabelResIds[vpFiveofaKindsCash] = VideoPokerLabel + 43;
  LabelResIds[vpPercCashHighPairs] = VideoPokerLabel + 32;
  LabelResIds[vpPercCashTwoPairs] = VideoPokerLabel + 33;
  LabelResIds[vpPercCashThreeofaKinds] = VideoPokerLabel + 34;
  LabelResIds[vpPercCashStraights] = VideoPokerLabel + 35;
  LabelResIds[vpPercCashFlushes] = VideoPokerLabel + 36;
  LabelResIds[vpPercCashFullHouses] = VideoPokerLabel + 37;
  LabelResIds[vpPercCashFourofaKinds] = VideoPokerLabel + 38;
  LabelResIds[vpPercCashStraightFlushes] = VideoPokerLabel + 39;
  LabelResIds[vpPercCashRoyalFlushesNatural] = VideoPokerLabel + 40;
  LabelResIds[vpPercCashRoyalFlushesWild] = VideoPokerLabel + 41;
  LabelResIds[vpPercCashFourDeuces] = VideoPokerLabel + 42;
  LabelResIds[vpPercCashFiveofaKinds] = VideoPokerLabel + 43;
  LabelResIds[vpNumIncPlays] = VideoPokerLabel + 73;
  LabelResIds[vpPercPlayingAccuracy] = VideoPokerLabel + 74;

  #if defined(ACVP_COMPILE)
  //add new stats here.
  LabelResIds[vpPair6] = VideoPokerLabel + 44;
//  LabelResIds[vpPairJacks] = VideoPokerLabel + 45;
//  LabelResIds[vpPairKings] = VideoPokerLabel + 46;
  LabelResIds[vpFlushDiamond] = VideoPokerLabel + 47;
  LabelResIds[vpFlushHeart] = VideoPokerLabel + 48;
  LabelResIds[vpFlushSpade] = VideoPokerLabel + 49;
  LabelResIds[vpFlushClub] = VideoPokerLabel + 50;
  LabelResIds[vpFlushAssault] = VideoPokerLabel + 51;
  LabelResIds[vpFourofaKind2_10] = VideoPokerLabel + 52;
  LabelResIds[vpFourFaces] = VideoPokerLabel + 53;
  LabelResIds[vpFourFacesWAceorFace] = VideoPokerLabel + 54;
  LabelResIds[vpFourFacesWAce] = VideoPokerLabel + 55;
  LabelResIds[vpFourAcesWFace] = VideoPokerLabel + 75;
  LabelResIds[vpFourofaKind4_K] = VideoPokerLabel + 56;
  LabelResIds[vpFourofaKind5_K] = VideoPokerLabel + 57;
  LabelResIds[vpFourofaKind2_4] = VideoPokerLabel + 58;
  LabelResIds[vpFourAces] = VideoPokerLabel + 76;
  LabelResIds[vpStraightDiamondFlush] = VideoPokerLabel + 59;
  LabelResIds[vpStraightHeartFlush] = VideoPokerLabel + 60;
  LabelResIds[vpStraightSpadeFlush] = VideoPokerLabel + 61;
  LabelResIds[vpStraightClubFlush] = VideoPokerLabel + 62;
  LabelResIds[vpRoyalDiamondFlush] = VideoPokerLabel + 63;
  LabelResIds[vpRoyalHeartFlush] = VideoPokerLabel + 64;
  LabelResIds[vpRoyalSpadeFlush] = VideoPokerLabel + 65;
  LabelResIds[vpRoyalClubFlush] = VideoPokerLabel + 66;
  LabelResIds[vpFiveofaKind6_K] = VideoPokerLabel + 67;
  LabelResIds[vpFiveofaKind3_5] = VideoPokerLabel + 68;
  LabelResIds[vpFiveAces] = VideoPokerLabel + 69;
  LabelResIds[vpFour2WAce] = VideoPokerLabel + 70;
  LabelResIds[vpFour7WJoker] = VideoPokerLabel + 71;
  LabelResIds[vpFiveJoker] = VideoPokerLabel + 72;
  LabelResIds[vpThreeofaKinds5] = VideoPokerLabel + 77;
  LabelResIds[vpThreeofaKinds8] = VideoPokerLabel + 78;
  LabelResIds[vpFourofaKinds5] = VideoPokerLabel + 79;
  LabelResIds[vpFourofaKinds7] = VideoPokerLabel + 80;
  LabelResIds[vpFourofaKinds8] = VideoPokerLabel + 81;

  LabelResIds[vpPair6Cash] = VideoPokerLabel + 44;
//  LabelResIds[vpPairJacksCash] = VideoPokerLabel + 45;
//  LabelResIds[vpPairKingsCash] = VideoPokerLabel + 46;
  LabelResIds[vpFlushDiamondCash] = VideoPokerLabel + 47;
  LabelResIds[vpFlushHeartCash] = VideoPokerLabel + 48;
  LabelResIds[vpFlushSpadeCash] = VideoPokerLabel + 49;
  LabelResIds[vpFlushClubCash] = VideoPokerLabel + 50;
  LabelResIds[vpFlushAssaultCash] = VideoPokerLabel + 51;
  LabelResIds[vpFourofaKind2_10Cash] = VideoPokerLabel + 52;
  LabelResIds[vpFourFacesCash] = VideoPokerLabel + 53;
  LabelResIds[vpFourFacesWAceorFaceCash] = VideoPokerLabel + 54;
  LabelResIds[vpFourFacesWAceCash] = VideoPokerLabel + 55;
  LabelResIds[vpFourAcesWFaceCash] = VideoPokerLabel + 75;
  LabelResIds[vpFourofaKind4_KCash] = VideoPokerLabel + 56;
  LabelResIds[vpFourofaKind5_KCash] = VideoPokerLabel + 57;
  LabelResIds[vpFourofaKind2_4Cash] = VideoPokerLabel + 58;
  LabelResIds[vpFourAcesCash] = VideoPokerLabel + 76;
  LabelResIds[vpStraightDiamondFlushCash] = VideoPokerLabel + 59;
  LabelResIds[vpStraightHeartFlushCash] = VideoPokerLabel + 60;
  LabelResIds[vpStraightSpadeFlushCash] = VideoPokerLabel + 61;
  LabelResIds[vpStraightClubFlushCash] = VideoPokerLabel + 62;
  LabelResIds[vpRoyalDiamondFlushCash] = VideoPokerLabel + 63;
  LabelResIds[vpRoyalHeartFlushCash] = VideoPokerLabel + 64;
  LabelResIds[vpRoyalSpadeFlushCash] = VideoPokerLabel + 65;
  LabelResIds[vpRoyalClubFlushCash] = VideoPokerLabel + 66;
  LabelResIds[vpFiveofaKind6_KCash] = VideoPokerLabel + 67;
  LabelResIds[vpFiveofaKind3_5Cash] = VideoPokerLabel + 68;
  LabelResIds[vpFiveAcesCash] = VideoPokerLabel + 69;
  LabelResIds[vpFour2WAceCash] = VideoPokerLabel + 70;
  LabelResIds[vpFour7WJokerCash] = VideoPokerLabel + 71;
  LabelResIds[vpFiveJokersCash] = VideoPokerLabel + 72;
  LabelResIds[vpThreeofaKinds5Cash] = VideoPokerLabel + 77;
  LabelResIds[vpThreeofaKinds8Cash] = VideoPokerLabel + 78;
  LabelResIds[vpFourofaKinds5Cash] = VideoPokerLabel + 79;
  LabelResIds[vpFourofaKinds7Cash] = VideoPokerLabel + 80;
  LabelResIds[vpFourofaKinds8Cash] = VideoPokerLabel + 81;

  LabelResIds[vpPercCashPair6] = VideoPokerLabel + 44;
//  LabelResIds[vpPercCashPairJacks] = VideoPokerLabel + 45;
//  LabelResIds[vpPercCashPairKings] = VideoPokerLabel + 46;
  LabelResIds[vpPercCashDiamondFlush] = VideoPokerLabel + 47;
  LabelResIds[vpPercCashHeartFlush] = VideoPokerLabel + 48;
  LabelResIds[vpPercCashSpadeFlush] = VideoPokerLabel + 49;
  LabelResIds[vpPercCashClubFlush] = VideoPokerLabel + 50;
  LabelResIds[vpPercCashFlushAssault] = VideoPokerLabel + 51;
  LabelResIds[vpPercCashFourofaKind2_10] = VideoPokerLabel + 52;
  LabelResIds[vpPercCashFourFaces] = VideoPokerLabel + 53;
  LabelResIds[vpPercCashFourFacesWAceorFace] = VideoPokerLabel + 54;
  LabelResIds[vpPercCashFourFacesWAce] = VideoPokerLabel + 55;
  LabelResIds[vpPercCashFourAcesWFace] = VideoPokerLabel + 75;
  LabelResIds[vpPercCashFourofaKind4_K] = VideoPokerLabel + 56;
  LabelResIds[vpPercCashFourofaKind5_K] = VideoPokerLabel + 57;
  LabelResIds[vpPercCashFourofaKind2_4] = VideoPokerLabel + 58;
  LabelResIds[vpPercCashFourAces] = VideoPokerLabel + 76;
  LabelResIds[vpPercCashStraightDiamondFlush] = VideoPokerLabel + 59;
  LabelResIds[vpPercCashStraightHeartFlush] = VideoPokerLabel + 60;
  LabelResIds[vpPercCashStraightSpadeFlush] = VideoPokerLabel + 61;
  LabelResIds[vpPercCashStraightClubFlush] = VideoPokerLabel + 62;
  LabelResIds[vpPercCashRoyalDiamondFlush] = VideoPokerLabel + 63;
  LabelResIds[vpPercCashRoyalHeartFlush] = VideoPokerLabel + 64;
  LabelResIds[vpPercCashRoyalSpadeFlush] = VideoPokerLabel + 65;
  LabelResIds[vpPercCashRoyalClubFlush] = VideoPokerLabel + 66;
  LabelResIds[vpPercCashFiveofaKind6_K] = VideoPokerLabel + 67;
  LabelResIds[vpPercCashFiveofaKind3_5] = VideoPokerLabel + 68;
  LabelResIds[vpPercCashFiveAces] = VideoPokerLabel + 69;
  LabelResIds[vpPercCashFour2WAce] = VideoPokerLabel + 70;
  LabelResIds[vpPercCashFour7WJoker] = VideoPokerLabel + 71;
  LabelResIds[vpPercCashFiveJoker] = VideoPokerLabel + 72;
  LabelResIds[vpPercCashThreeofaKinds5] = VideoPokerLabel + 77;
  LabelResIds[vpPercCashThreeofaKinds8] = VideoPokerLabel + 78;
  LabelResIds[vpPercCashFourofaKinds5] = VideoPokerLabel + 79;
  LabelResIds[vpPercCashFourofaKinds7] = VideoPokerLabel + 80;
  LabelResIds[vpPercCashFourofaKinds8] = VideoPokerLabel + 81;
  #endif


  for (int16 i=0;i<vpNumRows;i++)
  {
    if (LabelResIds[i] > 0)
    {
      LabelStrs[i] = TAppString(LabelResIds[i]);
    }
  }
}
//*******************************************************************
TStatCalcVP::~TStatCalcVP()
{
}
//*******************************************************************
void TStatCalcVP::GetSessionPtrs()
{
  if (PPlayerData == NULL) {return;}
  PData = &(PPlayerData->VPData);
  LastSessionNum = -1;
  for (int sessioniterator = GetVPData()->GetFirstValidSession();
       sessioniterator != -1;
       sessioniterator = GetVPData()->GetNextValidSession())
  {
    LastSessionNum++;
    PSessions[LastSessionNum] = &(GetVPData()->Sessions[sessioniterator]);
  }
}
//*******************************************************************
bool TStatCalcVP::GetValues(int statindex,float *values,int maxvalues)
{
  int num = GetMaxValueIndex(maxvalues);
  switch (statindex)
  {
    case vpTotalNumBets       :return GetTotalNumBets(values,num);
    case vpTotalCashBets      :return GetTotalCashBets(values,num);
    case vpAvgCashBet         :return GetAvgCashBets(values,num);
    case vpTotalCashWL        :return GetTotalCashWL(values,num);
    case vpCumCashWL          :return GetTotalCashWL(values,num);
    case vpHandsWon           :return GetWins(values,num);
    case vpHandsLost          :return GetLosses(values,num);
    case vpPercHandsWon       :return GetPercWins(values,num);
    case vpLongestWinStreak   :return GetLongestWinStreaks(values,num);
    case vpLongestLosingStreak:return GetLongestLosingStreak(values,num);
    case vpLargestCashWin     :return GetLargestCashWin(values,num);
    case vpPeakCashWinLevel   :return GetPeakWinLevel(values,num);
    case vpLowCashLossLevel   :return GetLowCashLostLevel(values,num);
    case vpNumIncPlays        :return GetNumBadStrategyPlays(values,num);
    case vpPercPlayingAccuracy:return GetPercPlayingAccuracy(values,num);
    default                   :return false;
  };
//  return false;
}
//*******************************************************************
bool TStatCalcVP::GetNumBadStrategyPlays(float *values ,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetVPSession(i)->IncorrectPlays;
  }
  return true;
}
//*******************************************************************
bool TStatCalcVP::GetPercPlayingAccuracy(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = PercPlayingAccuracy[i];
  }
  return true;
}
//*******************************************************************
string TStatCalcVP::MakeSubGameTypeStr(int subgametype,TMoney denom)
{
  string s = "Unk vp";
  switch (subgametype)
  {
    case cvpDeucesWild       : s = "DW";break;
    case cvpJokersWild       : s = "JW";break;
    case cvpJacksOrBetter    : s = "JB";break;
    case cvpTensOrBetter     : s = "TB";break;
    case cvpJacksProgressive : s = "JBP";break;
    #if defined(ACVP_COMPILE)
    case cvpFivesUp          : s = "5UP";break;
    case cvpBlackJacks       : s = "BJS";break;
    case cvpBonusDeluxe      : s = "BDX";break;
    case cvpClubBonus        : s = "CB";break;
    case cvpCrazy8s          : s = "C8";break;
    case cvpDDrawJB          : s = "DDJB";break;
    case cvpDDrawJokerPoker  : s = "DDJP";break;
    case cvpDDownStud        : s = "DDS";break;
    case cvpDeucesWildBonus  : s = "DWB";break;
    case cvpDeucesWildII     : s = "DW";break;
    case cvpDeucesWildProg   : s = "DWP";break;
    case cvpDiamondBonus     : s = "DB";break;
    case cvpDBonusPoker      : s = "DBP";break;
    case cvpFlushAssault     : s = "FA";break;
    case cvpFreeLook         : s = "FL";break;
    case cvpGamblersEdge     : s = "GE";break;
    case cvpHeartBonus       : s = "HB";break;
    case cvpJacksProgII      : s = "JBP";break;
    case cvpJackpotBonus     : s = "JP";break;
    case cvp96JacksOrBetterII: s = "JB9";break;
    case cvp85JacksOrBetter  : s = "JB8";break;
    case cvpDJacksOrBetter   : s = "DJB";break;
    case cvpJokersWildProg   : s = "JWP";break;
    case cvpJumpinJokers     : s = "JJ";break;
    case cvpTripleProg       : s = "TP";break;
    case cvpRoyalProg        : s = "RP";break;
    case cvpPowerProg        : s = "PHP";break;
    case cvpRaiseJokersWild  : s = "RJW";break;
    case cvpSpadeBonus       : s = "SB";break;
    case cvpSuperDJackpot    : s = "SDJ";break;
    case cvp7Wild1Joker      : s = "7WJ";break;
    case cvpTensOrBetterII   : s = "TB";break;
    case cvpTreasureTrove    : s = "TT";break;
    case cvpTriplePlay       : s = "TP";break;
    case cvpTripleBonus      : s = "TB";break;
    case cvpWheelDeal        : s = "WD";break;
    #endif
  }
  s += MakeDenomStr(denom);
  return s;
}
//*******************************************************************
void TStatCalcVP::Calc_TotalNumBets()
{
  Total_TotalNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalNumBets[i] = GetVPSession(i)->Wins + GetVPSession(i)->Losses;
    Total_TotalNumBets += TotalNumBets[i];
  }
}
//*******************************************************************
void TStatCalcVP::Make_CumulativeCashWLStrs()
{
  // use TotalCasWL
}
//*******************************************************************
void TStatCalcVP::Calc_CumulativeCashWL()
{
  // same as TotalCashWL
}
//*******************************************************************
void TStatCalcVP::Make_HandsPlayedStrs()
{
  // use TotalNumBets
}
//*******************************************************************
void TStatCalcVP::Calc_HandsPlayed()
{
  // same as TotalNumBets
}
//*******************************************************************
void TStatCalcVP::Make_NumGarbageHandsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    NumGarbageHandsStr[i] = MakeString(GetVPSession(i)->GarbageHands);
  }
  Total_NumGarbageHandsStr = MakeString(Total_NumGarbageHands);
}
//*******************************************************************
void TStatCalcVP::Calc_NumGarbageHands()
{
  Total_NumGarbageHands = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumGarbageHands += GetVPSession(i)->GarbageHands;
  }
}
//*******************************************************************
void TStatCalcVP::Make_HighPairsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HighPairsStr[i] = MakeString(GetVPSession(i)->HighPairs);
  }
  Total_HighPairsStr = MakeString(Total_HighPairs);
}
//*******************************************************************
void TStatCalcVP::Calc_HighPairs()
{
  Total_HighPairs = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_HighPairs += GetVPSession(i)->HighPairs;
  }
}
//*******************************************************************
void TStatCalcVP::Make_TwoPairsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoPairsStr[i] = MakeString(GetVPSession(i)->TwoPairs);
  }
  Total_TwoPairsStr = MakeString(Total_TwoPairs);
}
//*******************************************************************
void TStatCalcVP::Calc_TwoPairs()
{
  Total_TwoPairs = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoPairs += GetVPSession(i)->TwoPairs;
  }
}
//*******************************************************************
void TStatCalcVP::Make_ThreeOfKindsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeOfKindsStr[i] = MakeString(GetVPSession(i)->ThreeOfKinds);
  }
  Total_ThreeOfKindsStr = MakeString(Total_ThreeOfKinds);
}
//*******************************************************************
void TStatCalcVP::Calc_ThreeOfKinds()
{
  Total_ThreeOfKinds = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeOfKinds += GetVPSession(i)->ThreeOfKinds;
  }
}
//*******************************************************************
void TStatCalcVP::Make_StraightsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightsStr[i] = MakeString(GetVPSession(i)->Straights);
  }
  Total_StraightsStr = MakeString(Total_Straights);
}
//*******************************************************************
void TStatCalcVP::Calc_Straights()
{
  Total_Straights = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Straights += GetVPSession(i)->Straights;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FlushesStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushesStr[i] = MakeString(GetVPSession(i)->Flushes);
  }
  Total_FlushesStr = MakeString(Total_Flushes);
}
//*******************************************************************
void TStatCalcVP::Calc_Flushes()
{
  Total_Flushes = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Flushes += GetVPSession(i)->Flushes;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FullHousesStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FullHousesStr[i] = MakeString(GetVPSession(i)->FullHouses);
  }
  Total_FullHousesStr = MakeString(Total_FullHouses);
}
//*******************************************************************
void TStatCalcVP::Calc_FullHouses()
{
  Total_FullHouses = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FullHouses += GetVPSession(i)->FullHouses;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsStr[i] = MakeString(GetVPSession(i)->FourOfKinds);
  }
  Total_FourOfKindsStr = MakeString(Total_FourOfKinds);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds()
{
  Total_FourOfKinds = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds += GetVPSession(i)->FourOfKinds;
  }
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesStr[i] = MakeString(GetVPSession(i)->StraightFlushes);
  }
  Total_StraightFlushesStr = MakeString(Total_StraightFlushes);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushes()
{
  Total_StraightFlushes = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushes += GetVPSession(i)->StraightFlushes;
  }
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesNaturalStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesNaturalStr[i] = MakeString(GetVPSession(i)->NaturalRoyals);
  }
  Total_RoyalFlushesNaturalStr = MakeString(Total_RoyalFlushesNatural);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesNatural()
{
  Total_RoyalFlushesNatural = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesNatural += GetVPSession(i)->NaturalRoyals;
  }
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesWildStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesWildStr[i] = MakeString(GetVPSession(i)->WildRoyals);
  }
  Total_RoyalFlushesWildStr = MakeString(Total_RoyalFlushesWild);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesWild()
{
  Total_RoyalFlushesWild = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesWild += GetVPSession(i)->WildRoyals;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourDeucesStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourDeucesStr[i] = MakeString(GetVPSession(i)->FourDeuces);
  }
  Total_FourDeucesStr = MakeString(Total_FourDeuces);
}
//*******************************************************************
void TStatCalcVP::Calc_FourDeuces()
{
  Total_FourDeuces = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourDeuces += GetVPSession(i)->FourDeuces;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKindsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKindsStr[i] = MakeString(GetVPSession(i)->FiveOfKinds);
  }
  Total_FiveOfKindsStr = MakeString(Total_FiveOfKinds);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKinds()
{
  Total_FiveOfKinds = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKinds += GetVPSession(i)->FiveOfKinds;
  }
}
//*******************************************************************
void TStatCalcVP::Make_HighPairsCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HighPairsCashStr[i] = MakeString(GetVPSession(i)->TotalHighPairCash);
  }
  Total_HighPairsCashStr = MakeString(Total_HighPairsCash);
}
//*******************************************************************
void TStatCalcVP::Calc_HighPairsCash()
{
  Total_HighPairsCash = 0;
  for (int i=0;i<=LastSessionNum;i++)     
  {
    Total_HighPairsCash += GetVPSession(i)->TotalHighPairCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_TwoPairsCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TwoPairsCashStr[i] = MakeString(GetVPSession(i)->TotalTwoPairCash);
  }
  Total_TwoPairsCashStr = MakeString(Total_TwoPairsCash);
}
//*******************************************************************
void TStatCalcVP::Calc_TwoPairsCash()
{
  Total_TwoPairsCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TwoPairsCash += GetVPSession(i)->TotalTwoPairCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_ThreeOfKindsCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeOfKindsCashStr[i] = MakeString(GetVPSession(i)->TotalThreeOfKindCash);
  }
  Total_ThreeOfKindsCashStr = MakeString(Total_ThreeOfKindsCash);
}
//*******************************************************************
void TStatCalcVP::Calc_ThreeOfKindsCash()
{
  Total_ThreeOfKindsCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeOfKindsCash += GetVPSession(i)->TotalThreeOfKindCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_StraightsCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightsCashStr[i] = MakeString(GetVPSession(i)->TotalStraightCash);
  }
  Total_StraightsCashStr = MakeString(Total_StraightsCash);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightsCash()
{
  Total_StraightsCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_StraightsCash += GetVPSession(i)->TotalStraightCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FlushesCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushesCashStr[i] = MakeString(GetVPSession(i)->TotalFlushCash);
  }
  Total_FlushesCashStr = MakeString(Total_FlushesCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushesCash()
{
  Total_FlushesCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FlushesCash += GetVPSession(i)->TotalFlushCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FullHousesCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FullHousesCashStr[i] = MakeString(GetVPSession(i)->TotalFullHouseCash);
  }
  Total_FullHousesCashStr = MakeString(Total_FullHousesCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FullHousesCash()
{
  Total_FullHousesCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FullHousesCash += GetVPSession(i)->TotalFullHouseCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKindCash);
  }
  Total_FourOfKindsCashStr = MakeString(Total_FourOfKindsCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsCash()
{
  Total_FourOfKindsCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsCash += GetVPSession(i)->TotalFourOfKindCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesCashStr[i] = MakeString(GetVPSession(i)->TotalStraightFlushCash);
  }
  Total_StraightFlushesCashStr = MakeString(Total_StraightFlushesCash);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesCash()
{
  Total_StraightFlushesCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesCash += GetVPSession(i)->TotalStraightFlushCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesNaturalCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesNaturalCashStr[i] = MakeString(GetVPSession(i)->TotalNaturalRoyalCash);
  }
  Total_RoyalFlushesNaturalCashStr = MakeString(Total_RoyalFlushesNaturalCash);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesNaturalCash()
{
  Total_RoyalFlushesNaturalCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesNaturalCash += GetVPSession(i)->TotalNaturalRoyalCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesWildCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesWildCashStr[i] = MakeString(GetVPSession(i)->TotalWildRoyalCash);
  }
  Total_RoyalFlushesWildCashStr = MakeString(Total_RoyalFlushesWildCash);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesWildCash()
{
  Total_RoyalFlushesWildCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesWildCash += GetVPSession(i)->TotalWildRoyalCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourDeucesCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourDeucesCashStr[i] = MakeString(GetVPSession(i)->TotalFourDeuceCash);
  }
  Total_FourDeucesCashStr = MakeString(Total_FourDeucesCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourDeucesCash()
{
  Total_FourDeucesCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourDeucesCash += GetVPSession(i)->TotalFourDeuceCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKindsCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKindsCashStr[i] = MakeString(GetVPSession(i)->TotalFiveOfKindCash);
  }
  Total_FiveOfKindsCashStr = MakeString(Total_FiveOfKindsCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKindsCash()
{
  Total_FiveOfKindsCash = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKindsCash += GetVPSession(i)->TotalFiveOfKindCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_TotalCashWonStrs()
{
  for (int i = 0;i<=LastSessionNum;i++)
  {
    TotalCashWonStr[i] = MakeString(TotalCashWon[i]);
  }
  Total_TotalCashWonStr = MakeString(Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_TotalCashWon()
{
  Total_TotalCashWon = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWon[i] = GetVPSession(i)->TotalHighPairCash +
				          GetVPSession(i)->TotalTwoPairCash  +
				          GetVPSession(i)->TotalThreeOfKindCash +
				          GetVPSession(i)->TotalStraightCash +
				          GetVPSession(i)->TotalFlushCash +
				          GetVPSession(i)->TotalFullHouseCash +
				          GetVPSession(i)->TotalFourOfKindCash +
				          GetVPSession(i)->TotalStraightFlushCash +
				          GetVPSession(i)->TotalNaturalRoyalCash +
				          GetVPSession(i)->TotalWildRoyalCash +
				          GetVPSession(i)->TotalFourDeuceCash +
				          GetVPSession(i)->TotalFiveOfKindCash;
    #if defined(ACVP_COMPILE)
    TotalCashWon[i] += GetVPSession(i)->TotalPair6Cash +
                    //   GetVPSession(i)->TotalPairJacksCash +
                    //   GetVPSession(i)->TotalPairKingsCash +
                       GetVPSession(i)->TotalThreeOfKinds5Cash +
                       GetVPSession(i)->TotalThreeOfKinds8Cash +
                       GetVPSession(i)->TotalFlushDiamondCash +
                       GetVPSession(i)->TotalFlushHeartCash +
                       GetVPSession(i)->TotalFlushSpadeCash +
                       GetVPSession(i)->TotalFlushClubCash +
                       GetVPSession(i)->TotalFlushAssaultCash +
                       GetVPSession(i)->TotalFourOfKinds5Cash +
                       GetVPSession(i)->TotalFourOfKinds8Cash +
                       GetVPSession(i)->TotalFourOfKinds7Cash +
                       GetVPSession(i)->TotalFourOfKinds2_10Cash +
                       GetVPSession(i)->TotalFourOfKindsFacesCash +
                       GetVPSession(i)->TotalFourOfKindsFacesWAceOrFaceCash +
                       GetVPSession(i)->TotalFourOfKindsFacesWAceCash +
                       GetVPSession(i)->TotalFourOfKindsAcesWFaceCash +
                       GetVPSession(i)->TotalFourOfKinds4_KCash +
                       GetVPSession(i)->TotalFourOfKinds5_KCash +
                       GetVPSession(i)->TotalFourOfKinds2_4Cash +
                       GetVPSession(i)->TotalFourOfKindsAcesCash +
                       GetVPSession(i)->TotalStraightFlushesDiamondCash +
                       GetVPSession(i)->TotalStraightFlushesHeartCash +
                       GetVPSession(i)->TotalStraightFlushesSpadeCash +
                       GetVPSession(i)->TotalStraightFlushesClubCash +
                       GetVPSession(i)->TotalRoyalFlushesHeartCash +
                       GetVPSession(i)->TotalRoyalFlushesDiamondCash +
                       GetVPSession(i)->TotalRoyalFlushesSpadeCash +
                       GetVPSession(i)->TotalRoyalFlushesClubCash +
                       GetVPSession(i)->TotalFiveOfKinds6_KCash +
                       GetVPSession(i)->TotalFiveOfKinds3_5Cash +
                       GetVPSession(i)->TotalFiveOfKindsAcesCash +
                       GetVPSession(i)->TotalFourOfKinds2WAceCash +
                       GetVPSession(i)->TotalFourOfKinds7WJokerCash +
                       GetVPSession(i)->TotalFiveOfKindsJokerCash +
                       GetVPSession(i)->TotalThreeOfKinds5Cash +
                       GetVPSession(i)->TotalThreeOfKinds8Cash +
                       GetVPSession(i)->TotalFourOfKinds5Cash +
                       GetVPSession(i)->TotalFourOfKinds7Cash +
                       GetVPSession(i)->TotalFourOfKinds8Cash;
    #endif
    Total_TotalCashWon += TotalCashWon[i];
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercHighPairsCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercHighPairsCashWonStr[i] = MakeString(PercHighPairsCashWon[i]);
  }
  Total_PercHighPairsCashWonStr = MakeString(Total_PercHighPairsCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercHighPairsCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercHighPairsCashWon[i] = GetPerc(GetVPSession(i)->TotalHighPairCash,TotalCashWon[i]);
  }
  Total_PercHighPairsCashWon = GetPerc(Total_HighPairsCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercTwoPairsCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercTwoPairsCashWonStr[i] = MakeString(PercTwoPairsCashWon[i]);
  }
  Total_PercTwoPairsCashWonStr = MakeString(Total_PercTwoPairsCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercTwoPairsCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercTwoPairsCashWon[i] = GetPerc(GetVPSession(i)->TotalTwoPairCash,TotalCashWon[i]);
  }
  Total_PercTwoPairsCashWon = GetPerc(Total_TwoPairsCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercThreeOfKindsCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercThreeOfKindsCashWonStr[i] = MakeString(PercThreeOfKindsCashWon[i]);
  }
  Total_PercThreeOfKindsCashWonStr = MakeString(Total_PercThreeOfKindsCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercThreeOfKindsCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercThreeOfKindsCashWon[i] = GetPerc(GetVPSession(i)->TotalThreeOfKindCash,TotalCashWon[i]);
  }
  Total_PercThreeOfKindsCashWon = GetPerc(Total_ThreeOfKindsCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercStraightsCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightsCashWonStr[i] = MakeString(PercStraightsCashWon[i]);
  }
  Total_PercStraightsCashWonStr = MakeString(Total_PercStraightsCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercStraightsCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightsCashWon[i] = GetPerc(GetVPSession(i)->TotalStraightCash,TotalCashWon[i]);
  }
  Total_PercStraightsCashWon = GetPerc(Total_StraightsCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercFlushesCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushesCashWonStr[i] = MakeString(PercFlushesCashWon[i]);
  }
  Total_PercFlushesCashWonStr = MakeString(Total_PercFlushesCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFlushesCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushesCashWon[i] = GetPerc(GetVPSession(i)->TotalFlushCash,TotalCashWon[i]);
  }
  Total_PercFlushesCashWon = GetPerc(Total_FlushesCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercFullHousesCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFullHousesCashWonStr[i] = MakeString(PercFullHousesCashWon[i]);
  }
  Total_PercFullHousesCashWonStr = MakeString(Total_PercFullHousesCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFullHousesCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFullHousesCashWon[i] = GetPerc(GetVPSession(i)->TotalFullHouseCash,TotalCashWon[i]);
  }
  Total_PercFullHousesCashWon = GetPerc(Total_FullHousesCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKindsCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsCashWonStr[i] = MakeString(PercFourOfKindsCashWon[i]);
  }
  Total_PercFourOfKindsCashWonStr = MakeString(Total_PercFourOfKindsCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKindsCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKindCash,TotalCashWon[i]);
  }
  Total_PercFourOfKindsCashWon = GetPerc(Total_FourOfKindsCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercStraightFlushesCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesCashWonStr[i] = MakeString(PercStraightFlushesCashWon[i]);
  }
  Total_PercStraightFlushesCashWonStr = MakeString(Total_PercStraightFlushesCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercStraightFlushesCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesCashWon[i] = GetPerc(GetVPSession(i)->TotalStraightFlushCash,TotalCashWon[i]);
  }
  Total_PercStraightFlushesCashWon = GetPerc(Total_StraightFlushesCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercRoyalFlushesNaturalCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesNaturalCashWonStr[i] = MakeString(PercRoyalFlushesNaturalCashWon[i]);
  }
  Total_PercRoyalFlushesNaturalCashWonStr = MakeString(Total_PercRoyalFlushesNaturalCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercRoyalFlushesNaturalCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesNaturalCashWon[i] = GetPerc(GetVPSession(i)->TotalNaturalRoyalCash,
                                                TotalCashWon[i]);
  }
  Total_PercRoyalFlushesNaturalCashWon = GetPerc(Total_RoyalFlushesNaturalCash,
                                                 Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercRoyalFlushesWildCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesWildCashWonStr[i] = MakeString(PercRoyalFlushesWildCashWon[i]);
  }
  Total_PercRoyalFlushesWildCashWonStr = MakeString(Total_PercRoyalFlushesWildCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercRoyalFlushesWildCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesWildCashWon[i] = GetPerc(GetVPSession(i)->TotalWildRoyalCash,TotalCashWon[i]);
  }
  Total_PercRoyalFlushesWildCashWon = GetPerc(Total_RoyalFlushesWildCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercFourDeucesCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourDeucesCashWonStr[i] = MakeString(PercFourDeucesCashWon[i]);
  }
  Total_PercFourDeucesCashWonStr = MakeString(Total_PercFourDeucesCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourDeucesCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourDeucesCashWon[i] = GetPerc(GetVPSession(i)->TotalFourDeuceCash,TotalCashWon[i]);
  }
  Total_PercFourDeucesCashWon = GetPerc(Total_FourDeucesCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_PercFiveOfKindsCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKindsCashWonStr[i] = MakeString(PercFiveOfKindsCashWon[i]);
  }
  Total_PercFiveOfKindsCashWonStr = MakeString(Total_PercFiveOfKindsCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFiveOfKindsCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKindsCashWon[i] = GetPerc(GetVPSession(i)->TotalFiveOfKindCash,TotalCashWon[i]);
  }
  Total_PercFiveOfKindsCashWon = GetPerc(Total_FiveOfKindsCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_NumBadStrategyPlaysStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    NumBadStrategyPlaysStr[i] = MakeString(GetVPSession(i)->IncorrectPlays);
  }
  Total_NumBadStrategyPlaysStr = MakeString(Total_NumBadStrategyPlays);
}
//*******************************************************************
void TStatCalcVP::Calc_NumBadStrategyPlays()
{
  Total_NumBadStrategyPlays = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_NumBadStrategyPlays += GetVPSession(i)->IncorrectPlays;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercPlayingAccuracyStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercPlayingAccuracyStr[i] = MakeString(PercPlayingAccuracy[i]);
  }
  Total_PercPlayingAccuracyStr = MakeString(Total_PercPlayingAccuracy);
}
//*******************************************************************
void TStatCalcVP::Calc_PercPlayingAccuracy()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
	 PercPlayingAccuracy[i] = GetPerc(TotalNumBets[i]-GetVPSession(i)->IncorrectPlays,TotalNumBets[i]);
  }
  Total_PercPlayingAccuracy = GetPerc(Total_TotalNumBets-Total_NumBadStrategyPlays,Total_TotalNumBets);
}
//*******************************************************************
#if defined(ACVP_COMPILE)
void TStatCalcVP::Make_Pair6Strs()
{
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Pair6Str[i] = MakeString(GetVPSession(i)->Pair6);
  }
  Total_Pair6Str = MakeString(Total_Pair6);
}
//*******************************************************************
void TStatCalcVP::Calc_Pair6()
{
  Total_Pair6 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Pair6 += GetVPSession(i)->Pair6;
  }
}
//*******************************************************************
void TStatCalcVP::Make_Pair6CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Pair6CashStr[i] = MakeString(GetVPSession(i)->TotalPair6Cash);
  }
  Total_Pair6CashStr = MakeString(Total_Pair6Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_Pair6Cash()
{
  Total_Pair6Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_Pair6Cash += GetVPSession(i)->TotalPair6Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercPair6CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercPair6CashWonStr[i] = MakeString(PercPair6CashWon[i]);
  }
  Total_PercPair6CashWonStr = MakeString(Total_PercPair6CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercPair6CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercPair6CashWon[i] = GetPerc(GetVPSession(i)->TotalPair6Cash,TotalCashWon[i]);
  }
  Total_PercPair6CashWon = GetPerc(Total_Pair6Cash,Total_TotalCashWon);
}
//*******************************************************************
//void TStatCalcVP::Make_PairJacksStrs()
//{
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    PairJacksStr[i] = MakeString(GetVPSession(i)->PairJacks);
//  }
//  Total_PairJacksStr = MakeString(Total_PairJacks);
//}
////*******************************************************************
//void TStatCalcVP::Calc_PairJacks()
//{
//  Total_PairJacks = 0;
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    Total_PairJacks += GetVPSession(i)->PairJacks;
//  }
//}
////*******************************************************************
//void TStatCalcVP::Make_PairJacksCashStrs()
//{
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    PairJacksCashStr[i] = MakeString(GetVPSession(i)->TotalPairJacksCash);
//  }
//  Total_PairJacksCashStr = MakeString(Total_PairJacksCash);
//}
////*******************************************************************
//void TStatCalcVP::Calc_PairJacksCash()
//{
//  Total_PairJacksCash = 0;
//  for (int i = 0;i<=LastSessionNum;i++)
//  {
//    Total_PairJacksCash += GetVPSession(i)->TotalPairJacksCash;
//  }
//}
////*******************************************************************
//void TStatCalcVP::Make_PercPairJacksCashWonStrs()
//{
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    PercPairJacksCashWonStr[i] = MakeString(PercPairJacksCashWon[i]);
//  }
//  Total_PercPairJacksCashWonStr = MakeString(Total_PercPairJacksCashWon);
//}
////*******************************************************************
//void TStatCalcVP::Calc_PercPairJacksCashWon()
//{
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    PercPairJacksCashWon[i] = GetPerc(GetVPSession(i)->TotalPairJacksCash,TotalCashWon[i]);
//  }
//  Total_PercPairJacksCashWon = GetPerc(Total_PairJacksCash,Total_TotalCashWon);
//}
////*******************************************************************
//void TStatCalcVP::Make_PairKingsStrs()
//{
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    PairKingsStr[i] = MakeString(GetVPSession(i)->PairKings);
//  }
//  Total_PairKingsStr = MakeString(Total_PairKings);
//}
////*******************************************************************
//void TStatCalcVP::Calc_PairKings()
//{
//  Total_PairKings = 0;
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    Total_PairKings += GetVPSession(i)->PairKings;
//  }
//}
////*******************************************************************
//void TStatCalcVP::Make_PairKingsCashStrs()
//{
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    PairKingsCashStr[i] = MakeString(GetVPSession(i)->TotalPairKingsCash);
//  }
//  Total_PairKingsCashStr = MakeString(Total_PairKingsCash);
//}
////*******************************************************************
//void TStatCalcVP::Calc_PairKingsCash()
//{
//  Total_PairKingsCash = 0;
//  for (int i = 0;i<=LastSessionNum;i++)
//  {
//    Total_PairKingsCash += GetVPSession(i)->TotalPairKingsCash;
//  }
//}
////*******************************************************************
//void TStatCalcVP::Make_PercPairKingsCashWonStrs()
//{
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    PercPairKingsCashWonStr[i] = MakeString(PercPairKingsCashWon[i]);
//  }
//  Total_PercPairKingsCashWonStr = MakeString(Total_PercPairKingsCashWon);
//}
////*******************************************************************
//void TStatCalcVP::Calc_PercPairKingsCashWon()
//{
//  for (int i=0;i<=LastSessionNum;i++)
//  {
//    PercPairKingsCashWon[i] = GetPerc(GetVPSession(i)->TotalPairKingsCash,TotalCashWon[i]);
//  }
//  Total_PercPairKingsCashWon = GetPerc(Total_PairKingsCash,Total_TotalCashWon);
//}
//*******************************************************************
void TStatCalcVP::Make_ThreeOfKinds5Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeOfKinds5Str[i] = MakeString(GetVPSession(i)->ThreeOfKinds5);
  }
  Total_ThreeOfKinds5Str = MakeString(Total_ThreeOfKinds5);
}
//*******************************************************************
void TStatCalcVP::Calc_ThreeOfKinds5()
{
  Total_ThreeOfKinds5 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeOfKinds5 += GetVPSession(i)->ThreeOfKinds5;
  }
}
//*******************************************************************
void TStatCalcVP::Make_ThreeOfKinds5CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeOfKinds5CashStr[i] = MakeString(GetVPSession(i)->TotalThreeOfKinds5Cash);
  }
  Total_ThreeOfKinds5CashStr = MakeString(Total_ThreeOfKinds5Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_ThreeOfKinds5Cash()
{
  Total_ThreeOfKinds5Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_ThreeOfKinds5Cash += GetVPSession(i)->TotalThreeOfKinds5Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercThreeOfKinds5CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercThreeOfKinds5CashWonStr[i] = MakeString(PercThreeOfKinds5CashWon[i]);
  }
  Total_PercThreeOfKinds5CashWonStr = MakeString(Total_PercThreeOfKinds5CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercThreeOfKinds5CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercThreeOfKinds5CashWon[i] = GetPerc(GetVPSession(i)->TotalThreeOfKinds5Cash,TotalCashWon[i]);
  }
  Total_PercThreeOfKinds5CashWon = GetPerc(Total_ThreeOfKinds5Cash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_ThreeOfKinds8Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeOfKinds8Str[i] = MakeString(GetVPSession(i)->ThreeOfKinds8);
  }
  Total_ThreeOfKinds8Str = MakeString(Total_ThreeOfKinds8);
}
//*******************************************************************
void TStatCalcVP::Calc_ThreeOfKinds8()
{
  Total_ThreeOfKinds8 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_ThreeOfKinds8 += GetVPSession(i)->ThreeOfKinds8;
  }
}
//*******************************************************************
void TStatCalcVP::Make_ThreeOfKinds8CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    ThreeOfKinds8CashStr[i] = MakeString(GetVPSession(i)->TotalThreeOfKinds8Cash);
  }
  Total_ThreeOfKinds8CashStr = MakeString(Total_ThreeOfKinds8Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_ThreeOfKinds8Cash()
{
  Total_ThreeOfKinds8Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_ThreeOfKinds8Cash += GetVPSession(i)->TotalThreeOfKinds8Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercThreeOfKinds8CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercThreeOfKinds8CashWonStr[i] = MakeString(PercThreeOfKinds8CashWon[i]);
  }
  Total_PercThreeOfKinds8CashWonStr = MakeString(Total_PercThreeOfKinds8CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercThreeOfKinds8CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercThreeOfKinds8CashWon[i] = GetPerc(GetVPSession(i)->TotalThreeOfKinds8Cash,TotalCashWon[i]);
  }
  Total_PercThreeOfKinds8CashWon = GetPerc(Total_ThreeOfKinds8Cash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FlushDiamondStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushDiamondStr[i] = MakeString(GetVPSession(i)->FlushDiamond);
  }
  Total_FlushDiamondStr = MakeString(Total_FlushDiamond);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushDiamond()
{
  Total_FlushDiamond = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FlushDiamond += GetVPSession(i)->FlushDiamond;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FlushDiamondCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushDiamondCashStr[i] = MakeString(GetVPSession(i)->TotalFlushDiamondCash);
  }
  Total_FlushDiamondCashStr = MakeString(Total_FlushDiamondCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushDiamondCash()
{
  Total_FlushDiamondCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FlushDiamondCash += GetVPSession(i)->TotalFlushDiamondCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFlushDiamondCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushDiamondCashWonStr[i] = MakeString(PercFlushDiamondCashWon[i]);
  }
  Total_PercFlushDiamondCashWonStr = MakeString(Total_PercFlushDiamondCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFlushDiamondCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushDiamondCashWon[i] = GetPerc(GetVPSession(i)->TotalFlushDiamondCash,TotalCashWon[i]);
  }
  Total_PercFlushDiamondCashWon = GetPerc(Total_FlushDiamondCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FlushHeartStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushHeartStr[i] = MakeString(GetVPSession(i)->FlushHeart);
  }
  Total_FlushHeartStr = MakeString(Total_FlushHeart);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushHeart()
{
  Total_FlushHeart = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FlushHeart += GetVPSession(i)->FlushHeart;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FlushHeartCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushHeartCashStr[i] = MakeString(GetVPSession(i)->TotalFlushHeartCash);
  }
  Total_FlushHeartCashStr = MakeString(Total_FlushHeartCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushHeartCash()
{
  Total_FlushHeartCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FlushHeartCash += GetVPSession(i)->TotalFlushHeartCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFlushHeartCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushHeartCashWonStr[i] = MakeString(PercFlushHeartCashWon[i]);
  }
  Total_PercFlushHeartCashWonStr = MakeString(Total_PercFlushHeartCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFlushHeartCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushHeartCashWon[i] = GetPerc(GetVPSession(i)->TotalFlushHeartCash,TotalCashWon[i]);
  }
  Total_PercFlushHeartCashWon = GetPerc(Total_FlushHeartCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FlushSpadeStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushSpadeStr[i] = MakeString(GetVPSession(i)->FlushSpade);
  }
  Total_FlushSpadeStr = MakeString(Total_FlushSpade);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushSpade()
{
  Total_FlushSpade = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FlushSpade += GetVPSession(i)->FlushSpade;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FlushSpadeCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushSpadeCashStr[i] = MakeString(GetVPSession(i)->TotalFlushSpadeCash);
  }
  Total_FlushSpadeCashStr = MakeString(Total_FlushSpadeCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushSpadeCash()
{
  Total_FlushSpadeCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FlushSpadeCash += GetVPSession(i)->TotalFlushSpadeCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFlushSpadeCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushSpadeCashWonStr[i] = MakeString(PercFlushSpadeCashWon[i]);
  }
  Total_PercFlushSpadeCashWonStr = MakeString(Total_PercFlushSpadeCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFlushSpadeCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushSpadeCashWon[i] = GetPerc(GetVPSession(i)->TotalFlushSpadeCash,TotalCashWon[i]);
  }
  Total_PercFlushSpadeCashWon = GetPerc(Total_FlushSpadeCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FlushClubStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushClubStr[i] = MakeString(GetVPSession(i)->FlushClub);
  }
  Total_FlushClubStr = MakeString(Total_FlushClub);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushClub()
{
  Total_FlushClub = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FlushClub += GetVPSession(i)->FlushClub;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FlushClubCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushClubCashStr[i] = MakeString(GetVPSession(i)->TotalFlushClubCash);
  }
  Total_FlushClubCashStr = MakeString(Total_FlushClubCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushClubCash()
{
  Total_FlushClubCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FlushClubCash += GetVPSession(i)->TotalFlushClubCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFlushClubCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushClubCashWonStr[i] = MakeString(PercFlushClubCashWon[i]);
  }
  Total_PercFlushClubCashWonStr = MakeString(Total_PercFlushClubCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFlushClubCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushClubCashWon[i] = GetPerc(GetVPSession(i)->TotalFlushClubCash,TotalCashWon[i]);
  }
  Total_PercFlushClubCashWon = GetPerc(Total_FlushClubCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FlushAssaultStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushAssaultStr[i] = MakeString(GetVPSession(i)->FlushAssault);
  }
  Total_FlushAssaultStr = MakeString(Total_FlushAssault);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushAssault()
{
  Total_FlushAssault = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FlushAssault += GetVPSession(i)->FlushAssault;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FlushAssaultCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FlushAssaultCashStr[i] = MakeString(GetVPSession(i)->TotalFlushAssaultCash);
  }
  Total_FlushAssaultCashStr = MakeString(Total_FlushAssaultCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FlushAssaultCash()
{
  Total_FlushAssaultCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FlushAssaultCash += GetVPSession(i)->TotalFlushAssaultCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFlushAssaultCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushAssaultCashWonStr[i] = MakeString(PercFlushAssaultCashWon[i]);
  }
  Total_PercFlushAssaultCashWonStr = MakeString(Total_PercFlushAssaultCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFlushAssaultCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFlushAssaultCashWon[i] = GetPerc(GetVPSession(i)->TotalFlushAssaultCash,TotalCashWon[i]);
  }
  Total_PercFlushAssaultCashWon = GetPerc(Total_FlushAssaultCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds5Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds5Str[i] = MakeString(GetVPSession(i)->FourOfKinds5);
  }
  Total_FourOfKinds5Str = MakeString(Total_FourOfKinds5);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds5()
{
  Total_FourOfKinds5 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds5 += GetVPSession(i)->FourOfKinds5;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds5CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds5CashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds5Cash);
  }
  Total_FourOfKinds5CashStr = MakeString(Total_FourOfKinds5Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds5Cash()
{
  Total_FourOfKinds5Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds5Cash += GetVPSession(i)->TotalFourOfKinds5Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds5CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds5CashWonStr[i] = MakeString(PercFourOfKinds5CashWon[i]);
  }
  Total_PercFourOfKinds5CashWonStr = MakeString(Total_PercFourOfKinds5CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds5CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds5CashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds5Cash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds5CashWon = GetPerc(Total_FourOfKinds5Cash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds8Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds8Str[i] = MakeString(GetVPSession(i)->FourOfKinds8);
  }
  Total_FourOfKinds8Str = MakeString(Total_FourOfKinds8);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds8()
{
  Total_FourOfKinds8 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds8 += GetVPSession(i)->FourOfKinds8;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds8CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds8CashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds8Cash);
  }
  Total_FourOfKinds8CashStr = MakeString(Total_FourOfKinds8Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds8Cash()
{
  Total_FourOfKinds8Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds8Cash += GetVPSession(i)->TotalFourOfKinds8Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds8CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds8CashWonStr[i] = MakeString(PercFourOfKinds8CashWon[i]);
  }
  Total_PercFourOfKinds8CashWonStr = MakeString(Total_PercFourOfKinds8CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds8CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds8CashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds8Cash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds8CashWon = GetPerc(Total_FourOfKinds8Cash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds7Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds7Str[i] = MakeString(GetVPSession(i)->FourOfKinds7);
  }
  Total_FourOfKinds7Str = MakeString(Total_FourOfKinds7);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds7()
{
  Total_FourOfKinds7 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds7 += GetVPSession(i)->FourOfKinds7;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds7CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds7CashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds7Cash);
  }
  Total_FourOfKinds7CashStr = MakeString(Total_FourOfKinds7Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds7Cash()
{
  Total_FourOfKinds7Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds7Cash += GetVPSession(i)->TotalFourOfKinds7Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds7CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds7CashWonStr[i] = MakeString(PercFourOfKinds7CashWon[i]);
  }
  Total_PercFourOfKinds7CashWonStr = MakeString(Total_PercFourOfKinds7CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds7CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds7CashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds7Cash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds7CashWon = GetPerc(Total_FourOfKinds7Cash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds2_10Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds2_10Str[i] = MakeString(GetVPSession(i)->FourOfKinds2_10);
  }
  Total_FourOfKinds2_10Str = MakeString(Total_FourOfKinds2_10);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds2_10()
{
  Total_FourOfKinds2_10 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds2_10 += GetVPSession(i)->FourOfKinds2_10;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds2_10CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds2_10CashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds2_10Cash);
  }
  Total_FourOfKinds2_10CashStr = MakeString(Total_FourOfKinds2_10Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds2_10Cash()
{
  Total_FourOfKinds2_10Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds2_10Cash += GetVPSession(i)->TotalFourOfKinds2_10Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds2_10CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds2_10CashWonStr[i] = MakeString(PercFourOfKinds2_10CashWon[i]);
  }
  Total_PercFourOfKinds2_10CashWonStr = MakeString(Total_PercFourOfKinds2_10CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds2_10CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds2_10CashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds2_10Cash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds2_10CashWon = GetPerc(Total_FourOfKinds2_10Cash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsFacesStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsFacesStr[i] = MakeString(GetVPSession(i)->FourOfKindsFaces);
  }
  Total_FourOfKindsFacesStr = MakeString(Total_FourOfKindsFaces);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsFaces()
{
  Total_FourOfKindsFaces = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsFaces += GetVPSession(i)->FourOfKindsFaces;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsFacesCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsFacesCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKindsFacesCash);
  }
  Total_FourOfKindsFacesCashStr = MakeString(Total_FourOfKindsFacesCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsFacesCash()
{
  Total_FourOfKindsFacesCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsFacesCash += GetVPSession(i)->TotalFourOfKindsFacesCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKindsFacesCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsFacesCashWonStr[i] = MakeString(PercFourOfKindsFacesCashWon[i]);
  }
  Total_PercFourOfKindsFacesCashWonStr = MakeString(Total_PercFourOfKindsFacesCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKindsFacesCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsFacesCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKindsFacesCash,TotalCashWon[i]);
  }
  Total_PercFourOfKindsFacesCashWon = GetPerc(Total_FourOfKindsFacesCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsFacesWAceOrFaceStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsFacesWAceOrFaceStr[i] = MakeString(GetVPSession(i)->FourOfKindsFacesWAceOrFace);
  }
  Total_FourOfKindsFacesWAceOrFaceStr = MakeString(Total_FourOfKindsFacesWAceOrFace);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsFacesWAceOrFace()
{
  Total_FourOfKindsFacesWAceOrFace = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsFacesWAceOrFace += GetVPSession(i)->FourOfKindsFacesWAceOrFace;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsFacesWAceOrFaceCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsFacesWAceOrFaceCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKindsFacesWAceOrFaceCash);
  }
  Total_FourOfKindsFacesWAceOrFaceCashStr = MakeString(Total_FourOfKindsFacesWAceOrFaceCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsFacesWAceOrFaceCash()
{
  Total_FourOfKindsFacesWAceOrFaceCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsFacesWAceOrFaceCash += GetVPSession(i)->TotalFourOfKindsFacesWAceOrFaceCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKindsFacesWAceOrFaceCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsFacesWAceOrFaceCashWonStr[i] = MakeString(PercFourOfKindsFacesWAceOrFaceCashWon[i]);
  }
  Total_PercFourOfKindsFacesWAceOrFaceCashWonStr = MakeString(Total_PercFourOfKindsFacesWAceOrFaceCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKindsFacesWAceOrFaceCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsFacesWAceOrFaceCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKindsFacesWAceOrFaceCash,TotalCashWon[i]);
  }
  Total_PercFourOfKindsFacesWAceOrFaceCashWon = GetPerc(Total_FourOfKindsFacesWAceOrFaceCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsFacesWAceStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsFacesWAceStr[i] = MakeString(GetVPSession(i)->FourOfKindsFacesWAce);
  }
  Total_FourOfKindsFacesWAceStr = MakeString(Total_FourOfKindsFacesWAce);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsFacesWAce()
{
  Total_FourOfKindsFacesWAce = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsFacesWAce += GetVPSession(i)->FourOfKindsFacesWAce;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsFacesWAceCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsFacesWAceCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKindsFacesWAceCash);
  }
  Total_FourOfKindsFacesWAceCashStr = MakeString(Total_FourOfKindsFacesWAceCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsFacesWAceCash()
{
  Total_FourOfKindsFacesWAceCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsFacesWAceCash += GetVPSession(i)->TotalFourOfKindsFacesWAceCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKindsFacesWAceCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsFacesWAceCashWonStr[i] = MakeString(PercFourOfKindsFacesWAceCashWon[i]);
  }
  Total_PercFourOfKindsFacesWAceCashWonStr = MakeString(Total_PercFourOfKindsFacesWAceCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKindsFacesWAceCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsFacesWAceCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKindsFacesWAceCash,TotalCashWon[i]);
  }
  Total_PercFourOfKindsFacesWAceCashWon = GetPerc(Total_FourOfKindsFacesWAceCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsAcesWFaceStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsAcesWFaceStr[i] = MakeString(GetVPSession(i)->FourOfKindsAcesWFace);
  }
  Total_FourOfKindsAcesWFaceStr = MakeString(Total_FourOfKindsAcesWFace);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsAcesWFace()
{
  Total_FourOfKindsAcesWFace = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsAcesWFace += GetVPSession(i)->FourOfKindsAcesWFace;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsAcesWFaceCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsAcesWFaceCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKindsAcesWFaceCash);
  }
  Total_FourOfKindsAcesWFaceCashStr = MakeString(Total_FourOfKindsAcesWFaceCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsAcesWFaceCash()
{
  Total_FourOfKindsAcesWFaceCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsAcesWFaceCash += GetVPSession(i)->TotalFourOfKindsAcesWFaceCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKindsAcesWFaceCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsAcesWFaceCashWonStr[i] = MakeString(PercFourOfKindsAcesWFaceCashWon[i]);
  }
  Total_PercFourOfKindsAcesWFaceCashWonStr = MakeString(Total_PercFourOfKindsAcesWFaceCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKindsAcesWFaceCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsAcesWFaceCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKindsAcesWFaceCash,TotalCashWon[i]);
  }
  Total_PercFourOfKindsAcesWFaceCashWon = GetPerc(Total_FourOfKindsAcesWFaceCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds4_KStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds4_KStr[i] = MakeString(GetVPSession(i)->FourOfKinds4_K);
  }
  Total_FourOfKinds4_KStr = MakeString(Total_FourOfKinds4_K);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds4_K()
{
  Total_FourOfKinds4_K = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds4_K += GetVPSession(i)->FourOfKinds4_K;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds4_KCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds4_KCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds4_KCash);
  }
  Total_FourOfKinds4_KCashStr = MakeString(Total_FourOfKinds4_KCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds4_KCash()
{
  Total_FourOfKinds4_KCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds4_KCash += GetVPSession(i)->TotalFourOfKinds4_KCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds4_KCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds4_KCashWonStr[i] = MakeString(PercFourOfKinds4_KCashWon[i]);
  }
  Total_PercFourOfKinds4_KCashWonStr = MakeString(Total_PercFourOfKinds4_KCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds4_KCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds4_KCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds4_KCash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds4_KCashWon = GetPerc(Total_FourOfKinds4_KCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds5_KStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds5_KStr[i] = MakeString(GetVPSession(i)->FourOfKinds5_K);
  }
  Total_FourOfKinds5_KStr = MakeString(Total_FourOfKinds5_K);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds5_K()
{
  Total_FourOfKinds5_K = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds5_K += GetVPSession(i)->FourOfKinds5_K;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds5_KCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds5_KCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds5_KCash);
  }
  Total_FourOfKinds5_KCashStr = MakeString(Total_FourOfKinds5_KCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds5_KCash()
{
  Total_FourOfKinds5_KCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds5_KCash += GetVPSession(i)->TotalFourOfKinds5_KCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds5_KCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds5_KCashWonStr[i] = MakeString(PercFourOfKinds5_KCashWon[i]);
  }
  Total_PercFourOfKinds5_KCashWonStr = MakeString(Total_PercFourOfKinds5_KCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds5_KCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds5_KCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds5_KCash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds5_KCashWon = GetPerc(Total_FourOfKinds5_KCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds2_4Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds2_4Str[i] = MakeString(GetVPSession(i)->FourOfKinds2_4);
  }
  Total_FourOfKinds2_4Str = MakeString(Total_FourOfKinds2_4);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds2_4()
{
  Total_FourOfKinds2_4 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds2_4 += GetVPSession(i)->FourOfKinds2_4;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds2_4CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds2_4CashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds2_4Cash);
  }
  Total_FourOfKinds2_4CashStr = MakeString(Total_FourOfKinds2_4Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds2_4Cash()
{
  Total_FourOfKinds2_4Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds2_4Cash += GetVPSession(i)->TotalFourOfKinds2_4Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds2_4CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds2_4CashWonStr[i] = MakeString(PercFourOfKinds2_4CashWon[i]);
  }
  Total_PercFourOfKinds2_4CashWonStr = MakeString(Total_PercFourOfKinds2_4CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds2_4CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds2_4CashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds2_4Cash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds2_4CashWon = GetPerc(Total_FourOfKinds2_4Cash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsAcesStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsAcesStr[i] = MakeString(GetVPSession(i)->FourOfKindsAces);
  }
  Total_FourOfKindsAcesStr = MakeString(Total_FourOfKindsAces);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsAces()
{
  Total_FourOfKindsAces = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsAces += GetVPSession(i)->FourOfKindsAces;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKindsAcesCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKindsAcesCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKindsAcesCash);
  }
  Total_FourOfKindsAcesCashStr = MakeString(Total_FourOfKindsAcesCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKindsAcesCash()
{
  Total_FourOfKindsAcesCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKindsAcesCash += GetVPSession(i)->TotalFourOfKindsAcesCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKindsAcesCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsAcesCashWonStr[i] = MakeString(PercFourOfKindsAcesCashWon[i]);
  }
  Total_PercFourOfKindsAcesCashWonStr = MakeString(Total_PercFourOfKindsAcesCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKindsAcesCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKindsAcesCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKindsAcesCash,TotalCashWon[i]);
  }
  Total_PercFourOfKindsAcesCashWon = GetPerc(Total_FourOfKindsAcesCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesDiamondStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesDiamondStr[i] = MakeString(GetVPSession(i)->StraightFlushesDiamond);
  }
  Total_StraightFlushesDiamondStr = MakeString(Total_StraightFlushesDiamond);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesDiamond()
{
  Total_StraightFlushesDiamond = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesDiamond += GetVPSession(i)->StraightFlushesDiamond;
  }
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesDiamondCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesDiamondCashStr[i] = MakeString(GetVPSession(i)->TotalStraightFlushesDiamondCash);
  }
  Total_StraightFlushesDiamondCashStr = MakeString(Total_StraightFlushesDiamondCash);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesDiamondCash()
{
  Total_StraightFlushesDiamondCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesDiamondCash += GetVPSession(i)->TotalStraightFlushesDiamondCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercStraightFlushesDiamondCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesDiamondCashWonStr[i] = MakeString(PercStraightFlushesDiamondCashWon[i]);
  }
  Total_PercStraightFlushesDiamondCashWonStr = MakeString(Total_PercStraightFlushesDiamondCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercStraightFlushesDiamondCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesDiamondCashWon[i] = GetPerc(GetVPSession(i)->TotalStraightFlushesDiamondCash,TotalCashWon[i]);
  }
  Total_PercStraightFlushesDiamondCashWon = GetPerc(Total_StraightFlushesDiamondCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesHeartStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesHeartStr[i] = MakeString(GetVPSession(i)->StraightFlushesHeart);
  }
  Total_StraightFlushesHeartStr = MakeString(Total_StraightFlushesHeart);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesHeart()
{
  Total_StraightFlushesHeart = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesHeart += GetVPSession(i)->StraightFlushesHeart;
  }
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesHeartCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesHeartCashStr[i] = MakeString(GetVPSession(i)->TotalStraightFlushesHeartCash);
  }
  Total_StraightFlushesHeartCashStr = MakeString(Total_StraightFlushesHeartCash);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesHeartCash()
{
  Total_StraightFlushesHeartCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesHeartCash += GetVPSession(i)->TotalStraightFlushesHeartCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercStraightFlushesHeartCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesHeartCashWonStr[i] = MakeString(PercStraightFlushesHeartCashWon[i]);
  }
  Total_PercStraightFlushesHeartCashWonStr = MakeString(Total_PercStraightFlushesHeartCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercStraightFlushesHeartCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesHeartCashWon[i] = GetPerc(GetVPSession(i)->TotalStraightFlushesHeartCash,TotalCashWon[i]);
  }
  Total_PercStraightFlushesHeartCashWon = GetPerc(Total_StraightFlushesHeartCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesSpadeStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesSpadeStr[i] = MakeString(GetVPSession(i)->StraightFlushesSpade);
  }
  Total_StraightFlushesSpadeStr = MakeString(Total_StraightFlushesSpade);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesSpade()
{
  Total_StraightFlushesSpade = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesSpade += GetVPSession(i)->StraightFlushesSpade;
  }
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesSpadeCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesSpadeCashStr[i] = MakeString(GetVPSession(i)->TotalStraightFlushesSpadeCash);
  }
  Total_StraightFlushesSpadeCashStr = MakeString(Total_StraightFlushesSpadeCash);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesSpadeCash()
{
  Total_StraightFlushesSpadeCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesSpadeCash += GetVPSession(i)->TotalStraightFlushesSpadeCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercStraightFlushesSpadeCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesSpadeCashWonStr[i] = MakeString(PercStraightFlushesSpadeCashWon[i]);
  }
  Total_PercStraightFlushesSpadeCashWonStr = MakeString(Total_PercStraightFlushesSpadeCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercStraightFlushesSpadeCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesSpadeCashWon[i] = GetPerc(GetVPSession(i)->TotalStraightFlushesSpadeCash,TotalCashWon[i]);
  }
  Total_PercStraightFlushesSpadeCashWon = GetPerc(Total_StraightFlushesSpadeCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesClubStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesClubStr[i] = MakeString(GetVPSession(i)->StraightFlushesClub);
  }
  Total_StraightFlushesClubStr = MakeString(Total_StraightFlushesClub);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesClub()
{
  Total_StraightFlushesClub = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesClub += GetVPSession(i)->StraightFlushesClub;
  }
}
//*******************************************************************
void TStatCalcVP::Make_StraightFlushesClubCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StraightFlushesClubCashStr[i] = MakeString(GetVPSession(i)->TotalStraightFlushesClubCash);
  }
  Total_StraightFlushesClubCashStr = MakeString(Total_StraightFlushesClubCash);
}
//*******************************************************************
void TStatCalcVP::Calc_StraightFlushesClubCash()
{
  Total_StraightFlushesClubCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_StraightFlushesClubCash += GetVPSession(i)->TotalStraightFlushesClubCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercStraightFlushesClubCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesClubCashWonStr[i] = MakeString(PercStraightFlushesClubCashWon[i]);
  }
  Total_PercStraightFlushesClubCashWonStr = MakeString(Total_PercStraightFlushesClubCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercStraightFlushesClubCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercStraightFlushesClubCashWon[i] = GetPerc(GetVPSession(i)->TotalStraightFlushesClubCash,TotalCashWon[i]);
  }
  Total_PercStraightFlushesClubCashWon = GetPerc(Total_StraightFlushesClubCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesHeartStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesHeartStr[i] = MakeString(GetVPSession(i)->RoyalFlushesHeart);
  }
  Total_RoyalFlushesHeartStr = MakeString(Total_RoyalFlushesHeart);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesHeart()
{
  Total_RoyalFlushesHeart = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesHeart += GetVPSession(i)->RoyalFlushesHeart;
  }
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesHeartCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesHeartCashStr[i] = MakeString(GetVPSession(i)->TotalRoyalFlushesHeartCash);
  }
  Total_RoyalFlushesHeartCashStr = MakeString(Total_RoyalFlushesHeartCash);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesHeartCash()
{
  Total_RoyalFlushesHeartCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesHeartCash += GetVPSession(i)->TotalRoyalFlushesHeartCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercRoyalFlushesHeartCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesHeartCashWonStr[i] = MakeString(PercRoyalFlushesHeartCashWon[i]);
  }
  Total_PercRoyalFlushesHeartCashWonStr = MakeString(Total_PercRoyalFlushesHeartCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercRoyalFlushesHeartCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesHeartCashWon[i] = GetPerc(GetVPSession(i)->TotalRoyalFlushesHeartCash,TotalCashWon[i]);
  }
  Total_PercRoyalFlushesHeartCashWon = GetPerc(Total_RoyalFlushesHeartCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesDiamondStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesDiamondStr[i] = MakeString(GetVPSession(i)->RoyalFlushesDiamond);
  }
  Total_RoyalFlushesDiamondStr = MakeString(Total_RoyalFlushesDiamond);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesDiamond()
{
  Total_RoyalFlushesDiamond = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesDiamond += GetVPSession(i)->RoyalFlushesDiamond;
  }
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesDiamondCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesDiamondCashStr[i] = MakeString(GetVPSession(i)->TotalRoyalFlushesDiamondCash);
  }
  Total_RoyalFlushesDiamondCashStr = MakeString(Total_RoyalFlushesDiamondCash);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesDiamondCash()
{
  Total_RoyalFlushesDiamondCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesDiamondCash += GetVPSession(i)->TotalRoyalFlushesDiamondCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercRoyalFlushesDiamondCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesDiamondCashWonStr[i] = MakeString(PercRoyalFlushesDiamondCashWon[i]);
  }
  Total_PercRoyalFlushesDiamondCashWonStr = MakeString(Total_PercRoyalFlushesDiamondCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercRoyalFlushesDiamondCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesDiamondCashWon[i] = GetPerc(GetVPSession(i)->TotalRoyalFlushesDiamondCash,TotalCashWon[i]);
  }
  Total_PercRoyalFlushesDiamondCashWon = GetPerc(Total_RoyalFlushesDiamondCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesSpadeStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesSpadeStr[i] = MakeString(GetVPSession(i)->RoyalFlushesSpade);
  }
  Total_RoyalFlushesSpadeStr = MakeString(Total_RoyalFlushesSpade);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesSpade()
{
  Total_RoyalFlushesSpade = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesSpade += GetVPSession(i)->RoyalFlushesSpade;
  }
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesSpadeCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesSpadeCashStr[i] = MakeString(GetVPSession(i)->TotalRoyalFlushesSpadeCash);
  }
  Total_RoyalFlushesSpadeCashStr = MakeString(Total_RoyalFlushesSpadeCash);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesSpadeCash()
{
  Total_RoyalFlushesSpadeCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesSpadeCash += GetVPSession(i)->TotalRoyalFlushesSpadeCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercRoyalFlushesSpadeCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesSpadeCashWonStr[i] = MakeString(PercRoyalFlushesSpadeCashWon[i]);
  }
  Total_PercRoyalFlushesSpadeCashWonStr = MakeString(Total_PercRoyalFlushesSpadeCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercRoyalFlushesSpadeCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesSpadeCashWon[i] = GetPerc(GetVPSession(i)->TotalRoyalFlushesSpadeCash,TotalCashWon[i]);
  }
  Total_PercRoyalFlushesSpadeCashWon = GetPerc(Total_RoyalFlushesSpadeCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesClubStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesClubStr[i] = MakeString(GetVPSession(i)->RoyalFlushesClub);
  }
  Total_RoyalFlushesClubStr = MakeString(Total_RoyalFlushesClub);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesClub()
{
  Total_RoyalFlushesClub = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesClub += GetVPSession(i)->RoyalFlushesClub;
  }
}
//*******************************************************************
void TStatCalcVP::Make_RoyalFlushesClubCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    RoyalFlushesClubCashStr[i] = MakeString(GetVPSession(i)->TotalRoyalFlushesClubCash);
  }
  Total_RoyalFlushesClubCashStr = MakeString(Total_RoyalFlushesClubCash);
}
//*******************************************************************
void TStatCalcVP::Calc_RoyalFlushesClubCash()
{
  Total_RoyalFlushesClubCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_RoyalFlushesClubCash += GetVPSession(i)->TotalRoyalFlushesClubCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercRoyalFlushesClubCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesClubCashWonStr[i] = MakeString(PercRoyalFlushesClubCashWon[i]);
  }
  Total_PercRoyalFlushesClubCashWonStr = MakeString(Total_PercRoyalFlushesClubCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercRoyalFlushesClubCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRoyalFlushesClubCashWon[i] = GetPerc(GetVPSession(i)->TotalRoyalFlushesClubCash,TotalCashWon[i]);
  }
  Total_PercRoyalFlushesClubCashWon = GetPerc(Total_RoyalFlushesClubCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKinds6_KStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKinds6_KStr[i] = MakeString(GetVPSession(i)->FiveOfKinds6_K);
  }
  Total_FiveOfKinds6_KStr = MakeString(Total_FiveOfKinds6_K);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKinds6_K()
{
  Total_FiveOfKinds6_K = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKinds6_K += GetVPSession(i)->FiveOfKinds6_K;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKinds6_KCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKinds6_KCashStr[i] = MakeString(GetVPSession(i)->TotalFiveOfKinds6_KCash);
  }
  Total_FiveOfKinds6_KCashStr = MakeString(Total_FiveOfKinds6_KCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKinds6_KCash()
{
  Total_FiveOfKinds6_KCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKinds6_KCash += GetVPSession(i)->TotalFiveOfKinds6_KCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFiveOfKinds6_KCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKinds6_KCashWonStr[i] = MakeString(PercFiveOfKinds6_KCashWon[i]);
  }
  Total_PercFiveOfKinds6_KCashWonStr = MakeString(Total_PercFiveOfKinds6_KCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFiveOfKinds6_KCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKinds6_KCashWon[i] = GetPerc(GetVPSession(i)->TotalFiveOfKinds6_KCash,TotalCashWon[i]);
  }
  Total_PercFiveOfKinds6_KCashWon = GetPerc(Total_FiveOfKinds6_KCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKinds3_5Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKinds3_5Str[i] = MakeString(GetVPSession(i)->FiveOfKinds3_5);
  }
  Total_FiveOfKinds3_5Str = MakeString(Total_FiveOfKinds3_5);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKinds3_5()
{
  Total_FiveOfKinds3_5 = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKinds3_5 += GetVPSession(i)->FiveOfKinds3_5;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKinds3_5CashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKinds3_5CashStr[i] = MakeString(GetVPSession(i)->TotalFiveOfKinds3_5Cash);
  }
  Total_FiveOfKinds3_5CashStr = MakeString(Total_FiveOfKinds3_5Cash);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKinds3_5Cash()
{
  Total_FiveOfKinds3_5Cash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKinds3_5Cash += GetVPSession(i)->TotalFiveOfKinds3_5Cash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFiveOfKinds3_5CashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKinds3_5CashWonStr[i] = MakeString(PercFiveOfKinds3_5CashWon[i]);
  }
  Total_PercFiveOfKinds3_5CashWonStr = MakeString(Total_PercFiveOfKinds3_5CashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFiveOfKinds3_5CashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKinds3_5CashWon[i] = GetPerc(GetVPSession(i)->TotalFiveOfKinds3_5Cash,TotalCashWon[i]);
  }
  Total_PercFiveOfKinds3_5CashWon = GetPerc(Total_FiveOfKinds3_5Cash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKindsAcesStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKindsAcesStr[i] = MakeString(GetVPSession(i)->FiveOfKindsAces);
  }
  Total_FiveOfKindsAcesStr = MakeString(Total_FiveOfKindsAces);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKindsAces()
{
  Total_FiveOfKindsAces = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKindsAces += GetVPSession(i)->FiveOfKindsAces;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKindsAcesCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKindsAcesCashStr[i] = MakeString(GetVPSession(i)->TotalFiveOfKindsAcesCash);
  }
  Total_FiveOfKindsAcesCashStr = MakeString(Total_FiveOfKindsAcesCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKindsAcesCash()
{
  Total_FiveOfKindsAcesCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKindsAcesCash += GetVPSession(i)->TotalFiveOfKindsAcesCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFiveOfKindsAcesCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKindsAcesCashWonStr[i] = MakeString(PercFiveOfKindsAcesCashWon[i]);
  }
  Total_PercFiveOfKindsAcesCashWonStr = MakeString(Total_PercFiveOfKindsAcesCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFiveOfKindsAcesCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKindsAcesCashWon[i] = GetPerc(GetVPSession(i)->TotalFiveOfKindsAcesCash,TotalCashWon[i]);
  }
  Total_PercFiveOfKindsAcesCashWon = GetPerc(Total_FiveOfKindsAcesCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds2WAceStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds2WAceStr[i] = MakeString(GetVPSession(i)->FourOfKinds2WAce);
  }
  Total_FourOfKinds2WAceStr = MakeString(Total_FourOfKinds2WAce);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds2WAce()
{
  Total_FourOfKinds2WAce = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds2WAce += GetVPSession(i)->FourOfKinds2WAce;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds2WAceCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds2WAceCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds2WAceCash);
  }
  Total_FourOfKinds2WAceCashStr = MakeString(Total_FourOfKinds2WAceCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds2WAceCash()
{
  Total_FourOfKinds2WAceCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds2WAceCash += GetVPSession(i)->TotalFourOfKinds2WAceCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds2WAceCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds2WAceCashWonStr[i] = MakeString(PercFourOfKinds2WAceCashWon[i]);
  }
  Total_PercFourOfKinds2WAceCashWonStr = MakeString(Total_PercFourOfKinds2WAceCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds2WAceCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds2WAceCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds2WAceCash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds2WAceCashWon = GetPerc(Total_FourOfKinds2WAceCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds7WJokerStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds7WJokerStr[i] = MakeString(GetVPSession(i)->FourOfKinds7WJoker);
  }
  Total_FourOfKinds7WJokerStr = MakeString(Total_FourOfKinds7WJoker);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds7WJoker()
{
  Total_FourOfKinds7WJoker = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds7WJoker += GetVPSession(i)->FourOfKinds7WJoker;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FourOfKinds7WJokerCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FourOfKinds7WJokerCashStr[i] = MakeString(GetVPSession(i)->TotalFourOfKinds7WJokerCash);
  }
  Total_FourOfKinds7WJokerCashStr = MakeString(Total_FourOfKinds7WJokerCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FourOfKinds7WJokerCash()
{
  Total_FourOfKinds7WJokerCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FourOfKinds7WJokerCash += GetVPSession(i)->TotalFourOfKinds7WJokerCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFourOfKinds7WJokerCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds7WJokerCashWonStr[i] = MakeString(PercFourOfKinds7WJokerCashWon[i]);
  }
  Total_PercFourOfKinds7WJokerCashWonStr = MakeString(Total_PercFourOfKinds7WJokerCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFourOfKinds7WJokerCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFourOfKinds7WJokerCashWon[i] = GetPerc(GetVPSession(i)->TotalFourOfKinds7WJokerCash,TotalCashWon[i]);
  }
  Total_PercFourOfKinds7WJokerCashWon = GetPerc(Total_FourOfKinds7WJokerCash,Total_TotalCashWon);
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKindsJokerStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKindsJokerStr[i] = MakeString(GetVPSession(i)->FiveOfKindsJoker);
  }
  Total_FiveOfKindsJokerStr = MakeString(Total_FiveOfKindsJoker);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKindsJoker()
{
  Total_FiveOfKindsJoker = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKindsJoker += GetVPSession(i)->FiveOfKindsJoker;
  }
}
//*******************************************************************
void TStatCalcVP::Make_FiveOfKindsJokerCashStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    FiveOfKindsJokerCashStr[i] = MakeString(GetVPSession(i)->TotalFiveOfKindsJokerCash);
  }
  Total_FiveOfKindsJokerCashStr = MakeString(Total_FiveOfKindsJokerCash);
}
//*******************************************************************
void TStatCalcVP::Calc_FiveOfKindsJokerCash()
{
  Total_FiveOfKindsJokerCash = 0;
  for (int i = 0;i<=LastSessionNum;i++)
  {
    Total_FiveOfKindsJokerCash += GetVPSession(i)->TotalFiveOfKindsJokerCash;
  }
}
//*******************************************************************
void TStatCalcVP::Make_PercFiveOfKindsJokerCashWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKindsJokerCashWonStr[i] = MakeString(PercFiveOfKindsJokerCashWon[i]);
  }
  Total_PercFiveOfKindsJokerCashWonStr = MakeString(Total_PercFiveOfKindsJokerCashWon);
}
//*******************************************************************
void TStatCalcVP::Calc_PercFiveOfKindsJokerCashWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercFiveOfKindsJokerCashWon[i] = GetPerc(GetVPSession(i)->TotalFiveOfKindsJokerCash,TotalCashWon[i]);
  }
  Total_PercFiveOfKindsJokerCashWon = GetPerc(Total_FiveOfKindsJokerCash,Total_TotalCashWon);
}
//*******************************************************************
#endif
//*******************************************************************
void TStatCalcVP::LocalCalculate()
{
  TCasinoStatCalc::LocalCalculate();

  if (PPlayerData == NULL) {return;}

  Calc_CumulativeCashWL();
  Calc_HandsPlayed();
  Calc_NumGarbageHands();
  Calc_HighPairs();
  Calc_TwoPairs();
  Calc_ThreeOfKinds();
  Calc_Straights();
  Calc_Flushes();
  Calc_FullHouses();
  Calc_FourOfKinds();
  Calc_StraightFlushes();
  Calc_RoyalFlushesNatural();
  Calc_RoyalFlushesWild();
  Calc_FourDeuces();
  Calc_FiveOfKinds();

  #if defined(ACVP_COMPILE)
  Calc_Pair6();
//  Calc_PairJacks();
//  Calc_PairKings();
  Calc_ThreeOfKinds5();
  Calc_ThreeOfKinds8();
  Calc_FlushDiamond();
  Calc_FlushHeart();
  Calc_FlushSpade();
  Calc_FlushClub();
  Calc_FlushAssault();
  Calc_FourOfKinds5();
  Calc_FourOfKinds8();
  Calc_FourOfKinds7();
  Calc_FourOfKinds2_10();
  Calc_FourOfKindsFaces();
  Calc_FourOfKindsFacesWAceOrFace();
  Calc_FourOfKindsFacesWAce();
  Calc_FourOfKindsAcesWFace();
  Calc_FourOfKinds4_K();
  Calc_FourOfKinds5_K();
  Calc_FourOfKinds2_4();
  Calc_FourOfKindsAces();
  Calc_StraightFlushesDiamond();
  Calc_StraightFlushesHeart();
  Calc_StraightFlushesSpade();
  Calc_StraightFlushesClub();
  Calc_RoyalFlushesHeart();
  Calc_RoyalFlushesDiamond();
  Calc_RoyalFlushesSpade();
  Calc_RoyalFlushesClub();
  Calc_FiveOfKinds6_K();
  Calc_FiveOfKinds3_5();
  Calc_FiveOfKindsAces();
  Calc_FourOfKinds2WAce();
  Calc_FourOfKinds7WJoker();
  Calc_FiveOfKindsJoker();
  #endif

  Calc_HighPairsCash();
  Calc_TwoPairsCash();
  Calc_ThreeOfKindsCash();
  Calc_StraightsCash();
  Calc_FlushesCash();
  Calc_FullHousesCash();
  Calc_FourOfKindsCash();
  Calc_StraightFlushesCash();
  Calc_RoyalFlushesNaturalCash();
  Calc_RoyalFlushesWildCash();
  Calc_FourDeucesCash();
  Calc_FiveOfKindsCash();

  #if defined(ACVP_COMPILE)
  Calc_Pair6Cash();
//  Calc_PairJacksCash();
//  Calc_PairKingsCash();
  Calc_ThreeOfKinds5Cash();
  Calc_ThreeOfKinds8Cash();
  Calc_FlushDiamondCash();
  Calc_FlushHeartCash();
  Calc_FlushSpadeCash();
  Calc_FlushClubCash();
  Calc_FlushAssaultCash();
  Calc_FourOfKinds5Cash();
  Calc_FourOfKinds8Cash();
  Calc_FourOfKinds7Cash();
  Calc_FourOfKinds2_10Cash();
  Calc_FourOfKindsFacesCash();
  Calc_FourOfKindsFacesWAceOrFaceCash();
  Calc_FourOfKindsFacesWAceCash();
  Calc_FourOfKindsAcesWFaceCash();
  Calc_FourOfKinds4_KCash();
  Calc_FourOfKinds5_KCash();
  Calc_FourOfKinds2_4Cash();
  Calc_FourOfKindsAcesCash();
  Calc_StraightFlushesDiamondCash();
  Calc_StraightFlushesHeartCash();
  Calc_StraightFlushesSpadeCash();
  Calc_StraightFlushesClubCash();
  Calc_RoyalFlushesHeartCash();
  Calc_RoyalFlushesDiamondCash();
  Calc_RoyalFlushesSpadeCash();
  Calc_RoyalFlushesClubCash();
  Calc_FiveOfKinds6_KCash();
  Calc_FiveOfKinds3_5Cash();
  Calc_FiveOfKindsAcesCash();
  Calc_FourOfKinds2WAceCash();
  Calc_FourOfKinds7WJokerCash();
  Calc_FiveOfKindsJokerCash();
  #endif

  Calc_TotalCashWon();
  Calc_PercHighPairsCashWon();
  Calc_PercTwoPairsCashWon();
  Calc_PercThreeOfKindsCashWon();
  Calc_PercStraightsCashWon();
  Calc_PercFlushesCashWon();
  Calc_PercFullHousesCashWon();
  Calc_PercFourOfKindsCashWon();
  Calc_PercStraightFlushesCashWon();
  Calc_PercRoyalFlushesNaturalCashWon();
  Calc_PercRoyalFlushesWildCashWon();
  Calc_PercFourDeucesCashWon();
  Calc_PercFiveOfKindsCashWon();
  Calc_NumBadStrategyPlays();
  Calc_PercPlayingAccuracy();

  #if defined(ACVP_COMPILE)
  Calc_PercPair6CashWon();
//  Calc_PercPairJacksCashWon();
//  Calc_PercPairKingsCashWon();
  Calc_PercThreeOfKinds5CashWon();
  Calc_PercThreeOfKinds8CashWon();
  Calc_PercFlushDiamondCashWon();
  Calc_PercFlushHeartCashWon();
  Calc_PercFlushSpadeCashWon();
  Calc_PercFlushClubCashWon();
  Calc_PercFlushAssaultCashWon();
  Calc_PercFourOfKinds5CashWon();
  Calc_PercFourOfKinds8CashWon();
  Calc_PercFourOfKinds7CashWon();
  Calc_PercFourOfKinds2_10CashWon();
  Calc_PercFourOfKindsFacesCashWon();
  Calc_PercFourOfKindsFacesWAceOrFaceCashWon();
  Calc_PercFourOfKindsFacesWAceCashWon();
  Calc_PercFourOfKindsAcesWFaceCashWon();
  Calc_PercFourOfKinds4_KCashWon();
  Calc_PercFourOfKinds5_KCashWon();
  Calc_PercFourOfKinds2_4CashWon();
  Calc_PercFourOfKindsAcesCashWon();
  Calc_PercStraightFlushesDiamondCashWon();
  Calc_PercStraightFlushesHeartCashWon();
  Calc_PercStraightFlushesSpadeCashWon();
  Calc_PercStraightFlushesClubCashWon();
  Calc_PercRoyalFlushesHeartCashWon();
  Calc_PercRoyalFlushesDiamondCashWon();
  Calc_PercRoyalFlushesSpadeCashWon();
  Calc_PercRoyalFlushesClubCashWon();
  Calc_PercFiveOfKinds6_KCashWon();
  Calc_PercFiveOfKinds3_5CashWon();
  Calc_PercFiveOfKindsAcesCashWon();
  Calc_PercFourOfKinds2WAceCashWon();
  Calc_PercFourOfKinds7WJokerCashWon();
  Calc_PercFiveOfKindsJokerCashWon();
  #endif
}
//*******************************************************************
void TStatCalcVP::LocalMakeAllStrings()
{
  TCasinoStatCalc::LocalMakeAllStrings();

  if (PPlayerData == NULL) {return;}

  Make_CumulativeCashWLStrs();
  Make_HandsPlayedStrs();
  Make_NumGarbageHandsStrs();
  Make_HighPairsStrs();
  Make_TwoPairsStrs();
  Make_ThreeOfKindsStrs();
  Make_StraightsStrs();
  Make_FlushesStrs();
  Make_FullHousesStrs();
  Make_FourOfKindsStrs();
  Make_StraightFlushesStrs();
  Make_RoyalFlushesNaturalStrs();
  Make_RoyalFlushesWildStrs();
  Make_FourDeucesStrs();
  Make_FiveOfKindsStrs();

  #if defined(ACVP_COMPILE)
  Make_Pair6Strs();
//  Make_PairJacksStrs();
//  Make_PairKingsStrs();
  Make_ThreeOfKinds5Strs();
  Make_ThreeOfKinds8Strs();
  Make_FlushDiamondStrs();
  Make_FlushHeartStrs();
  Make_FlushSpadeStrs();
  Make_FlushClubStrs();
  Make_FlushAssaultStrs();
  Make_FourOfKinds5Strs();
  Make_FourOfKinds8Strs();
  Make_FourOfKinds7Strs();
  Make_FourOfKinds2_10Strs();
  Make_FourOfKindsFacesStrs();
  Make_FourOfKindsFacesWAceOrFaceStrs();
  Make_FourOfKindsFacesWAceStrs();
  Make_FourOfKindsAcesWFaceStrs();
  Make_FourOfKinds4_KStrs();
  Make_FourOfKinds5_KStrs();
  Make_FourOfKinds2_4Strs();
  Make_FourOfKindsAcesStrs();
  Make_StraightFlushesDiamondStrs();
  Make_StraightFlushesHeartStrs();
  Make_StraightFlushesSpadeStrs();
  Make_StraightFlushesClubStrs();
  Make_RoyalFlushesHeartStrs();
  Make_RoyalFlushesDiamondStrs();
  Make_RoyalFlushesSpadeStrs();
  Make_RoyalFlushesClubStrs();
  Make_FiveOfKinds6_KStrs();
  Make_FiveOfKinds3_5Strs();
  Make_FiveOfKindsAcesStrs();
  Make_FourOfKinds2WAceStrs();
  Make_FourOfKinds7WJokerStrs();
  Make_FiveOfKindsJokerStrs();
  #endif

  Make_HighPairsCashStrs();
  Make_TwoPairsCashStrs();
  Make_ThreeOfKindsCashStrs();
  Make_StraightsCashStrs();
  Make_FlushesCashStrs();
  Make_FullHousesCashStrs();
  Make_FourOfKindsCashStrs();
  Make_StraightFlushesCashStrs();
  Make_RoyalFlushesNaturalCashStrs();
  Make_RoyalFlushesWildCashStrs();
  Make_FourDeucesCashStrs();
  Make_FiveOfKindsCashStrs();
  Make_TotalCashWonStrs();

  #if defined(ACVP_COMPILE)
  Make_Pair6CashStrs();
//  Make_PairJacksCashStrs();
//  Make_PairKingsCashStrs();
  Make_ThreeOfKinds5CashStrs();
  Make_ThreeOfKinds8CashStrs();
  Make_FlushDiamondCashStrs();
  Make_FlushHeartCashStrs();
  Make_FlushSpadeCashStrs();
  Make_FlushClubCashStrs();
  Make_FlushAssaultCashStrs();
  Make_FourOfKinds5CashStrs();
  Make_FourOfKinds8CashStrs();
  Make_FourOfKinds7CashStrs();
  Make_FourOfKinds2_10CashStrs();
  Make_FourOfKindsFacesCashStrs();
  Make_FourOfKindsFacesWAceOrFaceCashStrs();
  Make_FourOfKindsFacesWAceCashStrs();
  Make_FourOfKindsAcesWFaceCashStrs();
  Make_FourOfKinds4_KCashStrs();
  Make_FourOfKinds5_KCashStrs();
  Make_FourOfKinds2_4CashStrs();
  Make_FourOfKindsAcesCashStrs();
  Make_StraightFlushesDiamondCashStrs();
  Make_StraightFlushesHeartCashStrs();
  Make_StraightFlushesSpadeCashStrs();
  Make_StraightFlushesClubCashStrs();
  Make_RoyalFlushesHeartCashStrs();
  Make_RoyalFlushesDiamondCashStrs();
  Make_RoyalFlushesSpadeCashStrs();
  Make_RoyalFlushesClubCashStrs();
  Make_FiveOfKinds6_KCashStrs();
  Make_FiveOfKinds3_5CashStrs();
  Make_FiveOfKindsAcesCashStrs();
  Make_FourOfKinds2WAceCashStrs();
  Make_FourOfKinds7WJokerCashStrs();
  Make_FiveOfKindsJokerCashStrs();
  #endif

  Make_PercHighPairsCashWonStrs();
  Make_PercTwoPairsCashWonStrs();
  Make_PercThreeOfKindsCashWonStrs();
  Make_PercStraightsCashWonStrs();
  Make_PercFlushesCashWonStrs();
  Make_PercFullHousesCashWonStrs();
  Make_PercFourOfKindsCashWonStrs();
  Make_PercStraightFlushesCashWonStrs();
  Make_PercRoyalFlushesNaturalCashWonStrs();
  Make_PercRoyalFlushesWildCashWonStrs();
  Make_PercFourDeucesCashWonStrs();
  Make_PercFiveOfKindsCashWonStrs();
  Make_NumBadStrategyPlaysStrs();
  Make_PercPlayingAccuracyStrs();

  #if defined(ACVP_COMPILE)
  Make_PercPair6CashWonStrs();
//  Make_PercPairJacksCashWonStrs();
//  Make_PercPairKingsCashWonStrs();
  Make_PercThreeOfKinds5CashWonStrs();
  Make_PercThreeOfKinds8CashWonStrs();
  Make_PercFlushDiamondCashWonStrs();
  Make_PercFlushHeartCashWonStrs();
  Make_PercFlushSpadeCashWonStrs();
  Make_PercFlushClubCashWonStrs();
  Make_PercFlushAssaultCashWonStrs();
  Make_PercFourOfKinds5CashWonStrs();
  Make_PercFourOfKinds8CashWonStrs();
  Make_PercFourOfKinds7CashWonStrs();
  Make_PercFourOfKinds2_10CashWonStrs();
  Make_PercFourOfKindsFacesCashWonStrs();
  Make_PercFourOfKindsFacesWAceOrFaceCashWonStrs();
  Make_PercFourOfKindsFacesWAceCashWonStrs();
  Make_PercFourOfKindsAcesWFaceCashWonStrs();
  Make_PercFourOfKinds4_KCashWonStrs();
  Make_PercFourOfKinds5_KCashWonStrs();
  Make_PercFourOfKinds2_4CashWonStrs();
  Make_PercFourOfKindsAcesCashWonStrs();
  Make_PercStraightFlushesDiamondCashWonStrs();
  Make_PercStraightFlushesHeartCashWonStrs();
  Make_PercStraightFlushesSpadeCashWonStrs();
  Make_PercStraightFlushesClubCashWonStrs();
  Make_PercRoyalFlushesHeartCashWonStrs();
  Make_PercRoyalFlushesDiamondCashWonStrs();
  Make_PercRoyalFlushesSpadeCashWonStrs();
  Make_PercRoyalFlushesClubCashWonStrs();
  Make_PercFiveOfKinds6_KCashWonStrs();
  Make_PercFiveOfKinds3_5CashWonStrs();
  Make_PercFiveOfKindsAcesCashWonStrs();
  Make_PercFourOfKinds2WAceCashWonStrs();
  Make_PercFourOfKinds7WJokerCashWonStrs();
  Make_PercFiveOfKindsJokerCashWonStrs();
  #endif
}
//*******************************************************************
void TStatCalcVP::LocalFillGrid()
{
  TCasinoStatCalc::LocalFillGrid();

  if (PPlayerData == NULL) {return;}

  for (int col = 0;col <= LastSessionNum;col++)
  {
    PStatDlg->cell[vpDate][col + 1] = DateStr[col];
    PStatDlg->cell[vpHoursPlayed][col + 1] = HoursPlayedStr[col];
    PStatDlg->cell[vpGameType][col + 1] = TypeGamePlayedStr[col];
    PStatDlg->cell[vpTotalNumBets][col + 1] = TotalNumBetsStr[col];
    PStatDlg->cell[vpTotalCashBets][col + 1] = TotalCashBetsStr[col];
    PStatDlg->cell[vpAvgCashBet][col + 1] = AvgCashBetsStr[col];
    PStatDlg->cell[vpStartingBankroll][col + 1] = StartingBankrollStr[col];
    PStatDlg->cell[vpEndingBankroll][col + 1] = EndingBankrollStr[col];
    PStatDlg->cell[vpTotalCashWL][col + 1] = TotalCashWLStr[col];
    PStatDlg->cell[vpRetPerc][col + 1] = ReturnPercStr[col];
    PStatDlg->cell[vpAvgWLPerHour][col + 1] = AvgWLPerHourStr[col];
    PStatDlg->cell[vpCumCashWL][col + 1] = TotalCashWLStr[col];
    PStatDlg->cell[vpHandsPlayed][col + 1] = TotalNumBetsStr[col];
    PStatDlg->cell[vpHandsWon][col + 1] = HandsWonStr[col];
    PStatDlg->cell[vpHandsLost][col + 1] = HandsLostStr[col];
    PStatDlg->cell[vpPercHandsWon][col + 1] = PercHandsWonStr[col];
    PStatDlg->cell[vpPercHandsLost][col + 1] = PercHandsLostStr[col];
    PStatDlg->cell[vpLongestWinStreak][col + 1] = LongestWinStreakStr[col];
    PStatDlg->cell[vpLongestLosingStreak][col + 1] = LongestLosingStreakStr[col];
    PStatDlg->cell[vpLargestCashWin][col + 1] = LargestCashWinStr[col];
    PStatDlg->cell[vpPeakCashWinLevel][col + 1] = PeakCashWinLevelStr[col];
    PStatDlg->cell[vpLowCashLossLevel][col + 1] = LowCashLostLevelStr[col];
    PStatDlg->cell[vpGarbageHands][col + 1] = NumGarbageHandsStr[col];
    PStatDlg->cell[vpHighPairs][col + 1] = HighPairsStr[col];
    PStatDlg->cell[vpTwoPairs][col + 1] = TwoPairsStr[col];
    PStatDlg->cell[vpThreeofaKinds][col + 1] = ThreeOfKindsStr[col];
    PStatDlg->cell[vpStraights][col + 1] = StraightsStr[col];
    PStatDlg->cell[vpFlushes][col + 1] = FlushesStr[col];
    PStatDlg->cell[vpFullHouses][col + 1] = FullHousesStr[col];
    PStatDlg->cell[vpFourofaKinds][col + 1] = FourOfKindsStr[col];
    PStatDlg->cell[vpStraightFlushes][col + 1] = StraightFlushesStr[col];
    PStatDlg->cell[vpRoyalFlushesNatural][col + 1] = RoyalFlushesNaturalStr[col];
    PStatDlg->cell[vpRoyalFlushesWild][col + 1] = RoyalFlushesWildStr[col];
    PStatDlg->cell[vpFiveofaKinds][col + 1] = FiveOfKindsStr[col];
    PStatDlg->cell[vpFourDeuces][col + 1] = FourDeucesStr[col];
    PStatDlg->cell[vpTotalCashBets2][col + 1] = TotalCashBetsStr[col];
    PStatDlg->cell[vpTotalCashWon][col + 1] = TotalCashWonStr[col];
    PStatDlg->cell[vpHighPairsCash][col + 1] = HighPairsCashStr[col];
    PStatDlg->cell[vpTwoPairsCash][col + 1] = TwoPairsCashStr[col];
    PStatDlg->cell[vpThreeofaKindsCash][col + 1] = ThreeOfKindsCashStr[col];
    PStatDlg->cell[vpStraightsCash][col + 1] = StraightsCashStr[col];
    PStatDlg->cell[vpFlushesCash][col + 1] = FlushesCashStr[col];
    PStatDlg->cell[vpFullHousesCash][col + 1] = FullHousesCashStr[col];
    PStatDlg->cell[vpFourofaKindsCash][col + 1] = FourOfKindsCashStr[col];
    PStatDlg->cell[vpStraightFlushesCash][col + 1] = StraightFlushesCashStr[col];
    PStatDlg->cell[vpRoyalFlushesNaturalCash][col + 1] = RoyalFlushesNaturalCashStr[col];
    PStatDlg->cell[vpRoyalFlushesWildCash][col + 1] = RoyalFlushesWildCashStr[col];
    PStatDlg->cell[vpFourDeucesCash][col + 1] = FourDeucesCashStr[col];
    PStatDlg->cell[vpFiveofaKindsCash][col + 1] = FiveOfKindsCashStr[col];
    PStatDlg->cell[vpPercCashHighPairs][col + 1] = PercHighPairsCashWonStr[col];
    PStatDlg->cell[vpPercCashTwoPairs][col + 1] = PercTwoPairsCashWonStr[col];
    PStatDlg->cell[vpPercCashThreeofaKinds][col + 1] = PercThreeOfKindsCashWonStr[col];
    PStatDlg->cell[vpPercCashStraights][col + 1] = PercStraightsCashWonStr[col];
    PStatDlg->cell[vpPercCashFlushes][col + 1] = PercFlushesCashWonStr[col];
    PStatDlg->cell[vpPercCashFullHouses][col + 1] = PercFullHousesCashWonStr[col];
    PStatDlg->cell[vpPercCashFourofaKinds][col + 1] = PercFourOfKindsCashWonStr[col];
    PStatDlg->cell[vpPercCashStraightFlushes][col + 1] = PercStraightFlushesCashWonStr[col];
    PStatDlg->cell[vpPercCashRoyalFlushesNatural][col + 1] = PercRoyalFlushesNaturalCashWonStr[col];
    PStatDlg->cell[vpPercCashRoyalFlushesWild][col + 1] = PercRoyalFlushesWildCashWonStr[col];
    PStatDlg->cell[vpPercCashFourDeuces][col + 1] = PercFourDeucesCashWonStr[col];
    PStatDlg->cell[vpPercCashFiveofaKinds][col + 1] = PercFiveOfKindsCashWonStr[col];
    PStatDlg->cell[vpNumIncPlays][col + 1] = NumBadStrategyPlaysStr[col];
    PStatDlg->cell[vpPercPlayingAccuracy][col + 1] = PercPlayingAccuracyStr[col];
    #if defined(ACVP_COMPILE)
    //add new stats here.
    PStatDlg->cell[vpPair6][col + 1] = Pair6Str[col];
//    PStatDlg->cell[vpPairJacks][col + 1] = PairJacksStr[col];
//    PStatDlg->cell[vpPairKings][col + 1] = PairKingsStr[col];
    PStatDlg->cell[vpFlushDiamond][col + 1] = FlushDiamondStr[col];
    PStatDlg->cell[vpFlushHeart][col + 1] = FlushHeartStr[col];
    PStatDlg->cell[vpFlushSpade][col + 1] = FlushSpadeStr[col];
    PStatDlg->cell[vpFlushClub][col + 1] = FlushClubStr[col];
    PStatDlg->cell[vpFlushAssault][col + 1] = FlushAssaultStr[col];
    PStatDlg->cell[vpFourofaKind2_10][col + 1] = FourOfKinds2_10Str[col];
    PStatDlg->cell[vpFourFaces][col + 1] = FourOfKindsFacesStr[col];
    PStatDlg->cell[vpFourFacesWAceorFace][col + 1] = FourOfKindsFacesWAceOrFaceStr[col];
    PStatDlg->cell[vpFourFacesWAce][col + 1] = FourOfKindsFacesWAceStr[col];
    PStatDlg->cell[vpFourAcesWFace][col + 1] = FourOfKindsAcesWFaceStr[col];
    PStatDlg->cell[vpFourofaKind4_K][col + 1] = FourOfKinds4_KStr[col];
    PStatDlg->cell[vpFourofaKind5_K][col + 1] = FourOfKinds5_KStr[col];
    PStatDlg->cell[vpFourofaKind2_4][col + 1] = FourOfKinds2_4Str[col];
    PStatDlg->cell[vpFourAces][col + 1] = FourOfKindsAcesStr[col];
    PStatDlg->cell[vpStraightDiamondFlush][col + 1] = StraightFlushesDiamondStr[col];
    PStatDlg->cell[vpStraightHeartFlush][col + 1] = StraightFlushesHeartStr[col];
    PStatDlg->cell[vpStraightSpadeFlush][col + 1] = StraightFlushesSpadeStr[col];
    PStatDlg->cell[vpStraightClubFlush][col + 1] = StraightFlushesClubStr[col];
    PStatDlg->cell[vpRoyalDiamondFlush][col + 1] = RoyalFlushesDiamondStr[col];
    PStatDlg->cell[vpRoyalHeartFlush][col + 1] = RoyalFlushesHeartStr[col];
    PStatDlg->cell[vpRoyalSpadeFlush][col + 1] = RoyalFlushesSpadeStr[col];
    PStatDlg->cell[vpRoyalClubFlush][col + 1] = RoyalFlushesClubStr[col];
    PStatDlg->cell[vpFiveofaKind6_K][col + 1] = FiveOfKinds6_KStr[col];
    PStatDlg->cell[vpFiveofaKind3_5][col + 1] = FiveOfKinds3_5Str[col];
    PStatDlg->cell[vpFiveAces][col + 1] = FiveOfKindsAcesStr[col];
    PStatDlg->cell[vpFour2WAce][col + 1] = FourOfKinds2WAceStr[col];
    PStatDlg->cell[vpFour7WJoker][col + 1] = FourOfKinds7WJokerStr[col];
    PStatDlg->cell[vpFiveJoker][col + 1] = FiveOfKindsJokerStr[col];

    PStatDlg->cell[vpThreeofaKinds5][col + 1] = ThreeOfKinds5Str[col];
    PStatDlg->cell[vpThreeofaKinds8][col + 1] = ThreeOfKinds8Str[col];
    PStatDlg->cell[vpFourofaKinds5][col + 1] = FourOfKinds5Str[col];
    PStatDlg->cell[vpFourofaKinds7][col + 1] = FourOfKinds7Str[col];
    PStatDlg->cell[vpFourofaKinds8][col + 1] = FourOfKinds8Str[col];
    //
    PStatDlg->cell[vpPair6Cash][col + 1] = Pair6CashStr[col];
//    PStatDlg->cell[vpPairJacksCash][col + 1] = PairJacksCashStr[col];
//    PStatDlg->cell[vpPairKingsCash][col + 1] = PairKingsCashStr[col];
    PStatDlg->cell[vpFlushDiamondCash][col + 1] = FlushDiamondCashStr[col];
    PStatDlg->cell[vpFlushHeartCash][col + 1] = FlushHeartCashStr[col];
    PStatDlg->cell[vpFlushSpadeCash][col + 1] = FlushSpadeCashStr[col];
    PStatDlg->cell[vpFlushClubCash][col + 1] = FlushClubCashStr[col];
    PStatDlg->cell[vpFlushAssaultCash][col + 1] = FlushAssaultCashStr[col];
    PStatDlg->cell[vpFourofaKind2_10Cash][col + 1] = FourOfKinds2_10CashStr[col];
    PStatDlg->cell[vpFourFacesCash][col + 1] = FourOfKindsFacesCashStr[col];
    PStatDlg->cell[vpFourFacesWAceorFaceCash][col + 1] = FourOfKindsFacesWAceOrFaceCashStr[col];
    PStatDlg->cell[vpFourFacesWAceCash][col + 1] = FourOfKindsFacesWAceCashStr[col];
    PStatDlg->cell[vpFourAcesWFaceCash][col + 1] = FourOfKindsAcesWFaceCashStr[col];
    PStatDlg->cell[vpFourofaKind4_KCash][col + 1] = FourOfKinds4_KCashStr[col];
    PStatDlg->cell[vpFourofaKind5_KCash][col + 1] = FourOfKinds5_KCashStr[col];
    PStatDlg->cell[vpFourofaKind2_4Cash][col + 1] = FourOfKinds2_4CashStr[col];
    PStatDlg->cell[vpFourAcesCash][col + 1] = FourOfKindsAcesCashStr[col];
    PStatDlg->cell[vpStraightDiamondFlushCash][col + 1] = StraightFlushesDiamondCashStr[col];
    PStatDlg->cell[vpStraightHeartFlushCash][col + 1] = StraightFlushesHeartCashStr[col];
    PStatDlg->cell[vpStraightClubFlushCash][col + 1] = StraightFlushesClubCashStr[col];
    PStatDlg->cell[vpStraightSpadeFlushCash][col + 1] = StraightFlushesSpadeCashStr[col];
    PStatDlg->cell[vpRoyalDiamondFlushCash][col + 1] = RoyalFlushesDiamondCashStr[col];
    PStatDlg->cell[vpRoyalHeartFlushCash][col + 1] = RoyalFlushesHeartCashStr[col];
    PStatDlg->cell[vpRoyalSpadeFlushCash][col + 1] = RoyalFlushesSpadeCashStr[col];
    PStatDlg->cell[vpRoyalClubFlushCash][col + 1] = RoyalFlushesClubCashStr[col];
    PStatDlg->cell[vpFiveofaKind6_KCash][col + 1] = FiveOfKinds6_KCashStr[col];
    PStatDlg->cell[vpFiveofaKind3_5Cash][col + 1] = FiveOfKinds3_5CashStr[col];
    PStatDlg->cell[vpFiveAcesCash][col + 1] = FiveOfKindsAcesCashStr[col];
    PStatDlg->cell[vpFour2WAceCash][col + 1] = FourOfKinds2WAceCashStr[col];
    PStatDlg->cell[vpFour7WJokerCash][col + 1] = FourOfKinds7WJokerCashStr[col];
    PStatDlg->cell[vpFiveJokersCash][col + 1] = FiveOfKindsJokerCashStr[col];

    PStatDlg->cell[vpThreeofaKinds5Cash][col + 1] = ThreeOfKinds5CashStr[col];
    PStatDlg->cell[vpThreeofaKinds8Cash][col + 1] = ThreeOfKinds8CashStr[col];
    PStatDlg->cell[vpFourofaKinds5Cash][col + 1] = FourOfKinds5CashStr[col];
    PStatDlg->cell[vpFourofaKinds7Cash][col + 1] = FourOfKinds7CashStr[col];
    PStatDlg->cell[vpFourofaKinds8Cash][col + 1] = FourOfKinds8CashStr[col];
    //
    PStatDlg->cell[vpPercCashPair6][col + 1] = PercPair6CashWonStr[col];
//    PStatDlg->cell[vpPercCashPairJacks][col + 1] = PercPairJacksCashWonStr[col];
//    PStatDlg->cell[vpPercCashPairKings][col + 1] = PercPairKingsCashWonStr[col];
    PStatDlg->cell[vpPercCashDiamondFlush][col + 1] = PercFlushDiamondCashWonStr[col];
    PStatDlg->cell[vpPercCashHeartFlush][col + 1] = PercFlushHeartCashWonStr[col];
    PStatDlg->cell[vpPercCashSpadeFlush][col + 1] = PercFlushSpadeCashWonStr[col];
    PStatDlg->cell[vpPercCashClubFlush][col + 1] = PercFlushClubCashWonStr[col];
    PStatDlg->cell[vpPercCashFlushAssault][col + 1] = PercFlushAssaultCashWonStr[col];
    PStatDlg->cell[vpPercCashFourofaKind2_10][col + 1] = PercFourOfKinds2_10CashWonStr[col];
    PStatDlg->cell[vpPercCashFourFaces][col + 1] = PercFourOfKindsFacesCashWonStr[col];
    PStatDlg->cell[vpPercCashFourFacesWAceorFace][col + 1] = PercFourOfKindsFacesWAceOrFaceCashWonStr[col];
    PStatDlg->cell[vpPercCashFourFacesWAce][col + 1] = PercFourOfKindsFacesWAceCashWonStr[col];
    PStatDlg->cell[vpPercCashFourAcesWFace][col + 1] = PercFourOfKindsAcesWFaceCashWonStr[col];
    PStatDlg->cell[vpPercCashFourofaKind4_K][col + 1] = PercFourOfKinds4_KCashWonStr[col];
    PStatDlg->cell[vpPercCashFourofaKind5_K][col + 1] = PercFourOfKinds5_KCashWonStr[col];
    PStatDlg->cell[vpPercCashFourofaKind2_4][col + 1] = PercFourOfKinds2_4CashWonStr[col];
    PStatDlg->cell[vpPercCashFourAces][col + 1] = PercFourOfKindsAcesCashWonStr[col];
    PStatDlg->cell[vpPercCashStraightDiamondFlush][col + 1] = PercStraightFlushesDiamondCashWonStr[col];
    PStatDlg->cell[vpPercCashStraightHeartFlush][col + 1] = PercStraightFlushesHeartCashWonStr[col];
    PStatDlg->cell[vpPercCashStraightClubFlush][col + 1] = PercStraightFlushesClubCashWonStr[col];
    PStatDlg->cell[vpPercCashStraightSpadeFlush][col + 1] = PercStraightFlushesSpadeCashWonStr[col];
    PStatDlg->cell[vpPercCashRoyalDiamondFlush][col + 1] = PercRoyalFlushesDiamondCashWonStr[col];
    PStatDlg->cell[vpPercCashRoyalHeartFlush][col + 1] = PercRoyalFlushesHeartCashWonStr[col];
    PStatDlg->cell[vpPercCashRoyalSpadeFlush][col + 1] = PercRoyalFlushesSpadeCashWonStr[col];
    PStatDlg->cell[vpPercCashRoyalClubFlush][col + 1] = PercRoyalFlushesClubCashWonStr[col];
    PStatDlg->cell[vpPercCashFiveofaKind6_K][col + 1] = PercFiveOfKinds6_KCashWonStr[col];
    PStatDlg->cell[vpPercCashFiveofaKind3_5][col + 1] = PercFiveOfKinds3_5CashWonStr[col];
    PStatDlg->cell[vpPercCashFiveAces][col + 1] = PercFiveOfKindsAcesCashWonStr[col];
    PStatDlg->cell[vpPercCashFour2WAce][col + 1] = PercFourOfKinds2WAceCashWonStr[col];
    PStatDlg->cell[vpPercCashFour7WJoker][col + 1] = PercFourOfKinds7WJokerCashWonStr[col];
    PStatDlg->cell[vpPercCashFiveJoker][col + 1] = PercFiveOfKindsJokerCashWonStr[col];

    PStatDlg->cell[vpPercCashThreeofaKinds5][col + 1] = PercThreeOfKinds5CashWonStr[col];
    PStatDlg->cell[vpPercCashThreeofaKinds8][col + 1] = PercThreeOfKinds8CashWonStr[col];
    PStatDlg->cell[vpPercCashFourofaKinds5][col + 1] = PercFourOfKinds5CashWonStr[col];
    PStatDlg->cell[vpPercCashFourofaKinds7][col + 1] = PercFourOfKinds7CashWonStr[col];
    PStatDlg->cell[vpPercCashFourofaKinds8][col + 1] = PercFourOfKinds8CashWonStr[col];
    #endif
  }
  PStatDlg->totalcol[vpDate] = Total_DateStr;
  PStatDlg->totalcol[vpHoursPlayed] = Total_HoursPlayedStr;
  PStatDlg->totalcol[vpGameType] = Total_TypeGamePlayedStr;
  PStatDlg->totalcol[vpTotalNumBets] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[vpTotalCashBets] = Total_TotalCashBetsStr;
  PStatDlg->totalcol[vpAvgCashBet] = Total_AvgCashBetsStr;
  PStatDlg->totalcol[vpStartingBankroll] = Total_StartingBankrollStr;
  PStatDlg->totalcol[vpEndingBankroll] = Total_EndingBankrollStr;
  PStatDlg->totalcol[vpTotalCashWL] = Total_TotalCashWLStr;
  PStatDlg->totalcol[vpRetPerc] = Total_ReturnPercStr;
  PStatDlg->totalcol[vpAvgWLPerHour] = Total_AvgWLPerHourStr;
  PStatDlg->totalcol[vpCumCashWL] = Total_TotalCashWLStr;
  PStatDlg->totalcol[vpHandsPlayed] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[vpHandsWon] = Total_HandsWonStr;
  PStatDlg->totalcol[vpHandsLost] = Total_HandsLostStr;
  PStatDlg->totalcol[vpPercHandsWon] = Total_PercHandsWonStr;
  PStatDlg->totalcol[vpPercHandsLost] = Total_PercHandsLostStr;
  PStatDlg->totalcol[vpLongestWinStreak] = Total_LongestWinStreakStr;
  PStatDlg->totalcol[vpLongestLosingStreak] = Total_LongestLosingStreakStr;
  PStatDlg->totalcol[vpLargestCashWin] = Total_LargestCashWinStr;
  PStatDlg->totalcol[vpPeakCashWinLevel] = Total_PeakCashWinLevelStr;
  PStatDlg->totalcol[vpLowCashLossLevel] = Total_LowCashLostLevelStr;
  PStatDlg->totalcol[vpGarbageHands] = Total_NumGarbageHandsStr;
  PStatDlg->totalcol[vpHighPairs] = Total_HighPairsStr;
  PStatDlg->totalcol[vpTwoPairs] = Total_TwoPairsStr;
  PStatDlg->totalcol[vpThreeofaKinds] = Total_ThreeOfKindsStr;
  PStatDlg->totalcol[vpStraights] = Total_StraightsStr;
  PStatDlg->totalcol[vpFlushes] = Total_FlushesStr;
  PStatDlg->totalcol[vpFullHouses] = Total_FullHousesStr;
  PStatDlg->totalcol[vpFourofaKinds] = Total_FourOfKindsStr;
  PStatDlg->totalcol[vpStraightFlushes] = Total_StraightFlushesStr;
  PStatDlg->totalcol[vpRoyalFlushesNatural] = Total_RoyalFlushesNaturalStr;
  PStatDlg->totalcol[vpRoyalFlushesWild] = Total_RoyalFlushesWildStr;
  PStatDlg->totalcol[vpFiveofaKinds] = Total_FiveOfKindsStr;
  PStatDlg->totalcol[vpFourDeuces] = Total_FourDeucesStr;
  PStatDlg->totalcol[vpTotalCashBets2] = Total_TotalCashBetsStr;
  PStatDlg->totalcol[vpTotalCashWon] = Total_TotalCashWonStr;
  PStatDlg->totalcol[vpHighPairsCash] = Total_HighPairsCashStr;
  PStatDlg->totalcol[vpTwoPairsCash] = Total_TwoPairsCashStr;
  PStatDlg->totalcol[vpThreeofaKindsCash] = Total_ThreeOfKindsCashStr;
  PStatDlg->totalcol[vpStraightsCash] = Total_StraightsCashStr;
  PStatDlg->totalcol[vpFlushesCash] = Total_FlushesCashStr;
  PStatDlg->totalcol[vpFullHousesCash] = Total_FullHousesCashStr;
  PStatDlg->totalcol[vpFourofaKindsCash] = Total_FourOfKindsCashStr;
  PStatDlg->totalcol[vpStraightFlushesCash] = Total_StraightFlushesCashStr;
  PStatDlg->totalcol[vpRoyalFlushesNaturalCash] = Total_RoyalFlushesNaturalCashStr;
  PStatDlg->totalcol[vpRoyalFlushesWildCash] = Total_RoyalFlushesWildCashStr;
  PStatDlg->totalcol[vpFourDeucesCash] = Total_FourDeucesCashStr;
  PStatDlg->totalcol[vpFiveofaKindsCash] = Total_FiveOfKindsCashStr;
  PStatDlg->totalcol[vpPercCashHighPairs] = Total_PercHighPairsCashWonStr;
  PStatDlg->totalcol[vpPercCashTwoPairs] = Total_PercTwoPairsCashWonStr;
  PStatDlg->totalcol[vpPercCashThreeofaKinds] = Total_PercThreeOfKindsCashWonStr;
  PStatDlg->totalcol[vpPercCashStraights] = Total_PercStraightsCashWonStr;
  PStatDlg->totalcol[vpPercCashFlushes] = Total_PercFlushesCashWonStr;
  PStatDlg->totalcol[vpPercCashFullHouses] = Total_PercFullHousesCashWonStr;
  PStatDlg->totalcol[vpPercCashFourofaKinds] = Total_PercFourOfKindsCashWonStr;
  PStatDlg->totalcol[vpPercCashStraightFlushes] = Total_PercStraightFlushesCashWonStr;
  PStatDlg->totalcol[vpPercCashRoyalFlushesNatural] = Total_PercRoyalFlushesNaturalCashWonStr;
  PStatDlg->totalcol[vpPercCashRoyalFlushesWild] = Total_PercRoyalFlushesWildCashWonStr;
  PStatDlg->totalcol[vpPercCashFourDeuces] = Total_PercFourDeucesCashWonStr;
  PStatDlg->totalcol[vpPercCashFiveofaKinds] = Total_PercFiveOfKindsCashWonStr;
  PStatDlg->totalcol[vpNumIncPlays] = Total_NumBadStrategyPlaysStr;
  PStatDlg->totalcol[vpPercPlayingAccuracy] = Total_PercPlayingAccuracyStr;
  #if defined(ACVP_COMPILE)
  // add new stats here.
  PStatDlg->totalcol[vpPair6] = Total_Pair6Str;
//  PStatDlg->totalcol[vpPairJacks] = Total_PairJacksStr;
//  PStatDlg->totalcol[vpPairKings] = Total_PairKingsStr;
  PStatDlg->totalcol[vpFlushDiamond] = Total_FlushDiamondStr;
  PStatDlg->totalcol[vpFlushHeart] = Total_FlushHeartStr;
  PStatDlg->totalcol[vpFlushSpade] = Total_FlushSpadeStr;
  PStatDlg->totalcol[vpFlushClub] = Total_FlushClubStr;
  PStatDlg->totalcol[vpFlushAssault] = Total_FlushAssaultStr;
  PStatDlg->totalcol[vpFourofaKind2_10] = Total_FourOfKinds2_10Str;
  PStatDlg->totalcol[vpFourFaces] = Total_FourOfKindsFacesStr;
  PStatDlg->totalcol[vpFourFacesWAceorFace] = Total_FourOfKindsFacesWAceOrFaceStr;
  PStatDlg->totalcol[vpFourFacesWAce] = Total_FourOfKindsFacesWAceStr;
  PStatDlg->totalcol[vpFourAcesWFace] = Total_FourOfKindsAcesWFaceStr;
  PStatDlg->totalcol[vpFourofaKind4_K] = Total_FourOfKinds4_KStr;
  PStatDlg->totalcol[vpFourofaKind5_K] = Total_FourOfKinds5_KStr;
  PStatDlg->totalcol[vpFourofaKind2_4] = Total_FourOfKinds2_4Str;
  PStatDlg->totalcol[vpFourAces] = Total_FourOfKindsAcesStr;
  PStatDlg->totalcol[vpStraightDiamondFlush] = Total_StraightFlushesDiamondStr;
  PStatDlg->totalcol[vpStraightHeartFlush] = Total_StraightFlushesHeartStr;
  PStatDlg->totalcol[vpStraightSpadeFlush] = Total_StraightFlushesSpadeStr;
  PStatDlg->totalcol[vpStraightClubFlush] = Total_StraightFlushesClubStr;
  PStatDlg->totalcol[vpRoyalDiamondFlush] = Total_RoyalFlushesDiamondStr;
  PStatDlg->totalcol[vpRoyalHeartFlush] = Total_RoyalFlushesHeartStr;
  PStatDlg->totalcol[vpRoyalSpadeFlush] = Total_RoyalFlushesSpadeStr;
  PStatDlg->totalcol[vpRoyalClubFlush] = Total_RoyalFlushesClubStr;
  PStatDlg->totalcol[vpFiveofaKind6_K] = Total_FiveOfKinds6_KStr;
  PStatDlg->totalcol[vpFiveofaKind3_5] = Total_FiveOfKinds3_5Str;
  PStatDlg->totalcol[vpFiveAces] = Total_FiveOfKindsAcesStr;
  PStatDlg->totalcol[vpFour2WAce] = Total_FourOfKinds2WAceStr;
  PStatDlg->totalcol[vpFour7WJoker] = Total_FourOfKinds7WJokerStr;
  PStatDlg->totalcol[vpFiveJoker] = Total_FiveOfKindsJokerStr;

  PStatDlg->totalcol[vpThreeofaKinds5] = Total_ThreeOfKinds5Str;
  PStatDlg->totalcol[vpThreeofaKinds8] = Total_ThreeOfKinds8Str;
  PStatDlg->totalcol[vpFourofaKinds5] = Total_FourOfKinds5Str;
  PStatDlg->totalcol[vpFourofaKinds7] = Total_FourOfKinds7Str;
  PStatDlg->totalcol[vpFourofaKinds8] = Total_FourOfKinds8Str;
  //
  PStatDlg->totalcol[vpPair6Cash] = Total_Pair6CashStr;
//  PStatDlg->totalcol[vpPairJacksCash] = Total_PairJacksCashStr;
//  PStatDlg->totalcol[vpPairKingsCash] = Total_PairKingsCashStr;
  PStatDlg->totalcol[vpFlushDiamondCash] = Total_FlushDiamondCashStr;
  PStatDlg->totalcol[vpFlushHeartCash] = Total_FlushHeartCashStr;
  PStatDlg->totalcol[vpFlushSpadeCash] = Total_FlushSpadeCashStr;
  PStatDlg->totalcol[vpFlushClubCash] = Total_FlushClubCashStr;
  PStatDlg->totalcol[vpFlushAssaultCash] = Total_FlushAssaultCashStr;
  PStatDlg->totalcol[vpFourofaKind2_10Cash] = Total_FourOfKinds2_10CashStr;
  PStatDlg->totalcol[vpFourFacesCash] = Total_FourOfKindsFacesCashStr;
  PStatDlg->totalcol[vpFourFacesWAceorFaceCash] = Total_FourOfKindsFacesWAceOrFaceCashStr;
  PStatDlg->totalcol[vpFourFacesWAceCash] = Total_FourOfKindsFacesWAceCashStr;
  PStatDlg->totalcol[vpFourAcesWFaceCash] = Total_FourOfKindsAcesWFaceCashStr;
  PStatDlg->totalcol[vpFourofaKind4_KCash] = Total_FourOfKinds4_KCashStr;
  PStatDlg->totalcol[vpFourofaKind5_KCash] = Total_FourOfKinds5_KCashStr;
  PStatDlg->totalcol[vpFourofaKind2_4Cash] = Total_FourOfKinds2_4CashStr;
  PStatDlg->totalcol[vpFourAcesCash] = Total_FourOfKindsAcesCashStr;
  PStatDlg->totalcol[vpStraightDiamondFlushCash] = Total_StraightFlushesDiamondCashStr;
  PStatDlg->totalcol[vpStraightHeartFlushCash] = Total_StraightFlushesHeartCashStr;
  PStatDlg->totalcol[vpStraightClubFlushCash] = Total_StraightFlushesClubCashStr;
  PStatDlg->totalcol[vpStraightSpadeFlushCash] = Total_StraightFlushesSpadeCashStr;
  PStatDlg->totalcol[vpRoyalDiamondFlushCash] = Total_RoyalFlushesDiamondCashStr;
  PStatDlg->totalcol[vpRoyalHeartFlushCash] = Total_RoyalFlushesHeartCashStr;
  PStatDlg->totalcol[vpRoyalSpadeFlushCash] = Total_RoyalFlushesSpadeCashStr;
  PStatDlg->totalcol[vpRoyalClubFlushCash] = Total_RoyalFlushesClubCashStr;
  PStatDlg->totalcol[vpFiveofaKind6_KCash] = Total_FiveOfKinds6_KCashStr;
  PStatDlg->totalcol[vpFiveofaKind3_5Cash] = Total_FiveOfKinds3_5CashStr;
  PStatDlg->totalcol[vpFiveAcesCash] = Total_FiveOfKindsAcesCashStr;
  PStatDlg->totalcol[vpFour2WAceCash] = Total_FourOfKinds2WAceCashStr;
  PStatDlg->totalcol[vpFour7WJokerCash] = Total_FourOfKinds7WJokerCashStr;
  PStatDlg->totalcol[vpFiveJokersCash] = Total_FiveOfKindsJokerCashStr;

  PStatDlg->totalcol[vpThreeofaKinds5Cash] = Total_ThreeOfKinds5CashStr;
  PStatDlg->totalcol[vpThreeofaKinds8Cash] = Total_ThreeOfKinds8CashStr;
  PStatDlg->totalcol[vpFourofaKinds5Cash] = Total_FourOfKinds5CashStr;
  PStatDlg->totalcol[vpFourofaKinds7Cash] = Total_FourOfKinds7CashStr;
  PStatDlg->totalcol[vpFourofaKinds8Cash] = Total_FourOfKinds8CashStr;
  //
  PStatDlg->totalcol[vpPercCashPair6] = Total_PercPair6CashWonStr;
//  PStatDlg->totalcol[vpPercCashPairJacks] = Total_PercPairJacksCashWonStr;
//  PStatDlg->totalcol[vpPercCashPairKings] = Total_PercPairKingsCashWonStr;
  PStatDlg->totalcol[vpPercCashDiamondFlush] = Total_PercFlushDiamondCashWonStr;
  PStatDlg->totalcol[vpPercCashHeartFlush] = Total_PercFlushHeartCashWonStr;
  PStatDlg->totalcol[vpPercCashSpadeFlush] = Total_PercFlushSpadeCashWonStr;
  PStatDlg->totalcol[vpPercCashClubFlush] = Total_PercFlushClubCashWonStr;
  PStatDlg->totalcol[vpPercCashFlushAssault] = Total_PercFlushAssaultCashWonStr;
  PStatDlg->totalcol[vpPercCashFourofaKind2_10] = Total_PercFourOfKinds2_10CashWonStr;
  PStatDlg->totalcol[vpPercCashFourFaces] = Total_PercFourOfKindsFacesCashWonStr;
  PStatDlg->totalcol[vpPercCashFourFacesWAceorFace] = Total_PercFourOfKindsFacesWAceOrFaceCashWonStr;
  PStatDlg->totalcol[vpPercCashFourFacesWAce] = Total_PercFourOfKindsFacesWAceCashWonStr;
  PStatDlg->totalcol[vpPercCashFourAcesWFace] = Total_PercFourOfKindsAcesWFaceCashWonStr;
  PStatDlg->totalcol[vpPercCashFourofaKind4_K] = Total_PercFourOfKinds4_KCashWonStr;
  PStatDlg->totalcol[vpPercCashFourofaKind5_K] = Total_PercFourOfKinds5_KCashWonStr;
  PStatDlg->totalcol[vpPercCashFourofaKind2_4] = Total_PercFourOfKinds2_4CashWonStr;
  PStatDlg->totalcol[vpPercCashFourAces] = Total_PercFourOfKindsAcesCashWonStr;
  PStatDlg->totalcol[vpPercCashStraightDiamondFlush] = Total_PercStraightFlushesDiamondCashWonStr;
  PStatDlg->totalcol[vpPercCashStraightHeartFlush] = Total_PercStraightFlushesHeartCashWonStr;
  PStatDlg->totalcol[vpPercCashStraightClubFlush] = Total_PercStraightFlushesClubCashWonStr;
  PStatDlg->totalcol[vpPercCashStraightSpadeFlush] = Total_PercStraightFlushesSpadeCashWonStr;
  PStatDlg->totalcol[vpPercCashRoyalDiamondFlush] = Total_PercRoyalFlushesDiamondCashWonStr;
  PStatDlg->totalcol[vpPercCashRoyalHeartFlush] = Total_PercRoyalFlushesHeartCashWonStr;
  PStatDlg->totalcol[vpPercCashRoyalSpadeFlush] = Total_PercRoyalFlushesSpadeCashWonStr;
  PStatDlg->totalcol[vpPercCashRoyalClubFlush] = Total_PercRoyalFlushesClubCashWonStr;
  PStatDlg->totalcol[vpPercCashFiveofaKind6_K] = Total_PercFiveOfKinds6_KCashWonStr;
  PStatDlg->totalcol[vpPercCashFiveofaKind3_5] = Total_PercFiveOfKinds3_5CashWonStr;
  PStatDlg->totalcol[vpPercCashFiveAces] = Total_PercFiveOfKindsAcesCashWonStr;
  PStatDlg->totalcol[vpPercCashFour2WAce] = Total_PercFourOfKinds2WAceCashWonStr;
  PStatDlg->totalcol[vpPercCashFour7WJoker] = Total_PercFourOfKinds7WJokerCashWonStr;
  PStatDlg->totalcol[vpPercCashFiveJoker] = Total_PercFiveOfKindsJokerCashWonStr;

  PStatDlg->totalcol[vpPercCashThreeofaKinds5] = Total_PercThreeOfKinds5CashWonStr;
  PStatDlg->totalcol[vpPercCashThreeofaKinds8] = Total_PercThreeOfKinds8CashWonStr;
  PStatDlg->totalcol[vpPercCashFourofaKinds5] = Total_PercFourOfKinds5CashWonStr;
  PStatDlg->totalcol[vpPercCashFourofaKinds7] = Total_PercFourOfKinds7CashWonStr;
  PStatDlg->totalcol[vpPercCashFourofaKinds8] = Total_PercFourOfKinds8CashWonStr;
  #endif
}
//*******************************************************************
//*******************************************************************
TVPGraph::TVPGraph(TCasinoStatDlg* pstatdlg,TStatCalcVP *pstatcalc)
:TCasinoGraphDlg(pstatdlg,pstatcalc)
{
  strName = "Video Poker";
  BaseStringID = VideoPokerLabel;

  StatTT[0] = TStatCalcVP::vpTotalNumBets;
  StatTT[1] = TStatCalcVP::vpTotalCashBets;
  StatTT[2] = TStatCalcVP::vpAvgCashBet;
  StatTT[3] = TStatCalcVP::vpTotalCashWL;
  StatTT[4] = TStatCalcVP::vpCumCashWL;
  StatTT[5] = TStatCalcVP::vpHandsWon;
  StatTT[6] = TStatCalcVP::vpHandsLost;
  StatTT[7] = TStatCalcVP::vpPercHandsWon;
  StatTT[8] = TStatCalcVP::vpLongestWinStreak;
  StatTT[9] = TStatCalcVP::vpLongestLosingStreak;
  StatTT[10] = TStatCalcVP::vpLargestCashWin;
  StatTT[11] = TStatCalcVP::vpPeakCashWinLevel;
  StatTT[12] = TStatCalcVP::vpLowCashLossLevel;
  StatTT[13] = TStatCalcVP::vpNumIncPlays;
  StatTT[14] = TStatCalcVP::vpPercPlayingAccuracy;

  LastStatIndex = 14;

  for (int i=0;i<=LastStatIndex;i++)
  {
    StatLabelStr[i] = pstatcalc->GetLabelStr(int16(StatTT[i]));
  }
  UseStatLabelStr = true;
}
//*******************************************************************
TVPGraph::~TVPGraph()
{
}
//*******************************************************************

