//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cdebtdlg.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.3     7/2/97     M.Perry    original baseline
//  2.0    08/31/97    M.Perry    32-bit conversion.
//********************************************************************
#include "cdebtdlg.h"
#include <owl\static.h>
#include <owl\button.h>
#include "cplaydat.h"
#include "cdebtdlg.rh"
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TGamblerDebtDlg,TOgleDialog)
  EV_COMMAND(IDC_GAMBLER_DEBT_BORROW_PB,CmBorrow),
  EV_COMMAND(IDC_GAMBLER_DEBT_REPAY_PB,CmRepay),
  EV_COMMAND(IDC_GAMBLER_DEBT_DONE_PB,CmDone),
END_RESPONSE_TABLE;
//*******************************************************************
TGamblerDebtDlg::TGamblerDebtDlg(TWindow *pwin,TCasinoPlayerData *pcasplayerdata)
:TOgleDialog(pwin,IDD_GAMBLER_DEBT),
 PCasPlayerData(pcasplayerdata)
{
  PName_STB = new TStatic(this,IDC_GAMBLER_DEBT_NAME_STB);
  PBankroll_STB = new TStatic(this,IDC_GAMBLER_DEBT_BANKROLL_STB);
  PDebt_STB = new TStatic(this,IDC_GAMBLER_DEBT_DEBT_STB);
  PLOC_STB = new TStatic(this,IDC_GAMBLER_DEBT_LOC_STB);
  PAvailNow_STB = new TStatic(this,IDC_GAMBLER_DEBT_AVAILNOW_STB);
  PBorrow_PB = new TButton(this,IDC_GAMBLER_DEBT_BORROW_PB);
  PRepay_PB = new TButton(this,IDC_GAMBLER_DEBT_REPAY_PB);
  PDone_PB = new TButton(this,IDC_GAMBLER_DEBT_DONE_PB);

  if (PCasPlayerData != NULL) // prime the pump, just in case first time.
  {
    PCasPlayerData->SetCashLimit();
    PCasPlayerData->CalcAvailNow();
    PCasPlayerData->CalcCreditLine();
  }
}
//*******************************************************************
TGamblerDebtDlg::~TGamblerDebtDlg()
{
  delete PName_STB;
  delete PBankroll_STB;
  delete PDebt_STB;
  delete PLOC_STB;
  delete PAvailNow_STB;
  delete PBorrow_PB;
  delete PRepay_PB;
  delete PDone_PB;
}
//*******************************************************************
void TGamblerDebtDlg::CmBorrow()
{
  if (PCasPlayerData->AvailNow <= 0.0) {return;}
  PCasPlayerData->Debt += 1000.0;
  PCasPlayerData->AddCash(TMoney(1000.0));
  PCasPlayerData->SetMaxMinDebt();
  PCasPlayerData->CalcAvailNow();
  HandleStatus();
}
//*******************************************************************
void TGamblerDebtDlg::CmRepay()
{
  if (PCasPlayerData->Debt < 1000.0) {return;}
  if (PCasPlayerData->Cash <= 1000.0) {return;}
  PCasPlayerData->SubCash(TMoney(1000.0));
  PCasPlayerData->Debt -= 1000.0;
  PCasPlayerData->SetMinDebt();
  PCasPlayerData->CalcAvailNow();
  PCasPlayerData->CalcCreditLine();
  HandleStatus();
}
//*******************************************************************
void TGamblerDebtDlg::CmDone()
{
  TOgleDialog::CmOk();
}
//*******************************************************************
void TGamblerDebtDlg::HandleNoDataStatus()
{
  HWND hwnd = GetFocus();
  if (hwnd == *PBorrow_PB)
  {
    PDone_PB->SetFocus();
  }
  PBorrow_PB->EnableWindow(false);

  if (hwnd == *PRepay_PB)
  {
    PDone_PB->SetFocus();
  }
  PRepay_PB->EnableWindow(false);
}
//*******************************************************************
void TGamblerDebtDlg::HandleStatus()
{
  if (PCasPlayerData == NULL)
  {
    HandleNoDataStatus();
    return;
  }

  HWND hwnd = GetFocus();
  bool borrowstatus = false;
  if (PCasPlayerData->AvailNow >= 1000.0)
  {
    borrowstatus = true;
  }
  if (!borrowstatus && hwnd == *PBorrow_PB)
  {
    PDone_PB->SetFocus();
  }
  PBorrow_PB->EnableWindow(borrowstatus);

  bool repaystatus = true;
  if (PCasPlayerData->Debt < 1000.0 || PCasPlayerData->Cash <= 1000.0)
  {
    repaystatus = false;
  }
  if (!repaystatus && hwnd == *PRepay_PB)
  {
    PDone_PB->SetFocus();
  }
  PRepay_PB->EnableWindow(repaystatus);

  PName_STB->SetText(PCasPlayerData->GetName().c_str());
  PBankroll_STB->SetText(PCasPlayerData->Cash.GetStr().c_str());
  PDebt_STB->SetText(PCasPlayerData->Debt.GetStr().c_str());
  PLOC_STB->SetText(PCasPlayerData->CreditLine.GetStr().c_str());
  PAvailNow_STB->SetText(PCasPlayerData->AvailNow.GetStr().c_str());
}
//*******************************************************************
void TGamblerDebtDlg::SetupWindow()
{
  TOgleDialog::SetupWindow();
  HandleStatus();
}
//*******************************************************************

