//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  csalgdlg.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.3     6/23/97    M.Perry    original baseline
//  2.0    08/31/97    M.Perry    32-bit conversion.
//********************************************************************
#include "csalgdlg.h"
#include <owl\button.h>
#include <owl\radiobut.h>
#include <owl\listbox.h>
#include <owl\edit.h>
#include <owl\validate.h>
#include "cplaymgr.h"
#include "cplaydat.h"
#include "oapp.h"
#include <assert.h>
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TSalariedGamblerDlg,TOgleDialog)
  EV_COMMAND(IDC_DEACTIVATE_PB,CmDeActivate),
  EV_COMMAND(IDC_ACTIVATE_PB,CmActivate),
  EV_COMMAND(IDC_CREATE_PB,CmCreate),
  EV_COMMAND(IDC_DELETE_PB,CmDelete),
  EV_COMMAND(IDC_STATISTICS_PB,CmStatistics),
  EV_COMMAND(IDC_HELP_PB,CmHelp),
  EV_COMMAND(IDC_DONE_PB,CmDone),
  EV_LBN_SELCHANGE(IDC_ACTIVEGAMBLERS_LB,CmActiveLBSelChange),
  EV_LBN_SELCHANGE(IDC_SAVEDGAMBLERS_LB,CmSavedLBSelChange),
  EV_LBN_DBLCLK(IDC_ACTIVEGAMBLERS_LB,CmActiveLBDblClk),
  EV_LBN_DBLCLK(IDC_SAVEDGAMBLERS_LB,CmSavedLBDblClk),
END_RESPONSE_TABLE;
//*******************************************************************
string TSalariedGamblerDlg::MakeStringOfSpaces(int16 numspaces)
{
  string strtemp("");
  for (int16 i=0;i<numspaces;i++)
  {
    strtemp += " ";
  }
  return strtemp;
}
//*******************************************************************
string TSalariedGamblerDlg::MakePlayerInfoString(int16 playernum)
{
  TCasinoPlayerData *pcpd = PCasinoPlayerMgr->GetCasinoPlayerDataPtr(playernum);
  if (pcpd == NULL) {return string("error getting playerdata");}

  string strname = pcpd->GetName();
  string strsex = "M";
  if (!pcpd->Male) {strsex = "F";}
  string strbankroll = pcpd->Cash.GetStr();
  string strsalary = pcpd->Salary.GetStr();
  string strdebt = pcpd->Debt.GetStr();

  // start born string.
  TTime time(pcpd->CreationSec);
  TDate date(time);
  TDate::HowToPrint htp = date.SetPrintOption(TDate::Numbers);
  string strborn = date.AsString();
  date.SetPrintOption(htp);
  // end born string.


  string s = strname;
  int16 lensofar = int16(strlen(s.c_str()));
  s += MakeStringOfSpaces(int16(SexStart - lensofar));
  s += strsex;
  lensofar = int16(strlen(s.c_str()));
  s += MakeStringOfSpaces(int16(BankrollStart - lensofar));
  s += strbankroll;
  lensofar = int16(strlen(s.c_str()));
  s += MakeStringOfSpaces(int16(SalaryStart - lensofar));
  s += strsalary;
  lensofar = int16(strlen(s.c_str()));
  s += MakeStringOfSpaces(int16(DebtStart - lensofar));
  s += strdebt;
  lensofar = int16(strlen(s.c_str()));
  s += MakeStringOfSpaces(int16(BornStart - lensofar));
  s += strborn;
  return s;
}
//*******************************************************************
TSalariedGamblerDlg::TSalariedGamblerDlg(TWindow *parent,TCasinoPlayerMgr *pplayermgr,TResId dlgid,dword helpid)
:TOgleDialog(parent,dlgid,true),
 PCasinoPlayerMgr(pplayermgr),
 HelpID(helpid)
{
  PActivate_Btn = new TButton(this,IDC_ACTIVATE_PB);
  PDeActivate_Btn = new TButton(this,IDC_DEACTIVATE_PB);
  PCreate_Btn = new TButton(this,IDC_CREATE_PB);
  PDelete_Btn = new TButton(this,IDC_DELETE_PB);
  PStatistics_Btn = new TButton(this,IDC_STATISTICS_PB);
  PDone_Btn = new TButton(this,IDC_DONE_PB);
  PHelp_Btn = new TButton(this,IDC_HELP_PB);
  PActivePlayers_LB = new TListBox(this,IDC_ACTIVEGAMBLERS_LB);
  PSavedPlayers_LB = new TListBox(this,IDC_SAVEDGAMBLERS_LB);

  SexStart = int16(PCasinoPlayerMgr->GetMaxNameLength() + 1);
  BankrollStart = int16(SexStart + 4);
  SalaryStart = int16(BankrollStart + 13);
  DebtStart = int16(SalaryStart + 11);
  BornStart = int16(DebtStart + 10);

  NumPlayersAllowed = PCasinoPlayerMgr->GetNumPlayersAllowed();
  NumActiveAllowed = PCasinoPlayerMgr->GetNumActiveAllowed();
  PSavedNumsTT = new int16[NumPlayersAllowed];
  PActiveNumsTT = new int16[NumActiveAllowed];
}
//*******************************************************************
TSalariedGamblerDlg::~TSalariedGamblerDlg()
{
  delete PActivate_Btn;
  delete PDeActivate_Btn;
  delete PCreate_Btn;
  delete PDelete_Btn;
  delete PStatistics_Btn;
  delete PDone_Btn;
  delete PHelp_Btn;
  delete PActivePlayers_LB;
  delete PSavedPlayers_LB;

  delete [] PSavedNumsTT;
  delete [] PActiveNumsTT;
}
//*******************************************************************
void TSalariedGamblerDlg::LoadSavedTT()
{
  UnLoadSavedTT();
  LastSavedIndex = -1;
  for (int16 i=0;i<NumPlayersAllowed;i++)
  {
    if (PCasinoPlayerMgr->IsPlayerValid(i))
    {
      LastSavedIndex++;
      PSavedNumsTT[LastSavedIndex] = i;
    }
  }
}
//*******************************************************************
void TSalariedGamblerDlg::UnLoadSavedTT()
{
  for (int16 i=0;i<NumPlayersAllowed;i++)
  {
    PSavedNumsTT[i] = -1;
  }
  LastSavedIndex = -1;
}
//*******************************************************************
void TSalariedGamblerDlg::LoadActiveTT()
{
  UnLoadActiveTT();
  LastActiveIndex = -1;
  for (int16 i=0;i<NumPlayersAllowed;i++)
  {
    if (PCasinoPlayerMgr->IsPlayerValid(i) && PCasinoPlayerMgr->IsPlayerActive(i))
    {
      LastActiveIndex++;
      PActiveNumsTT[LastActiveIndex] = i;
    }
  }
}
//*******************************************************************
void TSalariedGamblerDlg::UnLoadActiveTT()
{
  for (int16 i=0;i<NumActiveAllowed;i++)
  {
    PActiveNumsTT[i] = -1;
  }
  LastActiveIndex = -1;
}
//*******************************************************************
void TSalariedGamblerDlg::LoadSaved_LB()
{
  UnLoadSaved_LB();
  LoadSavedTT();
  for (int16 i=0;i<=LastSavedIndex;i++)
  {
    string s = MakePlayerInfoString(PSavedNumsTT[i]);
    PSavedPlayers_LB->AddString(s.c_str());
  }
}
//*******************************************************************
void TSalariedGamblerDlg::UnLoadSaved_LB()
{
  PSavedPlayers_LB->ClearList();
}
//*******************************************************************
void TSalariedGamblerDlg::LoadActive_LB()
{
  UnLoadActive_LB();
  LoadActiveTT();
  for (int16 i=0;i<=LastActiveIndex;i++)
  {
    string s = MakePlayerInfoString(PActiveNumsTT[i]);
    PActivePlayers_LB->AddString(s.c_str());
  }
}
//*******************************************************************
void TSalariedGamblerDlg::UnLoadActive_LB()
{
  PActivePlayers_LB->ClearList();
}
//*******************************************************************
bool TSalariedGamblerDlg::IsActive(int16 playernum)
{
  for (int16 i=0;i<=LastActiveIndex;i++)
  {
    if (PActiveNumsTT[i] == playernum) {return true;}
  }
  return false;
}
//*******************************************************************
void TSalariedGamblerDlg::HandleButtonStatus()
{
  HWND hwnd = GetFocus();

  int16 selactive = int16(PActivePlayers_LB->GetSelIndex());
  int16 selsaved = int16(PSavedPlayers_LB->GetSelIndex());
  int16 selsavedpn = TranslateSaved2PN(selsaved);

  bool deactivatestatus = false;
  if (CheckActiveIndex(selactive))
  {
    deactivatestatus = true;
  }
  if (!deactivatestatus && *PDeActivate_Btn == hwnd) {PDone_Btn->SetFocus();}
  PDeActivate_Btn->EnableWindow(deactivatestatus);

  bool activatestatus = false;
  if (selsavedpn != -1 && !IsActive(selsavedpn) && LastActiveIndex < (NumActiveAllowed - 1))
  {
    activatestatus = true;
  }
  if (!activatestatus && *PActivate_Btn == hwnd) {PDone_Btn->SetFocus();}
  PActivate_Btn->EnableWindow(activatestatus);

  bool createstatus = true;
  if (LastSavedIndex >= (NumPlayersAllowed - 1))
  {
    createstatus = false;
  }
  if (!createstatus && *PCreate_Btn == hwnd) {PDone_Btn->SetFocus();}
  PCreate_Btn->EnableWindow(createstatus);

  bool deletestatus = false;
  if (selsavedpn >= 0 && !IsActive(selsavedpn))
  {
    deletestatus = true;
  }
  if (!deletestatus && *PDelete_Btn == hwnd) {PDone_Btn->SetFocus();}
  PDelete_Btn->EnableWindow(deletestatus);

  bool statisticsstatus = false;
  if (selsavedpn >= 0)
  {
    statisticsstatus = true;
  }
  if (!statisticsstatus && *PStatistics_Btn == hwnd) {PDone_Btn->SetFocus();}
  PStatistics_Btn->EnableWindow(statisticsstatus);
}
//*******************************************************************
void TSalariedGamblerDlg::SetupWindow()
{
  TOgleDialog::SetupWindow();
  Refresh();
}
//*******************************************************************
void TSalariedGamblerDlg::CmActivate()
{
  int16 selsaved = int16(PSavedPlayers_LB->GetSelIndex());
  int16 selsavedpn = TranslateSaved2PN(selsaved);
  if (selsavedpn == -1) {return;}  // bad playernum.
  if (IsActive(selsavedpn)) {return;} // already active.

  if (!PCasinoPlayerMgr->ActivatePlayer(selsavedpn,this))
  {
    string strtext = "Unknown error";
    string strtitle = "Error - Activate Gambler";
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               strtext,
                                                               strtitle);
    psgmd->Execute(MB_TASKMODAL);
    delete psgmd;
  }
  else
  {
    Refresh();
  }
}
//*******************************************************************
void TSalariedGamblerDlg::CmDeActivate()
{
  int16 selactive = int16(PActivePlayers_LB->GetSelIndex());
  int16 selactivepn = TranslateActive2PN(selactive);

  if (!PCasinoPlayerMgr->DeActivatePlayer(selactivepn))
  {
    string strtext = "Unknown error";
    string strtitle = "Error - DeActivate Gambler";
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               strtext,
                                                               strtitle);
    psgmd->Execute(MB_TASKMODAL);
    delete psgmd;
  }
  LoadActive_LB();
  HandleButtonStatus();
}
//*******************************************************************
void TSalariedGamblerDlg::CmCreate()
{
  TSalariedGamblerCreateDlg *psgcd = new TSalariedGamblerCreateDlg(this,PCasinoPlayerMgr,IDD_SALARIEDGAMBLER_CREATE,HelpID);
  if (psgcd->Execute(MB_TASKMODAL) == IDOK)
  {
    Refresh();
  }
  delete psgcd;
}
//*******************************************************************
void TSalariedGamblerDlg::CmDelete()
{
  int16 selsaved = int16(PSavedPlayers_LB->GetSelIndex());
  int16 selsavedpn = TranslateSaved2PN(selsaved);
  if (selsavedpn == -1) {return;}
  TSalariedGamblerDeleteDlg *psgdd = new TSalariedGamblerDeleteDlg(this,PCasinoPlayerMgr,
                                                                   IDD_SALARIEDGAMBLER_DELETE,
                                                                   HelpID,selsavedpn);
  if (psgdd->Execute(MB_TASKMODAL) == IDOK)
  {
    PCasinoPlayerMgr->CreateDefPlayer();
    LoadSaved_LB();
    HandleButtonStatus();
  }
  delete psgdd;
}
//*******************************************************************
void TSalariedGamblerDlg::CmStatistics()
{
  int16 selsaved = int16(PSavedPlayers_LB->GetSelIndex());
  int16 selsavedpn = TranslateSaved2PN(selsaved);
  if (selsavedpn == -1) {return;}

  PCasinoPlayerMgr->DoStatisticsDialog(selsavedpn,this);
}
//*******************************************************************
void TSalariedGamblerDlg::CmHelp()
{
  TOgleApp::GetOgleApp()->Help(HELP_CONTEXT,HelpID);
}
//*******************************************************************
void TSalariedGamblerDlg::CmDone()
{
  if (PCasinoPlayerMgr->GetNumActivePlayers() == 0)
  {
    PCasinoPlayerMgr->CreateActivateDefPlayer();
  }
  CmOk();
}
//*******************************************************************
void TSalariedGamblerDlg::CmActiveLBSelChange()
{
  HandleButtonStatus();
}
//*******************************************************************
void TSalariedGamblerDlg::CmActiveLBDblClk()
{
  CmDeActivate();
}
//*******************************************************************
void TSalariedGamblerDlg::CmSavedLBSelChange()
{
  HandleButtonStatus();
}
//*******************************************************************
void TSalariedGamblerDlg::CmSavedLBDblClk()
{

  int16 selsaved = int16(PSavedPlayers_LB->GetSelIndex());
  int16 selsavedpn = TranslateSaved2PN(selsaved);
  if (!IsActive(selsavedpn) && LastActiveIndex < (NumActiveAllowed - 1))
  {
    CmActivate();
  }
}
//*******************************************************************
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TSalariedGamblerCreateDlg,TOgleDialog)
  EV_COMMAND(IDC_CREATE_SEX_MALE_RB,CmSexMale),
  EV_COMMAND(IDC_CREATE_SEX_FEMALE_RB,CmSexFemale),
  EV_COMMAND(IDC_CREATE_SALARY_0_RB,CmSalary0),
  EV_COMMAND(IDC_CREATE_SALARY_200_RB,CmSalary200),
  EV_COMMAND(IDC_CREATE_SALARY_500_RB,CmSalary500),
  EV_COMMAND(IDC_CREATE_SALARY_2500_RB,CmSalary2500),
  EV_COMMAND(IDC_CREATE_BANKROLL_200_RB,CmBankroll200),
  EV_COMMAND(IDC_CREATE_BANKROLL_500_RB,CmBankroll500),
  EV_COMMAND(IDC_CREATE_BANKROLL_2500_RB,CmBankroll2500),
  EV_COMMAND(IDC_CREATE_BANKROLL_5000_RB,CmBankroll5000),
  EV_COMMAND(IDC_CREATE_CANCEL_PB,CmCancel),
  EV_COMMAND(IDC_CREATE_HELP_PB,CmHelp),
  EV_COMMAND(IDC_CREATE_CREATE_PB,CmCreate),
END_RESPONSE_TABLE;
//*******************************************************************
TSalariedGamblerCreateDlg::TSalariedGamblerCreateDlg(TWindow *parent,TCasinoPlayerMgr *pplayermgr,
                                                     TResId dlgid,dword helpid)
:TOgleDialog(parent,dlgid,true),
 HelpID(helpid),
 PCasinoPlayerMgr(pplayermgr),
 bCheater(false)
{
  PSexMale_RB = new TRadioButton(this,IDC_CREATE_SEX_MALE_RB);
  PSexFemale_RB = new TRadioButton(this,IDC_CREATE_SEX_FEMALE_RB);
  PBankroll200_RB = new TRadioButton(this,IDC_CREATE_BANKROLL_200_RB);
  PBankroll500_RB = new TRadioButton(this,IDC_CREATE_BANKROLL_500_RB);
  PBankroll2500_RB = new TRadioButton(this,IDC_CREATE_BANKROLL_2500_RB);
  PBankroll5000_RB = new TRadioButton(this,IDC_CREATE_BANKROLL_5000_RB);
  PSalary0_RB = new TRadioButton(this,IDC_CREATE_SALARY_0_RB);
  PSalary200_RB = new TRadioButton(this,IDC_CREATE_SALARY_200_RB);
  PSalary500_RB = new TRadioButton(this,IDC_CREATE_SALARY_500_RB);
  PSalary2500_RB = new TRadioButton(this,IDC_CREATE_SALARY_2500_RB);

  int16 maxnamelength = int16(PCasinoPlayerMgr->GetMaxNameLength() + 1);
  PName_ETB = new TEdit(this,IDC_CREATE_GAMBLERNAME_ETB,maxnamelength);
  PName_ETB->SetValidator(new TFilterValidator("A-Za-z "));

  PCancel_PB = new TButton(this,IDC_CREATE_CANCEL_PB);
  PHelp_PB = new TButton(this,IDC_CREATE_HELP_PB);
  PCreate_PB = new TButton(this,IDC_CREATE_CREATE_PB);
}
//*******************************************************************
TSalariedGamblerCreateDlg::~TSalariedGamblerCreateDlg()
{
  delete PSexMale_RB;
  delete PSexFemale_RB;
  delete PBankroll200_RB;
  delete PBankroll500_RB;
  delete PBankroll2500_RB;
  delete PBankroll5000_RB;
  delete PSalary0_RB;
  delete PSalary200_RB;
  delete PSalary500_RB;
  delete PSalary2500_RB;
  delete PName_ETB;
  delete PCancel_PB;
  delete PHelp_PB;
  delete PCreate_PB;
}
//*******************************************************************
void TSalariedGamblerCreateDlg::SetupWindow()
{
  TOgleDialog::SetupWindow();
  PSexMale_RB->Check();
  PSexFemale_RB->Uncheck();
  PBankroll200_RB->Uncheck();
  PBankroll500_RB->Check();
  PBankroll2500_RB->Uncheck();
  PBankroll5000_RB->Uncheck();
  PSalary0_RB->Uncheck();
  PSalary200_RB->Check();
  PSalary500_RB->Uncheck();
  PSalary2500_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::Cheat()
{
  bCheater = !bCheater;
  if (bCheater)
  {
    PBankroll5000_RB->SetCaption("$100000");
  }
  else
  {
    PBankroll5000_RB->SetCaption("$5000");
  }
}
//*******************************************************************
bool TSalariedGamblerCreateDlg::PreProcessMsg(MSG& msg)
{
  if (msg.message == WM_KEYDOWN &&
      msg.wParam == '4' &&
      GetKeyState(VK_SHIFT) < 0)
  {
    Cheat();
    return true;
  }
  return TOgleDialog::PreProcessMsg(msg);
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmSexMale()
{
  PSexMale_RB->Check();
  PSexFemale_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmSexFemale()
{
  PSexFemale_RB->Check();
  PSexMale_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmBankroll200()
{
  PBankroll200_RB->Check();
  PBankroll500_RB->Uncheck();
  PBankroll2500_RB->Uncheck();
  PBankroll5000_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmBankroll500()
{
  PBankroll200_RB->Uncheck();
  PBankroll500_RB->Check();
  PBankroll2500_RB->Uncheck();
  PBankroll5000_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmBankroll2500()
{
  PBankroll200_RB->Uncheck();
  PBankroll500_RB->Uncheck();
  PBankroll2500_RB->Check();
  PBankroll5000_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmBankroll5000()
{
  PBankroll200_RB->Uncheck();
  PBankroll500_RB->Uncheck();
  PBankroll2500_RB->Uncheck();
  PBankroll5000_RB->Check();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmSalary0()
{
  PSalary0_RB->Check();
  PSalary200_RB->Uncheck();
  PSalary500_RB->Uncheck();
  PSalary2500_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmSalary200()
{
  PSalary0_RB->Uncheck();
  PSalary200_RB->Check();
  PSalary500_RB->Uncheck();
  PSalary2500_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmSalary500()
{
  PSalary0_RB->Uncheck();
  PSalary200_RB->Uncheck();
  PSalary500_RB->Check();
  PSalary2500_RB->Uncheck();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmSalary2500()
{
  PSalary0_RB->Uncheck();
  PSalary200_RB->Uncheck();
  PSalary500_RB->Uncheck();
  PSalary2500_RB->Check();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmCancel()
{
  TOgleDialog::CmCancel();
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmHelp()
{
  TOgleApp::GetOgleApp()->Help(HELP_CONTEXT,HelpID);
}
//*******************************************************************
string TSalariedGamblerCreateDlg::GetName()
{
  int16 len = int16(PName_ETB->GetTextLen());
  char *buffer = new char[len + 2];
  PName_ETB->GetText(buffer,len + 1);
  string s = buffer;
  delete [] buffer;
  s.strip(string::Both);
  return s;
}
//*******************************************************************
bool TSalariedGamblerCreateDlg::IsNameOk(string name)
{
  if (strlen(name.c_str()) == 0)
  {
    string strtext = "You must enter a name for your new gambler";
    string strtitle = "Ooops!";
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               strtext,
                                                               strtitle);
    psgmd->Execute(MB_TASKMODAL);
    delete psgmd;
    return false;
  }
  if (PCasinoPlayerMgr->IsPlayerValid(name))
  {
    string strtext = "You have entered a name that is already being used by another gambler. Please enter a new and unique name.";
    string strtitle = "Ooops!";
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               strtext,
                                                               strtitle);
    psgmd->Execute(MB_TASKMODAL);
    delete psgmd;

    return false;
  }
  return true;
}
//*******************************************************************
TMoney TSalariedGamblerCreateDlg::GetBankroll()
{
  if (PBankroll200_RB->GetCheck() == BF_CHECKED) {return 200.0;}
  if (PBankroll500_RB->GetCheck() == BF_CHECKED) {return 500.0;}
  if (PBankroll2500_RB->GetCheck() == BF_CHECKED) {return 2500.0;}
  if (bCheater) {return 100000.0;}
  return 5000.0;
}
//*******************************************************************
bool TSalariedGamblerCreateDlg::GetSex()
{
  if (PSexMale_RB->GetCheck() == BF_CHECKED)
  {
    return true;
  }
  else
  {
    return false;
  }
}
//*******************************************************************
TMoney TSalariedGamblerCreateDlg::GetSalary()
{
  if (PSalary0_RB->GetCheck() == BF_CHECKED) {return 0.0;}
  if (PSalary200_RB->GetCheck() == BF_CHECKED) {return 200.0;}
  if (PSalary500_RB->GetCheck() == BF_CHECKED) {return 500.0;}
  return 2500.0;
}
//*******************************************************************
void TSalariedGamblerCreateDlg::CmCreate()
{
  string name = GetName();
  if (!IsNameOk(name)) {return;}
  bool sex = GetSex();
  TMoney salary = GetSalary();
  TMoney bankroll = GetBankroll();
  if (!PCasinoPlayerMgr->CreatePlayer(name,sex,bankroll,salary))
  {
    string strtext = "Couldn't create player";
    string strtitle = "Error - Create Player";
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               strtext,
                                                               strtitle);
    psgmd->Execute(MB_TASKMODAL);
    delete psgmd;
    return;
  }
  bool activateok = false;
  if (PCasinoPlayerMgr->ActivatePlayer(name))
  {
    string strtext = "Gambler ";
    strtext += name;
    strtext += " was also activated for your convenience.";
    string strtitle = "Info - Create Gambler";
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               strtext,
                                                               strtitle);
    psgmd->Execute(MB_TASKMODAL);
    delete psgmd;
    activateok = true;
  }
  else if (PCasinoPlayerMgr->GetNumActivePlayers() == PCasinoPlayerMgr->GetNumActiveAllowed())
  {
    #if defined(PLAYERMGR_FORCEACTIVATEONCREATE)
    TPlayerData *pdat = PCasinoPlayerMgr->GetActivePlayerDataPtr(0);
    if (pdat != NULL)
    {
      string activename = pdat->GetName();
      PCasinoPlayerMgr->DeActivatePlayer(activename);
      if (PCasinoPlayerMgr->ActivatePlayer(name))
      {
        string strtext = "Gambler ";
        strtext += name;
        strtext += " was also activated for your convenience.";
        string strtitle = "Info - Create Gambler";
        TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                                   IDD_SALARIEDGAMBLER_MSG,
                                                                   strtext,
                                                                   strtitle);
        psgmd->Execute(MB_TASKMODAL);
        delete psgmd;
        activateok = true;
      }
    }
    #else
    char buffer[34];
    ltoa(PCasinoPlayerMgr->GetNumActiveAllowed(),buffer,10);
    string strtext = "Due to the ";
    strtext += buffer;
    strtext += " active gambler limit gambler ";
    strtext += name;
    strtext += " couldn't be activated at this time.";
    strtext += "\nYou must DeActivate a gambler and then Activate ";
    strtext += name;
    strtext += ".";
    string strtitle = "Info - Create Gambler";
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               strtext,
                                                               strtitle);
    psgmd->Execute(MB_TASKMODAL);
    delete psgmd;
    #endif
  }
  if (!activateok)
  {
    string strtext = "Due to unknown reason gambler ";
    strtext += name;
    strtext += "couldn't be activated at this time.";
    string strtitle = "Info - Create Gambler";
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(this,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               strtext,
                                                               strtitle);
    psgmd->Execute(MB_TASKMODAL);
    delete psgmd;
  }
  TOgleDialog::CmOk();
}
//*******************************************************************
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TSalariedGamblerMsgDlg,TOgleDialog)
 EV_COMMAND(IDC_MSG_DONE_PB,CmDone),
END_RESPONSE_TABLE;
//*******************************************************************
TSalariedGamblerMsgDlg::TSalariedGamblerMsgDlg(TWindow *parent,TResId dlgid,
                                               string text,string title)
:TOgleDialog(parent,dlgid,true),
 strText(text),
 strTitle(title)
{
  PDone_PB = new TButton(this,IDC_MSG_DONE_PB);
  PText_STB = new TStatic(this,IDC_MSG_TEXT_STB);
}
//*******************************************************************
TSalariedGamblerMsgDlg::~TSalariedGamblerMsgDlg()
{
  delete PDone_PB;
  delete PText_STB;
}
//*******************************************************************
void TSalariedGamblerMsgDlg::SetupWindow()
{
  TOgleDialog::SetupWindow();
  PText_STB->SetText(strText.c_str());
  if (strTitle != "") {SetCaption(strTitle.c_str());}
}
//*******************************************************************
void TSalariedGamblerMsgDlg::CmDone()
{
  TOgleDialog::CmOk();
}
//*******************************************************************
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TSalariedGamblerDeleteDlg,TOgleDialog)
  EV_COMMAND(IDC_DELETE_HELP_PB,CmHelp),
  EV_COMMAND(IDC_DELETE_CANCEL_PB,CmCancel),
  EV_COMMAND(IDC_DELETE_DELETE_PB,CmDelete),
END_RESPONSE_TABLE;
//*******************************************************************
TSalariedGamblerDeleteDlg::TSalariedGamblerDeleteDlg(TWindow *parent,TCasinoPlayerMgr *pplayermgr,
                                                     TResId dlgid,dword helpid,int16 playernum)
:TOgleDialog(parent,dlgid,true),
 HelpID(helpid),
 PlayerNum(playernum),
 PCasinoPlayerMgr(pplayermgr)
{
  PHelp_PB = new TButton(this,IDC_DELETE_HELP_PB);
  PCancel_PB = new TButton(this,IDC_DELETE_CANCEL_PB);
  PDelete_PB = new TButton(this,IDC_DELETE_DELETE_PB);
  PMsg_STB = new TStatic(this,IDC_DELETE_MSG_STB);
}
//*******************************************************************
TSalariedGamblerDeleteDlg::~TSalariedGamblerDeleteDlg()
{
  delete PHelp_PB;
  delete PCancel_PB;
  delete PDelete_PB;
  delete PMsg_STB;
}
//*******************************************************************
void TSalariedGamblerDeleteDlg::SetupWindow()
{
  TOgleDialog::SetupWindow();
  string msg = "About to delete ";
  msg += PCasinoPlayerMgr->GetPlayerName(PlayerNum);
  PMsg_STB->SetText(msg.c_str());
}
//*******************************************************************
void TSalariedGamblerDeleteDlg::CmHelp()
{
  TOgleApp::GetOgleApp()->Help(HELP_CONTEXT,HelpID);
}
//*******************************************************************
void TSalariedGamblerDeleteDlg::CmCancel()
{
  TOgleDialog::CmCancel();
}
//*******************************************************************
void TSalariedGamblerDeleteDlg::CmDelete()
{
  PCasinoPlayerMgr->RemovePlayer(PlayerNum);
  TOgleDialog::CmOk();
}
//*******************************************************************

