/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// grid.cpp                                                                //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.50 - 25 nov 1995                       //
// TGrid - Copyright  1994,1995 H. Picot, All Rights Reserved.            //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#include "griddll.h"
#pragma hdrstop
#include <ctl3d.h>
#include <owl/profile.h>
#include <values.h>

  
// TGrid ----------------------------------------------------------------- //
DEFINE_RESPONSE_TABLE1 (TGrid, TControl)
   EV_WM_ERASEBKGND,
   EV_WM_KILLFOCUS,
   EV_WM_NCPAINT,
   EV_WM_PAINT,
   EV_WM_SETFOCUS,
   EV_WM_WINDOWPOSCHANGING,
END_RESPONSE_TABLE;


// ----------------------------------------------------------------------- //
// constructors, destructor, initialization                                //
// ----------------------------------------------------------------------- //
TGrid::TGrid (TWindow *p, int id, int x0, int y0, int w0, int h0, TGEnterKey ek, TGTabKey tk, TGLineMarker lm, int max, int attrib, TGLayout l,
              TModule *module)
   : TControl  (p, id, "", x0, y0, w0, h0, module)
   {
   Init (max, ek, tk, lm, attrib, l);

   // by default, ini file is named grid; section title is the id of the grid.
   // default can be changed by calling SetFormInfo (const string &ini, const string &section);
   iniFile = "grid.ini";
   char s[16];
   sprintf (s, "%d", id);
   section = s;
   }


TGrid::TGrid (TWindow *p, int id, TGEnterKey ek, TGTabKey tk, TGLineMarker lm, int max, int attrib, TGLayout l, TModule *module)
    : TControl  (p, id, module)
   {
   Attr.Style = BS_OWNERDRAW|WS_CHILD|WS_VISIBLE|WS_TABSTOP|WS_CLIPSIBLINGS;
   Init (max, ek, tk, lm, attrib, l);

   iniFile = "grid.ini";
   char s[16];
   sprintf (s, "%d", id);
   section = s;
   }


void TGrid::Init (int max, TGEnterKey ek, TGTabKey tk, TGLineMarker lm, int attrib, TGLayout l)
   {
   readLayout = true;

   firstColumn = lastColumn = current = 0;
   nControls = hpos = 0;
   titleHeight = vbarWidth = 0;
   logicalWidth = clientWidth = 0;
   lineHeight = clientHeight = 0;
   firstLine = activeLine = visibleLines = 0;
   selectRow = -1;

   if (max >= 0)
      nLines = max;
   else
      nLines = 0;
   maxLines = max;

   bool bwcc = GetApplication()->BWCCEnabled ();
   bool b3D  = GetApplication()->Ctl3dEnabled ();
   formBkg        = TColor(GetSysColor (bwcc||isChicago||b3D ? COLOR_BTNFACE : COLOR_WINDOW));
   lineColor      = TColor::Gray;
   emptyGrid      = GetSysColor (COLOR_WINDOW);
   hilitetext     = GetSysColor (COLOR_HIGHLIGHTTEXT);
   hilite         = GetSysColor (COLOR_HIGHLIGHT);
   parentBkgColor = GetSysColor(COLOR_BTNFACE);

   calcPos = false;
   vscrollHidden = false;
   hscrollHidden = false;
   tempNoHMove = false;
   modified = false;
   echo = false;
   donotcomeback = false;
   noupdate = false;

   enterKey = ek;
   tabKey = tk;
   lineMarker = lm;
   layout = l;
   visible = gControl;

   lineSwapping   = (attrib & gLineSwapOn)      ? true : false;
   centerTitle    = (attrib & gCenterTitle)     ? true : false;
   disableHScroll = (attrib & gDisableHScroll)  ? true : false;
   disableVScroll = (attrib & gDisableVScroll)  ? true : false;
   hideHScroll    = (attrib & gHideHScroll)     ? true : false;
   hideVScroll    = (attrib & gHideVScroll)     ? true : false;
   sizeColumn     = (attrib & gSizeColumn)      ? true : false;
   horzShrink     = (attrib & gHorzShrink)      ? true : false;
   vertShrink     = (attrib & gVertShrink)      ? true : false;
   cursorType     = (attrib & gArrowCursor)     ? ArrowCursor : DefCursor;

   selectedLine = -1;
   nsel = 0;
   lineSelection = (attrib & gSingleSelection) ? Single : None;
   if (lineSelection == Single)
      {
      if ((attrib & gAutoSelection) != 0)
         lineSelection = SingleAutoSelect;
      selectedLines = 0;
      }
   else if ((attrib & gMultipleSelection) != 0)
      {
      lineSelection = Multiple;
      selectedLines = new TBitField (maxLines ? maxLines : 128);
      }
   else
      selectedLines = 0;

   // setup children
   client = new TGridClient (this);
   vscroll = (attrib & gVScroll) ? new TGridVScroll (this)     : 0;
   hscroll = (attrib & gHScroll) ? new TGridHScroll (this)     : 0;
   vbar    = (attrib & gVBar)    ? new TGridVerticalBar (this) : 0;
   title   = new TGridTitle (this, (attrib & gTitle) ? true : false);

   if (vscroll)
      cxvscroll = GetSystemMetrics(SM_CXVSCROLL);
   else
      cxvscroll = 0;
   if (hscroll)
      cyhscroll = GetSystemMetrics(SM_CYHSCROLL);
   else
      cyhscroll = 0;

   gridFont = 0;
   titleFont = 0;

   ComboBmp       =
   CheckBoxesBmp  =
   BwccCheckBmp   =
   BwccUncheckBmp =
   Ctl3dCheckBmp  = 0;

   titleLogFont = 0;
   gridLogFont = new LOGFONT;
   memset (gridLogFont, 0, sizeof(LOGFONT));
   strcpy (gridLogFont->lfFaceName, "MS Sans Serif");
   gridLogFont->lfHeight = MulDiv(TScreenDC().GetDeviceCaps (LOGPIXELSY),9,72);
   gridLogFont->lfWeight = FW_BOLD;

   buttonFace = new TBrush (TColor(GetSysColor (COLOR_BTNFACE)));
   }


TGrid::~TGrid ()
   {
   if (gridLogFont)
      delete gridLogFont;
   if (titleLogFont)
      delete titleLogFont;
   if (firstColumn)
      delete firstColumn;
   if (gridFont)
      delete gridFont;
   if (titleFont)
      delete titleFont;
   if (ComboBmp)
      delete ComboBmp;
   if (CheckBoxesBmp)
      delete CheckBoxesBmp;
   if (BwccCheckBmp)
      delete BwccCheckBmp;
   if (BwccUncheckBmp)
      delete BwccUncheckBmp;
   if (Ctl3dCheckBmp)
      delete Ctl3dCheckBmp;
   if (selectedLines)
      delete selectedLines;
   if (buttonFace)
      delete buttonFace;
   }


void TGrid::PerformCreate (int menuOrId)
   {
   Attr.Style = BS_OWNERDRAW|WS_CHILD|WS_VISIBLE|WS_TABSTOP|WS_CLIPSIBLINGS;
   TControl::PerformCreate (menuOrId);
   }


bool TGrid::Create ()
   {
   bool rc = TControl::Create ();

   if (rc)
      CalcPos ();

   return rc;
   }


char *TGrid::GetClassName()
   {
   #ifdef __WIN32__
   return "TGrid32";
   #else
   return "TGrid16";
   #endif
   }


void TGrid::GetWindowClass(WNDCLASS &wc)
   {
   TControl::GetWindowClass(wc);
   wc.style &= ~CS_GLOBALCLASS;
   wc.hCursor = 0;
   }


void TGrid::SetupWindow ()
   {
   TControl::SetupWindow ();

   TApplication *app = GetApplication ();
   if (app && app->Ctl3dEnabled())
      {
      bool FAR PASCAL (*SubclassDlg)(HWND, WORD);
      (FARPROC)SubclassDlg = app->GetCtl3dModule()->GetProcAddress("Ctl3dSubclassDlg");
      if (SubclassDlg)
         SubclassDlg (*this, CTL3D_ALL);
      }

   if (layout == aForm)
      Go (0);
   }


void TGrid::Add (TControl *p, const char *t, TGAlign a, bool hide)   
   {
   if (p)
      {
      TGridColumn *ge = new TGridColumn (p, lastColumn, this, t);
      lastColumn = ge;
      if (firstColumn == 0)
         firstColumn = ge;

      #if __BORLANDC__ >= 0x500
      if (TGDateEdit *de = dynamic_cast<TGDateEdit *>(p))          // must be before TGEdit
         {
         de->grid = this;
         ge->type = gDateEdit;
         }
      else if (TGTimeEdit *te = dynamic_cast<TGTimeEdit *>(p))     // must be before TGEdit
         {
         te->grid = this;
         ge->type = gTimeEdit;
         }
      else if (TGDoubleEdit *de = dynamic_cast<TGDoubleEdit *>(p)) // must be before TGEdit
         {
         de->grid = this;
         ge->type = gDoubleEdit;
         }
      else if (TGLongEdit *le = dynamic_cast<TGLongEdit *>(p))     // must be before TGEdit
         {
         le->grid = this;
         ge->type = gLongEdit;
         }
      else if (TGShortEdit *se = dynamic_cast<TGShortEdit *>(p))   // must be before TGEdit
         {
         se->grid = this;
         ge->type = gShortEdit;
         }
      else if (TGEdit *e = dynamic_cast<TGEdit *>(p))              // must be before TGStatic
         {
         e->grid = this;
         ge->type = gEdit;
         }
      else if (TGCheckBox *cb = dynamic_cast<TGCheckBox *>(p))   // must be before TGButton
         {
         cb->grid = this;
         ge->type = gCheck;
         }
      else if (TGButton *b = dynamic_cast<TGButton *>(p))
         {
         b->grid = this;
         ge->type = gButton;
         }
      else if (TGComboBox *cb = dynamic_cast<TGComboBox *>(p))
         {
         cb->grid = this;
         ge->type = gCombo;
         }
      else if (TGStatic *s = dynamic_cast<TGStatic *>(p))
         {
         s->grid = this;
         ge->type = gStatic;
         }
      #else
      if (dynamic_cast<TGDateEdit *>(p))         // must be before TGEdit
         {
         dynamic_cast<TGDateEdit *>(p)->grid = this;
         ge->type = gDateEdit;
         }
      else if (dynamic_cast<TGTimeEdit *>(p))    // must be before TGEdit
         {
         dynamic_cast<TGTimeEdit *>(p)->grid = this;
         ge->type = gTimeEdit;
         }
      else if (dynamic_cast<TGDoubleEdit *>(p))  // must be before TGEdit
         {
         dynamic_cast<TGDoubleEdit *>(p)->grid = this;
         ge->type = gDoubleEdit;
         }
      else if (dynamic_cast<TGLongEdit *>(p))    // must be before TGEdit
         {
         dynamic_cast<TGLongEdit *>(p)->grid = this;
         ge->type = gLongEdit;
         }
      else if (dynamic_cast<TGShortEdit *>(p))   // must be before TGEdit
         {
         dynamic_cast<TGShortEdit *>(p)->grid = this;
         ge->type = gShortEdit;
         }
      else if (dynamic_cast<TGEdit *>(p))        // must be before TGStatic
         {
         dynamic_cast<TGEdit *>(p)->grid = this;
         ge->type = gEdit;
         }
      else if (dynamic_cast<TGCheckBox *>(p))    // must be before TGButton
         {
         dynamic_cast<TGCheckBox *>(p)->grid = this;
         ge->type = gCheck;
         }
      else if (dynamic_cast<TGButton *>(p))
         {
         dynamic_cast<TGButton *>(p)->grid = this;
         ge->type = gButton;
         }
      else if (dynamic_cast<TGComboBox *>(p))
         {
         dynamic_cast<TGComboBox *>(p)->grid = this;
         ge->type = gCombo;
         }
      else if (dynamic_cast<TGStatic *>(p))
         {
         dynamic_cast<TGStatic *>(p)->grid = this;
         ge->type = gStatic;
         }
      #endif

      nControls++;

      if (ge->type == gButton)
         ge->align = Center;
      else
         ge->align = a;

      if( hide )                        
        ge->Hidden = -1;                

      // check if column added after grid creation
      if (calcPos)
         {
         AppendColumn ();
         if (title)
            title->Invalidate ();
         Refresh ();
         }
      }
   }


void TGrid::AppendColumn ()
   {
   int w = lastColumn->CtrlWidth();
   lastColumn->left = logicalWidth;
   lastColumn->width = w+1;

   if (lastColumn->type == gCheck)
      lastColumn->width += 2;

   if (lastColumn->left + lastColumn->width > logicalWidth)
      logicalWidth = lastColumn->left + lastColumn->width;

   lastColumn->control->SetParent (client);
   lastColumn->HideControl ();

   SetWindowPos (0, Attr.X, Attr.Y, Attr.W, Attr.H, SWP_NOZORDER);
   CheckHScroll ();
   }


// addition suggested by David M. Larson. -------------------------------- //
void TGrid::ClearModify (TControl *c)
   {
   #if __BORLANDC__ >= 0x500
   if (TGEdit *ed = dynamic_cast<TGEdit*>(c))
      ed->ClearModify ();
   else if (TGCheckBox *ck = dynamic_cast<TGCheckBox *>(c))
      ck->ClearModify ();
   else if (TGComboBox *cb = dynamic_cast<TGComboBox *>(c))
      cb->ClearModify ();
   #else
   TGEdit *ed = dynamic_cast<TGEdit*>(c);
   TGComboBox *cb = dynamic_cast<TGComboBox *>(c);
   TGCheckBox *ck = dynamic_cast<TGCheckBox *>(c);

   if (ed)
      ed->ClearModify ();
   else if (ck)
      ck->ClearModify ();
   else if (cb)
      cb->ClearModify ();
   #endif
   }


bool TGrid::StaticGrid ()
   {
   bool flag = true;
   for (TGridColumn *c = firstColumn; c && flag; c = c->next)
      {
      if (c->IsColumnHidden())
         continue;

      if (c->control)
         {
         #if __BORLANDC__ >= 0x500
         if (TEdit *ed = dynamic_cast<TEdit *>(c->control))       // if a read-only TEdit
         #else
         TEdit *ed = dynamic_cast<TEdit *>(c->control);
         if (ed)       // if a read-only TEdit
         #endif
            {
            if ((ed->Attr.Style & ES_READONLY) != 0)
               continue;
            }
         else           // if not a TEdit but still a TStatic
            {
            if (dynamic_cast<TStatic *>(c->control))
               continue;
            }

         if (c->control->IsWindowEnabled ())
            flag = false;
         }
      }

   return flag;
   }


// ----------------------------------------------------------------------- //
// Locating elements (i.e. columns)                                        //
// ----------------------------------------------------------------------- //
TGridColumn * TGrid::FindColumn (TPoint &p) const
   {
   for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
      {
      if ( ctrl->Hidden )
        continue;

      if (layout == aGrid && TRect (ctrl->left, 0, ctrl->left+ctrl->width, clientHeight).Contains (p))
         return ctrl;
      else if (layout == aForm && ctrl->control)
         {
         TRect rc (ctrl->control->GetClientRect ());
         ctrl->control->MapWindowPoints (*client, rc, 2);
         if (rc.Contains (p))
            return ctrl;
         }
      }
   return 0;
   }


TGridColumn *TGrid::FindColumn (TControl *c) const
   {
   for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
      {
      if ( ctrl->Hidden )
        continue;

      if (ctrl->control == c)
         return ctrl;
      }

   return 0;
   }


TGridColumn *TGrid::FindColumn (HWND hwnd) const
   {
   for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
      {
      if ( ctrl->Hidden )
        continue;

      if (ctrl->control->HWindow == hwnd)
         return ctrl;
      }

   return 0;
   }


TGridColumn *TGrid::FindColumn (int id) const
   {
   for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
      {
      if ( ctrl->Hidden )
        continue;                         

      if (ctrl->control->GetId() == id)
         return ctrl;
      }

   return 0;
   }


int TGrid::GetDataLine (int y)
   {
   if (layout == aForm)
      return firstLine;

   int y0 = 0;
   for (int i = 0; i < visibleLines; i++, y0 += lineHeight)
      {
      if (y >= y0 && y < y0+lineHeight)
         return i + firstLine; 
      }

   return 0;
   }


void TGrid::FindNearestUnlockedCell (TGridColumn *e)
   {
   TGridColumn *prev = e->previous;
   TGridColumn *next = e->next;
   while (prev || next)
      {
      if (prev)
         {
         if (!prev->Hidden && prev->SetFocus ())         
            {
            current = prev;
            return;
            }
         prev = prev->previous;
         }
      if (next)
         {
         if (!next->Hidden && next->SetFocus ())         
            {
            current = next;
            return;
            }
         next = next->next;
         }
      }
   }


// ----------------------------------------------------------------------- //
// Line functions                                                          //
// ----------------------------------------------------------------------- //
void TGrid::SetLineHeight (int h)
   {
   if (h == 0)
      return;
   if (layout == aGrid)
      SetVisibleLines (clientHeight / (lineHeight=h+1));
   else
      {
      SetVisibleLines (1);
      lineHeight = Attr.H;
      }
   }


void TGrid::SetNLines (int n)
   {
   nLines = n;

   if (GetCurrentLine() >= n)
      Go (n > 0 ? n-1 : 0);

   if (vscroll && vscroll->IsWindow())
      {
      if (maxLines < 0)
         vscroll->SetLRange (0, (nLines*10)/9+1, true);
      else
         vscroll->SetLRange (0, nLines-1, true);
      }
   }


void TGrid::SetVisibleLines (int nl)
   {
   if (layout == aForm)
      visibleLines = 1;
   else
      visibleLines = nl;

   if (vscroll)
      vscroll->PageMagnitude = visibleLines;
   }


void TGrid::SetMaxLines (int m)
   {
   if (m < 0)
      nLines = maxLines;
   else
      nLines = m;
   if (nLines < 0)
      nLines = 0;

   if (GetCurrentLine() >= nLines && nLines > 0)
      Go (nLines > 0 ? nLines-1 : 0);

   maxLines = m;
   if (vscroll && vscroll->IsWindow())
      {
      if (maxLines < 0)
         vscroll->SetLRange (0, (nLines*10)/9+1, true);
      else
         vscroll->SetLRange (0, maxLines-1, true);
      }
   }


void TGrid::ResetPosition ()
   {
   Go (0);
   GotoFirstColumn ();
   }


void TGrid::SwapLines (int l1, int l2)
   {
   if (l1 == l2 || l1 < 0 || l2 < 0)
      return;
   if (maxLines >= 0 && (l1 >= maxLines || l2 >= maxLines))
      return;

   title->SetFocus ();

   for (TGridColumn *e = firstColumn; e; e = e->next)
      {
      int id = e->control->GetId ();
      TGType colType = e->type;
      if (colType == gCombo && dynamic_cast<TGDataComboBox *>(e->control))
         {
         #ifdef __WIN32__
            colType = gLongEdit;
         #else
            colType = gShortEdit;
         #endif
         }
      switch (colType)
         {
         // strings (variable size)
         case gStatic:
         case gEdit:
         case gCombo:
         case gButton:           // ???
            {
            string t1 = static_cast<const char *>(TransferItem (l1, id));
            string t2 = static_cast<const char *>(TransferItem (l2, id));
            TransferItem (l1, id, t2.c_str());
            TransferItem (l2, id, t1.c_str());
            break;
            }

         // other types (fixed size)
         default:
            {
            int l = e->control->Transfer (0, tdSizeData);
            char *t1 = new char[l];
            char *t2 = new char[l];
            memcpy (t1, TransferItem (l1, id), l);
            memcpy (t2, TransferItem (l2, id), l);
            TransferItem (l1, id, t2);
            TransferItem (l2, id, t1);
            delete [] t1;
            delete [] t2;
            break;
            }
         }
      }

   int line = l1 - firstLine;
   if (line >= 0 && line < visibleLines)
      {
      int y = line*lineHeight;
      client->InvalidateRect (TRect (0, y, clientWidth, y+lineHeight), false);
      }

   line = l2 - firstLine;
   if (line >= 0 && line < visibleLines)
      {
      int y = line*lineHeight;
      client->InvalidateRect (TRect (0, y, clientWidth, y+lineHeight), false);
      }

   if (!noupdate)
      client->UpdateWindow ();

   if (current && !current->SetFocus ())
      FindNearestUnlockedCell (current);

   SetModify (true);
   }


void TGrid::Move (int delta, bool showControl)
   {
   int n = GetCurrentLine() + delta;
   Go (n >= 0 ? n : 0, showControl);
   }


bool TGrid::HasFocus () const
   {
   HWND currentFocus = GetFocus ();
   if (currentFocus == HWindow)
      return true;
   else if (currentFocus == client->HWindow)
      return true;
   else if (vbar && currentFocus == vbar->HWindow)
      return true;
   else if (title && currentFocus == title->HWindow)
      return true;
   else if (vscroll && currentFocus == vscroll->HWindow)
      return true;
   else if (hscroll && currentFocus == hscroll->HWindow)
      return true;
   else
      {
      for (TGridColumn *c = firstColumn; c; c = c->next)
         {
         if (c->control && currentFocus == c->control->HWindow)
            return true;
         }
      }
   return false;
   }


void TGrid::Go (int pos, bool showControl)
   {
   if (pos < 0)
      {
      firstLine = -1;
      activeLine = 0;
      return;
      }

   // does the grid or one of its components currently has focus? -------- //
   bool hasFocus = HasFocus ();

   // checking pos ------------------------------------------------------- //
   if (pos >= nLines)
      {
      if (maxLines > 0)
         pos = nLines-1;
      else
         {
         // Ask permission to append; -------------------------------------//
         // AppendLine may change pos (passed by reference) ---------------//
         if (!AppendNewLine (pos))
            {
            vscroll->SetLRange (0, nLines-1, true);
            return;
            }

         SetNLines (pos+1);
         }
      }

   if (nLines == 0)
      return;

   if (pos < 0)
      pos = 0;

   // if no change, abort function ----------------------------------------//
   int oldCurrent = GetCurrentLine();
   if (pos == oldCurrent)
      return;

   // check current cell --------------------------------------------------//
   if (hasFocus && !CanKillFocus (current))
      return;

   // check data ----------------------------------------------------------//
   if (!ValidateLine (oldCurrent))
      return;

   // set focus to this, while changing current line ---------------------//
   if (hasFocus && title->SetFocus () == NULL)
      return;     // WM_KILLFOCUS failed

   // remove markers ------------------------------------------------------//
   if (vbar && vbar->IsWindow() && layout == aGrid)
      vbar->ShowText (activeLine);

   // flush controls !!! --------------------------------------------------//
   for (TGridColumn *col = firstColumn; col; col = col->next)
      {
      if( col->Hidden )
         continue;                      

      #if __BORLANDC__ >= 0x500
      if (TStatic *s = dynamic_cast<TStatic *>(col->control))
      #else
      TStatic *s = dynamic_cast<TStatic *>(col->control);
      if (s)
      #endif
         s->SetText ("");
      else
         {
         #if __BORLANDC__ >= 0x500
         if (TGCheckBox *c = dynamic_cast<TGCheckBox *>(col->control))
         #else
         TGCheckBox *c = dynamic_cast<TGCheckBox *>(col->control);
         if (c)
         #endif
            {
            c->SetCheck (BF_GRAYED);
            c->ClearModify ();         /**/
            }
         else
            {
            #if __BORLANDC__ >= 0x500
            if (TComboBox *cb = dynamic_cast<TComboBox *>(col->control))
            #else
            TComboBox *cb = dynamic_cast<TComboBox *>(col->control);
            if (cb)
            #endif
               {
               if ( (Attr.Style & CBS_DROPDOWNLIST) != CBS_DROPDOWNLIST)
                  cb->SetSelIndex (-1);
               }
            else
               {
               #if __BORLANDC__ >= 0x500
               if (TButton *b = dynamic_cast<TButton *>(col->control))
               #else
               TButton *b = dynamic_cast<TButton *>(col->control);
               if (b)
               #endif
                  b->SetCaption ("");
               }
            }
         }
      }

   // compute new position ------------------------------------------------//
   int delta = pos - oldCurrent;
   int oldFirstLine = firstLine;
   if (delta + activeLine >= 0 && delta + activeLine < visibleLines)
      activeLine += delta;

   else
      {
      firstLine += delta;
      if (firstLine+visibleLines > nLines && maxLines > 0)
         firstLine = nLines-visibleLines;
      if (firstLine < 0)
         firstLine = 0;
      activeLine = pos - firstLine;
      if (activeLine < 0)
         activeLine = 0;
      else if (activeLine >= visibleLines)
         activeLine = visibleLines-1;
      }

   // redraw grid or scroll -----------------------------------------------//
   if (firstLine != oldFirstLine)
      {
      int scroll = oldFirstLine-firstLine;
      TRect clip (0, 0, clientWidth, clientHeight);  // ***
      if (scroll >= visibleLines || scroll <= -visibleLines)
         client->InvalidateRect (clip, false);
      else
         {
         int s = scroll*lineHeight;
         client->ScrollWindowEx (0, s, 0, &clip, 0,0, SW_INVALIDATE|SW_SCROLLCHILDREN);
         }
      if (!noupdate)
         client->UpdateWindow ();

      if (vbar && vbar->IsWindow())
         {
         TRect vbclip (0, 0, vbarWidth, clientHeight);
         vbar->InvalidateRect (vbclip, false);
         if (!noupdate)
            vbar->UpdateWindow ();
         }
      }

   // transfer data from memory -------------------------------------------//
   int newCurrent = GetCurrentLine ();

   // update scroller -----------------------------------------------------//
   if (vscroll && vscroll->IsWindow())
      vscroll->SetLPosition (newCurrent);

   // update vertical bar -------------------------------------------------//
   if (vbar && firstLine == oldFirstLine && layout == aGrid)
      vbar->ShowMarker (activeLine);

   // reset focus ---------------------------------------------------------//
   if (current && (hasFocus || current->type == gCombo))
      {
      if (showControl)
         {
         tempNoHMove = true;
         if (!current->SetFocus ())
            FindNearestUnlockedCell (current);
         tempNoHMove = false;
         }
      else if (current->type == gCombo && current->control)
         RefreshItem (oldCurrent, current->control->GetId ());
      }

   // Notify user ---------------------------------------------------------//
   if (firstLine != oldFirstLine)
      ScrollData (firstLine, firstLine+visibleLines-1);

   if (newCurrent != oldCurrent)
      {
      NewRowSelected (newCurrent, oldCurrent);
      if (lineSelection == SingleAutoSelect)
         {
         if (GetSelectedLine() != newCurrent)
            SelectLine (newCurrent);
         }
      }

   // If the grid is static (ie no column can get focus), set focus to the //
   // vertical scroller -------------------------------------------------- //
   if (hasFocus && vscroll && StaticGrid() && vscroll->IsWindow())
      Parent->SendMessage (WM_NEXTDLGCTL, WPARAM(vscroll->HWindow), 1);


   // Refresh all controls with relevant data (form only) ---------------- //
   if (layout == aForm)
      {
      for (TGridColumn *c = firstColumn; c; c = c->next)
         {
         if (c->control) 
            {
            int id = c->control->GetId ();
            int line = GetCurrentLine ();
            void *data = const_cast<void *>(TransferItem (line, id));
            if (data)
               {
               if (c->type == gCombo)
                  dynamic_cast<TGComboBox *>(c->control)->SetData (data);
               else if (c->type == gButton)
                  dynamic_cast<TGButton *>(c->control)->SetCaption ((char *)data);
               else
                  c->control->Transfer (data, tdSetData);
               }

            switch (VisibleCell (line, id)) 
               {
               case gControl:
               case gFlat:
                  c->control->ShowWindow (SW_SHOW);
                  break;

               case gLocked:
               case gEmpty:
                  c->control->ShowWindow (SW_HIDE);
                  break;
               }
            }
         }
      }
   }


// ----------------------------------------------------------------------- //
//                                                                         //
// Line selection functions                                                //
//                                                                         //
// enhanced by Dean Crawford                                               //
//                                                                         //
// ----------------------------------------------------------------------- //
bool TGrid::SelectLine (int line, uint modKeys)
   { 
   // Get keys state
   bool shifted = (modKeys & MK_SHIFT)   ? true : false;
   bool cntrled = (modKeys & MK_CONTROL) ? true : false;
   
   if (lineSelection == Single || lineSelection == SingleAutoSelect)
      {
      // selection outside range of valid lines
      if (line >= 0 && line >= nLines)
         return false;

      // unselecting currently selected line
      else if (line == selectedLine && line != -1)
         {
         int old = line;
         if (lineSelection != SingleAutoSelect)
            {
            selectedLine = -1;
            if (old >= 0)
               {
               RefreshLine (line);
               LineSelected (old, false);
               }
            }
         }

      // selecting a new line and unselecting the old one
      else
         {
         int old = selectedLine;
         selectedLine = -1;
         RefreshLine (old);
         if (old >= 0)
            LineSelected (old, false);

         selectedLine = line;
         RefreshLine (line);
         if (line >= 0)
            LineSelected (line, true);
         }

      return true;
      }
   else if (lineSelection == Multiple)
      {
      if (shifted && selectRow != -1)
         {
         int lower = min(selectRow+1, line);
         int upper = max(selectRow-1, line);
         for (int srows = lower; srows <= upper; srows++)
            {
            if (selectedLines->Read (srows) == false)
               {
               nsel++;
               selectedLines->Toggle (srows);
               RefreshLine (srows);
               LineSelected (srows, true);
               }
            }
         }
      else
         {
         if (!cntrled)
            UnselectAllLines ();
         bool s = selectedLines->Read (line);
         if (s)
            nsel--;
         else
            nsel++;
         selectedLines->Toggle (line);
         RefreshLine (line);
         LineSelected (line, !s);
         }

      selectRow = line;
      return true;
      }

   return false;
   }


bool TGrid::LineSelected (int line) const
   {
   if (selectedLine >= 0 && line == selectedLine)
      return true;
   else if (selectedLines)
      return selectedLines->Read (line);
   else
      return false;
   }


int TGrid::CountSelectedLines () const
   {
   if (selectedLine >= 0)
      return 1;
   else
      return nsel;
   }


int TGrid::GetSelectedLine ()   const
   {
   if (lineSelection != None)
      {
      if (selectedLine >= 0)
         return selectedLine;
      else if (selectedLines)
         {
         ulong n = selectedLines->FindFirst();
         if (n == bitNoFind)
            return -1;
         else
            return int(n);
         }
      }

   return -1;
   }


int TGrid::GetSelectedLines (int *list) const
   {
   if (list == 0)
      return 0;

   if (lineSelection == Single || lineSelection == SingleAutoSelect)
      {
      if (selectedLine >= 0)
         {
         list[0] = selectedLine;
         return 1;
         }
      }
   else if (lineSelection == Multiple)
      {
      int index = 0;
      for (ulong n = selectedLines->FindFirst(); index < nsel && n != bitNoFind; n = selectedLines->FindNext())
         list[index++] = int(n);
      return nsel;
      }

   return 0;
   }


int TGrid::GetSelectedLines (TBitField &bf) const
   {
   if (lineSelection == Single || lineSelection == SingleAutoSelect)
      {
      bf.ClearAll();
      if (selectedLine >= 0)
         {
         bf.Set (selectedLine);
         return 1;
         }
      }
   else if (lineSelection == Multiple)
      {
      bf = *selectedLines;
      return nsel;
      }

   return 0;
   }


void TGrid::UnselectAllLines ()
   {
   if (current)
      current->HideControl ();
   if (lineSelection == Single || lineSelection == SingleAutoSelect)
      {
      if (selectedLine >= 0)
         {
         int old = selectedLine;
         selectedLine = -1;
         RefreshLine (old);
         LineSelected (old, false);
         }
      }
   else if (lineSelection == Multiple)
      {
      nsel = 0;
      for (ulong n = selectedLines->FindFirst(); n != bitNoFind; n = selectedLines->FindNext())
         {
         selectedLines->Clear (n);
         RefreshLine (int(n));
         LineSelected (int(n), false);
         }
      selectRow = -1;
      }
   if (current)
      current->ShowControl ();
   }


bool TGrid::ValidateLine (int line)
   {
   // retrieve data from current control
   TControl *c = current ? current->control : 0;
   if (c)
      {
      // check if data has been touched
      bool touched = false;
      #if __BORLANDC__ >= 0x500
      if (TGEdit *ed = dynamic_cast<TGEdit *>(c))
         touched = ed->IsModified ();
      else if (TGCheckBox *ck = dynamic_cast<TGCheckBox *>(c))
         touched = ck->IsModified ();
      else if (TGComboBox *cb = dynamic_cast<TGComboBox *>(c))
         touched = cb->IsModified();
		#else
      TGEdit *ed = dynamic_cast<TGEdit *>(c);
      TGComboBox *cb = dynamic_cast<TGComboBox *>(c);
      TGCheckBox *ck = dynamic_cast<TGCheckBox *>(c);
      if (ed)
         touched = ed->IsModified ();
      else if (ck)
         touched = ck->IsModified();
      else if (cb)
         touched = cb->IsModified ();
      #endif

      if (touched)
         {
         void *data = 0;
         if (current->type == gCombo)
            data = dynamic_cast<TGComboBox *>(c)->GetData ();
         else
            {
            int l = c->Transfer (0, tdSizeData);
            if (l > 0)
               {
               data = new char[l];
               c->Transfer (data, tdGetData);
               }
            }

         int id = c->GetId ();
         bool rc = DataChanging (line, id, data);
         if (rc)
            {
            modified = true;
            TransferItem (line, id, data);
            DataChanged (line, id, data);
            ClearModify (c);
            }

         if (data)
            delete [] data;
         if (!rc)
            return false;
         }
      }

   // now, validate line
   return IsValidLine (line);
   }

// ----------------------------------------------------------------------- //
// Keyboard response                                                       //
// ----------------------------------------------------------------------- //
bool TGrid::KeyDown (uint key, int repeat)
   {
   bool ctrl  = (GetKeyState (VK_CONTROL) & 0x8000) ? true : false;
   bool shift = (GetKeyState (VK_SHIFT)   & 0x8000) ? true : false;

   switch (key)
      {
      case VK_ESCAPE:   return VkEscape (ctrl, shift, repeat);
      case VK_INSERT:   return VkInsert (ctrl, shift, repeat);
      case VK_DELETE:   return VkDelete (ctrl, shift, repeat);
      case VK_PRIOR:    return VkPrior  (ctrl, shift, repeat);
      case VK_NEXT:     return VkNext   (ctrl, shift, repeat);
      case VK_UP:       return VkUp     (ctrl, shift, repeat);
      case VK_DOWN:     return VkDown   (ctrl, shift, repeat);
      case VK_RIGHT:    return VkRight  (ctrl, shift, repeat);
      case VK_LEFT:     return VkLeft   (ctrl, shift, repeat);
      case VK_RETURN:   return VkReturn (ctrl, shift, repeat);
      case VK_TAB:      return VkTab    (ctrl, shift, repeat);
      case VK_HOME:     return VkHome   (ctrl, shift, repeat);
      case VK_END:      return VkEnd    (ctrl, shift, repeat);
      }

   return false;
   }


bool TGrid::VkEscape (bool ctrl, bool shift, int)
   {
   if (ctrl == false && shift == false)
      {
      for (TGridColumn *c = firstColumn; c; c = c->next)
         {
         #if __BORLANDC__ >= 0x500
         if (TEdit *ed = dynamic_cast<TEdit *>(c->control))
         #else
         TEdit *ed = dynamic_cast<TEdit *>(c->control);
         if (ed)
         #endif
            ed->SetValidator (0);
         }
      Parent->PostMessage (WM_COMMAND, IDCANCEL);
      return true;
      }

   return false;
   }


bool TGrid::VkEnd (bool ctrl, bool, int)
   {
   if (ctrl)
      {
      if (!CanKillFocus (current))
         return true;

      GotoLastColumn ();
      return true;
      }

   return false;
   }


bool TGrid::VkHome (bool ctrl, bool, int)
   {
   if (ctrl)
      {
      if (!CanKillFocus (current))
         return true;

      GotoFirstColumn ();
      return true;
      }

   return false;
   }


bool TGrid::VkRight (bool, bool, int repeat)
   {
   for (int i = 0; i < repeat; i++)
      if (!GotoNextColumn ())
         break;
   return true;
   }


bool TGrid::VkLeft (bool, bool, int repeat)
   {
   for (int i = 0; i < repeat; i++)
      if (!GotoPreviousColumn ())
         break;
   return true;
   }


bool TGrid::VkUp (bool ctrl, bool, int repeat)
   {
   if (ctrl)
      {
      if (lineSwapping == false)
         return false;
      for (int i = 0; i < repeat; i++)
         {
         int l = firstLine+activeLine;
         if (l <= 0)
            break;
         SwapLines (l-1, l);
         Move (-1);
         }
      }
   else
      Move (-repeat);
   return true;
   }


bool TGrid::VkDown (bool ctrl, bool, int repeat)
   {
   if (ctrl)
      {
      if (lineSwapping == false)
         return false;
      for (int i = 0; i < repeat; i++)
         {
         int l = firstLine+activeLine;
         if (l >= nLines-1)
            break;
         SwapLines (l, l+1);
         Move (1);
         }
      }
   else
      Move (repeat);
   return true;
   }


bool TGrid::VkNext (bool ctrl, bool, int repeat)
   {
   if (ctrl)
      {
      int n1 = nLines-visibleLines-1;
      Go (n1 >= 0 ? n1 : 0, false);
      Go (nLines > 0 ? nLines-1 : 0);
      }
   else
      Move (visibleLines*repeat);
   return true;
   }


bool TGrid::VkPrior (bool ctrl, bool, int repeat)
   {
   if (ctrl)
      Go (0);
   else
      Move (-visibleLines*repeat);
   return true;
   }


bool TGrid::VkReturn (bool ctrl, bool shift, int repeat)
   {
   if (ctrl)
      return false;

   else if (shift && lineSelection != None)
      return SelectLine (GetCurrentLine ());

   switch (enterKey)
      {
      case NextLine:
         {
         if (!CanKillFocus (current))
            return true;

         Move (repeat, false);
         GotoFirstColumn ();
         return true;
         }

      case DoNothing:
         return true;

      case ExitDialog:
         return false;

      case EnterNextColumn:
         {
         if (!CanKillFocus (current))
            return true;

         for (int i = 0; i < repeat; i++)
            {
            if (!GotoNextColumn ())
               {
               Move (1, false);
               GotoFirstColumn ();
               }
            }
         return true;
         }
      }
   return false;
   }


bool TGrid::VkTab (bool ctrl, bool shift, int repeat)
   {
   if (ctrl || tabKey == NextControl)
      {
      if (ValidateLine(activeLine+firstLine))
         {
         HWND hwnd = Parent->GetNextDlgTabItem (Parent->HWindow, shift);
         if (hwnd)
            {
            if (current)
               current->HideControl ();
            Parent->PostMessage (WM_NEXTDLGCTL, shift);
            }
         }
      return true;
      }
   else
      {
      if (!CanKillFocus (current))
         return true;

      if (tabKey == AutoNextLine)
         {
         if (shift)
            {
            for (int i = 0; i < repeat; i++)
               {
               if (!GotoPreviousColumn ())
                  {
                  Move (-1, false);
                  GotoLastColumn ();
                  }
               }
            }
         else
            {
            for (int i = 0; i < repeat; i++)
               {
               if (!GotoNextColumn ())
                  {
                  Move (1, false);
                  GotoFirstColumn ();
                  }
               }
            }
         return true;
         }
      else
         {
         if (shift)
            return VkLeft (ctrl, shift, repeat);
         else
            return VkRight (ctrl, shift, repeat);
         }
      }
   }


bool TGrid::VkDelete (bool ctrl, bool, int)
   {
   if (ctrl)
      {
      title->SetFocus ();
      if (DeleteLine (activeLine+firstLine) && activeLine >= 0 && activeLine < visibleLines)
         {
         // optimization made by Ren Mayer
         int y = activeLine*lineHeight;
         client->InvalidateRect (TRect (0, y, clientWidth, clientHeight), false);
         if (!noupdate)
            client->UpdateWindow ();
         // end of optimization
         }
      if (current && !current->SetFocus ())
         FindNearestUnlockedCell (current);
      SetModify (true);
      return true;
      }
   return false;
   }


bool TGrid::VkInsert (bool ctrl, bool, int repeat)
   {
   if (ctrl)
      {
      if (!CanKillFocus (current) || !ValidateLine (activeLine+firstLine))
         return true;
      title->SetFocus ();
      for (int i = 0; i < repeat; i++)
         {
         if (!InsertLine (activeLine+firstLine))
            return true;
         }
      // optimization made by Ren Mayer
      if (activeLine >= 0 && activeLine < visibleLines)
         {
         int y = activeLine*lineHeight;
         client->InvalidateRect (TRect (0, y, clientWidth, clientHeight), false);
         if (!noupdate)
            client->UpdateWindow ();
         }
      // end of optimization
      if (current && !current->SetFocus ())
         FindNearestUnlockedCell (current);
      SetModify (true);
      return true;
      }
   return false;
   }


// ----------------------------------------------------------------------- //
// Handling columns                                                        //
// ----------------------------------------------------------------------- //
bool TGrid::GotoPreviousColumn ()
   {
   TGridColumn *e = current;
   if (e == 0)
      e = lastColumn;
   else
      e = e->previous;
   while (e && e->Hidden)      
      e = e->previous;
   if (e == 0 && layout == aGrid)
      return false;
   for (; e != current; e = e->previous)
      {
      if (e == 0)
         {
         if (layout == aForm)
            e = lastColumn;
         else
            break;
         }

      if (e->SetFocus ())
         return true;
      }

   if (vscroll && vscroll->IsWindow() && (StaticGrid() || donotcomeback))
      Parent->PostMessage (WM_NEXTDLGCTL, WPARAM(vscroll->HWindow), 1);

   if (!donotcomeback)
      GotoFirstColumn ();

   return false;
   }


bool TGrid::GotoNextColumn ()
   {
   TGridColumn *e = current;
   if (e == 0)
      e = firstColumn;
   else
      e = e->next;
   while (e && e->Hidden)
      e = e->next;
   if (e == 0 && layout == aGrid)
      return false;
   for (; e != current; e = e->next)
      {
      if (e == 0)
         {
         if (layout == aForm)
            e = firstColumn;
         else
            break;
         }

      if (e->SetFocus ())
         return true;
      }

   if (vscroll && vscroll->IsWindow() && (StaticGrid() || donotcomeback))
      Parent->PostMessage (WM_NEXTDLGCTL, WPARAM(vscroll->HWindow), 1);

   if (!donotcomeback)
      GotoLastColumn ();

   return false;
   }


void TGrid::GotoLastColumn ()
   {
   if (current)
      current->HideControl ();
   current = lastColumn;
   if (current && !current->SetFocus ())
      {
      donotcomeback = true;
      GotoPreviousColumn ();
      donotcomeback = false;
      }
   }


void TGrid::GotoFirstColumn ()
   {
   if (current)
      current->HideControl ();
   current = firstColumn;
   if (current && !current->SetFocus ())
      {
      donotcomeback = true;
      GotoNextColumn ();
      donotcomeback = false;
      }
   }


void TGrid::SetHPos (int newpos)
   {
   MoveHPos (newpos - hpos);
   }


void TGrid::MoveHPos (int delta)
   {
   if (layout == aGrid)
      {
      int pos = hpos + delta;
      if (pos < 0)
         pos = 0;
      else if (clientWidth >= logicalWidth)
         pos = 0;
      else if (lastColumn->left+lastColumn->width-delta < clientWidth)
         pos = logicalWidth-clientWidth;
      delta = pos - hpos;

      for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
         {
         if (ctrl->Hidden)
           continue;

         ctrl->left -= delta;
         }

      TRect clip (0, 0, clientWidth, clientHeight);
      client->ScrollWindowEx (-delta, 0, 0, &clip, 0,0, SW_INVALIDATE|SW_SCROLLCHILDREN);
      if (!noupdate)
         client->UpdateWindow ();

      if (title && title->IsWindow() && titleHeight > 0)
         {
         title->Invalidate ();
         if (!noupdate)
            title->UpdateWindow ();
         }

      if (hscroll && hscroll->IsWindow())
         hscroll->SetPosition (hpos = pos);
      }
   }


void TGrid::SetColumnWidth (int id, int width)
   {
   if (layout == aGrid)
      {
      TGridColumn *e = FindColumn (id);
      if (e)
         {
         int delta = width - e->width;
         if (delta)
            ChangeColWidth (e, delta);
         }
      }
   }


void TGrid::ChangeColWidth (TGridColumn *e, int delta, int minw)
   {
   if (e && layout == aGrid)
      {
      if (e->Hidden)                         
         return;                             

      if (delta < 0 && delta < -(e->width-minw)) // min width is 8 pixels
         delta = -(e->width-minw);

      e->width += delta;
      for (TGridColumn *col = e->next; col; col = col->next)
         col->left += delta;
      logicalWidth += delta;

      CheckHScroll ();

      if (delta && IsWindow ())
         {
         int x = e->left+e->width;
         int lx = lastColumn->left+lastColumn->width;
         bool updateRight = true;
         if (lx < clientWidth)
            {
            int delta = clientWidth-lx;
            int pos = hpos + delta;
            if (pos < 0)
               pos = 0;
            else if (clientWidth >= logicalWidth)
               pos = 0;
            else if (lastColumn->left+lastColumn->width-delta < clientWidth)
               pos = logicalWidth-clientWidth;
            delta = pos - hpos;

            if (delta)
               {
               for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
                  ctrl->left -= delta;

               if (title && title->IsWindow() && titleHeight > 0)
                  title->InvalidateRect (TRect(0,0,e->left+e->width,titleHeight), false);

               if (hscroll && hscroll->IsWindow())
                  hscroll->SetPosition (hpos = pos);

               TRect clip (0, 0, e->left, clientHeight);
               client->ScrollWindowEx (-delta, 0, 0, &clip, 0,0, SW_INVALIDATE|SW_SCROLLCHILDREN);
               updateRight = false;
               }
            }
         if (updateRight)
            {
            if (title && title->IsWindow() && titleHeight > 0)
               title->InvalidateRect (TRect (e->left, 0, clientWidth, titleHeight), false);
            TRect clip (x, 0, clientWidth, clientHeight);
            client->ScrollWindowEx (delta, 0, 0, &clip, 0,0, SW_INVALIDATE|SW_SCROLLCHILDREN);
            }
         }

      client->InvalidateRect (TRect (e->left, 0, e->left+e->width, clientHeight), false);
      if (!noupdate)
         {
         if (title && title->IsWindow() && titleHeight > 0)
            title->UpdateWindow ();
         client->UpdateWindow ();
         }

      bool vs = (vscroll && vscroll->IsWindow () && !vscrollHidden) ? true : false;
      if (horzShrink && (logicalWidth < clientWidth || (clientWidth >= logicalWidth && Attr.W > vbarWidth+clientWidth+cxvscroll*vs)))
         {
         ChangeWindowPos (Attr.W, Attr.H, SWP_NOMOVE);
         InvalidateRect (TRect (vbarWidth+clientWidth-2, 0, Attr.W, Attr.H), false);
         if (!noupdate)
            UpdateWindow ();
         }
      }
   }


void TGrid::Remove (int id, bool destroy)
   {
   TGridColumn *e = FindColumn (id);
   if (e)
      Remove (e, destroy);
   }


void TGrid::Remove (TGridColumn *e, bool destroy)
   {
   if (e == 0)
      return;

   if ( ! e->Hidden )                
      {
      int w = e->width;
      for (TGridColumn *col = e->next; col; col = col->next)
         col->left -= w;
      logicalWidth -= w;

      CheckHScroll ();
      client->InvalidateRect (TRect (e->left, 0, clientWidth, clientHeight), false);
      if (!noupdate)
         client->UpdateWindow ();

      if (title && title->IsWindow() && titleHeight > 0)
         {
         title->InvalidateRect (TRect (e->left, 0, clientWidth, titleHeight), false);
         if (!noupdate)
            title->UpdateWindow ();
         }

      if (current == e)
         {
         if (e->next)
            current = e->next;
         else if (e->previous)
            current = e->previous;
         else
            current = 0;
         }

      if (e->next)
         e->next->previous = e->previous;
      if (e->previous)
         e->previous->next = e->next;
      if (lastColumn == e)
         lastColumn = e->previous;
      if (firstColumn == e)
         firstColumn = e->next;
      }

   if (destroy)
      delete e->control;

   e->next = e->previous = 0;
   delete e;
   }


// This section was suggested by Charles R. Cordell
void TGrid::HideColumn(int id)
   {
   if (layout == aGrid)
      {
      TGridColumn *e = FindColumn (id);
      if ( e )
         {
         if (current == e)
            FindNearestUnlockedCell (e);     // set current to closest column
         int w = e->width;
         ChangeColWidth (e, -w, 0);
         e->Hidden = w;
         }
      }
   }


void TGrid::UnHideColumn(int id)
   {
   if (layout == aGrid)
      {
      TGridColumn *e;
      for (e = firstColumn; e; e = e->next)
         if ( id == e->control->GetId() )
            break;

      if (e && e->Hidden)
        {
        int w = e->Hidden;
        e->Hidden = 0;
        ChangeColWidth (e, w, 0);
        }
      }
   }
// end of section

void TGrid::HideColumns (bool hide, int *ids, int ncol)
   {
   if (ids > 0 && layout == aGrid)
      {
      noupdate = true;

      if (hide)
         {
         for (int i = 0; i < ncol; i++)
            HideColumn (ids[i]);
         }
      else
         {
         for (int i = 0; i < ncol; i++)
            UnHideColumn (ids[i]);
         }

      noupdate = false;

      client->UpdateWindow ();
      vbar->UpdateWindow ();
      title->UpdateWindow ();
      }
   }


bool TGrid::IsColumnHidden (int id)  const
   {
   TGridColumn *c = FindColumn (id);
   if (c)
      return c->IsColumnHidden ();
   else
      return true;
   }


void TGrid::CheckHScroll ()
   {
   // check if horiz scroll has to be shown/hidden --------------------------
   if (hscroll && hscroll->IsWindow())
      {
      bool scrollable = ((logicalWidth > clientWidth) ? true : false);
      if (hideHScroll)
         {
         bool bVisible = !hscrollHidden;
         if ((bVisible ^ scrollable) != 0)
            SetWindowPos(0,0,0,Attr.W,Attr.H,SWP_NOZORDER|SWP_NOMOVE);  
         }
      else if (disableHScroll)
         {
         bool enabled = (hscroll->IsWindowEnabled () ? true : false);
         if ((enabled ^ scrollable) != 0)
            hscroll->EnableWindow (logicalWidth > clientWidth);
         }
      }

   SetHScrollRange ();
   }


// ----------------------------------------------------------------------- //
// Colors                                                                  //
// ----------------------------------------------------------------------- //
HBRUSH TGrid::CtlColor(HDC hDC, HWND hWndChild, uint ctlType)
   {
   TGridColumn *e = FindColumn (hWndChild);
   if (e)
      {
      TWindow *wptr = e->control;
      TColor bg, fg;
      HBRUSH brush;

      if (dynamic_cast<TGButton *>(wptr) != 0 && LineSelected (GetCurrentLine()))
         {
         static HBRUSH brush = 0;
         TDC dc(hDC);
         dc.SetTextColor (hilitetext);
         dc.SetBkColor (hilite);
         brush = TBrush (hilite);
         return brush;
         }

      bool customColors = CellColor (GetCurrentLine(), wptr->GetId(), true, bg, fg, brush);
      if (customColors)
         {
         TDC dc(hDC);
         dc.SetTextColor (fg);
         dc.SetBkColor (bg);
         return brush;
         }

      #if __BORLANDC__ >= 0x500
      if (TGEdit *e = dynamic_cast<TGEdit *>(wptr))
         return e->CtlColors (hDC);

      if (TGComboBox *cb = dynamic_cast<TGComboBox *>(wptr))
         return cb->CtlColors (hDC);

      if (TGStatic *s = dynamic_cast<TGStatic *>(wptr))
         return s->CtlColors (hDC);

      if (TGCheckBox *cb = dynamic_cast<TGCheckBox *>(wptr))
         return cb->CtlColors (hDC);
      #else
      if (dynamic_cast<TGEdit *>(wptr))
         return dynamic_cast<TGEdit *>(wptr)->CtlColors (hDC);

      if (dynamic_cast<TGComboBox *>(wptr))
         return dynamic_cast<TGComboBox *>(wptr)->CtlColors (hDC);

      if (dynamic_cast<TGStatic *>(wptr))
         return dynamic_cast<TGStatic *>(wptr)->CtlColors (hDC);

      if (dynamic_cast<TGCheckBox *>(wptr))
         return dynamic_cast<TGCheckBox *>(wptr)->CtlColors (hDC);
		#endif

      TApplication *app = GetApplication ();
      if (app->BWCCEnabled () || app->Ctl3dEnabled())
         {
         char buffer[32];
         ::GetClassName (hWndChild, buffer, 32);
         if (stricmp (buffer, "Static") == 0)
            {
            TDC(hDC).SetBkColor (GetSysColor (COLOR_BTNFACE));
            return HBRUSH(*buttonFace);
            }
         }

      if (app->Ctl3dEnabled())
         {
         HBRUSH WINAPI(*CtlColorEx)(uint, WPARAM, LPARAM);
         (FARPROC)CtlColorEx = app->GetCtl3dModule()->GetProcAddress("Ctl3dCtlColorEx");
         HBRUSH hb = 0;
         if (CtlColorEx)
         #if defined(__WIN32__)
           hb = CtlColorEx(WM_CTLCOLORMSGBOX+ctlType, WPARAM(hDC), LPARAM(hWndChild));
         #else
           hb = CtlColorEx(WM_CTLCOLOR, WPARAM(hDC), MAKELPARAM(hWndChild, ctlType));
         #endif

         if (hb != 0)
            return hb;
         }
      }

   return TWindow::EvCtlColor(hDC, hWndChild, ctlType);
   }


// ----------------------------------------------------------------------- //
// Switching focus                                                         //
// ----------------------------------------------------------------------- //
void TGrid::EvSetFocus (HWND)
   {
   if (current == 0)
      GotoFirstColumn ();
   else
      current->SetFocus ();

   RefreshLine (GetCurrentLine ());

   if (vscroll && StaticGrid() && vscroll->IsWindow())
      Parent->PostMessage (WM_NEXTDLGCTL, WPARAM(vscroll->HWindow), 1);
   }


void TGrid::EvKillFocus (HWND hWndGetFocus)
   {
   if (current && CanKillFocus (current) && current->control->HWindow != hWndGetFocus)
      current->HideControl ();
   }


void TGrid::SetFocusOnChild (TControl *child, HWND /*loosefocus*/)
   {
   TGridColumn *e = FindColumn (child);
   if (e && !e->Hidden)          
      {
      int delta = 0;

      current = e;

      if (e->left < 0)
         delta = e->left;
      else if (e->left+e->width > clientWidth)
         delta = e->left+e->width - clientWidth;
      if (delta && !tempNoHMove)
         MoveHPos (delta);

      int id = child->GetId ();
      int line = GetCurrentLine ();
      void *data = const_cast<void *>(TransferItem (line, id));
      if (data)               // fix made by Ren Mayer ------------------------
         {
         if (e->type == gCombo)
            dynamic_cast<TGComboBox *>(child)->SetData (data);
         // fix made by Ren Mayer ---------------------------------------------
         else if (e->type == gButton)
            dynamic_cast<TGButton *>(child)->SetCaption ((char *)data);
         // end of fix ------------------------------------------------------------
         else
            child->Transfer (data, tdSetData);
         }
      e->ShowControl ();
      }
   }


void TGrid::KillFocusFromChild (TControl *c, HWND /*getfocus*/, bool okToChange)
   {
   if (current)
      {
      TGridColumn *ctrl = FindColumn (c);
      if (ctrl)
         {
         void *data = 0;
         if (ctrl->type == gCombo)
            data = dynamic_cast<TGComboBox *>(ctrl->control)->GetData ();
         else
            {
            int l = ctrl->control->Transfer (0, tdSizeData);
            if (l > 0)
               {
               data = new char[l];
               ctrl->control->Transfer (data, tdGetData);
               }
            }
         if (okToChange)
            {
            modified = true;
            int l = GetCurrentLine ();
            int id = c->GetId ();
            TransferItem (l, id, data);
            DataChanged (l, id, data);
            ClearModify (c);
            }
         if (data)
            delete [] data;
         }
      current->HideControl ();
      if (!noupdate)
         client->UpdateWindow ();
      }
   }


bool TGrid::CanKillFocus (TGridColumn *e)
   {
   if (e)
      return e->CanKillFocus ();
   else
      return true;
   }

// ----------------------------------------------------------------------- //
// Painting                                                                //
// ----------------------------------------------------------------------- //
bool TGrid::EvEraseBkgnd (HDC)
   {
   if (layout == aGrid)
      {
      bool hs = (hscroll && hscroll->IsWindow () && !vscrollHidden) ? true : false;
      bool vs = (vscroll && vscroll->IsWindow () && !hscrollHidden) ? true : false;

      // upper right corner --------------------------------------------------//
      if (cxvscroll && titleHeight && vs)
         InvalidateRect (TRect (TPoint(vbarWidth+clientWidth, 0), TSize(cxvscroll, titleHeight)), false);

      // lower left corner ---------------------------------------------------//
      if (vbarWidth && cyhscroll && hs)
         InvalidateRect (TRect (TPoint(0, titleHeight+clientHeight), TSize(vbarWidth, cyhscroll)), false);

      // lower right corner --------------------------------------------------//
      if (cxvscroll && cyhscroll && vs && hs)
         InvalidateRect (TRect (TPoint(vbarWidth+clientWidth, titleHeight+clientHeight), TSize(cxvscroll, cyhscroll)), false);

      if (!noupdate)
         UpdateWindow ();

      return true;
      }
   else
      return false;
   }


void TGrid::EvNCPaint ()
   {
   }

void TGrid::EvPaint ()
   {
   if (layout == aGrid)
      {
      TPaintDC dc(*this);
      Paint(dc, dc.Ps.fErase, *(TRect*)&dc.Ps.rcPaint);
      }
   else
      TControl::EvPaint ();
   }

void TGrid::Paint (TDC &dc, bool, TRect &clip)
   {
   if (layout == aGrid)
      {
      TRegion rgnClip (clip);
      dc.SelectClipRgn (rgnClip);
      bool hs = (hscroll && hscroll->IsWindow () && !hscrollHidden) ? true : false;
      bool vs = (vscroll && vscroll->IsWindow () && !vscrollHidden) ? true : false;

      int cx = cxvscroll*vs;
      int cy = cyhscroll*hs;

      dc.FrameRect (TRect (0,0,vbarWidth+clientWidth+cx,titleHeight+clientHeight+cy), TBrush(TColor(GetTitleColor (TitleFrame))));

      // upper left corner -------------------------------------------------- //
      if (vbarWidth && titleHeight)
         DrawCorner (dc, TRect (TPoint(1, 1), TSize(vbarWidth-2, titleHeight-1)));

      // upper right corner --------------------------------------------------//
      if (cx && titleHeight)
         DrawCorner (dc, TRect (TPoint(vbarWidth+clientWidth-1, 1), TSize(cxvscroll, titleHeight-1)));

      // lower left corner ---------------------------------------------------//
      if (vbarWidth && cy)
         DrawCorner (dc, TRect (TPoint(1, titleHeight+clientHeight), TSize(vbarWidth-2, cyhscroll)));

      // lower right corner --------------------------------------------------//
      if (cx && cy)
         DrawCorner (dc, TRect (TPoint(vbarWidth+clientWidth-1, titleHeight+clientHeight), TSize(cxvscroll, cyhscroll)));

      TPen pen (GetTitleColor (TitleFrame));
      dc.SelectObject (pen);
      if (vbarWidth)
         {
         dc.MoveTo (vbarWidth-1, 0);
         dc.LineTo (vbarWidth-1, titleHeight+clientHeight+cy);
         }
      if (cx)
         {
         dc.MoveTo (vbarWidth+clientWidth-2, 0);
         dc.LineTo (vbarWidth+clientWidth-2, titleHeight+clientHeight+cy);
         }
      if (titleHeight)
         {
         dc.MoveTo (0, titleHeight);
         dc.LineTo (vbarWidth+clientWidth+cx, titleHeight);
         }
      if (cy)
         {
         dc.MoveTo (0, titleHeight+clientHeight-1);
         dc.LineTo (vbarWidth+clientWidth+cx, titleHeight+clientHeight-1);
         }
      dc.RestorePen ();

      dc.SaveDC ();
      TRegion rgn (GetClientRect());
      TRegion excl (TRect(0, 0, vbarWidth+clientWidth+cx, titleHeight+clientHeight+cy));
      rgn -= excl;
      TBrush *hbr;
		#if __BORLANDC__ >= 0x500
      if (TGDialog *p = dynamic_cast<TGDialog *>(Parent))
      #else
      TGDialog *p = dynamic_cast<TGDialog *>(Parent);
      if (p)
      #endif
         hbr = new TBrush(p->BkgBrush());
      else
         hbr = new TBrush(parentBkgColor);
      dc.SelectClipRgn (rgn);
      dc.FillRect (GetClientRect(), *hbr);
      delete hbr;
      dc.RestoreDC ();
      }
   }


void TGrid::DrawCorner (TDC &dc, TRect &rc)
   {
   dc.FillRect (rc, TBrush (GetTitleColor (TitleBackground)));
   Frame3d (dc, rc, 1, GetTitleColor (TitleHilite), GetTitleColor (TitleShadow));
   }


void TGrid::PaintClient (TDC &dc, bool, TRect &clip)
   {
   // clip drawing ----------------------------------------------------------
   dc.SelectClipRgn (TRegion(clip));

   // No column to paint ? --------------------------------------------------
   if (firstColumn == 0)
      {
      dc.FillRect (clip, TBrush (emptyGrid));
      return;
      }

   if (layout == aGrid)
      {
      // draw lines ------------------------------------------------------------
      int y = 0;
      int i;

      for (i = 0; i < visibleLines; i++, y += lineHeight)
         {
         int pos = firstLine+i;
         if (maxLines < 0 && pos >= nLines)
            {
            if (!AppendNewLine (pos))
               {
               TRect rc (0, y, clientWidth, clientHeight);
               dc.FillRect (rc, TBrush (emptyGrid));
               vscroll->SetLRange (0, nLines-1, true);
               break;
               }
            SetNLines (pos+1);
            }

         TRect currect (0, y, clientWidth-1, y+lineHeight-1);
         if (!currect.Touches (clip))
            continue;

         // text ---------------------------------------------------------------
         for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
            {
            if( ctrl->Hidden )
              continue;                        

            // Select font -----------------------------------------------------
            if (ctrl->font)
               dc.SelectObject (*ctrl->font);
            else
               dc.SelectObject (*gridFont);

            // do not draw elements scrolled out horizontally ------------------
            if (ctrl->left + ctrl->width <= 0)
               continue;
            if (ctrl->left >= clientWidth)
               continue;

            // drawing area ----------------------------------------------------
            TRect rc (ctrl->left, y, ctrl->left+ctrl->width-1, y+lineHeight-1);
            if (rc.Touches (clip))
               {
               TControl *c = ctrl->control;
               const void *data;
               TGVisible v;
               if (maxLines > 0 && pos >= maxLines)
                  {
                  data = 0;
                  v = gEmpty;
                  }
               else
                  {
                  data = TransferItem (i+firstLine, c->GetId ());
                  v = VisibleCell (i+firstLine, ctrl->control->GetId());
                  }
               if (v == gEmpty)
                  dc.FillRect (rc, TBrush (emptyGrid));
               else
                  {
                  switch (ctrl->type)
                     {
                     case gTimeEdit:
                     case gDateEdit:
                     case gShortEdit:
                     case gLongEdit:
                     case gDoubleEdit:
                     case gEdit:
                        dynamic_cast<TGEdit *>(c)->DrawCell (dc, rc, data, ctrl->align);
                        break;

                     case gStatic:
                        dynamic_cast<TGStatic *>(c)->DrawCell (dc, rc, data, ctrl->align);
                        break;

                     case gCombo:
                        dynamic_cast<TGComboBox *>(c)->DrawCell (dc, rc, data);
                        break;

                     case gCheck:
                        dynamic_cast<TGCheckBox *>(c)->DrawCell (dc, rc, data, ctrl->align);
                        break;

                     case gButton:
                        dynamic_cast<TGButton *>(c)->DrawCell (dc, rc, data);
                        break;
                     }
                  }
               }

            dc.RestoreFont ();
            }
         }

      // draw grid -------------------------------------------------------------
      // this section was enhanced by Dean Crawford ----------------------------
      int vh = visibleLines*lineHeight;
      if (logicalWidth <= clientWidth)
         {
         TRect rc (logicalWidth, 0, clientWidth, clientHeight);
         dc.FillRect (rc, TBrush(emptyGrid));
         dc.SelectObject (TPen (GetSysColor (COLOR_WINDOWFRAME)));
         int x = logicalWidth;
         dc.MoveTo (x, 0);
         dc.LineTo (x, vh);
         dc.RestorePen ();
         }
      clip.Inflate (1);
      dc.SelectClipRgn (TRegion(clip));
      dc.SelectObject (TPen (lineColor));
      y = lineHeight-1;

      if (vh < clientHeight)
         {
         TRect rc (0, vh-1, clientWidth, clientHeight);
         dc.FillRect (rc, TBrush (emptyGrid));
         }

      for (i = 0; i <= visibleLines; i++, y += lineHeight)
         {
         dc.MoveTo (0, y);
         dc.LineTo (clientWidth, y);
         }

      for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
         {
         if ( ctrl->Hidden )
            continue;                       

         int x = ctrl->left-1;
         if (x >= 0 && x < clientWidth)
            {
            dc.MoveTo (x, 0);
            dc.LineTo (x, vh);
            }
         }

      dc.RestorePen ();
      }
   else
      {
      for (TGridColumn *c = firstColumn; c; c = c->next)
         {
         if (c->control == 0)
            continue;
         c->ShowControl ();
         }
      }
   }


void TGrid::RefreshLine (int line)
   {
   if (line < firstLine || line >= firstLine+visibleLines)
      return;

   int gridline = line - firstLine;
   int y = lineHeight * gridline;
   TRect clip (0, y, clientWidth, y+lineHeight);
   client->InvalidateRect (clip, false);
   if (!noupdate)
      client->UpdateWindow ();

   // update vbar -----------------------------------------------------------
   RefreshItem (line, 0);
   }


void TGrid::RefreshItem (int line, int id)
   {
   if (line < firstLine || line >= firstLine+visibleLines)
      return;

   int gridline = line - firstLine;

   if (gridline == activeLine)
      {
      TGridColumn *e = FindColumn (id);
      if (e)
         {
         void *data = const_cast<void *>(TransferItem (line, id));
         if (e->type == gCombo)
            dynamic_cast<TGComboBox *>(e->control)->SetData (data);
         else
            e->control->Transfer (data, tdSetData);
         }
      }

   int y = (layout == aGrid ? lineHeight * gridline : Attr.H);
   if (id != 0)   // update client area
      {
      TGridColumn *e = FindColumn (id);
      if (e)
         {
         TRect clip (e->left, y, e->left+e->width, y+lineHeight);
         if (clip.right < 0 || clip.left >= clientWidth)
            return;
         if (clip.left < 0)
            clip.left = 0;
         if (clip.right >= clientWidth-1)
            clip.right = clientWidth;
         if (!clip.IsEmpty ())
            {
            client->InvalidateRect (clip, true);
            if (!noupdate)
               client->UpdateWindow ();
            }
         }
      }
   else if (layout == aGrid && vbar && vbar->IsWindow())  // update left vertical bar
      {
      TRect clip (0, y, vbarWidth, y+lineHeight);
      vbar->InvalidateRect (clip, false);
      if (!noupdate)
         vbar->UpdateWindow ();
      }
   else if (layout == aForm)
      {
      for (TGridColumn *c = firstColumn; c; c = c->next)
         {
         if (c->control) 
            {
            int id = c->control->GetId ();
            int line = GetCurrentLine ();

            switch (VisibleCell (line, id)) 
               {
               case gControl:
               case gFlat:
                  c->control->ShowWindow (SW_SHOW);
                  break;

               case gLocked:
               case gEmpty:
                  c->control->ShowWindow (SW_HIDE);
                  break;
               }
            }
         }
      }
   }


void TGrid::Refresh ()
   {
   for (int i = 0; i < visibleLines; i++)
      RefreshLine (firstLine+i);
   }


// ----------------------------------------------------------------------- //
// Changing grid size                                                      //
// ----------------------------------------------------------------------- //
void TGrid::EvWindowPosChanging (WINDOWPOS far &wpos)
   {
   ChangeWindowPos (wpos.cx, wpos.cy, wpos.flags);
   TControl::EvWindowPosChanging (wpos);
   }


// Utility functions added by Ren Mayer
bool TGrid::IsHScrollbarNeeded ()
   {
   if (layout == aGrid)
      {
      bool willNeedHScroll = false;
      if (hscroll)
         {
         willNeedHScroll = true;
         if (hideHScroll && logicalWidth <= clientWidth)
            willNeedHScroll = false;
         }
      return willNeedHScroll;
      }
   else
      return false;
   }


bool TGrid::IsVScrollbarNeeded ()
   {
   bool willNeedVScroll = false;
   if (vscroll)
      {
      willNeedVScroll = true;
      if (hideVScroll && lineHeight && maxLines > 0 && (clientHeight / lineHeight) >= maxLines)
         willNeedVScroll = false;
      }
   return willNeedVScroll;
   }
// end of update by Ren Mayer


void TGrid::ChangeWindowPos (int w, int h, uint flags)
   {
   if (layout == aForm)
      {
      Attr.W = w;
      Attr.H = h;
      }

   // if there are no columns in TGrid, don't do anything yet.
   if (firstColumn == 0 || w == 0 || h == 0)
      return;

   int oldFirstLine = firstLine;
   int oldVisibleLines = visibleLines;

   bool Move = ((flags & SWP_NOMOVE) == 0 ? true : false);
   bool Size = ((flags & SWP_NOSIZE) == 0 ? true : false);

   if (!Move && !Size)
      return;

   if (layout == aGrid)
      readLayout = true;

   // get size of elements
   titleHeight = (layout == aGrid ? title->CalcHeight() : 0);
   if (vbar && vbarWidth == 0)
      vbarWidth = vbar->CalcWidth();
   clientWidth = w - vbarWidth - 2;
   clientHeight = h - titleHeight - 2;

   // check if scroll bars are visible (Update by Ren Mayer)
   bool willNeedVScroll = IsVScrollbarNeeded ();
   bool willNeedHScroll = IsHScrollbarNeeded ();
   if (willNeedVScroll)
      {
      clientWidth -= cxvscroll;
      if (IsHScrollbarNeeded ())
         clientHeight -= cyhscroll;
      }
   else if (willNeedHScroll)
      {
      clientHeight -= cyhscroll;
      if (IsVScrollbarNeeded ())
         clientWidth -= cxvscroll;
      }
   // end of update by Ren Mayer

   if (layout == aGrid && lastColumn && firstColumn && lastColumn->left+lastColumn->width < clientWidth && firstColumn->left <= 0)
      MoveHPos (lastColumn->left+lastColumn->width - clientWidth);

   if (Size)
      {
      if (hscroll && hscroll->IsWindow())
         {
         if (logicalWidth <= clientWidth)
            {
            if (disableHScroll)
               hscroll->EnableWindow (false);
            if (hideHScroll)
               {
               if (!hscrollHidden)
                  {
                  hscroll->ShowWindow (SW_HIDE);
                  hscrollHidden = true;
                  HScrollHidden (true);  // notification to virtual function (R. Mayer)
                  }
               }
            }
         else
            {
            if (hscrollHidden)
               {
               hscroll->ShowWindow (SW_SHOW);
               hscrollHidden = false;
               HScrollHidden (false);  // notification to virtual function (R. Mayer)
               }
            hscroll->EnableWindow (true);
            }
         }

      }

   if (Move || Size)
      {
      uint flag = SWP_NOZORDER;

      if (!Size)
         flag |= SWP_NOSIZE;
      else if (!Move)
         flag |= SWP_NOMOVE;

      // adjust to integral line height -------------------------------------
      if (h && lineHeight)
         {
         int old = visibleLines;
         int curline = firstLine+activeLine;
         SetVisibleLines (clientHeight / lineHeight);
         if (maxLines > 0 && visibleLines >= maxLines)
            {
            if (vscroll && vscroll->IsWindow())
               {
               if (!vscrollHidden)
                  {
                  if (disableVScroll)
                     vscroll->EnableWindow (false);
                  if (hideVScroll)
                     {
                     vscroll->ShowWindow (SW_HIDE);
                     vscrollHidden = true;
                     VScrollHidden (true);  // notification to virtual function (R. Mayer)
                     }
                  }
               }
            if (vertShrink)
               SetVisibleLines (maxLines);
            }
         else if (vscroll && vscroll->IsWindow())
            {
            if (vscrollHidden)
               {
               vscroll->ShowWindow (SW_SHOW);
               vscrollHidden = false;
               VScrollHidden (false);  // notification to virtual function (R. Mayer)
               }
            vscroll->EnableWindow (true);
            }

         if (visibleLines <= 0)
            SetVisibleLines (1);

         if (visibleLines != old)
            {
            Go (0);
            if (curline > 0)
               Go (curline);
            }

         if (layout == aGrid)
            clientHeight = visibleLines * lineHeight;
         }

      // adjust related controls -----------------------------------------------
      if (layout == aGrid && horzShrink && clientWidth > logicalWidth)
         clientWidth = logicalWidth;

      flag &= (~SWP_NOMOVE);   

      client->SetWindowPos (0, vbarWidth, titleHeight, clientWidth-1, clientHeight, flag);

      if (vbar && vbar->IsWindow())
         vbar->SetWindowPos (0, 0, titleHeight, vbarWidth, layout == aGrid ? clientHeight : h - 2, flag);

      if (title && title->IsWindow())
         {
         if (titleHeight > 0)
            title->SetWindowPos (0, vbarWidth, 0, clientWidth-1, titleHeight, flag);
         else
            title->SetWindowPos (0, 0, 0, 0, 0, flag|SWP_HIDEWINDOW);
         }
      if (vscroll && vscroll->IsWindow() && !vscrollHidden)
         vscroll->SetWindowPos (0, vbarWidth+clientWidth-1, titleHeight, cxvscroll, clientHeight, flag);

      if (hscroll && hscroll->IsWindow() && !hscrollHidden)
         {
         hscroll->SetWindowPos (0, vbarWidth, titleHeight+clientHeight, clientWidth-1, cyhscroll, flag);
         SetHScrollRange ();
         }

      if (layout == aForm)
         {
         SetLineHeight (Attr.H);

         // Create statics for labels
         TScreenDC sdc;
         for (TGridColumn *c = firstColumn; c; c = c->next)
            {
            if (c->control)
               {
               if (c->label == 0)
                  {
                  TRect rc (0,0,MAXINT,MAXINT);
                  sdc.DrawText (c->title.c_str(), -1, rc, DT_LEFT|DT_SINGLELINE|DT_CALCRECT);
                  c->label = new TGStatic (client, -1, c->title.c_str(), 0, 0, rc.Width(), c->control->Attr.H, 0, 0);
                  c->label->Create ();
                  }
               else
                  c->label->ShowWindow (true);
               }
            }

         if (readLayout == true)
            {
            readLayout = false;

            struct SizeRect
               {
               int x;
               int y;
               int w;
               int h;
               SizeRect () : x(-1), y(-1), w(-1), h(-1) {}
               };

            SizeRect *controls = new SizeRect [nControls];
            SizeRect *labels   = new SizeRect [nControls];

            // read layout from file; if no layout info found, default layout
            TProfile prof (section.c_str(), iniFile.c_str());
            char buffer[256];
            char key[16];

            int i = 0;
            TGridColumn *c;
            for (c = firstColumn; c; c = c->next, ++i)
               {
               if (c->control == 0)
                  continue;

               sprintf (key, "%d", c->control->GetId ());
               prof.GetString (key, buffer, 256, "");

               if (*buffer == 0)
                  continue;

               char *b = strtok (buffer, ",");
               if (b)
                  controls[i].x = atoi (b);
               b = strtok (0, ",");
               if (b)
                  controls[i].y = atoi (b);
               b = strtok (0, ",");
               if (b)
                  controls[i].w = atoi (b);
               b = strtok (0, ",");
               if (b)
                  controls[i].h = atoi (b);
               b = strtok (buffer, ",");
               if (b)
                  labels[i].x = atoi (b);
               b = strtok (0, ",");
               if (b)
                  labels[i].y = atoi (b);
               b = strtok (0, ",");
               if (b)
                  labels[i].w = atoi (b);
               b = strtok (0, ",");
               if (b)
                  labels[i].h = atoi (b);
               else
                  labels[i].h = controls[i].h;
               }

            // find maximum width of controls and labels
            i = 0;
            c = firstColumn;
            int x0 = 3;
            do
               {
               int lw = 0;
               int cw = 0;
               int top = 0;
               int bottom = client->Attr.H;

               TGridColumn *c0 = c;
               int start = i;

               for (; c; c = c->next, ++i)
                  {
                  if (c->control)
                     {
                     if (controls[i].h < 0)
                        controls[i].h = c->control->Attr.H;

                     if (top+controls[i].h >= bottom)
                        break;

                     if (controls[i].w < 0)
                        {
                        controls[i].w = c->control->Attr.W;
                        if (controls[i].w > cw)
                           cw = controls[i].w;
                        }

                     if (labels[i].w < 0)
                        {
                        labels[i].w = c->label->Attr.W;
                        if (labels[i].w > lw)
                           lw = labels[i].w;
                        }

                     top += controls[i].h + 2;
                     }
                  }

               int cx = x0 + lw + 3;
               int j = start;
               top = 0;

               for (TGridColumn *col = c0; j < i && col; j++, col = col->next)
                  {
                  if (controls[j].x < 0 || controls[j].y < 0)
                     {
                     controls[j].x = cx;
                     controls[j].y = top;
                     top += controls[j].h+2;
                     }

                  col->control->SetWindowPos (0, controls[j].x, controls[j].y, controls[j].w, controls[j].h, SWP_NOZORDER);

                  if (labels[j].h < 0)
                     labels[j].h = controls[j].h;
                  if (labels[j].x < 0 || labels[j].y < 0)
                     {
                     labels[j].x = cx - labels[j].w - 3;
                     labels[j].y = controls[j].y;
                     }
                  col->label->SetWindowPos (0, labels[j].x, labels[j].y, labels[j].w, labels[j].h, SWP_NOZORDER);
                  }

               x0 += lw + cw + 8;
               } while (c);

            delete [] controls;
            delete [] labels;
            }
         }
      else
         {
         // hide labels
         for (TGridColumn *c = firstColumn; c; c = c->next)
            {
            if (c->label)
               c->label->ShowWindow (false);
            }
         }
      }

   if (firstLine != oldFirstLine || visibleLines != oldVisibleLines)
      ScrollData (firstLine, firstLine+visibleLines-1);
   }


bool TGrid::CalcPos ()
   {
   // If the grid has been created but the border controls aren't created yet,
   // we have to create them ourselves here                                 //
   if (IsWindow())
      {
      if (!client->IsWindow ())
         client->Create ();
      if (vbar && !vbar->IsWindow ())
         vbar->Create ();
      if (title && !title->IsWindow ())
         title->Create ();
      if (hscroll && !hscroll->IsWindow ())
         hscroll->Create ();
      if (vscroll && !vscroll->IsWindow ())
         vscroll->Create ();
      }

   // abort if:                                                             //
   //    - TGrid's HWindow is not valid                                     //
   // or - title's HWindow is not valid                                     //
   // or - vertical bar'sHWindow is not valid                               //
   // or - CalcPos has been successfully executed previously                //
   // Note: we need to check for multiple successful executions of CalcPos  //
   //       because CalcPos is called from Create(), which can be (and is)  //
   //       called more than once.                                          //
   if (calcPos || !IsWindow())
      return false;

   TGridColumn *ctrl;
	for (ctrl = firstColumn; ctrl; ctrl = ctrl->next)
		{
      if (!ctrl->control->IsWindow ())
         return false;
      }

   calcPos = true;

   // create the grid's TFont -----------------------------------------------
   if (gridFont == 0)
      {
      SetFont ();
      if (titleLogFont)
         SetTitleFont ();
      }

   // line height &  column width -------------------------------------------
   logicalWidth = 0;

   if (layout == aGrid)
      {
      int wasHidden = 0;                      // Charles R. Cordell
      for (ctrl = firstColumn; ctrl; ctrl = ctrl->next)
         {
         // section added by Charles R. Cordell
         if ( ctrl->Hidden == -1)
            {
            ctrl->Hidden = 0;   //unhide temp.
            wasHidden = true;   // remember
            }
         // end of contribution

         int h = ctrl->CtrlHeight ();
         if (h >= lineHeight)
            SetLineHeight (h);

         int w = ctrl->CtrlWidth();
         ctrl->left = logicalWidth;
         ctrl->width = w+1;

         if (ctrl->type == gCheck)
            ctrl->width += 2;

         if (ctrl->left + ctrl->width > logicalWidth)
            logicalWidth = ctrl->left + ctrl->width;

         // make the control a child of TGrid to clip it inside the grid -------
         ctrl->control->SetParent (client);
         ctrl->HideControl ();

         // section added by Charles R. Cordell
         if ( wasHidden )
            {
            wasHidden = 0;
            for (TGridColumn *col = ctrl->next; col; col = col->next)
               col->left -= ctrl->width;
            logicalWidth -= ctrl->width;
            ctrl->Hidden = ctrl->width;
            ctrl->width = 0;
            }
         // end of contribution
         }

      if (vscroll)
         {
         if (maxLines < 0)
            vscroll->SetLRange (0, (nLines*10)/9+1, true);
         else
            vscroll->SetLRange (0, maxLines-1, true);
         vscroll->SetLPosition (0);
         }

      if (hscroll)
         {
         hscroll->LineMagnitude = LOWORD(GetDialogBaseUnits());// approx. 1 char
         hscroll->SetPosition (0);
         }

      // SetWindowPos will make sure everything is in place --------------------
      SetWindowPos (0, Attr.X, Attr.Y, Attr.W, Attr.H, SWP_NOZORDER);  
      if (HasFocus ())
         GotoFirstColumn ();

      // Line selection --------------------------------------------------------
      if (lineSelection == Single || lineSelection == SingleAutoSelect)
         selectedLine = 0;
      }
   else
      {
      for (ctrl = firstColumn; ctrl; ctrl = ctrl->next)
         ctrl->control->SetParent (client);

      SetLineHeight (Attr.H);

      if (vscroll)
         {
         if (maxLines < 0)
            vscroll->SetLRange (0, (nLines*10)/9+1, true);
         else
            vscroll->SetLRange (0, maxLines-1, true);
         vscroll->SetLPosition (0);
         }

      // SetWindowPos will make sure everything is in place --------------------
      SetWindowPos (0, Attr.X, Attr.Y, Attr.W, Attr.H, SWP_NOZORDER);
      if (HasFocus ())
         GotoFirstColumn ();

      // Line selection --------------------------------------------------------
      if (lineSelection == Single || lineSelection == SingleAutoSelect)
         selectedLine = 0;
      }

   return true;
   }


void TGrid::SetHScrollRange ()
   {
   if (hscroll && hscroll->IsWindow())
      {
      ::SetScrollRange (*hscroll, SB_CTL, 0, logicalWidth - clientWidth -1, true);
      hscroll->PageMagnitude = clientWidth;
      }
   }

// ----------------------------------------------------------------------- //
// Data messages                                                           //
// ----------------------------------------------------------------------- //
bool TGrid::CanClose ()
	{
	int line = GetCurrentLine ();
   if (current && !CanKillFocus (current))
      return false;

   return ValidateLine (line);
	}


// ----------------------------------------------------------------------- //
// Mouse functions                                                         //
// ----------------------------------------------------------------------- //
bool TGrid::SetCursorShape (TPoint &p) // p is the mouse cursor position in client coordinates inside the 'client' control
   {
   if (cursorType == DefCursor)
      {
      int x = p.x+1;

      for (TGridColumn *e = firstColumn; e; e = e->next)
         {
         int right = e->left+e->width;
         if (x >= e->left && x < right)
            {
            TControl *c = e->control;
            switch (e->type)
               {
               default:
                  client->SetCursor (0, IDC_ARROW);
                  break;
               case gCombo:
                  {
                  int w = LOWORD(GetDialogBaseUnits());
                  if (x < right-w-w && (c->Attr.Style & CBS_DROPDOWNLIST) != CBS_DROPDOWNLIST)
                     client->SetCursor (0, IDC_IBEAM);
                  else
                     client->SetCursor (0, IDC_ARROW);
                  break;
                  }
               case gDateEdit:
               case gTimeEdit:
               case gShortEdit:
               case gLongEdit:
               case gDoubleEdit:
               case gEdit:
                  client->SetCursor (0, IDC_IBEAM);
                  break;
               }
            client->ClientToScreen (p);
            SetCursorPos (p.x, p.y);
            return true;
            }
         }
      }
   else
      {
      // if cursor is on the call with focus, use standard behaviour
      if (current && current->control && current->control->IsWindowVisible ())
         {
         TRect rc = current->control->GetWindowRect ();
         client->ClientToScreen (p);
         if (rc.Contains (p))
            return false;
         }
      client->SetCursor (0, IDC_ARROW);
      return true;
      }

   return false;
   }


void TGrid::ProcessMouse (uint modKeys, TPoint &point, uint message, bool sendToControl) 
   {
   if (echo)
      {
      echo = false;
      return;
      }

   TPoint p (point);
   MapWindowPoints (*client, &p, 1); // map coordinates to client area
   int line = (layout == aGrid ? p.y / lineHeight : firstLine);
   TGridColumn *c = FindColumn (p);
   int id = (c && c->control) ? c->control->GetId() : -1;

   if (vbar && p.x < 0 && p.y >= 0)  // mouse was pressed in vbar
      {
      TPoint p2(point);
      MapWindowPoints (*vbar, &p2, 1);
      if (MouseInVBar (modKeys, p2, message, line+firstLine))
         return;
      }
   else if (title && p.x >= 0 && p.y < 0) // mouse was pressed in title
      {
      TPoint p2(point);
      MapWindowPoints (*title, &p2, 1);
      if (MouseInTitle (modKeys, p2, message, id))
         return;
      }
   else if (p.x >= 0 && p.y >= 0) // mouse was pressed in client area
      {
      if (MouseInClient (modKeys, p, message, line, id))
         return;
      }

   if (line >= 0 && id >= 0 && message == WM_LBUTTONDOWN && layout == aGrid)
      {
      int old = activeLine;
      TGridColumn *e = FindColumn (p);
      if (CanKillFocus (current))
         {
         if (line != activeLine)
            {
            int n = firstLine+line;
            Go (n >= 0 ? n : 0, false);
            }

         if (e)
            {
            TControl *c = e->control;
            int ln = GetCurrentLine ();
            int id = c->GetId ();
            TGVisible v = VisibleCell (ln, id);
            if (e->type != gStatic && v != gEmpty && v != gLocked)
               {
               bool tabStop = ( (c->Attr.Style & WS_TABSTOP) ? true : false);
               if (dynamic_cast<TGCheckBox *>(c) && !tabStop)
                  {
                  const void *d = TransferItem (ln, id);
                  if (d)
                     {
                     bool val = *static_cast<const bool *>(d);
                     val ^= 1;
                     if (DataChanging (ln, id, static_cast<void *>(&val)))
                        {
                        modified = true;
                        TransferItem (ln, id, static_cast<void *>(&val));
                        DataChanged (ln, id, static_cast<void *>(&val));
                        ClearModify (c);
                        }
                     RefreshItem (ln, id);
                     }
                  }
               else if (dynamic_cast<TGButton *>(c) && !tabStop)
                  {
                  // who has focus ?
                  HWND hwnd = GetFocus ();
                  if (sendToControl)
                     {
                     echo = true;
                     client->MapWindowPoints (*c, &p, 1);
                     c->SendMessage (message, modKeys, MAKELONG(p.x,p.y));
                     }
                  // send focus back
                  TGridColumn *col;
                  for (col = firstColumn; col; col = col->next)
                     {
                     if (col->control && col->control->HWindow == hwnd)
                        {
                        col->SetFocus ();
                        break;
                        }
                     }
                  if (col == 0)
                     ::SetFocus (hwnd);
                  }
               else
                  {
                  if (e != current || line != old)
                     e->SetFocus ();
                  if (sendToControl)
                     {
                     echo = true;
                     client->MapWindowPoints (*c, &p, 1);
                     c->SendMessage (message, modKeys, MAKELONG(p.x,p.y));
                     }

                  if (e->type == gCombo)
                     dynamic_cast<TGComboBox *>(e->control)->ShowList ();
                  }
               }
            }

         if (StaticGrid() && lineSelection != SingleAutoSelect && sendToControl)
            SelectLine (GetCurrentLine (), modKeys);   // enhanced by Dean Crawford
         }
      }

   switch (message)
      {
      case WM_LBUTTONDOWN:
         LButtonDown (modKeys, point);
         break;
      case WM_LBUTTONUP:
         LButtonUp (modKeys, point);
         break;
      case WM_LBUTTONDBLCLK:
         LButtonDblClk (modKeys, point);
         break;
      case WM_RBUTTONDOWN:
         RButtonDown (modKeys, point);
         break;
      case WM_RBUTTONUP:
         RButtonUp (modKeys, point);
         break;
      case WM_RBUTTONDBLCLK:
         RButtonDblClk (modKeys, point);
         break;
      }
   }


// ----------------------------------------------------------------------- //
// Borders and attributes                                                  //
// ----------------------------------------------------------------------- //
void TGrid::ChangeTitle (int id, const char *newTitle)
   {
   TGridColumn *e = FindColumn (id);
   if (e)
      e->SetTitle (newTitle);
   }


const char *TGrid::GetTitle (int id) const
   {
   TGridColumn *e = FindColumn (id);
   if (e)
      return e->title.c_str();
   else
      return 0;
   }


void TGrid::SetLineMarker (TGLineMarker lm)
   {
   if (lm == lineMarker)
      return;

   // remove old markers ----------------------------------------------------
   if (vbar)
      vbar->ShowText (activeLine);

   lineMarker = lm;

   if (vbar)
      vbar->ShowMarker (activeLine);
   }


void TGrid::SetVBarText (int cwidth)
   {
   if (vbar)
      {
      vbar->SetUserText (cwidth);
      vbarWidth = vbar->CalcWidth();
      }
   }


void TGrid::SetVBarUserText (bool on)
   {
   if (vbar && on == vbar->lineNumber)
      {
      vbar->lineNumber = !vbar->lineNumber;
      vbar->Invalidate ();
      }
   }


void TGrid::SetVBarWidth (int pixels)
   {
   if (vbar)
      vbarWidth = pixels;
   }


void TGrid::SetFont (int id, const char *facename, int height, int weight, bool italic)
   {
   if (!IsWindow())
      {
      TGridColumn *e = FindColumn (id);
      if (e)
         e->SetFont (facename, height, weight, italic);
      }
   }


void TGrid::SetFont (const char *facename, int height, int weight, bool italic)
   {
   if (!IsWindow())
      {
      strcpy (gridLogFont->lfFaceName, facename);
      if (height)
         gridLogFont->lfHeight = height;
      gridLogFont->lfWeight = weight;
      gridLogFont->lfItalic = BYTE(italic);
      }
   }


void TGrid::SetTitleFont (const char *facename, int height, int weight, bool italic)
   {
   if (!IsWindow ())
      {
      if (titleLogFont == 0)
         {
         titleLogFont = new LOGFONT;
         memcpy (titleLogFont, gridLogFont, sizeof(LOGFONT));
         }
      strcpy (titleLogFont->lfFaceName, facename);
      if (height)
         titleLogFont->lfHeight = height;
      titleLogFont->lfWeight = weight;
      titleLogFont->lfItalic = BYTE(italic);
      }
   }


void TGrid::SetFont ()
   {
   if (gridFont)
      delete gridFont;
   gridFont = new TFont (gridLogFont);

   SetWindowFont (*gridFont, true);

   for (TGridColumn *ctrl = firstColumn; ctrl; ctrl = ctrl->next)
      ctrl->SetFont ();
   }


void TGrid::SetTitleFont ()
   {
   if (titleFont)
      delete titleFont;
   if (titleLogFont == 0)
      {
      titleLogFont = new LOGFONT;
      memcpy (titleLogFont, gridLogFont, sizeof(LOGFONT));
      }
   titleFont = new TFont (titleLogFont);
   }


TColor TGrid::GetTitleColor (TGTitleColor part) const
   {
   switch (part)
      {
      case TitleText:
         return GetSysColor (COLOR_BTNTEXT);
      case TitleBackground:
         return GetSysColor (COLOR_BTNFACE);
      case TitleHilite:
         return GetSysColor (COLOR_BTNHIGHLIGHT);
      case TitleShadow:
         return GetSysColor (COLOR_BTNSHADOW);
      case TitleFrame:
         return GetSysColor (COLOR_WINDOWFRAME);
      default:
         return TColor::Black;
      }
   }


TFont *TGrid::GetGridFont (int id) const
   {
   TGridColumn *e = FindColumn (id);
   if (e->font)
      return e->font;
   else
      return gridFont;
   }



void TGrid::SetVisibleCells (TGVisible v)
   {
   TGVisible old = visible;
   visible = v;
   if (old != v)
      Refresh ();
   }


void TGrid::HorzShrink ()
   {
   if (!IsWindow() || logicalWidth >= clientWidth)
      return;

   int delta = clientWidth - logicalWidth;
   SetWindowPos (0, 0, 0, Attr.W-delta, Attr.H, SWP_NOZORDER|SWP_NOMOVE);
   }


void TGrid::SetTitle (int id, const char *text)
   {
   TGridColumn *c = FindColumn (id);
   if (c)
      c->SetTitle (text);
   }


int TGrid::GetColumnWidth (int id) const
   {
   TGridColumn *c = FindColumn (id);
   if (c)
      return c->width;
   else
      return 0;
   }


int TGrid::GetTotalWidth  () const
   {
   bool vs = (vscroll && vscroll->IsWindow () && !hscrollHidden) ? true : false;
   return 2+vbarWidth+clientWidth+(vs?cxvscroll:0);
   }


int TGrid::GetTotalHeight () const
   {
   bool hs = (hscroll && hscroll->IsWindow () && !vscrollHidden) ? true : false;
   return 2+titleHeight+clientHeight+(hs?cyhscroll:0);
   }



// controlling layout
void TGrid::SetLayout (TGLayout l)
   {
   layout = l;
   calcPos = false;
   lineHeight = 0;
   if (title)
      {
      titleHeight = (layout == aGrid ? title->CalcHeight() : 0);
      title->SetWindowPos (0, title->Attr.X, title->Attr.Y, title->Attr.W, titleHeight, SWP_NOZORDER|SWP_NOMOVE);
      title->ShowWindow (l == aGrid ? SW_SHOW : SW_HIDE);
      }

   if (layout == aGrid)
      {
      for (TGridColumn *c = firstColumn; c; c = c->next)
         {
         if (c->label)
            c->label->ShowWindow (false);
         }
      }

   CalcPos ();
   ChangeWindowPos (Attr.W, Attr.H, SWP_NOMOVE);

   if (vscroll && layout == aForm)
      vscroll->SetPosition (GetCurrentLine());

   Invalidate ();
   UpdateWindow ();
   }

// EOF
