/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// grid.h                                                                  //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.50 - 25 nov 1995                       //
// TGrid - Copyright  1994,1995 H. Picot, All Rights Reserved.            //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#ifndef GRID_H
#define GRID_H

#define HRESIZE 500

#ifndef __OWL_SCROLLBAR_H
#include "owl\scrollba.h"
#endif
 
class _GRIDCLASS TGDialog : public TDialog
   {
   protected:
      TColor bkg;
      TBrush *brush;
      HBRUSH EvCtlColor(HDC hDC, HWND hWndChild, uint ctlType);
      virtual HBRUSH CtlColors (HDC hDC); 

   public:
      TGDialog (TWindow *p, TResId resId, TColor bkg, TModule *module = 0);
      TGDialog (TWindow *p, TResId resId, TModule *module = 0); 
      ~TGDialog ();
      virtual void   SetColor  (TColor bkg);
      virtual TColor BkgColor  () { return bkg; }
      virtual HBRUSH BkgBrush  () { return *brush; }

   DECLARE_RESPONSE_TABLE (TGDialog);
   };

class _GRIDCLASS TGCheckBox;
class _GRIDCLASS TGComboBox;
class _GRIDCLASS TGEdit;
class _GRIDCLASS TGStatic;
class _GRIDCLASS TGButton;

class _GRIDCLASS TGridClient;
class _GRIDCLASS TGridColumn;
class _GRIDCLASS TGridVScroll;
class _GRIDCLASS TGridHScroll;
class _GRIDCLASS TGridTitle;
class _GRIDCLASS TGridVerticalBar;

void _GRIDFUNCTION Frame3d (TDC &dc, TRect &rc, int w, TColor topleft, TColor bottomright);
void _GRIDFUNCTION Frame3d (TDC &dc, int x0, int y0, int x1, int y1, int w, TColor topleft, TColor bottomright);

class _GRIDCLASS TGrid;
enum TGAlign      { Left =DT_LEFT, Center =DT_CENTER, Right =DT_RIGHT };
// enumerations ----------------------------------------------------------
enum TGAttributes { gNoBorder         = 0x0000,
                    gVScroll          = 0x0001,
                    gHScroll          = 0x0002,
                    gTitle            = 0x0004,
                    gVBar             = 0x0008,
                    gNoScroll         = 0x000c,
                    gAllBorders       = 0x000f,
                    gHideHScroll      = 0x0010,
                    gHideVScroll      = 0x0020,
                    gHideScroll       = 0x0030,
                    gDisableHScroll   = 0x0040,
                    gDisableVScroll   = 0x0080,
                    gDisableScroll    = 0x00c0,
                    gCenterTitle      = 0x0104,
                    gLineSwapOn       = 0x0200,
                    gSizeColumn       = 0x0400,
                    gHorzShrink       = 0x0800,
                    gSingleSelection  = 0x1000,
                    gMultipleSelection= 0x2000,
                    gAutoSelection    = 0x3000,
                    gArrowCursor      = 0x4000,
                    gDefCursor        = 0x0000,
                    gVertShrink       = 0x8000
                  };
enum TGLayout     { aGrid, aForm };        // doc
enum TGLineMarker { ArrowOnly, ArrowAndText, ReverseColor };
enum TGLineSelect { None, Single, Multiple, SingleAutoSelect };
enum TGTitleColor { TitleText, TitleBackground, TitleHilite, TitleShadow, TitleFrame };
enum TGType       { gEdit, gStatic, gCombo, gButton, gCheck, gDateEdit, gTimeEdit, gDoubleEdit, gLongEdit, gShortEdit };
enum TGVisible    { gControl, gFlat, gLocked, gEmpty };
enum TGCursorType { DefCursor, ArrowCursor };
enum TGEnterKey   { EnterNextColumn, NextLine, ExitDialog, DoNothing };
enum TGTabKey     { TabNextColumn, AutoNextLine, NextControl };

class _GRIDCLASS TGridClient : public TControl
   {
   protected:
      TGrid *grid;
      void Paint (TDC &dc, bool erase, TRect &clip);
      HBRUSH EvCtlColor (HDC hDC, HWND hWndChild, uint ctlType);
      bool EvEraseBkgnd    (HDC hwnd) { return true; }
      void EvKillFocus     (HWND hWndGetFocus);
      void EvLButtonDown   (uint modKeys, TPoint &point);
      void EvLButtonUp     (uint modKeys, TPoint &point);
      void EvLButtonDblClk (uint modKeys, TPoint &point);
      void EvRButtonDown   (uint modKeys, TPoint &point);
      void EvRButtonUp     (uint modKeys, TPoint &point);
      void EvRButtonDblClk (uint modKeys, TPoint &point);
      bool EvSetCursor     (HWND hwnd, uint hitTest, uint mouseMsg);
      void EvSetFocus      (HWND hwndLooseFocus);
      LRESULT WindowProc   (uint msg, WPARAM wParam, LPARAM lParam);

   public:
      char * GetClassName ();
      TGridClient (TGrid *g);
      bool Create ();

   DECLARE_RESPONSE_TABLE (TGridClient);
   };


class _GRIDCLASS TGridColumn
   {
   public:
      TGrid *grid;
      TControl *control;
      TGridColumn *next;
      TGridColumn *previous;
      int left;
      int width;
      int Hidden;                  // Charles R. Cordell
      TGAlign align;
      TGType  type;
      string title;
      TFont *font;
      TGStatic *label;              // doc

      TGridColumn (TControl *c, TGridColumn *prev, TGrid *grid, const char *t);
     ~TGridColumn ();
      bool CanKillFocus ();
      int CtrlWidth () const;
      int CtrlHeight () const;
      void HideControl ();
      bool IsColumnHidden () const { return Hidden > 0 ? true : false; }
      bool SetFocus ();
      void SetFont ();
      void SetFont (const char *facename, int size, int weight, bool italic);
      void SetTitle (const char *newtitle);
      void ShowControl ();
      char * GetClassName ();
   };

class _GRIDCLASS TGridVScroll : public TScrollBar
   {
   protected:
      void EvKeyDown (uint key, uint repeat, uint flags);
      uint minRange;
      uint maxRange;
      int factor;

   public:
      TGrid *grid;

      TGridVScroll (TGrid *g);

      void SetLRange (uint min, uint max, bool draw);
      void SetLPosition (uint pos);

      void SBBottom ();
      void SBLineDown ();
      void SBLineUp ();
      void SBPageDown ();
      void SBPageUp ();
      void SBThumbPosition (int pos);
      void SBThumbTrack (int pos);
      void SBTop ();

      DECLARE_RESPONSE_TABLE(TGridVScroll);
   };

class _GRIDCLASS TGridHScroll : public TScrollBar
   {
   public:
      TGrid *grid;

      TGridHScroll (TGrid *g);

      void SBBottom ();
      void SBLineDown ();
      void SBLineUp ();
      void SBPageDown ();
      void SBPageUp ();
      void SBThumbPosition (int pos);
      void SBThumbTrack (int pos);
      void SBTop ();
   };

class _GRIDCLASS TGridTitle : public TControl
   {
   friend TGrid;

   private:
      bool capture;
      bool showBack;
      bool visible;
      int xorg;
      bool MoveMouseCursor ();
      TGridColumn *column;

   protected:
      TGrid *grid;

      bool  EvSetCursor (HWND hwnd, uint hitTest, uint mouseMsg);
      void  EvLButtonDown (uint modKeys, TPoint &p);
      void  EvLButtonUp (uint modKeys, TPoint &p);
      void  EvLButtonDblClk (uint modKeys, TPoint &p);
      void  EvRButtonDblClk (uint modKeys, TPoint &p);
      void  EvRButtonDown (uint modKeys, TPoint &p);
      void  EvRButtonUp (uint modKeys, TPoint &p);
      void  EvMouseMove (uint modKeys, TPoint &p);
      void  DrawCell (TDC &dc, int x, int w, const char *p, TGAlign a);
      char *GetClassName ();
      void  GetWindowClass (WNDCLASS &wc);
      int   CalcHeight ();
      void  Paint (TDC &dc, bool erase, TRect &);
      LRESULT WindowProc   (uint msg, WPARAM wParam, LPARAM lParam);

   public:
      TGridTitle (TGrid *g, bool visible);
      bool Create ();

      DECLARE_RESPONSE_TABLE (TGridTitle);
   };

class _GRIDCLASS TGridVerticalBar : public TControl
   {
   friend TGrid;

   protected:
      TGrid *grid;
      bool lineNumber;
      int charWidth;

      int CalcWidth ();
      void DrawCell (TDC &dc, TRect &rc, int line);
      void EvLButtonDblClk (uint modKeys, TPoint &point);
      void EvLButtonDown (uint modKeys, TPoint &point);
      void EvLButtonUp (uint modKeys, TPoint &point);
      void EvRButtonDblClk (uint modKeys, TPoint &point);
      void EvRButtonDown (uint modKeys, TPoint &point);
      void EvRButtonUp (uint modKeys, TPoint &point);
      TRect FillRect (TDC &dc, int line, bool erase) const;
      const char *Info (int line) const;
      void Paint (TDC &dc, bool erase, TRect &);
      void ShowMarker (int line) const;
      void ShowText (int line) const;

   public:
      TGridVerticalBar (TGrid *g);
      void SetUserText (int cwidth);
      bool Create ();
      char * GetClassName ();

      DECLARE_RESPONSE_TABLE (TGridVerticalBar);
   };


#define defAttrib (gAllBorders  | \
                   gHideScroll  | \
                   gCenterTitle | \
                   gLineSwapOn  | \
                   gHorzShrink  | \
                   gVertShrink)
#define MK_TGRID 0x20

class _GRIDCLASS TGrid : public TControl
   {
   public:
      // constructors and destructor -------------------------------------------
      TGrid (TWindow *p, int id, int x, int y, int w, int h, TGEnterKey ek =NextLine, TGTabKey =TabNextColumn,
             TGLineMarker lm =ReverseColor, int max =-1, int attrib =defAttrib, TGLayout l =aGrid, TModule *module = 0);        // doc
      TGrid (TWindow *p, int id, TGEnterKey ek =NextLine, TGTabKey =TabNextColumn, TGLineMarker lm =ReverseColor, int max =-1,
             int attrib =defAttrib, TGLayout l =aGrid, TModule *module = 0);                                                  // doc
     ~TGrid ();

      // public methods --------------------------------------------------------
      TGridColumn *FindColumn (HWND hwnd)      const;
      TGridColumn *FindColumn (int id)         const;
      TGridColumn *FindColumn (TControl *c)    const;
      TGridColumn *FindColumn (TPoint &p)      const;

      int        CalcVBarWidth ()         const { return vbar->CalcWidth(); }
      int        CountSelectedLines()     const;
      bool       Create ();
      int        GetActiveLine ()         const { return activeLine; }
      int        GetClientHeight ()       const { return clientHeight; }
      int        GetClientWidth ()        const { return clientWidth; }
      int        GetColumnWidth (int id)  const;
      int        GetCurrentLine ()        const { return firstLine+activeLine; }
      TColor     GetEmptyGridColor ()     const { return emptyGrid; }
      TGEnterKey GetEnterKey ()           const { return enterKey; }
      int        GetFirstLine ()          const { return firstLine;  }
      TColor     GetFormBkg ()            const { return formBkg; }     // doc
      TColor     GetHiliteColor ()        const { return hilite; }
      TColor     GetHiliteTextColor ()    const { return hilitetext; }
      TGLayout   GetLayout ()             const { return layout; }      // doc
      TColor     GetLineColor ()          const { return lineColor; }
      TGLineMarker GetLineMarker ()       const { return lineMarker; }
      bool       GetLineSwapping ()       const { return lineSwapping; }
      int        GetMaxLines ()           const { return maxLines; }
      int        GetNLines ()             const { return nLines; }
      TColor     GetParentBkgColor ()     const { return parentBkgColor; }
      TGTabKey   GetTabKey ()             const { return tabKey; }
      int        GetLogicalWidth ()       const { return logicalWidth; }
      int        GetSelectedLine ()       const;
      int        GetSelectedLines (int *lines) const;
      int        GetSelectedLines (TBitField &bf) const;
      int        GetTitleHeight ()        const { return titleHeight; }
      int        GetTotalWidth  ()        const;
      int        GetTotalHeight ()        const;
      int        GetVBarWidth ()          const { return vbarWidth; }
      int        GetVisibleLines ()       const { return visibleLines; }
      int        GetX ()                  const { return Attr.X; }
      int        GetY ()                  const { return Attr.Y; }
      bool       HasFocus ()              const;                                 // doc
      void       HideColumn (int id);           // Charles R. Cordell
      void       HideColumns (bool hide, int *ids, int ncol);
      bool       IsModified ()            const { return modified; }
      bool       IsColumnHidden (int id)  const;
      bool       IsVBarSetToUserText ()   const { return (vbar ? !vbar->lineNumber : false); }
      void       SetModify          (bool flag) { modified = flag; }
      bool       LineSelected       (int line) const;
      void       PointToClient      (TPoint &point) const { MapWindowPoints (*client, &point, 1); }
      void       SetEmptyGridColor  (TColor c)     { emptyGrid = c; }
      void       SetEnterKey        (TGEnterKey ek){ enterKey = ek; }
      void       SetFormBkg         (TColor c)     { formBkg = c; if (layout == aGrid) Invalidate (); }  // doc
      void       SetFormInfo        (const string &ini, const string &sect) { iniFile = ini; section = sect; } // doc
      void       SetHiliteColor     (TColor c)     { hilite = c; }
      void       SetHiliteTextColor (TColor c)     { hilitetext = c; }
      void       SetLayout          (TGLayout l);                         // doc
      void       SetLineColor       (TColor c)     { lineColor = c; }
      void       SetLineMarker      (TGLineMarker lm);
      void       SetLineSwapping    (bool t)       { lineSwapping = t; }
      void       SetParentBkgColor  (TColor c)     { parentBkgColor = c; }
      void       SetTabKey          (TGTabKey tk)  { tabKey = tk; }
      void       SetVBarUserText    (bool on);
      bool       StaticGrid         ();
      void       UnselectAllLines   ();
      void       UnHideColumn(int id);         // Charles R. Cordell

      // virtual public methods ------------------------------------------------
      virtual void   Add            (TControl *p, const char *title, TGAlign a = Left, bool hide=false);  // Charles R. Cordell
      virtual bool   CellColor      (int line, int id, bool active, TColor &bg, TColor &fg, HBRUSH &brush) { return false; }
      virtual void   ChangeColWidth (TGridColumn *e, int delta, int minw = 8);
      virtual void   ChangeTitle    (int id, const char *newTitle);
      virtual int    GetDataLine    (int y);
      virtual TColor GetTitleColor  (TGTitleColor part) const;
      virtual const char *GetTitle  (int id) const;
      virtual void   Go             (int pos, bool showControl = true);
      virtual void   GotoFirstColumn ();
      virtual void   GotoLastColumn ();
      virtual bool   GotoNextColumn ();
      virtual bool   GotoPreviousColumn ();
      virtual void   HorzShrink     ();
      virtual bool   MouseInClient  (uint modKeys, const TPoint &point, uint message, int line, int id) { return false; }
      virtual bool   MouseInTitle   (uint modKeys, const TPoint &point, uint message, int id)   { return false; }
      virtual bool   MouseInVBar    (uint modKeys, const TPoint &point, uint message, int line) { return false; }
      virtual void   Move           (int delta, bool showControl = true);
      virtual void   Refresh        ();
      virtual void   RefreshItem    (int line, int id);
      virtual void   RefreshLine    (int line);
      virtual void   ResetPosition  ();
      virtual void   Remove         (int id, bool destroy = true);
      virtual void   Remove         (TGridColumn *e, bool destroy = true);
      virtual bool   SelectLine     (int line, uint modKeys = 0);    // changed by Dean Crawford
      virtual void   SetColumnWidth (int id, int width);
      virtual void   SetFont        (const char *facename, int height=0, int weight=FW_BOLD, bool italic=false);
      virtual void   SetFont        (int id, const char *facename, int height=0, int weight=FW_BOLD, bool italic=false);
      virtual void   SetLineHeight  (int h);
      virtual void   SetMaxLines    (int m);
      virtual void   SetNLines      (int n);
      virtual void   SetTitle       (int id, const char *text);
      virtual void   SetVisibleCells(TGVisible v);
      virtual void   SwapLines      (int l1, int l2);
      virtual void   HScrollHidden  (bool hidden) {}          // doc
      virtual void   VScrollHidden  (bool hidden) {}          // doc

      // virtual public methods which will usually be overloaded -------------
      virtual void DataChanged      (int line, int id, const void *data) {}
      virtual bool DataChanging     (int line, int id, const void *data) { return true; }
      virtual bool DeleteLine       (int line)           { return false; }
      virtual bool InsertLine       (int line)           { return false; }
      virtual bool IsValidLine      (int line)           { return true; }
      virtual void LineSelected     (int line, bool selected) {}
      virtual void NewRowSelected   (int newrow, int oldrow)      {}
      virtual void ScrollData       (int firstline, int lastline) {}

      virtual const void *TransferItem (int line, int id) = 0;
      virtual void TransferItem        (int line, int id, const void *data) = 0;

      DECLARE_RESPONSE_TABLE (TGrid);

   protected:
      friend TGridClient;
      friend TGridColumn;
      friend TGridVScroll;
      friend TGridHScroll;
      friend TGridTitle;
      friend TGridVerticalBar;
      
      friend TGCheckBox;
      friend TGComboBox;
      friend TGEdit;
      friend TGStatic;
      friend TGButton;
      TGridTitle   *title;
      TGridColumn *firstColumn;
      TGridColumn *lastColumn;
      TGridColumn *current;
      bool calcPos;
      bool lineSwapping;
      bool centerTitle;
      bool disableHScroll;
      bool disableVScroll;
      bool hideHScroll;
      bool hideVScroll;
      bool vscrollHidden;
      bool hscrollHidden;
      bool sizeColumn;
      bool tempNoHMove;
      bool horzShrink;
      bool vertShrink;
      bool modified;
      bool echo;                 // used in ProcessMouse                                           /
      bool donotcomeback;        // used in GotoLastColumn and GotoFirstColumn                     /
      bool noupdate;             // temporarily disable screen updates                             /

      bool readLayout;           // use parameter file for layout of form grid if set to true      /   // doc
      string iniFile;            // .ini file holding position of controls                         /
      string section;            // name of section for this grid in the ini file                  /

      int lineHeight;            // lineHeight in pixels                                           /
      int firstLine;             // offset of first visible line                                   /
      int activeLine;            // offset of visible active line                                  /
      int visibleLines;          // number of visible lines                                        /
      int selectRow;             // last current line used by TGrid::Go() (added by Dean Crawford) /
      int maxLines;              // maximum number of lines (-1 = no limite)                       /
      int nLines;                // actual number of lines, if maxLines == -1                      /
      int nControls;             // number of columns with a control                               /
      int hpos;                  // current horizontal offset (0 <= hpos < lastColumn->left)       /
      int selectedLine;          // index of line currently selected (single selection)            /
      int nsel;                  // actual number of lines in selectedLines                        /
      int logicalWidth;          // total width for all columns                                    /
      int clientWidth;           // client width                                                   /
      int clientHeight;          // client height                                                  /
      int titleHeight;           // height of title bar                                            /
      int vbarWidth;             // width of vertical bar                                          /
      int cxvscroll;             // width of vertical scroller                                     /
      int cyhscroll;             // height of horizontal scroller                                  /

      TBitField *selectedLines;  // map of lines currently selected                                /
      TGCursorType cursorType;
      TGridVScroll *vscroll;
      TGridHScroll *hscroll;
      TGridVerticalBar *vbar;
      TGridClient *client;
      TColor lineColor;
      TColor emptyGrid;
      TColor hilite;
      TColor hilitetext;
      TColor parentBkgColor;
      TColor formBkg;            // used with aForm layout                                         /   // doc
      TGEnterKey enterKey;
      TGTabKey   tabKey;
      TGLineMarker lineMarker;
      TGVisible visible;
      TGLayout layout;             // doc
      TGLineSelect lineSelection;
      TFont *gridFont;
      TFont *titleFont;
      TBitmap *ComboBmp;
      TBitmap *CheckBoxesBmp;
      TBitmap *BwccCheckBmp;
      TBitmap *BwccUncheckBmp;
      TBitmap *Ctl3dCheckBmp;
      LOGFONT *gridLogFont;
      LOGFONT *titleLogFont;
      TBrush *buttonFace;

      // virtual protected methods which are usually not overriden -------------
      virtual void   AppendColumn ();
      virtual bool   CalcPos ();
      virtual bool   CanKillFocus (TGridColumn *e);
      virtual void   ChangeWindowPos (int w,int h, uint flags);
      virtual void   CheckHScroll ();
      virtual void   ClearModify (TControl *c);
      virtual void   DrawCorner (TDC &dc, TRect &rc);
      virtual void   FindNearestUnlockedCell (TGridColumn *e);
      virtual TFont *GetGridFont (int id) const;
      virtual void   Init (int max, TGEnterKey ek, TGTabKey tk, TGLineMarker lm, int attrib, TGLayout l);     // doc
      virtual bool   IsHScrollbarNeeded ();
      virtual bool   IsVScrollbarNeeded ();
      virtual bool   KeyDown (uint key, int repeat);
      virtual void   KillFocusFromChild (TControl *child, HWND getFocus, bool okToChange);
      virtual void   MoveHPos (int delta);
      virtual void   ProcessMouse (uint modKeys, TPoint &point, uint message, bool sendToControl = false);
      virtual bool   SetCursorShape (TPoint &p); 
      virtual void   SetFocusOnChild (TControl *child, HWND hWndLostFocus);
      virtual void   SetFont ();
      virtual void   SetHPos (int newpos);
      virtual void   SetHScrollRange ();
      virtual void   SetTitleFont ();
      virtual void   SetTitleFont (const char *facename, int height =0, int weight =FW_BOLD, bool italic =false);
      virtual void   SetupWindow ();
      virtual void   SetVisibleLines (int nl);
      virtual bool   ValidateLine (int line);

      // message processing and TControl's overloaded methods ------------------
      bool CanClose            ();
      HBRUSH CtlColor          (HDC hDC, HWND hWndChild, uint ctlType);
      bool EvEraseBkgnd        (HDC hwnd);
      void EvKillFocus         (HWND hWndGetFocus);
      void EvNCPaint           ();
      void EvPaint             ();
      void EvSetFocus          (HWND hWndLooseFocus);
      void EvWindowPosChanging (WINDOWPOS far &wpos);
      char *GetClassName       ();
      void GetWindowClass      (WNDCLASS &wc);
      void Paint               (TDC &dc, bool erase, TRect &rc);
      void PaintClient         (TDC &dc, bool erase, TRect &rc);
      void PerformCreate       (int menuOrId);
      void SetVBarText         (int cwidth = 4);
      void SetVBarWidth        (int pixels);

      // virtual protected methods which may be overriden ----------------------
      virtual bool AppendNewLine  (int &newline) { return true; }
      virtual void LButtonDown    (uint modKeys, TPoint &point) {}
      virtual void LButtonUp      (uint modKeys, TPoint &point) {}
      virtual void LButtonDblClk  (uint modKeys, TPoint &point) {}
      virtual void RButtonDown    (uint modKeys, TPoint &point) {}
      virtual void RButtonUp      (uint modKeys, TPoint &point) {}
      virtual void RButtonDblClk  (uint modKeys, TPoint &point) {}
      virtual TGVisible VisibleCell (int line, int id) { return visible; }

      // keyboard handling -----------------------------------------------------
      virtual bool VkDelete (bool ctrl, bool shift, int repeat);
      virtual bool VkDown   (bool ctrl, bool shift, int repeat);
      virtual bool VkEnd    (bool ctrl, bool shift, int repeat);
      virtual bool VkEscape (bool ctrl, bool shift, int repeat);
      virtual bool VkHome   (bool ctrl, bool shift, int repeat);
      virtual bool VkInsert (bool ctrl, bool shift, int repeat);
      virtual bool VkLeft   (bool ctrl, bool shift, int repeat);
      virtual bool VkNext   (bool ctrl, bool shift, int repeat);
      virtual bool VkPrior  (bool ctrl, bool shift, int repeat);
      virtual bool VkReturn (bool ctrl, bool shift, int repeat);
      virtual bool VkRight  (bool ctrl, bool shift, int repeat);
      virtual bool VkTab    (bool ctrl, bool shift, int repeat);
      virtual bool VkUp     (bool ctrl, bool shift, int repeat);
   };

// COLOR_CODE //////////////////////////////////////////////////////////////////
#define COLOR_CODE(type)                                                   \
HBRUSH type::CtlColors (HDC hDC)                                           \
   {                                                                       \
   TDC dc(hDC);                                                            \
   dc.SetTextColor (text);                                                 \
   dc.SetBkColor (bkg);                                                    \
   return *brush;                                                          \
   }                                                                       \
void type::SetColors (TColor t, TColor b)                                  \
   {                                                                       \
   text = t;                                                               \
   bkg = b;                                                                \
   delete brush;                                                           \
   brush = new TBrush (bkg);                                               \
   }

// COLOR_HEADER ////////////////////////////////////////////////////////////////
#define COLOR_HEADER                                                       \
   protected:                                                              \
      TColor text;                                                         \
      TColor bkg;                                                          \
      TBrush *brush;                                                       \
   public:                                                                 \
      virtual HBRUSH CtlColors (HDC hDC);                                  \
      virtual void   SetColors (TColor text,                               \
                                TColor bkg = GetSysColor(COLOR_WINDOW));   \
      virtual TColor BkgColor ()  { return bkg; }                          \
      virtual TColor TextColor () { return text; }                         \
      virtual HBRUSH BkgBrush  () { return *brush; }

// GRID_EV_GETDLGCODE //////////////////////////////////////////////////////////
#define GRID_EV_GETDLGCODE(obj)                                             \
   uint TG##obj::EvGetDlgCode (MSG far *msg)                                \
      {                                                                     \
      if (grid)                                                             \
         return T##obj::EvGetDlgCode (msg) | DLGC_WANTALLKEYS;              \
      else                                                                  \
         return T##obj::EvGetDlgCode (msg);                                 \
      }

// GRID_EV_KEYUP ///////////////////////////////////////////////////////////////
#define GRID_EV_KEYUP(obj)                                                  \
   void TG##obj::EvKeyUp (uint key, uint repeat, uint flags)                \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         switch (key)                                                       \
            {                                                               \
            case VK_INSERT:                                                 \
            case VK_DELETE:                                                 \
            case VK_HOME:                                                   \
            case VK_END:                                                    \
            case VK_ESCAPE:                                                 \
            case VK_TAB:                                                    \
            case VK_RETURN:                                                 \
            case VK_UP:                                                     \
            case VK_DOWN:                                                   \
            case VK_PRIOR:                                                  \
            case VK_NEXT:                                                   \
            case VK_LEFT:                                                   \
            case VK_RIGHT:                                                  \
               return;                                                      \
            }                                                               \
         }                                                                  \
      T##obj::EvKeyUp (key, repeat, flags);                                 \
      }


// GRID_EV_CHAR ////////////////////////////////////////////////////////////////
#define GRID_EV_CHAR(obj)                                                   \
   void TG##obj::EvChar (uint key, uint repeat, uint flags)                 \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         switch (key)                                                       \
            {                                                               \
            case VK_ESCAPE:                                                 \
            case VK_TAB:                                                    \
            case VK_RETURN:                                                 \
               return;                                                      \
            }                                                               \
         }                                                                  \
      T##obj::EvChar (key, repeat, flags);                                  \
      }


// GRID_EV_KEYDOWN /////////////////////////////////////////////////////////////
#define GRID_EV_KEYDOWN(obj)                                                \
   void TG##obj::EvKeyDown (uint key, uint repeat, uint flags)              \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         switch (key)                                                       \
            {                                                               \
            case VK_HOME:                                                   \
            case VK_END:                                                    \
            case VK_ESCAPE:                                                 \
            case VK_INSERT:                                                 \
            case VK_DELETE:                                                 \
            case VK_TAB:                                                    \
            case VK_RETURN:                                                 \
            case VK_UP:                                                     \
            case VK_DOWN:                                                   \
            case VK_PRIOR:                                                  \
            case VK_NEXT:                                                   \
            case VK_LEFT:                                                   \
            case VK_RIGHT:                                                  \
               if (grid->KeyDown (key, repeat))                             \
                  return;                                                   \
               break;                                                       \
            }                                                               \
         }                                                                  \
      T##obj::EvKeyDown (key, repeat, flags);                               \
      }


// MOUSE_HANDLER_CODE //////////////////////////////////////////////////////////
#define MOUSE_HANDLER_CODE(obj)                                             \
   void TG##obj::EvLButtonDown (uint modKeys, TPoint &point)                \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         MapWindowPoints (*grid, &point, 1);                                \
         grid->ProcessMouse (modKeys, point, WM_LBUTTONDOWN, false);        \
         }                                                                  \
      T##obj::EvLButtonDown (modKeys, point);                               \
      }                                                                     \
   void TG##obj::EvLButtonUp (uint modKeys, TPoint &point)                  \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         MapWindowPoints (*grid, &point, 1);                                \
         grid->ProcessMouse (modKeys, point, WM_LBUTTONUP, false);          \
         }                                                                  \
      T##obj::EvLButtonUp (modKeys, point);                                 \
      }                                                                     \
   void TG##obj::EvLButtonDblClk (uint modKeys, TPoint &point)              \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         MapWindowPoints (*grid, &point, 1);                                \
         grid->ProcessMouse (modKeys, point, WM_LBUTTONDBLCLK, false);      \
         }                                                                  \
      T##obj::EvLButtonDblClk (modKeys, point);                             \
      }                                                                     \
   void TG##obj::EvRButtonDown (uint modKeys, TPoint &point)                \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         MapWindowPoints (*grid, &point, 1);                                \
         grid->ProcessMouse (modKeys, point, WM_RBUTTONDOWN, false);        \
         }                                                                  \
      T##obj::EvRButtonDown (modKeys, point);                               \
      }                                                                     \
   void TG##obj::EvRButtonUp (uint modKeys, TPoint &point)                  \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         MapWindowPoints (*grid, &point, 1);                                \
         grid->ProcessMouse (modKeys, point, WM_RBUTTONUP, false);          \
         }                                                                  \
      T##obj::EvRButtonUp (modKeys, point);                                 \
      }                                                                     \
   void TG##obj::EvRButtonDblClk (uint modKeys, TPoint &point)              \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         MapWindowPoints (*grid, &point, 1);                                \
         grid->ProcessMouse (modKeys, point, WM_RBUTTONDBLCLK, false);      \
         }                                                                  \
      T##obj::EvRButtonDblClk (modKeys, point);                             \
      }


// MOUSE_HANDLER_HEADER ////////////////////////////////////////////////////////
#define MOUSE_HANDLER_HEADER                                                \
   protected:                                                               \
      void EvLButtonDown (uint modKeys, TPoint &point);                     \
      void EvLButtonUp (uint modKeys, TPoint &point);                       \
      void EvLButtonDblClk (uint modKeys, TPoint &point);                   \
      void EvRButtonDown (uint modKeys, TPoint &point);                     \
      void EvRButtonUp (uint modKeys, TPoint &point);                       \
      void EvRButtonDblClk (uint modKeys, TPoint &point);

#endif

// These macros make transfering data from a grid easier ///////////////////////
#define XFR_STRING(x)  static_cast<const char *>(x)
#define XFR_bool(x)   *static_cast<const bool *>(x)
#define XFR_DATE(x)   *static_cast<const IntlDate *>(x)
#define XFR_TIME(x)   *static_cast<const TGTime *>(x)
#define XFR_DOUBLE(x) *static_cast<const double *>(x)
#define XFR_LONG(x)   *static_cast<const long *>(x)
#define XFR_SHORT(x)  *static_cast<const short *>(x)


// EOF
