/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// gridbmp.cpp                                                             //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.50 - 25 nov 1995                       //
// TGrid - Copyright  1994,1995 H. Picot, All Rights Reserved.            //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#include "griddll.h"
#pragma hdrstop


TGBitmap::TGBitmap (TWindow *p, int id, TModule *bmpmod, int x, int y, int w, int h, TModule *module)
   : TGStatic (p, id, "", x, y, w, h, 0, module),
     bmpModule (bmpmod) 
   {
   }


TGBitmap::TGBitmap (TWindow *p, int id, TModule *bmpmod, TModule *module)
   : TGStatic (p, id, 0, module),
     bmpModule (bmpmod)
   {
   }


void TGBitmap::DrawCell (TDC &dc, TRect &rc, const void *p, TGAlign /*align*/)
   {
   TGStatic::DrawCell (dc, rc, "");    // to draw background with the desired color
   if (p)
      {
      try
         {
         ::DibToDC (dc, bmpModule ? HINSTANCE(*bmpModule) : HINSTANCE(0), *static_cast<const int *>(p),
                    rc.left, rc.top, rc.Width(), rc.Height(), false, true);
         }
      catch (...)
         {
         }
      }
   }


TGBitmapFile::TGBitmapFile (TWindow *p, int id, int x, int y, int w, int h, TModule *module)
   : TGStatic (p, id, "", x, y, w, h, 0, module)
   {
   }


TGBitmapFile::TGBitmapFile (TWindow *p, int id, TModule *module)
   : TGStatic (p, id, 0, module)
   {
   }


void TGBitmapFile::DrawCell (TDC &dc, TRect &rc, const void *p, TGAlign /*align*/)
   {
   TGStatic::DrawCell (dc, rc, "");    // to draw background with the desired color
   if (p)
      {
      try
         {
         ::DibToDC (dc, static_cast<const char *>(p), rc.left, rc.top, rc.Width(), rc.Height(), false, true);
         }
      catch (...)
         {
         }
      }
   }


TSize DibToDC (TDC &dc, const char *file, int x, int y, int w, int h, bool calc, bool keepRatio)
   {
   TDib *dib = new TDib (file);
   TSize sz = DibToDC (dc, *dib, x, y, w, h, calc, keepRatio);
   delete dib;
   return sz;
   }


TSize DibToDC (TDC &dc, HINSTANCE inst, int resid, int x, int y, int w, int h, bool calc, bool keepRatio)
   {
   TDib *dib = new TDib (inst, TResId(resid));
   TSize sz = DibToDC (dc, *dib, x, y, w, h, calc, keepRatio);
   delete dib;
   return sz;
   }


TSize DibToDC (TDC &dc, TDib &dib, int x, int y, int w, int h, bool calc, bool keepRatio)
   {
   if (w == 0 || h == 0 || dib.Width() == 0 || dib.Height() == 0)
      return TSize(w,h);

   TRect dest (TPoint(x,y), TSize(w,h));
   TRect src (0, 0, dib.Width(), dib.Height ());

   if (keepRatio)
      {
      int sh = src.Height();
      int sw = src.Width ();

      // if bitmap < w,h, center it
      if (sh <= h && sw <= w)
         {
         dest.left += (w - sw) / 2;
         dest.top += (h - sh) / 2;
         dest.right = dest.left + sw;
         dest.bottom = dest.top + sh;
         }

      // else, shrink to fit and center it, keeping size ratio
      else
         {
         double rh = double(w) / sw;
         double rv = double(h) / sh;

         if (rh < rv)     // center vertically
            {
            rv -= rh;
            rv /= 2;
            dest.top += sh*rv;
            dest.bottom = dest.top + sh*rh;
            }
         else if (rv < rh)  // center horizontally
            {
            rh -= rv;
            rh /= 2;
            dest.left += sw*rh;
            dest.right = dest.left + sw*rv;
            }
         }
      }

   if (!calc)
      dc.StretchDIBits (dest, src, dib);

   return TSize (dest.Width(), dest.Height());
   }

// EOF
