struct TETHeader
{
  uint32  Size;      // total size of msg.
  int16   MT;        // Message Type.
  uint8   MajVer;    // msg ver.
  uint8   MinVer;    // msg ver.
};
//*******************************************************************
// note: Actual ET Msg formats (encryption tunnel):
//
// Size == ?;
// MT == 1;                      // handshake request
// MajVer == 1;
// MinVer == 0;
// char[25] SysName == "CARDOZA INTERNET CASINO";
// uint8 Client_MajVer == ?;
// uint8 Client_MinVer == ?;
// int16 Client_Public_Key_Len == ?;
// char [] Client_Public_Key == ?;
//*******************************************************************
// Size == ?;
// MT == 2                      // handshake reply
// MinVer == 1;
// MajVer == 0;
// int16 ErrorCode == ?;
// uint8 GW_MajVer == ?;
// uint8 GW_MinVer == ?;
// int16 GW_Public_Key_Len == ?;
// char [] GW_Public_Key == ?;
//*******************************************************************
// Size == ?;
// MT == 3                      // Heartbeat Request
// MajVer == 1;
// MinVer == 0;
// int16 Status == ?;           // Requester's Status
//*******************************************************************
// Size == ?;
// MT == 4;                      // Heartbeat Reply
// MajVer == 1;
// MinVer == 0;
// int16 Status == ?;                       // Replier's Status
// int16 ErrorCode == ?;
//*******************************************************************
// Size == ?;
// MT == 5;                      // encrypted CE Msg wrapper
// MajVer == 1;
// MinVer == 0;
// int16 Method == ?;
// int16 KeyLen == ?;
// char [] Key == ?;
// int16 Encrypted_Data_Len == ?;
// byte [] Encrypted_Data == ?;
//*******************************************************************
#define ET_MT_HANDSHAKE_REQ 1
#define ET_MT_HANDSHAKE_REP 2
#define ET_MT_HEARBEAT_REQ  3
#define ET_MT_HEARTBEAT_REP 4
#define ET_MT_ENCRYPTED_CE  5
//*******************************************************************
class TETMsg
{
public:
  int16 ConnectionID;
  TETHeader Header;
  byte *PData;

public:
  int16 GetConnectionID() {return ConnectionID;}
  uint32 GetSize();
  uint32 GetDataSize() {return (GetSize() - sizeof(TETHeader));}
  int16 GetMT();
  uint8 GetMajVer();
  uint8 GetMinVer();

  // handshake request functions.
  string Get_HSReq_SysName();
  uint8 Get_HSREq_ClientMajVer();
  uint8 Get_HSReq_ClientMinVer();
  string Get_HSReq_ClientPublicKey();

  // encrypted CE Msg functions.
  int16 Get_ECE_Method();
  string Get_ECE_Key();
  int16 Get_ECE_Data_Len();
  byte *Get_ECE_Data_Ptr();
};
//*******************************************************************