create or replace package body PCasSession as
//*******************************************************************
procedure GetdArray(gw_ipaddr IN,cs_array OUT,cs_num OUT,ec OUT)
{
  // create array from Tbl_CasSession
  // where (cs.GatewayID == gw_ipaddr &&
            cs.InUse == true)
}
//*******************************************************************
procedure Create(uan IN PDef.TUAN,
                 gatewayid IN PDef.TGatewayID,
                 cscounter IN PDef.TLgCount,
                 cs IN OUT TCasSession,
                 errorcode OUT PDef.TErrorCode)
{
  TCasSession cs = TCasSession(uan,
                               cscounter,
                               CasSessionID_Seq.Next,
                               gatewayid,
                               0,
                               PDef.GT_NONE,
                               true,
                               true,
                               SYSDATE,
                               0,
                               0,
                               0,
                               SYSDATE,
                               0);
  errorcode = ICAS.EC_NONE;
}
//*****************************************************************
procedure Add(cs IN OUT TCasSession,errorcode OUT PDef.TErrorCode)
{
  insert into Tbl_CasSession
    cs
  errorcode = ICAS.EC_NONE;
}
//*****************************************************************
procedure CreateAdd(uan IN PDef.TUAN,
                    gatewayid IN PDef.TGatewayID,
                    cscounter IN PDef.TLgCount,
                    cs IN OUT TCasSession,
                    errorcode OUT PDef.TErrorCode)
{
  Create(uan,gatewayid,cscounter,cs,errorcode);
  if (errorcode == PDef.EC_NONE)
  {
    Add(cs,errorcode);
  }
}
//*****************************************************************
procedure Lock(csid IN PDef.TCSID,
               cs IN OUT TCasSession,
               errorcode OUT PDef.TErrorCode)
{
  errorcode = PDef.EC_NONE;

  select cs
  from Tbl_CasSession
  where rec.CasSessionID == csid
  for update

  if (not found)
  {
    errorcode = PDef.EC_CASSESSION_NOT_FOUND;
  }
}
//*******************************************************************
function IsStopSafe(cs IN TCasSession) return PDef.TBool
{
  PDef.TBool rc = true;
  TGameSession gs;
  TErrorCode ec;
  PGameSession.Lock(cs.GameSessionID,gs,ec);
  if (ec == PDef.EC_NONE)
  {
    rc = PGameSession.IsStopSafe(gs);
    PGameSession.UnLock(gs,ec);
  }
  return rc;
}
//*****************************************************************
procedure Start(user IN OUT TUser,
                cs IN OUT TCasSession,
                gametype IN PDef.TGame,
                machineid IN PDef.TMID,
                ec OUT PDef.TErrorCode)
{
  TGameSession gs;
  cs.OverallStartDate = SYSDATE;
  cs.LastStartDate = cs.OverallStartDate;
  cs.GameSessionCounter += 1;
  PGameSession.CreateAdd(user.UAN,
                         cs.CasSessionID,
                         cs.CasSessionCounter,
                         cs.GameSessionCounter,
                         gs,
                         ec);
  if (ec == PDef.EC_NONE)
  {
    cs.GameSessionID = gs.GameSessionID;
    PGameSession.Start(user,gs,gametype,machineid,ec);
    if (ec == PDef.EC_NONE)
    {
      PGameSession.UnLock(gs,ec);
    }
  }
}
//*****************************************************************
procedure Start(user IN OUT TUser,
                cs IN OUT TCasSession,
                ec OUT PDef.TErrorCode)
{
  Start(user,cs,PDef.GT_LOBBY,0,ec);
}
//*****************************************************************
procedure Pause(user IN OUT TUser,
                cs IN OUT TCasSession,
                ec OUT PDef.TErrorCode)
{
  TGameSession gs;
  cs.LastEndDate = SYSDATE;
  cs.TimePlayedSec += cs.LastEndDate - cs.LastStartDate;
  cs.DisconnectCounter += 1;
  PGameSession.Lock(cs.GameSessionID,gs,ec);
  if (ec == PDef.EC_NONE)
  {
    PGameSession.Pause(user,gs,ec);
    if (ec == PDef.EC_NONE)
    {
      PGameSession.UnLock(gs,ec);
    }
  }
}
//*****************************************************************
procedure ReStart(user IN OUT TUser,
                  cs IN OUT TCasSession,
                  ec OUT PDef.TErrorCode)
{
  TGameSession gs;
  cs.LastStartDate = SYSDATE;
  PGameSession.Lock(cs.GameSessionID,gs,ec);
  if (ec == PDef.EC_NONE)
  {
    PGameSession.ReStart(user,gs,ec);
    if (ec == PDef.EC_NONE)
    {
      PGameSession.UnLock(gs,ec);
    }
  }
}
//*****************************************************************
procedure Stop(user IN OUT TUser,
               cs IN OUT TCasSession,
               ec OUT PDef.TErrorCode)
{
  TGameSession gs;
  cs.LastEndDate = SYSDATE;
  cs.OverallEndDate = cs.LastEndDate;
  cs.TimePlayedSec += cs.LastEndDate - cs.LastStartDate;
  PGameSession.Lock(cs.GameSessionID,gs,ec);
  if (ec == PDef.EC_NONE)
  {
    PGameSession.Stop(user,gs,ec);
    if (ec == PDef.EC_NONE)
    {
      PGameSession.UnLock(gs,ec);
    }
  }
}
//*****************************************************************
procedure SwitchGame(user IN OUT TUser,
                     cs IN OUT TCasSession,
                     gametype IN PDef.TGame,
                     mid IN PDef.TMID,
                     endgs IN PDef.TBool,
                     ec OUT PDef.TErrorCode)
{
  TGameSession gs;
  PGameSession.Lock(cs.GameSessionID,gs,ec);
  if (ec == PDef.EC_NONE)
  {
    if (!PGameSession.IsSafeStop(gs) && !endgs)
    {
      ec = PDef.EC_SWITCH_UNSAFE;
    }
    else
    {
      Stop(user,cs,ec);
      if (ec == PDef.EC_NONE)
      {
        Start(cs,gametype,mid,ec);
      }
    }
  }
}
//*******************************************************************
end;