create or replace package body PUser as
//*******************************************************************
procedure GetArray(gw_ipaddr IN,user_array OUT,user_num OUT,ec OUT)
{
  user_num = 0;

  PCasSession.GetArray(gw_ipaddr,cs_array,cs_num,ec);
  for (int i=0;i<cs_num;i++)
  {
    Get(cs_array[i].UAN,user,ec);
    user_array[user_num] = user;
    user_num++;
  }
}
//*******************************************************************
procedure LostGW(ipaddr IN,ec OUT)
{
  GetArray(ipaddr,user_array,user_num,ec);

  for (int i = 0;i<user_num;i++)
  {
    Logoff(user_array[i].UAN,false,ec);
  }
}
//*******************************************************************
function Exists(username IN PDef.TUserNameStr) return TBool
{
  TUser user;
  PDef.TBool rc = true;

  select from Tbl_Users
    into user
  where rec.UserName == username

  if (not found)
  {
    rc = false;
  }
  return rc;
}
//*****************************************************************
procedure Lock(username IN PDef.TUserNameStr,
               user IN OUT TUser,
               errorcode OUT PDef.TErrorCode)
{
  errorcode = PDef.EC_NONE;

  select from Tbl_Users
    into user
  where rec.UserName == username
  for update  // lock it.

  if (not found)
  {
    errorcode = PDef.EC_USER_NOT_FOUND;
  }
}
//*****************************************************************
procedure Lock(uan IN PDef.TUAN,
               user IN OUT TUser,
               errorcode OUT PDef.TErrorCode)
{
  errorcode = PDef.EC_NONE;
  select from Tbl_Users
   into user
  where rec.UAN == uan
  for update

  if (not found)
  {
    errorcode = PDef.EC_USER_NOT_FOUND;
  }
}
//*****************************************************************
procedure UnLock(user IN OUT TUser,errorcode OUT PDef.TErrorCode)
{
  errorcode = PDef.EC_NONE;

  update Tbl_Users
   set rec = user
  where rec.UAN == user.UAN

  if (update failed)
  {
    errorcode = PDef.EC_USER_SET_FAIL;
  }
}
//*****************************************************************
procedure Add(user IN TUser,errorcode OUT PDef.TErrorCode)
{
  if (Exists(user.UserName))
  {
    ec = EC_USER_USERNAME_INUSE;
  }
  insert user into Tbl_User;

  errorcode = PDef.EC_NONE;
}
//*****************************************************************
procedure Create(username IN PDef.TUserName,
                 pw IN PDef.TPWStr,
                 name IN TName,
                 address IN TAddress,
                 email IN,
                 dayphone IN,
                 eveningphone IN,
                 dob IN,
                 sex_male IN,
                 mothersmaiden IN,
                 hometown IN,
                 user IN OUT TUser,
                 errorcode OUT PDef.TErrorCode)
{
  PRegion.PreProcessCreate(address.State,address.Country,ec);
  if (ec == EC_NONE)
  {
    user = TUser(Seq_UAN.next(),
                 username,
                 true,
                 false,
                 false,
                 0,
                 false,
                 0,
                 0,
                 TDate(0),
                 TRegLockData(true,true,true,true,false,0));

    // check username for uniqueness before doing all this stuff.
    // if another thread's adduser() sneeks in before this is done
    // then this work should get rolled back.
    if (Exists(username))
    {
      errorcode = PDef.EC_USER_USERNAME_INUSE;
    }
    else
    {
      PWallet.CreateAdd(user.UAN,errorcode);
      if (errorcode == PDef.EC_NONE)
      {
        PPassword.CreateAdd(user.UAN,pw,errorcode);
        if (errorcode == PDef.EC_NONE)
        {
          PUser_Info.CreateAdd(user.UAN,name,address,
                               email,dayphone,eveningphone,
                               dob,sex_male,mothersmaiden,hometown,
                               errorcode);
          if (errorcode == PDef.EC_NONE)
          {
            PAccounts.CreateAdd(user.UAN,errorcode);
            PRegion.CheckLocks(RLC_MA,user,address.State,address.Country,ec);
          }
        }
      }
    }
  }
}
//*****************************************************************
procedure CreateAdd(username IN PDef.TUserName,
                    password IN PDef.TPWStr,
                    name IN TName,
                    address IN TAddress,
                    email IN,
                    dayphone IN,
                    eveningphone IN,
                    dob IN,
                    sex_male IN,
                    mothersmaiden IN,
                    hometown IN,
                    user IN OUT TUser,
                    errorcode OUT PDef.TErrorCode)
{
  // note: if this function fails then we need rollback.

  Create(username,password,name,address,misc_address,personal,user,errorcode);
  if (errorcode == PDef.EC_NONE)
  {
    Add(user,errorcode);
    if (ec == EC_NONE)
    {
      Lock(user.UAN,user,ec); // lock and return it.
    }
  }
}
//*****************************************************************
procedure GetModifiedUserNames(username IN PDef.TUserNameStr,
                               name IN TName,
                               un1 IN OUT PDef.TUserNameStr,
                               un2 IN OUT PDef.TUserNameStr,
                               un3 IN OUT PDef.TUserNameStr,
                               un4 IN OUT PDef.TUserNameStr)
{
  // note: snaphshot usernames.
  //       For speed up ->
  //       make a sorted array of usernames and then
  //       instead of using Exists(username) use Find()
  //       on array.

  PDef.TUserNameStr un = username;
  un1 = GetModifiedUserName(un);
   un = name.First;
  un2 = GetModifiedUserName(un);
   un = name.First + name.Last[0];
  un3 = GetModifiedUserName(un);
   un = name.First[0] + name.Last;
  un4 = GetModifiedUserName(un);
}
//*****************************************************************
function GetModifiedUserName(username IN PDef.TUserNameStr) return TUserNameStr
{
  string s = username;

  // make sure we're not beyond our max size.
  s = s.truncate(14);
  PDef.TSmNum len = length(s);

  // also, make sure we're not under our min size.
  if (len < 4)
  {
    return "";
  }

  // try name once before we start appending numbers.
  if (!Exists(s))
  {
    return s;
  }

  // temporary strings.
  string ts = s;
  string ts2;

  // we're at our max size, & we know name didn't work, so truncate by 1.
  if (len == 14)
  {
    // don't even try this name, because tried above.
    ts = ts.truncate(13);
  }

  len = ts.length();
  if (len == 13)
  {
    // try this name before appending numbers, might occasionally be redundant.
    if (!Exists(ts))
    {
      return ts;
    }
    // start numbering using 1 digit at end.
    ts2 = ts;
    for (int i = 1;i <= 9;i++)
    {
      ts2 = ts + itoa(i);
      if (!Exists(ts2))
      {
        return ts2;
      }
    }
    // well, let's make room for 2 digits.
    ts = ts.truncate(12);
  }

  len = ts.length();
  if (len == 12)
  {
    // try this name before appending numbers, might occasionally be redundant.
    if (!Exists(ts))
    {
      return ts;
    }
    // start numbering using up to 2 digits at end.
    for (int i = 1;i <= 99;i++)
    {
      ts2 = ts + itoa(i);
      if (!Exists(ts2))
      {
        return ts2;
      }
    }
    // well, let's make room for 3 digits.
    ts = ts.truncate(11);
  }

  // let's try this name before numbering, might occasionally be redundant.
  if (!Exists(ts))
  {
    return ts;
  }
  // start numbering using up to 3 digits at end.
  for (int i = 1;i <= 999;i++)
  {
    ts2 = ts + itoa(i);
    if (!Exists(ts2))
    {
      return ts2;
    }
  }
  // well, we never found a usable name.
  return "";
}
//*****************************************************************
procedure ValidateUser(user IN OUT TUser,
                       pw IN PDef.TPWStr,
                       mustchangepw OUT PDef.TBool,
                       errorcode OUT PDef.TErrorCode)
{
  errorcode = PDef.EC_NONE;
  mustchangepw = false;

  if (!user.Open)
  {
    errorcode = PDef.EC_USER_NOT_OPEN;
  }
  else if (user.SysLock)
  {
    errorcode = PDef.EC_USER_SYSLOCK;
  }
  else if (user.SysError)
  {
    errorcode = PDef.EC_USER_SYSERROR;
  }
  else if (user.CasLock)
  {
    ec = EC_USER_CASLOCK;
  }
  else if (!RegLockData.CashPlay && !RegLockData.FreePlay)
  {
    ec = EC_USER_REGLOCK;
  }
  else
  {
    TPassword password;
    PPassword.Lock(user.UAN,password,ec);
    if (ec == EC_NONE)
    {
      PPassword.Verify(password,pw,ec);
      PPassword.UnLock(password,ec);
    }
  }
}
//*******************************************************************
procedure Login(username IN PDef.TUserNameStr,
                pw IN PDef.TPWStr,
                gatewayid IN OUT PDef.TGatewayID,
                mustchangepw OUT PDef.TBool,
                gt OUT PDef.TGame,
                mid OUT PDef.TMID,
                uan OUT PDef.TUAN,
                ec OUT PDef.TErrorCode)
{
  TUser user;
  Lock(username,user,ec);
  if (ec == PDef.EC_NONE)
  {
    uan = user.GetUAN();
    Login(user,pw,gatewayid,gt,mid,mustchangepw,ec);

    UnLock(user,ec);
  }
}
//*****************************************************************
procedure NewLogin(username IN PDef.TUserNameStr,
                   pw IN PDef.TPWStr,
                   gwid IN PDef.TGatewayID,
                   fname IN PDef.TNameStr,
                   lname IN PDef.TNameStr,
                   street IN PDef.TStreetStr,
                   city IN PDef.TCityStr,
                   state IN PDef.TStateStr,
                   country IN PDef.TCountryStr,
                   zip IN PDef.TZipStr,
                   email IN PDef.TNameStr,
                   dayphone IN PDef.TNameStr,
                   eveningphone IN PDef.TNameStr,
                   dob IN PDef.TDate,
                   sex_male IN PDef.TBool,
                   mothersmaiden IN PDef.TNameStr,
                   hometown IN PDef.TCityStr,
                   uan OUT PDef.TUAN,
                   gt OUT PDef.TGame,
                   mid OUT PDef.TMID,
                   modun1 OUT PDef.TUserNameStr,
                   modun2 OUT PDef.TUserNameStr,
                   modun3 OUT PDef.TUserNameStr,
                   modun4 OUT PDef.TUserNameStr,
                   ec OUT PDef.TErrorCode)
{

  TUser user;

  // try to create user and add to Tbl_User
  CreateAdd(username,
            pw,
            fname,
            lname,
            street,
            city,
            state,
            country,
            zip,
            email,
            dayphone,
            eveningphone,
            dob,
            sex_male,
            mothersmaiden,
            hometown,
            user,
            ec);
  // successful, so try to login user.
  if (ec == PDef.EC_NONE)
  {
    TBool mustchangepw = false;  // not needed.
    Login(user,
          pw,
          gwid,
          gt,
          mid,
          mustchangepw,
          ec);

    if (ec == PDef.EC_NONE)
    {
      Lock(user,ec);
      if (ec == EC_NONE)
      {
        Commit();
      }
    }
  }
  // unsuccessful, because not unique username, recommend some.
  else if (ec == PDef.EC_USER_USERNAME_INUSE)
  {
    GetModifiedUserNames(username,
                         fname,
                         lname,
                         modun1,
                         modun2,
                         modun3,
                         modun4);
    Rollback();
  }
}
//*****************************************************************
procedure Login(user IN OUT TUser,
                pw IN TPWStr,
                gatewayid IN OUT TGatewayID,
                gt IN OUT TGame,
                mid IN OUT TMID,
                mustchangepw IN OUT TBool,
                ec OUT TErrorCode)
{
  ValidateUser(user,pw,mustchangepw,errorcode);

  // passed validation.
  if (errorcode == PDef.EC_NONE)
  {
    user.IncLoginCounter();

    TCasSession cs;
    PCasSession.Lock(user.CasSessionID,cs,errorcode);
    if (errorcode == PDef.EC_NONE)
    {
      if (cs.IsInUse())
      {
        if (cs.IsLoggedIn())
        {
          gatewayid = cs.GetGatewayID();
          errorcode = PDef.EC_USER_LOGGEDIN;
        }
        else
        {
          PCasSession.ReStart(user,cs,gatewayid,errorcode);
          if (errorcode == PDef.EC_NONE)
          {
            PCasSession.UnLock(cs,errorcode)
          }
        }
      }
      else  // CasSessionID should have been NULL or 0.
      {
        user.CasSessionCounter++;
        PCasSession.CreateAdd(user.UAN,
                              gatewayid,
                              user.CasSessionCounter,
                              cs,
                              errorcode);
        if (errorcode == PDef.EC_NONE)
        {
          PCasSession.Start(user,cs,errorcode);
          if (errorcode == PDef.EC_NONE)
          {
            PCasSession.UnLock(cs,errorcode);
          }
        }
      }
    }
    else if (errorcode == PDef.EC_CASSESSION_NOTFOUND)
    {
      user.CasSessionCounter++;
      PCasSession.CreateAdd(user.UAN,
                            gatewayid,
                            user.CasSessionCounter,
                            cs,
                            errorcode);
      if (errorcode == PDef.EC_NONE)
      {
        PCasSession.Start(user,cs,errorcode);
        if (errorcode == PDef.EC_NONE)
        {
          PCasSession.UnLock(cs,errorcode);
        }
      }
    }
    else
    {
      // do nothing, just fail and return errorcode.
    }
  }
}
//*******************************************************************
procedure Logoff(uan IN PDef.TUAN,
                 endcassession IN PDef.TBool,
                 ec OUT PDef.TErrorCode)
{
  TErrorCode ec1,ec2 = EC_NONE;

  TUser user;
  Lock(uan,user,ec);

  if (ec == PDef.EC_NONE)
  {
    TCasSession cs;
    PCasSession.Lock(cs,ec);
    if (ec == PDef.EC_NONE)
    {
      if (!data.EndCasSession && !PCasSession.IsSafeStop(cs))
      {
        PCasSession.Pause(user,cs,ec);
      }
      else
      {
        PCasSession.Stop(user,cs,ec);
        if (ec == PDef.EC_NONE)
        {
          user.CasSessionID = 0;
        }
      }
    }
    PCasSession.UnLock(cs,ec1);
    if (ec == PDef.EC_NONE)
    {
      ec = ec1;
    }
    UnLock(user,ec2);
    if (ec == PDef.EC_NONE)
    {
      ec = ec2;
    }
  }
}
//*****************************************************************
procedure SwitchGame(uan IN PDef.TUAN,
                     endgamesession IN PDef.TBool,
                     gt IN OUT PDef.TGame,
                     mid IN OUT PDef.TMID,
                     ec OUT PDef.TErrorCode)
{
  TErrorCode ec1,ec2;
  TUser user;
  Lock(uan,user,ec);
  if (ec == PDef.EC_NONE)
  {
    TCasSession cs;
    PCasSession.Lock(user.UAN,cs,ec);
    if (ec == PDef.EC_NONE)
    {
      PCasSession.SwitchGame(user,
                             cs,
                             gt,
                             mid,
                             endgamesession,
                             ec);
    }
    PCasSession.UnLock(cs,ec1);
    if (ec == PDef.EC_NONE)
    {
      ec = ec1;
    }
    UnLock(user,ec2);
    if (ec == PDef.EC_NONE)
    {
      ec = ec2;
    }
  }
}
//*******************************************************************
procedure LockUserObjects(uan IN PDef.TUAN,
                          user IN OUT TUser,
                          wallet IN OUT TWallet,
                          cs IN OUT TCasSession,
                          gs IN OUT TGameSession,
                          ec OUT PDef.TErrorCode)
{
  TErrorCode ec2;
  Lock(uan,user,ec);
  if (ec == PDef.EC_NONE)
  {
    PWallet.Lock(user.UAN,wallet,ec);
    if (ec != PDef.EC_NONE)
    {
      UnLock(user,ec2);
    }
    else
    {
      PCasSession.Lock(user.CasSessionID,cs,ec);
      if (ec != PDef.EC_NONE)
      {
        PWallet.UnLock(wallet,ec2);
        UnLock(user,ec2);
      }
      else
      {
        PGameSession.Lock(cs.GameSessionID,gs,ec);
        if (ec != PDef.EC_NONE)
        {
          PCasSession.UnLock(cs,ec2);
          PWallet.UnLock(wallet,ec2);
          UnLock(user,ec2);
        }
      }
    }
  }
}
//*****************************************************************
procedure UnLockUserObjects(user IN OUT TUser,
                            wallet IN OUT TWallet,
                            cs IN OUT TCasSession,
                            gs IN OUT TGameSession,
                            ec OUT PDef.TErrorCode)
{
  PGameSession.UnLock(gs,ec);
  PCasSession.UnLock(cs,ec);
  PWallet.UnLock(wallet,ec);
  UnLock(user,ec);
}
//*******************************************************************
procedure SetRegLocks(user IN OUT,region IN,rlcode IN,ec OUT)
{
  if (rlcode == RLC_NONE)
  {
    // do nothing.
  }
  else if (rlcode == RLC_MA)
  {
    user.RegLockData.SetLocks(region.MA);
  }
  else if (rlcode == RLC_CHECK)
  {
    user.RegLockData.SetLocks(region.Check);
  }
  else if (rlcode == RLC_CC)
  {
    user.RegLockData.SetLocks(region.CC);
  }
  else if (rlcode == RLC_WT)
  {
    user.RegLockData.SetLocks(region.WT);
  }
  else if (rlcode == RLC_WU)
  {
    user.RegLockData.SetLocks(region.WU);
  }
}
//*******************************************************************
end PUser;