create or replace package body PSlots as
//*****************************************************************
procedure BeginRound(si IN OUT TSlots_Instance,
                     ec OUT PDef.TErrorCode)
{
  si.Round.Begin();
  si.State = PDef.STATE_BET;
  ec = PDef.EC_NONE;
}
//*****************************************************************
procedure EndRound(si IN OUT TSlots_Instance,
                   ec OUT PDef.TErrorCode)
{
  if (si.Round.
  si.State = PDef.STATE_END;
  si.Round.End();
  PSlot_Round.CreateAdd(si.Round,ec);
}
//*****************************************************************
procedure Start(user IN OUT TUser,
                mid IN PDef.TMID,
                freeplay IN PDef.TBool,
                iid OUT PDef.TIID,
                ec OUT PDef.TErrorCode)
{
  TMachine sm;
  TSlot_Machine.Get(mid,sm,ec);
  if (ec == PDef.EC_NONE)
  {
    TSlot_Instance si;
    PSlot_Instance.Create(mid,user.UAN,freeplay,si,ec);
    if (ec == PDef.EC_NONE)
    {
      iid = si.Round.InstanceID;
      BeginRound(si,ec);
      if (ec == PDef.EC_NONE)
      {
        PSlot_Instance.Add(si,ec);
      }
    }
  }
}
//*****************************************************************
procedure Stop(user IN OUT TUser,
               iid IN PDef.TIID,
               cscounter IN PDef.TLgCount,
               wagered IN OUT PDef.TCurrency,
               wonlost IN OUT PDef.TCurrency,
               ec OUT PDef.TErrorCode)
{
  TSlot_Instance si;
  PSlot_Instance.Lock(iid,si,ec);
  if (ec == PDef.EC_NONE)
  {
    TWallet w;
    PWallet.Lock(uan,w,ec);
    if (ec == PDef.EC_NONE)
    {
      TSlot_Machine sm;
      PSlot_Machine.Get(si.MachineID,sm,ec);
      if (ec == PDef.EC_NONE)
      {
        if (si.Round.UAN != uan)
        {
          ec = PDef.EC_USER_INSTANCE_MISMATCH;
        }
        else
        {
          TLgNum cred = si.Round.CoinsPlayed + si.Round.Credits;
          si.Round.CoinsPlayed = 0;
          si.Round.Credits = 0;
          if (cred > 0)
          {
            w.IncChips(cred * sm.CoinSize);
          }
          EndRound(si,ec);
          if (ec == PDef.EC_NONE)
          {
            wagered = si.Wagered;
            wonlost = si.WonLost;
            si.InUse = false;
          }
        }
      }
    }
    PWallet.UnLock(w,ec);
    PSlot_Instance.UnLock(si,ec);
  }
}
//*****************************************************************
function IsStopSafe(iid IN PDef.TIID,
                    uan IN PDef.TUAN) return TBool
{
  PDef.TBool rc = true;
  PDef.TErrorCode ec = PDef.EC_NONE;
  TSlot_Instance si;
  PSlot_Instance.Get(iid,si,ec);
  if (ec == PDef.EC_NONE)
  {
    if (si.State != PDef.STATE_BET)
    {
      rc = false;
    }
  }
}
//*****************************************************************
procedure BetOne(uan IN PDef.TUAN,
                 reply OUT PDef.TBetOne_Reply)
{
  reply = (0,0,0,PDef.EC_NONE);
  TErrorCode ec1 = PDef.EC_NONE;
  TErrorCode ec2 = PDef.EC_NONE;

  TUser user;
  TWallet wallet;
  TCasSession cs;
  TGameSession gs;
  TSlot_Instance si;
  TSlot_Machine sm;

  PUser.LockUserObjects(uan,user,wallet,cs,gs,reply.ErrorCode);
  if (reply.ErrorCode == PDef.EC_NONE)
  {
    if (gs.GetGameType() != PDef.GT_SLOTS)
    {
      reply.ErrorCode = PDef.EC_GAME_INV;
    }
    else
    {
      PSlot_Instance.Lock(gs.GetInstanceID(),si,reply.ErrorCode);
      if (reply.ErrorCode == PDef.EC_NONE)
      {
        PSlot_Machine.Get(si.Round.MachineID,reply.ErrorCode);
        if (reply.ErrorCode == PDef.EC_NONE)
        {
          if (si.Round.UAN != user.UAN)
          {
            reply.ErrorCode = PDef.EC_USER_INSTANCE_MISMATCH;
          }
          else if (si.State != PDef.STATE_BET)
          {
            reply.ErrorCode = PDef.EC_STATE_INV;
          }
          else
          {
            if (si.Round.CoinsPlayed >= sm.MaxCoinsPlayable)
            {
              reply.ErrorCode = PDef.EC_SLOTS_COINSPLAYABLE_MAX;
            }
            else
            {
              if (si.Round.Credits > 0)
              {
                si.Round.Credits--;
                si.Round.CoinsPlayed++;
                si.Round.BetOneCounter++;
              }
              else if (wallet.GetChips() >= sm.CoinSize)
              {
                wallet.DecChips(sm.CoinSize);
                si.Round.CoinsPlayed++;
                si.Round.BetOneCounter++;
              }
              else
              {
                reply.ErrorCode = PDef.EC_WALLET_CHIPS_NSF;
              }
            }
            reply.CoinsPlayed = si.Round.CoinsPlayed;
            reply.Credits = si.Round.Credits;
            reply.Chips = wallet.GetChips();
          }
          PSlot_Instance.UnLock(si,ec1);
        }
      }
    }
    PUser.UnLockUserObjects(user,wallet,cs,gs,ec2);
  }

  if (ec1 == PDef.EC_NONE && ec2 == PDef.EC_NONE)
  {
    Commit();
  }
  else
  {
    reply.ErrorCode = PDef.EC_SAVE_FAILURE;
    Rollback();
  }
}
//*****************************************************************
procedure BetMax(uan IN PDef.TUAN,
                 reply OUT TBetMax_Reply)
{
  reply = (0,0,0,PDef.EC_NONE);
  TErrorCode ec1 = PDef.EC_NONE;
  TErrorCode ec2 = PDef.EC_NONE;

  TUser user;
  TWallet wallet;
  TCasSession cs;
  TGameSession gs;
  TInstance si;
  TMachine sm;

  PUser.LockUserObjects(uan,user,wallet,cs,gs,reply.ErrorCode);

  if (reply.ErrorCode == PDef.EC_NONE)
  {
    PSlot_Instance.Lock(gs.GetInstanceID(),si,reply.ErrorCode);
    if (reply.ErrorCode == PDef.EC_NONE)
    {
      if (si.Round.UAN != user.UAN)
      {
        reply.ErrorCode = PDef.EC_USER_INSTANCE_MISMATCH;
      }
      else if (si.State != PDef.STATE_BET)
      {
        reply.ErrorCode = PDef.EC_STATE_INV;
      }
      else
      {
        PSlot_Machine.Get(Round.MachineID,sm,reply.ErrorCode);
        if (reply.ErrorCode == PDef.EC_NONE)
        {
          if (si.Round.Credits >= sm.MaxCoinsPlayable - si.Round.CoinsPlayed))
          {
            si.Round.Credits -= (sm.MaxCoinsPlayable - si.Round.CoinsPlayed);
            si.Round.CoinsPlayed = sm.MaxCoinsPlayable;
          }
          else if (si.Round.Credits > 0)
          {
            si.Round.CoinsPlayed += si.Round.Credits;
            si.Round.Credits = 0;
          }
          if (si.Round.CoinsPlayed < sm.MaxCoinsPlayable)
          {
            if (((sm.MaxCoinsPlayable - si.Round.CoinsPlayed) * sm.CoinSize) <
                  wallet.GetChips())
            {
              wallet.DecChips(sm.MaxCoinsPlayed - si.Round.CoinsPlayed) * sm.CoinsSize);
              si.Round.CoinsPlayed = sm.MaxCoinsPlayable;
            }
          }
        }
        reply.CoinsPlayed = si.Round.CoinsPlayed;
        reply.Credits = si.Round.Credits;
        reply.Chips = wallet.GetChips();

        if (si.Round.CoinsPlayed == sm.MaxCoinsPlayable)
        {
          si.Round.BetMax = true;
        }
        else
        {
          reply.ErrorCode = PDef.EC_WALLET_CHIPS_NSF;
        }
      }
      PSlot_Instance.UnLock(si,ec1);
    }

    PUser.UnLockUserObjects(user,wallet,cs,gs,ec2);
  }

  if (ec1 == PDef.EC_NONE && ec2 == PDef.EC_NONE)
  {
    Commit();
  }
  else
  {
    reply.ErrorCode = PDef.EC_SAVE_FAILURE;
    Rollback();
  }
}
//*****************************************************************
procedure Cashout(uan IN PDef.TUAN,
                  reply OUT PDef.TCashout_Reply)
{
  reply = (0,0,0,PDef.EC_NONE);
  TErrorCode ec1 = PDef.EC_NONE;
  TErrorCode ec2 = PDef.EC_NONE;

  TUser user;
  TWallet wallet;
  TCasSession cs;
  TGameSession gs;
  TInstance si;
  TMachine sm;

  PUser.LockUserObjects(uan,user,wallet,cs,gs,reply.ErrorCode);

  if (reply.ErrorCode == PDef.EC_NONE)
  {
    PSlot_Instance.Lock(gs.GetInstanceID(),si,reply.ErrorCode)
    if (reply.ErrorCode == PDef.EC_NONE)
    {
      if (si.Round.UAN != user.UAN)
      {
        reply.ErrorCode = PDef.EC_USER_INSTANCE_MISMATCH;
      }
      else if (si.State != PDef.STATE_BET)
      {
        reply.ErrorCode = PDef.EC_STATE_INV;
      }
      else
      {
        PSlot_Machine.Get(Round.MachineID,sm,reply.ErrorCode);
        if (reply.ErrorCode == PDef.EC_NONE)
        {
          if (si.Round.Credits > 0)
          {
            wallet.IncChips(si.Round.Credits * sm.CoinSize);
            si.Round.Credits = 0;
          }
          si.Round.CashOut = true;
          EndRound(si,reply.ErrorCode);
          if (reply.ErrorCode == PDef.EC_NONE)
          {
            BeginRound(si,reply.ErrorCode);
          }
        }
        reply.Credits = si.Round.Credits;
        reply.Chips = wallet.GetChips();
        reply.CoinsPlayed = si.Round.CoinsPlayed;
      }
      PSlot_Instance.UnLock(si,ec1);
    }
    PUser.UnLockUserObjects(user,wallet,cs,gs,ec2);
  }

  if (ec1 == PDef.EC_NONE && ec2 == PDef.EC_NONE)
  {
    Commit();
  }
  else
  {
    reply.ErrorCode = PDef.EC_SAVE_FAILURE;
    Rollback();
  }
}
//*****************************************************************
procedure Spin(uan IN PDef.TUAN,
               reply OUT PDef.TSpin_Reply)
{
  reply = TSpin_Reply();
  TUser user;
  TWallet wallet;
  TCasSession cs;
  TGameSession gs;
  TInstance si;
  TMachine sm;
  TReel r1;
  TReel r2;
  TReel r3;
  TPayoff payoff;

  PUser.LockUserObjects(uan,user,wallet,cs,gs,reply.ErrorCode);
  if (reply.ErrorCode == PDef.EC_NONE)
  {
    PSlot_Instance.Lock(gs.GetInstanceID(),si,reply.ErrorCode);
    if (reply.ErrorCode == PDef.EC_NONE)
    {
      if (si.Round.UAN != user.UAN)
      {
        reply.ErrorCode = PDef.EC_USER_INSTANCE_MISMATCH;
      }
      else if (si.State != PDef.STATE_BET)
      {
        reply.ErrorCode = PDef.EC_STATE_INV;
      }
      else
      {
        if (si.Round.CoinsPlayed <= 0)
        {
          reply.ErrorCode = PDef.EC_SLOTS_COINSPLAYED_INSUF;
        }
        else
        {
          PSlot_Machine.Get(si.Round.MachineID,sm,reply.ErrorCode);
          if (reply.ErrorCode == PDef.EC_NONE)
          {
            PSlot_Reel.Get(sm.ReelTypeID,1,rand(sm.MaxLookupNum),r1,reply.ErrorCode);
            if (reply.ErrorCode == PDef.EC_NONE)
            {
              PSlot_Reel.Get(sm.ReelTypeID,2,rand(sm.MaxLookupNum),r2,reply.ErrorCode);
              if (reply.ErrorCode == PDef.EC_NONE)
              {
                PSlot_Reel.Get(sm.ReelTypeID,3,rand(sm.MaxLookupNum),r3,reply.ErrorCode);
              }
            }
            if (reply.ErrorCode == PDef.EC_NONE)
            {
              si.Round.LastReel1 = r1.ReelStopNum;
              si.Round.LastReel2 = r2.ReelStopNum;
              si.Round.LastReel3 = r3.ReelStopNum;

              TErrorCode pec;
              PSlot_Payoff.Get(sm.PayoffTypeID,
                               si.Round.CoinsPlayed,
                               r1.Symbol,
                               r2.Symbol,
                               r3.Symbol,
                               payoff,
                               pec);
              si.Wagered += si.Round.CoinsPlayed * sm.CoinSize;
              if (pec == PDef.EC_NONE)
              {
                reply.PayoffHiLiteNum = payoff.HiLitNum;
                si.Round.WinnerPaid = payoff.Amount;
                si.Round.Credits += payoff.Amount;
                if (payoff.Amount > 0)
                {
                  si.WonLost += payoff.Amount * sm.CoinSize;
                }
                else
                {
                  si.WonLost -= si.Round.CoinsPlayed * sm.CoinSize;
                }
              }
              else // not found
              {
                si.Round.WinnerPaid = 0;
                si.WonLost -= si.Round.CoinsPlayed * sm.CoinSize;
              }
              si.Round.Spin = true;
              reply.WinnerPaid = si.Round.WinnerPaid;
              reply.Credits = si.Round.Credits;
              EndRound(si,ec);
              if (ec == PDef.EC_NONE)
              {
                BeginRound(si,ec);
              }
            }
          }
        }
      }
      PSlot_Instance.UnLock(si,ec1);
    }
    PUser.UnLockUserObjects(user,wallet,cs,gs,ec2);
  }
  if (ec1 == PDef.EC_NONE && ec2 == PDef.EC_NONE)
  {
    Commit();
  }
  else
  {
    reply.ErrorCode = PDef.EC_SAVE_FAILURE;
    Rollback();
  }
}
//*****************************************************************
end;
