//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  inifile.h
//    Project Name:  Ogle                    
// Original Author:  Adam Krist
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/19/95    A.Krist    original baseline
//          9/20/95    M.Perry    reorg'd
//  2.0     8/25/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__inifile_h)
#define __inifile_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__CSTRING_H)
#include <cstring.h>
#endif

#if !defined(__STDLIB_H)
#include <stdlib.h>
#endif
//*******************************************************************
class TIniFile
{
protected:
  string BaseIniFileName;
  string FullIniFileName;
  string HDDrive;
  string HDDir;

  TIniFile() {}

public:
  // couple of helper functions.
  int boolToint(bool b) const;
  bool intTobool(int i) const;

  void SetHDDrive(const string &hddrive);
//  void SetHDDrive(const string &hddrive);
  const string &GetHDDrive() const;
  void SetHDDir(const string &hddir);
//  void SetHDDir(const string &hddir);
  const string &GetHDDir() const;
  void SetIniFile(const string &inifile);
//  void SetIniFile(const string &inifile);
  const string &GetIniFile() const;
  void SetFullIniFile();
  const string &GetFullIniFile() const;

  TIniFile(string const &hddrive,string const &hddir,string const &inifilename);
//  TIniFile(string fullpathname);
  TIniFile(const string &fullpathname);
  ~TIniFile() {}

  bool WriteString(const string &inisection,const string &entrykey,const string &newentry);
  bool WriteString(const string &inisection,int entrykey,const string &newentry);
//  bool WriteString(const char *inisection,const char *entrykey,const char *newentry);
//  bool WriteString(const char *inisection,int entrykey,const string &newentry);
//  bool WriteString(const char *inisection,const char *entrykey,const string &newentry);
//  bool WriteString(const string &inisection,const char *entrykey,const string &newentry);
//  bool WriteString(const string &inisection,const string &entrykey,const char *newentry);
//  bool WriteString(const string &inisection,const char *entrykey,const char *newentry);
//  bool WriteString(const char *inisection,const string &entrykey,const string &newentry);
//  bool WriteString(const char *inisection,const string &entrykey,const char *newentry);

  string ReadString(const string &inisection,const string &entrykey,const string &defaultstring) const;
//  string ReadString(string &inisection,const char *entrykey,const char *defaultstring);
  string ReadString(const string &inisection,int entrykey,const string &defaultstring) const;
//  string ReadString(const char *inisection,int entrykey,string &defaultstring);
//  string ReadString(const char *inisection,int entrykey,const char *defaultstring);
//  string ReadString(const char *inisection,const char *entrykey,const char *defaultstring);
//  string ReadString(const char *inisection,const char *entrykey,string &defaultstring);
//  string ReadString(string &inisection,int entrykey,const char *defaultstring);
//  string ReadString(const char *inisection,string &entrykey,const char *defaultstring);

  int ReadInt(const string &inisection,const string &entrykey,int defaultint) const;
  int ReadInt(const string &inisection,int entrykey,int defaultint) const;
//  int ReadInt(const char *inisection,const char *entrykey,int defaultint);
//  int ReadInt(string &inisection,const char *entrykey,int defaultint);

  bool WriteInt(const string &inisection,const string &entrykey,int newint);
  bool WriteInt(const string &inisection,int entrykey,int newint);
//  bool WriteInt(const char *inisection,const char *entrykey,int newint);
//  bool WriteInt(string &inisection,const char *entrykey,int newint);

  bool Writebool(const string &inisection,const string &entrykey,bool newbool);
//  bool Writebool(const char *inisection,const char *entrykey,bool newbool);
//  bool Writebool(const string &inisection,const char *entrykey,bool newbool);

  bool Readbool(const string &inisection,const string &entrykey,bool defbool) const;
//  bool Readbool(const char *inisection,const char *entrykey,bool defbool);
//  bool Readbool(const string &inisection,const char *entrykey,bool defbool);

  bool Flush();
  bool ClearSection(const string &inisection);
//  bool ClearSection(const char *inisection);

  bool ClearEntry(const string &inisection,const string &entrykey);
//  bool ClearEntry(string &inisection,const char *entrykey);
//  bool ClearEntry(const char *inisection,const char *entrykey);
};
//*******************************************************************
//************************** inlines ********************************
//*******************************************************************
inline int TIniFile::boolToint(bool b) const
{
  return (b?1:0);
}
//*******************************************************************
inline bool TIniFile::intTobool(int i) const
{
  return (i?true:false);
}
//*******************************************************************
inline void TIniFile::SetHDDrive(const string &hddrive)
{
  HDDrive = hddrive;
}
//*******************************************************************
//inline void TIniFile::SetHDDrive(const string &hddrive)
//{
//  HDDrive = hddrive;
//}
//*******************************************************************
inline const string &TIniFile::GetHDDrive() const
{
  return HDDrive;
}
//*******************************************************************
inline void TIniFile::SetHDDir(const string &hddir)
{
  HDDir = hddir;
}
//*******************************************************************
//inline void TIniFile::SetHDDir(const string &hddir)
//{
//  HDDir = hddir;
//}
//*******************************************************************
inline const string &TIniFile::GetHDDir() const
{
  return HDDir;
}
//*******************************************************************
inline void TIniFile::SetIniFile(const string &inifile)
{
  BaseIniFileName = inifile;
}
//*******************************************************************
//inline void TIniFile::SetIniFile(const string &inifile)
//{
//  BaseIniFileName = inifile;
//}
//*******************************************************************
inline const string &TIniFile::GetIniFile() const
{
  return BaseIniFileName;
}
//*******************************************************************
inline void TIniFile::SetFullIniFile()
{
  FullIniFileName = HDDrive + HDDir + BaseIniFileName;
}
//*******************************************************************
inline const string &TIniFile::GetFullIniFile() const
{
  return FullIniFileName;
}
//*******************************************************************
inline bool TIniFile::Writebool(const string &inisection,const string &entrykey,bool newbool)
{
  return WriteInt(inisection,entrykey,boolToint(newbool));
}
//*******************************************************************
//inline bool TIniFile::Writebool(const char *inisection,const char *entrykey,bool newbool)
//{
//  return WriteInt(inisection,entrykey,boolToint(newbool));
//}
//*******************************************************************
//inline bool TIniFile::Writebool(const string &inisection,const char *entrykey,bool newbool)
//{
//  return WriteInt(inisection,entrykey,boolToint(newbool));
//}
//*******************************************************************
inline bool TIniFile::Readbool(const string &inisection,const string &entrykey,bool defbool) const
{
  return intTobool(ReadInt(inisection,entrykey,boolToint(defbool)));
}
//*******************************************************************
//inline bool TIniFile::Readbool(const char *inisection,const char *entrykey,bool defbool)
//{
//  return intTobool(ReadInt(inisection,entrykey,boolToint(defbool)));
//}
//*******************************************************************
//inline bool TIniFile::Readbool(const string &inisection,const char *entrykey,bool defbool)
//{
//  return intTobool(ReadInt(inisection,entrykey,boolToint(defbool)));
//}
//*******************************************************************
inline bool TIniFile::Flush()
{
  return (bool)WritePrivateProfileString(NULL,
                                         NULL,
                                         NULL,
                                         GetFullIniFile().c_str());
}
//*******************************************************************
inline bool TIniFile::ClearSection(const string &inisection)
{
  return (bool)WritePrivateProfileString(inisection.c_str(),
                                         NULL,
                                         NULL,
                                         GetFullIniFile().c_str());
}
//*******************************************************************
//inline bool TIniFile::ClearSection(const char *inisection)
//{
//  return (bool)WritePrivateProfileString(inisection,
//                                         NULL,
//                                         NULL,
//                                         GetFullIniFile().c_str());
//}
//*******************************************************************
inline bool TIniFile::ClearEntry(const string &inisection,const string &entrykey)
{
  return (bool)WritePrivateProfileString(inisection.c_str(),
                                         entrykey.c_str(),
                                         NULL,
                                         GetFullIniFile().c_str());
}
//*******************************************************************
//inline bool TIniFile::ClearEntry(string &inisection,const char *entrykey)
//{
//  return (bool)WritePrivateProfileString(inisection.c_str(),
//                                         entrykey,
//                                         NULL,
//                                         GetFullIniFile().c_str());
//}
//*******************************************************************
//inline bool TIniFile::ClearEntry(const char *inisection,const char *entrykey)
//{
//  return (bool)WritePrivateProfileString(inisection,
//                                         entrykey,
//                                         NULL,
//                                         GetFullIniFile().c_str());
//}
//*******************************************************************
#endif
