//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  obasef.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     3/2/95     M.Perry     original baseline
//  2.0     8/20/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__obasef_h) // Sentry.
#define __obasef_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__ofmode_h)
#include "ofmode.h"
#endif

#if !defined(__omisc_h)
#include "omisc.h"
#endif

#if !defined(__oflags_h)
#include "oflags.h"
#endif

#if !defined(__omemmgr_h)
#include "omemmgr.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__ofilenam_h)
#include "ofilenam.h"
#endif

// start of obf stat flags
#define obfFIRST (fBit0)
#define obfHDBUFFERING (obfFIRST)
#define obfHDAVAIL ((WORD)(obfHDBUFFERING * 2))
#define obfOWNEDBYFILEMGR ((WORD)(obfHDAVAIL * 2))
#define obfSHAREABLE ((WORD)(obfOWNEDBYFILEMGR * 2))
#define obfLAST (obfSHAREABLE)
// end of obf stat flags

class TDirEntry;
class TOgleFileMgr;
//********************************************************************
typedef enum TagLoadResult
{
  lrCDReadFail,
  lrMemFail,
  lrHDWriteFail,
  lrHDBufferingOff,
  lrOk
}TLoadResult;
// *******************************************************************
class TOgleBaseFile
{
private:
//  TOgleBaseFile() {} // keep from being used.
  friend class TOgleFileMgr;
  friend class TFileList;
  friend class TFMFLData;

public:
  void huge *RamPtr;  // Conscious decision to make public, but be careful.

private:
  int16 ListNum;
  int16 OwnerCount;
  int16 HDImmedCount;
  int16 HDBGHiCount;
  int16 HDBGMedCount;
  int16 HDBGLoCount;
  int16 HDNoneCount;
  int16 RamImmedCount;
  int16 RamBGHiCount;
  int16 RamBGMedCount;
  int16 RamBGLoCount;
  int16 RamNoneCount;
  TFMStatus RamStatus;
  TFMStatus HDStatus;

protected:
  dword Offset;
//  TOgleApp *POgleApp;
  dword Size;
  string BaseFileName;
  string CDRomFullPathName;
  string HDFullPathName;
  TOgleWORDFlag StatFlags;
  uint AllocFlags;      // set by ctor.

public:
  void SetSize(dword size);
  dword GetSize();
  TOgleApp *GetOgleApp();
  TOgleFileMgr *GetFileMgr();
  bool GetHDBuffering() const;
  void SetHDBuffering(bool hdbuffering = true);
  void ClearHDBuffering();
  TOgleBaseFile();//v2.3
//  TOgleBaseFile(TOgleApp *papp);
  virtual ~TOgleBaseFile();
  bool ReadData(int32 ByteOffset,byte *PDest,int32 NumBytes);
  void huge *GetRamPtr();
  dword GetSizeRamPtr();
  bool IsRamAvail();
  bool AllocRamPtr();
  bool DeAllocRamPtr();
  bool DeleteHD();
  string &GetBaseName();
  string &GetHDName();
  bool GetHDAvail() const;
  bool IsHDAvail() const;
  void SetHDAvail();
  void ClearHDAvail();
  string &GetCDRomName();
  TFMStatus GetRamStatus();
  TFMStatus GetHDStatus();
  int16 GetListNum();
  TLoadResult LoadRam();
  TLoadResult LoadHD();
  bool UnLoadRam();
  bool UnLoadHD();
  bool UnLoadRamHD();

protected:
  void SetOffset(dword offset);
  dword GetOffset();
  void SetBaseName(const string &basename);
//  void SetCDRomName(string &cdname);
  void SetCDRomName(const string &cdname);
  void SetHDName(string &hdname);

private:
  void IncTotalHDAllocated(dword size);
  void DecTotalHDAllocated(dword size);

  // Start of FileMgr caller only !!
  void SetOwnedByFileMgr();
  void ClearOwnedByFileMgr();
  bool GetOwnedByFileMgr() const;
  bool IsOwnedByFileMgr() const;

  void IncOwnerCount();
  void DecOwnerCount();
  int16 GetOwnerCount();

  void IncHDImmedCount();
  void DecHDImmedCount();
  int16 GetHDImmedCount();

  void IncHDBGHiCount();
  void DecHDBGHiCount();
  int16 GetHDBGHiCount();

  void IncHDBGMedCount();
  void DecHDBGMedCount();
  int16 GetHDBGMedCount();

  void IncHDBGLoCount();
  void DecHDBGLoCount();
  int16 GetHDBGLoCount();

  void IncHDNoneCount();
  void DecHDNoneCount();
  int16 GetHDNoneCount();

  void IncRamImmedCount();
  void DecRamImmedCount();
  int16 GetRamImmedCount();

  void IncRamBGHiCount();
  void DecRamBGHiCount();
  int16 GetRamBGHiCount();

  void IncRamBGMedCount();
  void DecRamBGMedCount();
  int16 GetRamBGMedCount();

  void IncRamBGLoCount();
  void DecRamBGLoCount();
  int16 GetRamBGLoCount();

  void IncRamNoneCount();
  void DecRamNoneCount();
  int16 GetRamNoneCount();

  TFMStatus &SetRamStatus();
  TFMStatus &SetHDStatus();

  TFMStatus IncHDStatus(TFMStatus hdstatus);
  TFMStatus DecHDStatus(TFMStatus hdstatus);
  TFMStatus IncRamStatus(TFMStatus ramstatus);
  TFMStatus DecRamStatus(TFMStatus ramstatus);

  void IncStatus(TFMStatus hdstatus,TFMStatus ramstatus);
  void DecStatus(TFMStatus hdstatus,TFMStatus ramstatus);

  void SetShareable();
  void ClearShareable();
  bool IsShareable() const;
  bool GetShareable() const;

  void SetListNum(int16 num);
  // End of FileMgr caller only.
};
// ******************************************************************
//************************  inlines  ********************************
//*******************************************************************
inline void TOgleBaseFile::SetOffset(dword offset)
{
  Offset = offset;
}
//*******************************************************************
inline dword TOgleBaseFile::GetOffset()
{
  return Offset;
}
//*******************************************************************
inline void TOgleBaseFile::SetSize(dword size)
{
  Size = size;
}
//*******************************************************************
inline dword TOgleBaseFile::GetSize()
{
  return Size;
}
//*******************************************************************
inline TOgleApp *TOgleBaseFile::GetOgleApp()
{
  return TOgleApp::GetOgleApp();//POgleApp;
}
//*******************************************************************
inline TOgleFileMgr *TOgleBaseFile::GetFileMgr()
{
  return GetOgleApp()->GetFileMgr();
}
//*******************************************************************
inline bool TOgleBaseFile::GetHDBuffering() const
{
  return StatFlags.IsSet(obfHDBUFFERING);
}
//*******************************************************************
inline void TOgleBaseFile::SetHDBuffering(bool hdbuffering)
{
  if (hdbuffering) {StatFlags.Set(obfHDBUFFERING);}
  else             {StatFlags.Clear(obfHDBUFFERING);}
}
//*******************************************************************
inline void TOgleBaseFile::ClearHDBuffering()
{
  StatFlags.Clear(obfHDBUFFERING);
}
//*******************************************************************
inline void huge *TOgleBaseFile::GetRamPtr()
{
  return RamPtr;
}
//*******************************************************************
inline dword TOgleBaseFile::GetSizeRamPtr()
{
  return TOgleMemMgr::GetSizePtr(RamPtr);
}
//*******************************************************************
inline bool TOgleBaseFile::IsRamAvail()
{
  return (RamPtr != 0);
}
//*******************************************************************
inline bool TOgleBaseFile::AllocRamPtr()
{
  RamPtr = TOgleMemMgr::AllocPtr(AllocFlags,GetSize());
  return IsRamAvail();
}
//*******************************************************************
inline bool TOgleBaseFile::DeAllocRamPtr()
{
  if (!IsRamAvail()) {return true;}
  TOgleMemMgr::DeAllocPtr(RamPtr);
  RamPtr = 0;
  return true;
}
//*******************************************************************
inline bool TOgleBaseFile::DeleteHD()
{
  TOgleFileName::DeleteFile(GetHDName());
  ClearHDAvail();
  DecTotalHDAllocated(GetSize());
  return true;
}
//*******************************************************************
inline string &TOgleBaseFile::GetBaseName()
{
  return BaseFileName;
}
//*******************************************************************
inline string &TOgleBaseFile::GetHDName()
{
  return HDFullPathName;
}
//*******************************************************************
inline bool TOgleBaseFile::GetHDAvail() const
{
  return StatFlags.IsSet(obfHDAVAIL);
}
//*******************************************************************
inline bool TOgleBaseFile::IsHDAvail() const
{
  return StatFlags.IsSet(obfHDAVAIL);
}
//*******************************************************************
inline void TOgleBaseFile::SetHDAvail()
{
  StatFlags.Set(obfHDAVAIL);
}
//*******************************************************************
inline void TOgleBaseFile::ClearHDAvail()
{
  StatFlags.Clear(obfHDAVAIL);
}
//*******************************************************************
inline string &TOgleBaseFile::GetCDRomName()
{
  return CDRomFullPathName;
}
//*******************************************************************
inline TFMStatus TOgleBaseFile::GetRamStatus()
{
  return RamStatus;
}
//*******************************************************************
inline TFMStatus TOgleBaseFile::GetHDStatus()
{
  return HDStatus;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetListNum()
{
  return ListNum;
}
//*******************************************************************
inline bool TOgleBaseFile::UnLoadRam()
{
  return DeAllocRamPtr();
}
//*******************************************************************
inline bool TOgleBaseFile::UnLoadHD()
{
  return DeleteHD();
}
//*******************************************************************
inline bool TOgleBaseFile::UnLoadRamHD()
{
  bool ram = UnLoadRam();
  bool hd = UnLoadHD();
  return (ram && hd);
}
//*******************************************************************
inline void TOgleBaseFile::SetBaseName(const string &basename)
{
  BaseFileName = basename;
}
//*******************************************************************
inline void TOgleBaseFile::SetCDRomName(const string &cdname)
{
  CDRomFullPathName = cdname;
}
//*******************************************************************
inline void TOgleBaseFile::SetHDName(string &hdname)
{
  HDFullPathName = hdname;
}
//*******************************************************************
inline void TOgleBaseFile::SetOwnedByFileMgr()
{
  StatFlags.Set(obfOWNEDBYFILEMGR);
}
//*******************************************************************
inline void TOgleBaseFile::ClearOwnedByFileMgr()
{
  StatFlags.Clear(obfOWNEDBYFILEMGR);
}
//*******************************************************************
inline bool TOgleBaseFile::GetOwnedByFileMgr() const
{
  return StatFlags.IsSet(obfOWNEDBYFILEMGR);
}
//*******************************************************************
inline bool TOgleBaseFile::IsOwnedByFileMgr() const
{
  return StatFlags.IsSet(obfOWNEDBYFILEMGR);
}
//*******************************************************************
inline void TOgleBaseFile::IncOwnerCount()
{
  OwnerCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecOwnerCount()
{
  OwnerCount--;
  if (OwnerCount < 0) OwnerCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetOwnerCount()
{
  return OwnerCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncHDImmedCount()
{
  HDImmedCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecHDImmedCount()
{
  HDImmedCount--;
  if(HDImmedCount < 0) HDImmedCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetHDImmedCount()
{
  return HDImmedCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncHDBGHiCount()
{
  HDBGHiCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecHDBGHiCount()
{
  HDBGHiCount--;
  if (HDBGHiCount < 0) HDBGHiCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetHDBGHiCount()
{
  return HDBGHiCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncHDBGMedCount()
{
  HDBGMedCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecHDBGMedCount()
{
  HDBGMedCount--;
  if (HDBGMedCount < 0) HDBGMedCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetHDBGMedCount()
{
  return HDBGMedCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncHDBGLoCount()
{
  HDBGLoCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecHDBGLoCount()
{
  HDBGLoCount--;
  if (HDBGLoCount < 0) HDBGLoCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetHDBGLoCount()
{
  return HDBGLoCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncHDNoneCount()
{
  HDNoneCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecHDNoneCount()
{
  HDNoneCount--;
  if (HDNoneCount < 0) HDNoneCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetHDNoneCount()
{
  return HDNoneCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncRamImmedCount()
{
  RamImmedCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecRamImmedCount()
{
  RamImmedCount--;
  if (RamImmedCount < 0) RamImmedCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetRamImmedCount()
{
  return RamImmedCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncRamBGHiCount()
{
  RamBGHiCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecRamBGHiCount()
{
  RamBGHiCount--;
  if (RamBGHiCount < 0) RamBGHiCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetRamBGHiCount()
{
  return RamBGHiCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncRamBGMedCount()
{
  RamBGMedCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecRamBGMedCount()
{
  RamBGMedCount--;
  if (RamBGMedCount < 0) RamBGMedCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetRamBGMedCount()
{
  return RamBGMedCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncRamBGLoCount()
{
  RamBGLoCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecRamBGLoCount()
{
  RamBGLoCount--;
  if (RamBGLoCount < 0) RamBGLoCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetRamBGLoCount()
{
  return RamBGLoCount;
}
//*******************************************************************
inline void TOgleBaseFile::IncRamNoneCount()
{
  RamNoneCount++;
}
//*******************************************************************
inline void TOgleBaseFile::DecRamNoneCount()
{
  RamNoneCount--;
  if (RamNoneCount < 0) RamNoneCount = 0;
}
//*******************************************************************
inline int16 TOgleBaseFile::GetRamNoneCount()
{
  return RamNoneCount;             
}
//*******************************************************************
inline TFMStatus &TOgleBaseFile::SetRamStatus()
{
  if (GetRamImmedCount() > 0) RamStatus = TFileMode::fm_Immed;
  else if (GetRamBGHiCount() > 0) RamStatus = TFileMode::fm_BG_Hi;
  else if (GetRamBGMedCount() > 0) RamStatus = TFileMode::fm_BG_Med;
  else if (GetRamBGLoCount() > 0) RamStatus = TFileMode::fm_BG_Lo;
  else RamStatus = TFileMode::fm_None;
  return RamStatus;
}
//*******************************************************************
inline TFMStatus &TOgleBaseFile::SetHDStatus()
{
  if (GetHDImmedCount() > 0) HDStatus = TFileMode::fm_Immed;
  else if (GetHDBGHiCount() > 0) HDStatus = TFileMode::fm_BG_Hi;
  else if (GetHDBGMedCount() > 0) HDStatus = TFileMode::fm_BG_Med;
  else if (GetHDBGLoCount() > 0) HDStatus = TFileMode::fm_BG_Lo;
  else HDStatus = TFileMode::fm_None;
  return HDStatus;
}
//*******************************************************************
inline void TOgleBaseFile::SetShareable()
{
  StatFlags.Set(obfSHAREABLE);
}
//*******************************************************************
inline void TOgleBaseFile::ClearShareable()
{
  StatFlags.Clear(obfSHAREABLE);
}
//*******************************************************************
inline bool TOgleBaseFile::IsShareable() const
{
  return StatFlags.IsSet(obfSHAREABLE);
}
//*******************************************************************
inline bool TOgleBaseFile::GetShareable() const
{
  return StatFlags.IsSet(obfSHAREABLE);
}
//*******************************************************************
inline void TOgleBaseFile::SetListNum(int16 num)
{
  ListNum = num;
}
//*******************************************************************
#endif
