//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name: obmp.h
//    Project Name: Ogle
// Original Author: Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0    4/28/95     M.Perry     original baseline
//  2.0    8/21/97     M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__obmp_h) 
#define __obmp_h

#if !defined(__ogr_h)
#include "ogr.h"
#endif

#if !defined(__opal_h)
#include "opal.h"
#endif
//*******************************************************************
class TBitmapFileHeader
{
public:
  BITMAPFILEHEADER BitMapFileHeader;

public:
  TBitmapFileHeader();
  virtual ~TBitmapFileHeader();
  operator const BITMAPFILEHEADER();
  BITMAPFILEHEADER &operator=(BITMAPFILEHEADER &bitmapfileheader);
};
//*******************************************************************
class TBitmapInfo
{
protected:
  BITMAPINFO *PBitMapInfo;

public:
  TBitmapInfo();
  TBitmapInfo(int16 numquads);
  TBitmapInfo(BITMAPINFO *pbitmapinfo);
  ~TBitmapInfo();
  bool CreateBITMAPINFO(int16 numquads);
  bool DestroyBITMAPINFO();
  BITMAPINFO *GetBITMAPINFO();
  operator const BITMAPINFO();
  operator const BITMAPINFO *();
  bool SetupBitmapInfoHeader(int16 numquads);
  TBitmapInfo &operator=(TPal &pal);
  int16 LookupNumQuads(int16 bitcount);
  TBitmapInfo &operator=(BITMAPINFO &bitmapinfo);
  BITMAPINFOHEADER *GetHeader();
  RGBQUAD *GetQuads();
  bool SetAllQuads(RGBQUAD rgbquad);
  int16 GetNumQuads();
  int32 GetWidth();
  void SetWidth(int32 width);
  int32 GetHeight();
  void SetHeight(int32 height);
  word GetNumColorPlanes();
  word GetBitCount();
  dword GetCompression();
  bool IsCompressed();
  bool IsTopDown();
  void SetTopDown(bool topdown);
  dword GetSizeStruct();
};
//*******************************************************************
class TBmp:public TGr
{
public:
  HBITMAP hBitmap;
  TBitmapFileHeader *PBitmapFileHeader;
  TBitmapInfo *PBitmapInfo;

public:
  TBmp();
  TBmp(TOgleModeWindow *pmw,const TFileMode &fm,bool AutoRead = true,bool KeepBaseFile = true);
  TBmp(TOgleModeWindow *pmw);
  virtual ~TBmp();
  virtual string AddExt();
  bool IsHBITMAPValid();
  operator HBITMAP() const;
  virtual bool CreateHBITMAP();
  virtual bool DeleteHBITMAP();
  int32 GetHeight();
  int32 GetWidth();
  word GetNumColorPlanes();
  word GetBitCount();
  byte *FindBits();
  virtual bool BltTo(TOgleVideoDC &vdc,TOgleRect &clipvrect);
  virtual bool BltTo(TGr &gr,TOgleRect &clipvrect);
protected:
  virtual bool ReadLocalHeader();
  virtual bool WriteLocalHeader();
  virtual bool ReadLocal();
  virtual bool WriteLocal();
};
//*******************************************************************
//**************** TBitmapFileHeader inlines  ***********************
//*******************************************************************
inline TBitmapFileHeader::operator const BITMAPFILEHEADER()
{
  return BitMapFileHeader;
}
//*******************************************************************
//******************  TBitmapInfo inlines ***************************
//*******************************************************************
inline BITMAPINFO *TBitmapInfo::GetBITMAPINFO()
{
  return PBitMapInfo;
}
//*******************************************************************
inline TBitmapInfo::operator const BITMAPINFO()
{
  return *PBitMapInfo;
}
//*******************************************************************
inline TBitmapInfo::operator const BITMAPINFO *()
{
  return PBitMapInfo;
}
//*******************************************************************
inline BITMAPINFOHEADER *TBitmapInfo::GetHeader()
{
  return (BITMAPINFOHEADER *)PBitMapInfo;
}
//*******************************************************************
inline RGBQUAD *TBitmapInfo::GetQuads()
{
  return (RGBQUAD *)((byte *)PBitMapInfo + (word)GetHeader()->biSize);
}
//*******************************************************************
inline int16 TBitmapInfo::GetNumQuads()
{
  if (GetHeader() == NULL) {return 0;}
  return int16(GetHeader()->biClrUsed);
}
//*******************************************************************
inline int32 TBitmapInfo::GetWidth()
{
  if (GetHeader() == NULL) {return 0;}
  return GetHeader()->biWidth;
}
//*******************************************************************
inline void TBitmapInfo::SetWidth(int32 width)
{
  if (GetHeader() != NULL) {GetHeader()->biWidth = width;}
}
//*******************************************************************
inline int32 TBitmapInfo::GetHeight()
{
  if (GetHeader() == NULL) {return 0;}
  return GetHeader()->biHeight;
}
//*******************************************************************
inline void TBitmapInfo::SetHeight(int32 height)
{
  if (GetHeader() != NULL) {GetHeader()->biHeight = height;}
}
//*******************************************************************
inline word TBitmapInfo::GetNumColorPlanes()
{
  if (GetHeader() == NULL) {return 1;}
  return GetHeader()->biPlanes;
}
//*******************************************************************
inline word TBitmapInfo::GetBitCount()
{
  if (GetHeader() == NULL) {return 8;}
  return GetHeader()->biBitCount;
}
//*******************************************************************
inline dword TBitmapInfo::GetCompression()
{
  if (GetHeader() == NULL) {return BI_RGB;}
  return GetHeader()->biCompression;
}
//*******************************************************************
inline bool TBitmapInfo::IsCompressed()
{
  return bool(GetCompression() != BI_RGB);
}
//*******************************************************************
inline bool TBitmapInfo::IsTopDown()
{
  if (GetHeader() == NULL) {return true;}
  return bool(GetHeader()->biHeight < 0);
}
//*******************************************************************
inline void TBitmapInfo::SetTopDown(bool topdown)
{
  if (topdown && !IsTopDown())
  {
    SetHeight(-labs(GetHeight()));
  }
  else if (!topdown && IsTopDown())
  {
    SetHeight(labs(GetHeight()));
  }
}
//*******************************************************************
inline dword TBitmapInfo::GetSizeStruct()
{
  if (PBitMapInfo == NULL) {return 0;}
  return dword(sizeof(BITMAPINFO) + (GetNumQuads() - 1)*sizeof(RGBQUAD));
}
//*******************************************************************
//*********************** TBmp inlines  *****************************
//*******************************************************************
inline bool TBmp::IsHBITMAPValid()
{
  return bool(hBitmap != NULL);
}
//*******************************************************************
inline TBmp::operator HBITMAP() const
{
  return hBitmap;
}
//*******************************************************************
inline int32 TBmp::GetHeight()
{
  if (PBitmapInfo == NULL) {return 0;}
  return PBitmapInfo->GetHeight();
}
//*******************************************************************
inline int32 TBmp::GetWidth()
{
  if (PBitmapInfo == NULL) {return 0;}
  return PBitmapInfo->GetWidth();
}
//*******************************************************************
inline word TBmp::GetNumColorPlanes()
{
  if (PBitmapInfo == NULL) {return 1;}
  return PBitmapInfo->GetNumColorPlanes();
}
//*******************************************************************
inline word TBmp::GetBitCount()
{
  if (PBitmapInfo == NULL) {return 8;}
  return PBitmapInfo->GetBitCount();
}
//*******************************************************************
#endif

