//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ocurmgr.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0      2/23/96   M.Perry    original baseline
//  2.0      8/26/97   M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__ocurmgr_h)
#define __ocurmgr_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__ocursors_rh)
#include "ocursors.rh"
#endif

#if !defined(__ogeo_h)
#include "ogeo.h"
#endif

class OGLEOWLCLASS TOgleApp;
class OGLEOWLCLASS TModule;
//*******************************************************************
typedef enum TagCursorTypes
{
  ctHiPri = 0,
  ctFileMgr = 1,
  ctAction = 2,
  ctArea = 3,
  ctNormal = 4
}TCursorTypes;
#define MAX_CURSOR_TYPES (ctNormal + 1)
//*******************************************************************
class TCursorData
{
public:
  TModule *PModule;
  TResId ResId;

public:
  TCursorData();
  TCursorData(TModule *pmod,TResId ri);
  ~TCursorData();
  void Set(TModule *pmod,TResId ri);
};
//*******************************************************************
class TCursorMgr
{
private:
  TOgleApp *POgleApp;

public:
  TCursorData CursorData[MAX_CURSOR_TYPES];

public:
  TCursorMgr(TOgleApp *papp);
  ~TCursorMgr();
  void ModeDeActivated();
  bool CheckIndex(TCursorTypes type);
  void SetCursor();
  void SetCursor(TCursorTypes CT,TModule *pmodule,TResId resId,bool force = true);//false);
  void SetCursor(TCursorTypes CT,TCursorData &cd,bool force = true);//false);
  TCursorData &GetCursor(TCursorTypes CT);
  TCursorData &GetCursor();
};
//*******************************************************************
//******************** TCursorData inlines  *************************
//*******************************************************************
inline TCursorData::TCursorData()
{
  PModule = 0;
  ResId = 0;
}
//*******************************************************************
inline TCursorData::TCursorData(TModule *pmod,TResId ri)
{
  PModule = pmod;
  ResId = ri;
}
//*******************************************************************
inline TCursorData::~TCursorData()
{
}
//*******************************************************************
inline void TCursorData::Set(TModule *pmod,TResId ri)
{
  PModule = pmod;
  ResId = ri;
}
//*******************************************************************
//******************** TCursorMgr inlines ***************************
//*******************************************************************
inline void TCursorMgr::ModeDeActivated()
{
  SetCursor(ctAction,0,0);
  SetCursor(ctArea,0,0);
}
//*******************************************************************
inline bool TCursorMgr::CheckIndex(TCursorTypes type)
{
  return ((type >= ctHiPri) && (type <= ctNormal));
}
//*******************************************************************
inline TCursorData &TCursorMgr::GetCursor(TCursorTypes CT)
{
  return CursorData[CT];
}
//*******************************************************************
inline TCursorData &TCursorMgr::GetCursor()
{
  if (GetCursor(ctHiPri).ResId != 0) {return GetCursor(ctHiPri);}
  if (GetCursor(ctFileMgr).ResId != 0) {return GetCursor(ctFileMgr);}
  if (GetCursor(ctAction).ResId != 0) {return GetCursor(ctAction);}
  if (GetCursor(ctArea).ResId != 0) {return GetCursor(ctArea);}
  return GetCursor(ctNormal);
}
//*******************************************************************
#endif

