//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  odetlist.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     5/9/96     M.Perry    original baseline
//  2.0     8/26/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined (__odetlist_h)
#define __odetlist_h

#if !defined(__olist_h)
#include "olist.h"
#endif

#if !defined(__odet_h)
#include "odet.h"
#endif

#if defined(GED_COMPILE) && !defined(__odetlscr_h)
  #include "odetlscr.h"
#endif

#if defined(GED_COMPILE)
class OGLEOWLCLASS TDetListDlg;
#endif

class TDirEntry;
//*******************************************************************
class TDetList:public TOgleList
{
#if defined(GED_COMPILE)
protected:
  string HDDrive;
  string HDDir;
  TDetListScript *PDetListScript;
  void DoDlg(TWindow *pwin);
  void CreateDetListScript(string &hddrive,string &hddir,string &appname);
  void DestroyDetListScript();
#endif

public:
  TDetList();
  virtual ~TDetList();
  TDirEntry *GetDirEntry(string &pathname);
  int16 GetNumDets();
  TDet *GetDetPtr(int16 detindex);
  TDet *GetDetPtr(const string &detfname);
  bool IsDetAdded(const string &detfname);
  bool AddDet(TDet *pdet);
  bool RemoveDet(int16 detindex);
  bool IncDetUsage(const string &detfname);
  bool DecDetUsage(const string &detfname);
  void ReleaseUnUsed();
  #ifdef GED_COMPILE
  TDetListScript *GetDetListScript();
  void Edit(TWindow *pwin,string &hddrive,string &hddir,string &appname);
  string &GetSource();
  void CreateAll(TWindow *pwin);
  void CreateAll(TDetListDlg *pdlg);
  int16 GetNumNames();
  void Create(TWindow *pwin,int16 index);
  void Create(TDetListDlg *pdlg,int16 index);
  void AddName(string &name);
  void RemoveName(int16 index);
  void EditDes(TWindow *pwin,int16 index);
  void SetDest(string &dest);
  string &GetDest();
  string &GetName(int16 index);
  #endif
};
//*******************************************************************
//******************** inlines **************************************
//*******************************************************************
inline int16 TDetList::GetNumDets()
{
  return int16(LastAddedElementNum + 1);
}
//*******************************************************************
inline TDet *TDetList::GetDetPtr(int16 detindex)
{
  return ((TDet *)GetElementPtr(detindex));
}
//*******************************************************************
inline TDet *TDetList::GetDetPtr(const string &detfname)
{
  return ((TDet *)GetElementPtr(detfname));
}
//*******************************************************************
inline bool TDetList::IsDetAdded(const string &detfname)
{
  return (GetDetPtr(detfname) != NULL);
}
//*******************************************************************
inline bool TDetList::AddDet(TDet *pdet)
{
  return AddElement(pdet);
}
//*******************************************************************
inline bool TDetList::RemoveDet(int16 detindex)
{
  return RemoveElement(detindex);
}
//*******************************************************************
inline bool TDetList::IncDetUsage(const string &detfname)
{
  TDet *pdet = GetDetPtr(detfname);
  if (pdet == NULL) {return false;}
  pdet->IncUsage();
  return true;
}
//*******************************************************************
inline bool TDetList::DecDetUsage(const string &detfname)
{
  TDet *pdet = GetDetPtr(detfname);
  if (pdet == NULL) {return false;}
  pdet->DecUsage();
  return true;
}
//*******************************************************************
#if defined(GED_COMPILE)
inline TDetListScript *TDetList::GetDetListScript()
{
  return PDetListScript;
}
//*******************************************************************
inline string &TDetList::GetSource()
{
  return PDetListScript->GetSource();
}
//*******************************************************************
inline int16 TDetList::GetNumNames()
{
  return PDetListScript->GetNumNames();
}
//*******************************************************************
inline void TDetList::AddName(string &name)
{
  PDetListScript->AddName(name);
}
//*******************************************************************
inline void TDetList::RemoveName(int16 index)
{
  PDetListScript->RemoveName(index);
}
//*******************************************************************
inline void TDetList::SetDest(string &dest)
{
  PDetListScript->SetDest(dest);
}
//*******************************************************************
inline string &TDetList::GetDest()
{
  return PDetListScript->GetDest();
}
//*******************************************************************
inline string &TDetList::GetName(int16 index)
{
  return PDetListScript->GetName(index);
}
#endif
//*******************************************************************
#endif
