//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  odetscr.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/27/96    M.Perry    original baseline
//  2.0     8/26/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__odetscr_h)
#define __odetscr_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__inifile_h)
#include "inifile.h"
#endif

#define MAX_EXTS 25
#define MAX_INCS 200
#define MAX_EXCS 25
#define MAX_EXC_NAMES 150                            
//*******************************************************************
class TDetExcData
{
public:
  string Name;
  bool Recurse;
  TDetExcData();
  ~TDetExcData();
  TDetExcData(TDetExcData &ded);
  TDetExcData(const TDetExcData &ded);
  TDetExcData &operator=(TDetExcData &ded);
  TDetExcData &operator=(const TDetExcData &ded);
  bool ContainsDir(string &dir);
};
//*******************************************************************
class TDetIncData
{
public:
  int16 NumExts;
  string Exts[MAX_EXTS];
  string Name;
  bool Recurse;
  TDetIncData();
  TDetIncData(TDetIncData &did);
  TDetIncData(const TDetIncData &did);
  ~TDetIncData();

  TDetIncData &operator=(TDetIncData &did);
  TDetIncData &operator=(const TDetIncData &did);
  bool CheckIndex(int16 index) {return ((index >=0) && (index < MAX_EXTS));}
  string GetExt(int16 index);
  void SetExt(int16 index,string &ext);
  bool AddExt(string &ext);
  bool RemoveExt(int16 index);
  bool ContainsExt(string &ext);
  bool ContainsDir(string &dir);
};
//*******************************************************************
class TDetExcNameData
{
public:
  string Name;
  TDetExcNameData();
  ~TDetExcNameData();
  TDetExcNameData(TDetExcNameData &dend);
  TDetExcNameData(const TDetExcNameData &dend);
  TDetExcNameData &operator=(TDetExcNameData &dend);
  TDetExcNameData &operator=(const TDetExcNameData &dend);
  bool IsMatch(string &dir,string &name);
};
//*******************************************************************
class TDetScript
{
public:
  bool Initialized;
  string HDDrive;
  string HDDir;
  string FileName;
  string FileNameandExt;
  string &GetFileName() {return FileName;}
  TDetScript(string &hddrive,string &hddir,string &filename);
  virtual ~TDetScript();
  int16 NumIncs;
  int16 GetNumIncs() {return NumIncs;}
  bool AddInc(TDetIncData &did);
  bool RemoveInc(int16 index);
  bool CheckIncIndex(int16 index) {return ((index >= 0) && (index < MAX_INCS));}
  string GetIncSectionStr(int16 index);
  string GetExcSectionStr(int16 index);
  TDetIncData DetIncData[MAX_INCS];
  bool SetIncData(int16 index,TDetIncData &did)
  {
    if (!CheckIncIndex(index)) {return false;}
    DetIncData[index] = did;
    return true;
  }
  TDetIncData *GetIncData(int16 index)
  {
    if (!CheckIncIndex(index)) {return NULL;}
    return &(DetIncData[index]);
  }
  int16 NumExcs;
  int16 GetNumExcs() {return NumExcs;}
  bool AddExc(TDetExcData &ded);
  bool RemoveExc(int16 index);
  bool CheckExcIndex(int16 index) {return ((index >= 0) && (index < MAX_EXCS));}
  TDetExcData DetExcData[MAX_EXCS];
  bool SetExcData(int16 index,TDetExcData &ded)
  {
    if (!CheckExcIndex(index)) {return false;}
    DetExcData[index] = ded;
    return true;
  }
  TDetExcData *GetExcData(int16 index)
  {
    if (!CheckExcIndex(index)) {return NULL;}
    return &(DetExcData[index]);
  }

  int16 NumExcNames;
  int16 GetNumExcNames() {return NumExcNames;}
  TDetExcNameData DetExcNameData[MAX_EXC_NAMES];
  bool AddExcName(TDetExcNameData &dend);
  bool RemoveExcName(int16 index);
  string GetExcNameSectionStr(int16 index);
  bool ReadExcNames();
  bool WriteExcNames();
  bool CheckExcNameIndex(int16 index) {return ((index >= 0) && (index < MAX_EXC_NAMES));}
  bool SetExcNameData(int16 index,TDetExcNameData &dend)
  {
    if (!CheckExcNameIndex(index)) {return false;}
    DetExcNameData[index] = dend;
    return true;
  }
  TDetExcNameData *GetExcNameData(int16 index)
  {
    if (!CheckExcNameIndex(index)) {return NULL;}
    return &(DetExcNameData[index]);
  }

  TIniFile *PScript;
  bool Read();
  bool Write();
  bool Initialize();
  bool DeInitialize();
protected:
  bool ReadInfo();
  bool ReadIncs();
  bool ReadExcs();
  bool WriteInfo();
  bool WriteIncs();
  bool WriteExcs();
};
//*******************************************************************
#endif

