//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  odll.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     7/7/96     M.Perry    original baseline
//  2.0     8/23/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__odll_h)
#define __odll_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif
//*******************************************************************
class TOgleDll
{
protected:
  string Name;
  HINSTANCE hInst;
  bool LoadErrorFatal;
  int16 RefCount;
  void IncRefCount();
  void DecRefCount();
  int16 GetRefCount();

public:
  TOgleDll(const char *dllName,bool loaderrorfatal = true);
  virtual ~TOgleDll();

  bool Initialized;
  bool IsInitialized();
  bool Initialize();
  bool DeInitialize();
  void *GetProcAddress(string &functName);
  void *GetProcAddress(const char *functName);
  void *GetProcAddress(WORD ordinalNum);
  virtual bool SetupProcs();
  virtual bool CleanupProcs();
  bool LoadLibrary();
  bool FreeLibrary();
  string GetInstanceError();
  bool CheckInstance();
  int16 GetSysUsage();
  static int16 GetSysUsage(HINSTANCE hinst);
  bool SysReLoad();

protected:
  // the following are wrappers around initialize/deinitialize using refcount for control.
  virtual bool Load();
  virtual bool UnLoad();
  virtual bool HandleInstanceError();
};
//*******************************************************************
//*************************  inlines  *******************************
//*******************************************************************
inline void TOgleDll::IncRefCount()
{
  RefCount++;
}
//*******************************************************************
inline void TOgleDll::DecRefCount()
{
  RefCount--;
  if (RefCount < 0) {RefCount = 0;}
}
//*******************************************************************
inline int16 TOgleDll::GetRefCount()
{
  return RefCount;
}
//*******************************************************************
inline bool TOgleDll::IsInitialized()
{
  return Initialized;
}
//*******************************************************************
inline void *TOgleDll::GetProcAddress(string &functName)
{
  return ::GetProcAddress(hInst,functName.c_str());
}
//*******************************************************************
inline void *TOgleDll::GetProcAddress(const char *functName)
{
  return ::GetProcAddress(hInst,functName);
}
//*******************************************************************
inline void *TOgleDll::GetProcAddress(WORD ordinalNum)
{
  return ::GetProcAddress(hInst,MAKEINTRESOURCE(ordinalNum));
}
//*******************************************************************
inline bool TOgleDll::CheckInstance()
{
  #if defined(BI_PLAT_WIN16)
  if (hInst < HINSTANCE_ERROR) {return false;}
  #else
  if (hInst == NULL) {return false;}
  #endif
  return true;
}
//*******************************************************************
inline int16 TOgleDll::GetSysUsage()
{
  #if defined(BI_PLAT_WIN16)
  return ::GetModuleUsage(hInst);
  #else
  return 1; // maybe should do a ::GetModuleHandle() to see if loaded.
  #endif
}
//*******************************************************************
inline int16 TOgleDll::GetSysUsage(HINSTANCE hinst)
{
  #if defined(BI_PLAT_WIN16)
  return ::GetModuleUsage(hinst);
  #else
  return 1; // maybe should do a ::GetModuleHandle() to see if loaded.
  #endif
}
//*******************************************************************
#endif

