//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ogambler.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/3/96    M.Perry      changed dialog.
//  2.0     8/23/97   M.Perry      32-bit conversion.
//********************************************************************
#if !defined(__ogambler_h)
#define __ogambler_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(OWL_LISTBOX_H)
#include <owl\listbox.h>
#endif

#if !defined(OWL_DIALOG_H)
#include <owl\dialog.h>
#endif
//#include <owl\edit.h>
//#include <owl\checkbox.h>
//#include <owl\radiobut.h>
//#include <owl\validate.h>
#if !defined(__ogambler_rh)
#include "ogambler.rh"
#endif

class TDialog;
class TListBox;
class TStatic;
class TEdit;
class TButton;
class TRadioButton;
class TPlayerMgr;
//*******************************************************************
class OGLEOWLCLASS TGamblerDialog : public TDialog
{
private:
  int16 ActiveStart;
  int16 NameStart;
  int16 SexStart;
  int16 BankrollStart;
  int16 BankrollEnd;
  int16 NumPlayersAllowed;
  int16 *PlayerNumsTT;
  int16 LastPlayerIndex;
  TPlayerMgr *PPlayerMgr;
  TButton *PActDeAct_Btn;
  string ActivateCaption;
  string DeActivateCaption;
  TButton *PCreate_Btn;
  TButton *PDelete_Btn;
  TButton *PStatistics_Btn;
  TButton *PDone_Btn;
  TListBox* PGambler_LB;

public:
  dword HelpID;

public:
  TGamblerDialog(TWindow *parent,TResId resId,TPlayerMgr *pplayermgr,dword helpid);
  virtual ~TGamblerDialog();

  void CmActDeAct();
  void CmDone();
  void CmCreate();
  void CmDelete();
  void CmStatistics();
  void CmHelp();

  void CmLBSelChange();
  void CmLBKillFocus();
  void CmLBDblClk();
  virtual void SetupWindow();

private:
  bool CheckPNTTIndex(int16 index);
  bool CheckPNIndex(int16 index);
  int16 GetSelPlayerNum();
  void HandleActDeActStatus();
  void HandleCreateStatus();
  void HandleDeleteStatus();
  void HandleStatisticsStatus();
  void HandleButtonStatus();
  void ReLoadLB();
  void LoadLB();
  void UnLoadLB();
  void ClearPlayerNums();
  void LoadPlayerNums();
  string StringOfSpaces(int16 numspaces);
  string MakeLBString(int16 playernum);

  DECLARE_RESPONSE_TABLE(TGamblerDialog);
};
//*******************************************************************
class OGLEOWLCLASS TGamblerDeleteDialog: public TDialog
{
private:
  int16 PlayerNum;
  TPlayerMgr *PPlayerMgr;
  TButton *PDelete_Btn;
  TButton *PCancel_Btn;
  TButton *PHelp_Btn;
  TStatic *PName_STB;
  string AboutToDeleteStr;

public:
  dword HelpID;

public:
  TGamblerDeleteDialog(TWindow *parent,TPlayerMgr *pplayermgr,int16 playernum,dword helpid);
  virtual ~TGamblerDeleteDialog();
  virtual void SetupWindow();
  void CmDelete();
  void CmCancel();
  void CmHelp();

  DECLARE_RESPONSE_TABLE(TGamblerDeleteDialog);
};
//*******************************************************************
class OGLEOWLCLASS TGamblerCreateDialog:public TDialog
{
private:
  TPlayerMgr *PPlayerMgr;
  TButton *PCreate_Btn;
  TButton *PCancel_Btn;
  TButton *PHelp_Btn;
  TEdit *PName_ETB;
  TRadioButton *P200_RBtn;
  TRadioButton *P500_RBtn;
  TRadioButton *P2500_RBtn;
  TRadioButton *PMale_RBtn;
  TRadioButton *PFemale_RBtn;
  string OoopsStr;
  string MustEnterNameStr;
  string EnteredUsedNameStr;
  string UnknownErrorStr;

public:
  dword HelpID;

public:
  TGamblerCreateDialog(TWindow *parent,TPlayerMgr *pplayermgr,dword helpid);
  virtual ~TGamblerCreateDialog();
  virtual void SetupWindow();
  void CmMale();
  void CmFemale();
  void Cm200();
  void Cm500();
  void Cm2500();
  void CmCreate();
  void CmCancel();
  void CmHelp();

private:
  string GetName();
  bool IsNameOk(string name);
  bool GetSex();
  int16 GetBankroll();

  DECLARE_RESPONSE_TABLE(TGamblerCreateDialog);
};
//*******************************************************************
class OGLEOWLCLASS TGamblerMsgDlg:public TDialog
{
private:
  TStatic *PMsg_STB;
  string strTitle;
  string strMsg;

public:
  TGamblerMsgDlg(TWindow *pwin,string title,string msg);
  virtual ~TGamblerMsgDlg();

protected:
  virtual void SetupWindow();
};
//*******************************************************************
//********************* TGamblerDialog inlines **********************
//*******************************************************************
inline bool TGamblerDialog::CheckPNTTIndex(int16 index)
{
  if ((index >= 0) && (index < NumPlayersAllowed)) {return true;}
  return false;
}
//*******************************************************************
inline bool TGamblerDialog::CheckPNIndex(int16 index)
{
  if ((index >=0) && (index < NumPlayersAllowed)) {return true;}
  return false;
}
//*******************************************************************
inline int16 TGamblerDialog::GetSelPlayerNum()
{
  int16 index = int16(PGambler_LB->GetSelIndex());
  if (!CheckPNTTIndex(index)) {return -1;}
  return PlayerNumsTT[index];
}
//*******************************************************************
inline void TGamblerDialog::HandleButtonStatus()
{
  HandleActDeActStatus();
  HandleCreateStatus();
  HandleDeleteStatus();
  HandleStatisticsStatus();
}
//*******************************************************************
#endif

