//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ogeo.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     10/26/95   M.Perry    original baseline
//  2.0     8/21/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__ogeo_h)
#define __ogeo_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if defined(OGLEOWLVER5)
  #if !defined(WINSYS_GEOMETRY_H)
    #include <winsys\geometry.h>
  #endif
#else
  // ??? OSL_GEOMETRY_H ??? 
  #include <osl\geometry.h>
#endif
//*******************************************************************
class TTriplet
{
public:
  int16 X;
  int16 Y;
  int16 Z;
  TTriplet();
  TTriplet(int16 x,int16 y,int16 z);
  ~TTriplet();
  TTriplet(const TTriplet &t);
  int16 GetX();
  int16 GetY();
  int16 GetZ();

  void SetX(int16 x);
  void SetY(int16 y);
  void SetZ(int16 z);
  TTriplet operator+(const TTriplet &triplet) const;
  TTriplet operator-(const TTriplet &triplet) const;
  TTriplet &operator+=(const TTriplet &triplet);
  bool operator!=(const TTriplet &triplet);
  bool operator==(const TTriplet &triplet);
  TTriplet &operator=(TTriplet const &triplet);
//  TTriplet &operator=(TTriplet triplet);
};
//*******************************************************************
class TOglePoint
{
public:
  int16 x;
  int16 y;
  TOglePoint();
  TOglePoint(int16 newx,int16 newy);
  TOglePoint(const TOglePoint &point);
  TOglePoint(const tagPOINT &point);
  TOglePoint(const TPoint &point);
  TOglePoint(uint32 u32);
  TOglePoint(const TSize &size);

  bool operator==(const TOglePoint &pt);
  bool operator!=(const TOglePoint &pt);
  bool operator==(const TPoint &pt);
  bool operator!=(const TPoint &pt);
  bool operator==(const tagPOINT &pt);
  bool operator!=(const tagPOINT &pt);

  TOglePoint OffsetBy(int16 dx,int16 dy) const;
  TOglePoint operator +(const TSize& size) const;
  TSize operator -(const TPoint& point) const;
  TSize operator -(const TOglePoint &point) const;
  TOglePoint operator -(const TSize &size) const;
  TOglePoint operator -() const;

  TOglePoint &Offset(int16 dx,int16 dy);
  TOglePoint &operator+=(const TSize &size);
  TOglePoint &operator-=(const TSize &size);

  operator const TPoint();
};
//*******************************************************************
class TOgleRect//:public tagRECT
{
public:
  int16 left;
  int16 top;
  int16 right;
  int16 bottom;

  TOgleRect(int16 leftx,int16 topy,int16 rightx,int16 bottomy);
  TOgleRect(const TPoint &tl,const TPoint &br);
  TOgleRect(const TOglePoint &tl,const TOglePoint &br);
  TOgleRect();
  TOgleRect(TRect const &rect);
  TOgleRect(TOgleRect &rect);
  TOgleRect(TPoint const &pt,TSize const &size,bool dummy);
  ~TOgleRect();
  void Set(int16 leftx,int16 topy,int16 rightx,int16 bottomy);
  void SetNull();
  bool IsNull();
  void SetEmpty();
  bool IsEmpty();
  int16 GetWidth();
  void SetWidth(int16 w);
  int16 GetHeight();
  void SetHeight(int16 h);
  long GetArea();
  bool IsEqual(const TOgleRect &rect) const;
  bool IsEqual(const TRect &rect) const;
  bool operator==(const TOgleRect &rect) const;
  bool operator==(const TRect &rect) const;
  bool operator!=(const TOgleRect &rect) const;
  bool operator!=(const TRect &rect) const;
  bool TestSect(TOgleRect &rect); // test for overlap.
  bool Encloses(TPoint &pt);      // pt is inside of rect.
  bool Encloses(TOglePoint &pt);
  bool Contains(TPoint &pt);
  bool Contains(TOglePoint &pt);
  bool Contains(TOgleRect &rect);
  bool Encloses(TOgleRect &rect);
  bool EnclosedBy(TOgleRect &rect); // fits inside passed rect.
  TOgleRect Intersect(TOgleRect &rect);
  TOgleRect SmallestEnclosing(TOgleRect &rect); // smallest orect enclosing both orects.
  bool Disjoint(TOgleRect &rect, TOgleRect &r1,TOgleRect &r2,TOgleRect &r3,TOgleRect &r4,
                                 bool &vr1,bool &vr2,bool &vr3,bool &vr4);
  TOglePoint GetTL();
  void SetTL(TPoint &pt);
  void SetTL(TOglePoint &pt);
  TOglePoint GetBR();
  void SetBR(TPoint &pt);
  void SetBR(TOglePoint &pt);
  TOglePoint GetTR();
  void SetTR(TPoint &pt);
  void SetTR(TOglePoint &pt);
  TOglePoint GetBL();
  void SetBL(TPoint &pt);
  void SetBL(TOglePoint &pt);
  TOgleRect &operator+=(TOgleRect &rect);
  TSize GetSize();
  TOgleRect OffsetBy(int16 dx,int16 dy);
  void OffsetRectBy(TPoint &pt);
  void OffsetRectBy(TOglePoint &pt);
  TOgleRect InflateBy(int16 dx,int16 dy);
  TOgleRect Normalize();
  TOgleRect &SetNormalized();

  operator const TRect();
};
//*******************************************************************
//****************** TTriplets inlines start ************************
//*******************************************************************
inline TTriplet::TTriplet()
:X(0),
 Y(0),
 Z(0)
{
}
//*******************************************************************
inline TTriplet::TTriplet(int16 x,int16 y,int16 z)
:X(x),
 Y(y),
 Z(z)
{
}
//*******************************************************************
inline TTriplet::~TTriplet()
{
}
//*******************************************************************
inline TTriplet::TTriplet(const TTriplet &t)
:X(t.X),
 Y(t.Y),
 Z(t.Z)
{
}
//*******************************************************************
inline int16 TTriplet::GetX()
{
  return X;
}
//*******************************************************************
inline int16 TTriplet::GetY()
{
  return Y;
}
//*******************************************************************
inline int16 TTriplet::GetZ()
{
  return Z;
}
//*******************************************************************
inline void TTriplet::SetX(int16 x)
{
  X = x;
}
//*******************************************************************
inline void TTriplet::SetY(int16 y)
{
  Y = y;
}
//*******************************************************************
inline void TTriplet::SetZ(int16 z)
{
  Z = z;
}
//*******************************************************************
inline TTriplet TTriplet::operator+(const TTriplet &triplet) const
{
  return TTriplet(int16(X + triplet.X),
                  int16(Y + triplet.Y),
                  int16(Z + triplet.Z));
}
//*******************************************************************
inline TTriplet TTriplet::operator-(const TTriplet &triplet) const
{
  return TTriplet(int16(X - triplet.X),
                  int16(Y - triplet.Y),
                  int16(Z - triplet.Z));
}
//*******************************************************************
inline TTriplet &TTriplet::operator+=(const TTriplet &triplet)
{
  X += triplet.X;
  Y += triplet.Y;
  Z += triplet.Z;
  return *this;
}
//*******************************************************************
inline bool TTriplet::operator!=(const TTriplet &triplet)
{
  return (X != triplet.X || Y != triplet.Y || Z != triplet.Z);
}
//*******************************************************************
inline bool TTriplet::operator==(const TTriplet &triplet)
{
  return (X == triplet.X && Y == triplet.Y && Z == triplet.Z);
}
//*******************************************************************
inline TTriplet &TTriplet::operator=(TTriplet const &triplet)
{
  if (&triplet == this) {return *this;} // I think this is needed.
  X = triplet.X;
  Y = triplet.Y;
  Z = triplet.Z;
  return *this;
}
//*******************************************************************
//inline TTriplet &TTriplet::operator=(TTriplet triplet)
//{
//  if (&triplet == this) {return *this;} // I don't know if this is needed.
//  X = triplet.X;
//  Y = triplet.Y;
//  Z = triplet.Z;
//  return *this;
//}
//*******************************************************************
//****************** TOglePoint inlines start ***********************
//*******************************************************************
inline TOglePoint::TOglePoint()
:x(0),
 y(0)
{
}
//*******************************************************************
inline TOglePoint::TOglePoint(int16 newx,int16 newy)
:x(newx),
 y(newy)
{
}
//*******************************************************************
inline TOglePoint::TOglePoint(const TOglePoint &point)
:x(point.x),
 y(point.y)
{
}
//*******************************************************************
inline TOglePoint::TOglePoint(const tagPOINT &point)
:x(int16(point.x)),
 y(int16(point.y))
{
}
//*******************************************************************
inline TOglePoint::TOglePoint(const TPoint &point)
:x(int16(point.x)),
 y(int16(point.y))
{
}
//*******************************************************************
inline TOglePoint::TOglePoint(uint32 u32)
{
  x = loint16(u32);
  y = hiint16(u32);
}
//*******************************************************************
inline TOglePoint::TOglePoint(const TSize &size)
:x(int16(size.cx)),
 y(int16(size.cy))
{
}
//*******************************************************************
inline bool TOglePoint::operator==(const TOglePoint &pt)
{
  return bool(x == pt.x && y == pt.y);
}
//*******************************************************************
inline bool TOglePoint::operator!=(const TOglePoint &pt)
{
  return bool(x != pt.x || y != pt.y);
}
//*******************************************************************
inline bool TOglePoint::operator==(const TPoint &pt)
{
  return bool(x == pt.x && y == pt.y);
}
//*******************************************************************
inline bool TOglePoint::operator!=(const TPoint &pt)
{
  return bool(x != pt.x || y != pt.y);
}
//*******************************************************************
inline bool TOglePoint::operator==(const tagPOINT &pt)
{
  return bool(x == pt.x && y == pt.y);
}
//*******************************************************************
inline bool TOglePoint::operator!=(const tagPOINT &pt)
{
  return bool(x != pt.x || y != pt.y);
}
//*******************************************************************
inline TOglePoint TOglePoint::OffsetBy(int16 dx,int16 dy) const
{
  return TOglePoint(int16(x + dx),int16(y + dy));
}
//*******************************************************************
inline TOglePoint TOglePoint::operator +(const TSize& size) const
{
  return TOglePoint(int16(x + size.cx),int16(y + size.cy));
}
//*******************************************************************
inline TSize TOglePoint::operator -(const TPoint& point) const
{
  return TSize(x - point.x,y - point.y);
}
//*******************************************************************
inline TSize TOglePoint::operator -(const TOglePoint &point) const
{
  return TSize(x - point.x,y - point.y);
}
//*******************************************************************
inline TOglePoint TOglePoint::operator -(const TSize &size) const
{
  return TOglePoint(int16(x - size.cx),int16(y - size.cy));
}
//*******************************************************************
inline TOglePoint TOglePoint::operator -() const
{
  return TOglePoint(int16(-x),int16(-y));
}
//*******************************************************************
inline TOglePoint &TOglePoint::Offset(int16 dx,int16 dy)
{
  x += dx;
  y += dy;
  return *this;
}
//*******************************************************************
inline TOglePoint &TOglePoint::operator+=(const TSize &size)
{
  x += int16(size.cx);
  y += int16(size.cy);
  return *this;
}
//*******************************************************************
inline TOglePoint &TOglePoint::operator-=(const TSize &size)
{
  x -= int16(size.cx);
  y -= int16(size.cy);
  return *this;
}
//*******************************************************************
inline TOglePoint::operator const TPoint()
{
  return TPoint(x,y);
}
//*******************************************************************
//****************** TOgleRect inlines start ************************
//*******************************************************************
inline TOgleRect::TOgleRect(int16 leftx,int16 topy,int16 rightx,int16 bottomy)
:left(leftx),
 top(topy),
 right(rightx),
 bottom(bottomy)
{
}
//*******************************************************************
inline TOgleRect::TOgleRect(const TPoint &tl,const TPoint &br)
:left(int16(tl.x)),
 top(int16(tl.y)),
 right(int16(br.x)),
 bottom(int16(br.y))
{
}
//*******************************************************************
inline TOgleRect::TOgleRect(const TOglePoint &tl,const TOglePoint &br)
:left(tl.x),
 top(tl.y),
 right(br.x),
 bottom(br.y)
{
}
//*******************************************************************
inline TOgleRect::TOgleRect(TPoint const &pt,TSize const &size,bool dummy)
:left(int16(pt.x)),
 top(int16(pt.y))
{
  SetWidth(int16(size.cx));
  SetHeight(int16(size.cy));
}
//*******************************************************************
inline TOgleRect::TOgleRect()
:left(0),
 top(0),
 right(0),
 bottom(0)
{
}
//*******************************************************************
inline TOgleRect::TOgleRect(TRect const &rect)
:left(int16(rect.left)),
 top(int16(rect.top)),
 right(int16(rect.right)),
 bottom(int16(rect.bottom))
{
}
//*******************************************************************
inline TOgleRect::TOgleRect(TOgleRect &rect)
:left(rect.left),
 top(rect.top),
 right(rect.right),
 bottom(rect.bottom)
{
}
//*******************************************************************
inline TOgleRect::~TOgleRect()
{
}
//*******************************************************************
inline void TOgleRect::SetNull()
{
  left = right = top = bottom = 0;
}
//*******************************************************************
inline bool TOgleRect::IsNull()
{
  return (!left && !right && !top && !bottom);
}
//*******************************************************************
inline void TOgleRect::SetEmpty()
{
  SetNull();
}
//*******************************************************************
inline bool TOgleRect::IsEmpty()
{
  return ((left >= right) || (top >= bottom));
}
//*******************************************************************
inline int16 TOgleRect::GetWidth()
{
  return int16(right-left);
}
//*******************************************************************
inline void TOgleRect::SetWidth(int16 w)
{
  right = int16(left + w);
//  right = int16(left + w - 1); // needs to include starting point.
}
//*******************************************************************
inline int16 TOgleRect::GetHeight()
{
  return int16(bottom - top);
}
//*******************************************************************
inline void TOgleRect::SetHeight(int16 h)
{
  bottom = int16(top + h);
//  bottom = int16(top + h - 1); // needs to include starting point.
}
//*******************************************************************
inline long TOgleRect::GetArea()
{
  return (long(left - right) * long(bottom - top));
}
//*******************************************************************
inline bool TOgleRect::IsEqual(const TOgleRect &rect) const
{
  return (left == rect.left &&
          right == rect.right &&
          top == rect.top &&
          bottom == rect.bottom);
}
//*******************************************************************
inline bool TOgleRect::IsEqual(const TRect &rect) const
{
  return (left == rect.left &&
          right == rect.right &&
          top == rect.top &&
          bottom == rect.bottom);
}
//*******************************************************************
inline bool TOgleRect::operator==(const TOgleRect &rect) const
{
  return IsEqual(rect);
}
//*******************************************************************
inline bool TOgleRect::operator==(const TRect &rect) const
{
  return IsEqual(rect);
}
//*******************************************************************
inline bool TOgleRect::operator!=(const TOgleRect &rect) const
{
  return !IsEqual(rect);
}
//*******************************************************************
inline bool TOgleRect::operator!=(const TRect &rect) const
{
  return !IsEqual(rect);
}
//*******************************************************************
inline TOglePoint TOgleRect::GetTL()
{
  return TOglePoint(left,top);
}
//*******************************************************************
inline void TOgleRect::SetTL(TPoint &pt)
{
  top = int16(pt.y);
  left = int16(pt.x);
}
//*******************************************************************
inline void TOgleRect::SetTL(TOglePoint &pt)
{
  left = pt.x;
  top = pt.y;
}
//*******************************************************************
inline TOglePoint TOgleRect::GetBR()
{
  return TOglePoint(right,bottom);
}
//*******************************************************************
inline void TOgleRect::SetBR(TPoint &pt)
{
  bottom = int16(pt.y);
  right = int16(pt.x);
}
//*******************************************************************
inline void TOgleRect::SetBR(TOglePoint &pt)
{
  right = pt.x;
  bottom = pt.y;
}
//*******************************************************************
inline TOglePoint TOgleRect::GetTR()
{
  return TOglePoint(right,top);
}
//*******************************************************************
inline void TOgleRect::SetTR(TOglePoint &pt)
{
  top = pt.y;
  right = pt.x;
}
//*******************************************************************
inline TOglePoint TOgleRect::GetBL()
{
  return TOglePoint(left,bottom);
}
//*******************************************************************
inline void TOgleRect::SetBL(TOglePoint &pt)
{
  bottom = pt.y;
  left = pt.x;
}
//*******************************************************************
inline TOgleRect::operator const TRect()
{
  return TRect(left,top,right,bottom);
}
//*******************************************************************
inline void TOgleRect::SetTR(TPoint &pt)
{
  top = int16(pt.y);
  right = int16(pt.x);
}
//*******************************************************************
inline void TOgleRect::SetBL(TPoint &pt)
{
  bottom = int16(pt.y);
  left = int16(pt.x);
}
//*******************************************************************
inline TSize TOgleRect::GetSize()
{
  return TSize(GetWidth(),GetHeight());
}
//*******************************************************************
inline TOgleRect TOgleRect::OffsetBy(int16 dx,int16 dy)
{
  return TOgleRect(int16(left + dx),
                   int16(top + dy),
                   int16(right + dx),
                   int16(bottom + dy));
}
//*******************************************************************
inline void TOgleRect::OffsetRectBy(TPoint &pt)
{
  left += int16(pt.x);
  right += int16(pt.x);
  top += int16(pt.y);
  bottom += int16(pt.y);
}
//*******************************************************************
inline void TOgleRect::OffsetRectBy(TOglePoint &pt)
{
  left += pt.x;
  right += pt.x;
  top += pt.y;
  bottom += pt.y;
}
//*******************************************************************
inline TOgleRect TOgleRect::InflateBy(int16 dx,int16 dy)
{
  return TOgleRect(int16(left - dx),
                   int16(top - dy),
                   int16(right + dx),
                   int16(bottom + dy));
}
//*******************************************************************
inline TOgleRect TOgleRect::Normalize()
{
  return TOgleRect(Min(left,right),Min(top,bottom),Max(left,right),Max(top,bottom));
}
//*******************************************************************
inline TOgleRect &TOgleRect::SetNormalized()
{
  left = Min(left,right);
  top = Min(top,bottom);
  right = Max(left,right);
  bottom = Max(top,bottom);
  return *this;
}
//*******************************************************************
inline TOgleRect &TOgleRect::operator+=(TOgleRect &rect)
{
  *this = SmallestEnclosing(rect);
  return *this;
}
//*******************************************************************
inline void TOgleRect::Set(int16 leftx,int16 topy,int16 rightx,int16 bottomy)
{
  left = leftx;
  top = topy;
  right = rightx;
  bottom = bottomy;
}
//*******************************************************************
inline bool TOgleRect::TestSect(TOgleRect &rect)
// test for overlap.
{
  return ((left < rect.right) &&
          (right > rect.left) &&
          (top < rect.bottom) &&
          (bottom > rect.top));
}
//*******************************************************************
inline bool TOgleRect::Encloses(TPoint &pt)
// pt is inside of rect.
{
  return ((pt.x >= left) &&
          (pt.x < right) &&
          (pt.y >= top) &&
          (pt.y < bottom));
}
//*******************************************************************
inline bool TOgleRect::Encloses(TOglePoint &pt)
{
  return ((pt.x >= left) &&
          (pt.x < right) &&
          (pt.y >= top) &&
          (pt.y < bottom));
}
//*******************************************************************
inline bool TOgleRect::Encloses(TOgleRect &rect)
{
  return ((rect.left >= left) &&
          (rect.right <= right) &&
          (rect.top >= top) &&
          (rect.bottom <= bottom));
}
//*******************************************************************
inline bool TOgleRect::Contains(TPoint &pt)
{
  return Encloses(pt);
}
//*******************************************************************
inline bool TOgleRect::Contains(TOglePoint &pt)
{
  return Encloses(pt);
}
//*******************************************************************
inline bool TOgleRect::Contains(TOgleRect &rect)
{
  return TestSect(rect);
}
//*******************************************************************
inline bool TOgleRect::EnclosedBy(TOgleRect &rect)
// fits inside passed rect.
{
  return ((left >= rect.left) &&
          (right <= rect.right) &&
          (top >= rect.top) &&
          (bottom <= rect.bottom));
}
//*******************************************************************
inline TOgleRect TOgleRect::Intersect(TOgleRect &rect)
{
  return TOgleRect(left > rect.left ? left : rect.left,
                   top > rect.top ? top : rect.top,
                   right < rect.right ? right : rect.right,
                   bottom < rect.bottom ? bottom : rect.bottom);
}
//*******************************************************************
inline TOgleRect TOgleRect::SmallestEnclosing(TOgleRect &rect)
// smallest orect enclosing both orects.
{
  return TOgleRect(left < rect.left ? left : rect.left,
                   top < rect.top ? top : rect.top,
                   right > rect.right ? right : rect.right,
                   bottom > rect.bottom ? bottom : rect.bottom);
}
//*******************************************************************
inline bool TOgleRect::Disjoint(TOgleRect &rect,
                                TOgleRect &r1,TOgleRect &r2,TOgleRect &r3,TOgleRect &r4,
                                bool &vr1,bool &vr2,bool &vr3,bool &vr4)
// taking the 2 original rects, it will create up to 4 rects that do not overlap and still
// completely enclose original rects.
{
  TOgleRect rs = Intersect(rect);
  vr1 = vr2 = vr3 = vr4 = false;
  int16 numused = 0;

  if (top < rs.top)
  {
    r1.left = left;
    r1.top = top;
    r1.right = right;
    r1.bottom = rs.top;
    vr1 = true;
    numused++;
  }
  if (left < rs.left)
  {
    r2.left = left;
    r2.top = top > rs.top ? top : rs.top;
    r2.right = rs.left;
    r2.bottom = bottom < rs.bottom ? bottom : rs.bottom;
    vr2 = true;
    numused++;
  }
  if (right > rs.right)
  {
    r3.left = rs.right;
    r3.top = top > rs.top ? top : rs.top;
    r3.right = right;
    r3.bottom = bottom < rs.bottom ? bottom : rs.bottom;
    vr3 = true;
    numused++;
  }
  if (bottom > rs.bottom)
  {
    r4.left = left;
    r4.top = rs.bottom;
    r4.right = right;
    r4.bottom = bottom;
    vr4 = true;
    numused++;
  }
  return (numused > 0);
}
//*******************************************************************
//******************* TORect inlines end ****************************
//*******************************************************************
#endif

