//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oinifile.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/20/95    M.Perry     original baseline
//  2.0     8/25/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__oinifile_h)
#define __oinifile_h

#if !defined(__inifile_h)
#include "inifile.h"
#endif
//*******************************************************************
class TOgleIniFile:public TIniFile
{
protected:
  TOgleIniFile() {}

public:
  TOgleIniFile(string const &hddrive,string const &hddir,string const &exename,
               string const &inifilename = string("ogle.ini"),
               bool setdefs = false);

  virtual ~TOgleIniFile();
  virtual void SetToDefaults();
  virtual void ReadData();
  virtual void WriteData();

  //filemgr
//  void SetExeName(string &exename);
  void SetExeName(const string &exename);
  const string &GetExeName();

  const string &SetCDDrive(const string &cddrive);
  const string &GetCDDrive();
  const string &ReadCDDrive();
  void WriteCDDrive();

  const string &SetCDDir(const string &cddir);
  const string &GetCDDir();
  const string &ReadCDDir();
  void WriteCDDir();

  int GetHDBuffering();
  int SetHDBuffering(int buffering);
  int ReadHDBuffering();
  void WriteHDBuffering();

  //soundmgr
  int SetMusic(int on);
  int GetMusic();
  int ReadMusic();
  void WriteMusic();

  int SetEffects(int on);
  int GetEffects();
  int ReadEffects();
  void WriteEffects();

  //playermgr
  string ReadPlayerName(int slotnum);
  void WritePlayerName(int slotnum,string &name);

  // modewin
  int GetDetailLevel(string &modename,int deflevel);
  void SetDetailLevel(string &modename,int level);

  bool GetPrevRun();
  void SetPrevRun(bool prev = true);

  enum TWarnEnum
  {
    BPP_LT_RECOM = 0,
    BPP_GT_RECOM = 1,
    WARNENUM_MAX = 2
  };

  bool CheckWarnNum(TWarnEnum warn)
  {
    return bool(warn >= 0 && warn < WARNENUM_MAX);
  }
  string GetWarnSection();
  bool GetWarnEnbDef(TWarnEnum warn);
  string GetWarnKeys(TWarnEnum warn);
  bool GetWarnMsg(TWarnEnum warn);
  bool SetWarnMsg(TWarnEnum warn,bool enb);

protected:
  //filemgr
  string DirSect;
  string CDDriveKey;
  string CDDirKey;

  string ExeName;
  string CDDrive;
  string CDDir;

  string SysSect;
  string HDBufferKey;
  int HDBuffering;

  //soundmgr
  string SoundSect;
  string EffectsKey;
  string MusicKey;



  int Effects;
  int Music;

  //playermgr
  string PlayerSect;

  // modewin
  string DetailLevelKey;
};
//*******************************************************************
//*********************** inlines ***********************************
//*******************************************************************
inline bool TOgleIniFile::GetPrevRun()
{
  return Readbool(SysSect,string("PrevRun"),false);
}
//*******************************************************************
inline void TOgleIniFile::SetPrevRun(bool prev)
{
  Writebool(SysSect,string("PrevRun"),prev);
}
//*******************************************************************
//inline void TOgleIniFile::SetExeName(string &exename)
inline void TOgleIniFile::SetExeName(const string &exename)
{
  ExeName = exename;
}
//*******************************************************************
inline const string &TOgleIniFile::GetExeName()
{
  return ExeName;
}
//*******************************************************************
inline const string &TOgleIniFile::GetCDDrive()
{
  return CDDrive;
}
//*******************************************************************
inline const string &TOgleIniFile::ReadCDDrive()
{
  return SetCDDrive(ReadString(DirSect,CDDriveKey,GetCDDrive()));
}
//*******************************************************************
inline void TOgleIniFile::WriteCDDrive()
{
  WriteString(DirSect,CDDriveKey,GetCDDrive());
}
//*******************************************************************
inline const string &TOgleIniFile::GetCDDir()
{
  return CDDir;
}
//*******************************************************************
inline const string &TOgleIniFile::ReadCDDir()
{
  return SetCDDir(ReadString(DirSect,CDDirKey,GetCDDir()));
}
//*******************************************************************
inline void TOgleIniFile::WriteCDDir()
{
  WriteString(DirSect,CDDirKey,GetCDDir());
}
//*******************************************************************
inline int TOgleIniFile::GetHDBuffering()
{
  return HDBuffering;
}
//*******************************************************************
inline int TOgleIniFile::SetHDBuffering(int buffering)
{
  HDBuffering = buffering;
  return HDBuffering;
}
//*******************************************************************
inline int TOgleIniFile::ReadHDBuffering()
{
  return SetHDBuffering(ReadInt(SysSect,HDBufferKey,GetHDBuffering()));
}
//*******************************************************************
inline void TOgleIniFile::WriteHDBuffering()
{
  WriteInt(SysSect,HDBufferKey,GetHDBuffering());
}
//*******************************************************************
inline int TOgleIniFile::SetMusic(int on)
{
  Music = on;
  return Music;
}
//*******************************************************************
inline int TOgleIniFile::GetMusic()
{
  return Music;
}
//*******************************************************************
inline int TOgleIniFile::ReadMusic()
{
  return SetMusic(ReadInt(SoundSect,MusicKey,GetMusic()));
}
//*******************************************************************
inline void TOgleIniFile::WriteMusic()
{
  WriteInt(SoundSect,MusicKey,GetMusic());
}
//*******************************************************************
inline int TOgleIniFile::SetEffects(int on)
{
  Effects = on;
  return Effects;
}
//*******************************************************************
inline int TOgleIniFile::GetEffects()
{
  return Effects;
}
//*******************************************************************
inline int TOgleIniFile::ReadEffects()
{
  return SetEffects(ReadInt(SoundSect,EffectsKey,GetEffects()));
}
//*******************************************************************
inline void TOgleIniFile::WriteEffects()
{
  WriteInt(SoundSect,EffectsKey,GetEffects());
}
//*******************************************************************
inline void TOgleIniFile::WritePlayerName(int slotnum,string &name)
{
  WriteString(PlayerSect,slotnum,name);
}
//*******************************************************************
inline int TOgleIniFile::GetDetailLevel(string &modename,int deflevel)
{
  return ReadInt(modename,DetailLevelKey,deflevel);
}
//*******************************************************************
inline void TOgleIniFile::SetDetailLevel(string &modename,int level)
{
  WriteInt(modename,DetailLevelKey,level);
}
//*******************************************************************
#endif

