//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  omdat.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.3     6/5/97     M.Perry    original baseline
//  2.0     8/20/97    M.Perry    32-bit conversion
//********************************************************************
#if !defined(__omdat_h)
#define __omdat_h

#if !defined(__obase_h)
#include "obase.h"
#endif

#if defined(ENB_MISCDATA_CHECK)
  #include <assert.h>
#endif
//*******************************************************************
class TMiscData:public TOgleBase
{
protected:
  friend class TMiscDataDlg;

  int16 NumArrays;
  int16 *PSizes;
  int16 **PPInts;
  void DestroySizes();
  void DestroyInts();
  bool GrowNumArraysTo(int16 size);
  bool GrowNumArraysBy(int16 num);
  bool GrowArrayTo(int16 arraynum,int16 size);
  bool GrowArrayBy(int16 arraynum,int16 num);
  bool Setint16(int16 arraynum,int16 index,int16 value);

public:
  TMiscData(TOgleModeWindow *pmw,const TFileMode &fm,bool AutoRead = true,bool KeepBaseFile = false);
  TMiscData(bool dummy,TOgleModeWindow *pmw,TFileMode &fm);
  TMiscData(TOgleModeWindow *pmw);
  virtual ~TMiscData();
  virtual string AddExt();
  virtual bool ReadLocal();
  virtual bool WriteLocal();
  virtual const char *GetIDChars() const;

  int16 GetNumArrays() {return NumArrays;}
  bool CheckArrayNum(int16 arraynum);
  int16 GetNumint16s(int16 arraynum);
  int16 Getint16(int16 arraynum,int16 index);
};
//*******************************************************************
//********************  inlines *************************************
//*******************************************************************
inline bool TMiscData::GrowNumArraysBy(int16 num)
{
  return GrowNumArraysTo(int16(NumArrays + num));
}
//*******************************************************************
inline bool TMiscData::GrowArrayBy(int16 arraynum,int16 num)
{
  return GrowArrayTo(arraynum,int16(GetNumint16s(arraynum) + num));
}
//*******************************************************************
inline bool TMiscData::Setint16(int16 arraynum,int16 index,int16 value)
{
  if (arraynum < 0 || arraynum >= NumArrays) {return false;}
  if (PSizes == NULL || PSizes[arraynum] <= index) {return false;}
  if (PPInts == NULL || PPInts[arraynum] == NULL) {return false;}
  PPInts[arraynum][index] = value;
  return true;
}
//*******************************************************************
inline bool TMiscData::CheckArrayNum(int16 arraynum)
{
  if (arraynum >= 0 && arraynum < NumArrays) {return true;}
  return false;
}
//*******************************************************************
inline int16 TMiscData::GetNumint16s(int16 arraynum)
{
  if (PSizes == NULL) {return 0;}
  if (!CheckArrayNum(arraynum)) {return 0;}
  return PSizes[arraynum];
}
//*******************************************************************
inline int16 TMiscData::Getint16(int16 arraynum,int16 index)
{
  if (index < 0 || index >= GetNumint16s(arraynum) ||
      PPInts == NULL || PPInts[arraynum] == NULL)
  {
    #if defined(ENB_MISCDATA_CHECK)
    assert(false);
    #endif
    return 0;
  }
  return PPInts[arraynum][index];
}
//*******************************************************************
#endif