//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  omemmgr.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     8/22/96    M.Perry    original baseline
//  2.0     8/18/97    M.Perry    32-bit rework.
//********************************************************************
#if !defined(__omemmgr_h)
#define __omemmgr_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif
//*******************************************************************
class TOgleMemMgr
{
protected:
  static dword BytesAllocated;
  static bool CompactPending;
  static dword WorkingSize;
  static dword AudioSize;

public:
  TOgleMemMgr();
  ~TOgleMemMgr();
  static dword GetWorkingSize();
  static dword GetAudioSize();
  static void IncAudioSize(dword size);
  static void DecAudioSize(dword size);
  static dword GetBytesAllocated();
  static dword GetAppBytesAllocated();
  static dword GetSizePtr(void huge *ptr);
  static void huge *AllocPtr(uint allocflags,dword numbytes);
  static void DeAllocPtr(void huge *ptr);
  static bool WillFit(dword size);
  static bool IsCompactPending();
  static void SetCompactPending(bool pending = true);
  static void Update();
  static void PostUpdate();
  static bool GetWorkingAreaOverflow();
  static dword GetWorkingAreaOverflowSize();

protected:
  static void IncBytes(dword numbytes);
  static void DecBytes(dword numbytes);
};
//*******************************************************************
//********************** inlines ************************************
//*******************************************************************
inline void TOgleMemMgr::IncBytes(dword numbytes)
{
  BytesAllocated += numbytes;
}
//*******************************************************************
inline void TOgleMemMgr::DecBytes(dword numbytes)
{
  if (numbytes > BytesAllocated) {BytesAllocated = 0;}
  else {BytesAllocated -= numbytes;}
}
//*******************************************************************
inline dword TOgleMemMgr::GetSizePtr(void huge *ptr)
{
  return GlobalSize(GlobalPtrHandle(ptr));
}
//*******************************************************************
inline void huge *TOgleMemMgr::AllocPtr(uint allocflags,dword numbytes)
{
  void *ptr = GlobalAllocPtr(allocflags,numbytes);
  if (ptr != 0) {IncBytes(GetSizePtr(ptr));}
  return ptr;
}
//*******************************************************************
inline void TOgleMemMgr::DeAllocPtr(void huge *ptr)
{
  if (ptr == NULL) {return;}
  DecBytes(GetSizePtr(ptr));
  GlobalFreePtr(ptr);
}
//*******************************************************************
inline bool TOgleMemMgr::WillFit(dword size)
{
  return ((size + GetAppBytesAllocated()) <= WorkingSize);
}
//*******************************************************************
inline void TOgleMemMgr::Update()
{
//  Sys.RefreshMemInfo();
}
//*******************************************************************
inline void TOgleMemMgr::PostUpdate()
{
}
//*******************************************************************
inline bool TOgleMemMgr::GetWorkingAreaOverflow()
{
  return (GetAppBytesAllocated() > WorkingSize)?true:false;
}
//*******************************************************************
inline dword TOgleMemMgr::GetWorkingAreaOverflowSize()
{
  dword size = GetAppBytesAllocated();
  if (size <= WorkingSize) {return 0;}
  return (size - WorkingSize);
}
//*******************************************************************
inline dword TOgleMemMgr::GetWorkingSize()
{
  return WorkingSize;
}
//*******************************************************************
inline dword TOgleMemMgr::GetAudioSize()
{
  return AudioSize;
}
//*******************************************************************
inline void TOgleMemMgr::IncAudioSize(dword size)
{
  AudioSize += size;
}
//*******************************************************************
inline void TOgleMemMgr::DecAudioSize(dword size)
{
  if (AudioSize > size)
  {
    AudioSize -= size;
  }
  else
  {
    AudioSize = 0;
  }
}
//*******************************************************************
inline dword TOgleMemMgr::GetBytesAllocated()
{
  return BytesAllocated;
}
//*******************************************************************
inline dword TOgleMemMgr::GetAppBytesAllocated()
{
  return (BytesAllocated + AudioSize);
}
//*******************************************************************
inline bool TOgleMemMgr::IsCompactPending()
{
  return CompactPending;
}
//*******************************************************************
inline void TOgleMemMgr::SetCompactPending(bool pending)
{
  CompactPending = pending;
}
//*******************************************************************
#endif
