//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  omodewin.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     02/27/95   M. Perry    original baseline
//  1.2     03/27/97   M.Perry     changed loading dialog subsystem.
//  2.0     08/28/97   M.Perry     32-bit conversion.
//  2.5     09/23/99   M.Perry     mod's for setupgobjinfo proc's.
//********************************************************************
#if !defined(__omodewin_h)
#define __omodewin_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(OWL_WINDOW_H)
#include <owl\window.h>
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__otimers_h)
#include "otimers.h"
#endif

#if !defined(__omsg_h)
#include "omsg.h"
#endif

#if !defined(__ocurmgr_h)
#include "ocurmgr.h"
#endif

#if defined(DEBUGUPDATES) && !defined(__oupdsum_h)
  #include "oupdsum.h"
#endif

class TFileMode;
class TPlayer;
class TGameLord;
class OGLEOWLCLASS TOgleGameFrameWindow;
class TPlayerMgr;
class TOgleGroupGaugeDlg;
class TPal;
class TBmp;
class TOgleScreenMgr;
class TOgleSoundMgr;
class TOgleFileMgr;
class TGObjList;
class TOgleMode;
class TModeState;
class TGameObject;
//*******************************************************************
class OGLEOWLCLASS TOgleModeWindow : public TWindow
{
private:
  int16 DetailLevel;

protected:
  //mrp - v2.5 - 09/23/99
  bool GObjInfoInitialized,GObjIDSInitialized,GObjStringsInitialized;
  bool UseBmpMono;
  int16 MonoColorNDX;
  bool ShowWindowBeforeGObjList;
  bool DiscardBeforeLoading;
  bool Initialized;
  bool StartNowOK;
  string strModeName;
  TOgleGroupGaugeDlg *PGroupGaugeDlg;
  TOgleMode *PMode;
  TPlayer **GObj;
  TGameLord *pLord;
  TModeState *PState;

public:
  #if defined(GED_COMPILE)
  bool CheckSpriteIDConflict();
  #endif
  #if defined(GED_COMPILE)
  bool CheckSpriteIDUsage();
  #endif
  friend class TOgleMode;
  bool HideGauge;  // MRP
  bool AllowFadeIn;

  bool UpdatePaused;
  bool Active;
  uint ModeHelpCommand;
  uint ModeHelpData;
  TOgleTime *PTime;
  string ModeWindowFileName;
  int16 NumPlayers;
  bool ModeWinInitOK;
  #if defined(DEBUGUPDATES)
  TMWUpdateSum DebugUpdate;
  #endif

protected:
  virtual string GetDetName();
  bool RegisterDet();
  bool UnRegisterDet();
  TOgleGroupGaugeDlg *CreateHiddenGrpGauge();
  virtual void CreateGroupGaugeDlg();
  virtual void DestroyGroupGaugeDlg();
  virtual void FadeIn(TBmp *pbmp);
  void PreLoad();
  void PostLoad();
  void CreateTimer();
  void DestroyTimer();

public:
  bool GetInitialized() {return Initialized;}
  TModeState *GetState() {return PState;}
  void SetMode(TOgleMode *pmode) {PMode = pmode;} // used by TOgleMode until mode init separated from ctor.
  virtual void TimeChanged();
  TOgleGroupGaugeDlg *GetGroupGaugeDlg();
  void GetIniDetailLevel();
  void SetIniDetailLevel();
  string GetModeName();
  TOgleMode *GetMode() {return PMode;}
  void SetModeName(string &name);
  void DoDetailLevelDlg();
  int16 GetDetailLevel();
  void SetDetailLevel(int16 level);
  void ScrollMsg(string title,string *ptext,int16 num,bool pause = true);
  bool Help(uint command,uint32 data = 0);
  void ModeHelp();

  string GetResString(uint id);
  virtual int GetMenuID();
  virtual TModule *GetMenuModule();
  void InitMenu();
  void DeInitMenu();

  TOgleSoundMgr *GetSoundMgr();
  TOgleFileMgr *GetFileMgr();
  TOgleApp *GetOgleApp(bool fatal = true);
  TOgleGameFrameWindow *GetFW();
  TCursorMgr *GetCursorMgr();
  TPlayerMgr *GetPlayerMgr();

  #if defined(DEBUGUPDATES)
  void ShowTime();
  #endif

  virtual void SetIndexes();
  int MessageBox(HWND hWnd,const char far* text,const char far* caption,uint type);
  int MessageBox(const char far* text,const char far* caption,uint type);

  virtual void Initialize(dword StartFUC);
  virtual void DeInitialize();
  #if defined(DEBUGSPRITES)
  void RTReport();
  #endif
  virtual bool IdleAction(long count);
  virtual bool CanClose();
  virtual void ShuttingDown();
  virtual bool AskQuitMode();

  bool IsPlayerChangeOk();
  void NotifyModePlayerActivation(int16 slotnum);
  void NotifyModePlayerDeActivation(int16 slotnum);

  void GObjError(const string &GObjDescr,const string &GObjFName);

  virtual int16 AskCriteriaQuestion(int16 QNum);

  //mrp - v2.5 - 09/23/99
  virtual void SetupGObjInfo();
  virtual void SetupGObjIDS();
  virtual void SetupGObjStrings();

  virtual void SetupGObjs();
  TGObjList *GetGObjList();
  TGameObject *GetGObjPtr(string &GObjName);
  TGameObject *GetGObjPtr(int16 GObjIndex);
  int16 GetGObjIndex(string GObjName);
  int16 GetSpriteIndex(int16 GObjIndex,string SpriteName);

  virtual bool CanActivateSprite(int16 GObjIndex,int16 ListNum,dword StartFUC);
  virtual bool ActivateSprite(int16 GObjIndex,int16 ListNum,dword StartFUC);
  virtual bool ActivateSpriteNow(int16 GObjIndex,int16 ListNum);
  virtual bool IsActiveSprite(int16 GObjIndex,int16 ListNum);
  virtual bool ActivateSkitNow(int16 GObjIndex,int16 SkitCatIndex,int16 SkitIndex);
  virtual bool ActivateSkit(int16 GObjIndex,int16 SkitCatIndex,int16 SkitIndex,dword StartFUC);
  dword GetSkitLength(int16 GObjIndex,int16 SkitCatIndex,int16 SkitIndex);
  dword GetSpriteLength(int16 GObjIndex,int16 SpriteIndex);

  void SetUpdatePause();
  void ClearUpdatePause();
  bool GetUpdatePause();
  virtual void ResumeUpdates() {}

  bool CanStartNow();
  void SetStartNow();
  void ClearStartNow();
  dword GetFUC();
  dword GetAdjFUC();
  bool Update();
  virtual void Update(dword NewFUC);
  virtual void UpdateAllGObjs(dword NewFUC);

  virtual void UpdateState(dword NewFUC);
  virtual void PostUpdateState(dword NewFUC);
  virtual void UpdateAllStates(dword NewFUC);
  virtual void PostUpdateAllStates(dword NewFUC);

  virtual void UpdateMgrs(dword NewFUC);

  virtual void PostUpdateAllGObjs(dword NewFUC);

  virtual void RunState();
  virtual void RunAllStates();

  virtual void Slenderize();
  virtual void CreatePlayerList(int16 numplayers = 3);
  void DestroyPlayerList();
  TPlayer *GetPlayer(int16 playernum)
  {
    // MRP - 06/26/99 - v2.3 - needed '>=' not just '>' condition
    // the old off by one problem.
    if (playernum < 0 || playernum >= NumPlayers) {return NULL;}
    return GObj[playernum];
  }
  TGameLord *GetGameLord()
  {
    return pLord;
  }
  virtual void CreateMode();
  void DestroyMode();
  virtual TGameObject *CreateGObj(TFileMode &fm,int16 id,void *psource = NULL);
  virtual bool DestroyGObj(TGameObject *pgobj);

  virtual void CreateModeState();
  virtual void DeleteModeState();

  TOgleModeWindow(TOgleApp *papp,TOgleGameFrameWindow *pgfw,
                  string modename,TPal *PPal,string title = "");
  virtual ~TOgleModeWindow ();

  bool EvQueryNewPalette();
  void EvPaletteChanged(HWND hWndPalChg);
  void EvLButtonDown(uint modKeys, TPoint& point);
  void EvLButtonUp(uint modKeys,TPoint &point);
  void EvLButtonDblClk(uint modKeys, TPoint& point);
  void EvRButtonDown(uint modKeys, TPoint& point);
  void EvRButtonUp(uint modKeys,TPoint &point);
  void EvRButtonDblClk(uint modKeys,TPoint &point);
  void EvChar(uint key,uint repeatCount,uint flags);
  void ReSendEnterArea();
  void SendToEnterAreaResponder(int16 msg);

  void EvSetFocus(HWND hWndLostFocus);
  void EvKillFocus(HWND hWndGetFocus);
  void Activate();
  void DeActivate();

  bool GetActive();
  void SetActive(bool active = true);

  void EvTimer(uint timerId);
  void Paint(TDC& dc,bool erase,TRect& rect);
  bool EvEraseBkgnd(HDC hDC);
  bool EvSetCursor(HWND hWndCursor, uint hitTest, uint mouseMsg);
  virtual void GetWindowClass(WNDCLASS& wndClass);
  void SetCursorData(TCursorData &cd,bool check = false);

  virtual void SetupWindow();
  virtual void CleanupWindow();

  virtual void BeenSwitchedTo();

  virtual bool SwitchDenom(TMoney &money);// {return false;}
  void PostSwitchToMode(int16 ModeIndex);
  void PostSwitchToMode(string modename);
  void PostCommandToFW(int command);
  DECLARE_RESPONSE_TABLE(TOgleModeWindow);
};
//*******************************************************************
//************************** inlines ********************************
//*******************************************************************
inline TOgleGroupGaugeDlg *TOgleModeWindow::GetGroupGaugeDlg()
{
  return PGroupGaugeDlg;
}
//*******************************************************************
inline string TOgleModeWindow::GetModeName()
{
  return strModeName;
}
//*******************************************************************
inline void TOgleModeWindow::SetModeName(string &name)
{
  strModeName = name;
}
//*******************************************************************
inline int16 TOgleModeWindow::GetDetailLevel()
{
  return DetailLevel;
}
//*******************************************************************
inline void TOgleModeWindow::SetDetailLevel(int16 level)
{
  DetailLevel = level;
  if (DetailLevel > 3) {DetailLevel = 3;}
  if (DetailLevel < 0) {DetailLevel = 0;}
}
//*******************************************************************
inline string TOgleModeWindow::GetResString(uint id)
{
  return GetOgleApp()->LoadString(id);
//  return TAppString(id);
}
//*******************************************************************
inline TOgleSoundMgr *TOgleModeWindow::GetSoundMgr()
{
//  return POgleApp->GetSoundMgr();
  return GetOgleApp()->GetSoundMgr();
}
//*******************************************************************
inline TOgleFileMgr *TOgleModeWindow::GetFileMgr()
{
//  return POgleApp->GetFileMgr();
  return GetOgleApp()->GetFileMgr();
}
//*******************************************************************
inline TOgleApp *TOgleModeWindow::GetOgleApp(bool fatal)
{
//  return POgleApp;
  return TOgleApp::GetOgleApp();
}
//*******************************************************************
inline TOgleGameFrameWindow *TOgleModeWindow::GetFW()
{
//  return PGameFW;
  return GetOgleApp()->GetFW();
}
//*******************************************************************
inline int TOgleModeWindow::MessageBox(const char far* text,const char far* caption,uint type)
{
  return TWindow::MessageBox(text,caption,type);
}
//*******************************************************************
//inline TPal *TOgleModeWindow::GetPal()
//{
//  return PMasterPalette;
//}
//*******************************************************************
//inline TBmp *TOgleModeWindow::GetBG()
//{
//  return PModeBackground;
//}
//*******************************************************************
inline TGameObject *TOgleModeWindow::GetGObjPtr(string &GObjName)
{
  return GetGObjPtr(GetGObjIndex(GObjName));
}
//*******************************************************************
inline void TOgleModeWindow::SetUpdatePause()
{
  UpdatePaused = true;
}
//*******************************************************************
inline void TOgleModeWindow::ClearUpdatePause()
{
  UpdatePaused = false;
  ResumeUpdates();
}
//*******************************************************************
inline bool TOgleModeWindow::GetUpdatePause()
{
  return UpdatePaused;
}
//*******************************************************************
inline bool TOgleModeWindow::CanStartNow()
{
  return (StartNowOK);
}
//*******************************************************************
inline void TOgleModeWindow::SetStartNow()
{
  StartNowOK = true;
}
//*******************************************************************
inline void TOgleModeWindow::ClearStartNow()
{
  StartNowOK = false;
}
//*******************************************************************
inline dword TOgleModeWindow::GetFUC()
{
  return ((PTime == NULL) ? 0 : PTime->GetFUC());
}
//*******************************************************************
inline dword TOgleModeWindow::GetAdjFUC()
{
  dword fuc = GetFUC();
  if (!CanStartNow()) {fuc += 1;}
  return fuc;
}
//*******************************************************************
inline bool TOgleModeWindow::GetActive()
{
  return Active;
}
//*******************************************************************
inline void TOgleModeWindow::SetActive(bool active)
{
  Active = active;
}
//*******************************************************************
#endif

