//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  omoney.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.2     5/6/97     M.Perry    added to handle change.
//  2.0     8/26/97    M.Perry    32-bit conversion.
// note: need to switch from double to BCD class for accuracy.
//  2.3    07/14/99    M.Perry    added functs w/ const params
//  2.5    09/29/99    M.Perry    added denom id support.
//********************************************************************
#if !defined(__omoney_h)
#define __omoney_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__MATH_H)
#include "math.h"
#endif
//*******************************************************************
class TMoney
{
public:
  //used w/ special ctor
  enum {ctorint16 = 0,ctorDenomID = 1};

  //Denom ID's
  enum {did_Invalid = -1,
        did_None = 0,
        did_0_00 = 0,
        did_0_01 = 1,
        did_0_05 = 2,
        did_0_10 = 3,
        did_0_25 = 4,
        did_0_50 = 5,
        did_1_00 = 6,
        did_2_00 = 7,
        did_5_00 = 8,
        did_10_00 = 9,
        did_25_00 = 10,
        did_50_00 = 11,
        did_100_00 = 12,
        did_500_00 = 13,
        did_1000_00 = 14,
        did_NumIDs = 15};

protected:
  double Value;

  static bool MoneyDenomIDInitialized;
  static TMoney MoneyDenomID[did_NumIDs];

public:
  TMoney();
  TMoney(TMoney &money);
  TMoney(const TMoney &money);
  TMoney(int32 cash);
  TMoney(double cash);
  TMoney(int16 cash);
  TMoney(int cash);
  TMoney(int16 ctortype,int16 data);
  ~TMoney();

  static bool CheckDenomID(int16 did) {return bool(did >= 0 && did < did_NumIDs);}
  int16 GetDenomID();
  void SetDenomID(int16 did);

  void Truncate() {Assign(int32(Value));}

  // helper op's
  void RoundTo5Cent();

  void Assign(int32 cash);
  void Assign(TMoney &money);
  void Assign(const TMoney &money);
  void Assign(double cash);
  void Assign(int16 cash);
  void Assign(int cash);

  void FlipSign();
  int16 GetSign() const;
  int16 GetCents() const;
  int32 GetDollars() const;
  string GetStr(bool showcents = true,
                bool showzerocents = true,
                bool showcommas = false) const;

  void Sub(double cash);
  void Sub(int32 cash);
  void Sub(TMoney &money);
  void Sub(const TMoney &money);
  void Sub(int16 cash);
  void Sub(int cash);

  void Add(double cash);
  void Add(int32 cash);
  void Add(TMoney &money);
  void Add(const TMoney &money);
  void Add(int16 cash);
  void Add(int cash);

  void Mult(int32 cash);
  void Mult(double cash);
  void Mult(TMoney &money);
  void Mult(const TMoney &money);
  void Mult(int16 cash);
  void Mult(int cash);

  void Div(int32 cash);
  void Div(double cash);
  void Div(TMoney &money);
  void Div(const TMoney &money);
  void Div(int16 cash);
  void Div(int cash);

  void Mod(int32 cash);
  void Mod(double cash);
  void Mod(TMoney &money);
  void Mod(const TMoney &money);
  void Mod(int16 cash);
  void Mod(int cash);

  void Abs();
  void RoundToDollar();

  bool IsEqual(const TMoney &money);
  bool IsEqual(TMoney &money);
  bool IsEqual(double cash);
  bool IsEqual(int32 cash);
  bool IsEqual(int16 cash);
  bool IsEqual(int cash);

  // conversion op's
  operator int16() const {return (int16)Value;}
  operator int() const {return (int)Value;}
  operator int32() const {return (int32)Value;}
  operator double() const {return Value;}
  operator bool() const {return bool(Value > 0.0);}

  // unary op's
  TMoney &operator-() {FlipSign();return *this;}

  // assignment op's
  TMoney &operator=(TMoney &money);
  TMoney &operator=(const TMoney &money);
  TMoney &operator=(double cash);
  TMoney &operator=(int32 cash);
  TMoney &operator=(int16 cash);
  TMoney &operator=(int cash);

  TMoney &operator+=(TMoney &money);
  TMoney &operator+=(const TMoney &money);
  TMoney &operator+=(double cash);
  TMoney &operator+=(int32 cash);
  TMoney &operator+=(int16 cash);
  TMoney &operator+=(int cash);

  TMoney &operator-=(TMoney &money);
  TMoney &operator-=(const TMoney &money);
  TMoney &operator-=(double cash);
  TMoney &operator-=(int32 cash);
  TMoney &operator-=(int16 cash);
  TMoney &operator-=(int cash);

  TMoney &operator*=(TMoney &money);
  TMoney &operator*=(const TMoney &money);
  TMoney &operator*=(double cash);
  TMoney &operator*=(int32 cash);
  TMoney &operator*=(int16 cash);
  TMoney &operator*=(int cash);

  TMoney &operator/=(TMoney &money);
  TMoney &operator/=(const TMoney &money);
  TMoney &operator/=(double cash);
  TMoney &operator/=(int32 cash);
  TMoney &operator/=(int16 cash);
  TMoney &operator/=(int cash);

  TMoney &operator++();
  TMoney &operator++(int);

  TMoney &operator--();
  TMoney &operator--(int);

  TMoney &operator%=(int32 cash);
  TMoney &operator%=(TMoney &money);
  TMoney &operator%=(const TMoney &money);
  TMoney &operator%=(double cash);
  TMoney &operator%=(int16 cash);
  TMoney &operator%=(int cash);

  // relational op's
  bool operator==(TMoney &money);
  bool operator==(const TMoney &money);
  bool operator==(double cash);
  bool operator==(int32 cash);
  bool operator==(int16 cash);
  bool operator==(int cash);

  bool operator!=(TMoney &money);
  bool operator!=(const TMoney &money);
  bool operator!=(double cash);
  bool operator!=(int32 cash);
  bool operator!=(int16 cash);
  bool operator!=(int cash);

  bool operator>(TMoney &money);
  bool operator>(const TMoney &money);
  bool operator>(double cash);
  bool operator>(int32 cash);
  bool operator>(int16 cash);
  bool operator>(int cash);

  bool operator>=(TMoney &money);
  bool operator>=(const TMoney &money);
  bool operator>=(double cash);
  bool operator>=(int32 cash);
  bool operator>=(int16 cash);
  bool operator>=(int cash);

  bool operator<(TMoney &money);
  bool operator<(const TMoney &money);
  bool operator<(double cash);
  bool operator<(int32 cash);
  bool operator<(int16 cash);
  bool operator<(int cash);

  bool operator<=(TMoney &money);
  bool operator<=(const TMoney &money);
  bool operator<=(double cash);
  bool operator<=(int32 cash);
  bool operator<=(int16 cash);
  bool operator<=(int cash);

  // ********* friendly functions ***********

  // math op's
  friend TMoney operator+(const TMoney &m1,const TMoney &m2);
  friend TMoney operator+(const TMoney &m1,double cash);
  friend TMoney operator+(const TMoney &m1,int32 cash);
  friend TMoney operator+(const TMoney &m1,int16 cash);
  friend TMoney operator+(const TMoney &m1,int cash);

  friend int operator+(int cash,const TMoney &m);
  friend int16 operator+(int16 cash,const TMoney &m);
  friend int32 operator+(int32 cash,const TMoney &m);
  friend double operator+(double cash,const TMoney &m);

  friend TMoney operator-(const TMoney &m1,const TMoney &m2);
  friend TMoney operator-(const TMoney &m1,double cash);
  friend TMoney operator-(const TMoney &m1,int32 cash);
  friend TMoney operator-(const TMoney &m1,int16 cash);
  friend TMoney operator-(const TMoney &m1,int cash);

  friend int operator-(int cash,const TMoney &m);
  friend int16 operator-(int16 cash,const TMoney &m);
  friend int32 operator-(int32 cash,const TMoney &m);
  friend double operator-(double cash,const TMoney &m);

  friend TMoney operator*(const TMoney &m1,const TMoney &m2);
  friend TMoney operator*(const TMoney &m1,double cash);
  friend TMoney operator*(const TMoney &m1,int32 cash);
  friend TMoney operator*(const TMoney &m1,int16 cash);
  friend TMoney operator*(const TMoney &m1,int cash);

  friend int operator*(int cash,const TMoney &m);
  friend int16 operator*(int16 cash,const TMoney &m);
  friend int32 operator*(int32 cash,const TMoney &m);
  friend double operator*(double cash,const TMoney &m);

  friend TMoney operator/(const TMoney &m1,const TMoney &m2);
  friend TMoney operator/(const TMoney &m1,double cash);
  friend TMoney operator/(const TMoney &m1,int32 cash);
  friend TMoney operator/(const TMoney &m1,int16 cash);
  friend TMoney operator/(const TMoney &m1,int cash);

  friend int operator/(int cash,const TMoney &m);
  friend int16 operator/(int16 cash,const TMoney &m);
  friend int32 operator/(int32 cash,const TMoney &m);
  friend double operator/(double cash,const TMoney &m);

  friend TMoney operator%(const TMoney &m1,int32 cash);
  friend TMoney operator%(const TMoney &m1,double cash);
  friend TMoney operator%(const TMoney &m1,TMoney &m2);
  friend TMoney operator%(const TMoney &m1,int16 cash);
  friend TMoney operator%(const TMoney &m1,int cash);

  friend bool operator>(int32 cash,TMoney &m1);
  friend bool operator>(int32 cash,const TMoney &m1);

  friend bool operator>=(int32 cash,TMoney &m1);
  friend bool operator>=(int32 cash,const TMoney &m1);

  friend bool operator<(int32 cash,TMoney &m1);
  friend bool operator<(int32 cash,const TMoney &m1);

  friend bool operator<=(int32 cash,TMoney &m1);
  friend bool operator<=(int32 cash,const TMoney &m1);

  friend bool operator==(int32 cash,TMoney &m1);
  friend bool operator==(int32 cash,const TMoney &m1);

  friend bool operator!=(int32 cash,TMoney &m1);
  friend bool operator!=(int32 cash,const TMoney &m1);


  // misc op's
  friend TMoney Abs(const TMoney &money);
  friend int16 GetSign(const TMoney &money);
  friend int32 RoundToDollar(const TMoney &money);
};
//*******************************************************************
//******************  inlines  **************************************
//*******************************************************************
inline TMoney::TMoney()
{
  Assign(0l);
}
//*******************************************************************
inline TMoney::TMoney(const TMoney &money)
{
  Assign(money);
}
//*******************************************************************
inline TMoney::TMoney(TMoney &money)
{
  Assign(money);
}
//*******************************************************************
inline TMoney::TMoney(int32 cash)
{
  Assign(cash);
}
//*******************************************************************
inline TMoney::TMoney(int16 cash)
{
  Assign(cash);
}
//*******************************************************************
inline TMoney::TMoney(int cash)
{
  Assign(cash);
}
//*******************************************************************
inline TMoney::TMoney(double cash)
{
  Assign(cash);
}
//*******************************************************************
inline TMoney::~TMoney()
{
}
//*******************************************************************
inline void TMoney::Assign(int32 cash)
{
  Value = cash;
}
//*******************************************************************
inline void TMoney::Assign(int16 cash)
{
  Value = (int32)cash;
}
//*******************************************************************
inline void TMoney::Assign(int cash)
{
  Value = (int32)cash;
}
//*******************************************************************
inline void TMoney::Assign(const TMoney &money)
{
  Value = money.Value;
}
//*******************************************************************
inline void TMoney::Assign(TMoney &money)
{
  Value = money.Value;
}
//*******************************************************************
inline void TMoney::Assign(double cash)
{
  Value = cash;
}
//*******************************************************************
inline void TMoney::FlipSign()
{
  Value *= -1;
}
//*******************************************************************
inline int16 TMoney::GetSign() const
{
  if (Value < 0.0) {return -1;}
  return 1;
}
//*******************************************************************
inline int32 TMoney::GetDollars() const
{
  int32 l = (int32)Value;
  if (l < 0) {l *= -1;}
  return l;
}
//*******************************************************************
inline void TMoney::Sub(double cash)
{
  Assign(Value - cash);
}
//*******************************************************************
inline void TMoney::Sub(int32 cash)
{
  Assign(Value - (double)cash);
}
//*******************************************************************
inline void TMoney::Sub(int16 cash)
{
  Assign(Value - (double)cash);
}
//*******************************************************************
inline void TMoney::Sub(int cash)
{
  Assign(Value - (double)cash);
}
//*******************************************************************
inline void TMoney::Sub(TMoney &money)
{
  Assign(Value - money.Value);
}
//*******************************************************************
inline void TMoney::Sub(const TMoney &money)
{
  Assign(Value - money.Value);
}
//*******************************************************************
inline void TMoney::Add(double cash)
{
  Assign(Value + cash);
}
//*******************************************************************
inline void TMoney::Add(int32 cash)
{
  Assign(Value + (double)cash);
}
//*******************************************************************
inline void TMoney::Add(int16 cash)
{
  Assign(Value + (double)cash);
}
//*******************************************************************
inline void TMoney::Add(int cash)
{
  Assign(Value + (double)cash);
}
//*******************************************************************
inline void TMoney::Add(TMoney &money)
{
  Assign(Value + money.Value);
}
//*******************************************************************
inline void TMoney::Add(const TMoney &money)
{
  Assign(Value + money.Value);
}
//*******************************************************************
inline void TMoney::Mult(int32 cash)
{
  Assign(Value * cash);
}
//*******************************************************************
inline void TMoney::Mult(int16 cash)
{
  Assign(Value * cash);
}
//*******************************************************************
inline void TMoney::Mult(int cash)
{
  Assign(Value * cash);
}
//*******************************************************************
inline void TMoney::Mult(double cash)
{
  Assign(Value * cash);
}
//*******************************************************************
inline void TMoney::Mult(TMoney &money)
{
  Assign(Value * money.Value);
}
//*******************************************************************
inline void TMoney::Mult(const TMoney &money)
{
  Assign(Value * money.Value);
}
//*******************************************************************
inline void TMoney::Div(int32 cash)
{
  if (cash == 0l) {Assign(0.0);}
  Assign(Value / cash);
}
//*******************************************************************
inline void TMoney::Div(int16 cash)
{
  if (cash == 0) {Assign(0.0);}
  Assign(Value / cash);
}
//*******************************************************************
inline void TMoney::Div(int cash)
{
  if (cash == 0) {Assign(0.0);}
  Assign(Value / cash);
}
//*******************************************************************
inline void TMoney::Div(double cash)
{
  if (cash == 0.0) {Assign(0.0);}
  Assign(Value / cash);
}
//*******************************************************************
inline void TMoney::Div(TMoney &money)
{
  if (money.Value == 0.0) {Assign(0.0);}
  Assign(Value / money.Value);
}
//*******************************************************************
inline void TMoney::Div(const TMoney &money)
{
  if (money.Value == 0.0) {Assign(0.0);}
  Assign(Value / money.Value);
}
//*******************************************************************
inline void TMoney::Mod(int32 cash)
{
  if (cash == 0l) {Assign(0.0);}
  Assign((int32)Value % cash);
}
//*******************************************************************
inline void TMoney::Mod(int16 cash)
{
  if (cash == 0) {Assign(0.0);}
  Assign((int32)Value % cash);
}
//*******************************************************************
inline void TMoney::Mod(int cash)
{
  if (cash == 0) {Assign(0.0);}
  Assign((int32)Value % cash);
}
//*******************************************************************
inline void TMoney::Mod(double cash)
{
  if (cash == 0.0) {Assign(0.0);}
  Assign((int32)Value % (int32)cash);
}
//*******************************************************************
inline void TMoney::Mod(TMoney &money)
{
  if (money.Value == 0.0) {Assign(0.0);}
  Assign((int32)Value % (int32)money);
}
//*******************************************************************
inline void TMoney::Mod(const TMoney &money)
{
  if (money.Value == 0.0) {Assign(0.0);}
  Assign((int32)Value % (int32)money);
}
//*******************************************************************
inline void TMoney::Abs()
{
  Assign(fabs(Value));
}
//*******************************************************************
inline void TMoney::RoundToDollar()
{
  int32 cash = (int32)Value;
  Value = cash;
}
//*******************************************************************
inline bool TMoney::IsEqual(const TMoney &money)
{
  if (&money == this) {return true;}
  if (Value == money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::IsEqual(TMoney &money)
{
  if (&money == this) {return true;}
  if (Value == money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::IsEqual(double cash)
{
  if (Value == cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::IsEqual(int32 cash)
{
  if ((int32)Value == cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::IsEqual(int16 cash)
{
  if ((int32)Value == (int32)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::IsEqual(int cash)
{
  if ((int32)Value == (int32)cash) {return true;}
  return false;
}
//*******************************************************************
inline TMoney &TMoney::operator=(TMoney &money)
{
  if (&money == this) {return *this;}
  Assign(money);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator=(const TMoney &money)
{
  if (&money == this) {return *this;}
  Assign(money);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator=(double cash)
{
  Assign(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator=(int32 cash)
{
  Assign(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator=(int16 cash)
{
  Assign(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator=(int cash)
{
  Assign(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator+=(TMoney &money)
{
  Assign(Value + money.Value);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator+=(const TMoney &money)
{
  Assign(Value + money.Value);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator+=(double cash)
{
  Assign(Value + cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator+=(int32 cash)
{
  Assign(Value + (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator+=(int16 cash)
{
  Assign(Value + (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator+=(int cash)
{
  Assign(Value + (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator-=(TMoney &money)
{
  Assign(Value - money.Value);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator-=(const TMoney &money)
{
  Assign(Value - money.Value);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator-=(double cash)
{
  Assign(Value - cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator-=(int32 cash)
{
  Assign(Value - (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator-=(int16 cash)
{
  Assign(Value - (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator-=(int cash)
{
  Assign(Value - (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator*=(TMoney &money)
{
  Assign(Value * money.Value);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator*=(const TMoney &money)
{
  Assign(Value * money.Value);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator*=(double cash)
{
  Assign(Value * cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator*=(int32 cash)
{
  Assign(Value * (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator*=(int16 cash)
{
  Assign(Value * (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator*=(int cash)
{
  Assign(Value * (double)cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator/=(TMoney &money)
{
  Div(money);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator/=(const TMoney &money)
{
  Div(money);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator/=(double cash)
{
  Div(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator/=(int32 cash)
{
  Div(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator/=(int16 cash)
{
  Div(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator/=(int cash)
{
  Div(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator++()
{
  Assign(Value + 1.0);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator++(int)
{
  Assign(Value + 1.0);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator--()
{
  Assign(Value - 1.0);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator--(int)
{
  Assign(Value - 1.0);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator%=(int32 cash)
{
  Mod(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator%=(int16 cash)
{
  Mod(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator%=(int cash)
{
  Mod(cash);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator%=(TMoney &money)
{
  Mod(money);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator%=(const TMoney &money)
{
  Mod(money);
  return *this;
}
//*******************************************************************
inline TMoney &TMoney::operator%=(double cash)
{
  Mod(cash);
  return *this;
}
//*******************************************************************
inline bool TMoney::operator==(TMoney &money)
{
  return IsEqual(money);
}
//*******************************************************************
inline bool TMoney::operator==(const TMoney &money)
{
  return IsEqual(money);
}
//*******************************************************************
inline bool TMoney::operator==(double cash)
{
  return IsEqual(cash);
}
//*******************************************************************
inline bool TMoney::operator==(int32 cash)
{
  return IsEqual(cash);
}
//*******************************************************************
inline bool TMoney::operator==(int16 cash)
{
  return IsEqual(cash);
}
//*******************************************************************
inline bool TMoney::operator==(int cash)
{
  return IsEqual(cash);
}
//*******************************************************************
inline bool TMoney::operator!=(TMoney &money)
{
  return !IsEqual(money);
}
//*******************************************************************
inline bool TMoney::operator!=(const TMoney &money)
{
  return !IsEqual(money);
}
//*******************************************************************
inline bool TMoney::operator!=(double cash)
{
  return !IsEqual(cash);
}
//*******************************************************************
inline bool TMoney::operator!=(int32 cash)
{
  return !IsEqual(cash);
}
//*******************************************************************
inline bool TMoney::operator!=(int16 cash)
{
  return !IsEqual(cash);
}
//*******************************************************************
inline bool TMoney::operator!=(int cash)
{
  return !IsEqual(cash);
}
//*******************************************************************
inline bool TMoney::operator>(TMoney &money)
{
  if (Value > money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>(const TMoney &money)
{
  if (Value > money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>(double cash)
{
  if (Value > cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>(int32 cash)
{
  if (Value > (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>(int16 cash)
{
  if (Value > (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>(int cash)
{
  if (Value > (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>=(TMoney &money)
{
  if (Value >= money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>=(const TMoney &money)
{
  if (Value >= money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>=(double cash)
{
  if (Value >= cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>=(int32 cash)
{
  if (Value >= (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>=(int16 cash)
{
  if (Value >= (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator>=(int cash)
{
  if (Value >= (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<(TMoney &money)
{
  if (Value < money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<(const TMoney &money)
{
  if (Value < money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<(double cash)
{
  if (Value < cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<(int32 cash)
{
  if (Value < (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<(int16 cash)
{
  if (Value < (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<(int cash)
{
  if (Value < (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<=(TMoney &money)
{
  if (Value <= money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<=(const TMoney &money)
{
  if (Value <= money.Value) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<=(double cash)
{
  if (Value <= cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<=(int32 cash)
{
  if (Value <= (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<=(int16 cash)
{
  if (Value <= (double)cash) {return true;}
  return false;
}
//*******************************************************************
inline bool TMoney::operator<=(int cash)
{
  if (Value <= (double)cash) {return true;}
  return false;
}
//*******************************************************************
// ********* friendly functions ***********
inline TMoney operator+(const TMoney &m1,const TMoney &m2)
{
  return TMoney(m1.Value + m2.Value);
}
//*******************************************************************
inline TMoney operator+(const TMoney &m1,double cash)
{
  return TMoney(m1.Value + cash);
}
//*******************************************************************
inline TMoney operator+(const TMoney &m1,int32 cash)
{
  return TMoney(m1.Value + (double)cash);
}
//*******************************************************************
inline TMoney operator+(const TMoney &m1,int16 cash)
{
  return TMoney(m1.Value + (double)cash);
}
//*******************************************************************
inline TMoney operator+(const TMoney &m1,int cash)
{
  return TMoney(m1.Value + (double)cash);
}
//*******************************************************************
inline TMoney operator-(const TMoney &m1,const TMoney &m2)
{
  return TMoney(m1.Value - m2.Value);
}
//*******************************************************************
inline TMoney operator-(const TMoney &m1,double cash)
{
  return TMoney(m1.Value - cash);
}
//*******************************************************************
inline TMoney operator-(const TMoney &m1,int32 cash)
{
  return TMoney(m1.Value - (double)cash);
}
//*******************************************************************
inline TMoney operator-(const TMoney &m1,int16 cash)
{
  return TMoney(m1.Value - (double)cash);
}
//*******************************************************************
inline TMoney operator-(const TMoney &m1,int cash)
{
  return TMoney(m1.Value - (double)cash);
}
//*******************************************************************
inline TMoney operator*(const TMoney &m1,const TMoney &m2)
{
  return TMoney(m1.Value * m2.Value);
}
//*******************************************************************
inline TMoney operator*(const TMoney &m1,double cash)
{
  return TMoney(m1.Value * cash);
}
//*******************************************************************
inline TMoney operator*(const TMoney &m1,int32 cash)
{
  return TMoney(m1.Value * (double)cash);
}
//*******************************************************************
inline TMoney operator*(const TMoney &m1,int16 cash)
{
  return TMoney(m1.Value * (double)cash);
}
//*******************************************************************
inline TMoney operator*(const TMoney &m1,int cash)
{
  return TMoney(m1.Value * (double)cash);
}
//*******************************************************************
inline TMoney operator/(const TMoney &m1,const TMoney &m2)
{
  return TMoney(m1.Value / m2.Value);
}
//*******************************************************************
inline TMoney operator/(const TMoney &m1,double cash)
{
  return TMoney(m1.Value / cash);
}
//*******************************************************************
inline TMoney operator/(const TMoney &m1,int32 cash)
{
  return TMoney(m1.Value / (double)cash);
}
//*******************************************************************
inline TMoney operator/(const TMoney &m1,int16 cash)
{
  return TMoney(m1.Value / (double)cash);
}
//*******************************************************************
inline TMoney operator/(const TMoney &m1,int cash)
{
  return TMoney(m1.Value / (double)cash);
}
//*******************************************************************
inline int16 GetSign(const TMoney &money)
{
  return money.GetSign();
}
//*******************************************************************
inline int32 RoundToDollar(const TMoney &money)
{
  return (int32)money.Value;
}
//*******************************************************************
inline bool operator>(int32 cash,TMoney &m1)
{
  return ((double)cash > m1.Value);
}
//*******************************************************************
inline bool operator>(int32 cash,const TMoney &m1)
{
  return ((double)cash > m1.Value);
}
//*******************************************************************
inline bool operator>=(int32 cash,TMoney &m1)
{
  return ((double)cash >= m1.Value);
}
//*******************************************************************
inline bool operator>=(int32 cash,const TMoney &m1)
{
  return ((double)cash >= m1.Value);
}
//*******************************************************************
inline bool operator<(int32 cash,TMoney &m1)
{
  return ((double)cash < m1.Value);
}
//*******************************************************************
inline bool operator<(int32 cash,const TMoney &m1)
{
  return ((double)cash < m1.Value);
}
//*******************************************************************
inline bool operator<=(int32 cash,TMoney &m1)
{
  return ((double)cash <= m1.Value);
}
//*******************************************************************
inline bool operator<=(int32 cash,const TMoney &m1)
{
  return ((double)cash <= m1.Value);
}
//*******************************************************************
inline bool operator==(int32 cash,TMoney &m1)
{
  return ((double)cash == m1.Value);
}
//*******************************************************************
inline bool operator==(int32 cash,const TMoney &m1)
{
  return ((double)cash == m1.Value);
}
//*******************************************************************
inline bool operator!=(int32 cash,TMoney &m1)
{
  return ((double)cash != m1.Value);
}
//*******************************************************************
inline bool operator!=(int32 cash,const TMoney &m1)
{
  return ((double)cash != m1.Value);
}
//*******************************************************************
inline int operator+(int cash,const TMoney &m)
{
  return int(cash + int(m));
}
//*******************************************************************
inline int16 operator+(int16 cash,const TMoney &m)
{
  return int16(cash + int16(m));
}
//*******************************************************************
inline int32 operator+(int32 cash,const TMoney &m)
{
  return int32(cash + int32(m));
}
//*******************************************************************
inline double operator+(double cash,const TMoney &m)
{
  return double(cash + double(m));
}
//*******************************************************************
inline int operator-(int cash,const TMoney &m)
{
  return int(cash - int(m));
}
//*******************************************************************
inline int16 operator-(int16 cash,const TMoney &m)
{
  return int16(cash - int16(m));
}
//*******************************************************************
inline int32 operator-(int32 cash,const TMoney &m)
{
  return int32(cash - int32(m));
}
//*******************************************************************
inline double operator-(double cash,const TMoney &m)
{
  return double(cash - double(m));
}
//*******************************************************************
inline int operator*(int cash,const TMoney &m)
{
  return int(cash * int(m));
}
//*******************************************************************
inline int16 operator*(int16 cash,const TMoney &m)
{
  return int16(cash * int16(m));
}
//*******************************************************************
inline int32 operator*(int32 cash,const TMoney &m)
{
  return int32(cash * int32(m));
}
//*******************************************************************
inline double operator*(double cash,const TMoney &m)
{
  return double(cash * double(m));
}
//*******************************************************************
inline int operator/(int cash,const TMoney &m)
{
  return int(cash / int(m));
}
//*******************************************************************
inline int16 operator/(int16 cash,const TMoney &m)
{
  return int16(cash / int16(m));
}
//*******************************************************************
inline int32 operator/(int32 cash,const TMoney &m)
{
  return int32(cash / int32(m));
}
//*******************************************************************
inline double operator/(double cash,const TMoney &m)
{
  return double(cash / double(m));
}
//*******************************************************************
#endif

