//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oPal.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/28/95    M.Perry     original baseline
//  2.0     8/21/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__opal_h)
#define __opal_h

#if !defined(__ogr_h)
#include "ogr.h"
#endif

class TLogicalPalette;
//*******************************************************************
class TRGBQuadPalette
{
protected:
  friend TLogicalPalette;
  RGBQUAD *PRGBQuad;
  int16 NumQuads;

public:
  TRGBQuadPalette();
  TRGBQuadPalette(RGBQUAD *prgbquad,int16 numquads);
  TRGBQuadPalette(TRGBQuadPalette &rgbquadpalette);
  ~TRGBQuadPalette();
  bool IsEqual(TRGBQuadPalette &rgbquadpalette);
  bool operator==(TRGBQuadPalette &rgbquadpalette);
  bool operator!=(TRGBQuadPalette &rgbquadpalette);
  int16 GetNumQuads() {return NumQuads;}
  bool CreateQuadStruct(int16 numquads);
  void DestroyQuadStruct();
  operator RGBQUAD() const {return *PRGBQuad;}
  bool SetQuad(int16 index,RGBQUAD rgbquad);
  RGBQUAD GetQuad(int16 index);
  TRGBQuadPalette &operator=(TRGBQuadPalette &rgbquadpalette);
  TRGBQuadPalette &operator=(TLogicalPalette &logicalpalette);
  bool SetAllQuads(RGBQUAD rgbquad);
};
//*******************************************************************
class TLogicalPalette
{
protected:
  friend TRGBQuadPalette;
  LOGPALETTE *PLogPal;

public:
  TLogicalPalette(LOGPALETTE &logpalette);
  TLogicalPalette(TLogicalPalette &logicalpalette);
  TLogicalPalette(TRGBQuadPalette &rgbquadpalette);
  TLogicalPalette(int16 numentries);
  virtual ~TLogicalPalette();
  bool CreateLogPalStruct(int16 numentries);
  void DestroyLogPalStruct();
  int16 GetNumEntries();
  operator const LOGPALETTE() const;
  operator const PLOGPALETTE() const;
  bool SetPalEntry(int16 index,PALETTEENTRY palentry);
  PALETTEENTRY GetPalEntry(int16 index); // returns black if invalid index.
  TLogicalPalette &operator=(TLogicalPalette &logicalpalette);
  TLogicalPalette &operator=(TRGBQuadPalette &rgbquadpalette);
  bool IsEqual(LOGPALETTE &logpalette);
  bool operator==(LOGPALETTE &logpalette);
  bool operator!=(LOGPALETTE &logpalette);
  bool SetAllEntries(PALETTEENTRY palentry);
  bool SetAllFlags(byte flag);
  bool SetFlag(int16 index,byte flag);
  bool SetToSystem(int16 start,int16 num);

  bool IsPalColorsEqual(PALETTEENTRY pal1,PALETTEENTRY pal2);
  bool IsSysColor(PALETTEENTRY palentry);
  void ModifyColor(int16 palindex);
  bool CheckSysColors();
};
//*******************************************************************
class TIdentityPalette:public TLogicalPalette
{
public:
  TIdentityPalette(TLogicalPalette &logicalpalette);
  virtual ~TIdentityPalette();
};
//*******************************************************************
class TPal:public TGr
{
protected:
  HPALETTE hPalette;
  TLogicalPalette *PLogicalPalette;
  TIdentityPalette *PIdentityPalette;
  TRGBQuadPalette *PRGBQuadPalette;

public:
  TPal();
  TPal(TOgleModeWindow *pmw,const TFileMode &fm,bool AutoRead = true,bool KeepBaseFile = true);
  TPal(TPal &pal);
  TPal(TOgleModeWindow *pmw,TPal &pal);
  TPal(RGBQUAD *prgbquads,int16 numquads);
  TPal(TOgleModeWindow *pmw,RGBQUAD *prgbquads,int16 numquads);
  virtual ~TPal();
  virtual string AddExt();
  operator HPALETTE() const;
  operator TRGBQuadPalette() const;
  TRGBQuadPalette *GetRGBQuadPalette();
  bool CreateRGBQuadPalette();
  void DestroyRGBQuadPalette();
  bool CreateLogicalPalette();
  void DestroyLogicalPalette();
  bool CreateIdentityPalette();
  void DestroyIdentityPalette();
  bool CreateHPALETTE();
  bool DeleteHPALETTE();
  bool IsHPALETTEValid();
  bool SetModeWindow(TOgleModeWindow *pmw);
  bool ClearSystemPalette();

protected:
  virtual bool ReadLocalHeader();
  virtual bool WriteLocalHeader();
  virtual bool ReadLocal();
  virtual bool WriteLocal();
};
//*******************************************************************
//******************** TLogicalPalette inlines  *********************
//*******************************************************************
inline int16 TLogicalPalette::GetNumEntries()
{
  if (PLogPal == NULL) {return 0;}
  return int16(PLogPal->palNumEntries);
//  return ((PLogPal !=NULL) ? PLogPal->palNumEntries : 0);
}
//*******************************************************************
inline TLogicalPalette::operator const LOGPALETTE() const
{
  return (*PLogPal);
}
//*******************************************************************
inline TLogicalPalette::operator const PLOGPALETTE() const
{
  return PLogPal;
}
//*******************************************************************
inline bool TLogicalPalette::IsPalColorsEqual(PALETTEENTRY pal1,PALETTEENTRY pal2)
{
  if (pal1.peRed == pal2.peRed &&
      pal1.peBlue == pal2.peBlue &&
      pal1.peGreen == pal2.peGreen)
  {
    return true;
  }
  return false;
}
//*******************************************************************
//************************ TPal inlines *****************************
//*******************************************************************
inline TPal::operator HPALETTE() const
{
  return hPalette;
}
//*******************************************************************
inline TPal::operator TRGBQuadPalette() const
{
  return *PRGBQuadPalette;
}
//*******************************************************************
inline TRGBQuadPalette *TPal::GetRGBQuadPalette()
{
  return PRGBQuadPalette;
}
//*******************************************************************
inline bool TPal::IsHPALETTEValid()
{
  return bool(hPalette != NULL);
}
//*******************************************************************
#endif

