//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oplaymgr.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     10/12/95   M.Perry    original baseline
//  2.0      8/24/97   M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__oplaymgr_h)
#define __oplaymgr_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__ogamedat_h)
#include "ogamedat.h"
#endif

#if !defined(__oplaydat_h)
#include "oplaydat.h"
#endif

#if !defined(__ERRNO_H)
#include <errno.h>
#endif

#if !defined(CLASSLIB_TIME_H)
#include <classlib\time.h>
#endif

class TWindow;
class OGLEOWLCLASS TOgleGameFrameWindow;
class TOgleIniFile;
class OGLEOWLCLASS TOgleApp;
//*******************************************************************
class TPlayerMgr
{
public:
  enum {MAXNUMPLAYERS = 20};
  
private:
  TOgleApp *POgleApp;
  TGameHeader *PGameHeader;
  TGameData *PGameData;
  int32 GameDataOffset;
  TPlayerData BasePlayerData[MAXNUMPLAYERS];
  TPlayerData *PPlayerData[MAXNUMPLAYERS];
  int32 PlayerDataOffsets[MAXNUMPLAYERS];
  int16 ActivePlayers[MAXNUMPLAYERS];
  string DriveDir;
  string FileName;
  int16 NumPlayersAllowed;
  int16 NumActiveAllowed;

public:
  int16 GetActivePlayerNum(int16 slotnum);
  TOgleApp *GetOgleApp() {return POgleApp;}
  TPlayerMgr(TOgleGameFrameWindow *parent,string drivedir,int16 numactiveallowed = 3,int16 numplayers = 20);
  virtual ~TPlayerMgr();
  virtual void ModeDeActivated();

  // Start of dialog interface.
  virtual void DoManagementDialog();
  virtual void DoStatisticsDialog(int16 playernum = -1,TWindow *parent = NULL);
  int16 GetPlayerNum(const string &name);
  int16 NameToIndex(const string &name);
  bool DoesNameExist(const string &name);
  int16 GetMaxNameLength();
  int16 GetNumPlayersAllowed();
  int16 GetNumActiveAllowed();
  int16 GetNumActivePlayers();
  bool IsPlayerValid(int16 playernum);
  bool IsPlayerValid(const string &name);
  string GetPlayerName(int16 playernum);
  TMoney GetPlayerCash(int16 playernum);
  TMoney GetPlayerCash(const string &name);
  bool IsPlayerMale(int16 playernum);
  bool IsPlayerMale(const string &name);
  bool IsPlayerActive(int16 playernum);
  bool IsPlayerActive(const string &name);
  virtual void PrePlayerActivation(int16 playernum,TWindow *pwin);
  bool ActivatePlayer(int16 playernum,TWindow *pwin = NULL);
  bool ActivatePlayer(const string &name);
  void ForceDeActivatePlayer(int16 slotnum);
  bool DeActivatePlayer(int16 playernum);
  bool DeActivatePlayer(const string &name);
  bool CreatePlayer(const string &name,bool Male,TMoney cash);
  bool RemovePlayer(int16 playernum);
  bool RemovePlayer(const string &name);
  bool CanCreatePlayer();
  bool CanActivatePlayer();
  void ReleaseUnActivePlayerData();

  virtual void CreateActivateDefPlayer();
  virtual void CreateDefPlayer();
  // End of dialog interface.

  // Start of TPlayer  & TGameLord Interface.
  TGameHeader *GetGameHeaderPtr();

  TGameData *GetGameDataPtr();

  TPlayerData *GetActivePlayerDataPtr(int16 slotnum);
  TPlayerData *GetPlayerDataPtr(int16 playernum);
  // End of TPlayer & TGameLord Interface.

protected:
  string strDefPlayerName;
  void SetDefPlayerName(string &name);
  void SetDefPlayerName(const char *pzname);
  TOgleGameFrameWindow *Parent;
  TOgleIniFile *GetIniFile();

  bool UpdateAllActiveBasePlayerData();
  bool UpdateBasePlayerData(int16 playernum);
  bool ReadBasePlayerData(int16 playernum);
  bool ReadAllBasePlayerData();

  bool CheckPlayerNum(int16 playernum);
  bool IsPlayerDataNull(int16 playernum);
  bool CheckSlotNum(int16 slotnum);
  // Query gamelord.
  bool CanChangePlayers();

  // Notify gamelord & players.
  void NotifyModePlayerActivation(int16 slotnum);
  void NotifyModePlayerDeActivation(int16 slotnum);

  bool Init();
  bool Save();

  // Header stuff.
  bool ReadGameHeader();
  bool WriteGameHeader();
  void DeleteGameHeader();
  virtual int32 GetGameHeaderSize();
  virtual TGameHeader *GetNewGameHeader();
  void ReleaseNewGameHeader(TGameHeader *pgh);

  // Game Data stuff.
  bool ReadGameData();
  bool WriteGameData();
  void DeleteGameData();
  virtual int32 GetGameDataSize();
  virtual TGameData *GetNewGameData();
  void ReleaseNewGameData(TGameData *pgd);

  // Player data stuff.
  bool ReadPlayerData(int16 playernum);
  bool WritePlayerData(int16 playernum);
  bool WriteAllPlayerData();
  void DeletePlayerData(int16 playernum);
  void DeleteAllPlayerData();
  virtual int32 GetPlayerDataSize();
  virtual TPlayerData *GetNewPlayerData();
  void ReleaseNewPlayerData(TPlayerData *ppd);
  virtual void ClearPlayerData(int16 playernum);

  virtual string GetSaveName();

  bool IsSlotActive(int16 slotnum);
  int16 GetNextOpenActiveSlot();
  bool SetActiveSlot(int16 slotnum,int16 playernum);
  bool SetNextOpenActiveSlot(int16 playernum);
  void ClearActiveSlot(int16 slotnum);
  int16 GetPlayerActiveSlotNum(int16 playernum);

  int16 GetNumValidPlayers();
  int16 GetOpenPlayerNum();

  string GetActivePlayerName(int16 slotnum);
  string GetSavedActiveName(int16 slotnum);
  void ActivateSavedPlayers();

private:
  bool ReadData(void *pdest,int32 offset,int32 size);
  void *AllocReadData(int32 offset,int32 size);
  void DeAllocData(void *p);
  bool WriteData(void *psource,int32 offset,int32 size);

  virtual int32 GetFileSize();
  bool CheckFileExists();
  bool CheckFileSize();
  bool VerifyFile();
  bool CheckVersion();

  void SetOffsets();
  bool ReadFile();
  bool WriteFile();
  bool CreateFileSpace();
  bool DeleteOldFile();
  bool CreateNewFile();
  bool SaveDefaultsDirectToFile();

  void SaveActiveNames();
};
//*******************************************************************
//************************  inlines *********************************
//*******************************************************************
inline int16 TPlayerMgr::NameToIndex(const string &name)
{
  return GetPlayerNum(name);
}
//*******************************************************************
inline bool TPlayerMgr::DoesNameExist(const string &name)
{
  return (CheckPlayerNum(GetPlayerNum(name)));
}
//*******************************************************************
inline int16 TPlayerMgr::GetMaxNameLength()
{
  return (TPlayerData::NAMELENGTH - 1);
}
//*******************************************************************
inline int16 TPlayerMgr::GetNumPlayersAllowed()
{
  return NumPlayersAllowed;
}
//*******************************************************************
inline int16 TPlayerMgr::GetNumActiveAllowed()
{
  return NumActiveAllowed;
}
//*******************************************************************
inline bool TPlayerMgr::IsPlayerValid(const string &name)
{
  return IsPlayerValid(GetPlayerNum(name));
}
//*******************************************************************
inline TMoney TPlayerMgr::GetPlayerCash(const string &name)
{
  return GetPlayerCash(GetPlayerNum(name));
}
//*******************************************************************
inline bool TPlayerMgr::IsPlayerMale(const string &name)
{
  return IsPlayerMale(GetPlayerNum(name));
}
//*******************************************************************
inline bool TPlayerMgr::IsPlayerActive(const string &name)
{
  return IsPlayerActive(GetPlayerNum(name));
}
//*******************************************************************
inline bool TPlayerMgr::ActivatePlayer(const string &name)
{
  return ActivatePlayer(GetPlayerNum(name));
}
//*******************************************************************
inline bool TPlayerMgr::DeActivatePlayer(const string &name)
{
  return DeActivatePlayer(GetPlayerNum(name));
}
//*******************************************************************
inline bool TPlayerMgr::RemovePlayer(const string &name)
{
  return RemovePlayer(GetPlayerNum(name));
}
//*******************************************************************
inline bool TPlayerMgr::CanActivatePlayer()
{
  return (GetNumActivePlayers() < GetNumActiveAllowed());
}
//*******************************************************************
inline TGameHeader *TPlayerMgr::GetGameHeaderPtr()
{
  return PGameHeader;
}
//*******************************************************************
inline TGameData *TPlayerMgr::GetGameDataPtr()
{
  return PGameData;
}
//*******************************************************************
inline TPlayerData *TPlayerMgr::GetActivePlayerDataPtr(int16 slotnum)
{
  if (!IsSlotActive(slotnum)) {return NULL;}
  if (!CheckPlayerNum(ActivePlayers[slotnum])) {return NULL;}
  return PPlayerData[ActivePlayers[slotnum]];
}
//*******************************************************************
inline TPlayerData *TPlayerMgr::GetPlayerDataPtr(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return NULL;}
  if (IsPlayerDataNull(playernum)) {ReadPlayerData(playernum);}
  return PPlayerData[playernum];
}
//*******************************************************************
inline void TPlayerMgr::SetDefPlayerName(string &name)
{
  strDefPlayerName = name;
}
//*******************************************************************
inline void TPlayerMgr::SetDefPlayerName(const char *pzname)
{
  strDefPlayerName = pzname;
}
//*******************************************************************
inline bool TPlayerMgr::CheckPlayerNum(int16 playernum)
{
  if ((playernum < 0) || (playernum >= NumPlayersAllowed)) {return false;}
  return true;
}
//*******************************************************************
inline bool TPlayerMgr::IsPlayerDataNull(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return true;}
  return (PPlayerData[playernum] == NULL);
}
//*******************************************************************
inline bool TPlayerMgr::CheckSlotNum(int16 slotnum)
{
  if ((slotnum < 0) || (slotnum >= NumActiveAllowed)) {return false;}
  return true;
}
//*******************************************************************
inline void TPlayerMgr::DeleteGameHeader()
{
  DeAllocData(PGameHeader);
  PGameHeader = NULL;
}
//*******************************************************************
inline void TPlayerMgr::DeleteGameData()
{
  DeAllocData(PGameData);
  PGameData = NULL;
}
//*******************************************************************
inline string TPlayerMgr::GetActivePlayerName(int16 slotnum)
{
  return GetPlayerName(GetActivePlayerNum(slotnum));
}
//*******************************************************************
#endif

