//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osprcell.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     6/20/95    M.Perry    original baseline
//  2.0     8/21/97    M.Perry    32-bit conversion.
//  2.1     12/6/97    M.Perry    filename mod's.
//********************************************************************
#if !defined(__osprcell_h) 
#define __osprcell_h

#if !defined(__ocell_h)
#include "ocell.h"
#endif

#if !defined(__oanm_h)
#include "oanm.h"
#endif

#if !defined(__owav_h)
#include "owav.h"
#endif

#if !defined(__osprdefs_h)
#include "osprdefs.h"
#endif

class TSprite;
//*******************************************************************
class TSpriteCell:public TOgleCell
{
public:
  TSprite *POwnerSprite;
  bool IsAnm;
//  bool IsForward;
  bool OverrideDefaultXY;
  int16 X,Y,Z;
  TDirection Direction;
  int16 StartFrame;

public:
  TSpriteCell();
  TSpriteCell(TOgleModeWindow *pmw,TFileMode &NewFileMode);
  TSpriteCell(TOgleModeWindow *pmw,TFileMode &NewFileMode,TSprite *PSprite,
              void *PReadData,bool AutoRead = true);
  virtual ~TSpriteCell();
  TSpriteCell &operator=(const TSpriteCell &sprcell);
  virtual void Assign(const TSpriteCell &sprcell);
  virtual void ClearCell();

  virtual bool ReadLocal();
  bool ReadLocal1_2();
  bool ReadLocal2_0();
  bool ReadLocal2_3();
  bool ReadLocal2_4();
  virtual bool WriteLocal();
  virtual void SetMinVer();
  virtual void SetMajVer();
  virtual void AddToOwnersList(int16 RowNum);
  TAnm *CreateAnm(int16 RowNum);
  TWav *CreateWav(int16 RowNum);

  void SetX(int16 NewX);
  int16 GetX() const;
  void SetY(int16 NewY);
  int16 GetY() const;
  void SetZ(int16 NewZ);
  int16 GetZ() const;
  void SetXYZ(int16 NewX,int16 NewY,int16 NewZ);

  void SetIsAnm(bool NewIsAnm);
  bool GetIsAnm() const;
  bool GetIsWav() const;
  void SetIsAnm();
  void SetIsWav();
  bool GetForward() const;
  void SetIsForward(bool NewIsForward);
  bool GetIsForward() const;
  bool GetIsReverse() const;
  void SetForward();
  void SetReverse();
  void SetOverrideDefaultXY();
  void ClearOverrideDefaultXY();
  bool GetOverrideDefaultXY() const;

  TDirection GetDirection() const
  {
    return Direction;
  }
  void SetDirection(TDirection direction)
  {
    Direction = direction;
  }
  int16 GetStartFrame() const {return StartFrame;}
  void SetStartFrame(int16 startframe) {StartFrame = startframe;}
};
//*******************************************************************
//*********************  inlines  ***********************************
//*******************************************************************
inline void TSpriteCell::SetX(int16 NewX)
{
  X = NewX;
}
//*******************************************************************
inline int16 TSpriteCell::GetX() const
{
  return X;
}
//*******************************************************************
inline void TSpriteCell::SetY(int16 NewY)
{
  Y = NewY;
}
//*******************************************************************
inline int16 TSpriteCell::GetY() const
{
  return Y;
}
//*******************************************************************
inline void TSpriteCell::SetZ(int16 NewZ)
{
  Z = NewZ;
}
//*******************************************************************
inline int16 TSpriteCell::GetZ() const
{
  return Z;
}
//*******************************************************************
inline void TSpriteCell::SetXYZ(int16 NewX,int16 NewY,int16 NewZ)
{
  X = NewX;
  Y = NewY;
  Z = NewZ;
}
//*******************************************************************
inline void TSpriteCell::SetIsAnm(bool NewIsAnm)
{
  IsAnm = NewIsAnm;
}
//*******************************************************************
inline bool TSpriteCell::GetIsAnm() const
{
  return IsAnm;
}
//*******************************************************************
inline bool TSpriteCell::GetIsWav() const
{
  return !IsAnm;
}
//*******************************************************************
inline void TSpriteCell::SetIsAnm()
{
  IsAnm = true;
}
//*******************************************************************
inline void TSpriteCell::SetIsWav()
{
  IsAnm = false;
}
//*******************************************************************
inline bool TSpriteCell::GetForward() const
{
//  return IsForward;
  return bool(GetDirection() == dForward);
}
//*******************************************************************
inline void TSpriteCell::SetIsForward(bool NewIsForward)
{
//  IsForward = NewIsForward;
  if (NewIsForward)
  {
    Direction = dForward;
  }
  else
  {
    Direction = dReverse;
  }
}
//*******************************************************************
inline bool TSpriteCell::GetIsForward() const
{
//  return IsForward;
  return bool(GetDirection() == dForward);
}
//*******************************************************************
inline bool TSpriteCell::GetIsReverse() const
{
//  return !IsForward;
  return bool(GetDirection() == dReverse);
}
//*******************************************************************
inline void TSpriteCell::SetForward()
{
//  IsForward = true;
  Direction = dForward;
}
//*******************************************************************
inline void TSpriteCell::SetReverse()
{
//  IsForward = false;
  Direction = dReverse;
}
//*******************************************************************
inline void TSpriteCell::SetOverrideDefaultXY()
{
  OverrideDefaultXY = true;
}
//*******************************************************************
inline void TSpriteCell::ClearOverrideDefaultXY()
{
  OverrideDefaultXY = false;
}
//*******************************************************************
inline bool TSpriteCell::GetOverrideDefaultXY() const
{
  return OverrideDefaultXY;
}
//*******************************************************************
#endif

