//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oSprite.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/11/95    M.Perry     original baseline
//  1.3     5/29/97    M.Perry     added idcode.
//  2.0     8/21/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__osprite_h)
#define __osprite_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__obase_h)
#include "obase.h"
#endif

#if !defined(__oanmlst_h)
#include "oanmlst.h"
#endif

#if !defined(__owavlst_h)
#include "owavlst.h"
#endif

#if defined(DEBUGREPORTS) || defined(DEBUGSPRITES)
  #include <iostream.h>
  #include <fstream.h>
#endif

#if !defined(__ogeo_h)
#include "ogeo.h"
#endif

// possible future work.
// change unk to -1 when wanting to add support for old spritelists
// that have sprite id's as -1, which means unk, whereas 0 means
// none. None means it's been looked at, where -1 means fake it.
#define SPRITEID_UNK  0
#define SPRITEID_NONE  0

// start sprite stat flags
#define osprFIRST ((WORD)(obLAST * 2))
#define osprMANUAL (osprFIRST)
#define osprOVERRIDEDEFAULTFUCLENGTH ((WORD)(osprMANUAL * 2))
#define osprANMLISTSTOPPED ((WORD)(osprOVERRIDEDEFAULTFUCLENGTH * 2))
#define osprWAVLISTSTOPPED ((WORD)(osprANMLISTSTOPPED * 2))
#define osprLAST (osprWAVLISTSTOPPED)
// end sprite stat flags

class OGLEOWLCLASS TOgleModeWindow;
class TAnm;
class TWav;
class TSpriteTable;
class TAnmData;
//*******************************************************************
class TSprite:public TOgleBase
{
protected:
  TAnmList *PAnmList;
  TWavList *PWavList;
  TSpriteTable *PSpriteTable;
  TTriplet StartVPt,CurrentVPt;
  int16 IDCode;

public:
  int16 GetIDCode() {return IDCode;}
  void SetIDCode(int16 idcode) {IDCode = idcode;}
  void LoadRamAnmList(TFMStatus ramstatus1,TFMStatus ramstatus2);
  void LoadRamWavList(TFMStatus ramstatus1,TFMStatus ramstatus2);
  int16 GetCurrentAnmFrame(int16 anmnum = 0);
  void SetCurrentAnmFrame(int16 framenum,int16 anmnum = 0);
  bool Priority(int16 count = 1);
  void SetToManual();
  void SetManual();
  void ClearManual();
  bool GetManual();
  #ifdef DEBUGREPORTS
  void Report(ofstream &os);
  #endif
  #ifdef DEBUGSPRITES
  void RTReport(ofstream &os);
  #endif
  void SetAllAnmCADSRelative();
  void SetStartingX(int16 x);
  int16 GetStartingX();
  void SetStartingY(int16 y);
  int16 GetStartingY();
  void SetStartingZ(int16 z);
  int16 GetStartingZ();
  void SetStartingXYZ(int16 x,int16 y,int16 z);
  void SetStartingVPt(TTriplet &vpt);
  TTriplet &GetStartingVPt();
  void SetCurrentX(int16 x);
  int16 GetCurrentX();
  void SetCurrentY(int16 y);
  int16 GetCurrentY();
  void SetCurrentZ(int16 z);
  int16 GetCurrentZ();
  void SetCurrentXYZ(int16 x,int16 y,int16 z);
  void SetCurrentVPt(TTriplet &vpt);
  TTriplet &GetCurrentVPt();
  void ResetCurrentXYZ();

  dword GetCalcFUCLength();
  bool Is_FUC_Override();
  void Set_FUC_Override();
  void Clear_FUC_Override();

  virtual string AddExt();
  virtual void SetMinVer();
  virtual bool ReadLocal();
  virtual bool WriteLocal();
  virtual const char *GetIDChars() const;
  TSprite(TOgleModeWindow *pmw,const TFileMode &NewMode,void *psource);
  TSprite(TOgleModeWindow *pmw,const TFileMode &NewMode,
          bool AutoRead = true,bool KeepBaseFile = false);
  TSprite(TOgleModeWindow *pmw);
  virtual ~TSprite();
  virtual void Update(dword NewFUC);
  virtual void PostUpdate(dword NewFUC);
  virtual void Start(dword StartFUC);
  virtual void Start();
  virtual void Stop();
  virtual void Show();
  virtual void NoShow();
  virtual void Activate(dword StartFUC);
  virtual void DeActivate();
  virtual bool Save();
  bool AddAnm(TAnm *PNewAnm);
  bool AddWav(TWav *PNewWav);
  bool AddAnm(TFileMode NewFileMode,int16 NewScreenZ = 1);
  bool AddAnm(TFileMode NewFileMode,int16 NewScreenX,int16 NewScreenY, int16 NewScreenZ);
  bool AddWav(TFileMode NewFileMode,int16 NewFUCLength = -1);
  void StartWavNow(int16 wavindex);
  TSpriteTable *GetTable() {return PSpriteTable;}
  void ReleaseTable();
  virtual void Slenderize();
  TAnmList *GetAnmList() {return PAnmList;}
  TWavList *GetWavList() {return PWavList;}
  TAnm *GetAnm(int16 AnmIndex);
  TAnm *GetAnm(string &AnmFName);
  TWav *GetWav(int16 WavIndex);
  TWav *GetWav(string &WavFName);
  virtual TAnm *CreateAnm(TFileMode &fm,TAnmData &ad);
  virtual TWav *CreateWav(TFileMode &fm,int16 fuclength = -1);
  void NotifyStop(TAnmList *panmlist);
  void NotifyStop(TWavList *pwavlist);
  void SetAnmListStopped();
  void ClearAnmListStopped();
  bool GetAnmListStopped();
  void SetWavListStopped();
  void ClearWavListStopped();
  bool GetWavListStopped();
protected:
  void AnmError(string spritename,string descr,int16 num = -1);
  void WavError(string spritename,string descr,int16 num = -1);
  bool ReadLocal1_0();
  bool ReadLocal1_1();
};
//*******************************************************************
//*********************** inlines ***********************************
//*******************************************************************
inline void TSprite::NotifyStop(TAnmList *panmlist)
// if wav list is also stopped then we're stopped.
{
  SetAnmListStopped();
  if (GetManual()) {return;}
  if (GetWavListStopped())
  {
    SetStopped();
  }
}
//*******************************************************************
inline void TSprite::NotifyStop(TWavList *pwavlist)
// if anm list is also stopped then we're stopped.
{
  SetWavListStopped();
  if (GetManual()) {return;}
  if (GetAnmListStopped())
  {
    SetStopped();
  }
}
//*******************************************************************
inline void TSprite::SetAnmListStopped()
{
  StatFlags.Set(osprANMLISTSTOPPED);
}
//*******************************************************************
inline void TSprite::ClearAnmListStopped()
{
  StatFlags.Clear(osprANMLISTSTOPPED);
}
//*******************************************************************
inline bool TSprite::GetAnmListStopped()
{
  return StatFlags.IsSet(osprANMLISTSTOPPED);
}
//*******************************************************************
inline void TSprite::SetWavListStopped()
{
  StatFlags.Set(osprWAVLISTSTOPPED);
}
//*******************************************************************
inline void TSprite::ClearWavListStopped()
{
  StatFlags.Clear(osprWAVLISTSTOPPED);
}
//*******************************************************************
inline bool TSprite::GetWavListStopped()
{
  return StatFlags.IsSet(osprWAVLISTSTOPPED);
}
//*******************************************************************
inline void TSprite::SetManual()
{
  StatFlags.Set(osprMANUAL);
}
//*******************************************************************
inline void TSprite::ClearManual()
{
  StatFlags.Clear(osprMANUAL);
}
//*******************************************************************
inline bool TSprite::GetManual()
{
  return StatFlags.IsSet(osprMANUAL);
}
//*******************************************************************
inline void TSprite::SetStartingX(int16 x)
{
  StartVPt.X = x;
}
//*******************************************************************
inline int16 TSprite::GetStartingX()
{
  return StartVPt.X;
}
//*******************************************************************
inline void TSprite::SetStartingY(int16 y)
{
  StartVPt.Y = y;
}
//*******************************************************************
inline int16 TSprite::GetStartingY()
{
  return StartVPt.Y;
}
//*******************************************************************
inline void TSprite::SetStartingZ(int16 z)
{
  StartVPt.Z = z;
}
//*******************************************************************
inline int16 TSprite::GetStartingZ()
{
  return StartVPt.Z;
}
//*******************************************************************
inline void TSprite::SetStartingXYZ(int16 x,int16 y,int16 z)
{
  StartVPt.X = x;
  StartVPt.Y = y;
  StartVPt.Z = z;
}
//*******************************************************************
inline void TSprite::SetStartingVPt(TTriplet &vpt)
{
  StartVPt = vpt;
}
//*******************************************************************
inline TTriplet &TSprite::GetStartingVPt()
{
  return StartVPt;
}
//*******************************************************************
inline void TSprite::SetCurrentX(int16 x)
{
  CurrentVPt.X = x;
}
//*******************************************************************
inline int16 TSprite::GetCurrentX()
{
  return CurrentVPt.X;
}
//*******************************************************************
inline void TSprite::SetCurrentY(int16 y)
{
  CurrentVPt.Y = y;
}
//*******************************************************************
inline int16 TSprite::GetCurrentY()
{
  return CurrentVPt.Y;
}
//*******************************************************************
inline void TSprite::SetCurrentZ(int16 z)
{
  CurrentVPt.Z = z;
}
//*******************************************************************
inline int16 TSprite::GetCurrentZ()
{
  return CurrentVPt.Z;
}
//*******************************************************************
inline void TSprite::SetCurrentXYZ(int16 x,int16 y,int16 z)
{
  CurrentVPt.X = x;
  CurrentVPt.Y = y;
  CurrentVPt.Z = z;
}
//*******************************************************************
inline void TSprite::SetCurrentVPt(TTriplet &vpt)
{
  CurrentVPt = vpt;
}
//*******************************************************************
inline TTriplet &TSprite::GetCurrentVPt()
{
  return CurrentVPt;
}
//*******************************************************************
inline void TSprite::ResetCurrentXYZ()
{
  CurrentVPt = StartVPt;
}
//*******************************************************************
inline bool TSprite::Is_FUC_Override()
{
  return StatFlags.IsSet(osprOVERRIDEDEFAULTFUCLENGTH);
}
//*******************************************************************
inline void TSprite::Set_FUC_Override()
{
  StatFlags.Set(osprOVERRIDEDEFAULTFUCLENGTH);
}
//*******************************************************************
inline void TSprite::Clear_FUC_Override()
{
  StatFlags.Clear(osprOVERRIDEDEFAULTFUCLENGTH);
}
//*******************************************************************
inline TAnm *TSprite::GetAnm(int16 AnmIndex)
{
  if (GetAnmList() == NULL) {return NULL;}
  return (GetAnmList()->GetAnmPtr(AnmIndex));
}
//*******************************************************************
inline TAnm *TSprite::GetAnm(string &AnmFName)
{
  if (GetAnmList() == NULL) {return NULL;}
  return (GetAnmList()->GetAnmPtr(AnmFName));
}
//*******************************************************************
inline TWav *TSprite::GetWav(int16 WavIndex)
{
  if (GetWavList() == NULL) {return NULL;}
  return (GetWavList()->GetWavPtr(WavIndex));
}
//*******************************************************************
inline TWav *TSprite::GetWav(string &WavFName)
{
  if (GetWavList() == NULL) {return NULL;}
  return (GetWavList()->GetWavPtr(WavFName));
}
//*******************************************************************
#endif

