//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oinifile.cpp
//    Project Name:  Ogle                          
// Original Author:  Michael R. Perry
//                                      
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/5/96     M.Perry     original baseline
//  2.0     8/24/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__oinifile_h)
#include "oinifile.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif
//*******************************************************************
TOgleIniFile::TOgleIniFile(string const &hddrive,string const &hddir,
                           string const &exename,string const &inifilename,
                           bool setdefs)
:TIniFile(hddrive,hddir,inifilename)
{
  SetExeName(exename);
  if (setdefs) {SetToDefaults();}
}
//*******************************************************************
TOgleIniFile::~TOgleIniFile()
{
}
//*******************************************************************
void TOgleIniFile::SetToDefaults()
{
  // soundmgr
  SetEffects(1);
  SetMusic(1);
  SoundSect = "sound";
  EffectsKey = "effects";
  MusicKey = "music";

  //filemgr
  SetCDDrive(string("c:"));
  SetCDDir(string("\\"));
  DirSect = "location";
  CDDriveKey = "cddrive";
  CDDirKey = "cddir";
  SysSect = "System";
  HDBufferKey = "HDBuffering";
  SetHDBuffering(1);

  //playermgr
  PlayerSect = "Players";

  // modewin
  DetailLevelKey = "Detail Level";
}
//*******************************************************************
void TOgleIniFile::ReadData()
{
  ReadCDDrive();
  ReadCDDir();
  ReadHDBuffering();
  ReadEffects();
  ReadMusic();
}
//*******************************************************************
void TOgleIniFile::WriteData()
{
  WriteCDDrive();
  WriteCDDir();
  WriteHDBuffering();
  WriteEffects();
  WriteMusic();
}
//*******************************************************************
const string &TOgleIniFile::SetCDDrive(const string &cddrive)
{
  string s = cddrive;
  CDDrive = s.strip(string::Trailing,'\\');
  return CDDrive;
}
//*******************************************************************
const string &TOgleIniFile::SetCDDir(const string &cddir)
{
  string s = cddir;
  if (s.find_first_of("\\") != 0)
  {
    s.prepend('\\');
  }
  CDDir = s;
  return CDDir;
}
//*******************************************************************
string TOgleIniFile::ReadPlayerName(int slotnum)
{
  return ReadString(PlayerSect,slotnum,string(""));
}
//*******************************************************************
string TOgleIniFile::GetWarnSection()
{
  return string("Warning Msgs");
}
//*******************************************************************
bool TOgleIniFile::GetWarnEnbDef(TWarnEnum warn)
{
//  static bool WarnEnumDef[WARNENUM_MAX] = {false,false};
//  if (!CheckWarnNum(warn)) {return true;}
//  return WarnEnumDef[warn];
  TWarnEnum w = warn;
  return true;
}
//*******************************************************************
string TOgleIniFile::GetWarnKeys(TWarnEnum warn)
{
  static string strKeys[] = {"BPP_LT_RECOM","BPP_GT_RECOM"};
  if (!CheckWarnNum(warn)) {return TAppString::strEmpty;}
  return strKeys[warn];
}
//*******************************************************************
bool TOgleIniFile::GetWarnMsg(TWarnEnum warn)
{
  if (!CheckWarnNum(warn)) {return true;}
  return Readbool(GetWarnSection(),
                  GetWarnKeys(warn),
                  GetWarnEnbDef(warn));
}
//*******************************************************************
bool TOgleIniFile::SetWarnMsg(TWarnEnum warn,bool enb)
{
  if (!CheckWarnNum(warn)) {return false;}
  return Writebool(GetWarnSection(),
                   GetWarnKeys(warn),
                   enb);
}
//*******************************************************************


