//*******************************f*************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oapp.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0    02/27/95    M.Perry     original baseline
//  2.0    08/25/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__omisc_h)
#include "omisc.h"
#endif

#if !defined(__ofilemgr_h)
#include "ofilemgr.h"
#endif

#include <assert.h>

#if !defined(__ogamefw_h)
#include "ogamefw.h"
#endif

#if !defined(__ocurmgr_h)
#include "ocurmgr.h"
#endif

#if !defined(__ogame_h)
#include "ogame.h"
#endif

#if !defined(__COMPLEX_H)
#include <complex.h>
#endif

#if !defined(__oinifile_h)
#include "oinifile.h"
#endif

#if !defined(__opal_h)
#include "opal.h"
#endif

#if !defined(__odc_h)
#include "odc.h"
#endif

#if !defined(__owarnmsg_h)
#include "owarnmsg.h"
#endif

DIAG_DEFINE_GROUP(App,1,0);
//*******************************************************************
#if !defined(OGLE_DLL)
TOgleApp *TOgleApp::PAppHolder = NULL;
#endif
//*******************************************************************
const string TAppString::strEmpty = "";
//*******************************************************************
void TOgleApp::GetAppInfo(TOgleAppInfo &appinfo)
{
  appinfo.MinDisplaySize = TSize(640,480);
  appinfo.MinBitsPerPixel = 8;
  appinfo.MinNumColors = 256;
  appinfo.MainTitle = "Ogle App";
  appinfo.TechFlags.Set(oaiVT_WING);
  appinfo.TechFlags.Set(oaiAT_WAVEMIX);
  appinfo.TechFlags.Set(oaiMT_MCI);
  appinfo.MajVer = GetAppType();
  appinfo.MinVer = GetAppSubType();
}
//*******************************************************************
int16 TOgleApp::GetAppType()
{
  return 0;
}
//*******************************************************************
int16 TOgleApp::GetAppSubType()
{
  return 0;
}
//*******************************************************************
string TOgleApp::GetDetName()
{
  return string("");
}
//*******************************************************************
void TOgleApp::RegisterDet()
{
  PFileMgr->RegisterDet(GetDetName());
}
//*******************************************************************
string TOgleApp::GetHelpName()
{
  return string(AppName + ".hlp");
}
//*******************************************************************
string TOgleApp::GetHelpPath()
{
  string s = GetFileMgr()->PHDD->GetDrive();
  s += GetFileMgr()->PHDD->GetDir();
  return s;
}
//*******************************************************************
string TOgleApp::GetHelpPathName()
{
  string s = GetHelpPath();
  s += GetHelpName();
  return s;
}
//*******************************************************************
bool TOgleApp::Help(uint command,uint32 data)
{
  if (GetFW() == NULL) {return false;}
  return GetFW()->WinHelp(GetHelpPathName().c_str(),command,data);
}
//*******************************************************************
bool TOgleApp::CreateCursorMgr()
{
  PCursorMgr = new TCursorMgr(this);
  return true;
}
//*******************************************************************
bool TOgleApp::DestroyCursorMgr()
{
  delete PCursorMgr;
  PCursorMgr = NULL;
  return true;
}
//*******************************************************************
#if defined(OGLE_DLL)
TOgleApp *TOgleApp::GetOgleApp(bool fatal)
{
  // ask owl's global app dictionary for the current process's app ptr.
  TApplication *papp = ::OwlAppDictionary.GetApplication();
  if (papp)
  {
    // dynamically cast it to our derived (TOgleApp *) and see if it worked.
    TOgleApp *poapp = dynamic_cast<TOgleApp *>(papp);

    if (poapp != 0) // hey! It worked! Let's return it to the caller.
    {
      return poapp;
    }
    // oops! Wasn't our kind of ptr, now what?
    else if (fatal) // did caller say fatal?
    {
      // something weird happened,
      TXBase("Invalid application type").Throw();
    }
    else // oops? But not fatal.
    {
      return NULL;
    }
  }
  else if (fatal) // couldn't find it in the dictionary,and caller said fatal, so bail.
  {
    TXBase("Missing Ogle App Ptr").Throw();
  }
  // couldn't find it in the dictionary, but caller didn't want fatal, so return NULL.
  return NULL;
}
#endif
//*******************************************************************
void TOgleApp::CreateFileMgr()
{
  PFileMgr = new TOgleFileMgr(this);
}
//*******************************************************************
void TOgleApp::DestroyFileMgr()
{
  delete PFileMgr;
  PFileMgr = NULL;
}
//*******************************************************************
void TOgleApp::ChangePalette(string NewPaletteName)
{
  PGame->SetPaletteName(NewPaletteName);
  DestroyPalette();
  CreatePalette();
}
//*******************************************************************
void TOgleApp::CreatePalette()
{
  MasterPaletteFileName = PGame->GetPaletteName();

  PMasterPalette = new TPal(NULL,TFileMode(MasterPaletteFileName,
                                           TFileMode::fm_None,
                                           TFileMode::fm_Immed));

  assert(!PMasterPalette->IsFileNotFound());
}
//*******************************************************************
void TOgleApp::DestroyPalette()
{
  if (PMasterPalette != NULL)
  {
    delete PMasterPalette;
    PMasterPalette = NULL;
  }
}
//*******************************************************************
void TOgleApp::CreateGameFile()
{
  PGame = new TOgleGame(GameFileName);
}
//*******************************************************************
void TOgleApp::DestroyGameFile()
{
  if (PGame != NULL)
  {
    delete PGame;
    PGame = NULL;
  }
}
//*******************************************************************
void TOgleApp::CreateIniFile()
{
  PIniFile = new TOgleIniFile(HDDrive,HDDir,AppName);
}
//*******************************************************************
void TOgleApp::DestroyIniFile()
{
  if (PIniFile != NULL)
  {
    delete PIniFile;
    PIniFile = NULL;
  }
}
//*******************************************************************
void TOgleApp::MaxPrevApp(HWND hwnd)
{
  HWND lastpopup = GetLastActivePopup(hwnd);
  ::BringWindowToTop(lastpopup);
  if (::IsIconic(lastpopup))
  {
    ::OpenIcon(lastpopup);
    RECT rc;
    ::GetWindowRect(lastpopup,&rc);
    if (MaxFX)
    {
      float f;
      for (f=0.0;f<6.28;f+=0.15)
      {
        ::MoveWindow(lastpopup,
                     rc.left - (cos(6.28)*100) + (cos(f)*100),
                     rc.top - (sin(6.28)*100) + (sin(f)*100),
                     rc.right,
                     rc.bottom,
                     true);
        ::UpdateWindow(lastpopup);
      }
      for(f = 6.28;f>0.0;f-=0.15)
      {
        ::MoveWindow(lastpopup,
                     rc.left + (cos(6.28)*100) - (cos(f)*100),
                     rc.top + (sin(6.28)*100) - (sin(f)*100),
                     rc.right,
                     rc.bottom,
                     true);
        ::UpdateWindow(lastpopup);
      }
    }
    ::MoveWindow(lastpopup,
                 rc.left,
                 rc.top,
                 rc.right - rc.left,
                 rc.bottom - rc.top,
                 true);
    ::UpdateWindow(lastpopup);
  }
}
//*******************************************************************
bool TOgleApp::CheckPrevInst()
//note: return true if already running.
{
  #if defined(BI_PLAT_WIN16)
  if (hPrevInstance == 0) {return false;} // only works with Win 3.x
                                          // was only a quick out anyway.
  #endif

  TOgleAppInfo appinfo;
  GetAppInfo(appinfo);
  HWND prevFW = (HWND)::FindWindow(NULL,appinfo.MainTitle.c_str());
  if (prevFW != NULL)
  {
    MaxPrevApp(prevFW);
    return true;
  }
  return false;
}
//*******************************************************************
bool TOgleApp::CheckDisplayRes()
{
  string strxres,stryres,strbpp,strnumcolors;
  TOgleAppInfo appinfo;
  GetAppInfo(appinfo);
  char buffer[17];
  itoa(appinfo.MinDisplaySize.cx,buffer,10);
  strxres = buffer;
  itoa(appinfo.MinDisplaySize.cy,buffer,10);
  stryres = buffer;
  itoa(appinfo.MinBitsPerPixel,buffer,10);
  strbpp = buffer;
  itoa(int(appinfo.MinNumColors),buffer,10);
  strnumcolors = buffer;
  TOgleScreenDC *pscreendc = new TOgleScreenDC;
  TSize DisplaySize(pscreendc->GetDeviceCaps(HORZRES),
                    pscreendc->GetDeviceCaps(VERTRES));
  if (DisplaySize.cx < appinfo.MinDisplaySize.cx || DisplaySize.cy < appinfo.MinDisplaySize.cy)
  {
    string s = "Display resolution must be at least ";
    s += strxres;
    s += " by ";
    s += stryres;
    s += ", and the number of colors recommended is ";
    s += strnumcolors;
    s += ".\n";
    s += "Please install an appropriate SVGA video driver.";
    ::MessageBox(NULL,s.c_str(),"Fatal Error",MB_OK);
    delete pscreendc;
    return true;
  }
  int bpp = pscreendc->GetDeviceCaps(BITSPIXEL);
  if (bpp != appinfo.MinBitsPerPixel)
  {
    if (bpp < appinfo.MinBitsPerPixel && GetIniFile() != NULL &&
        GetIniFile()->GetWarnMsg(TOgleIniFile::BPP_LT_RECOM))
    {
      string s = "The number of colors detected for the current video driver is less than ";
      s += strnumcolors;
      s += ".\nThis will result in extremely poor color quality and poor performance.";
      string title = "Warning";
      TWarnMsgDlg::CreateExecute(s,title,TOgleIniFile::BPP_LT_RECOM,true);
//      ::MessageBox(NULL,s.c_str(),"Warning",MB_OK);
    }
    else if ((GetIniFile() != NULL &&
              GetIniFile()->GetWarnMsg(TOgleIniFile::BPP_GT_RECOM)) ||
              GetIniFile() == NULL)
         // would be nice to have a checkbox on dlg,ie. "don't show anymore" flag.
         // if (GetIniFile()->GetWarnMsgEnb(TIniFile::ENB_BPP_GT_RECOM_WARN_MSG))
    {
      string s = "The number of colors detected for the current video driver is greater than ";
      s += strnumcolors;
      s += ". Depending on your system this could result in slightly slower performance.";
      string title = "Warning";
      TWarnMsgDlg::CreateExecute(s,title,TOgleIniFile::BPP_GT_RECOM,true);
//      ::MessageBox(NULL,s.c_str(),"Warning",MB_OK);
    }
  }
  delete pscreendc;
  return false;
}
//*******************************************************************
bool TOgleApp::Initialize()
{
  if (IsInitialized()) {return true;}

  CreateIniFile();

  if (CheckPrevInst()) {return false;}
  if (CheckDisplayRes()) {return false;}

  CreateCursorMgr();
//  CreateIniFile(); // moved before checkxxx()'s
  CreateFileMgr();
  RegisterDet();
  CreateGameFile();
  CreatePalette();
  Initialized = true;
  return true;
}
//*******************************************************************
bool TOgleApp::DeInitialize()
{
  if (!IsInitialized()) {return true;}

  delete PMasterPalette;
  PMasterPalette = NULL;

  delete PGame;
  PGame = NULL;

  DestroyFileMgr();

  // mrp - v2.4 - 09/03/99 - added for 1st run and save file creation msgs.
  if (GetIniFile() != NULL)
  {
    GetIniFile()->SetPrevRun(true);
  }

  DestroyIniFile();
  DestroyCursorMgr();

  Initialized = false;
  return true;
}
//*******************************************************************
TOgleSoundMgr *TOgleApp::GetSoundMgr()
{
//  return PSoundMgr;
  if (GetFW() == NULL)
  {
    return NULL;
  }
  return GetFW()->GetSoundMgr();
}
//*******************************************************************
TOgleApp::TOgleApp (string NewHDDrive,string NewHDDir,string NewAppName)
:TApplication(NewAppName.c_str())
{
  PGame = NULL;
  PFileMgr = NULL;
  PMasterPalette = NULL;
//  PSoundMgr = NULL;
  PCursorMgr = NULL;
  PIniFile = NULL;
  Initialized = false;
  HDDrive = NewHDDrive;
  HDDir = NewHDDir;
  AppName = NewAppName;
  MaxFX = true;

  #if !defined(OGLE_DLL)
  PAppHolder = this;
  #endif
}
//*******************************************************************
TOgleApp::~TOgleApp ()
{
  DeInitialize();
  #if !defined(OGLE_DLL)
  PAppHolder = NULL;
  #endif
}
//*******************************************************************
bool TOgleApp::CanClose ()
{
  TRACEX(App,0,"CanClose");
  bool result = TApplication::CanClose();

  return result;
}
//*******************************************************************
TOgleGameFrameWindow *TOgleApp::CreateGameFrameWindow()
{
  TOgleAppInfo appinfo;
  GetAppInfo(appinfo);
  return (new TOgleGameFrameWindow(this,appinfo.MainTitle,true));
}
//*******************************************************************
void TOgleApp::DoShow()
// note: badly named, this var is used by TApplication to do a ShowWindow on the framewindow.
{
  if (GetCmdShow() != SW_HIDE)
  {
    if (GetCmdShow() != SW_SHOWMINNOACTIVE)
    {
      SetCmdShow(SW_SHOWNORMAL);
    }
  }
}
//*******************************************************************
void TOgleApp::InitMainWindow ()
{
  TRACEX(App,0,"InitMainWindow");

  DoShow();

  TOgleGameFrameWindow *frame = CreateGameFrameWindow();
  frame->Initialize();

  SetMainWindow(frame);

  // Borland Windows custom controls.
  EnableBWCC();
}
//*******************************************************************

