//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ogame.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     7/18/95    M.Perry    original baseline
//  2.0     8/24/97    M.Perry    32-bit conversion.
//  2.1     12/6/97    M.Perry    filename mod's.
//********************************************************************
#include "ogame.h"

//*******************************************************************
void TOgleGame::SetMajVer()
{
  MajVer = 2;
}
//*******************************************************************
void TOgleGame::SetMinVer()
{
  MinVer = 3;
}
//*******************************************************************
string TOgleGame::AddExt()
{
  return string(".g");
}
//*******************************************************************
const char *TOgleGame::GetIDChars() const
{
  return "GAME";
}
//*******************************************************************
TOgleGame::TOgleGame(string GameName)
:TOgleBase(NULL)
{
  if (!GameName.is_null())
  {
    FileMode = TFileMode(GameName,TFileMode::fm_None,TFileMode::fm_Immed);
    Init(FileMode,true,false);
  }
}
//*******************************************************************
TOgleGame::TOgleGame()
:TOgleBase(NULL)
{
}
//*******************************************************************
TOgleGame::~TOgleGame()
{
  string *pstr = NULL;
  while (Modes.GetRecCount() > 0)
  {
    pstr = (string *)Modes.Pop();
    delete pstr;
  }
}
//*******************************************************************
void TOgleGame::SetName(string NewName)
{
  FileMode.Name = NewName;
//  FileMode.Name += AddExt();
  FileMode.Name.SetExt(AddExt());
}
//*******************************************************************
int16 TOgleGame::GetModeIndex(string ModeName)
{
  int16 Count = 0;
  string *ps;
  for (ps = GetFirstMode();
       ps != NULL;
       ps = GetNextMode(),Count++)
  {
    if (ModeName == *ps) {return Count;}
  }
  return -1;
}
//*******************************************************************
bool TOgleGame::RemoveModeName(string &modename)
{
  int16 mi = GetModeIndex(modename);
  if (mi < 0) {return false;}
  string *ps = (string *)Modes.IndexRemove(mi);
  delete ps;
  return true;
}
//*******************************************************************
string TOgleGame::GetModeName(int16 ModeNum)
{
  if (ModeNum >= GetNumModes()) {return string();}
  int16 count = 0;
  string *ps;
  for (ps = GetFirstMode();
       ps != NULL;
       ps = GetNextMode(),count++)
  {
    if (ModeNum == count) {return *ps;}
  }
  return string();
}
//*******************************************************************
TCompareResult CompareStrings(void *firstpstr,void *secpstr)
{
  if ((firstpstr == NULL) || (secpstr == NULL)) {return EQUAL_TO;}
  string *pstr1 = (string *)firstpstr;
  string *pstr2 = (string *)secpstr;
  if (*pstr1 > *pstr2) {return GREATER_THAN;}
  else if (*pstr1 < *pstr2) {return LESS_THAN;}
  else {return EQUAL_TO;}
}
//*******************************************************************
void TOgleGame::SortByName()
{
  TCompareFunct OldCompareFunct = Modes.SetCompareFunct(CompareStrings);
  Modes.SliqSort();
  Modes.SetCompareFunct(OldCompareFunct);
}
//*******************************************************************
bool TOgleGame::ReadLocal1_0()
{
  if (!ReadFNameString(PaletteName)) {return false;}
  // don't need to add dir to PaletteName because it was in logroot.
  PaletteName += ".pal";

  uint16 NumModes;
  if (!Readuint16(&NumModes)) {return false;}

  for (int16 i = 0;i<NumModes;i++)
  {
    string *PStr = new string;
    if (!ReadFNameString(*PStr))
    {
      delete PStr;
      return false;
    }
    // don't need to add dir to modename because it was in logroot.
    (*PStr) += ".m";
    
    AddModeName(PStr);
  }
  SortByName();
  return true;
}
//*******************************************************************
bool TOgleGame::ReadLocal2_0()
//also used by ver 2.3
{
  if (!ReadFileName(PaletteName)) {return false;}
  uint16 nummodes;
  if (!Readuint16(&nummodes)) {return false;}

  for (int16 i = 0;i<nummodes;i++)
  {
    string *pstr = new string;
    if (!ReadFileName(*pstr))
    {
      delete pstr;
      return false;
    }
    AddModeName(pstr);
  }
  SortByName();
  return true;
}
//*******************************************************************
bool TOgleGame::ReadLocal()
{
  switch (MajVer)
  {
    case 1: return ReadLocal1_0();
    case 2:
           switch (MinVer)
           {
             case 0: return ReadLocal2_0();
             case 3: return ReadLocal2_0(); // shares reader, inc'd for release.
             default:return false;
           }
    default: return false;
  }
}
//*******************************************************************
bool TOgleGame::WriteLocal()
{
//  if (!WriteFNameString(PaletteName)) {return false;}
//
//  uint16 NumModes = GetNumModes();
//  if (!Writeuint16(&NumModes)) {return false;}
//
//  for (int16 i = 0;i<NumModes;i++)
//  {
//    if (!WriteFNameString(GetModeName(i))) {return false;}
//  }
//  return true;

  if (!WriteFileName(PaletteName)) {return false;}

  uint16 nummodes = GetNumModes();
  if (!Writeuint16(&nummodes)) {return false;}

  string modename;
  for (int16 i = 0;i < nummodes;i++)
  {
    modename = GetModeName(i);
    if (!WriteFileName(modename)) {return false;}
  }
  return true;
}
//*******************************************************************

