//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  olistwal.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     1/8/96     M.Perry    original baseline
//********************************************************************
#if !defined(__olistwal_h)
#include "olistwal.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif
//*******************************************************************
TOgleActiveElement::TOgleActiveElement(TOgleBase *pbase,bool active)
:PBase(pbase),
 Active(active),
 StartFUC(0)
{
}
//*******************************************************************
TOgleActiveElement::TOgleActiveElement()
:Active(false),
 PBase(NULL),
 StartFUC(0)
{
}
//*******************************************************************
TOgleActiveElement::TOgleActiveElement(TOgleBase *PNewBase,dword NewStartFUC)
:Active(true),
 PBase(PNewBase),
 StartFUC(NewStartFUC)
{
  if (PBase != NULL) {PBase->Activate(NewStartFUC);}
}
//*******************************************************************
TOgleActiveElement::~TOgleActiveElement()
{
  Stop();
  if (PBase != NULL) {PBase->DeActivate();}
  Active = false;
  PBase = NULL;
}
//*******************************************************************
void TOgleActiveElement::PostUpdate(dword NewFUC)
{
  if (!Active) {return;}

  if (PBase == NULL)
  {
    Active = false;
    return;
  }

  if (StartFUC > NewFUC) {return;}

  if (!PBase->GetStarted() || PBase->GetStopped())
  {
    Active = false;
    return;
  }

  PBase->PostUpdate(NewFUC);
  if (PBase->GetStopped())
  {
    Active = false;
  }
}
//*******************************************************************
//*******************************************************************
TOgleActiveList::TOgleActiveList(TOgleModeWindow *pmw,const string &name)
:TSliq(false),
 Name(name),
 PModeWin(pmw)
{
}
//*******************************************************************
TOgleActiveList::~TOgleActiveList()
{
  TOgleActiveElement *PElementHolder;
  while ((PElementHolder = (TOgleActiveElement *)Pop()) != NULL)
  {
    delete PElementHolder;
  }
}
//*******************************************************************
TOgleActiveElement *TOgleActiveList::CreateNewElement()
{
  return (new TOgleActiveElement);
}
//*******************************************************************
TOgleActiveElement *TOgleActiveList::CreateNewElement(TOgleBase *PNewBase,dword NewStartFUC)
{
  return (new TOgleActiveElement(PNewBase,NewStartFUC));
}
//*******************************************************************
TCompareResult CompareHiStartFUC(void *FirstObj,void *SecObj)
{
  if ((FirstObj == NULL) || (SecObj == NULL)) {return EQUAL_TO;}

  dword First = ((TOgleActiveElement *)FirstObj)->StartFUC;
  dword Second = ((TOgleActiveElement *)SecObj)->StartFUC;

  if (First > Second) {return GREATER_THAN;}
  else if (First < Second) {return LESS_THAN;}
  else {return EQUAL_TO;}
}
//*******************************************************************
void TOgleActiveList::SortByStartFUC()
{
  TCompareFunct OldCompareFunct = SetCompareFunct(CompareHiStartFUC);
  SliqSort();
  SetCompareFunct(OldCompareFunct);
}
//*******************************************************************
bool TOgleActiveList::AddElement(TOgleBase *PNewBase,dword NewStartFUC)
{
  TOgleActiveElement *PNewElement = CreateNewElement(PNewBase,NewStartFUC);

  if (Push(PNewElement)) {return true;}

  delete PNewElement;
  return false;
}
//*******************************************************************
TOgleActiveElement *TOgleActiveList::FindElement(TOgleBase *POldBase)
{
  for (TOgleActiveElement *PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    if (PElementHolder->PBase == POldBase)
    {
      return (PElementHolder);
    }
  }
  return NULL;
}
//*******************************************************************
bool TOgleActiveList::RemoveElement(TOgleBase *POldBase)
{
  return (RemoveElement(FindElement(POldBase)));
}
//*******************************************************************
bool TOgleActiveList::RemoveElement(TOgleActiveElement *PActiveElement)
{
  if (PActiveElement == NULL) {return false;}

  if (LMatchRemove(PActiveElement))
  {
    delete PActiveElement;
    return true;
  }
  else
  {
    return false;
  }
}
//*******************************************************************
void TOgleActiveList::RemoveNotActiveElements()
{
  for (TOgleActiveElement *PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    if (!PElementHolder->Active)
    {
      RemoveElement(PElementHolder);
    }
  }
}
//*******************************************************************
void TOgleActiveList::Update(dword NewFUC)
{
  for (TOgleActiveElement *PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    PElementHolder->Update(NewFUC);
  }
}
//*******************************************************************
void TOgleActiveList::PostUpdate(dword NewFUC)
{
  for (TOgleActiveElement *PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    PElementHolder->PostUpdate(NewFUC);
  }

  RemoveNotActiveElements();
}
//*******************************************************************
bool TOgleActiveList::IsActive(TOgleBase *POldBase)
{
  for (TOgleActiveElement *PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    if (PElementHolder->PBase == POldBase)
    {
      if (PElementHolder->Active) {return true;}
    }
  }
  return false;
}
//*******************************************************************
bool TOgleActiveList::HasGoingElement()
{
  for (TOgleActiveElement *PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    if (PElementHolder->GetStarted() && !PElementHolder->GetStopped())
    {
      return true;
    }
  }
  return false;
}
//*******************************************************************
bool TOgleActiveList::HasActiveElement()
{
  for (TOgleActiveElement *PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    if (PElementHolder->Active) {return true;}
  }
  return false;
}
//*******************************************************************
//*******************************************************************
bool TOgleListWAL::CanActivateElement(int16 ElementNum,dword StartFUC)
{
  return CanActivateElement(GetElementPtr(ElementNum),StartFUC);
}
//*******************************************************************
#pragma argsused
bool TOgleListWAL::CanActivateElement(TOgleBase *PBase,dword StartFUC)
{
  return true;
}
//*******************************************************************
bool TOgleListWAL::IsActiveElement(int16 ElementNum)
{
  return TOgleList::IsActiveElement(ElementNum);
}
//*******************************************************************
bool TOgleListWAL::ActivateElement(int16 ElementNum,dword StartFUC)
{
  return ActivateElement(GetElementPtr(ElementNum),StartFUC);
}
//*******************************************************************
void TOgleListWAL::DeActivateElement(int16 ElementNum)
{
  TOgleList::DeActivateElement(ElementNum);
}
//*******************************************************************
void TOgleListWAL::Activate(dword StartFUC)
{
  TOgleList::Activate(StartFUC);
}
//*******************************************************************
void TOgleListWAL::DeActivate()
{
  TOgleList::DeActivate();
}
//*******************************************************************
#pragma argsused
int16 TOgleListWAL::GetActiveListIndex(TOgleBase *PBase)
{
  return 0;
}
//*******************************************************************
TOgleListWAL::TOgleListWAL(TOgleModeWindow *pmw):TOgleList(pmw)
{
  PActiveList = NULL;
}
//*******************************************************************
TOgleListWAL::~TOgleListWAL()
{
  for (int16 i=0;i<NumActiveLists;i++)
  {
    delete PActiveList[i];
  }
  delete [] PActiveList;
  PActiveList = NULL;
}
//*******************************************************************
TOgleActiveList *TOgleListWAL::CreateActiveList(const string &name)
{
  return (new TOgleActiveList(PModeWin,name));
}
//*******************************************************************
bool TOgleListWAL::AddActiveList(const string &name)
{
  if ((NumActiveLists == 0) || (PActiveList == NULL))
  {
    PActiveList = new TOgleActiveList *[1];
    NumActiveLists = 1;
    PActiveList[0] = CreateActiveList(name);
    return true;
  }
  else
  {
    TOgleActiveList **Holder = PActiveList;
    int16 OldNumActiveLists = NumActiveLists;
//    NumActiveLists += 1;
    NumActiveLists++;
    PActiveList = new TOgleActiveList *[NumActiveLists];

    for (int16 i = 0;i < OldNumActiveLists;i++)
    {
      PActiveList[i] = Holder[i];
    }
    PActiveList[NumActiveLists - 1] = CreateActiveList(name);
    delete [] Holder;
    return true;
  }
}
//*******************************************************************
bool TOgleListWAL::RemoveActiveList(int16 index)
{
  if ((index >= NumActiveLists) || (index < 0)) {return false;}
  if (PActiveList == NULL) {return false;}

  if (NumActiveLists == 1)
  {
    delete PActiveList[0];
    delete [] PActiveList;
    PActiveList = NULL;
    NumActiveLists = 0;
    return true;
  }
  else
  {
    TOgleActiveList **Holder = PActiveList;
    int16 OldNumActiveLists = NumActiveLists;
//    NumActiveLists -= 1;
    NumActiveLists--;
    PActiveList = new TOgleActiveList *[NumActiveLists];
    int16 CurrentIndex = 0;
    for (int16 i=0;i<OldNumActiveLists;i++)
    {
      if (i != index)
      {
        PActiveList[CurrentIndex] = Holder[i];
        CurrentIndex++;
      }
    }
    delete Holder[index];
    delete [] Holder;
    return true;
  }
}
//*******************************************************************
bool TOgleListWAL::ActivateElement(TOgleBase *PBase,dword StartFUC)
{
  if (PBase == NULL) {return false;}
  if (!CanActivateElement(PBase,StartFUC)) {return false;}
  int16 ndx = GetActiveListIndex(PBase);
  if (GetActiveList(ndx) == NULL) {return false;}
  return (GetActiveList(ndx)->AddElement(PBase,StartFUC));
}
//*******************************************************************
void TOgleListWAL::DeActivateElement(TOgleBase *PBase)
{
  if (PBase == NULL) {return;}
  int16 ndx = GetActiveListIndex(PBase);
  if (GetActiveList(ndx) == NULL) {return;}
  GetActiveList(ndx)->RemoveElement(PBase);
}
//*******************************************************************
bool TOgleListWAL::IsActiveElement(TOgleBase *PBase)
{
  if (PBase == NULL) {return false;}
  int16 ndx = GetActiveListIndex(PBase);
  if (GetActiveList(ndx) == NULL) {return false;}
  return (GetActiveList(ndx)->IsActive(PBase));
}
//*******************************************************************
void TOgleListWAL::Update(dword NewFUC)
{
  TOgleActiveList *pal = NULL;
  for (int16 i=0;i<NumActiveLists;i++)
  {
    pal = GetActiveList(i);

    #if defined(DEBUGUPDATES)
    PModeWin->DebugUpdate.StartUListWAL(i,pal->GetName());
    #endif

    if (pal != NULL) {pal->Update(NewFUC);}

    #if defined(DEBUGUPDATES)
    PModeWin->DebugUpdate.EndUListWAL(i);
    #endif
  }
}
//*******************************************************************
void TOgleListWAL::PostUpdate(dword NewFUC)
{
  TOgleActiveList *pal = NULL;
  for (int16 i=0;i<NumActiveLists;i++)
  {
    pal = GetActiveList(i);

    #if defined(DEBUGUPDATES)
    PModeWin->DebugUpdate.StartPUListWAL(i,pal->GetName());
    #endif

    if (pal != NULL) {pal->PostUpdate(NewFUC);}

    #if defined(DEBUGUPDATES)
    PModeWin->DebugUpdate.EndPUListWAL(i);
    #endif
  }
}
//*******************************************************************

