//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  omdat.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.3     6/5/97     M.Perry    original baseline
//  2.0     8/20/97    M.Perry    32-bit conversion.
//********************************************************************
#include "omdat.h"

//*******************************************************************
TMiscData::TMiscData(TOgleModeWindow *pmw,const TFileMode &fm,bool AutoRead,bool KeepBaseFile)
:TOgleBase(pmw),
 NumArrays(0),
 PSizes(NULL),
 PPInts(NULL)
{
  Init(fm,AutoRead,KeepBaseFile);
}
//*******************************************************************
#pragma argsused
TMiscData::TMiscData(bool dummy,TOgleModeWindow *pmw,TFileMode &fm)
:TOgleBase(pmw),
 PSizes(NULL),
 PPInts(NULL),
 NumArrays(0)
{
  FileMode = fm;
//  FileMode.Name += AddExt();
  FileMode.Name.SetExt(AddExt());
}
//*******************************************************************
TMiscData::TMiscData(TOgleModeWindow *pmw)
:TOgleBase(pmw),
 NumArrays(0),
 PSizes(NULL),
 PPInts(NULL)
{
}
//*******************************************************************
TMiscData::~TMiscData()
{
  DestroySizes();
  DestroyInts();
}
//*******************************************************************
string TMiscData::AddExt()
{
  return string(".md");
}
//*******************************************************************
const char *TMiscData::GetIDChars() const
{
  return "MDAT";
}
//*******************************************************************
void TMiscData::DestroySizes()
{
  if (PSizes != NULL)
  {
    delete [] PSizes;
    PSizes = NULL;
  }
}
//*******************************************************************
void TMiscData::DestroyInts()
{
  if (PPInts == NULL) {return;}

  for (int16 i = 0;i<NumArrays;i++)
  {
    if (PPInts[i] != NULL)
    {
      delete [] PPInts[i];
    }
  }
  delete [] PPInts;
  PPInts = NULL;
}
//*******************************************************************
bool TMiscData::ReadLocal()
{
  if (!Readint16(&NumArrays)) {return false;}
  if (NumArrays <= 0) {return true;}

  PPInts = new int16 *[NumArrays];
  PSizes = new int16[NumArrays];

  for (int16 i=0;i<NumArrays;i++)
  {
    PPInts[i] = NULL;
    int16 numints;
    if (!Readint16(&numints)) {return false;}
    PSizes[i] = numints;
    if (numints > 0)
    {
      PPInts[i] = new int16[numints];
      for (int16 j = 0;j<numints;j++)
      {
        int16 num;
        if (!Readint16(&num)) {return false;}
        PPInts[i][j] = num;
      }
    }
  }
  return true;
}
//*******************************************************************
bool TMiscData::WriteLocal()
{
  if (!Writeint16(&NumArrays)) {return false;}
  if (NumArrays <= 0) {return true;}
  for (int16 i=0;i<NumArrays;i++)
  {
    int16 numints = GetNumint16s(i);
    if (!Writeint16(&numints)) {return false;}
    if (numints > 0)
    {
      for (int16 j=0;j<numints;j++)
      {
        int16 num = Getint16(i,j);
        if (!Writeint16(&num)) {return false;}
      }
    }
  }
  return true;
}
//*******************************************************************
bool TMiscData::GrowNumArraysTo(int16 size)
{
  if (size == NumArrays) {return true;}

  int16 **ppints = new int16 *[size];
  int16 *psizes = new int16[size];

  if (size < NumArrays)
  {
    for (int16 i=0;i<size;i++)
    {
      ppints[i] = PPInts[i];
      psizes[i] = PSizes[i];
    }
    // delete truncated arrays.
    for (int16 j=size;j<NumArrays;j++)
    {
      if (PPInts[j] != NULL)
      {
        delete [] PPInts[j];
        PPInts[j] = NULL;
        PSizes[j] = 0;
      }
    }
  }
  else
  {
    for (int16 i=0;i<NumArrays;i++)
    {
      ppints[i] = PPInts[i];
      psizes[i] = PSizes[i];
    }
    for (int16 j=NumArrays;j<size;j++)
    {
      ppints[j] = NULL;
      psizes[j] = 0;
    }
  }
  int16 **tppints = PPInts;
  PPInts = ppints;
  if (tppints != NULL)
  {
    delete [] tppints;
  }

  int16 *tpsizes = PSizes;
  PSizes = psizes;
  if (tpsizes != NULL)
  {
    delete [] tpsizes;
  }

  NumArrays = size;
  return true;
}
//*******************************************************************
bool TMiscData::GrowArrayTo(int16 arraynum,int16 size)
{
  if (!CheckArrayNum(arraynum)) {return false;}
  if (size == GetNumint16s(arraynum)) {return true;}

  int16 *pints = new int16[size];

  if (size < GetNumint16s(arraynum))
  {
    for (int16 i=0;i<size;i++)
    {
      pints[i] = Getint16(arraynum,i);
    }
  }
  else
  {
    for (int16 i=0;i<GetNumint16s(arraynum);i++)
    {
      pints[i] = Getint16(arraynum,i);
    }
    for (int16 j=GetNumint16s(arraynum);j<size;j++)
    {
      pints[j] = 0;
    }
  }
  int16 *tpints = PPInts[arraynum];
  PPInts[arraynum] = pints;
  PSizes[arraynum] = size;
  if (tpints != NULL)
  {
    delete [] tpints;
  }
  return true;
}
//*******************************************************************

