//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oplaymgr.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     10/12/95   M.Perry     original baseline
//  2.0     8/24/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__oplaymgr_h)
#include "oplaymgr.h"
#endif

#if !defined(__ogamefw_h)
#include "ogamefw.h"
#endif

#if !defined(__omisc_h)
#include "omisc.h"
#endif

#if !defined(__ofilenam_h)
#include "ofilenam.h"
#endif

#include <assert.h>

#if !defined(__ofile_h)
#include "ofile.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__oinifile_h)
#include "oinifile.h"
#endif

#if !defined(__omemmgr_h)
#include "omemmgr.h"
#endif

char ErrSaveFile[] = "Error - Save File";
//*******************************************************************
TPlayerMgr::TPlayerMgr(TOgleGameFrameWindow *parent,string drivedir,int16 numactiveallowed,int16 numplayers)
{
  SetDefPlayerName("John Dough");
  POgleApp = TOgleApp::GetOgleApp();//parent->GetOgleApp();
//  if (POgleApp == NULL) {POgleApp = TOgleApp::GetOgleApp();}

  assert(numplayers <= MAXNUMPLAYERS);

  Parent = parent;
  NumActiveAllowed = numactiveallowed;
  NumPlayersAllowed = numplayers;
  DriveDir = drivedir;
  PGameHeader = NULL;
  PGameData = NULL;

  for (int16 i = 0;i<MAXNUMPLAYERS;i++)
  {
    PPlayerData[i] = NULL;
    ActivePlayers[i] = -1;
  }
}
//*******************************************************************
TPlayerMgr::~TPlayerMgr()
{
  DeleteGameHeader();
  DeleteGameData();
  DeleteAllPlayerData();
}
//*******************************************************************
TOgleIniFile *TPlayerMgr::GetIniFile()
{
  return GetOgleApp()->GetIniFile();
}
//*******************************************************************
void TPlayerMgr::DoManagementDialog()
{
  UpdateAllActiveBasePlayerData();
  Parent->MessageBox("This is a default dialog.","Player Management Dialog",MB_OK);
}
//*******************************************************************
#pragma argsused
void TPlayerMgr::DoStatisticsDialog(int16 playernum,TWindow *parent)
{
  Parent->MessageBox("This is a default dialog.","Player Statistics Dialog",MB_OK);
}
//*******************************************************************
int16 TPlayerMgr::GetPlayerNum(const string &name)
{
  for (int16 i=0;i<GetNumPlayersAllowed();i++)
  {
    if (BasePlayerData[i].Valid && (name == BasePlayerData[i].Name))
    {
      return i;
    }
  }
  return -1;
}
//*******************************************************************
string TPlayerMgr::GetPlayerName(int16 playernum)
{
  string name;
  if (!CheckPlayerNum(playernum)) {return name;}

  name = BasePlayerData[playernum].Name;
  return name;
}
//*******************************************************************
bool TPlayerMgr::IsPlayerValid(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return false;}
  if (BasePlayerData[playernum].Valid) {return true;}
  return false;
}
//*******************************************************************
TMoney TPlayerMgr::GetPlayerCash(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return 0l;}
  return (BasePlayerData[playernum].Cash);
}
//*******************************************************************
bool TPlayerMgr::IsPlayerMale(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return true;}
  return (BasePlayerData[playernum].Male);
}
//*******************************************************************
#pragma argsused
void TPlayerMgr::PrePlayerActivation(int16 playernum,TWindow *pwin)
{
  // placeholder
}
//*******************************************************************
bool TPlayerMgr::ActivatePlayer(int16 playernum,TWindow *pwin)
{
  if (!CanChangePlayers()) {return false;}

  if (!ReadPlayerData(playernum)) {return false;}

  if (SetNextOpenActiveSlot(playernum))
  {
    PrePlayerActivation(playernum,pwin);
    NotifyModePlayerActivation(GetPlayerActiveSlotNum(playernum));
    return true;
  }
  return false;
}
//*******************************************************************
void TPlayerMgr::ForceDeActivatePlayer(int16 slotnum)
{
  if (!CheckSlotNum(slotnum)) {return;}
  int16 playernum = GetActivePlayerNum(slotnum);
  NotifyModePlayerDeActivation(slotnum);
  ClearActiveSlot(slotnum);
  WritePlayerData(playernum);
  DeletePlayerData(playernum);
}
//*******************************************************************
bool TPlayerMgr::DeActivatePlayer(int16 playernum)
{
  if (!CanChangePlayers()) {return false;}
  int16 slotnum = GetPlayerActiveSlotNum(playernum);
  ForceDeActivatePlayer(slotnum);
  return true;
}
//*******************************************************************
void TPlayerMgr::ClearPlayerData(int16 playernum)
{
  if (!IsPlayerDataNull(playernum))
  {
    PPlayerData[playernum]->ClearData();
  }
}
//*******************************************************************
bool TPlayerMgr::CreatePlayer(const string &name,bool Male,TMoney cash)
{
  if (!CanCreatePlayer()) {return false;}

  if (DoesNameExist(name)) {return false;}

  int16 playernum = GetOpenPlayerNum();
  if (!CheckPlayerNum(playernum)) {return false;}

  if (!ReadPlayerData(playernum)) {return false;}

  ClearPlayerData(playernum);
  PPlayerData[playernum]->SetName(name);
  PPlayerData[playernum]->SetMale(Male);
  PPlayerData[playernum]->SetCash(cash);
  PPlayerData[playernum]->SetInitialCash(cash);
  PPlayerData[playernum]->SetValid(true);
  if (!WritePlayerData(playernum)) {return false;}
  UpdateBasePlayerData(playernum);
  DeletePlayerData(playernum);
  return true;
}
//*******************************************************************
bool TPlayerMgr::RemovePlayer(int16 playernum)
{
  if (!IsPlayerValid(playernum)) {return true;} // not valid anyway.

  if (IsPlayerActive(playernum)) {return false;}

  if (IsPlayerDataNull(playernum))
  {
    if (!ReadPlayerData(playernum))
    {
      PPlayerData[playernum] = GetNewPlayerData();
      if (PPlayerData[playernum] == NULL) {return false;}
    }
  }
  ClearPlayerData(playernum);
  if (!WritePlayerData(playernum)) {return false;}
  UpdateBasePlayerData(playernum);
  DeletePlayerData(playernum);
  return true;
}
//*******************************************************************
bool TPlayerMgr::CanCreatePlayer()
{
  return (CheckPlayerNum(GetOpenPlayerNum()));
}
//*******************************************************************
bool TPlayerMgr::CanChangePlayers()
{
  return (Parent->IsPlayerChangeOk());
}
//*******************************************************************
void TPlayerMgr::NotifyModePlayerActivation(int16 slotnum)
{
  Parent->NotifyModePlayerActivation(slotnum);
}
//*******************************************************************
void TPlayerMgr::NotifyModePlayerDeActivation(int16 slotnum)
{
  Parent->NotifyModePlayerDeActivation(slotnum);
}
//*******************************************************************
int16 TPlayerMgr::GetNumValidPlayers()
{
  int16 count = 0;
  for (int16 i=0;i<NumPlayersAllowed;i++)
  {
    if (BasePlayerData[i].Valid) {count++;}
  }
  return count;
}
//*******************************************************************
int16 TPlayerMgr::GetOpenPlayerNum()
{
  for (int16 i =0;i<NumPlayersAllowed;i++)
  {
    if (!BasePlayerData[i].Valid) {return i;}
  }
  return -1;
}
//*******************************************************************
int32 TPlayerMgr::GetFileSize()
{
  int32 size = GetGameHeaderSize();
  size += GetGameDataSize();
  size += (GetPlayerDataSize() * NumPlayersAllowed);
  return size;
}
//*******************************************************************
TGameHeader *TPlayerMgr::GetNewGameHeader()
{
  TGameHeader *pgh = new TGameHeader;
  return pgh;
}
//*******************************************************************
void TPlayerMgr::ReleaseNewGameHeader(TGameHeader *pgh)
{
  delete pgh;
}
//*******************************************************************
TGameData *TPlayerMgr::GetNewGameData()
{
  TGameData *pgd = new TGameData;
  return pgd;
}
//*******************************************************************
void TPlayerMgr::ReleaseNewGameData(TGameData *pgd)
{
  delete pgd;
}
//*******************************************************************
TPlayerData *TPlayerMgr::GetNewPlayerData()
{
  TPlayerData *ppd = new TPlayerData;
  return ppd;
}
//*******************************************************************
void TPlayerMgr::ReleaseNewPlayerData(TPlayerData *ppd)
{
  delete ppd;
}
//*******************************************************************
bool TPlayerMgr::ReadFile()
{
  if (!ReadGameHeader())
  {
    Parent->MessageBox("Reading File Header Failed",ErrSaveFile,MB_OK);
    return false;
  }
  if (!ReadGameData())
  {
    Parent->MessageBox("Reading Game Data Failed",ErrSaveFile,MB_OK);
    return false;
  }
  if (!ReadAllBasePlayerData())
  {
    Parent->MessageBox("Reading Player Data Failed",ErrSaveFile,MB_OK);
    return false;
  }
  return true;
}
//*******************************************************************
bool TPlayerMgr::WriteFile()
{
  if (!WriteGameHeader())
  {
    Parent->MessageBox("Write File Header Failed",ErrSaveFile,MB_OK);
    return false;
  }
  if (!WriteGameData())
  {
    Parent->MessageBox("Write Game Data Failed",ErrSaveFile,MB_OK);
    return false;
  }
  if (!WriteAllPlayerData())
  {
    Parent->MessageBox("Write Player Data Failed",ErrSaveFile,MB_OK);
    return false;
  }
  return true;
}
//*******************************************************************
void TPlayerMgr::SetOffsets()
{
  GameDataOffset = GetGameHeaderSize();
  int32 counter = GameDataOffset + GetGameDataSize();

  for (int16 i = 0;i<NumPlayersAllowed;i++)
  {
    PlayerDataOffsets[i] = counter;
    counter += GetPlayerDataSize();
  }
}
//*******************************************************************
bool TPlayerMgr::CheckFileExists()
{
  if (!TOgleFileName::FileExists(FileName))
  {
    if (GetIniFile()->GetPrevRun()) // mrp - v2.4 - 09/03/99 - only show if not 1st time run.
    {
      Parent->MessageBox("Save File Not Found",ErrSaveFile,MB_OK);
    }
    return false;
  }
  return true;
}
//*******************************************************************
bool TPlayerMgr::CheckFileSize()
{
  if (TOgleFileName::GetFileSize(FileName) != GetFileSize())
  {
    Parent->MessageBox("Save File Corrupted",ErrSaveFile,MB_OK);
    return false;
  }
  return true;
}
//*******************************************************************
bool TPlayerMgr::CheckVersion()
{
  if (!ReadGameHeader()) {return false;}
  if (GetGameHeaderPtr() == NULL) {return false;}

  TGameHeader *pgh = GetNewGameHeader();
  bool rc = true;
  if (GetGameHeaderPtr()->MajVer != pgh->MajVer) {rc = false;}
  if (GetGameHeaderPtr()->MinVer != pgh->MinVer) {rc = false;}
  ReleaseNewGameHeader(pgh);
  DeleteGameHeader();
  return rc;
}
//*******************************************************************
bool TPlayerMgr::VerifyFile()
{
  if (!CheckFileExists()) {return false;}
  if (!CheckFileSize()) {return false;}
  if (!CheckVersion()) {return false;}
  return true;
}
//*******************************************************************
bool TPlayerMgr::SaveDefaultsDirectToFile()
{
  TOgleFile SaveFile(FileName,OFA_READWRITE,OFS_EXCLUSIVE,OFC_OPEN_EXISTING);//OF_READWRITE);
  if (!SaveFile.GetFileHandleValid()) {return false;} // v2.3

  // write header.
  TGameHeader *gh = GetNewGameHeader();
  if (!SaveFile.Write(gh,GetGameHeaderSize())) // mike
  {
    ReleaseNewGameHeader(gh);gh = NULL;
    return false;
  }
  ReleaseNewGameHeader(gh);gh = NULL;

  // write game data.
  SaveFile.Seek(GameDataOffset,OFSD_BEGIN);//beg); // mike
  TGameData *gd = GetNewGameData();
  if (!SaveFile.Write(gd,GetGameDataSize())) // mike
  {
    ReleaseNewGameData(gd);gd = NULL;
    return false;
  }
  ReleaseNewGameData(gd);gd = NULL;

  // write player data.
  TPlayerData *pd = GetNewPlayerData();
  for (int16 i = 0;i<NumPlayersAllowed;i++)
  {
    SaveFile.Seek(PlayerDataOffsets[i],OFSD_BEGIN);//beg); // mike
    if (!SaveFile.Write(pd,GetPlayerDataSize()))
    {
      ReleaseNewPlayerData(pd);pd = NULL;
      return false;
    }
  }
  ReleaseNewPlayerData(pd);pd = NULL;

  return true;
}
//*******************************************************************
bool TPlayerMgr::CreateFileSpace()
{
  TFileWriteBuf SaveFile(FileName);
  char buffer = '\0';
  int32 size = GetFileSize();
  for (int32 i=0;i<size;i++)
  {
    if (!SaveFile.Write(&buffer,1)) {return false;} // mike
  }
  return true;
}
//*******************************************************************
bool TPlayerMgr::DeleteOldFile()
{
  int16 rc = int16(::remove(FileName.c_str()));
  if (rc == 0) {return true;}         // file found and deleted, good.
  if (errno == ENOENT) {return true;} // file not found, good.
  return false;                       // permission denied, bad.
}
//*******************************************************************
bool TPlayerMgr::CreateNewFile()
{
  if (!DeleteOldFile())
  {
    Parent->MessageBox("Delete Old Save File Failed",ErrSaveFile,MB_OK);
    return false;
  }

  // mrp - v2.4 - 09/03/99 - don't show on 1st run.
  if (GetIniFile()->GetPrevRun())
  {
    Parent->MessageBox("Creating New Save File",ErrSaveFile,MB_OK);
  }
  bool rc = CreateFileSpace();
  if (!rc)
  {
    Parent->MessageBox("Create New Save File Failed",ErrSaveFile,MB_OK);
    return false;
  }
  rc = SaveDefaultsDirectToFile();
  if (!rc)
  {
    Parent->MessageBox("Writing to New Save File Failed",ErrSaveFile,MB_OK);
    return false;
  }
  return true;
}
//*******************************************************************
bool TPlayerMgr::Init()
{
  FileName = DriveDir + GetSaveName();

  SetOffsets();

  if (!VerifyFile())
  {
    if (!CreateNewFile()) {return false;}
  }

  return ReadFile();
}
//*******************************************************************
void TPlayerMgr::SaveActiveNames()
{
  TOgleIniFile *pinifile = GetIniFile();
  if (pinifile == NULL) {return;}

  for (int16 i = 0;i<GetNumActiveAllowed();i++)
  {
    pinifile->WritePlayerName(i,GetActivePlayerName(i));
  }
}
//*******************************************************************
string TPlayerMgr::GetSavedActiveName(int16 slotnum)
{
  TOgleIniFile *pinifile = GetIniFile();
  if (pinifile == NULL) {return string("");}
  return pinifile->ReadPlayerName(slotnum);
}
//*******************************************************************
void TPlayerMgr::ActivateSavedPlayers()
{
  for (int16 i=0;i<GetNumActiveAllowed();i++)
  {
    string name = GetSavedActiveName(i);
    if (name != "")
    {
      ActivatePlayer(name);
    }
  }
}
//*******************************************************************
bool TPlayerMgr::Save()
{
  SaveActiveNames();

  if (!WriteFile())
  {
    Parent->MessageBox("Creating New Save File",ErrSaveFile,MB_OK);
    if (!CreateNewFile())
    {
      Parent->MessageBox("Create New File Failed",ErrSaveFile,MB_OK);
      return false;
    }
    else
    {
      if (!WriteFile())
      {
        Parent->MessageBox("Write Save File Failed",ErrSaveFile,MB_OK);
        return false;
      }
    }
  }
  return true;
}
//*******************************************************************
void TPlayerMgr::ModeDeActivated()
{
// let derived playermgr handle whether or not we save.
//  Save();
}
//*******************************************************************
bool TPlayerMgr::ReadData(void *pdest,int32 offset,int32 size)
{
  TOgleFile SaveFile(FileName,OFA_READ,OFS_READ,OFC_OPEN_EXISTING);//OF_READ); // mike
  SaveFile.Seek(offset,OFSD_BEGIN);//beg); // mike
  if (!SaveFile.Read(pdest,size)) {return false;}
  else {return true;}
}
//*******************************************************************
void *TPlayerMgr::AllocReadData(int32 offset,int32 size)
{
  void *p = NULL;
  p = TOgleMemMgr::AllocPtr(GMEM_MOVEABLE,size);
  if (p == NULL) {return NULL;}
  if (!ReadData(p,offset,size))
  {
    DeAllocData(p);
    p = NULL;
  }
  return p;
}
//*******************************************************************
void TPlayerMgr::DeAllocData(void *p)
{
  TOgleMemMgr::DeAllocPtr(p);
}
//*******************************************************************
void TPlayerMgr::DeletePlayerData(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return;}
  if (IsPlayerDataNull(playernum)) {return;}
  DeAllocData(PPlayerData[playernum]);
  PPlayerData[playernum] = NULL;
}
//*******************************************************************
bool TPlayerMgr::WriteData(void *psource,int32 offset,int32 size)
{
  TOgleFile SaveFile(FileName,OFA_WRITE,OFS_EXCLUSIVE,OFC_OPEN_EXISTING);//OF_WRITE); // mike
  if (!SaveFile.GetFileHandleValid()) {return false;} //v2.3

  SaveFile.Seek(offset,OFSD_BEGIN);//beg); // mike
  if (!SaveFile.Write(psource,size)) {return false;} // mike
  return true;
}
//*******************************************************************
bool TPlayerMgr::ReadGameHeader()
{
  if (PGameHeader != NULL) {DeleteGameHeader();}

  PGameHeader = (TGameHeader *)AllocReadData(0,GetGameHeaderSize());
  return (PGameHeader != NULL);
}
//*******************************************************************
bool TPlayerMgr::WriteGameHeader()
{
  if (PGameHeader == NULL) {return false;}
  return WriteData(PGameHeader,0,GetGameHeaderSize());
}
//*******************************************************************
bool TPlayerMgr::ReadGameData()
{
  if (PGameData != NULL) {DeleteGameData();}
  PGameData = (TGameData *)AllocReadData(GameDataOffset,GetGameDataSize());
  return (PGameData != NULL);
}
//*******************************************************************
bool TPlayerMgr::WriteGameData()
{
  if (PGameData == NULL) {return false;}
  return WriteData(PGameData,GameDataOffset,GetGameDataSize());
}
//*******************************************************************
bool TPlayerMgr::ReadPlayerData(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return false;}
  if (!IsPlayerDataNull(playernum)) {DeletePlayerData(playernum);}
  PPlayerData[playernum] = (TPlayerData *)AllocReadData(PlayerDataOffsets[playernum],GetPlayerDataSize());
  return (!IsPlayerDataNull(playernum));
}
//*******************************************************************
bool TPlayerMgr::ReadBasePlayerData(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return false;}
  TPlayerData *pd = (TPlayerData *)AllocReadData(PlayerDataOffsets[playernum],GetPlayerDataSize());
  if (pd == NULL) {return false;}
  BasePlayerData[playernum] = *pd;
  DeAllocData(pd);
  return true;
}
//*******************************************************************
bool TPlayerMgr::ReadAllBasePlayerData()
{
  for (int16 i=0;i<NumPlayersAllowed;i++)
  {
    if (!ReadBasePlayerData(i)) {return false;}
  }
  return true;
}
//*******************************************************************
bool TPlayerMgr::UpdateBasePlayerData(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return false;}
  if (!IsPlayerDataNull(playernum))
  {
    BasePlayerData[playernum] = *(PPlayerData[playernum]);
  }
  return true;
}
//*******************************************************************
bool TPlayerMgr::UpdateAllActiveBasePlayerData()
{
  for (int16 i=0;i<NumActiveAllowed;i++)
  {
    int16 playernum = ActivePlayers[i];
    if (CheckPlayerNum(playernum)) {UpdateBasePlayerData(playernum);}
  }
  return true;
}
//*******************************************************************
bool TPlayerMgr::WritePlayerData(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return false;}
  if (IsPlayerDataNull(playernum)) {return false;}
  return WriteData(PPlayerData[playernum],PlayerDataOffsets[playernum],GetPlayerDataSize());
}
//*******************************************************************
bool TPlayerMgr::WriteAllPlayerData()
{
  for (int16 i = 0;i<NumPlayersAllowed;i++)
  {
    if (!IsPlayerDataNull(i)) {if (!WritePlayerData(i)) {return false;}}
  }
  return true;
}
//*******************************************************************
void TPlayerMgr::DeleteAllPlayerData()
{
  for (int16 i = 0;i<NumPlayersAllowed;i++)
  {
    DeletePlayerData(i);
  }
}
//*******************************************************************
void TPlayerMgr::ReleaseUnActivePlayerData()
{
  for (int16 i = 0;i<NumPlayersAllowed;i++)
  {
    if (!IsPlayerDataNull(i))
    {
      if (!IsPlayerActive(i))
      {
        WritePlayerData(i);
        DeletePlayerData(i);
      }
    }
  }
}
//*******************************************************************
int16 TPlayerMgr::GetNumActivePlayers()
{
  int16 counter = 0;
  for (int16 i=0;i<NumActiveAllowed;i++)
  {
    if (IsSlotActive(i)) {counter++;}
  }
  return counter;
}
//*******************************************************************
int16 TPlayerMgr::GetPlayerActiveSlotNum(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return -1;}
  for (int16 i=0;i<NumActiveAllowed;i++)
  {
    if (GetActivePlayerNum(i) == playernum) {return i;}
  }
  return -1;
}
//*******************************************************************
bool TPlayerMgr::IsPlayerActive(int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return false;}

  for (int16 i = 0;i<NumActiveAllowed;i++)
  {
    if (GetActivePlayerNum(i) == playernum) {return true;}
  }
  return false;
}
//*******************************************************************
int16 TPlayerMgr::GetActivePlayerNum(int16 slotnum)
{
  if (CheckSlotNum(slotnum)) {return ActivePlayers[slotnum];}
  return -1;
}
//*******************************************************************
bool TPlayerMgr::IsSlotActive(int16 slotnum)
{
  if (CheckSlotNum(slotnum))
  {
    return (CheckPlayerNum(ActivePlayers[slotnum]));
  }
  return false;
}
//*******************************************************************
int16 TPlayerMgr::GetNextOpenActiveSlot()
{
  for (int16 i=0;i<NumActiveAllowed;i++)
  {
    if (!IsSlotActive(i)) {return i;}
  }
  return -1;
}
//*******************************************************************
bool TPlayerMgr::SetActiveSlot(int16 slotnum,int16 playernum)
{
  if (!CheckPlayerNum(playernum)) {return false;}

  if (!CheckSlotNum(slotnum)) {return false;}
  if (!IsSlotActive(slotnum))
  {
    ActivePlayers[slotnum] = playernum;
    return true;
  }
  return false;
}
//*******************************************************************
bool TPlayerMgr::SetNextOpenActiveSlot(int16 playernum)
{
  return (SetActiveSlot(GetNextOpenActiveSlot(),playernum));
}
//*******************************************************************
void TPlayerMgr::ClearActiveSlot(int16 slotnum)
{
  if (IsSlotActive(slotnum))
  {
    ActivePlayers[slotnum] = -1;
  }
}
//*******************************************************************
void TPlayerMgr::CreateActivateDefPlayer()
{
}
//*******************************************************************
void TPlayerMgr::CreateDefPlayer()
{
}
//*******************************************************************
int32 TPlayerMgr::GetGameHeaderSize()
{
  return (sizeof(TGameHeader));
}
//*******************************************************************
int32 TPlayerMgr::GetGameDataSize()
{
  return (sizeof(TGameData));
}
//*******************************************************************
int32 TPlayerMgr::GetPlayerDataSize()
{
  return (sizeof(TPlayerData));
}
//*******************************************************************
string TPlayerMgr::GetSaveName()
{
  return string("ogle.sav");
}
//*******************************************************************

