//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  odc.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     7/25/96    M.Perry     original baseline
//  2.0     8/22/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__odc_h)
#include "odc.h"
#endif

#if !defined(__opal_h)
#include "opal.h"
#endif

#if !defined(__obmp_h)
#include "obmp.h"
#endif

//*******************************************************************
TOgleDC::TOgleDC(HDC hdc,TAutoDelete autodelete)
:TDC(hdc,autodelete)
{
}
//*******************************************************************
TOgleDC::~TOgleDC()
{
}
//*******************************************************************
bool TOgleDC::SelectPal(TPal &pal,bool forcebg)
{
  if (!pal.IsHPALETTEValid()) {return false;}
  HPALETTE oldPalette = ::SelectPalette(GetHDC(),pal,forcebg);
  PaletteRefs(oldPalette,pal);
  return true;
}
//*******************************************************************
void TOgleDC::PaletteRefs(HPALETTE oldpal,HPALETTE newpal)
{
  if (oldpal)
  {
    OBJ_REF_INC(newpal);
    if ((uint)oldpal > 1)
    {
      if (!OrgPalette) {OrgPalette = oldpal;}
      else             {OBJ_REF_DEC(oldpal, false);}
    }
  }
}
//*******************************************************************
void TOgleDC::AlignRect(TOgleRect &rect)
// attempts to align to DWORD boundary.
// does not clip to any bounding rect.
{
  TOgleRect ar(rect);
  TPoint pt;
  GetDCOrg(pt);
  ar.OffsetRectBy(pt);
  rect.left -= int16(ar.left % 4);
  rect.right += int16(ar.right % 4);
}
//*******************************************************************
//*******************************************************************
TOgleCreatedDC::TOgleCreatedDC(HDC hdc,TAutoDelete autodelete)
:TOgleDC(hdc,autodelete)
{
}
//*******************************************************************
TOgleCreatedDC::~TOgleCreatedDC()
{
  RestoreObjects();
  if (ShouldDelete) {::DeleteDC(HDC(Handle));}
  Handle = 0;
}
//*******************************************************************
//*******************************************************************
TOgleWindowDC::TOgleWindowDC(HDC hdc,HWND hwnd,TAutoDelete autodelete)
:TOgleDC(hdc,autodelete),
 HWnd(hwnd)
{
}
//*******************************************************************
TOgleWindowDC::TOgleWindowDC(HWND hwnd)
:TOgleDC(::GetWindowDC(hwnd),AutoDelete),
 HWnd(hwnd)
{
}
//*******************************************************************
TOgleWindowDC::~TOgleWindowDC()
{
  RestoreObjects();
  if (ShouldDelete) {::ReleaseDC(HWnd, HDC(Handle));}
  Handle = 0;
}
//*******************************************************************
//*******************************************************************
TOgleScreenDC::TOgleScreenDC()
:TOgleWindowDC(0)
{
}
//*******************************************************************
TOgleScreenDC::~TOgleScreenDC()
{
}
//*******************************************************************
//*******************************************************************
TOgleClientDC::TOgleClientDC(HWND hwnd)
:TOgleWindowDC(::GetDC(hwnd),hwnd,AutoDelete)
{
}
//*******************************************************************
TOgleClientDC::~TOgleClientDC()
{
}
//*******************************************************************
//*******************************************************************
TOgleDesktopDC::TOgleDesktopDC()
:TOgleWindowDC(::GetDesktopWindow())
{
}
//*******************************************************************
TOgleDesktopDC::~TOgleDesktopDC()
{
}
//*******************************************************************
//*******************************************************************
TOgleVideoDC::TOgleVideoDC(HDC hdc)
:TOgleCreatedDC(hdc,NoAutoDelete),
 PBmp(NULL),
 OrgBitmap(0)
{
}
//*******************************************************************
TOgleVideoDC::~TOgleVideoDC()
{
  RestoreObjects();
}
//*******************************************************************
void TOgleVideoDC::RestoreBmp()
{
  PBmp = NULL;
  if (OrgBitmap)
  {
    OBJ_REF_DEC(::SelectObject(HDC(Handle), OrgBitmap), false);
    OrgBitmap = 0;
  }
}
//*******************************************************************
bool TOgleVideoDC::SelectBmp(TBmp &bmp)
{
  if (!bmp.IsHBITMAPValid()) {return false;}
  PBmp = &bmp;
  HBITMAP oldbmp = (HBITMAP)::SelectObject(GetHDC(),bmp);
  BitmapRefs(oldbmp,bmp);
  return true;
}
//*******************************************************************
void TOgleVideoDC::BitmapRefs(HBITMAP oldbmp,HBITMAP newbmp)
{
  if (oldbmp)
  {
    OBJ_REF_INC(newbmp);
    if ((bool)oldbmp > 1)
    {
      if (!OrgBitmap) {OrgBitmap = oldbmp;}
      else            {OBJ_REF_DEC(oldbmp, false);}
    }
  }
}
//*******************************************************************
void TOgleVideoDC::RestoreObjects()
{
  RestoreBmp();
  TOgleCreatedDC::RestoreObjects();
}
//*******************************************************************
#pragma argsused
HBITMAP TOgleVideoDC::CreateBitmap(TBitmapInfo &bitmapinfo,void **ppbits)
{
  return NULL;
}
//*******************************************************************
#pragma argsused
bool TOgleVideoDC::SelectPal(TPal &pal,bool forcebg)
{
  return false;
}
//*******************************************************************
//*******************************************************************
TOglePaintDC::TOglePaintDC(TDC &dc)
:TOgleCreatedDC(dc,NoAutoDelete)
{
}
//*******************************************************************
TOglePaintDC::~TOglePaintDC()
{
}
//*******************************************************************
#pragma argsused
bool TOglePaintDC::BitBlt(TOgleRect &rect,TOgleVideoDC &vdc)
{
  return false;
}
//*******************************************************************

