//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:   opal.cpp
//    Project Name:   Ogle
// Original Author:   Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/28/95    M.Perry    original baseline
//********************************************************************
#if !defined(__opal_h)
#include "opal.h"
#endif

#if !defined(__odc_h)
#include "odc.h"
#endif
//*******************************************************************
TRGBQuadPalette::TRGBQuadPalette()
:PRGBQuad(NULL),NumQuads(0)
{
}
//*******************************************************************
TRGBQuadPalette::TRGBQuadPalette(TRGBQuadPalette &rgbquadpalette)
:PRGBQuad(NULL),NumQuads(0)
{
  CreateQuadStruct(rgbquadpalette.GetNumQuads());
  for (int16 i=0;i<GetNumQuads();i++)
  {
    PRGBQuad[i] = rgbquadpalette.PRGBQuad[i];
  }
}
//*******************************************************************
TRGBQuadPalette::TRGBQuadPalette(RGBQUAD *prgbquad,int16 numquads)
:PRGBQuad(NULL),NumQuads(0)
{
  CreateQuadStruct(numquads);
  for(int16 i=0;i<NumQuads;i++)
  {
    SetQuad(i,prgbquad[i]);
  }
}
//*******************************************************************
TRGBQuadPalette::~TRGBQuadPalette()
{
  DestroyQuadStruct();
}
//*******************************************************************
bool TRGBQuadPalette::IsEqual(TRGBQuadPalette &rgbquadpalette)
{
  if (&rgbquadpalette == this) {return true;}
  if (PRGBQuad == NULL && rgbquadpalette.PRGBQuad == NULL) {return true;}
  if (PRGBQuad == NULL || rgbquadpalette.PRGBQuad == NULL) {return false;}
  if (NumQuads != rgbquadpalette.GetNumQuads()) {return false;}
  for (int16 i=0;i<NumQuads;i++)
  {
    RGBQUAD &q1 = PRGBQuad[i];
    RGBQUAD &q2 = rgbquadpalette.PRGBQuad[i];
    if (q1.rgbBlue != q2.rgbBlue) {return false;}
    if (q1.rgbGreen != q2.rgbGreen) {return false;}
    if (q1.rgbRed != q2.rgbRed) {return false;}
    if (q1.rgbReserved != q2.rgbReserved) {return false;}
  }
  return true;
}
//*******************************************************************
bool TRGBQuadPalette::operator==(TRGBQuadPalette &rgbquadpalette)
{
  return IsEqual(rgbquadpalette);
}
//*******************************************************************
bool TRGBQuadPalette::operator!=(TRGBQuadPalette &rgbquadpalette)
{
  return !IsEqual(rgbquadpalette);
}
//*******************************************************************
bool TRGBQuadPalette::CreateQuadStruct(int16 numquads)
{
  if (PRGBQuad != NULL) {DestroyQuadStruct();}

  PRGBQuad = new RGBQUAD[numquads];
  NumQuads = numquads;
  for (int16 i=0;i<numquads;i++)
  {
    PRGBQuad[i].rgbRed = 0;
    PRGBQuad[i].rgbBlue = 0;
    PRGBQuad[i].rgbGreen = 0;
    PRGBQuad[i].rgbReserved = 0;
  }
  return true;
}
//*******************************************************************
void TRGBQuadPalette::DestroyQuadStruct()
{
  delete [] PRGBQuad;
  PRGBQuad = NULL;
}
//*******************************************************************
bool TRGBQuadPalette::SetQuad(int16 index,RGBQUAD rgbquad)
{
  if ((index < 0) || (index >= NumQuads)) {return false;}
  PRGBQuad[index] = rgbquad;
  return true;
}
//*******************************************************************
RGBQUAD TRGBQuadPalette::GetQuad(int16 index)
{
  if ((index < 0) || (index >= NumQuads)) {return TRgbQuad(0,0,0);}
  return PRGBQuad[index];
}
//*******************************************************************
TRGBQuadPalette &TRGBQuadPalette::operator=(TRGBQuadPalette &rgbquadpalette)
{
  if (&rgbquadpalette == this) {return *this;}
  int16 num = rgbquadpalette.GetNumQuads();
  CreateQuadStruct(num);
  for (int16 i=0;i<num;i++)
  {
    PRGBQuad[i] = rgbquadpalette.PRGBQuad[i];
  }
  return *this;
}
//*******************************************************************
TRGBQuadPalette &TRGBQuadPalette::operator=(TLogicalPalette &logicalpalette)
{
  int16 numpalentries = logicalpalette.GetNumEntries();
  CreateQuadStruct(numpalentries);
  for (int16 i=0;i<numpalentries;i++)
  {
    PRGBQuad[i].rgbRed = logicalpalette.PLogPal->palPalEntry[i].peRed;
    PRGBQuad[i].rgbBlue = logicalpalette.PLogPal->palPalEntry[i].peBlue;
    PRGBQuad[i].rgbGreen = logicalpalette.PLogPal->palPalEntry[i].peGreen;
  }
  return *this;
}
//*******************************************************************
bool TRGBQuadPalette::SetAllQuads(RGBQUAD rgbquad)
{
  for (int16 i=0;i<NumQuads;i++)
  {
    PRGBQuad[i] = rgbquad;
  }
  return true;
}
//*******************************************************************
//*******************************************************************
TLogicalPalette::TLogicalPalette(TRGBQuadPalette &rgbquadpalette)
:PLogPal(NULL)
{
  int16 numquads = rgbquadpalette.GetNumQuads();
  CreateLogPalStruct(numquads);
  for (int16 i=0;i<numquads;i++)
  {
    PLogPal->palPalEntry[i].peRed = rgbquadpalette.PRGBQuad[i].rgbRed;
    PLogPal->palPalEntry[i].peBlue = rgbquadpalette.PRGBQuad[i].rgbBlue;
    PLogPal->palPalEntry[i].peGreen = rgbquadpalette.PRGBQuad[i].rgbGreen;
  }
}
//*******************************************************************
TLogicalPalette::TLogicalPalette(int16 numentries)
:PLogPal(NULL)
{
  CreateLogPalStruct(numentries);
}
//*******************************************************************
TLogicalPalette::TLogicalPalette(LOGPALETTE &logpalette)
:PLogPal(NULL)
{
  int16 num = logpalette.palNumEntries;
  CreateLogPalStruct(num);
  for(int16 i=0;i<num;i++)
  {
    PLogPal->palPalEntry[i] = logpalette.palPalEntry[i];
  }
  PLogPal->palVersion = logpalette.palVersion;
}
//*******************************************************************
TLogicalPalette::TLogicalPalette(TLogicalPalette &logicalpalette)
:PLogPal(NULL)
{
  int16 num = logicalpalette.GetNumEntries();
  CreateLogPalStruct(num);
  for (int16 i = 0;i<num;i++)
  {
    PLogPal->palPalEntry[i] = logicalpalette.PLogPal->palPalEntry[i];
  }
  PLogPal->palVersion = logicalpalette.PLogPal->palVersion;
}
//*******************************************************************
TLogicalPalette::~TLogicalPalette()
{
  DestroyLogPalStruct();
}
//*******************************************************************
TLogicalPalette &TLogicalPalette::operator=(TLogicalPalette &logicalpalette)
{
  if (&logicalpalette == this) {return *this;}

  DestroyLogPalStruct();
  int16 num = logicalpalette.GetNumEntries();
  CreateLogPalStruct(num);
  for(int16 i=0;i<num;i++)
  {
    PLogPal->palPalEntry[i] = logicalpalette.PLogPal->palPalEntry[i];
  }
  return *this;
}
//*******************************************************************
TLogicalPalette &TLogicalPalette::operator=(TRGBQuadPalette &rgbquadpalette)
{
  int16 numquads = rgbquadpalette.GetNumQuads();
  CreateLogPalStruct(numquads);
  for (int16 i=0;i<numquads;i++)
  {
    PLogPal->palPalEntry[i].peRed = rgbquadpalette.PRGBQuad[i].rgbRed;
    PLogPal->palPalEntry[i].peBlue = rgbquadpalette.PRGBQuad[i].rgbBlue;
    PLogPal->palPalEntry[i].peGreen = rgbquadpalette.PRGBQuad[i].rgbGreen;
  }
  return *this;
}
//*******************************************************************
bool TLogicalPalette::IsEqual(LOGPALETTE &logpalette)
{
  if (PLogPal == NULL) {return false;}
  if (PLogPal->palVersion != logpalette.palVersion) {return false;}
  if (PLogPal->palNumEntries != logpalette.palNumEntries) {return false;}
  int16 num = logpalette.palNumEntries;
  for (int16 i=0;i<num;i++)
  {
    if (PLogPal->palPalEntry[i].peRed != logpalette.palPalEntry[i].peRed) {return false;}
    if (PLogPal->palPalEntry[i].peGreen != logpalette.palPalEntry[i].peGreen) {return false;}
    if (PLogPal->palPalEntry[i].peBlue != logpalette.palPalEntry[i].peBlue) {return false;}
    if (PLogPal->palPalEntry[i].peFlags != logpalette.palPalEntry[i].peFlags) {return false;}
  }
  return true;
}
//*******************************************************************
bool TLogicalPalette::operator==(LOGPALETTE &logpalette)
{
  return IsEqual(logpalette);
}
//*******************************************************************
bool TLogicalPalette::operator!=(LOGPALETTE &logpalette)
{
  return !IsEqual(logpalette);
}
//*******************************************************************
bool TLogicalPalette::CreateLogPalStruct(int16 numentries)
{
  if (PLogPal != NULL) {DestroyLogPalStruct();} // prevent memory leakage.

  PLogPal = (LOGPALETTE *) new uint8[sizeof(LOGPALETTE) + (numentries - 1)*sizeof(PALETTEENTRY)];
  PLogPal->palVersion = 0x300;
  PLogPal->palNumEntries = numentries;
  for (int16 i=0;i<numentries;i++)
  {
    PLogPal->palPalEntry[i].peRed = 0;
    PLogPal->palPalEntry[i].peGreen = 0;
    PLogPal->palPalEntry[i].peBlue = 0;
    PLogPal->palPalEntry[i].peFlags = 0;
  }
  return true;
}
//*******************************************************************
void TLogicalPalette::DestroyLogPalStruct()
{
  delete [] PLogPal;
  PLogPal = NULL;
}
//*******************************************************************
bool TLogicalPalette::SetPalEntry(int16 index,PALETTEENTRY palentry)
{
  if (PLogPal == NULL) {return false;}
  if ((index < 0) || (index >= PLogPal->palNumEntries)) {return false;}
  PLogPal->palPalEntry[index] = palentry;
  return true;
}
//*******************************************************************
PALETTEENTRY TLogicalPalette::GetPalEntry(int16 index)
{
  if ((PLogPal == NULL) || (index < 0) || (index >= PLogPal->palNumEntries))
  {
    return TPaletteEntry(0,0,0,0);
  }
  return PLogPal->palPalEntry[index];
}
//*******************************************************************
bool TLogicalPalette::SetAllEntries(PALETTEENTRY palentry)
{
  if (PLogPal == NULL) {return false;}
  int16 num = PLogPal->palNumEntries;
  for (int16 i = 0;i<num;i++)
  {
    PLogPal->palPalEntry[i] = palentry;
  }
  return true;
}
//*******************************************************************
bool TLogicalPalette::SetAllFlags(byte flag)
{
  if (PLogPal == NULL) {return false;}
  int16 num = GetNumEntries();
  for (int16 i=0;i<num;i++)
  {
    PLogPal->palPalEntry[i].peFlags = flag;
  }
  return true;
}
//*******************************************************************
bool TLogicalPalette::SetFlag(int16 index,byte flag)
{
 if (PLogPal == NULL) {return false;}
 if ((index < 0) || (index >= GetNumEntries())) {return false;}
 PLogPal->palPalEntry[index].peFlags = flag;
 return true;
}
//*******************************************************************
bool TLogicalPalette::SetToSystem(int16 start,int16 num)
{
  if ((start < 0) || (start >= GetNumEntries())) {return false;}
  if ((start + num) >= GetNumEntries()) {return false;}
  TOgleScreenDC *pscreenDC = new TOgleScreenDC();
  uint16 numchanged = uint16(pscreenDC->GetSystemPaletteEntries(start,num,&(PLogPal->palPalEntry[start])));
  delete pscreenDC;
  return bool(numchanged == num);
}
//*******************************************************************
bool TLogicalPalette::IsSysColor(PALETTEENTRY palentry)
{
  if (GetNumEntries() < 256) {return false;}
  for (int16 i = 0;i<10;i++)
  {
    if (IsPalColorsEqual(palentry,GetPalEntry(i))) {return true;}
  }
  for (int16 j = 246;j<256;j++)
  {
    if (IsPalColorsEqual(palentry,GetPalEntry(j))) {return true;}
  }
  return false;
}
//*******************************************************************
void TLogicalPalette::ModifyColor(int16 palindex)
{
  for (int16 i=0;i<10;i++)
  {
    if (IsPalColorsEqual(GetPalEntry(palindex),GetPalEntry(i)))
    {
      PLogPal->palPalEntry[palindex].peBlue += byte(1);
      ModifyColor(palindex); // recursive check if change matched a different sys color.
      return;
    }
  }
  for (int16 j=246;j<256;j++)
  {
    if (IsPalColorsEqual(GetPalEntry(palindex),GetPalEntry(j)))
    {
      PLogPal->palPalEntry[palindex].peBlue += byte(1);
      ModifyColor(palindex); // recursive check if change matched a different sys color.
      return;
    }
  }
}
//*******************************************************************
bool TLogicalPalette::CheckSysColors()
{
  if (PLogPal == NULL) {return false;}
  if (GetNumEntries() < 256) {return false;}
  for (int16 i=10;i<246;i++)
  {
    if (IsSysColor(GetPalEntry(i)))
    {
      ModifyColor(i);
    }
  }
  return true;
}
//*******************************************************************
//*******************************************************************
TIdentityPalette::TIdentityPalette(TLogicalPalette &logicalpalette)
:TLogicalPalette(logicalpalette)
{
  SetToSystem(0,10);
  for(int16 i=0;i<10;i++) {SetFlag(i,0);}

  SetToSystem(246,10);
  for (int16 j=0;j<10;j++) {SetFlag(j,0);}

  CheckSysColors(); // modifies any colors matching the sys colors.
}
//*******************************************************************
TIdentityPalette::~TIdentityPalette()
{
}
//*******************************************************************
//*******************************************************************
string TPal::AddExt()
{
  return string(".pal");
}
//*******************************************************************
TPal::TPal()
:TGr(NULL),
 hPalette(NULL),
 PLogicalPalette(NULL),
 PRGBQuadPalette(NULL),
 PIdentityPalette(NULL)
{
}
//*******************************************************************
TPal::TPal(TOgleModeWindow *pmw,const TFileMode &fm,bool AutoRead,bool KeepBaseFile)
:TGr(pmw,fm),
 hPalette(NULL),
 PLogicalPalette(NULL),
 PRGBQuadPalette(NULL),
 PIdentityPalette(NULL)
{
  Init(fm,AutoRead,KeepBaseFile);
}
//*******************************************************************
bool TPal::ReadLocal()
{
  if (CreateRGBQuadPalette())
  {
    CreateLogicalPalette();
  }
  return true;
}
//*******************************************************************
bool TPal::ReadLocalHeader()
{
  MajVer = 0;
  MinVer = 0;
  return true;
}
//*******************************************************************
bool TPal::WriteLocalHeader()
{
  return true;
}
//*******************************************************************
bool TPal::WriteLocal()
{
  return true;
}
//*******************************************************************
TPal::~TPal()
{
  DeleteHPALETTE();
  DestroyIdentityPalette();
  DestroyLogicalPalette();
  DestroyRGBQuadPalette();
}
//*******************************************************************
bool TPal::CreateRGBQuadPalette()
// note: added nasty getptr instead of getramptr because might be called from init(psource).
{
  if (GetPtr() == NULL) {return false;}
  if (PRGBQuadPalette != NULL) {DestroyRGBQuadPalette();}
  dword filesize = GetFileSize();
  int16 numquads = int16(filesize / sizeof(RGBQUAD));
  RGBQUAD *prgbquad = (RGBQUAD *)GetPtr();
  PRGBQuadPalette = new TRGBQuadPalette(prgbquad,numquads);
  return true;
}
//*******************************************************************
void TPal::DestroyRGBQuadPalette()
{
  delete PRGBQuadPalette;
  PRGBQuadPalette = NULL;
}
//*******************************************************************
bool TPal::CreateLogicalPalette()
{
  if (PLogicalPalette != NULL) {DestroyLogicalPalette();}
  if (PRGBQuadPalette == NULL) {return false;}
  PLogicalPalette = new TLogicalPalette(*PRGBQuadPalette);
  return true;
}
//*******************************************************************
void TPal::DestroyLogicalPalette()
{
  delete PLogicalPalette;
  PLogicalPalette = NULL;
}
//*******************************************************************
bool TPal::CreateIdentityPalette()
{
  if (PLogicalPalette == NULL) {return false;}
  if (PIdentityPalette != NULL) {DestroyIdentityPalette();}
  PIdentityPalette = new TIdentityPalette(*PLogicalPalette);
  if (PRGBQuadPalette != NULL) {*PRGBQuadPalette = *PIdentityPalette;}
  return true;
}
//*******************************************************************
void TPal::DestroyIdentityPalette()
{
  delete PIdentityPalette;
  PIdentityPalette = NULL;
}
//*******************************************************************
bool TPal::CreateHPALETTE()
{
  if (hPalette != NULL) {DeleteHPALETTE();}
  if (PIdentityPalette != NULL)
  {
    hPalette = ::CreatePalette(*PIdentityPalette);
    return (hPalette != NULL);
  }
  if (PLogicalPalette != NULL)
  {
    hPalette = ::CreatePalette(*PLogicalPalette);
    return (hPalette != NULL);
  }
  return false;
}
//*******************************************************************
bool TPal::DeleteHPALETTE()
{
  bool rc = true;
  if (hPalette != NULL)
  {
    rc = ::DeleteObject(hPalette);
    hPalette = NULL;
  }
  return rc;
}
//*******************************************************************
bool TPal::SetModeWindow(TOgleModeWindow *pmw)
{
  PModeWin = pmw;
  return true;
}
//*******************************************************************
TPal::TPal(TPal &pal)
:TGr(pal.PModeWin),
 hPalette(NULL),
 PRGBQuadPalette(NULL),
 PLogicalPalette(NULL),
 PIdentityPalette(NULL)
{
  if (pal.PRGBQuadPalette != NULL)
  {
    PRGBQuadPalette = new TRGBQuadPalette(*(pal.PRGBQuadPalette));
  }
  if (pal.PLogicalPalette != NULL)
  {
    PLogicalPalette = new TLogicalPalette(*(pal.PLogicalPalette));
  }
  if (pal.PIdentityPalette != NULL)
  {
    PIdentityPalette = new TIdentityPalette(*(pal.PIdentityPalette));
  }
  if (pal.hPalette != NULL)
  {
    CreateHPALETTE();
  }
}
//*******************************************************************
TPal::TPal(TOgleModeWindow *pmw,TPal &pal)
:TGr(pmw),
 hPalette(NULL),
 PRGBQuadPalette(NULL),
 PLogicalPalette(NULL),
 PIdentityPalette(NULL)
{
  if (pal.PRGBQuadPalette != NULL)
  {
    PRGBQuadPalette = new TRGBQuadPalette(*(pal.PRGBQuadPalette));
  }
  if (pal.PLogicalPalette != NULL)
  {
    PLogicalPalette = new TLogicalPalette(*(pal.PLogicalPalette));
  }
  if (pal.PIdentityPalette != NULL)
  {
    PIdentityPalette = new TIdentityPalette(*(pal.PIdentityPalette));
  }
  if (pal.hPalette != NULL)
  {
    CreateHPALETTE();
  }
}
//*******************************************************************
TPal::TPal(RGBQUAD *prgbquads,int16 numquads)
:TGr(NULL),
 hPalette(NULL),
 PRGBQuadPalette(NULL),
 PLogicalPalette(NULL),
 PIdentityPalette(NULL)
{
  PRGBQuadPalette = new TRGBQuadPalette(prgbquads,numquads);
  PLogicalPalette = new TLogicalPalette(*PRGBQuadPalette);
}
//*******************************************************************
TPal::TPal(TOgleModeWindow *pmw,RGBQUAD *prgbquads,int16 numquads)
:TGr(pmw),
 hPalette(NULL),
 PRGBQuadPalette(NULL),
 PLogicalPalette(NULL),
 PIdentityPalette(NULL)
{
  PRGBQuadPalette = new TRGBQuadPalette(prgbquads,numquads);
  PLogicalPalette = new TLogicalPalette(*PRGBQuadPalette);
}
//*******************************************************************
bool TPal::ClearSystemPalette()
{
  TLogicalPalette *psyspal = new TLogicalPalette(256);
  TPaletteEntry PaletteEntry(0,0,0,PC_NOCOLLAPSE);
  psyspal->SetAllEntries(PaletteEntry);
  TOgleScreenDC *pscreenDC = new TOgleScreenDC;

  HPALETTE hsyspal = ::CreatePalette(*psyspal);
  if (hsyspal == NULL)
  {
    delete pscreenDC;
    delete psyspal;
    return false;
  }

  HPALETTE oldpal = ::SelectPalette(*pscreenDC,hsyspal,false);
  pscreenDC->RealizePalette();
  ::SelectPalette(*pscreenDC,oldpal,false);
  ::DeleteObject(hsyspal);

  delete pscreenDC;
  delete psyspal;
  return true;
}
//*******************************************************************

