//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osktdlst.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     10/16/96   M.Perry    original baseline
//  2.0     8/24/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__osktdlst_h)
#include "osktdlst.h"
#endif

#if !defined(__ogobjsk_h)
#include "ogobjsk.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if !defined(__oskit_h) && defined(OGLE_DEBUG_SKITS) && defined(OGLE_DEBUG_SKITS_DATA)
#include "oskit.h"
#endif
//*******************************************************************
TSkitDataList::TSkitDataList(TSkit *pskit,TGObjSk *pgobjsk,int16 num)
:POwnerSkit(pskit),
 POwnerGObj(pgobjsk),
 Size(num),
 PData(NULL),
 LastAddedElementNum(-1)
{
  if (Size > 0)
  {
    PData = new TSkitData[Size];
  }
}
//*******************************************************************
TSkitDataList::~TSkitDataList()
{
  if (PData != NULL)
  {
    delete [] PData;
    PData = NULL;
  }
  LastAddedElementNum = -1;
  Size = 0;
}
//*******************************************************************
bool TSkitDataList::Add(int16 gobjindex,int16 spriteindex,int16 fuc_offset)
{
  if (PData == NULL) {return false;}
  if ((LastAddedElementNum + 1) >= Size) {return false;}
  LastAddedElementNum++;
  PData[LastAddedElementNum].Set(gobjindex,spriteindex,fuc_offset);
  return true;
}
//*******************************************************************
bool TSkitDataList::CanActivate(dword StartFUC)
{
  if (POwnerGObj == NULL)
  {
    #if defined(OGLE_DEBUG_SKITS) && defined(OGLE_DEBUG_SKITS_DATA)
    string title = "Warning - Skit";
    string msg = "TSkitDataList::CanActivate() \nPOwnerGObj == NULL";
    ::MessageBox(NULL,msg.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
    #endif
    return false;
  }

  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    // mrp - v2.3 - no change: this should be the correct calc for fuc param.
    if (!POwnerGObj->CanActivateSprite(PData[i].GObjIndex,
                                       PData[i].SpriteIndex,
                                       dword(StartFUC + PData[i].FUC_Offset)))
    {
      #if defined(OGLE_DEBUG_SKITS) && defined(OGLE_DEBUG_SKITS_DATA)
      string title = "Warning - Skit";
      string pstrmsg[8];
      pstrmsg[0] = "TSkitDataList::CanActivate()";
      pstrmsg[1] = "POwnerGObj->CanActivateSprite() failed";

      pstrmsg[2] = "POwnerSkit Name: ";
      if (POwnerSkit != NULL) {pstrmsg[2] += POwnerSkit->FileMode.Name.GetStr();}
      else                    {pstrmsg[2] += "unknown";}

      pstrmsg[3] = "GObjIndex: ";
      char buffer[50];
      itoa(PData[i].GObjIndex,buffer,10);
      pstrmsg[3] += buffer;

      pstrmsg[4] = "SpriteIndex: ";
      itoa(PData[i].SpriteIndex,buffer,10);
      pstrmsg[4] += buffer;

      pstrmsg[5] = "StartFUC: ";
      ultoa(StartFUC,buffer,10);
      pstrmsg[5] += buffer;

      pstrmsg[6] = "FUC_Offset: ";
      itoa(PData[i].FUC_Offset,buffer,10);
      pstrmsg[6] += buffer;

      pstrmsg[7] = "Actual FUC: ";
      ultoa(dword(StartFUC + PData[i].FUC_Offset),buffer,10);
      pstrmsg[7] += buffer;

      if (POwnerGObj->GetModeWin() != NULL)
      {
        POwnerGObj->GetModeWin()->ScrollMsg(title,pstrmsg,8,true);
      }
      else
      {
        string s;
        for (int16 i=0;i<8;i++)
        {
          s += pstrmsg[i] + "\n";
          ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK|MB_TASKMODAL);
        }
      }
      #endif
      return false;
    }
  }
  return true;
}
//*******************************************************************
void TSkitDataList::Start(dword StartFUC)
{
  if (POwnerGObj == NULL) {return;}
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    #if defined(OGLE_DEBUG_SKITS) && defined(OGLE_DEBUG_SKITS_DATA)
    bool ok =
    #endif
    POwnerGObj->ActivateSprite(PData[i].GObjIndex,
                               PData[i].SpriteIndex,
                               dword(StartFUC + PData[i].FUC_Offset));
    #if defined(OGLE_DEBUG_SKITS) && defined(OGLE_DEBUG_SKITS_DATA)
    if (!ok)
    {
      char buffer[17];
      string str = "Couldn't start skit cell, (GObj -> ";
      itoa(PData[i].GObjIndex,buffer,10);
      str += "# ";
      str += buffer;
      str += ") (Sprite -> ";
      itoa(PData[i].SpriteIndex,buffer,10);
      str += buffer;
      str += ")";

      POwnerGObj->PModeWin->MessageBox(str.c_str(),"Error - Skit Cell",MB_OK|MB_SYSTEMMODAL);
    }
    #endif
  }
}
//*******************************************************************

