//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oskitcat.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     8/27/95    M.Perry    original baseline
//  1.2     03/27/97   M.Perry    changed load dialog subsystem.
//  2.0     8/24/97    M.Perry    32-bit conversion.
//  2.1     12/6/97    M.Perry    filename mod's
//********************************************************************
#if !defined(__oskitcat_h)
#include "oskitcat.h"
#endif

#if !defined(__ogrpgdlg_h)
#include "ogrpgdlg.h"
#endif

#if !defined(__ofilemgr_h)
#include "ofilemgr.h"
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TSkitCat::CriteriaReport(ofstream &os)
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSkitList *pskitlist = GetSkitList(i);
    if (pskitlist != NULL)
    {
      pskitlist->CriteriaReport(os);
    }
    else
    {
      os<<"Skit list index: "<<i<<" was not there";
    }
  }
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TSkitCat::ReportSpriteUsage(ofstream &os,int16 gobjindex,int16 skitcatindex,
                                 int16 skitindex,int16 spriteindex)
{
  if (skitcatindex == -1)
  {
    int16 numskitlists = GetLastNumSkitLists();
    for (int16 i=0;i<=numskitlists;i++)
    {
      TSkitList *pskitlist = GetSkitList(i);
      if (pskitlist == NULL)
      {
        os<<"Skit list index: "<<i<<" was not there\n";
      }
      else
      {
        pskitlist->ReportSpriteUsage(os,gobjindex,skitindex,spriteindex);
      }
    }
  }
  else
  {
    TSkitList *pskitlist = GetSkitList(skitcatindex);
    if (pskitlist == NULL)
    {
      os<<"Skit list index: "<<skitcatindex<<" was not there\n";
    }
    else
    {
      pskitlist->ReportSpriteUsage(os,gobjindex,skitindex,spriteindex);
    }
  }
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
bool TSkitCat::IsSpriteUsedBySkit(int16 gobjindex,int16 spriteindex,int16 &usedskci,
                                  int16 &usedski)
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSkitList *pskitlist = GetSkitList(i);
    if (pskitlist != NULL)
    {
      if (pskitlist->IsSpriteUsedBySkit(gobjindex,spriteindex,usedski))
      {
        usedskci = i;
        return true;
      }
    }
  }
  return false;
}
#endif
//*******************************************************************
const char *TSkitCat::GetIDChars() const
{
  return "CAT ";
}
//*******************************************************************
string TSkitCat::AddExt()
{
  return string(".skc");
}
//*******************************************************************
void TSkitCat::SetMajVer()
{
  MajVer = 2;
}
//*******************************************************************
void TSkitCat::SetIndexes()
{
  if (LastAddedElementNum < 0) {return;}

  TOgleGroupGaugeDlg *pggd = NULL;

  pggd = GetGaugeDlg("Indexing Skits");
  pggd->SetActiveGauge(2);
  pggd->SetGaugeRange(0,LastAddedElementNum);
  for (int16 i = 0;i<= LastAddedElementNum;i++)
  {
    if (GetSkitList(i) != NULL)
    {
//      pggd->SetGaugeText(GetSkitList(i)->GetFName());
      pggd->SetGaugeText(GetSkitList(i)->FileMode.Name.GetStr());
      GetSkitList(i)->SetIndexes(i);
    }
    pggd->IncGaugeValue();
  }
  ReleaseGaugeDlg(pggd);                  
  pggd = NULL;
}
//*******************************************************************
void TSkitCat::Slenderize()
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSkitList *psl = GetSkitList(i);
    if (psl != NULL) {psl->Slenderize();}
  }
}
//*******************************************************************
TSkitCat::TSkitCat(TOgleModeWindow *pmw)
:TOgleList(pmw),
 POwnerGObj(NULL)
{
//  POwnerGObj = NULL;
}
//*******************************************************************
TSkitCat::TSkitCat(TOgleModeWindow *pmw,TFileMode NewFileMode,TGObjSk *PNewGObj,
                   bool AutoRead,bool KeepBaseFile)
:TOgleList(pmw),
 POwnerGObj(PNewGObj)
{
//  POwnerGObj = PNewGObj;
  Init(NewFileMode,AutoRead,KeepBaseFile);
}
//*******************************************************************
TSkitCat::~TSkitCat()
{
}
//*******************************************************************
bool TSkitCat::DeleteSkitCatList(string skitcatfname)
{
  int16 sci = GetSkitListIndex(skitcatfname);
  if (sci >= 0)
  {
    RemoveElement(sci);
    return true;
  }
  return false;
}
//*******************************************************************
bool TSkitCat::ActivateSkit(int16 SkitCatIndex,int16 SkitIndex,dword StartFUC)
{
  TSkitList *psl = GetSkitList(SkitCatIndex);
  if (psl == NULL) {return false;}
  return (psl->ActivateSkit(SkitIndex,StartFUC));
}
//*******************************************************************
bool TSkitCat::CanActivateSkit(int16 SkitCatIndex,int16 SkitIndex,dword StartFUC)
{
  TSkitList *psl = GetSkitList(SkitCatIndex);
  if (psl == NULL) {return false;}
  return psl->CanActivateSkit(SkitIndex,StartFUC);
}
//*******************************************************************
bool TSkitCat::ChooseStartSkit(int16 SkitCatIndex,dword &StartFUC,dword &FUCLen,int16 &skitindex)
{
  TSkitList *psl = GetSkitList(SkitCatIndex);
  if (psl == NULL) {return false;}
  return psl->ChooseStartSkit(StartFUC,FUCLen,skitindex);
}
//*******************************************************************
dword TSkitCat::GetSkitLength(int16 SkitCatIndex,int16 SkitIndex)
{
  TSkitList *psl = GetSkitList(SkitCatIndex);
  if (psl == NULL) {return 0;}
  return psl->GetSkitLength(SkitIndex);
}
//*******************************************************************
bool TSkitCat::ReadLocal1_0()
{
  // start filename mod's - MRP v2.1
  TOgleFileName ofn;
  string dirpath = GetFileMgr()->PDat->GetDirName(FileMode.GetDirIndex());
  ofn.SetDirPath(dirpath);
  // end
  
  TOgleGroupGaugeDlg *pggd = NULL;

  uint16 NumSkitLists;
  if (!(Readuint16(&NumSkitLists))) {return false;}

  GrowBy(NumSkitLists); // MRP 12/12/96

  if (NumSkitLists > 0)
  {
    pggd = GetGaugeDlg("Loading Skits");
    pggd->SetActiveGauge(2);
    pggd->SetGaugeRange(0,NumSkitLists - 1);
    string str;
    for (int16 i=0;i<NumSkitLists;i++)
    {
      if (!ReadFNameString(str)) {return false;}

      // start filename mod's - MRP v2.1
      ofn.SetFName(str);
      ofn.SetExt(string(".skl"));
      str = ofn.GetStr();
      // end

      pggd->SetGaugeText(str);
      AddElement(CreateSkitList(str));
      pggd->IncGaugeValue();
    }
    ReleaseGaugeDlg(pggd);
    pggd = NULL;
  }
  Sort();
  return true;
}
//*******************************************************************
bool TSkitCat::ReadLocal2_0()
{
  TOgleGroupGaugeDlg *pggd = NULL;

  uint16 NumSkitLists;
  if (!(Readuint16(&NumSkitLists))) {return false;}

  GrowBy(NumSkitLists); // MRP 12/12/96

  if (NumSkitLists > 0)
  {
    pggd = GetGaugeDlg("Loading Skits");
    pggd->SetActiveGauge(2);
    pggd->SetGaugeRange(0,NumSkitLists - 1);
    string str;
    for (int16 i=0;i<NumSkitLists;i++)
    {
      if (!ReadFileName(str)) {return false;}

      pggd->SetGaugeText(str);
      AddElement(CreateSkitList(str));
      pggd->IncGaugeValue();
    }
    ReleaseGaugeDlg(pggd);
    pggd = NULL;
  }
  Sort();
  return true;
}
//*******************************************************************
bool TSkitCat::ReadLocal()
{
  switch (MajVer)
  {
    case 1:return ReadLocal1_0();
    case 2:return ReadLocal2_0();
    default:return false;
  }
}
//*******************************************************************
bool TSkitCat::WriteLocal()
{
  uint16 NumSkitLists = uint16(LastAddedElementNum + 1);
  if (!Writeuint16(&NumSkitLists)) {return false;}
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
//    string tempStr = List[i]->GetFName();
    string tempStr = List[i]->FileMode.Name.GetStr();
//    if (!WriteFNameString(tempStr)) {return false;}
    if (!WriteFileName(tempStr)) {return false;}
  }
  return true;
}
//*******************************************************************
bool TSkitCat::Save()
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSkitList *psl = GetSkitList(i);
    if (psl != NULL) {psl->Save();}
  }
  Write();
  return true;
}
//*******************************************************************
TSkitList *TSkitCat::CreateSkitList(const string &SkitListName)
{
  TFileMode FM(SkitListName,
               TFileMode::fm_None,
               TFileMode::fm_Immed,
               FileMode.GetDirIndex());
  TSkitList *PSkitList = new TSkitList(PModeWin,FM,POwnerGObj);
  return PSkitList;
}
//*******************************************************************

