//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osktport.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/8/95     M.Perry    original baseline
//  2.0     8/24/97    M.Perry    32-bit conversion.
//  2.1     12/6/97    M.Perry    filename mod's
//********************************************************************
#if !defined(__osktport_h)
#include "osktport.h"
#endif

#if !defined(__ogobjsk_h)
#include "ogobjsk.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__oinifile_h)
#include "oinifile.h"
#endif

#if !defined(__ofilemgr_h)
#include "ofilemgr.h"
#endif

#if !defined(__omodewin_h) && defined(OGLE_DEBUG_SKITS)
#include "omodewin.h"
#endif

#include "assert.h"
//*******************************************************************
TSkitPortfolioList::TSkitPortfolioList(TGObjSk *PGObj,string &skitlistname,int16 portnum)
:TSliq(false,true),
 POwnerGObj(PGObj),
 SkitListName(skitlistname),
 PortNum(portnum)
{
  #if defined(GED_COMPILE)
  CurrentPosition = 0;
  #else
  CurrentPosition = ReadCurPos();
  #endif
}
//*******************************************************************
TSkitPortfolioList::TSkitPortfolioList(TSkitPortfolioList &spl)
:TSliq(false,true),
 POwnerGObj(spl.POwnerGObj),
 SkitListName(spl.SkitListName),
 PortNum(spl.PortNum)
{
  CurrentPosition = spl.GetCurrentPosition();
  OwnObjs = spl.OwnObjs;
  string Str;
  TSkitPortfolioElement *pspe;
  for (pspe = spl.GetFirst();
       pspe != NULL;
       pspe = spl.GetNext())
  {
    AddName(pspe->GetSkitName(),pspe->SkitIndex);
  }
}
//*******************************************************************
TSkitPortfolioList::~TSkitPortfolioList()
{
  #if !defined(GED_COMPILE)
  WriteCurPos();
  #endif

  TSkitPortfolioElement *pspe;
  while ((pspe = (TSkitPortfolioElement *)Pop()) != NULL)
  {
    delete pspe;
  }
}
//*******************************************************************
int16 TSkitPortfolioList::ReadCurPos()
{
  if (SkitListName != "")
  {
    return int16(TOgleApp::GetOgleApp()->GetIniFile()->ReadInt(SkitListName,PortNum,0));
  }
  else
  {
    return 0;
  }
}
//*******************************************************************
void TSkitPortfolioList::WriteCurPos()
{
  if (SkitListName != "")
  {
    TOgleApp::GetOgleApp()->GetIniFile()->WriteInt(SkitListName,PortNum,CurrentPosition);
  }
}
//*******************************************************************
void TSkitPortfolioList::SetIndexes(int16 skitlistindex)
{
  if (POwnerGObj == NULL)
  {
    #if defined(OGLE_DEBUG_SKITS)
    ::MessageBox(NULL,"TSkitPortfolioList::SetIndexes() \nPOwnerGObj == NULL",
                 "Warning - Skits",MB_OK | MB_TASKMODAL);
    #endif
    return;
  }

  TSkitPortfolioElement *pspe = NULL;
  for (pspe = GetFirst();
       pspe != NULL;
       pspe = GetNext())
  {
    pspe->SkitIndex = POwnerGObj->GetSkitIndex(skitlistindex,pspe->GetSkitName());
    #if defined(OGLE_DEBUG_SKITS)
    if (pspe->SkitIndex < 0)
    {
      char buffer[50];
      string title = "Warning - Skits";
      string pstrmsg[4];
      pstrmsg[0] = "TSkitPortfolioList::SetIndexes(int16 skitlistindex)";
      pstrmsg[1] = "SkitIndex < 0";
      itoa(skitlistindex,buffer,10);
      pstrmsg[2] = "skitlistindex == ";
      pstrmsg[2] += buffer;
      pstrmsg[3] = "Skit Name: ";
      pstrmsg[3] += pspe->GetSkitName();
      if (POwnerGObj->GetModeWin() != NULL)
      {
        POwnerGObj->GetModeWin()->ScrollMsg(title,pstrmsg,4,true);
      }
      else
      {
        string s;
        for (int16 i=0;i<4;i++)
        {
          s += pstrmsg[i];
          s += "\n";
        }
        ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
      }
    }
    #endif
  }
}
//*******************************************************************
void TSkitPortfolioList::ReleaseStrings()
{
  TSkitPortfolioElement *pspe = NULL;
  for (pspe = GetFirst();
       pspe != NULL;
       pspe = GetNext())
  {
    pspe->ReleaseString();
  }
}
//*******************************************************************
bool TSkitPortfolioList::AddName(const string &Str,int16 skitindex)
{
  return QFrontAdd(new TSkitPortfolioElement(Str,skitindex));
}
//*******************************************************************
bool TSkitPortfolioList::IndexAddName(int16 indexnum,const string &Str,int16 skitindex)
{
  return IndexAdd(indexnum,new TSkitPortfolioElement(Str,skitindex));
}
//*******************************************************************
//*******************************************************************
void TSkitPortfolio::SetIndexes()
{
  TOgleBase::SetIndexes();
}
//*******************************************************************
const char *TSkitPortfolio::GetIDChars() const
{
  return "PORT";
}
//*******************************************************************
void TSkitPortfolio::SetIndexes(int16 skitlistindex)
{
  for (int16 i = 0;i<NUMPORTFOLIOLISTS;i++)
  {
    PortfolioLists[i]->SetIndexes(skitlistindex);
  }
}
//*******************************************************************
void TSkitPortfolio::SetMinVer()
{
  MinVer = 4;
}
//*******************************************************************
void TSkitPortfolio::SetMajVer()
{
  MajVer = 2;
}
//*******************************************************************
bool TSkitPortfolio::ReadLocal1_0()
{
  // start filename mod's - MRP v2.1
  TOgleFileName ofn;
//  string dirpath = GetFileMgr()->PDat->GetDirName(FileMode.GetDirIndex());
  if (POwnerGObj != NULL)
  {
    ofn.SetDirPath(POwnerGObj->FileMode.Name.GetFName());
  }
  #if defined(OGLE_DEBUG_SKITS)
  else
  {
    ::MessageBox(NULL,
                 "TSkitPortfolio::ReadLocal1_0() \nPOwnerGObj == NULL",
                 "Warning - Skits",
                 MB_OK | MB_TASKMODAL);
  }
  #endif

//  ofn.SetDirPath(dirpath);
  ofn.SetExt(".sk");
  // end

  int16 NumLists;
  if (!Readint16(&NumLists)) {return false;}
  for (int16 i=0;i<NumLists;i++)
  {
    int16 NumNames;
    if (!Readint16(&NumNames)) {return false;}
    string Str;
    for (int16 j=0;j<NumNames;j++)
    {
      if (!ReadFNameString(Str)) {return false;}

      // start filename mod's - MRP v2.1
      ofn.SetFName(Str);
      Str = ofn.GetStr();
      // end

      AddToList(i,Str);
    }
  }
  return true;
}
//*******************************************************************
bool TSkitPortfolio::ReadLocal1_1()
{
  // start filename mod's - MRP v2.1
  TOgleFileName ofn;
//  string dirpath = GetFileMgr()->PDat->GetDirName(FileMode.GetDirIndex());
  if (POwnerGObj != NULL)
  {
    ofn.SetDirPath(POwnerGObj->FileMode.Name.GetFName());
  }
  #if defined(OGLE_DEBUG_SKITS)
  else
  {
    ::MessageBox(NULL,
                 "TSkitPortfolio::ReadLocal1_1() \nPOwnerGObj == NULL",
                 "Warning - Skits",
                 MB_OK | MB_TASKMODAL);
  }
  #endif
  ofn.SetExt(".sk");
//  ofn.SetDirPath(dirpath);
  // end

  int16 NumLists;
  if (!Readint16(&NumLists)) {return false;}
  for (int16 i=0;i<NumLists;i++)
  {
    int16 NumNames;
    if (!Readint16(&NumNames)) {return false;}
    string Str;
    int16 skitindex;
    for (int16 j=0;j<NumNames;j++)
    {
      if (!ReadFNameString(Str)) {return false;}

      // start filename mod's - MRP v2.1
      ofn.SetFName(Str);
      Str = ofn.GetStr();
      // end

      if (!Readint16(&skitindex)) {return false;}
      AddToList(i,Str,skitindex);
    }
  }
  return true;
}
//*******************************************************************
bool TSkitPortfolio::ReadLocal2_0()
{
  TOgleFileName ofn;
  if (POwnerGObj != NULL)
  {
    ofn.SetDirPath(POwnerGObj->FileMode.Name.GetFName());
  }
  #if defined(OGLE_DEBUG_SKITS)
  else
  {
    ::MessageBox(NULL,"TSkitPortfolio::ReadLocal2_0() \nPOwnerGObj == NULL",
                 "Warning - Skits",MB_OK | MB_TASKMODAL);
  }
  #endif
  ofn.SetExt(".sk");
  
  int16 NumLists;
  if (!Readint16(&NumLists)) {return false;}
  for (int16 i=0;i<NumLists;i++)
  {
    int16 NumNames;
    if (!Readint16(&NumNames)) {return false;}
    string Str;
    int16 skitindex;
    for (int16 j=0;j<NumNames;j++)
    {
      if (!ReadFileName(Str)) {return false;}
      ofn.SetFName(Str);
      Str = ofn.GetStr();
      if (!Readint16(&skitindex)) {return false;}

      AddToList(i,Str,skitindex);
    }
  }
  return true;
}
//*******************************************************************
bool TSkitPortfolio::ReadLocal2_4()
{
  int16 NumLists;
  if (!Readint16(&NumLists)) {return false;}
  for (int16 i=0;i<NumLists;i++)
  {
    int16 NumNames;
    if (!Readint16(&NumNames)) {return false;}
    string Str;
    int16 skitindex;
    for (int16 j=0;j<NumNames;j++)
    {
      if (!ReadFileName(Str)) {return false;}
      if (!Readint16(&skitindex)) {return false;}

      AddToList(i,Str,skitindex);
    }
  }
  return true;
}
//*******************************************************************
bool TSkitPortfolio::ReadLocal()
{
  switch (MajVer)
  {
    case 1:
           switch (MinVer)
           {
             case 0:return ReadLocal1_0();
             case 1:return ReadLocal1_1();
             default:return false;
           }
    case 2:
           switch(MinVer)
           {
             case 0:return ReadLocal2_0();
             case 4:return ReadLocal2_4();
           }
    default:return false;
  }
}
//*******************************************************************
// mrp - v2.4 - WriteLocal() not actually changed from 2.0,
//              just release version help.
bool TSkitPortfolio::WriteLocal()
{
  int16 NumLists = NUMPORTFOLIOLISTS;

  if (!Writeint16(&NumLists)) {return false;}
  for (int16 i=0;i<NumLists;i++)
  {
    int16 NumNames = PortfolioLists[i]->GetNumNames();
    if (!Writeint16(&NumNames)) {return false;}
    string Str;

    int16 counter;
    TSkitPortfolioElement *pspe;
    for (pspe = PortfolioLists[i]->GetFirst(),counter = 0;
         pspe != NULL;
         pspe = PortfolioLists[i]->GetNext(),counter++)
    {
      Str = pspe->GetSkitName();
      int16 skitindex = pspe->SkitIndex;
      if (!WriteFileName(Str)) {return false;}
      if (!Writeint16(&skitindex)) {return false;}
    }

//    if (NumNames != counter)
//    {
//      assert(false);
//    }
    assert(NumNames == counter);
  }
  return true;
}
//*******************************************************************
TSkitPortfolio::TSkitPortfolio(TGObjSk *PGObj)
:TOgleBase(NULL),
 POwnerGObj(PGObj)
{
//  POwnerGObj = PGObj;
  for (int16 i=0;i<NUMPORTFOLIOLISTS;i++)
  {
    PortfolioLists[i] = new TSkitPortfolioList(POwnerGObj,SkitListName,i);
  }
}
//*******************************************************************
TSkitPortfolio::TSkitPortfolio(TGObjSk *PGObj,string const &skitlistname,void *PReadData,bool AutoRead)
:TOgleBase(NULL),
 POwnerGObj(PGObj),
 SkitListName(skitlistname)
{
//  POwnerGObj = PGObj;
//  SkitListName = skitlistname;
  for (int16 i=0;i<NUMPORTFOLIOLISTS;i++)
  {
    PortfolioLists[i] = new TSkitPortfolioList(POwnerGObj,SkitListName,i);
  }

  Init(PReadData,AutoRead);
}
//*******************************************************************
TSkitPortfolio::~TSkitPortfolio()
{
  for (int16 i=0;i<NUMPORTFOLIOLISTS;i++)
  {
    delete PortfolioLists[i];
    PortfolioLists[i] = NULL;
  }
}
//*******************************************************************
bool TSkitPortfolio::AddToList(int16 listnum,const string &Str,int16 skitindex)
{
  if ((listnum < 0) || (listnum >= NUMPORTFOLIOLISTS)) {return false;}

  return PortfolioLists[listnum]->AddName(Str,skitindex);
}
//*******************************************************************
string TSkitPortfolio::GetCopyFromList(int16 listnum,int16 nameindex)
// note: not very efficient, but only used by GEd ??
{
  if ((listnum < 0) || (listnum > NUMPORTFOLIOLISTS)) {return string("");}
  int16 numnames = PortfolioLists[listnum]->GetLastNum();
  if ((nameindex < 0) || (nameindex > numnames)) {return string("");}

  int16 i = 0;
  TSkitPortfolioElement *pspe;
  for (pspe = PortfolioLists[i]->GetFirst();
       ((pspe != NULL) && (i < nameindex));
       pspe = PortfolioLists[i]->GetNext(),i++)
  {}

  if (pspe != NULL)
  {
    return pspe->GetSkitName();
  }
  else
  {
    return string("");
  }
}
//*******************************************************************
int16 TSkitPortfolio::GetNextValidList(int16 StartingAtList)
{
  if (StartingAtList < 0) {StartingAtList = 0;}
  if (StartingAtList >= NUMPORTFOLIOLISTS) {StartingAtList = (NUMPORTFOLIOLISTS - 1);}

  for (int16 CheckList = StartingAtList;
       CheckList < NUMPORTFOLIOLISTS;
       CheckList++)
  {
    if (PortfolioLists[CheckList]->GetNumNames() > 0)
    {
      return CheckList;
    }
  }
  return (NUMPORTFOLIOLISTS - 1);
}
//*******************************************************************
void TSkitPortfolio::ReleaseStrings()
{
  for (int16 i =0;i<NUMPORTFOLIOLISTS;i++)
  {
    PortfolioLists[i]->ReleaseStrings();
  }
}
//*******************************************************************

