//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  owav.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     5/15/95    M.Perry     original baseline
//  2.0     8/23/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__owav_h)
#include "owav.h"
#endif

#if !defined(__osndmgr_h)
#include "osndmgr.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if !defined(__owavlst_h)
#include "owavlst.h"
#endif
//*******************************************************************
string TWav::AddExt()
{
  return string(".wav");
}
//*******************************************************************
#ifdef DEBUGSPRITES
void TWav::RTReport(ofstream &os)
{
//  os<<"Wav Name: "<<GetFName()<<"\n";
  os << "Wav Name: " << FileMode.Name.GetStr() << "\n";
  os<<"Ram loaded: "<<GetRamLoaded()<<"\n";    
  os<<"Started: "<<GetStarted()<<"\n";
  os<<"Stopped: "<<GetStopped()<<"\n";
  os<<"BeginFUC: "<<BeginFUC<<"\n";
  os<<"FUC Length: "<<Get_FUC_Length()<<"\n";
}
#endif
//*******************************************************************
#ifdef DEBUGREPORTS
void TWav::Report(ofstream &os)
{
//  os<<"Wav Name: "<<GetFName()<<"\n";
  os<<"Wav Name: "<<FileMode.Name.GetStr()<<"\n";
}
#endif
//*******************************************************************
TWav::TWav(TOgleModeWindow *pmw)
:TSound(pmw),
 PWavList(NULL),
 PSndMgrData(NULL),
 LoopCount(0)
{
  FUC_Length = 0;
  #if defined(ENB_WAV_CLIP_END)
  SetClipEnd();//mrp - v2.3
  #endif
}
//*******************************************************************
TWav::TWav(TOgleModeWindow *pmw,
           const TFileMode &NewFileMode,
           int16 NewFUCLength,
           bool AutoRead,
           bool KeepBaseFile)
:TSound(pmw,NewFileMode),
 PSndMgrData(NULL),
 LoopCount(0),
 PWavList(NULL)
{
  #if defined(ENB_WAV_CLIP_END)
  SetClipEnd();//mrp - v2.3
  #endif
  FUC_Length = NewFUCLength;
  Init(NewFileMode,AutoRead,KeepBaseFile);
  GetSoundMgr()->Register(this);
}
//*******************************************************************
TWav::~TWav()
{
  RemoveFromSndMgr();
  GetSoundMgr()->UnRegister(this);
}
//*******************************************************************
void TWav::Start()
{
  bool added = AddToSndMgr();
  // set flags after add call, because it might change flags on us.
  SetStarted();
  if (added)
  {
    ClearStopped();
  }
  else // if it couldn't be added then pretend already done.
  {
    SetStopped();
    WLNotifyStop();
  }
}
//*******************************************************************
void TWav::Update(dword NewFUC)
// just has to get it going.
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartWavUpdate();
  #endif
  if (BeginFUC == NewFUC)
  {
    Start();
  }
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndWavUpdate();
  #endif
}
//*******************************************************************
void TWav::PostUpdate(dword NewFUC)
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartWavPUpdate();
  #endif

  if (FUC_Length <= 0 || !GetStarted() || GetStopped())
  {
    #if defined(DEBUGUPDATES)
    PModeWin->DebugUpdate.EndWavPUpdate();
    #endif
    return;
  }

  dword StopFUC = BeginFUC + FUC_Length - 1; // correcting off by one.
  if (StopFUC < BeginFUC) {StopFUC = BeginFUC;}

  if (NewFUC >= StopFUC) // stopping time
  {
    if (GetClipEnd())
    {
      Stop();
    }
    else
    {
      SetStopped();  // let it finish if not actually finished yet.
      GetSoundMgr()->ClearNotifyStop(this);
      ClearAddedToSndMgr();
      GetSoundMgr()->ChangeStatus(this,FileMode.GetRamStatus());
      WLNotifyStop();
    }
  }
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndWavPUpdate();
  #endif
}
//*******************************************************************
void TWav::Start(dword StartFUC)
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartWavStart();
  #endif

  BeginFUC = StartFUC + FUC_Offset;
  if (GetStarted() && !GetStopped()) {Stop();}
  ClearStarted();
  ClearStopped();
  dword curFUC = AskCurrentFUC();
  WLNotifyStart();
  if (BeginFUC == curFUC) {Start();}

  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndWavStart();
  #endif
}
//*******************************************************************
void TWav::Stop()
{
  RemoveFromSndMgr();
  SetStopped();
  WLNotifyStop();
}
//*******************************************************************
void TWav::WLNotifyStart()
{
  if (PWavList == NULL) {return;}
  if (!GetWLNotifiedStartScheduled())
  {
    SetWLNotifiedStartScheduled();
    PWavList->NotifyStart(this);
  }
}
//*******************************************************************
void TWav::WLNotifyStop()
{
  if (PWavList == NULL) {return;}
  if (GetWLNotifiedStartScheduled())
  {
    PWavList->NotifyStop(this);
    ClearWLNotifiedStartScheduled();
  }
}
//*******************************************************************
void TWav::NotifyStop()
// note: called from sndmgr.
{
  // sndmgr telling us we're done.
  if (GetAddedToSndMgr())
  {
    ClearAddedToSndMgr();
    GetSoundMgr()->ChangeStatus(this,FileMode.GetRamStatus());
  }
  SetStopped();
  WLNotifyStop();
}
//*******************************************************************
void TWav::Activate(dword StartFUC)
{
  TSound::Activate(StartFUC);
  ACLoad();
  if (DoLoadAhead(StartFUC))
  {
    GetSoundMgr()->ChangeStatus(this,TFileMode::fm_Immed);
  }
}
//*******************************************************************
void TWav::DeActivate()
{
  TSound::DeActivate();
  ACLoad();
}
//*******************************************************************
bool TWav::operator==(const TWav& wav)const
{
  return (this == &wav);
}
//*******************************************************************
bool TWav::AddToSndMgr()
{
  if (GetAddedToSndMgr()) {RemoveFromSndMgr();}
  if (GetSoundMgr()->IsAudioMuted())
  {
    WLNotifyStop();
    return false;
  }
  GetSoundMgr()->ChangeStatus(this,TFileMode::fm_Immed);   // this might call our LoadRamImmed();
  bool added = GetSoundMgr()->Play(this,LoopCount);
  if (added)
  {
    SetAddedToSndMgr();
  }
  else
  {
    ClearAddedToSndMgr();
    ACLoad();
    WLNotifyStop();
  }
  return added;
}
//*******************************************************************
bool TWav::RemoveFromSndMgr()
{
  if (!GetAddedToSndMgr())
  {
    WLNotifyStop();
    return true;
  }
  ClearAddedToSndMgr();
  GetSoundMgr()->Stop(this); // this might call notifystop.
  GetSoundMgr()->ChangeStatus(this,FileMode.GetRamStatus());
  WLNotifyStop();
  return true;
}
//*******************************************************************

