//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osprcell.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     6/20/95    M.Perry    original baseline
//  2.0                M.Perry    32-bit conversion.
//  2.1     12/6/97    M.Perry    filename mod's.
//********************************************************************
#if !defined(__osprcell_h)
#include "osprcell.h"
#endif

#if !defined(__oanmlst_h)
#include "oanmlst.h"
#endif

#if !defined(__owavlst_h)
#include "owavlst.h"
#endif

#if !defined(__osprite_h)
#include "osprite.h"
#endif

#if !defined(__ofilemgr_h)
#include "ofilemgr.h"
#endif

#include <assert.h>
//*******************************************************************
void TSpriteCell::ClearCell()
{
  SetIsAnm();
  SetType(0);
  Set_FUC_Length(0);
  SetForward();
  OverrideDefaultXY = false;
  SetXYZ(0,0,0);
  SetStartFrame(0);
  TOgleCell::ClearCell(); // it clears CellFileName
}
//*******************************************************************
TSpriteCell &TSpriteCell::operator=(const TSpriteCell &sprcell)
{
  Assign(sprcell);
  return *this;
}
//*******************************************************************
void TSpriteCell::Assign(const TSpriteCell &sprcell)
{
  if (&sprcell == this) {return;}
  FileMode = sprcell.FileMode;
  SetIsAnm(sprcell.GetIsAnm());
  SetType(sprcell.GetType());
  Set_FUC_Length(sprcell.Get_FUC_Length());
  CellFileName = sprcell.CellFileName;
//  SetIsForward(sprcell.GetIsForward());
  SetDirection(sprcell.GetDirection());
  OverrideDefaultXY = sprcell.OverrideDefaultXY;
  SetX(sprcell.GetX());
  SetY(sprcell.GetY());
  SetZ(sprcell.GetZ());
  SetStartFrame(sprcell.GetStartFrame());
}
//*******************************************************************
void TSpriteCell::SetMinVer()
{
  MinVer = 4;
}
//*******************************************************************
void TSpriteCell::SetMajVer()
{
  MajVer = 2;
}
//*******************************************************************
bool TSpriteCell::ReadLocal1_2()
{
  if (!(ReadFNameString(CellFileName))) {return false;}
  // (MRP v2.1)
  // start of filename mod's
  string dirpath = GetFileMgr()->PDat->GetDirName(FileMode.GetDirIndex());
  TOgleFileName ofn;
  ofn.SetFName(CellFileName);
  ofn.SetDirPath(dirpath);

  // end of filename mod's.

  if (!(Readdword(&FUC_Length))) {return false;}

  if (!(Readbool(&IsAnm))) {return false;}
  // start MRP v2.1
  if (IsAnm)
  {
    ofn.SetExt(string(".anm"));
  }
  else
  {
    ofn.SetExt(string(".wav"));
  }
  // end

  bool IsForward;
  if (!(Readbool(&IsForward))) {return false;}
  if (IsForward)
  {
    SetDirection(dForward);
  }
  else
  {
    SetDirection(dReverse);
  }

  if (!(Readbool(&OverrideDefaultXY))) {return false;}

  if (!(Readint16(&X))) {return false;}
  if (!(Readint16(&Y))) {return false;}
  if (!(Readint16(&Z))) {return false;}

  // start MRP v2.1
  // moved down here.
  CellFileName = ofn.GetStr();
  // end
  return true;
}
//*******************************************************************
bool TSpriteCell::ReadLocal2_0()
{
  if (!ReadFileName(CellFileName)) {return false;}
  if (!Readdword(&FUC_Length)) {return false;}
  if (!Readbool(&IsAnm)) {return false;}
  bool IsForward;
  if (!Readbool(&IsForward)) {return false;}
  if (IsForward)
  {
    SetDirection(dForward);
  }
  else
  {
    SetDirection(dReverse);
  }
  if (!Readbool(&OverrideDefaultXY)) {return false;}
  if (!Readint16(&X)) {return false;}
  if (!Readint16(&Y)) {return false;}
  if (!Readint16(&Z)) {return false;}
  return true;
}
//*******************************************************************
bool TSpriteCell::ReadLocal2_3()
{
  if (!ReadFileName(CellFileName)) {return false;}
  if (!Readdword(&FUC_Length)) {return false;}
  if (!Readbool(&IsAnm)) {return false;}
  int16 direction;
  if (!Readint16(&direction)) {return false;}
  SetDirection(TDirection(direction));
  if (!Readbool(&OverrideDefaultXY)) {return false;}
  if (!Readint16(&X)) {return false;}
  if (!Readint16(&Y)) {return false;}
  if (!Readint16(&Z)) {return false;}
  return true;
}
//*******************************************************************
bool TSpriteCell::ReadLocal2_4()
{
  if (!ReadFileName(CellFileName)) {return false;}
  #if defined(GED_COMPILE)
// the following removes spaces in fname part of filename.
  TOgleFileName ofn(CellFileName);
  string s = ofn.GetFName();
  s = s.strip(string::Both);
  ofn.SetFName(s);
  CellFileName = ofn.GetStr();
  #endif

  if (!Readdword(&FUC_Length)) {return false;}
  if (!Readbool(&IsAnm)) {return false;}
  int16 direction;
  if (!Readint16(&direction)) {return false;}
  SetDirection(TDirection(direction));
  if (!Readbool(&OverrideDefaultXY)) {return false;}
  if (!Readint16(&X)) {return false;}
  if (!Readint16(&Y)) {return false;}
  if (!Readint16(&Z)) {return false;}
  if (!Readint16(&StartFrame)) {return false;}
  return true;
}
//*******************************************************************
bool TSpriteCell::ReadLocal()
{
  switch (MajVer)
  {
    case 1:
            switch (MinVer)
            {
              case 2:return ReadLocal1_2();
              default:return false;
            }
    case 2:
            switch (MinVer)
            {
              case 0: return ReadLocal2_0();
              case 3: return ReadLocal2_3();
              case 4: return ReadLocal2_4();
              default:return false;
            }
    default: return false;
  }
}
//*******************************************************************
bool TSpriteCell::WriteLocal()
{
//  if (!WriteFNameString(CellFileName)) {return false;}
//  if (!Writedword(&FUC_Length)) {return false;}
//  if (!Writebool(&IsAnm)) {return false;}
//  if (!Writebool(&IsForward)) {return false;}
//  if (!Writebool(&OverrideDefaultXY)) {return false;}
//  if (!Writeint16(&X)) {return false;}
//  if (!Writeint16(&Y)) {return false;}
//  if (!Writeint16(&Z)) {return false;}
//  return true;

  if (!WriteFileName(CellFileName)) {return false;}
  if (!Writedword(&FUC_Length)) {return false;}
  if (!Writebool(&IsAnm)) {return false;}
//  if (!Writebool(&IsForward)) {return false;}
  int16 direction = (int16)GetDirection();
  if (!Writeint16(&direction)) {return false;}
  if (!Writebool(&OverrideDefaultXY)) {return false;}
  if (!Writeint16(&X)) {return false;}
  if (!Writeint16(&Y)) {return false;}
  if (!Writeint16(&Z)) {return false;}
  if (!Writeint16(&StartFrame)) {return false;}
  return true;
}
//*******************************************************************
TSpriteCell::TSpriteCell()
:TOgleCell(),
 POwnerSprite(NULL),
 IsAnm(true),
 Direction(dForward),
 OverrideDefaultXY(false),
 X(0),
 Y(0),
 Z(0),
 StartFrame(0)
{
}
//*******************************************************************
TSpriteCell::TSpriteCell(TOgleModeWindow *pmw,TFileMode &NewFileMode)
:TOgleCell(pmw,NewFileMode),
 POwnerSprite(NULL),
 IsAnm(true),
 Direction(dForward),
// IsForward(true),
 OverrideDefaultXY(false),
 X(0),
 Y(0),
 Z(0),
 StartFrame(0)
{
}
//*******************************************************************
TSpriteCell::TSpriteCell(TOgleModeWindow *pmw,TFileMode &NewFileMode,
                         TSprite *PSprite,void *PReadData,bool AutoRead)
:TOgleCell(pmw,NewFileMode),
 POwnerSprite(PSprite),
 IsAnm(true),
 Direction(dForward),
// IsForward(true),
 OverrideDefaultXY(false),
 X(0),
 Y(0),
 Z(0),
 StartFrame(0)
{
  Init(PReadData,AutoRead);
}
//*******************************************************************
TSpriteCell::~TSpriteCell()
{
  #if defined(GED_COMPILE)
  ClearCell();
  #endif
}
//*******************************************************************
TAnm *TSpriteCell::CreateAnm(int16 RowNum)
{
  TFileMode FM;
  FM.Name = CellFileName;
//  FM.DirIndex = FileMode.DirIndex;
  FM.SetDirIndex(FileMode.GetDirIndex());
//  #ifdef GED_COMPILE
//  FM.SetRamImmed();
//  #else
  FM.SetBGLo();
//  #endif

  TAnmData ad(X,Y,Z);
  ad.SetDirection(GetDirection());

  // mrp - v2.3
  ad.SetStartFrame(GetStartFrame());

//  if (IsForward) {ad.SetDirFor();}
//  else {ad.SetDirRev();}
  ad.SetUseDefaultXY(!OverrideDefaultXY);

//  int16 spx = POwnerSprite->GetStartingX();
//  int16 spy = POwnerSprite->GetStartingY();
//  int16 spz = POwnerSprite->GetStartingZ();
//  TTriplet triplet(spx,spy,spz);
//  ad.SetOffsetVPt(triplet);

  ad.SetOffsetVPt(POwnerSprite->GetStartingVPt());

  TAnm *PTempAnm = POwnerSprite->CreateAnm(FM,ad);

  #ifdef GED_COMPILE
  PTempAnm->LoadRamImmed();  // just in case.

  if (!OverrideDefaultXY)
  {
    SetX(PTempAnm->GetDefaultScreenX());
    SetY(PTempAnm->GetDefaultScreenY());
  }
  
  if (Get_FUC_Length() == 0 && GetDirection() == dForward) // MRP - v2.3 - jump case added
  {
    int16 fl = PTempAnm->GetFrameNum() - StartFrame;
    if (fl < 0) {fl = 0;}

    Set_FUC_Length(fl);//PTempAnm->GetFrameNum() - StartFrame);  // if ged then get fuc length.
  }
  else if (Get_FUC_Length() == 0 && GetDirection() == dReverse)
  {
    int16 fl = GetStartFrame() + 1;
    if (fl > PTempAnm->GetFrameNum()) {fl = PTempAnm->GetFrameNum();}
    Set_FUC_Length(fl);
  }

  PTempAnm->LoadNone(); // could set to bg lo, later, after memmgr can request filemgr to unload.
  #endif

  // MRP - future - add fuclen to TAnmData.
  PTempAnm->Set_FUC_Length(Get_FUC_Length());  // if not ged, then we don't know real fuc len,
                                                 // so get saved value.
  PTempAnm->SetFUCOffset(RowNum);

  return PTempAnm;
}
//*******************************************************************
TWav *TSpriteCell::CreateWav(int16 RowNum)
{
  TFileMode FM;
  FM.Name = CellFileName;
//  FM.DirIndex = FileMode.DirIndex;
  FM.SetDirIndex(FileMode.GetDirIndex());
  FM.SetBGLo();

  TWav *PTempWav;
  PTempWav = POwnerSprite->CreateWav(FM,int16(FUC_Length));

  #if defined(GED_COMPILE)
  if (FUC_Length == 0 && PTempWav != NULL)
  {
    TOgleFileName ofn(GetFileMgr()->GetCDRomPath(-1));
    ofn.Merge(CellFileName);
    FUC_Length = TWav::GetEstimateFUCLength(ofn.GetFileSize());
    PTempWav->Set_FUC_Length(FUC_Length);
  }
  #endif

  PTempWav->SetFUCOffset(RowNum);
  return PTempWav;
}
//*******************************************************************
void TSpriteCell::AddToOwnersList(int16 RowNum)
{
  #if !defined(NDEBUG)
  bool rc = false;
  assert(POwnerSprite != NULL);
  #endif
  if (IsAnm)
  {
    #if !defined(NDEBUG)
    rc =
    #endif
    POwnerSprite->AddAnm(CreateAnm(RowNum));
  }
  else
  {
    #if !defined(NDEBUG)
    rc =
    #endif
    POwnerSprite->AddWav(CreateWav(RowNum));
  }
  #if !defined(NDEBUG)
  assert(rc);
  #endif
}
//*******************************************************************

