//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oSprite.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/11/95    M.Perry     original baseline
//  2.0     8/21/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__osprite_h)
#include "osprite.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#include <assert.h>

#if !defined(__oanm_h)
#include "oanm.h"
#endif

#if !defined(__owav_h)
#include "owav.h"
#endif

#if !defined(__oanmlst_h)
#include "oanmlst.h"
#endif

#if !defined(__owavlst_h)
#include "owavlst.h"
#endif

#if !defined(__osprtbl_h)
#include "osprtbl.h"
#endif

DIAG_DEFINE_GROUP(OSPRITE,1,0); //name,enabled,level
//*******************************************************************
void TSprite::StartWavNow(int16 wavindex)
{
  TWav *pwav = GetWav(wavindex);
  if (pwav != NULL) {pwav->Start(AskCurrentFUC());}
}
//*******************************************************************
void TSprite::LoadRamAnmList(TFMStatus ramstatus1,TFMStatus ramstatus2)
{
  if (GetAnmList() != NULL) {GetAnmList()->LoadRamAll(ramstatus1,ramstatus2);}
}
//*******************************************************************
void TSprite::LoadRamWavList(TFMStatus ramstatus1,TFMStatus ramstatus2)
{
  if (GetWavList() != NULL) {GetWavList()->LoadRamAll(ramstatus1,ramstatus2);}
}
//*******************************************************************
void TSprite::ReleaseTable()
{
  delete PSpriteTable;
  PSpriteTable = NULL;
}
//*******************************************************************
void TSprite::Slenderize()
{
  if (IsSlenderized()) {return;}
  ReleaseTable();
  SetSlenderized();
}
//*******************************************************************
string TSprite::AddExt()
{
  return string(".sp");
}
//*******************************************************************
const char *TSprite::GetIDChars() const
{
  return "SPR ";
}
//*******************************************************************
int16 TSprite::GetCurrentAnmFrame(int16 anmnum)
{
  TAnm *panm = GetAnm(anmnum);
  if (panm == NULL) {return -1;}
  return panm->CAD.GetCurrentFrame();
}
//*******************************************************************
void TSprite::SetCurrentAnmFrame(int16 framenum,int16 anmnum)
{
  TAnm *panm = GetAnm(anmnum);
  if (panm == NULL) {return;}
  panm->CAD.SetCurrentFrame(framenum);
}
//*******************************************************************
bool TSprite::Priority(int16 count)
{
  int16 i=0;
  if (count > 0) // increase priority.
  {
    dword fuc = AskCurrentFUC() + STARTLOAD_DELTA + 3;  // past autoload kicking in.
    for (i=0;i<count;i++) {Activate(fuc);}
  }
  else          // decrease priority.
  {
    for (i=count;i<0;i++) {DeActivate();}
  }
  return true;
}
//*******************************************************************
#ifdef DEBUGSPRITES
void TSprite::RTReport(ofstream &os)
{
  // name and other data rtreported by TActiveSpriteElement before calling this function.
  os<<"Started: "<<GetStarted()<<"\n";
  os<<"Stopped: "<<GetStopped()<<"\n";
  if (GetAnmList() == NULL) {os<<"Anm List is NULL\n";}
  else                      {GetAnmList()->RTReport(os);}
  if (GetWavList() == NULL) {os<<"Wav List is NULL\n";}
  else                      {GetWavList()->RTReport(os);}
}
#endif
//*******************************************************************
void TSprite::SetToManual()
{
  if (GetAnmList() != NULL) {GetAnmList()->SetAllFUCOffsets(0);}
  if (GetWavList() != NULL) {GetWavList()->SetAllFUCOffsets(0);}
  SetManual();
}
//*******************************************************************
#ifdef DEBUGREPORTS
void TSprite::Report(ofstream &os)
{
  os<<"Sprite Name: "<<FileMode.Name.GetStr()<<"\n";
  os<<"ID Code: "<<GetIDCode()<<"\n";
  os<<"Type: "<<GetType()<<"\n";
  os<<"Length: "<<Get_FUC_Length()<<"\n\n";
  if (GetAnmList() != NULL) {GetAnmList()->Report(os);}
  os<<"\n";
  if (GetWavList() != NULL) {GetWavList()->Report(os);}
}
#endif
//*******************************************************************
void TSprite::AnmError(string spritename,string descr,int16 num)
{
  string str = "Sprite name: ";
  str += spritename;
  str += "\n Sprite file name: ";
  str += FileMode.Name.GetStr();
  str += "\n Description: \n";
  str += descr;
  if (num >= 0)
  {
    str += "\n Num: ";
    char buffer[17];
    itoa(num,buffer,10);
    str += buffer;
  }
  str += "\n Exiting Program";
  FatalError(str,string("Fatal Error - Anm Setup"));
//  ::MessageBox(NULL,str.c_str(),"Error Anm Setup",MB_OK);
//  exit(EXIT_FAILURE);
}
//*******************************************************************
void TSprite::WavError(string spritename,string descr,int16 num)
{
  string str = "Sprite name: ";
  str += spritename;
  str += "\n Sprite file name: ";
  str += FileMode.Name.GetStr();
  str += "\n Description: \n";
  str += descr;
  if (num >= 0)
  {
    str += "\n Num: ";
    char buffer[17];
    itoa(num,buffer,10);
    str += buffer;
  }
  str += "\n Exiting Program";
  FatalError(str,string("Fatal Error - Wav Setup"));
//  ::MessageBox(NULL,str.c_str(),"Error Wav Setup",MB_OK);
//  exit(EXIT_FAILURE);
}
//*******************************************************************
bool TSprite::ReadLocal1_0()
{
  TRACEX(OSPRITE,1,"ReadLocal1_0");
  if (!(Readint16(&WhatType))) {return false;}
  if (GetType() < 0) {SetType(0);}

  bool OverrideDefaultFUCLength;
  if (!Readbool(&OverrideDefaultFUCLength)) {return false;}
  if (OverrideDefaultFUCLength) {Set_FUC_Override();}
  else                          {Clear_FUC_Override();}

  dword TempFUCLength;
  if (!Readdword(&TempFUCLength)) {return false;}
  Set_FUC_Length(TempFUCLength);

  if (!Readint16(&StartVPt.X)) {return false;}
  if (!Readint16(&StartVPt.Y)) {return false;}
  if (!Readint16(&StartVPt.Z)) {return false;}

  PSpriteTable = new TSpriteTable(PModeWin,FileMode,this,GetPtr());
  FileOffset += PSpriteTable->ReadSize;
  #if defined(GED_COMPILE) && defined(WARN_EMPTY_TABLE)
  if (GetTable()->GetFrameCount() <= 0)
  {
    string msg = "Reading sprite '" + FileMode.Name.GetStr() + "' with zero rows in table.";
    ::MessageBox(NULL,msg.c_str(),"Warning !!  Sprite Read",MB_OK|MB_TASKMODAL);
  }
  #endif
  return true;
}
//*******************************************************************
bool TSprite::ReadLocal1_1()
{
  TRACEX(OSPRITE,1,"ReadLocal1_1");
  if (!(Readint16(&WhatType))) {return false;}
  if (GetType() < 0) {SetType(0);}
  if (!Readint16(&IDCode)) {return false;} // this added/different from 1_0
  bool OverrideDefaultFUCLength;
  if (!Readbool(&OverrideDefaultFUCLength)) {return false;}
  if (OverrideDefaultFUCLength) {Set_FUC_Override();}
  else                          {Clear_FUC_Override();}

  dword TempFUCLength;
  if (!Readdword(&TempFUCLength)) {return false;}
  Set_FUC_Length(TempFUCLength);

  if (!Readint16(&StartVPt.X)) {return false;}
  if (!Readint16(&StartVPt.Y)) {return false;}
  if (!Readint16(&StartVPt.Z)) {return false;}

  PSpriteTable = new TSpriteTable(PModeWin,FileMode,this,GetPtr());
  FileOffset += PSpriteTable->ReadSize;
  #if defined(GED_COMPILE) && defined(WARN_EMPTY_TABLE)
  if (GetTable()->GetFrameCount() <= 0)
  {
    string msg = "Reading sprite '" + FileMode.Name.GetStr() + "' with zero rows in table.";
    ::MessageBox(NULL,msg.c_str(),"Warning !!  Sprite Read",MB_OK|MB_TASKMODAL);
  }
  #endif
  return true;
}
//*******************************************************************
bool TSprite::ReadLocal()
{
  switch (MinVer)
  {
    case 1:return ReadLocal1_1();
    default:return ReadLocal1_0();
  }
}
//*******************************************************************
bool TSprite::WriteLocal()
{
  if (GetType() < 0) {SetType(0);}

  if (!Writeint16(&WhatType)) {return false;}
  if (!Writeint16(&IDCode)) {return false;} // MRP 5/29/97
  bool OverrideDefaultFUCLength = Is_FUC_Override();
  if (!Writebool(&OverrideDefaultFUCLength)) {return false;}
  if (!Writedword(&FUC_Length)) {return false;}

  if (!Writeint16(&StartVPt.X)) {return false;}
  if (!Writeint16(&StartVPt.Y)) {return false;}
  if (!Writeint16(&StartVPt.Z)) {return false;}

  #if defined(GED_COMPILE) && defined(WARN_EMPTY_SPRITE)
  if (GetTable() !== NULL && GetTable()->GetFrameCount() <= 0)
  {
    string msg = "Saving sprite '" + GetFName() + "' with zero rows in table.";
    ::MessageBox(NULL,msg.c_str(),"Warning !! Sprite Write",MB_OK|MB_TASKMODAL);
  }
  #endif

  #if defined(GED_COMPILE)
  if (GetTable() == NULL)
  {
    string msg = "(Sprite Table == NULL) while doing write()";
    string title = "Fatal Error - Sprite Write";
    FatalError(msg,title);
  }
  #endif

  if (GetTable() == NULL) {return false;}

  if (!GetTable()->Write(PWriteBuf)) {return false;}

  return true;
}
//*******************************************************************
void TSprite::SetMinVer()
{
  MinVer = 1;
}
//*******************************************************************
bool TSprite::Save()
{
  if (!Is_FUC_Override()) {Set_FUC_Length(GetCalcFUCLength());}
  if (!Write())
  {
    assert(false);
    return false;
  }
  return true;
}
//*******************************************************************
void TSprite::SetAllAnmCADSRelative()
{
  if (GetAnmList() != NULL)
  {
    GetAnmList()->SetAllCADRelative(CurrentVPt);
  }
}
//*******************************************************************
TAnm *TSprite::CreateAnm(TFileMode &fm,TAnmData &ad)
{
  #ifdef DEBUG_PMODEWIN_SPRITE
  if (PModeWin == NULL)
  {
    string s = "CreateAnm()\n";
    s += "PModeWin == NULL,\n";
    s += "Sprite Name - ";
    s += FileMode.Name;
    s += "\nAnm Name - ";
    s += fm.Name;
    ::MessageBox(NULL,s.c_str(),"Warning - Sprite !!!",MB_OK);
  }
  #endif
  return (new TAnm(PModeWin,fm,ad));
}
//*******************************************************************
TWav *TSprite::CreateWav(TFileMode &fm,int16 fuclength)
{
  if (fuclength > 0)
  {
    return (new TWav(PModeWin,fm,fuclength));
  }
  else
  {
    return (new TWav(PModeWin,fm));
  }
}
//*******************************************************************
TSprite::TSprite(TOgleModeWindow *pmw,const TFileMode &NewMode,void *psource)
:TOgleBase(pmw),PSpriteTable(NULL),
 IDCode(0)
{
  PAnmList = new TAnmList(PModeWin,this);
  PWavList = new TWavList(PModeWin,this);

  // since this init doesn't use filemode we have to set up correctly ourselves.
  FileMode = NewMode;
//  FileMode.Name += AddExt();
  FileMode.Name.SetExt(AddExt());

  Init(psource,true);  // read from ptr.
  ResetCurrentXYZ();

  #if defined(GED_COMPILE)
  if (GetTable() == NULL)
  {
    FatalError(string("(Sprite Table == NULL) when needed to createlists"),
               string("Fatal Error - Sprite Ctor"));
  }
  #else
  if (GetTable() != NULL)
  {
    GetTable()->CreateLists();
  }
  #endif

  #if !defined(GED_COMPILE)
  Slenderize();
  #endif

  #ifdef GED_COMPILE
  if (!Is_FUC_Override()) {Set_FUC_Length(GetCalcFUCLength());}
  #endif
}
//*******************************************************************
TSprite::TSprite(TOgleModeWindow *pmw,const TFileMode &NewFileMode,bool AutoRead,bool KeepBaseFile)
:TOgleBase(pmw),PSpriteTable(NULL),
 IDCode(0)
{
  TRACEX(OSPRITE,0,"ctor, name: " << NewFileMode.Name.GetStr());
  PAnmList = new TAnmList(PModeWin,this);
  PWavList = new TWavList(PModeWin,this);
  Init(NewFileMode,AutoRead,KeepBaseFile);
  ResetCurrentXYZ();

  #if !defined(GED_COMPILE)
  if (GetTable() == NULL)
  {
    FatalError(string("(Sprite Table == NULL) when needed to createlists"),
               string("Fatal Error - Sprite Ctor"));
  }
  #else
  if (GetTable() != NULL)
  {
    GetTable()->CreateLists();
  }
  #endif

  #if !defined(GED_COMPILE)
  Slenderize();
  #endif

  #ifdef GED_COMPILE
  if (!Is_FUC_Override()) {Set_FUC_Length(GetCalcFUCLength());}
  #endif
}
//*******************************************************************
TSprite::TSprite(TOgleModeWindow *pmw)
:TOgleBase(pmw),
 IDCode(0)
{
  PAnmList = new TAnmList(PModeWin,this);
  PWavList = new TWavList(PModeWin,this);
  PSpriteTable = new TSpriteTable(PModeWin,this);
}
//*******************************************************************
TSprite::~TSprite()
{
  if (PSpriteTable != NULL)
  {
    delete PSpriteTable;
    PSpriteTable = NULL;
  }

  if (PWavList != NULL)
  {
    delete PWavList;
    PWavList = NULL;
  }

  if (PAnmList != NULL)
  {
    delete PAnmList;
    PAnmList = NULL;
  }
}
//*******************************************************************
dword TSprite::GetCalcFUCLength()
{
  dword AnmLastFUC = 0;
  if (GetAnmList() != NULL) {AnmLastFUC = GetAnmList()->GetLastFUC();}
  dword WavLastFUC = 0;
  if (GetWavList() != NULL) {WavLastFUC = GetWavList()->GetLastFUC();}

  if (AnmLastFUC > WavLastFUC) {return dword(AnmLastFUC + 1);}
  return dword(WavLastFUC + 1);
}
//*******************************************************************
void TSprite::Activate(dword StartFUC)
{
  TOgleBase::Activate(StartFUC);
  if (GetAnmList() != NULL) {GetAnmList()->Activate(StartFUC);}
  if (GetWavList() != NULL) {GetWavList()->Activate(StartFUC);}
}
//*******************************************************************
void TSprite::DeActivate()
{
  if (GetAnmList() != NULL) {GetAnmList()->DeActivate();}
  if (GetWavList() != NULL) {GetWavList()->DeActivate();}
  TOgleBase::DeActivate();
}
//*******************************************************************
bool TSprite::AddAnm(TAnm *PNewAnm)
{
  if (GetAnmList() == NULL) {return false;}
  return (GetAnmList()->AddAnm(PNewAnm));
}
//*******************************************************************
bool TSprite::AddWav(TWav *PNewWav)
{
  if (GetWavList() == NULL) {return false;}
  return (GetWavList()->AddWav(PNewWav));
}
//*******************************************************************
bool TSprite::AddAnm(TFileMode NewFileMode,int16 NewScreenZ)
{
  if (GetAnmList() == NULL) {return false;}
  return (GetAnmList()->AddAnm(NewFileMode,NewScreenZ));
}
//*******************************************************************
bool TSprite::AddAnm(TFileMode NewFileMode,int16 NewScreenX,int16 NewScreenY,int16 NewScreenZ)
{
  if (GetAnmList() == NULL) {return false;}
  return (GetAnmList()->AddAnm(NewFileMode,NewScreenX,NewScreenY,NewScreenZ));
}
//*******************************************************************
bool TSprite::AddWav(TFileMode NewFileMode,int16 NewFUCLength)
{
  if (GetWavList() == NULL) {return false;}
  return (GetWavList()->AddWav(NewFileMode,NewFUCLength));
}
//*******************************************************************
void TSprite::Show()
{
  TOgleBase::Show(); // skip TOgleList::Show() which shows every element.
  if (GetAnmList() != NULL) {GetAnmList()->Show();}
  if (GetWavList() != NULL) {GetWavList()->Show();}
}
//*******************************************************************
void TSprite::NoShow()
{
  TOgleBase::NoShow();  // skip TOgleList::Show() which noshows every element.
  if (GetAnmList() != NULL) {GetAnmList()->NoShow();}
  if (GetWavList() != NULL) {GetWavList()->NoShow();}
}
//*******************************************************************
void TSprite::Start()
{
  TOgleBase::Start();
//  ClearAnmWavListStopped();
  ClearAnmListStopped();
  ClearWavListStopped();
}
//*******************************************************************
void TSprite::Start(dword StartFUC)
{
  #if defined(DEBUGUPDATES)
  BeginFUC = StartFUC;
  PModeWin->DebugUpdate.StartSpriteStart(GetType());
  #endif


  Start();
  if (GetAnmList() != NULL)
  {
    GetAnmList()->Start(StartFUC);
  }

  // Manual is checked because otherwise the sound
  // will play right when sprite activated.
  if (!GetManual() && GetWavList() != NULL)
  {
    GetWavList()->Start(StartFUC);
  }


  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndSpriteStart(GetType());
  #endif
}
//*******************************************************************
void TSprite::Stop()
{
  // right now these won't notify of stoppage.
  // because anm/wav lists don't override stop().

  if (GetAnmList() != NULL)
  {
    GetAnmList()->Stop();
  }

  if (GetWavList() != NULL)
  {
    GetWavList()->Stop();
  }

  TOgleBase::Stop();
//  SetAnmWavListStopped();
  SetAnmListStopped();
  SetWavListStopped();
}
//*******************************************************************
void TSprite::Update(dword NewFUC)
{
  #if defined(DEBUGUPDATES)
  if (PModeWin != NULL)
  {
    PModeWin->DebugUpdate.StartSpriteUpdate(GetType());
  }
  #endif


  if (GetStopped())
  {
    #if defined(DEBUGUPDATES)
    if (PModeWin != NULL)
    {
      PModeWin->DebugUpdate.EndSpriteUpdate(GetType());
    }
    #endif
    return;
  }
  if (GetAnmList() != NULL)
  {
    GetAnmList()->Update(NewFUC);
  }
  if (GetWavList() != NULL)
  {
    GetWavList()->Update(NewFUC);
  }


  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndSpriteUpdate(GetType());
  #endif
}
//*******************************************************************
void TSprite::PostUpdate(dword NewFUC)
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartSpritePUpdate(GetType());
  #endif


  if (GetStopped())
  {
    #if defined(DEBUGUPDATES)
    PModeWin->DebugUpdate.EndSpritePUpdate(GetType());
    #endif
    return;
  }

  if (GetAnmList() != NULL)
  {
    GetAnmList()->PostUpdate(NewFUC);
  }

  if (GetWavList() != NULL)
  {
    GetWavList()->PostUpdate(NewFUC);
  }


  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndSpritePUpdate(GetType());
  if (!GetManual())
  {
    dword fl = BeginFUC + FUC_Length;
    WARN((fl < NewFUC),FileMode.Name.GetStr() << " Sprite still going after fuc_length");
  }
  #endif
}
//*******************************************************************

