//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ofilenam.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//                                         
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  2.0     11/7/97    M.Perry    original baseline
//********************************************************************
#if !defined(__ofilenam_h)
#include "ofilenam.h"
#endif

//#include "oapp.h"

#if !defined(__STDLIB_H)
#include <stdlib.h>
#endif

#if !defined(__IO_H)
#include <io.h>
#endif

#if !defined(__FCNTL_H)
#include <fcntl.h>
#endif

#if !defined(__DIR_H)
#include <dir.h>
#endif

#if !defined(__DIRENT_H)
#include <dirent.h>
#endif

#if !defined(__ofile_h)
#include "ofile.h"
#endif

#if !defined(__omemmgr_h)
#include "omemmgr.h"
#endif
//*******************************************************************
bool TOgleFileName::RemoveRoot(const TOgleFileName &ofnroot)
{
  return RemoveRoot(ofnroot,*this);
}
//*******************************************************************
bool TOgleFileName::RemoveRoot(const string &strroot)
{
  TOgleFileName ofnroot(strroot);
  return RemoveRoot(ofnroot);
}
//*******************************************************************
//  statics
bool TOgleFileName::RemoveRoot(const TOgleFileName &root,TOgleFileName &local)
{
  TOgleFileName ofnlocal(local);
  TOgleFileName ofnroot(root);

  if (!ofnlocal.HasRoot(ofnroot)) {return false;}

  string root_pulleddir,root_restofpath;
  string local_pulleddir,local_restofpath;

  // Remove leftmost dir component from both root and local and then compare
  // pulled dir components. If mismatch or local pull failed then fail_bail.
  // By pulling from root first we know to stop when there is no more
  // leftmost dir's in the root.
  while (ofnroot.PullFirstDir(root_pulleddir,root_restofpath))
  {
    if (!ofnlocal.PullFirstDir(local_pulleddir,local_restofpath)) {return false;}

    root_pulleddir.to_lower();
    local_pulleddir.to_lower();
    
    if (root_pulleddir != local_pulleddir) {return false;}

    ofnroot = root_restofpath;
    ofnlocal = local_restofpath;
  }
  ofnlocal.MassageDir();
  // never failed, so I guess we pass.
  local = ofnlocal;
  return true;
}
//*******************************************************************
//static
bool TOgleFileName::RemoveRoot(const string &strroot,string &strlocal)
{
  TOgleFileName ofnroot(strroot);
  TOgleFileName ofnlocal(strlocal);
  return RemoveRoot(ofnroot,ofnlocal);
}
//*******************************************************************
bool TOgleFileName::LoadBuffer(void *p,int32 size,int32 fileoffset) const
{
  return LoadBuffer(*this,p,size,fileoffset);
}
//*******************************************************************
bool TOgleFileName::WriteBuffer(void *p,int32 size,int32 fileoffset) const
{
  return WriteBuffer(*this,p,size,fileoffset);
}
//*******************************************************************
//static
bool TOgleFileName::LoadBuffer(const TOgleFileName &ofn,void *p,
                               int32 size,int32 fileoffset)
{
  TOgleFile of(ofn,OFA_READ,OFS_READ,OFC_OPEN_EXISTING);
  if (!of.GetFileHandleValid()) {return false;}

  if (of.Seek(fileoffset) == OF_INVALID_SEEK) {return false;}
  if (!of.Read(p,size)) {return false;}
  return true;
}
//*******************************************************************
//static
bool TOgleFileName::WriteBuffer(const TOgleFileName &ofn,void *p,
                                int32 size,int32 fileoffset)
{
  TOgleFile of(ofn,OFA_WRITE,OFS_EXCLUSIVE,OFC_OPEN_ALWAYS);
  if (!of.GetFileHandleValid()) {return false;}

  if (of.Seek(fileoffset) == OF_INVALID_SEEK) {return false;}
  if (!of.Write(p,size)) {return false;}
  return true;
}
//*******************************************************************
bool TOgleFileName::LoadBuffer(const string &s,void *p,int32 size,int32 fileoffset)
{
  TOgleFileName ofn(s);
  return LoadBuffer(ofn,p,size,fileoffset);
}
//*******************************************************************
bool TOgleFileName::WriteBuffer(const string &s,void *p,int32 size,int32 fileoffset)
{
  TOgleFileName ofn(s);
  return WriteBuffer(ofn,p,size,fileoffset);
}
//*******************************************************************
TOgleFileName::TOgleFileName()
{
}
//*******************************************************************
TOgleFileName::TOgleFileName(string filename)
{
  SetFullName(filename);
}
//*******************************************************************
TOgleFileName::TOgleFileName(const TOgleFileName &ofilename)
:strName(ofilename.strName)
{
}
//*******************************************************************
TOgleFileName::~TOgleFileName()
{
}
//*******************************************************************
bool TOgleFileName::CopyFile(const string &dest)
{
  return CopyFile(dest,strName);
}
//*******************************************************************
string TOgleFileName::GetNameParts(TOgleBYTEFlag parts)
{
  string s;
  if (parts.IsSet(ofnDRIVE))
  {
    s = GetDrive();
  }
  if (parts.IsSet(ofnDIRPATH))
  {
    s += GetDirPath();
  }
  if (parts.IsSet(ofnFNAME))
  {
    s += GetFName();
  }
  if (parts.IsSet(ofnEXT))
  {
    s += GetExt();
  }
  return s;
}
//*******************************************************************
void TOgleFileName::ClearParts(TOgleBYTEFlag parts)
{
  if (parts.IsSet(ofnDRIVE))
  {
    SetDrive(TAppString::strEmpty);
  }
  if (parts.IsSet(ofnDIRPATH))
  {
    SetDirPath(TAppString::strEmpty);
  }
  if (parts.IsSet(ofnFNAME))
  {
    SetFName(TAppString::strEmpty);
  }
  if (parts.IsSet(ofnEXT))
  {
    SetExt(TAppString::strEmpty);
  }
}
//*******************************************************************
string TOgleFileName::GetDrive() const
{
  string strdrive,strdir,strfname,strext;
  SplitUp(strdrive,strdir,strfname,strext);
  return strdrive;
}
//*******************************************************************
void TOgleFileName::SetDrive(const string &drive)
{
  string strdrive,strdrive2,strdir,strfname,strext;
  strdrive2 = MassageDriveStr(drive);
  if (strdrive2 != TAppString::strEmpty)
  {
    SplitUp(strdrive,strdir,strfname,strext);         // separate original.
    strName = strdrive2 + strdir + strfname + strext; // piece back together with new drive.
  }
}
//*******************************************************************
string TOgleFileName::GetDirPath() const
{
  string strdrive,strdir,strfname,strext;
  SplitUp(strdrive,strdir,strfname,strext);
  return strdir;
}
//*******************************************************************
void TOgleFileName::SetDirPath(const string &strdirpath)
{
  string s = MassageDirStr(strdirpath);
  string strdrive,strdrive2,strdir,strdir2,strfname,strext;
  SplitUp(s,strdrive,strdir,strfname,strext);
  SplitUp(strdrive2,strdir2,strfname,strext);
  if (!strdrive.is_null())
  {
    strName = strdrive;
  }
  else
  {
    strName = strdrive2;
  }
  strName += strdir + strfname + strext;
}
//*******************************************************************
void TOgleFileName::AppendDirPath(const string &strdirpath)
{
  string strdp(strdirpath);
  string dp = GetDirPath();
  strdp = strdp.strip(string::Leading,'\\');
  dp += strdp;
  SetDirPath(dp);
}
//*******************************************************************
void TOgleFileName::PrePendDirPath(const string &strdirpath)
{
  string dp = strdirpath;
  dp = dp.strip(string::Trailing,'\\');
  dp += GetDirPath();
  SetDirPath(dp);
}
//*******************************************************************
string TOgleFileName::GetFName() const
{
  string strdrive,strdir,strfname,strext;
  SplitUp(strdrive,strdir,strfname,strext);
  return strfname;
}
//*******************************************************************
void TOgleFileName::SetFName(const string &fname)
{
  string strdrive,strdir,strfname,strfname2,strext;
  SplitUp(fname,strdrive,strdir,strfname,strext);
  SplitUp(strdrive,strdir,strfname2,strext);
  strName = strdrive + strdir + strfname + strext;
}
//*******************************************************************
string TOgleFileName::GetExt(bool stripperiod) const
{
  string strdrive,strdir,strfname,strext;
  SplitUp(strdrive,strdir,strfname,strext);
  if (stripperiod)
  {
    strext = strext.strip(string::Leading,'.');
  }
  return strext;
}
//*******************************************************************
void TOgleFileName::SetExt(const string &ext)
{
  string strdrive,strdir,strfname,strext;
  string strdrive2,strdir2,strfname2,strext2;

  SplitUp(ext,strdrive,strdir,strfname,strext); //split param name
  SplitUp(strdrive2,strdir2,strfname2,strext2); // split existing name

  // try to catch ext component passed in without '.' and
  // splitup puts ext into fname component.
  if (strfname.length() > 0 && !ext.contains('.'))
  {
    strext = ".";
    strext += strfname;
  }

//  // limit ext to dot + 3 chars, might be limiting.
//  if (strext.length() > 4) {strext.resize(4);}

  strName = strdrive2 + strdir2 + strfname2 + strext;
}
//*******************************************************************
string TOgleFileName::GetFNameExt() const
{
  string strdrive,strdir,strfname,strext,strfnameext;
  SplitUp(strdrive,strdir,strfname,strext);
  strfnameext = strfname + strext;
  return strfnameext;
}
//*******************************************************************
void TOgleFileName::SetFNameExt(const string &strfnameext)
{
  string strdrive,strdir,strfname,strfname2,strext,strext2;
  SplitUp(strfnameext,strdrive,strdir,strfname,strext);
  SplitUp(strdrive,strdir,strfname2,strext2);
  strName = strdrive + strdir + strfname + strext;
}
//*******************************************************************
void TOgleFileName::SetFullName(const string &strfullname)
{
  strName = strfullname;
}
//*******************************************************************
void TOgleFileName::SetLogicalRoot(const string &strlogicalroot)
{
  string strdrive,strdir,strfname,strext,strtemp;
  strtemp = strlogicalroot;
  strtemp = strtemp.strip(string::Trailing,'\\');
  strtemp += "\\";
  SplitUp(strdrive,strdir,strfname,strext);
  strdir = strdir.strip(string::Leading,'\\');
  strName = strtemp + strdir + strfname + strext;
}
//*******************************************************************
void TOgleFileName::Merge(const string &filename)
{
  string strdrive,strdrive2,strdir,strdir2,strfname,strfname2,strext,strext2;
  SplitUp(strdrive,strdir,strfname,strext);
  SplitUp(filename,strdrive2,strdir2,strfname2,strext2);
  if (!strdir.is_null())
  {
    strdir = strdir.strip(string::Trailing,'\\');
    strdir += "\\";
  }
  if (!strdir2.is_null())
  {
    strdir2 = strdir2.strip(string::Leading,'\\');
  }
  strName = strdrive + strdir + strdir2 + strfname2 + strext2;
}
//*******************************************************************
void TOgleFileName::Merge(const TOgleFileName &ofilename)
{
  Merge(ofilename.strName);
}
//*******************************************************************
TOgleFileName &TOgleFileName::operator=(const TOgleFileName &ofilename)
{
  if (&ofilename == this) {return *this;}
  strName = ofilename.strName;
  return *this;
}
//*******************************************************************
TOgleFileName &TOgleFileName::operator=(const string &filename)
{
  strName = filename;
  return *this;
}
//*******************************************************************
TOgleFileName &TOgleFileName::operator+=(const string &filename)
{
  strName += filename;
  return *this;
}
//*******************************************************************
bool TOgleFileName::PullFirstDir(string &first,string &rest) const
{
  return PullFirstDir(strName,first,rest);
}
//*******************************************************************
bool TOgleFileName::HasRoot(const string &strroot) const
{
  TOgleFileName ofn(strroot);
  return HasRoot(ofn);
}
//*******************************************************************
bool TOgleFileName::HasRoot(const TOgleFileName &ofnroot) const
{
  TOgleFileName ofn(strName);
  TOgleFileName ofnroottemp(ofnroot);

  string d1 = ofnroottemp.GetDrive();
  string d2 = ofn.GetDrive();
  d1.to_lower();
  d2.to_lower();
//  if (ofnroottemp.GetDrive() != ofn.GetDrive()) {return false;}
  if (d1 != d2) {return false;}

  string f1,f2,r1,r2;
  while (ofnroottemp.PullFirstDir(f1,r1))
  {
    if (!ofn.PullFirstDir(f2,r2)) {return false;}
    f1.to_lower();
    f2.to_lower();
    if (f1 != f2) {return false;}
    ofn = r2;
    ofnroottemp = r1;
  }
  return true;
}
//***************** static functions ********************************
void TOgleFileName::SplitUp(const string &fullpath,string &drive,string &dir,string &fname,string &ext)
{
  char szdrive[MAXFILEPARTSIZE];
  char szdir[MAXFILEPARTSIZE];
  char szfname[MAXFILEPARTSIZE];
  char szext[MAXFILEPARTSIZE];

  // mrp - v2.4 - 09/03/99 - trying to make sure everything's lowercase.
  string fp = fullpath;
  fp = fp.strip(string::Both); // strip whitespace from left and right of string.
  fp.to_lower(); // we want to return the parts in lower case.

//  _splitpath(fullpath.c_str(),szdrive,szdir,szfname,szext);
  _splitpath(fp.c_str(),szdrive,szdir,szfname,szext);

  drive = szdrive;
  if (!drive.is_null())
  {
    drive = drive.strip(string::Trailing,':');
    drive += ":";
  }
  dir = szdir;
  dir = dir.strip(string::Both,'\\');
  if (!dir.is_null())
  {
    dir.prepend('\\');
    dir.append('\\');
  }
  else if (!drive.is_null())
  {
    dir = "\\";
  }
  fname = szfname;
  ext = szext;
}
//*******************************************************************
string TOgleFileName::GetExt(const string &pathname,bool stripperiod)
{
  string strdrive,strdir,strfname,strext;
  SplitUp(pathname,strdrive,strdir,strfname,strext);
  if (stripperiod)
  {
    strext = strext.strip(string::Leading,'.');
  }
  return strext;
}
//*******************************************************************
bool TOgleFileName::PullFirstDir(const string &pathname,string &first,string &rest)
// looking back: it seems that if a drive component is present it is completely
//               ignored.
// returns: if there is any dir components passed in by pathname then,
//          rc == true,
//          first == first dir component,
//          rest == rest of path (no leading backslashes)
//
//          if only fname and/or fext then
//          rc == false;
//          first == ""
//          rest == fnameext
{
  string strdrive,strdir,strfname,strext,strtemp;
  SplitUp(pathname,strdrive,strdir,strfname,strext);  //remove drive
  strtemp = strdir + strfname + strext;               //path & fname & ext
  strtemp = strtemp.strip(string::Leading,'\\');
  size_t firstslash = strtemp.find_first_of("\\");
  if (firstslash == NPOS) // no slashes, all that's left is fnameext.
  {
    first = TAppString::strEmpty;
    rest = strtemp;
    return false;
  }
  else
  {
    first = strtemp;
    first.resize(firstslash + 1);
    rest = strtemp.substr(firstslash + 1);
    return true;
  }
}
//*******************************************************************
string TOgleFileName::PullLastDir(const string &pathname,bool keeplastslash)
{
  string strlastdir = TAppString::strEmpty;
  string strdrive,strdir,strfname,strext;

  SplitUp(pathname,strdrive,strdir,strfname,strext);
  if (strdir == TAppString::strEmpty) {return strlastdir;}

  strdir = strdir.strip(string::Trailing,'\\');  //remove trailing slash.

  size_t lastslash = strdir.find_last_of("\\");

  if (lastslash == NPOS)
  {
    strlastdir = strdir;
  }
  else
  {
    strlastdir = strdir.substr(lastslash + 1);
  }

  if (keeplastslash) {strlastdir += "\\";}

  return strlastdir;
}
//*******************************************************************
bool TOgleFileName::DirExists(const string &fulldir)
{
  string strdrive,strdir,strfname,strext;
  SplitUp(fulldir,strdrive,strdir,strfname,strext);
  if (strdir == TAppString::strEmpty) {return false;}

  strdir = strdir.strip(string::Trailing,'\\');
  string drivedir = strdrive + strdir;
  DIR *pdir = NULL;
  pdir = opendir(drivedir.c_str());
  if (pdir == NULL) {return false;}
  closedir(pdir);
  return true;
}
//*******************************************************************
bool TOgleFileName::CreateDir(const string &dir)
// note: walk thru dir parts creating each subdir as we go.
{
  string strdrive,strdir,strfname,strext;
  SplitUp(dir,strdrive,strdir,strfname,strext);

  string quickcheck = strdrive + strdir;
  if (DirExists(quickcheck)) {return true;}

  string strfirst,strrest,strfront;
  strfront = strdrive;
  if (!strdrive.is_null()) {strfront += "\\";}

  while (PullFirstDir(strdir,strfirst,strrest))
  {
    strfront += strfirst;
    strdir = strrest;
    string str = strfront;
//    str = str.strip(string::Trailing,'\\');
    if (!DirExists(str))
    {
      if (mkdir(str.c_str()) != 0) {return false;}
    }
  }
  return true;
}
//*******************************************************************
//char TOgleFileName::ConvertDriveStrToChar(string drive)
//{
//  char D;
//  string strdrive,strdir,strfname,strext;
//  SplitUp(drive,strdrive,strdir,strfname,strext);
//  if (strdrive.is_null())
//  {
//    if (drive.is_null()) {D = '\0';}
//    else {D = drive[0];}
//  }
//  else
//  {
//    D = strdrive[0];
//  }
//  return D;
//}
//*******************************************************************
int TOgleFileName::ConvertDriveLetterToInt(const string &drive)
{
  char D;
//  string strdrive,strdir,strfname,strext;
//  SplitUp(drive,strdrive,strdir,strfname,strext);
//  if (strdrive.is_null())
//  {
//    if (drive.is_null()) {return -1;}
//    D = drive[0];
//  }
//  else
//  {
//    D = strdrive[0];
//  }
//  D = ConvertDriveStrToChar(drive);
  D = GetDriveChar(drive);
  return (ConvertDriveLetterToInt(D));
}
//*******************************************************************
int TOgleFileName::ConvertDriveLetterToInt(char drive)
{
  char TempChar = (char)toupper(drive);
  if (!isalpha(TempChar)) {return -1;}

  int i = toascii(TempChar) - toascii('A') + 1;
  return (i);
}
//*******************************************************************
char TOgleFileName::GetDriveChar(const string &drive)
{
//  return ConvertIntToDriveLetter(ConvertDriveLetterToInt(drive));
//  return ConvertDriveStrToChar(drive);
  char D;
  string strdrive,strdir,strfname,strext;
  SplitUp(drive,strdrive,strdir,strfname,strext);
  if (strdrive.is_null())
  {
    if (drive.is_null()) {D = '\0';}
    else {D = drive[0];}
  }
  else
  {
    D = strdrive[0];
  }
  return D;
}
//*******************************************************************
char TOgleFileName::ConvertIntToDriveLetter(int drive)
{
  char buffer[17];
  if ((drive > 26) || (drive < 0)) {return 'a';}
  itoa(drive,buffer,10);
  return buffer[0];
}
//*******************************************************************
int32 TOgleFileName::GetFileSize(const string &fullpathname)
{
  TOgleFile ofile(fullpathname);
  return ofile.GetFileSize();
}
//*******************************************************************
int32 TOgleFileName::GetFileSize(const char *filename)
{
  return GetFileSize(string(filename));
}
//*******************************************************************
bool TOgleFileName::CopyFile(const string &dest,const string &source)
{
  if (dest == source) {return true;} // don't want to copy one unto oneself,
                                     // but it's still desired result,
                                     // so rc == true

  bool rc = false;
  TOgleFile ofsrc(source);
  if (!ofsrc.GetFileHandleValid()) {return false;}
//  TOgleFile ofdest(dest,OF_CREATE | OF_WRITE);
  TOgleFile ofdest(TOgleFileName(dest),
                   OFA_WRITE,
                   OFS_EXCLUSIVE,
                   OFC_CREATE_ALWAYS);
//                   OFC_OPEN_ALWAYS | OFC_TRUNCATE_EXISTING);
  if (!ofdest.GetFileHandleValid()) {return false;}
  int32 size = ofsrc.GetFileSize();
  void huge *p = TOgleMemMgr::AllocPtr(GMEM_MOVEABLE,size);
  if (p == NULL) {return false;}
  if (ofsrc.Read(p,size))
  {
    rc = ofdest.Write(p,size);
  }
  TOgleMemMgr::DeAllocPtr(p);
  return rc;
}
//*******************************************************************
bool TOgleFileName::DeleteFile(const string &fullpathname)
{
  return bool(::remove(fullpathname.c_str()) == 0);
}
//*******************************************************************
bool TOgleFileName::FileExists(const char *fullpathname)
{
//  OFSTRUCT tempOFSTRUCT;
//  if (OpenFile(fullpathname,&tempOFSTRUCT,OF_EXIST) == -1) {return false;}
//  return true;

  string s(fullpathname);
  return FileExists(s);
}
//*******************************************************************
bool TOgleFileName::FileExists(const string &fullpathname)
{
//  return FileExists(fullpathname.c_str());
  TOgleFileName ofn(fullpathname);
  return FileExists(ofn);
}
//*******************************************************************
//static
bool TOgleFileName::FileExists(const TOgleFileName &ofn)
{
  TOgleFile of(ofn,OFA_NONE,OFS_READWRITE,OFC_OPEN_EXISTING);
  return of.GetFileHandleValid();
}
//*******************************************************************
string TOgleFileName::MassageDriveStr(const string &drive)
{
  char d = GetDriveChar(drive);
  if (d == '\0') {return TAppString::strEmpty;}
  string s = d;
  s += ":";
  return s;
}
//*******************************************************************
string TOgleFileName::MassageDirStr(const string &dir)
{
  string s;
  if (dir.contains('.'))
  {
    string strdrive,strdir,strfname,strext;
    SplitUp(dir,strdrive,strdir,strfname,strext);
    s = strdrive + strdir;
  }
  else
  {
    s = dir;
    s = s.strip(string::Trailing,'\\');
    s += "\\";
  }
  return s;
}
//***************** friend functions ********************************
TOgleFileName operator+(const TOgleFileName &ofilename1,const TOgleFileName &ofilename2)
{
  TOgleFileName ofilename3(ofilename1);
  ofilename3.Merge(ofilename2);
//  ofilename3 += ofilename2;
  return ofilename3;
}
//*******************************************************************
