//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  omoney.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//   1.3    5/15/97     M.Perry    changed money from long to TMoney.
//   2.0    8/26/97     M.Perry    32-bit conversion.
//   2.3    07/14/99    M.Perry    added functs w/ const params
//********************************************************************
#include "omoney.h"

//*******************************************************************
bool TMoney::MoneyDenomIDInitialized = false;
TMoney TMoney::MoneyDenomID[did_NumIDs] = {0.0,
                                           0.01,
                                           0.05,
                                           0.10,
                                           0.25,
                                           0.50,
                                           1.0,
                                           2.0,
                                           5.0,
                                           10.0,
                                           25.0,
                                           50.0,
                                           100.0,
                                           500.00,
                                           1000.0};
//*******************************************************************
TMoney::TMoney(int16 ctortype,int16 data)
:Value(0.0)
{
  switch(ctortype)
  {
    case ctorint16:Assign(data);break;
    case ctorDenomID:SetDenomID(data);break;
    default:break;
  }
}
//*******************************************************************
int16 TMoney::GetDenomID()
{
  for (int16 i = 0;i<did_NumIDs;i++)
  {
    if (MoneyDenomID[i] == Value)
    {
      return i;
    }
  }
  return did_Invalid;
}
//*******************************************************************
void TMoney::SetDenomID(int16 did)
{
  if (CheckDenomID(did))
  {
    Assign(MoneyDenomID[did]);
  }
  else
  {
    Assign(0.0);
  }
}
//*******************************************************************
void TMoney::RoundTo5Cent()
{
  if (Value == 0.0) {return;} // quick out.

  int16 cents = GetCents();
  if (cents == 0) {return;}   // quick out.

  int16 sign = GetSign();
  int32 dollars = GetDollars();

  int16 x = int16(cents % 5);
  if (x == 4) {cents++;}
  else if (x == 3) {cents += int16(2);}
  else if (x == 2) {cents -= int16(2);}
  else if (x == 1) {cents -= int16(1);}

  double d = cents;
  d /= 100.0;
  d += dollars;
  d *= sign;
  Value = d;
}
//*******************************************************************
int16 TMoney::GetCents() const
{
  double d = Value;
  if (d < 0) {d *= -1;}  // make positive.
  d *= 100.0;
  d += 0.1;  // fucked up work around.
  int32 l = (int32)d;
  l %= 100;
  return (int16)l;
}
//*******************************************************************
string TMoney::GetStr(bool showcents,bool showzerocents,bool showcommas) const
{
  char buffer[35];
  string s;
  if (GetSign() == -1)
  {
    s = "-";
  }
  s += "$";
  ltoa(GetDollars(),buffer,10);
  s += buffer;

  string digitsonly(buffer);

  if (showcents)
  {
    int16 cents = GetCents();
    if (cents <= 0)
    {
      if (showzerocents)
      {
        s += ".00";
      }
    }
    else if (cents < 10)
    {
      s += ".0";
      ltoa(cents,buffer,10);
      s += buffer;
    }
    else
    {
      s += ".";
      ltoa(cents,buffer,10);
      s += buffer;
    }
  }

  if (showcommas)
  {
    int16 do_len = int16(digitsonly.length());
    int16 s_len = int16(s.length());
    if (do_len > 3)
    {
      s.insert(s_len - 6,string(','));
    }
    if (do_len > 6)
    {
      s.insert(s_len - 9,string(','));
    }
    if (do_len > 9)
    {
      s.insert(s_len - 12,string(','));
    }
  }

  return s;
}
//*******************************************************************
TMoney Abs(const TMoney &money)
{
  TMoney m(money);
  m.Abs();
  return m;
}
//*******************************************************************
TMoney operator%(const TMoney &m1,int32 cash)
{
  TMoney m(m1);
  m %= cash;
  return m;
}
//*******************************************************************
TMoney operator%(const TMoney &m1,int16 cash)
{
  TMoney m(m1);
  m %= cash;
  return m;
}
//*******************************************************************
TMoney operator%(const TMoney &m1,int cash)
{
  TMoney m(m1);
  m %= cash;
  return m;
}
//*******************************************************************
TMoney operator%(const TMoney &m1,double cash)
{
  TMoney m(m1);
  m %= cash;
  return m;
}
//*******************************************************************
TMoney operator%(const TMoney &m1,TMoney &m2)
{
  TMoney m(m1);
  m %= m2;
  return m;
}
//*******************************************************************

