//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name: osliq.cpp
//    Project Name: Ogle
// Original Author: Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     3/13/95    M.Perry     original baseline
//  2.0     8/21/97    M.Perry     32-bit conversion.
//********************************************************************
#include "osliq.h"

//*******************************************************************
#pragma argsused
TCompareResult DefaultCompareFunct(void *FirstObj,void *SecObj)
// -1 less than, 0 equal to, 1 greater than
{
  return EQUAL_TO;
}
//*******************************************************************
TSliq::TSliq(bool AllowEqObjPtr,bool ownobjs)
:OwnObjs(ownobjs),
 Left(NULL),
 Right(NULL),
 TravPtr(NULL),
 RemovedEndNode(false),
 LDirLToR(true),
 AllowEq(AllowEqObjPtr),
 RecCount(0),
 CachingRecHolders(true),
 CachedRecHolders(NULL)
{
  PCompareFunct = DefaultCompareFunct;
}
//*******************************************************************
TSliq::~TSliq()
// note: base class doesn't use OwnObjs because deleting a void ptr is pretty shitty.
{
//  if (OwnObjs)
//  {
//    void *p;
//    while ((p = Pop()) != NULL)
//    {
//      delete p;
//    }
//  }
//  else
//  {
//    do {} while (Pop() != NULL);
//  }
  while (Pop() != NULL) {}  // empty out the list so we can release recholders.

  Left = NULL;
  Right = NULL;
  TravPtr = NULL;
  LDirLToR = true;
  RecCount = 0;
  PCompareFunct = NULL;
  while (CachedRecHolders != NULL)
  {
    TRecHolder *prh = CachedRecHolders;
    if (prh != NULL)
    {
      CachedRecHolders = prh->Next;
      delete prh;
    }
  }
}
//*******************************************************************
bool TSliq::CheckObjPtr(void *NewObj)
{
  TRecHolder *L;
  TRecHolder *ListHead;

  ListHead = Left;
  L = ListHead;

  while (L != NULL)
  {
    if (L->ObjHolder == NewObj)
    {
      return true;
    }
    else {L = L->Next;}
  }//end while
  return false;
}
//*******************************************************************
bool TSliq::Push(void *NewObj)
{
  TRecHolder *RecHolder;
  TRecHolder *StackTop;
  TRecHolder *StackBottom;

  if (!AllowEq)
  {
    if (CheckObjPtr(NewObj))
    {
      return false;
    }
  }

  StackTop = Left;
  StackBottom = Right;

  RecHolder = GetRecHolder();
  if (RecHolder == NULL)
  {
    return false;
  }

  if (StackTop == NULL)
  {
    RecHolder->ObjHolder = NewObj;
    RecHolder->Next = NULL;
    RecHolder->Prev = NULL;
    StackTop = RecHolder;
    StackBottom = RecHolder;
  }
  else
  {
    RecHolder->ObjHolder = NewObj;
    RecHolder->Next = StackTop;
    RecHolder->Prev = NULL;
    StackTop->Prev = RecHolder;
    StackTop = RecHolder;
  }
  Left = StackTop;
  Right = StackBottom;
  IncRecCount();

  return true;
}
//*******************************************************************
bool TSliq::IndexAdd(int16 indexnum,void *NewObj)
{
  if (indexnum == 0)
  {
    return Push(NewObj);
  }
  else if (indexnum >= GetRecCount())
  {
    return false;
  }
  else if (indexnum < 0)
  {
    return false;
  }
  else
  {
    TRecHolder *L;
    TRecHolder *ListHead;
    ListHead = Left;
    int16 Counter;
    for (L=ListHead,Counter = 0;
         ((L != NULL) && (Counter < indexnum));
         L = L->Next,Counter++)
    {}
    if (L!=NULL)
    {
      TRecHolder *PRecHolder = GetRecHolder();
      PRecHolder->ObjHolder = NewObj;
      PRecHolder->Next = L;
      PRecHolder->Prev = L->Prev;
      L->Prev->Next = PRecHolder;
      L->Prev = PRecHolder;
      IncRecCount();
      return true;
    }
    else
    {
      return false;
    }
  }
}
//*******************************************************************
bool TSliq::QFrontAdd(void *NewObj)
{
  TRecHolder *RecHolder;
  TRecHolder *QueFront;
  TRecHolder *QueBack;

  if (!AllowEq)
  {
    if (CheckObjPtr(NewObj))
    {
      return false;
    }
  }

  QueFront = Right;
  QueBack = Left;

  RecHolder = GetRecHolder();
  if (RecHolder == NULL)
  {
    return false;
  }

  if (QueFront == NULL)
  {
    RecHolder->ObjHolder = NewObj;
    RecHolder->Next = NULL;
    RecHolder->Prev = NULL;
    QueBack = RecHolder;
    QueFront = RecHolder;
  }
  else
  {
    RecHolder->ObjHolder = NewObj;
    RecHolder->Next = NULL;
    RecHolder->Prev = QueFront;
    QueFront->Next = RecHolder;
    QueFront = RecHolder;
  }
  Left=QueBack;
  Right=QueFront;
  IncRecCount();

  return true;
}
//*******************************************************************
void *TSliq::Pop()
{
  void *OldObj = NULL;
  TRecHolder *RecHolder;
  TRecHolder *StackTop;
  TRecHolder *StackBottom;

  StackTop = Left;
  StackBottom = Right;


  if (StackTop == NULL)
  {
    return NULL;
  }

  RecHolder = StackTop;
  OldObj = RecHolder->ObjHolder;
  StackTop = RecHolder->Next;
  if (StackTop != NULL) {StackTop -> Prev = NULL;}
  if (StackTop == NULL) {StackBottom = NULL;}
  if (TravPtr == RecHolder)
  {
    if (LDirLToR) {TravPtr = RecHolder->Prev;}
    else {TravPtr = RecHolder->Next;}
  }
  ReleaseRecHolder(RecHolder);
  Left = StackTop;
  Right = StackBottom;
  DecRecCount();

  return OldObj;
}
//*******************************************************************
void *TSliq::QRemove()
{
  void *OldObj = NULL;
  TRecHolder *RecHolder;
  TRecHolder *QueFront;
  TRecHolder *QueBack;

  QueFront = Right;
  QueBack = Left;

  if (QueFront == NULL)
  {
    return NULL;
  }

  RecHolder = QueFront;
  OldObj = RecHolder->ObjHolder;
  QueFront = RecHolder->Prev;
  if (QueFront != NULL) {QueFront->Next = NULL;}
  if (QueFront == NULL) {QueBack = NULL;}
  if (TravPtr == RecHolder)
  {
    if (LDirLToR) {TravPtr = RecHolder->Prev;}
    else {TravPtr = RecHolder->Next;}
  }
  ReleaseRecHolder(RecHolder);
  Left = QueBack;
  Right = QueFront;
  DecRecCount();

  return OldObj;
}
//*******************************************************************
bool TSliq::LMatchRemove(void *OldObj)
{
  TRecHolder *ListHead;
  TRecHolder *ListTail;
  TRecHolder *L;
  ListHead = Left;
  ListTail = Right;

  if ((OldObj == NULL) || (ListHead == NULL))
  {
    return false;
  }

  L = ListHead;
  bool Found = false;
  bool Out = false;
  while (!Out)
  {
    if (L == NULL) {Out=true;}
    else
    {
     if (L->ObjHolder == OldObj)
     {
       Found = true;
       Out = true;
       if (L == ListHead)
       {
         ListHead = L->Next;
         if (ListHead == NULL) {ListTail = NULL;}
         else {ListHead->Prev = NULL;}
       }
       else if (L == ListTail)
       {
         ListTail = L->Prev;
         if (ListTail == NULL) {ListHead = NULL;}
         else {ListTail->Next = NULL;}
       }
       else
       {
          (L->Prev)->Next = L->Next;
          (L->Next)->Prev = L->Prev;
       }
     }
     else {L = L -> Next;}
    }
  }// end while


  if (Found)
  {
    if (TravPtr == L)
    {
      if (LDirLToR) {TravPtr = L->Prev;}
      else {TravPtr = L->Next;}
      if (TravPtr == NULL) {RemovedEndNode = true;}
    }
    ReleaseRecHolder(L);
    Left = ListHead;
    Right = ListTail;
    DecRecCount();
  }

  return Found;
}
//*******************************************************************
void *TSliq::IndexGet(int16 indexnum)
{
  TRecHolder *L;
  TRecHolder *ListHead = Left;
  int16 Counter;
  for (L=ListHead,Counter = 0;
       ((L != NULL) && (Counter < indexnum));
       L=L->Next,Counter++)
  {}
  if (L!=NULL) {return L->ObjHolder;}
  else {return NULL;}
}
//*******************************************************************
void *TSliq::IndexRemove(int16 indexnum)
{
  if ((indexnum < 0) || (indexnum >= GetRecCount()))
  {
    return NULL;
  }
  else if (indexnum == 0)
  {
    return Pop();
  }
  else if (indexnum == GetLastNum())
  {
    return QRemove();
  }
  else
  {
    TRecHolder *L;
    TRecHolder *ListHead = Left;
    int16 Counter;
    for (L=ListHead,Counter = 0;
         ((L != NULL) && (Counter < indexnum));
         L = L->Next,Counter++)
    {}
    if (L != NULL)
    {
      (L->Prev)->Next = L->Next;
      (L->Next)->Prev = L->Prev;
      void *OH = L->ObjHolder;
      ReleaseRecHolder(L);
      DecRecCount();
      return OH;
    }
    else
    {
      return NULL;
    }
  }
}
//*******************************************************************
void TSliq::Insertion_Sort(void **a,int16 n)
{
  int16 j,p;
  void  *tmp;

  for(p=1;p < n;p++)
  {
    tmp = a[p];
	 for(j=p;(PCompareFunct(tmp,a[j-1]) == LESS_THAN);j--)
	 {
      a[j] = a[j-1];
	   if(j == 1 )
	   {
        j--;
        break;
      }
    }
    a[j] = tmp;
  }
}
//*******************************************************************
void TSliq::Q_Sort(void **a,int16 left,int16 right)
{
	int16 i,j;
	int16 pivot;

	if((left + SORT_CUTOFF) <= right)
   {
     pivot = Median3(a,left,int16(right-1));
     i=left;j=right;
     for(;;)
     {
       while(PCompareFunct(a[++i],a[pivot])== LESS_THAN);
       while(PCompareFunct(a[--j],a[pivot])== GREATER_THAN);
       if(i < j)
       {
         Swap(a,i,j);
       }
       else
       {
         break; // Out of infinite loop.
       }
     }
     Swap(a,i,int16(right-1));  // Restore pivot
     Q_Sort(a,left,int16(i-1));
     Q_Sort(a,int16(i+1),right);
   }
}
//*******************************************************************
bool TSliq::SliqSort()
{
  int16 NumOfElements;
  TRecHolder *ListHead;

  NumOfElements = GetRecCount();

  if (NumOfElements > 0)
  {
    void **P;
    P = new void *[NumOfElements];

    if (P == NULL)
    {
      return false;
    }

    ListHead = Left;

    for (int16 i=0;i<NumOfElements;i++)
    {
      P[i] = ListHead->ObjHolder;
      ListHead = ListHead->Next;
    }

    Quick_Sort(P,NumOfElements);

    ListHead = Left;
    for (int16 j=0;j<NumOfElements;j++)
    {
      ListHead->ObjHolder = P[j];
      ListHead = ListHead->Next;
    }

    delete [] P;
  }
  return true;
}
//*******************************************************************

