//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  odet.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//                          
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/28/96    M.Perry     original baseline
//  2.0     8/26/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__odet_h)
#include "odet.h"
#endif

#if !defined(__odirent_h)
#include "odirent.h"
#endif

#if !defined(__omisc_h)
#include "omisc.h"
#endif

#ifdef GED_COMPILE
#if !defined(__odetdlg_h)
  #include "odetdlg.h"
#endif
#if !defined(__IOSTREAM_H)
  #include <iostream.h>
#endif
#if !defined(__FSTREAM_H)
  #include <fstream.h>
#endif
#endif
//*******************************************************************
string TDet::AddExt()
{
  return string(".det");
}
//*******************************************************************
const char *TDet::GetIDChars() const
{
  return "DET ";
}
//*******************************************************************
TDet::TDet(const TFileMode &fm,bool AutoRead,bool KeepBaseFile)
:TOgleBase(NULL),
 UsageCount(0),
 PDirEntry(NULL),
// Name(fm.Name)
 Name(fm.Name)
{
//  DewName = fm.Name + ".dew";
  DewName.SetFName(fm.Name.GetFName());
  DewName.SetExt(".dew");
  Init(fm,AutoRead,KeepBaseFile);
}
//*******************************************************************
TDet::TDet()
:TOgleBase(NULL),
 UsageCount(0),
 #if defined(GED_COMPILE)
 PDetScript(NULL),
 PWin(NULL),
 PDewFile(NULL),
 #endif
 PDirEntry(NULL)
{
}
//*******************************************************************
TDet::~TDet()
{
  DestroyDirEntry();
}
//*******************************************************************
TDirEntry *TDet::GetDirEntry(string &pathname)
{
  if (PDirEntry == NULL) {return NULL;}
  string first;
  string rest;
  TOgleFileName::PullFirstDir(pathname,first,rest);
  if ((first == "") && (rest == "")) {return PDirEntry;}
  return PDirEntry->GetDirEntry(pathname);
}
//*******************************************************************
#ifdef GED_COMPILE
void TDet::DoDlg(TWindow *pwin)
{
  TDetDlg *pdd = new TDetDlg(PDetScript,pwin,Root);
  pdd->Execute();
  delete pdd;
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
bool TDet::WriteOpenCDRomFile()
{
//  string CDRomFullPathName;
//  CDRomFullPathName = DestDir + Name + AddExt();

  TOgleFileName CDRomFullPathName;
  CDRomFullPathName.SetDirPath(DestDir);
  CDRomFullPathName.SetFName(Name.GetFName());
  CDRomFullPathName.SetExt(AddExt());
  PWriteBuf = new TFileWriteBuf(CDRomFullPathName.GetStr());
  return (PWriteBuf->IsOpen());
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
bool TDet::WriteLocal()
{
  if (PDirEntry != NULL)
  {
    if (!PDirEntry->Write()) {return false;}
  }
  return true;
}
#endif
//*******************************************************************
bool TDet::ReadLocal()
{
  PDirEntry = new TDirEntry(this);
  return PDirEntry->Read();
}
//*******************************************************************
#ifdef GED_COMPILE
void TDet::Edit(TWindow *pwin,string &hddrive,string &hddir,string &desname)
{
  Root = hddrive + hddir;
  CreateDetScript(hddrive,hddir,desname);
  DoDlg(pwin);
  DestroyDetScript();
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TDet::CreateDetScript(string &hddrive,string &hddir,string &desname)
{
  if (PDetScript != NULL) {return;}
  PDetScript = new TDetScript(hddrive,hddir,desname);
  PDetScript->Initialize();
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TDet::DestroyDetScript()
{
  if (PDetScript != NULL)
  {
    PDetScript->DeInitialize();
    delete PDetScript;
    PDetScript = NULL;
  }
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TDet::Create(TWindow *pwin,string &hddrive,string &hddir,string &desname,const string &destdir)
{
  PWin = pwin;
  Name = desname;

  DestDir = destdir;
  // add last slash if not there because we're assuming no filename on end.
//  if (DestDir.find_last_of("\\") != (strlen(DestDir.c_str()) - 1))
//  {
//    DestDir += "\\";
//  }
  DestDir = DestDir.strip(string::Trailing,'\\');
  DestDir += "\\";

  // verify/create dest dir.
  if (!TOgleFileName::DirExists(DestDir))
  {
    pwin->MessageBox("Creating Dir","Wad Create",MB_OK);
    TOgleFileName::CreateDir(DestDir);
  }

  CreateDetScript(hddrive,hddir,desname); // read script
  Root = hddrive + hddir;                 // set root to where scripts are found.
  CreateDirEntry();                       // using script read dir structure.
  CreateDewFile();                        // actual wad.
  CreateDetDataFile();                    // wad table, using write's.

  DestroyDirEntry();
  DestroyDetScript();                     // get rid of script.
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
bool TDet::IsDirOk(string &dirfromroot,string &dir)
{
  if (PDetScript == NULL) {return false;}

  if ((dir == ".\\") || (dir == "..\\")) {return false;}

  string fulldir = dirfromroot + dir;

  // check excludes
  int16 numexcs = PDetScript->GetNumExcs();
  for (int16 i=0;i<numexcs;i++)
  {
    TDetExcData *pexcdata = PDetScript->GetExcData(i);
    if ((pexcdata != NULL) &&
        pexcdata->ContainsDir(fulldir))
    {
      return false;
    }
  }

  // check includes.
  int16 numincs = PDetScript->GetNumIncs();
  for (int16 j=0;j<numincs;j++)
  {
    TDetIncData *pincdata = PDetScript->GetIncData(j);
    if ((pincdata != NULL) &&
        pincdata->ContainsDir(fulldir))
    {
      return true;
    }
  }
  return false;
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
bool TDet::IsExtOk(string &dirfromroot,string &name)
// not checking if dir is excluded because we shouldn't be here if excluded.
{
  if (PDetScript == NULL) {return false;}

  string ext = TOgleFileName::GetExt(name,true);

  int16 numincs = PDetScript->GetNumIncs();
  for (int16 i=0;i<numincs;i++)
  {
    TDetIncData *pincdata = PDetScript->GetIncData(i);
    if ((pincdata != NULL) &&
        pincdata->ContainsDir(dirfromroot) &&
        pincdata->ContainsExt(ext))
    {
      return true;
    }
  }
  return false;
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
bool TDet::IsNameOk(string &dirfromroot,string &name)
{
  if (PDetScript == NULL) {return false;}

  int16 numexcnames = PDetScript->GetNumExcNames();
  for (int16 i=0;i<numexcnames;i++)
  {
    TDetExcNameData *pdend = PDetScript->GetExcNameData(i);
    if (pdend != NULL &&
        pdend->IsMatch(dirfromroot,name))
    {
      return false;
    }
  }
  return true;
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TDet::CreateDirEntry()
{
//  PDirEntry = new TDirEntry(this,string(""),string(""));
  PDirEntry = new TDirEntry(this,TAppString::strEmpty,TAppString::strEmpty);
  PDirEntry->Sort();


//  string str = GetRoot() + Name + ".txt";
  TOgleFileName fn;
  fn.SetLogicalRoot(GetRoot());
  fn.SetFName(Name.GetFName());
  fn.SetExt(".txt");
  ofstream os(fn.GetFullName().c_str());
//  ofstream os(str.c_str());
  if (!os)
  {
    if (PWin != NULL) {PWin->MessageBox("couldn't create output stream","Error - DirEntry Report",MB_OK);}
    else {::MessageBox(NULL,"couldn't create output stream","Error - DirEntry Report",MB_OK);}
  }
  else
  {
    PDirEntry->Report(os);
  }
}
#endif
//*******************************************************************
void TDet::DestroyDirEntry()
{
  delete PDirEntry;
  PDirEntry = NULL;
}
//*******************************************************************
#ifdef GED_COMPILE
void TDet::CreateDetDataFile()
{
  Write();
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TDet::CreateDewFile()
{
//  string pathname = DestDir + Name + ".dew";
  TOgleFileName pathname;
  pathname.SetLogicalRoot(DestDir);
  pathname.SetFName(Name.GetFName());
  pathname.SetExt(".dew");
//  PDewFile = new TOgleFile(pathname.GetFullName(),OF_CREATE | OF_READWRITE);
  PDewFile = new TOgleFile(pathname,OFA_READWRITE,OFS_EXCLUSIVE,OFC_CREATE_ALWAYS);
  bool rc = PDirEntry->CreateDewFile();
  delete PDewFile;
  PDewFile = NULL;
  if (!rc) {PWin->MessageBox("TDirEntry failure.","Error - Create Dew File",MB_OK);}
}
#endif
//*******************************************************************

