//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  odetscr.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/27/96    M.Perry    original baseline
//  2.0     8/26/97    M.Perry    32-bit conversion.
//********************************************************************
#include "odetscr.h"

#define SCRIPTFILEEXT ".des"
#define INFOSECTION "info"
#define EXTSECTION "ext"
#define INCSECTION "inc"
#define EXCSECTION "exc"
#define EXCNAMESECTION "excname"
#define ROOTKEY "root"
#define DESTKEY "dest"
#define NAMEKEY "name"
#define NUMKEY "num"
#define NUMINCKEY "numinc"
#define NUMEXCKEY "numexc"
#define NUMEXCNAMEKEY "numexcname"
#define RECURSEKEY "recurse"
#define NUMEXTSKEY "numexts"
//*******************************************************************
TDetExcData::TDetExcData()
{
  Recurse = false;
}
//*******************************************************************
TDetExcData::~TDetExcData()
{
}
//*******************************************************************
TDetExcData::TDetExcData(TDetExcData &ded)
{
  Name = ded.Name;
  Recurse = ded.Recurse;
}
//*******************************************************************
TDetExcData::TDetExcData(const TDetExcData &ded)
{
  Name = ded.Name;
  Recurse = ded.Recurse;
}
//*******************************************************************
TDetExcData &TDetExcData::operator=(TDetExcData &ded)
{
  Name = ded.Name;
  Recurse = ded.Recurse;
  return *this;
}
//*******************************************************************
TDetExcData &TDetExcData::operator=(const TDetExcData &ded)
{
  Name = ded.Name;
  Recurse = ded.Recurse;
  return *this;
}
//*******************************************************************
bool TDetExcData::ContainsDir(string &dir)
{
  if (dir == Name) {return true;}  // same
  else if ((Name == "") && Recurse) {return true;} // root is recursive
  else if ((dir.find_first_of(Name) == 0) && Recurse) {return true;} // incdir is beg of dir && incdir is recursive
  else {return false;}  // no match.
}
//*******************************************************************
//*******************************************************************
TDetIncData::TDetIncData()
{
  NumExts = 13;
  Recurse = false;
  Exts[0] = "bmp";
  Exts[1] = "m";
  Exts[2] = "g";
  Exts[3] = "anm";
  Exts[4] = "go";
  Exts[5] = "sp";
  Exts[6] = "sk";
  Exts[7] = "spl";
  Exts[8] = "skc";
  Exts[9] = "skl";
  Exts[10] = "wav";
  Exts[11] = "pal";
  Exts[12] = "md";
}
//*******************************************************************
TDetIncData::TDetIncData(TDetIncData &did)
{
  Name = did.Name;
  Recurse = did.Recurse;
  NumExts = did.NumExts;
  for (int16 i=0;i<NumExts;i++)
  {
    Exts[i] = did.Exts[i];
  }
}
//*******************************************************************
TDetIncData::TDetIncData(const TDetIncData &did)
{
  Name = did.Name;
  Recurse = did.Recurse;
  NumExts = did.NumExts;
  for (int16 i=0;i<NumExts;i++)
  {
    Exts[i] = did.Exts[i];
  }
}
//*******************************************************************
TDetIncData::~TDetIncData()
{
}
//*******************************************************************
TDetIncData &TDetIncData::operator=(TDetIncData &did)
{
  Name = did.Name;
  Recurse = did.Recurse;
  NumExts = did.NumExts;
  for (int16 i=0;i<NumExts;i++)
  {
    Exts[i] = did.Exts[i];
  }
  return *this;
}
//*******************************************************************
TDetIncData &TDetIncData::operator=(const TDetIncData &did)
{
  Name = did.Name;
  Recurse = did.Recurse;
  NumExts = did.NumExts;
  for (int16 i=0;i<NumExts;i++)
  {
    Exts[i] = did.Exts[i];
  }
  return *this;
}
//*******************************************************************
string TDetIncData::GetExt(int16 index)
{
  if (!CheckIndex(index)) {return string();}
  return Exts[index];
}
//*******************************************************************
void TDetIncData::SetExt(int16 index,string &ext)
{
  if (!CheckIndex(index)) {return;}
  Exts[index] = ext;
}
//*******************************************************************
bool TDetIncData::AddExt(string &ext)
{
  if (NumExts >= MAX_EXTS) {return false;}
  SetExt(NumExts,ext);
  NumExts++;
  return true;
}
//*******************************************************************
bool TDetIncData::RemoveExt(int16 index)
{
  string *pexts = new string[NumExts];
  for (int16 i=0;i<NumExts;i++)
  {
    pexts[i] = Exts[i];
  }
  int16 counter = 0;
  for (int16 j=0;j<NumExts;j++)
  {
    if (index != j)
    {
      Exts[counter] = pexts[j];
      counter++;
    }
  }
  NumExts--;
  delete [] pexts;
  return true;
}
//*******************************************************************
bool TDetIncData::ContainsExt(string &ext)
{
  for (int16 i=0;i<NumExts;i++)
  {
    if (Exts[i] == ext) {return true;}
  }
  return false;
}
//*******************************************************************
bool TDetIncData::ContainsDir(string &dir)
{
  if (dir == Name) {return true;}  // same
  else if ((Name == "") && Recurse) {return true;} // root is recursive
  else if ((dir.find_first_of(Name) == 0) && Recurse) {return true;} // incdir is beg of dir && incdir is recursive
  else {return false;}  // no match.
}
//*******************************************************************
//*******************************************************************
TDetExcNameData::TDetExcNameData()
{
}
//*******************************************************************
TDetExcNameData::~TDetExcNameData()
{
}
//*******************************************************************
TDetExcNameData::TDetExcNameData(TDetExcNameData &dend)
{
  Name = dend.Name;
}
//*******************************************************************
TDetExcNameData::TDetExcNameData(const TDetExcNameData &dend)
{
  Name = dend.Name;
}
//*******************************************************************
TDetExcNameData &TDetExcNameData::operator=(TDetExcNameData &dend)
{
  if (&dend == this) {return *this;}
  Name = dend.Name;
  return *this;
}
//*******************************************************************
TDetExcNameData &TDetExcNameData::operator=(const TDetExcNameData &dend)
{
  if (&dend == this) {return *this;}
  Name = dend.Name;
  return *this;
}
//*******************************************************************
bool TDetExcNameData::IsMatch(string &dirfromroot,string &name)
{
  string s = dirfromroot + name;
  if (s == Name) {return true;}
  return false;
}
//*******************************************************************
//*******************************************************************
bool TDetScript::AddInc(TDetIncData &did)
{
  if ((NumIncs + 1) > MAX_INCS) {return false;}
  bool rc = SetIncData(NumIncs,did);
  if (rc) {NumIncs++;}
  return rc;
}
//*******************************************************************
bool TDetScript::RemoveInc(int16 index)
{
  TDetIncData *pdetincdata = new TDetIncData[NumIncs];
  for (int16 i=0;i<NumIncs;i++)
  {
    pdetincdata[i] = DetIncData[i];
  }
  int16 counter = 0;
  for (int16 j=0;j<NumIncs;j++)
  {
    if (j != index)
    {
      DetIncData[counter] = pdetincdata[j];
      counter++;
    }
  }
  NumIncs--;
  delete [] pdetincdata;
  return true;
}
//*******************************************************************
bool TDetScript::AddExc(TDetExcData &ded)
{
  if (( NumExcs + 1) > MAX_EXCS) {return false;}
  bool rc = SetExcData(NumExcs,ded);
  if (rc) {NumExcs++;}
  return rc;
}
//*******************************************************************
bool TDetScript::RemoveExc(int16 index)
{
  TDetExcData *pdetexcdata = new TDetExcData[NumExcs];
  for (int16 i=0;i<NumExcs;i++)
  {
    pdetexcdata[i] = DetExcData[i];
  }
  int16 counter = 0;
  for (int16 j=0;j<NumExcs;j++)
  {
    if (j != index)
    {
      DetExcData[counter] = pdetexcdata[j];
      counter++;
    }
  }
  NumExcs--;
  delete [] pdetexcdata;
  return true;
}
//*******************************************************************
bool TDetScript::AddExcName(TDetExcNameData &dend)
{
  if ((NumExcNames + 1) >= MAX_EXC_NAMES) {return false;}
  bool rc = SetExcNameData(NumExcNames,dend);
  if (rc) {NumExcNames++;}
  return rc;
}
//*******************************************************************
bool TDetScript::RemoveExcName(int16 index)
{
  TDetExcNameData *pdend = new TDetExcNameData[NumExcNames];
  for (int16 i=0;i<NumExcNames;i++)
  {
    pdend[i] = DetExcNameData[i];
  }
  int16 counter = 0;
  for (int16 j=0;j<NumExcNames;j++)
  {
    if (j != index)
    {
      DetExcNameData[counter] = pdend[j];
      counter++;
    }
  }
  NumExcNames--;
  delete [] pdend;
  return true;
}
//*******************************************************************
TDetScript::TDetScript(string &hddrive,string &hddir,string &filename)
{
  PScript = NULL;
  Initialized = false;
  HDDrive = hddrive;
  HDDir = hddir;
  FileName = filename;
  FileName.to_lower();
  FileNameandExt = FileName + SCRIPTFILEEXT;
}
//*******************************************************************
TDetScript::~TDetScript()
{
  if (Initialized) {DeInitialize();}
}
//*******************************************************************
bool TDetScript::Initialize()
{
  PScript = new TIniFile(HDDrive,HDDir,FileNameandExt);
  Initialized = true;
  bool rc = Read();
  return rc;
}
//*******************************************************************
bool TDetScript::DeInitialize()
{
  Write();
  delete PScript;
  PScript = NULL;
  Initialized = false;
  return true;
}
//*******************************************************************
bool TDetScript::Read()
{
  if (PScript == NULL) {return false;}
  ReadInfo();
  ReadIncs();
  ReadExcs();
  ReadExcNames();
  return true;
}
//*******************************************************************
bool TDetScript::Write()
{
  if (PScript == NULL) {return false;}
  WriteInfo();
  WriteIncs();
  WriteExcs();
  WriteExcNames();
  return true;
}
//*******************************************************************
bool TDetScript::ReadInfo()
{
  NumIncs = (int16)PScript->ReadInt(INFOSECTION,NUMINCKEY,0);
  NumExcs = (int16)PScript->ReadInt(INFOSECTION,NUMEXCKEY,0);
  NumExcNames = (int16)PScript->ReadInt(INFOSECTION,NUMEXCNAMEKEY,0);
  return true;
}
//*******************************************************************
bool TDetScript::WriteInfo()
{
  PScript->WriteInt(INFOSECTION,NUMINCKEY,NumIncs);
  PScript->WriteInt(INFOSECTION,NUMEXCKEY,NumExcs);
  PScript->WriteInt(INFOSECTION,NUMEXCNAMEKEY,NumExcNames);
  return true;
}
//*******************************************************************
string TDetScript::GetIncSectionStr(int16 index)
{
  string s = INCSECTION;
  char buffer[17];
  itoa(index,buffer,10);
  s += buffer;
  return s;
}
//*******************************************************************
string TDetScript::GetExcSectionStr(int16 index)
{
  string s = EXCSECTION;
  char buffer[17];
  itoa(index,buffer,10);
  s += buffer;
  return s;
}
//*******************************************************************
string TDetScript::GetExcNameSectionStr(int16 index)
{
  string s = EXCNAMESECTION;
  char buffer[17];
  itoa(index,buffer,10);
  s += buffer;
  return s;
}
//*******************************************************************
bool TDetScript::ReadIncs()
{
  for (int16 i=0;i<NumIncs;i++)
  {
    TDetIncData *pdid = new TDetIncData;
    string IncSection = GetIncSectionStr(i);
    pdid->Name = PScript->ReadString(IncSection,NAMEKEY,"");
    pdid->Recurse = PScript->ReadInt(IncSection,RECURSEKEY,0);
    pdid->NumExts = (int16)PScript->ReadInt(IncSection,NUMEXTSKEY,0);
    for (int16 j=0;j<pdid->NumExts;j++)
    {
      string extname = PScript->ReadString(IncSection,j,"");
      pdid->SetExt(j,extname);
    }
    SetIncData(i,*pdid);
    delete pdid;
    pdid = NULL;
  }
  return true;
}
//*******************************************************************
bool TDetScript::WriteIncs()
{
  for (int16 i=0;i<NumIncs;i++)
  {
    string IncSection = GetIncSectionStr(i);
    TDetIncData *pdid = GetIncData(i);
    if (pdid != NULL)
    {
      PScript->WriteString(IncSection,NAMEKEY,pdid->Name);
      PScript->WriteInt(IncSection,RECURSEKEY,pdid->Recurse);
      PScript->WriteInt(IncSection,NUMEXTSKEY,pdid->NumExts);
      for (int16 j=0;j<pdid->NumExts;j++)
      {
        string strext = pdid->GetExt(j);
        PScript->WriteString(IncSection,j,strext);//pdid->GetExt(j));
      }
    }
  }
  return true;
}
//*******************************************************************
bool TDetScript::ReadExcs()
{
  for (int16 i=0;i<NumExcs;i++)
  {
    string ExcSection = GetExcSectionStr(i);
    TDetExcData *pded = new TDetExcData;
    pded->Name = PScript->ReadString(ExcSection,NAMEKEY,"");
    pded->Recurse = PScript->ReadInt(ExcSection,RECURSEKEY,0);
    SetExcData(i,*pded);
    delete pded;
    pded = NULL;
  }
  return true;
}
//*******************************************************************
bool TDetScript::WriteExcs()
{
  for (int16 i=0;i<NumExcs;i++)
  {
    string ExcSection = GetExcSectionStr(i);
    TDetExcData *pded = GetExcData(i);
    if (pded != NULL)
    {
      PScript->WriteString(ExcSection,NAMEKEY,pded->Name);
      PScript->WriteInt(ExcSection,RECURSEKEY,pded->Recurse);
    }
  }
  return true;
}
//*******************************************************************
bool TDetScript::ReadExcNames()
{
  for (int16 i=0;i<NumExcNames;i++)
  {
    TDetExcNameData *pdend = new TDetExcNameData;
    string excnamesection = GetExcNameSectionStr(i);
    pdend->Name = PScript->ReadString(excnamesection,NAMEKEY,"");
    SetExcNameData(i,*pdend);
    delete pdend;
    pdend = NULL;
  }
  return true;
}
//*******************************************************************
bool TDetScript::WriteExcNames()
{
  for (int16 i=0;i<NumExcNames;i++)
  {
    string s = GetExcNameSectionStr(i);
    TDetExcNameData *pdend = GetExcNameData(i);
    if (pdend != NULL)
    {
      PScript->WriteString(s,NAMEKEY,pdend->Name);
    }
  }
  return true;
}
//*******************************************************************

