//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oaudwmix.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     10/18/96   M.Perry    original baseline
//  2.0     8/23/97    M.Perry    32-bit conversion.
//  2.2    06/01/99    M.Perry    fixed focus problem.
//********************************************************************
#if !defined(__oaudwmix_h)
#include "oaudwmix.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__osndmgr_rh)
#include "osndmgr.rh"
#endif

#if !defined(__owav_h)
#include "owav.h"
#endif

#if !defined(__oinifile_h)
#include "oinifile.h"
#endif

#if !defined(__osys_h)
#include "osys.h"
#endif

#include <assert.h>
//*******************************************************************
TAudioData::TAudioData(TAudioWaveMix *paudiomgr,string filename)
:PAudioMgr(paudiomgr)
{
  FileName = filename;
  PMixWave = NULL;
  OwnerCount = 0;
  Size = 0;
  Status = TFileMode::fm_None;
//  for (int16 i=0;i<NUM_TFMSTATUS;i++)
  for (int16 i=0;i<TFileMode::fm_Num_Status;i++)
  {
    StatusCounts[i] = 0;
  }
}
//*******************************************************************
TAudioData::~TAudioData()
{
  if (PMixWave != NULL) {UnLoad();}
}
//*******************************************************************
bool TAudioData::Load(TWav *pwav)
{
  if (PMixWave != NULL) {return true;}
  if (pwav == NULL || !pwav->GetRamLoaded()) {return false;}

  MMIOINFO mmioInfo;
  setmem(&mmioInfo,sizeof(MMIOINFO),'\0');
  mmioInfo.fccIOProc = FOURCC_MEM;
  mmioInfo.pchBuffer= (char huge *)pwav->GetRamPtr();
  dword sizeptr = pwav->GetSizeRamPtr();
  mmioInfo.cchBuffer = sizeptr;
  Size = sizeptr;
  PAudioMgr->IncSizeUsed(sizeptr);
  PMixWave = TWaveMix::Dll()->OpenWave(PAudioMgr->hMixSession,
                                       (LPSTR )(&mmioInfo),
                                       NULL,
                                       WMIX_MEMORY);
  return (PMixWave != NULL);
}
//*******************************************************************
void TAudioData::UnLoad()
{
  if (PMixWave == NULL) {return;}
  PAudioMgr->DecSizeUsed(Size);
  Size = 0;
  TWaveMix::Dll()->FreeWave(PAudioMgr->hMixSession,PMixWave);
  PMixWave = NULL;
}
//*******************************************************************
//*******************************************************************
TAudioObj::TAudioObj()
{
  Status = TFileMode::fm_None;
  PAudioData = NULL;
}
//*******************************************************************
TAudioObj::~TAudioObj()
{
}
//*******************************************************************
//*******************************************************************
TAudioDataSliq::TAudioDataSliq(TAudioWaveMix *paudiomgr)
:TSliq(false,true),
 PAudioMgr(paudiomgr)
{
}
//*******************************************************************
TAudioDataSliq::~TAudioDataSliq()
{
  for (TAudioData *pad = PopAD();pad != NULL;pad = PopAD())
  {
    delete pad;
  }
}
//*******************************************************************
TAudioData *TAudioDataSliq::GetAudioData(const string &name)
{
  for (TAudioData *pad = GetFirst();pad != NULL;pad = GetNext())
  {
    if (pad->FileName == name) {return pad;}
  }
  return NULL;
}
//*******************************************************************
bool TAudioDataSliq::UnLoadBelow(TFMStatus status,dword sizeneeded,dword &sizesofar)
{
  for (TAudioData *pad = GetFirst();pad != NULL;pad = GetNext())
  {
    if (pad->IsLoaded() && pad->Status <= status && !PAudioMgr->IsPlaying(pad->PMixWave))
    {
      sizesofar += pad->Size;
      pad->UnLoad();
      if (sizesofar >= sizeneeded) {return true;}
    }
  }
  return false;
}
//*******************************************************************
void TAudioDataSliq::DestroyNoOwners()
{
  for (TAudioData *pad = GetFirst();pad != NULL;pad = GetNext())
  {
    if (pad->GetOwnerCount() == 0 &&             // no owners
        !PAudioMgr->IsPlaying(pad->PMixWave) &&  // not playing
        LMatchRemove(pad))                       // remove from list
    {
      delete pad;                                // delete it.
    }
  }
}
//*******************************************************************
//*******************************************************************
TAudioDSArray::TAudioDSArray(TAudioWaveMix *paudiomgr)
:PAudioMgr(paudiomgr),
 CurrentList(0)
{
  for (int16 i=0;i<NUMAUDIODATASLIQS;i++)
  {
    PDS[i] = new TAudioDataSliq(paudiomgr);
  }
}
//*******************************************************************
TAudioDSArray::~TAudioDSArray()
{
  for (int16 i=0;i<NUMAUDIODATASLIQS;i++)
  {
    delete PDS[i];
    PDS[i] = NULL;
  }
}
//*******************************************************************
int16 TAudioDSArray::Hash(const string &name)
{
  unsigned long n = 0;
  for (const char *ptr = name.c_str();*ptr;++ptr)
  {
    n <<= 1;
    n += *ptr;
  }
  return (int16)(n % (unsigned long)NUMAUDIODATASLIQS);
}
//*******************************************************************
bool TAudioDSArray::UnLoadBelow(TFMStatus status,dword sizeneeded,dword &sizesofar)
{
  int16 counter = NUMAUDIODATASLIQS - 1;
  while (counter > 0)
  {
    if (PDS[CurrentList]->UnLoadBelow(status,sizeneeded,sizesofar))
    {
      return true;
    }
    CurrentList++;
    if (CurrentList >= NUMAUDIODATASLIQS) {CurrentList = 0;}
    counter--;
  }
  return false;
}
//*******************************************************************
void TAudioDSArray::DestroyNoOwners()
{
  for (int16 i=0;i<NUMAUDIODATASLIQS;i++)
  {
    PDS[i]->DestroyNoOwners();
  }
}
//*******************************************************************
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TChannelWindow,TOgleHideWin/*TDialog*/)
	EV_MESSAGE(MM_WOM_DONE,EvMMWomDone),
END_RESPONSE_TABLE;
//*******************************************************************
TChannelWindow::TChannelWindow(TWaveMixControlData *pcontroldata,TWindow *pwin)
//:TDialog(pwin,SND_DIALOG_1),
:TOgleHideWin(pwin,string("Hidden Audio Window")),
 PWin(pwin),
 PControlData(pcontroldata),
 Active(false)
{
}
//*******************************************************************
TChannelWindow::~TChannelWindow()
{
}
//*******************************************************************
LRESULT TChannelWindow::EvMMWomDone(WPARAM wParam,LPARAM lParam)
{
  int16 channel = (int16)wParam;
  LPMIXWAVE pmixwave = (LPMIXWAVE)lParam;
  if (Active) {PControlData->NotifyStop(channel,pmixwave);}
  return true;
}
//*******************************************************************
//*******************************************************************
TWaveMixControlData::TWaveMixControlData(TAudioWaveMix *paudio,int16 channel)
:PAudioWaveMix(paudio),
 Channel(channel),
 Valid(false),
 PWav(NULL),
 LoopCount(0),
 Submitted(false),
 PChannelWindow(NULL)
{
  PChannelWindow = new TChannelWindow(this,PAudioWaveMix->PWin);
  PChannelWindow->Create();

  MixPlayParams.wSize = sizeof(MIXPLAYPARAMS);
  MixPlayParams.hMixSession = PAudioWaveMix->hMixSession;
  MixPlayParams.hWndNotify = PChannelWindow->HWindow;
  MixPlayParams.iChannel = Channel;
}
//********************************************************************
TWaveMixControlData::~TWaveMixControlData()
{
  // stop any possible wavs first.
  delete PChannelWindow;
  PChannelWindow = NULL;
  Valid = false;
}
//*******************************************************************
void TWaveMixControlData::Prep(TWav *pwav,WORD loopcount)
{
  PWav = pwav;
  LoopCount = loopcount;
  MixPlayParams.wLoops = loopcount;
  Submitted = false;
  WavWantsNotifyStop = true;

  TAudioObj *pao = (TAudioObj *)PWav->PSndMgrData;
  MixPlayParams.lpMixWave = pao->PAudioData->PMixWave;

  assert(MixPlayParams.lpMixWave); // shouldn't happen :-)

  Valid = true;
}
//*******************************************************************
void TWaveMixControlData::Submit(dword flags)
{
  MixPlayParams.dwFlags = flags;

  assert(MixPlayParams.lpMixWave);

  // last check for valid data.
  if (MixPlayParams.lpMixWave == NULL)
  {
    Valid = false;
    if (PWav != NULL && WavWantsNotifyStop)
    {
      PWav->NotifyStop();
    }
    return;
  }

  uint result = TWaveMix::Dll()->Play(&MixPlayParams);

  if (result != MMSYSERR_NOERROR) // error playing data.
  {
    PAudioWaveMix->MMErrorBox(result);
    if (PWav != NULL && WavWantsNotifyStop)
    {
      PWav->NotifyStop();
    }
    Valid = false;
    return;
  }

  PChannelWindow->SetActive(true);
  Submitted = true;
}
//*******************************************************************
void TWaveMixControlData::Stop()
// just clear channel window's pwav ptr so that late coming WOM_DONE msg's don't hose us.
{
  if (!Valid) {return;}
  if (WavWantsNotifyStop) {PWav->NotifyStop();}
  PChannelWindow->SetActive(false);
  if (Submitted)
  {
    TWaveMix::Dll()->FlushChannel(PAudioWaveMix->hMixSession,Channel,0);
  }
  MixPlayParams.lpMixWave = NULL;
  PWav = NULL;
  Submitted = false;
  Valid = false;
}
//*******************************************************************
#pragma argsused
void TWaveMixControlData::NotifyStop(int16 channel,LPMIXWAVE pmixwave)
{
  // make sure not an old notify coming down the pike.
  if (pmixwave == MixPlayParams.lpMixWave)
  {
    Submitted = false; //keeps from flushing channel when notified by wavemix that it's done.
    Stop();
  }
}
//*******************************************************************
//*******************************************************************
TPlayQ::TPlayQ()
:TSliq(false,false)
{
}
//*******************************************************************
TPlayQ::~TPlayQ()
{
  Stop();
}
//*******************************************************************
void TPlayQ::Stop(bool notifystop)
{
  TPlayQData *pdata = Remove();
  while (pdata != NULL)
  {
    #if defined(OGLE_DEBUG_SOUND)
    ::MessageBox(NULL,"playq entry being stopped","Warning! Audio",MB_OK | MB_TASKMODAL);
    #endif
    if (notifystop && pdata->PWav != NULL) {pdata->PWav->NotifyStop();}
    delete pdata;

    // get next element
    pdata = Remove(); //mrp - v2.4 - 09/15/99 - was missing,cause of sound gpf?
  }
}
//*******************************************************************
bool TPlayQ::Remove(TWav *pwav,bool allmatches)
{
  bool found = false;
  for (TPlayQData *pdata = GetFirstData();
       pdata != NULL;
       pdata = GetNextData())
  {
    if (pdata->PWav == pwav)
    {
      found = true;
      LMatchRemove(pdata);
      delete pdata;
      if (!allmatches) {return true;}
    }
  }

  return found;
}
//*******************************************************************
bool TPlayQ::IsPlaying(LPMIXWAVE lpmixwave)
{
  TAudioObj *pao = NULL;
  for (TPlayQData *pdata = GetFirstData();pdata != NULL;pdata = GetNextData())
  {
    if (pdata->PWav != NULL)
    {
      pao = (TAudioObj *)pdata->PWav->PSndMgrData;
      if (lpmixwave == pao->PAudioData->PMixWave) {return true;}
    }
  }
  return false;
}
//*******************************************************************
//*******************************************************************
TAudioWaveMix::TAudioWaveMix(TWindow *pwin)
:PWin(pwin),
 Active(false),
 hMixSession(NULL),
 Muted(false),
 Initialized(false),
 PDSArray(NULL),
 SizeUsed(0),
 ErrorCount(0),
 LastTimeCount(0)
{
  POgleApp = TOgleApp::GetOgleApp(true);
  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    PControlData[i] = NULL;
  }
}
//*******************************************************************
TAudioWaveMix::~TAudioWaveMix()
{
  DeInitialize();
}
//*******************************************************************
bool TAudioWaveMix::Initialize()
{
  if (IsInitialized()) {return true;}
  ErrorCount = 0;
  LastTimeCount = 0;
  if (GetNumDevs() == 0)
  {
    ErrorBox("System reports there is no wave out devices");
  }

  if (!TWaveMix::LoadDll()) {return false;}

  Configure(); // sets hMixSession
  if (hMixSession == NULL)
  {
    ErrorBox("Couldn't initialize WaveMix");
    TWaveMix::UnLoadDll();
    return false;
  }
  Activate();

  if (!Active)
  {
    TWaveMix::Dll()->CloseSession(hMixSession);
    ErrorBox("Attempting to reload WaveMix");
    TWaveMix::Dll()->SysReLoad(); // force windows to dump from memory and reload library.
    Configure(); // sets hMixSession
    if (hMixSession == NULL)
    {
      ErrorBox("Couldn't initialize WaveMix");
      TWaveMix::UnLoadDll();
      return false;
    }
    Activate();
    if (!Active) {return false;}
  }
  // open all channels right away.
  uint result = TWaveMix::Dll()->OpenChannel(hMixSession,NUM_AUDIO_CHANNELS,WMIX_ALL);
  if (result != MMSYSERR_NOERROR)
  {
    MMErrorBox(result);
    return false;
  }
  CreateControlData();
  PDSArray = new TAudioDSArray(this); // set up audio data sliqs.
  GetIniValues();

  Initialized = true;
  return true;
}
//*******************************************************************
bool TAudioWaveMix::DeInitialize()
{
  if (!IsInitialized()) {return true;}

  // since we opened all channels then we must close all channels.
  SetIniValues();
  Stop();

  DestroyControlData();
  TWaveMix::Dll()->CloseChannel(hMixSession,NUM_AUDIO_CHANNELS,WMIX_ALL);

  delete PDSArray;   // let go of loaded wav's before closing session.
  PDSArray = NULL;

  TWaveMix::Dll()->CloseSession(hMixSession);

  TWaveMix::UnLoadDll();
  Initialized = false;
  return true;
}
//*******************************************************************
void TAudioWaveMix::CreateControlData()
{
  for (int16 i = 0;i < NUM_AUDIO_CHANNELS;i++)
  {
    PControlData[i] = new TWaveMixControlData(this,i);
  }
}
//*******************************************************************
void TAudioWaveMix::DestroyControlData()
{
  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    delete PControlData[i];
    PControlData[i] = NULL;
  }
}
//*******************************************************************
string TAudioWaveMix::GetMMErrorStr(uint err)
{
  string s("Unknown error");
  switch (err)
  {
    case MMSYSERR_NOERROR      :s = "No error";break;
    case MMSYSERR_ERROR        :s = "Unspecified error";break;
    case MMSYSERR_BADDEVICEID  :s = "Device ID out of range";break;
    case MMSYSERR_NOTENABLED   :s = "Driver failed enable";break;
    case MMSYSERR_ALLOCATED    :s = "Device already allocated";break;
    case MMSYSERR_INVALHANDLE  :s = "Device Handle is invalid";break;
    case MMSYSERR_NODRIVER     :s = "No Device Driver present";break;
    case MMSYSERR_NOMEM        :s = "Memory allocation error";break;
    case MMSYSERR_NOTSUPPORTED :s = "Function isn't supported";break;
    case MMSYSERR_BADERRNUM    :s = "Error value out of range";break;
    case MMSYSERR_INVALFLAG    :s = "Invalid flag passed";break;
    case MMSYSERR_INVALPARAM   :s = "Invalid parameter passed";break;
  }
  return s;
}
//*******************************************************************
void TAudioWaveMix::ErrorBox(string msg)
{
  ErrorCount++;
  if (ErrorCount <= 5) // we limit the number of error's shown to ErrorCount.
  {
    PWin->MessageBox(msg.c_str(),"Error - Audio",MB_OK);
  }
}
//*******************************************************************
void TAudioWaveMix::Activate()
{
  if (hMixSession == NULL) {return;}

  if (Active) {return;}
  uint result = TWaveMix::Dll()->Activate(hMixSession,true);
  if (result != MMSYSERR_NOERROR)
  {
    MMErrorBox(result);
  }
  else
  {
    Active = true;
  }
}
//*******************************************************************
void TAudioWaveMix::DeActivate()
{
  if (hMixSession == NULL) {return;}

  if (!Active) {return;}
  TWaveMix::Dll()->Activate(hMixSession,false);
  Active = false;
}
//*******************************************************************
void TAudioWaveMix::SetMute(bool mute)
{
  // possibly free up/acquire resources.
  if (IsMuted() == mute) {return;}
  if (mute) {Stop(true);} // send NotifyStop's.
  Muted = mute;
  SetIniValues();
}
//*******************************************************************
void TAudioWaveMix::GetIniValues()
{
  TOgleIniFile *pini = TOgleApp::GetOgleApp(true)->GetIniFile();
  int16 on = int16(pini->GetEffects());
  if (on == 0)
  {
    Muted = true;
  }
  else
  {
    Muted = false;
  }
}
//*******************************************************************
void TAudioWaveMix::SetIniValues()
{
  TOgleIniFile *pini = TOgleApp::GetOgleApp(true)->GetIniFile();
  int16 on = Muted?int16(0):int16(1);
  pini->SetEffects(on);
}
//*******************************************************************
void TAudioWaveMix::Stop(bool notifystop)
{
  if (hMixSession == NULL) {return;}

  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (!notifystop) {PControlData[i]->WavWantsNotifyStop = false;}
    PControlData[i]->Stop();
  }
  TWaveMix::Dll()->FlushChannel(hMixSession,NUM_AUDIO_CHANNELS,WMIX_ALL);
  PlayQ.Stop(notifystop);
}
//*******************************************************************
bool TAudioWaveMix::CheckLoadStatus(TWav *pwav)
{
  // the following sets up PMixWave if not already loaded.
  TAudioObj *pao = (TAudioObj *)pwav->PSndMgrData;
  if (pao == NULL || pao->PAudioData == NULL) {return false;}
  pao->PAudioData->DecCount(pao->Status);
  pao->PAudioData->IncCount(TFileMode::fm_Immed);
  pao->PAudioData->SetStatus();
  pao->Status = TFileMode::fm_Immed;
  if (!pao->PAudioData->IsLoaded())
  {
    if (!pwav->GetRamLoaded())
    {
      TFMStatus status = pwav->FileMode.GetRamStatus();
      pwav->LoadRamImmed();

      assert(pwav->GetRamLoaded()); // just in case.

      pao->PAudioData->Load(pwav);
      pwav->LoadRam(status);
    }
  }
  return true;
}
//*******************************************************************
bool TAudioWaveMix::Add(TWav *pwav,WORD loopcount)
{
  if (!Active || hMixSession == NULL || IsMuted()) {return false;}

  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (PControlData[i]->Valid &&
        PControlData[i]->PWav == pwav &&
        PControlData[i]->WavWantsNotifyStop) // if false then wav told sndmgr just finish out
    {                                        // but it doesn't want to know about it.
      return false; // already playing.
    }
  }

  if (!CheckLoadStatus(pwav)) {return false;}

  return PlayQ.Add(new TPlayQData(pwav,loopcount));
}
//*******************************************************************
bool TAudioWaveMix::Remove(TWav *pwav)
{
  bool found = false;
  if (hMixSession == NULL) {return false;}

  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (PControlData[i]->Valid && PControlData[i]->PWav == pwav)
    {
      PControlData[i]->WavWantsNotifyStop = false;
      PControlData[i]->Stop();
      found = true;
    }
  }
  if (PlayQ.Remove(pwav,true))
  {
    found = true;
  }
  return found;
}
//*******************************************************************
int16 TAudioWaveMix::GetNumOpenChannels()
{
  if (hMixSession == NULL) {return 0;}

  int16 num = 0;
  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (!PControlData[i]->Valid) {num++;}
  }
  return num;
}
//*******************************************************************
int16 TAudioWaveMix::GetNextOpenChannel()
{
  if (hMixSession == NULL) {return -1;}

  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (!PControlData[i]->Valid) {return i;}
  }
  return -1;
}
//*******************************************************************
bool TAudioWaveMix::PrepPlayQ()
{
  bool prepped = false;
  int16 idiotcheck = NUM_AUDIO_CHANNELS - 1;
  int16 openchannel = GetNextOpenChannel();
  while (PlayQ.GetRecCount() > 0 && openchannel >= 0 && idiotcheck >= 0)
  {
    idiotcheck--;
    TPlayQData *pdata = PlayQ.Remove();
    if (pdata != NULL)
    {
      PControlData[openchannel]->Prep(pdata->PWav,pdata->LoopCount);
      prepped = true;
      delete pdata;
    }
    openchannel = GetNextOpenChannel();
  }
  return prepped;
}
//*******************************************************************
void TAudioWaveMix::SubmitPrepped()
{
  int16 hipri = -1;

  // find index of last controldata, it will be submitted as hipri.
  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    // find prepped and not submitted.
    if (PControlData[i]->Valid && !PControlData[i]->Submitted)
    {
      hipri = i;
    }
  }

  if (hipri == -1) {return;}

  int16 j;
  for (j=0;j<hipri;j++)
  {
    if (PControlData[j]->Valid && !PControlData[j]->Submitted)
    {
      PControlData[j]->Submit(WMIX_WAIT | WMIX_CLEARQUEUE);
    }
  }
  PControlData[j]->Submit(WMIX_HIPRIORITY | WMIX_CLEARQUEUE);
}
//*******************************************************************
int16 TAudioWaveMix::UnStickControlData()
//look for valid,submitted wav's that think they should be done.
{
  int16 numstuck = 0;
  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (PControlData[i]->Valid &&
        PControlData[i]->Submitted &&
        !PControlData[i]->WavWantsNotifyStop)
    {
      PControlData[i]->Stop();  // flushes channel.
      numstuck++;
    }
  }
  return numstuck;
}
//*******************************************************************
bool TAudioWaveMix::CheckControlData()
// if 6 channels full or more wav's in playq than channels avail then try
// to find some channels to flush.
{
  int16 numopen = GetNumOpenChannels();
  if (numopen < PlayQ.GetRecCount() || numopen < 2)
  {
    if (UnStickControlData() == 0)
    {
      Pump(); // if we didn't see any stuck, but we want to do something.
      return true;
    }
  }
  return false;
}
//*******************************************************************
#pragma argsused
void TAudioWaveMix::Update(dword fuc)
{
  if (!Active) {return;}
  if (hMixSession == NULL) {return;}
  dword oldtime = LastTimeCount;
  LastTimeCount = TSys::GetMSec();
  bool pumpmix = CheckControlData();

  #if defined(OGLE_DEBUG_SOUND)
  int16 numopenchannels = GetNumOpenChannels();
  int16 reccount = PlayQ.GetRecCount();
  if (numopenchannels < reccount)
  {
    char buffer[50];
    string s = "More wav's than open channels.\nNum Open Channels: ";
    itoa(numopenchannels,buffer,10);
    s += buffer;
    s += ".\nRecCount: ";
    itoa(reccount,buffer,10);
    s += buffer;
    ::MessageBox(NULL,s.c_str(),"Warning! - Audio",MB_OK);
  }
  #endif

  if (PrepPlayQ())
  {
    SubmitPrepped();
  }
  else if (!pumpmix && ((LastTimeCount - oldtime) > 333)) // 2/6 of sec.
  {
    Pump(); // we didn't find any stuck,we didn't submit any,but the last update was long ago.
  }
  #if defined(OGLE_DEBUG_SOUND)
  if (PlayQ.GetRecCount() > 0)
  {
    ::MessageBox(NULL,"PlayQ's reccount > 0 after submitted","Warning! - Audio",MB_OK | MB_TASKMODAL);
  }
  #endif
}
//*******************************************************************
void TAudioWaveMix::Configure()
{
  TOgleIniFile *pini = TOgleApp::GetOgleApp(true)->GetIniFile();

  int16 ch = int16(pini->ReadInt("WaveMix","NumChannels",0));
  if (ch < 0 || ch > 2) {ch = 0;}

  int16 sr = int16(pini->ReadInt("WaveMix","SampleRate",0));
  if (sr != 11 && sr != 22 && sr != 44) {sr = 0;}

  if (ch == 0 && sr == 0)
  {
    hMixSession = TWaveMix::Dll()->Init();
    return;
  }
  MIXCONFIG MixConfig;
  MixConfig.wSize = sizeof(MIXCONFIG);
  MixConfig.wChannels = ch;
  MixConfig.wSamplingRate = sr;

  if (ch != 0 && sr != 0)
  {
    MixConfig.dwFlags = WMIX_CONFIG_CHANNELS | WMIX_CONFIG_SAMPLINGRATE;
  }
  else if (ch != 0)
  {
    MixConfig.dwFlags = WMIX_CONFIG_CHANNELS;
  }
  else
  {
    MixConfig.dwFlags = WMIX_CONFIG_SAMPLINGRATE;
  }
  hMixSession = TWaveMix::Dll()->ConfigureInit(&MixConfig);
}
//*******************************************************************
bool TAudioWaveMix::IsPlaying()
{
  if (hMixSession == NULL) {return false;}

  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (PControlData[i]->IsPlaying()) {return true;}
  }
  return false;
}
//*******************************************************************
void TAudioWaveMix::ClearNotifyStop(TWav *pwav)
{
  // search playq for wav -> just remove w/out playing & w/out notifying pwav
  // search controldata for wav -> clear notify flag but let it continue to play to end.
  if (hMixSession == NULL) {return;}
  // ??? should we check for initialized here ???

  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (PControlData[i]->Valid && PControlData[i]->PWav == pwav)
    {
      PControlData[i]->WavWantsNotifyStop = false;
    }
  }
  PlayQ.Remove(pwav,true); // remove all instances of this wav.
}
//*******************************************************************
void TAudioWaveMix::Register(TWav *pwav)
{
  if (hMixSession == NULL || !IsInitialized()) {return;}
  if (pwav == NULL) {return;}

  TAudioObj *pao = new TAudioObj;
  pao->PAudioData = PDSArray->GetAudioData(pwav->FileMode.Name.GetStr());
  if (pao->PAudioData == NULL)
  {
    pao->PAudioData = new TAudioData(this,pwav->FileMode.Name.GetStr());
    PDSArray->Add(pwav->FileMode.Name.GetStr(),pao->PAudioData);
  }
  pao->PAudioData->IncOwnerCount();
  pao->PAudioData->IncCount(TFileMode::fm_None);
  pao->PAudioData->SetStatus();
  pwav->PSndMgrData = pao;
}
//*******************************************************************
void TAudioWaveMix::UnRegister(TWav *pwav)
{
  if (pwav == NULL) {return;}

  // first check if submitted
  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (PControlData[i]->Valid && PControlData[i]->PWav == pwav)
    {
      PControlData[i]->WavWantsNotifyStop = false;
      PControlData[i]->Stop();
    }
  }
  // check if in play queue.
  while (PlayQ.Remove(pwav)) {} // make sure not in play queue

  // now do cleanup of wav's sndmgr data.
  TAudioObj *pao = (TAudioObj *)pwav->PSndMgrData;
  if (pao == NULL) {return;}
  if (pao->PAudioData != NULL)
  {
    pao->PAudioData->DecOwnerCount();
    pao->PAudioData->DecCount(pao->Status);
    pao->PAudioData->SetStatus();
  }
  delete pao;
  pwav->PSndMgrData = NULL;
}
//*******************************************************************
void TAudioWaveMix::ChangeStatus(TWav *pwav,TFMStatus status)
{
  if (pwav == NULL) {return;}
  TAudioObj *pao = (TAudioObj *)pwav->PSndMgrData;
  if (pao == NULL || pao->PAudioData == NULL) {return;}
  if (pao->Status != status)
  {
    pao->PAudioData->DecCount(pao->Status);
    pao->PAudioData->IncCount(status);
    pao->PAudioData->SetStatus();
    pao->Status = status;
  }
  if (pao->PAudioData->Status == TFileMode::fm_Immed &&
      pao->PAudioData->PMixWave == NULL)
  {
    TFMStatus status = pwav->FileMode.GetRamStatus();
    pwav->LoadRamImmed();
    pao->PAudioData->Load(pwav);
    pwav->LoadRam(status);
  }
}
//*******************************************************************
bool TAudioWaveMix::IsPlaying(LPMIXWAVE pmixwave)
{
  for (int16 i=0;i<NUM_AUDIO_CHANNELS;i++)
  {
    if (PControlData[i]->IsPlaying(pmixwave))
    {
      return true;
    }
  }
  return PlayQ.IsPlaying(pmixwave);
}
//*******************************************************************
void TAudioWaveMix::ModeDeActivated()
{
  PDSArray->DestroyNoOwners();
  #if defined(OGLE_DEBUG_SOUND)
  if (PlayQ.GetRecCount() > 0)
  {
    ::MessageBox(NULL,"playq still has entries","Warning! - Audio",MB_OK);
  }
  #endif
}
//*******************************************************************

