//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oresplst.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/12/95    M.Perry    original baseline
//  2.0    08/28/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__oresplst_h)
#include "oresplst.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif
//*******************************************************************
TResponderList::TResponderList(TOgleModeWindow *pownerwindow)
:TSliq(false,false)
{
  POwnerWindow = pownerwindow;
  PResponderPriority = NULL;
  PAreaResponder = NULL;
  PApp = TOgleApp::GetOgleApp();
}
//*******************************************************************
TResponderList::~TResponderList()
{
  PResponderPriority = NULL;
}
//*******************************************************************
bool TResponderList::SetPriority(TResponder *presponder)
{
  if (PResponderPriority != NULL) {return false;}
  POwnerWindow->SetCapture();
  PResponderPriority = presponder;
  return true;
}
//*******************************************************************
void TResponderList::ClearPriority(TResponder *presponder)
{
  if (PResponderPriority == presponder)
  {
    PResponderPriority = NULL;
    POwnerWindow->ReleaseCapture();
  }
}
//*******************************************************************
TCompareResult ResponderCompareHiToLoZ(void *FirstObj,void *SecObj)
{
  if ((FirstObj == NULL) || (SecObj == NULL)) {return EQUAL_TO;}

  int16 First = ((TResponder *)FirstObj)->GetZ();
  int16 Second = ((TResponder *)SecObj)->GetZ();

  if (First < Second) {return GREATER_THAN;}
  else if (First > Second) {return LESS_THAN;}
  return EQUAL_TO;
}
//*******************************************************************
void TResponderList::SortByZOrder()
{
  TCompareFunct OldCompareFunct = SetCompareFunct(ResponderCompareHiToLoZ);
  SliqSort();
  SetCompareFunct(OldCompareFunct);
  ClearZChanged();
}
//*******************************************************************
void TResponderList::FindKeyHandler(uint key,uint repeatCount,uint flags)
// note: we already know revent is reChar, so we don't need to pass to function.
{
  #if defined(DEBUGUPDATES)
  POwnerWindow->DebugUpdate.StartKey();
  #endif

  if (PResponderPriority) // mouse event in progress.
  {
    #if defined(DEBUGUPDATES)
    POwnerWindow->DebugUpdate.EndKey();
    #endif

    return;
  }

  if (ZChanged) {SortByZOrder();}

  for(TResponder *PResponder = GetFirst();
      PResponder != NULL;
      PResponder = GetNext())
  {
    if (PResponder->IsChar(key))
    {
      // if handled then break loop.
      if (PResponder->HandleKeyEvent(reChar,repeatCount,flags))
      {
        #if defined(DEBUGUPDATES)
        POwnerWindow->DebugUpdate.EndKey();
        #endif

        return;
      }
    }
  }

  #if defined(DEBUGUPDATES)
  POwnerWindow->DebugUpdate.EndKey();
  #endif
}
//*******************************************************************
void TResponderList::SendToEnterAreaResponder(int msg)
{
  if (PAreaResponder == NULL) {return;}
  TPoint point;
  POwnerWindow->GetCursorPos(point);
  POwnerWindow->ScreenToClient(point);
  PAreaResponder->HandleMouseEvent((TResponseEvent)msg,0,point);
}
//*******************************************************************
void TResponderList::FindAreaResponder(TPoint &point)
// find first responder that handles enterleaves && contains point.
{
  #if defined(DEBUGUPDATES)
  POwnerWindow->DebugUpdate.StartArea();
  #endif

  // send leaving msg.
  if (PAreaResponder != NULL && !PAreaResponder->Contains(point))
  {
    PAreaResponder->HandleMouseEvent(reLeaveArea,0,point);
    PAreaResponder = NULL;
  }

  // send entering msg if found.
  for (TResponder *PResponder = GetFirst();
       PResponder != NULL;
       PResponder = GetNext())
  {
    if (PResponder->IsEnterLeave() && PResponder->Contains(point))
    {
      if (PAreaResponder == PResponder)
      {
        #if defined(DEBUGUPDATES)
        POwnerWindow->DebugUpdate.EndArea();
        #endif

        return;
      }
      // handle situation where new responder is above old area responder.
      if (PAreaResponder != NULL) {PAreaResponder->HandleMouseEvent(reLeaveArea,0,point);}
      PResponder->HandleMouseEvent(reEnterArea,0,point);
      PAreaResponder = PResponder;

      #if defined(DEBUGUPDATES)
      POwnerWindow->DebugUpdate.EndArea();
      #endif

      return;
    }
  }

  #if defined(DEBUGUPDATES)
  POwnerWindow->DebugUpdate.EndArea();
  #endif
}
//*******************************************************************
void TResponderList::FindMouseHandler(TResponseEvent revent,uint modkeys,TPoint &point)
{
  #if defined(DEBUGUPDATES)
  POwnerWindow->DebugUpdate.StartMouse();
  #endif

  // check sorted.
  if (ZChanged) {SortByZOrder();}

  // make entering / leaving calls.
  if (revent == reSetCursor) {FindAreaResponder(point);}

  // check responder with priority.
  if (PResponderPriority)
  {
    if (PResponderPriority->HandleMouseEvent(revent,modkeys,point))
    {
      #if defined(DEBUGUPDATES)
      POwnerWindow->DebugUpdate.EndMouse();
      #endif
      return;
    }
  }

  for(TResponder *PResponder = GetFirst();
      PResponder != NULL;
      PResponder = GetNext())
  {
    if (PResponder->Contains(point))
    {
      // if handled then break loop.
      if (PResponder->HandleMouseEvent(revent,modkeys,point))
      {
        #if defined(DEBUGUPDATES)
        POwnerWindow->DebugUpdate.EndMouse();
        #endif
        return;
      }
    }
  }
  // if nobody handled setcursor then clear area cursor.
  if (revent == reSetCursor) {PApp->GetCursorMgr()->SetCursor(ctArea,0,0);}
  #if defined(DEBUGUPDATES)
  POwnerWindow->DebugUpdate.EndMouse();
  #endif
}
//*******************************************************************

