//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  gedctrl.cpp
//    Project Name:  GEd
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     6/12/96    M.Perry    original baseline
//  2.0    10/28/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__gedctrl_h)
#include "gedctrl.h"
#endif
//*******************************************************************
bool TGedControl::GetLockStatus()
{
  Flush();
  return ReadInt(INFOSECTION,FILELOCKENTRY,boolToint(false));
}
//*******************************************************************
bool TGedControl::WriteLockStatus(bool lock)
{
  bool result = WriteInt(INFOSECTION,FILELOCKENTRY,boolToint(lock));
  if (result) {Flush();}
  return result;
}
//*******************************************************************
TGedControl::TGedControl(string &hddrive,string &hddir,string fname,string &username)
:TIniFile(hddrive,hddir,fname + GEDCTRLEXT),UserName(username)
{
  Master = false;
  LockedByMe = false;
  Register();
}
//*******************************************************************
TGedControl::~TGedControl()
{
  UnRegister();
}
//*******************************************************************
bool TGedControl::WriteLockName()
{
  return WriteString(INFOSECTION,FILELOCKNAMEENTRY,UserName);
}
//*******************************************************************
bool TGedControl::ClearLockName()
{
  return WriteString(INFOSECTION,FILELOCKNAMEENTRY,NONE_STR);
}
//*******************************************************************
string TGedControl::GetLockName()
{
  Flush();
  return ReadString(INFOSECTION,FILELOCKNAMEENTRY,NONE_STR);
}
//*******************************************************************
int TGedControl::GetUserCount()
{
  Flush();
  return ReadInt(INFOSECTION,USERCOUNTENTRY,0);
}
//*******************************************************************
bool TGedControl::WriteUserCount(int count)
{
  bool result = WriteInt(INFOSECTION,USERCOUNTENTRY,count);
  Flush();
  return result;
}
//*******************************************************************
bool TGedControl::IncUserCount()
{
  if (!WaitForLock()) {return false;}
  int uc = GetUserCount();
  uc++;
  bool result = WriteUserCount(uc);
  ReleaseLock();
  return result;
}
//*******************************************************************
bool TGedControl::DecUserCount()
{
  if (!WaitForLock()) {return false;}
  int uc = GetUserCount();
  uc--;
  if (uc < 0) {uc = 0;}
  bool result = WriteUserCount(uc);
  ReleaseLock();
  return result;
}
//*******************************************************************
bool TGedControl::SetMasterName()
{
  if (!WaitForLock()) {return false;}
  bool result = WriteString(INFOSECTION,MASTERENTRY,UserName);
  ReleaseLock();
  return result;
}
//*******************************************************************
string TGedControl::GetMasterName()
{
  if (IsMaster()) {return UserName;}
  return ReadString(INFOSECTION,MASTERENTRY,NONE_STR);
}
//*******************************************************************
bool TGedControl::ClearMasterName()
{
  if (!WaitForLock()) {return false;}
  bool result = WriteString(INFOSECTION,MASTERENTRY,NONE_STR);
  ReleaseLock();
  return result;
}
//*******************************************************************
// mrp - v2.4 - 08/19/99 - modified to let graceful restart after crash.
bool TGedControl::IsModeLocked(string &modename)
{
  if (!WaitForLock()) {return false;}
  string modeusername = GetModeLockUserName(modename);//ReadString(LOCKMODESECTION,modename,NONE_STR);
  bool result = bool(modeusername != string(NONE_STR));
  if (UserName == modeusername) {result = false;}
  ReleaseLock();
  return result;
}
//*******************************************************************
bool TGedControl::SetModeLock(string &modename)
{
  if (!WaitForLock()) {return false;}
  bool result = WriteString(LOCKMODESECTION,modename,UserName);
  if (result) {ModeName = modename;}
  ReleaseLock();
  return result;
}
//*******************************************************************
bool TGedControl::ClearModeLock(string &modename)
{
  if (modename == "") {return true;}
  if (!WaitForLock()) {return false;}
  bool result = WriteString(LOCKMODESECTION,modename,NONE_STR);
  if (result) {ModeName = "";}
  ReleaseLock();
  return result;
}
//*******************************************************************
bool TGedControl::ClearModeLock()
{
  return ClearModeLock(ModeName);
}
//*******************************************************************
string TGedControl::GetModeLockUserName(string &modename)
{
//  if (!WaitForLock()) {return false;}
//  string s =
  return ReadString(LOCKMODESECTION,modename,NONE_STR);
//  ReleaseLock();
//  return s;
}
//*******************************************************************
bool TGedControl::SetLock(bool lock)
{
  if (LockedByMe)
  {
    if (lock) {return true;}
    ClearLockName();
    if (!WriteLockStatus(false)) {return false;}
    LockedByMe = false;
    return true;
  }
  else // !LockedByMe
  {
    if (!lock) {return true;}
    if (GetLockStatus()) {return false;}
    if (!WriteLockStatus(true)) {return false;}
    WriteLockName();
    LockedByMe = true;
    return true;
  }
}
//*******************************************************************
void TGedControl::Register()
{
  bool restarting = false;
  if (WaitForLock())
  {
    if (GetUserCount() == 0)
    {
      Master = true;
      SetMasterName();
    }
    // mrp - v2.4 - 08/19/99 - modified to add graceful restart after crash.
    else
    {
      string m = GetMasterName();
      if (m == UserName)
      {
        int result = ::MessageBox(NULL,
  "User name is same as current Master User Name.\nDid Previous execution do a fatal exit?",
                                  "Query - GEd Control",
                                  MB_YESNO | MB_TASKMODAL);
        if (result == IDYES)
        {
          Master = true;
          SetMasterName();
          restarting = true;
        }
      }
    }
    ReleaseLock();
  }
  if (!restarting)
  {
    IncUserCount();
  }
  Registered = true;
}
//*******************************************************************
void TGedControl::UnRegister()
{
  if (Registered)
  {
    if (ModeName != "") {ClearModeLock(ModeName);}
    if (IsMaster()) {ClearMasterName();Master = false;}
    DecUserCount();
    Registered = false;
  }
}
//*******************************************************************
bool TGedControl::WaitForLock()
{
  if (LockedByMe) {return true;}
  while (true)
  {
    if (SetLock(true)) {return true;}
    Flush();
    string lockname = GetLockName();
    string msg = "Locked by: " + lockname;
    msg += "\nTry Again ?";
    if (::MessageBox(NULL,msg.c_str(),"Waiting For Lock",MB_YESNO | MB_TASKMODAL) == IDNO)
    {
      return false;
    }
  }
}
//*******************************************************************

