//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  csglord.h
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     10/11/95   M.Perry     original baseline
//  2.0     09/25/97   M.Perry     32-bit conversion.
//  2.3    07/14/99    M.Perry    gobjlist removal
//********************************************************************
#if !defined(__csglord_h)
#define __csglord_h

#include "casglord.h"
#include "csnames.rh"
#include "casbutsp.h"  // Button
#include "cascdsp.h"    //Cash Display
#include "csposp.h"  // payoff in coins

class TSlotsModeWin;
class TSLordMainState;

#include "cscpsp.h"
#include "cgametyp.h"
#include "cswpsp.h"
#include "cscrsp.h"
#include "csjpsp.h"
#include "caschsp.h"
#include "cnamesp.h"
#include "csreelsp.h"
#include "cssdsp.h"
//*******************************************************************
enum TGLordRID
{
  idButton0,
  idButton1,
  idButton2,
  idButton3,
  idButton4,
  idButton5,
  idButton6,
  idButton7,

  idAddCoins,
  idOneChip,
  idFiveChip,
  idTwentyFiveChip,
  idOneHundredChip,
  idOneThouChip,
  idTenThouChip,

  idSpeedClick,

  idNamePlate,
  idStatBar,
  idProgressive
};
//*******************************************************************
class TCasinoSGameLord:public TCasinoGameLord
{
public:
  enum {MAXBET = 3,NUMOFBUTTONS = 8,NumResponders = 9};

  TSlotsModeWin  *pMWin;
  TSSound        *Sounds;
  TSCoinsPlayed  *CoinsPlayed;
  TSWinPaid      *WinPaid;
  TSCredits      *Credits;
  TCasChips      *Chips;
  TSReel         *Reel;
  TCasNamePlate  *NamePlate1;
  TCasNamePlate  *NamePlate2;
  TCasNamePlate  *NamePlate3;
  TSProgressive  *Progressive;
  TCasButton     *Button0;     //ask
  TCasButton     *Button1;     //stat
  TCasButton     *Button2;     //menu
  TCasButton     *Button3;     //help
  TCasButton     *Button4;     //CashOut
  TCasButton     *Button5;     //Bet1
  TCasButton     *Button6;     //BetMax
  TCasButton     *Button7;     //spin
  TCasCashDisplay *BankCashDisplay;
  TSCoinPay      *CoinPay;
  TResponder *Responders[NumResponders];
  TMoney credits;
  TMoney coinsplayed;
  TMoney coinsize;
  TMoney winnerpaid;
  TMoney maxbet;
  TMoney pot;
  TMoney progressive;
  int16 playersin;
  int16 r1;
  int16 r2;
  int16 r3;
  int16 Song;
  int16 shutdownstate;
  int16 machinenum;
  int16 PayOffSound;
  int16 ButtonInfo;
  bool  ButtonHit[NUMOFBUTTONS];


public:
  TCasinoSGameLord(TSlotsModeWin *PNewOwnerWindow,const TFileMode &fm,
						  int16 subgametype,int16 coindenom);
  virtual ~TCasinoSGameLord();
  virtual TSprite *CreateSprite(string spritename,int16 id = 0,void *psource = NULL);

  virtual bool AddCredits(int16 AddCredits);
  void EnableAllButtons();
  void FreezeAllButtons();
  void UnFreezeAllButtons();
  void DisEnableAllButtons();
  void EnableButton(int16 button);
  void DisEnableButton(int16 button);
  void ButtonClear(int16 button);
  virtual bool Respond(int16 Button,int16 info = 0,void *pdata = NULL);
  virtual bool Respond(TResponder &responder);
  void SetupResponders();
  void CleanupResponders();

  int16 ShowWinner();
  int16 GetBet();
  int16 Bet1();
  TMoney BetX(TMoney x);
  int16 BetMax();
  int16 CashOut();
  void Spin();
  virtual void SetReel();
  int16 Switch();
  int16 AddCoins();
  int16 GoodToGo();
  TMoney Winner();
  int16 Clear();
  int16 InitS();
  void ClearActions();
  void AskExpert();
  int16 PayOutProgressive();
  virtual TMoney PayOut();
  int16 GetPlayin() {return playersin;};
  int16 ChangeGame();
  void  SwitchToPlayer(int16 player);
  void  CyclePlayers();
  virtual void SetupActivePlayer(int16 slot);
  virtual void ShuttingDown();
  virtual void NotifyPlayerActivation(int16 slotnum);
  virtual void NotifyPlayerDeActivation(int16 slotnum);
  virtual void SetupSprites();
  virtual void CleanupSprites();
  #if defined(SUPPORT_OLD_VER)
//  virtual void SetupUniqueIDStrings();
//  virtual void SetUIDS();
  virtual void SetUIDSStrings();
  virtual int16 GetUniqueID(const string &s);
  string strButton0,strButton1,strButton2,strButton3,strButton4,
         strButton5,strButton6,strButton7,strBankCashDisplay,
         strChips,strCoinPay,strCredits,strNP1,strNP2,strNP3,
         strReel,strSound,strWinPaid,strProgressive,strCoinsPlayed,
         strSong;

  int16 idsButton0,idsButton1,idsButton2,idsButton3,idsButton4,
        idsButton5,idsButton6,idsButton7,idsBankCashDisplay,
        idsChips,idsCoinPay,idsCredits,idsNP1,idsNP2,
        idsNP3,idsReel,idsSound,idsWinPaid,idsProgressive,
        idsCoinsPlayed,idsSong;
  #endif
};
//*******************************************************************
#endif