//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ckcalc.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/6/96    M.Perry      derived from Dave's code.
//  2.0    09/17/97   M.Perry      32-bit conversion.
//********************************************************************
#include "ckcalc.h"         
#include "cstatdlg.h"
#include <stdio.h>
#include <classlib\time.h>

//*******************************************************************
TStatCalcKeno::TStatCalcKeno(TCasinoStatDlg *pstatdlg)
:TCasinoStatCalc(pstatdlg)
{
}
//*******************************************************************
TStatCalcKeno::~TStatCalcKeno()
{
}
//*******************************************************************
void TStatCalcKeno::GetSessionPtrs()
{
  if (PPlayerData == NULL) {return;}
  PData = &(PPlayerData->KData);

  LastSessionNum = -1;
  for (int sessioniterator = GetKData()->GetFirstValidSession();
       sessioniterator != -1;
       sessioniterator = GetKData()->GetNextValidSession())
  {
    LastSessionNum++;
    PSessions[LastSessionNum] = &(GetKData()->Sessions[sessioniterator]);
  }
}
//*******************************************************************
bool TStatCalcKeno::GetValues(int statindex,float *values,int maxvalues)
{
  int num = GetMaxValueIndex(maxvalues);
  switch (statindex)
  {
    case kTotalNumBets         :return GetTotalNumBets(values,num);
    case kTotalCashBets        :return GetTotalCashBets(values,num);
    case kAvgCashBet           :return GetAvgCashBets(values,num);
    case kTotalCashWL          :return GetTotalCashWL(values,num);
    case kCumCashWL            :return GetTotalCashWL(values,num);
    case kBetsWon              :return GetWins(values,num);
    case kBetsLost             :return GetLosses(values,num);
    case kPercBetsWon          :return GetPercWins(values,num);
    case kPercBetsLost         :return GetPercLosses(values,num);
    case kLongestWinStreak     :return GetLongestWinStreaks(values,num);
    case kLongestLosingStreak  :return GetLongestLosingStreak(values,num);
    case kLargestCashWin       :return GetLargestCashWin(values,num);
    case kLargestCashLoss      :return GetLargestCashLost(values,num);
    case kPeakCashWinLevel     :return GetPeakWinLevel(values,num);
    case kLowCashLossLevel     :return GetLowCashLostLevel(values,num);
    default                    :return false;
  };
//  return false;
}
//*******************************************************************
#pragma argsused
string TStatCalcKeno::MakeSubGameTypeStr(int subgametype,TMoney gamedenom)
{
  string s = "Keno";
  return s;
}
//*******************************************************************
void TStatCalcKeno::Make_CumulativeCashWLStrs()
{
  // use TotalCasWL
}
//*******************************************************************
void TStatCalcKeno::Calc_CumulativeCashWL()
{
  // same as TotalCashWL
}
//*******************************************************************
void TStatCalcKeno::Make_Num1SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num1SpotBetsStr[i] = MakeString(GetKSession(i)->nNum1SpotBets);
  }
  Total_Num1SpotBetsStr = MakeString(Total_Num1SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num1SpotBets()
{
  Total_Num1SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num1SpotBets += GetKSession(i)->nNum1SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num2SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num2SpotBetsStr[i] = MakeString(GetKSession(i)->nNum2SpotBets);
  }
  Total_Num2SpotBetsStr = MakeString(Total_Num2SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num2SpotBets()
{
  Total_Num2SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num2SpotBets += GetKSession(i)->nNum2SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num3SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num3SpotBetsStr[i] = MakeString(GetKSession(i)->nNum3SpotBets);
  }
  Total_Num3SpotBetsStr = MakeString(Total_Num3SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num3SpotBets()
{
  Total_Num3SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num3SpotBets += GetKSession(i)->nNum3SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num4SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num4SpotBetsStr[i] = MakeString(GetKSession(i)->nNum4SpotBets);
  }
  Total_Num4SpotBetsStr = MakeString(Total_Num4SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num4SpotBets()
{
  Total_Num4SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num4SpotBets += GetKSession(i)->nNum4SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num5SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num5SpotBetsStr[i] = MakeString(GetKSession(i)->nNum5SpotBets);
  }
  Total_Num5SpotBetsStr = MakeString(Total_Num5SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num5SpotBets()
{
  Total_Num5SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num5SpotBets += GetKSession(i)->nNum5SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num6SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num6SpotBetsStr[i] = MakeString(GetKSession(i)->nNum6SpotBets);
  }
  Total_Num6SpotBetsStr = MakeString(Total_Num6SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num6SpotBets()
{
  Total_Num6SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num6SpotBets += GetKSession(i)->nNum6SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num7SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num7SpotBetsStr[i] = MakeString(GetKSession(i)->nNum7SpotBets);
  }
  Total_Num7SpotBetsStr = MakeString(Total_Num7SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num7SpotBets()
{
  Total_Num7SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num7SpotBets += GetKSession(i)->nNum7SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num8SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num8SpotBetsStr[i] = MakeString(GetKSession(i)->nNum8SpotBets);
  }
  Total_Num8SpotBetsStr = MakeString(Total_Num8SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num8SpotBets()
{
  Total_Num8SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num8SpotBets += GetKSession(i)->nNum8SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num9SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num9SpotBetsStr[i] = MakeString(GetKSession(i)->nNum9SpotBets);
  }
  Total_Num9SpotBetsStr = MakeString(Total_Num9SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num9SpotBets()
{
  Total_Num9SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num9SpotBets += GetKSession(i)->nNum9SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num10SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num10SpotBetsStr[i] = MakeString(GetKSession(i)->nNum10SpotBets);
  }
  Total_Num10SpotBetsStr = MakeString(Total_Num10SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num10SpotBets()
{
  Total_Num10SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num10SpotBets += GetKSession(i)->nNum10SpotBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num3SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num3SpotSpecBetsStr[i] = MakeString(GetKSession(i)->nNum3SpotSpecialBets);
  }
  Total_Num3SpotSpecBetsStr = MakeString(Total_Num3SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num3SpotSpecBets()
{
  Total_Num3SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num3SpotSpecBets += GetKSession(i)->nNum3SpotSpecialBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num4SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num4SpotSpecBetsStr[i] = MakeString(GetKSession(i)->nNum4SpotSpecialBets);
  }
  Total_Num4SpotSpecBetsStr = MakeString(Total_Num4SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num4SpotSpecBets()
{
  Total_Num4SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num4SpotSpecBets += GetKSession(i)->nNum4SpotSpecialBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_Num6SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    Num6SpotSpecBetsStr[i] = MakeString(GetKSession(i)->nNum6SpotSpecialBets);
  }
  Total_Num6SpotSpecBetsStr = MakeString(Total_Num6SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_Num6SpotSpecBets()
{
  Total_Num6SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_Num6SpotSpecBets += GetKSession(i)->nNum6SpotSpecialBets;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL1SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL1SpotBetsStr[i] = MakeString(GetKSession(i)->l1SpotBetsCashWL);
  }
  Total_TotalCashWL1SpotBetsStr = MakeString(Total_TotalCashWL1SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL1SpotBets()
{
  Total_TotalCashWL1SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL1SpotBets += GetKSession(i)->l1SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL2SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL2SpotBetsStr[i] = MakeString(GetKSession(i)->l2SpotBetsCashWL);
  }
  Total_TotalCashWL2SpotBetsStr = MakeString(Total_TotalCashWL2SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL2SpotBets()
{
  Total_TotalCashWL2SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL2SpotBets += GetKSession(i)->l2SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL3SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL3SpotBetsStr[i] = MakeString(GetKSession(i)->l3SpotBetsCashWL);
  }
  Total_TotalCashWL3SpotBetsStr = MakeString(Total_TotalCashWL3SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL3SpotBets()
{
  Total_TotalCashWL3SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL3SpotBets += GetKSession(i)->l3SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL4SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL4SpotBetsStr[i] = MakeString(GetKSession(i)->l4SpotBetsCashWL);
  }
  Total_TotalCashWL4SpotBetsStr = MakeString(Total_TotalCashWL4SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL4SpotBets()
{
  Total_TotalCashWL4SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL4SpotBets += GetKSession(i)->l4SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL5SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL5SpotBetsStr[i] = MakeString(GetKSession(i)->l5SpotBetsCashWL);
  }
  Total_TotalCashWL5SpotBetsStr = MakeString(Total_TotalCashWL5SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL5SpotBets()
{
  Total_TotalCashWL5SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL5SpotBets += GetKSession(i)->l5SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL6SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL6SpotBetsStr[i] = MakeString(GetKSession(i)->l6SpotBetsCashWL);
  }
  Total_TotalCashWL6SpotBetsStr = MakeString(Total_TotalCashWL6SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL6SpotBets()
{
  Total_TotalCashWL6SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL6SpotBets += GetKSession(i)->l6SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL7SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL7SpotBetsStr[i] = MakeString(GetKSession(i)->l7SpotBetsCashWL);
  }
  Total_TotalCashWL7SpotBetsStr = MakeString(Total_TotalCashWL7SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL7SpotBets()
{
  Total_TotalCashWL7SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL7SpotBets += GetKSession(i)->l7SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL8SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL8SpotBetsStr[i] = MakeString(GetKSession(i)->l8SpotBetsCashWL);
  }
  Total_TotalCashWL8SpotBetsStr = MakeString(Total_TotalCashWL8SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL8SpotBets()
{
  Total_TotalCashWL8SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL8SpotBets += GetKSession(i)->l8SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL9SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL9SpotBetsStr[i] = MakeString(GetKSession(i)->l9SpotBetsCashWL);
  }
  Total_TotalCashWL9SpotBetsStr = MakeString(Total_TotalCashWL9SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL9SpotBets()
{
  Total_TotalCashWL9SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL9SpotBets += GetKSession(i)->l9SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL10SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL10SpotBetsStr[i] = MakeString(GetKSession(i)->l10SpotBetsCashWL);
  }
  Total_TotalCashWL10SpotBetsStr = MakeString(Total_TotalCashWL10SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL10SpotBets()
{
  Total_TotalCashWL10SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL10SpotBets += GetKSession(i)->l10SpotBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL3SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL3SpotSpecBetsStr[i] = MakeString(GetKSession(i)->l3SpotSpecialBetsCashWL);
  }
  Total_TotalCashWL3SpotSpecBetsStr = MakeString(Total_TotalCashWL3SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL3SpotSpecBets()
{
  Total_TotalCashWL3SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL3SpotSpecBets += GetKSession(i)->l3SpotSpecialBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL4SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL4SpotSpecBetsStr[i] = MakeString(GetKSession(i)->l4SpotSpecialBetsCashWL);
  }
  Total_TotalCashWL4SpotSpecBetsStr = MakeString(Total_TotalCashWL4SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL4SpotSpecBets()
{
  Total_TotalCashWL4SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL4SpotSpecBets += GetKSession(i)->l4SpotSpecialBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_TotalCashWL6SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalCashWL6SpotSpecBetsStr[i] = MakeString(GetKSession(i)->l6SpotSpecialBetsCashWL);
  }
  Total_TotalCashWL6SpotSpecBetsStr = MakeString(Total_TotalCashWL6SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCashWL6SpotSpecBets()
{
  Total_TotalCashWL6SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashWL6SpotSpecBets += GetKSession(i)->l6SpotSpecialBetsCashWL;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash1SpotBets()
{
  Total_TotalCash1SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash1SpotBets += GetKSession(i)->l1SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash2SpotBets()
{
  Total_TotalCash2SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash2SpotBets += GetKSession(i)->l2SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash3SpotBets()
{
  Total_TotalCash3SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash3SpotBets += GetKSession(i)->l3SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash4SpotBets()
{
  Total_TotalCash4SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash4SpotBets += GetKSession(i)->l4SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash5SpotBets()
{
  Total_TotalCash5SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash5SpotBets += GetKSession(i)->l5SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash6SpotBets()
{
  Total_TotalCash6SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash6SpotBets += GetKSession(i)->l6SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash7SpotBets()
{
  Total_TotalCash7SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash7SpotBets += GetKSession(i)->l7SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash8SpotBets()
{
  Total_TotalCash8SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash8SpotBets += GetKSession(i)->l8SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash9SpotBets()
{
  Total_TotalCash9SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash9SpotBets += GetKSession(i)->l9SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash10SpotBets()
{
  Total_TotalCash10SpotBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash10SpotBets += GetKSession(i)->l10SpotBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash3SpotSpecBets()
{
  Total_TotalCash3SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash3SpotSpecBets += GetKSession(i)->l3SpotSpecialBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash4SpotSpecBets()
{
  Total_TotalCash4SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash4SpotSpecBets += GetKSession(i)->l4SpotSpecialBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalCash6SpotSpecBets()
{
  Total_TotalCash6SpotSpecBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCash6SpotSpecBets += GetKSession(i)->l6SpotSpecialBetsCash;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet1SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet1SpotBetsStr[i] = MakeString(PercRet1SpotBets[i]);
  }
  Total_PercRet1SpotBetsStr = MakeString(Total_PercRet1SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet1SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet1SpotBets[i] = GetPercRet(GetKSession(i)->l1SpotBetsCashWL,
                                     GetKSession(i)->l1SpotBetsCash);
  }
  Total_PercRet1SpotBets = GetPercRet(Total_TotalCashWL1SpotBets,Total_TotalCash1SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet2SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet2SpotBetsStr[i] = MakeString(PercRet2SpotBets[i]);
  }
  Total_PercRet2SpotBetsStr = MakeString(Total_PercRet2SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet2SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet2SpotBets[i] = GetPercRet(GetKSession(i)->l2SpotBetsCashWL,
                                     GetKSession(i)->l2SpotBetsCash);
  }
  Total_PercRet2SpotBets = GetPercRet(Total_TotalCashWL2SpotBets,Total_TotalCash2SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet3SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet3SpotBetsStr[i] = MakeString(PercRet3SpotBets[i]);
  }
  Total_PercRet3SpotBetsStr = MakeString(Total_PercRet3SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet3SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet3SpotBets[i] = GetPercRet(GetKSession(i)->l3SpotBetsCashWL,
                                     GetKSession(i)->l3SpotBetsCash);
  }
  Total_PercRet3SpotBets = GetPercRet(Total_TotalCashWL3SpotBets,Total_TotalCash3SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet4SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet4SpotBetsStr[i] = MakeString(PercRet4SpotBets[i]);
  }
  Total_PercRet4SpotBetsStr = MakeString(Total_PercRet4SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet4SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet4SpotBets[i] = GetPercRet(GetKSession(i)->l4SpotBetsCashWL,
                                     GetKSession(i)->l4SpotBetsCash);
  }
  Total_PercRet4SpotBets = GetPercRet(Total_TotalCashWL4SpotBets,Total_TotalCash4SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet5SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet5SpotBetsStr[i] = MakeString(PercRet5SpotBets[i]);
  }
  Total_PercRet5SpotBetsStr = MakeString(Total_PercRet5SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet5SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet5SpotBets[i] = GetPercRet(GetKSession(i)->l5SpotBetsCashWL,
                                     GetKSession(i)->l5SpotBetsCash);
  }
  Total_PercRet5SpotBets = GetPercRet(Total_TotalCashWL5SpotBets,Total_TotalCash5SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet6SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet6SpotBetsStr[i] = MakeString(PercRet6SpotBets[i]);
  }
  Total_PercRet6SpotBetsStr = MakeString(Total_PercRet6SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet6SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet6SpotBets[i] = GetPercRet(GetKSession(i)->l6SpotBetsCashWL,
                                     GetKSession(i)->l6SpotBetsCash);

  }
  Total_PercRet6SpotBets = GetPercRet(Total_TotalCashWL6SpotBets,Total_TotalCash6SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet7SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet7SpotBetsStr[i] = MakeString(PercRet7SpotBets[i]);
  }
  Total_PercRet7SpotBetsStr = MakeString(Total_PercRet7SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet7SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet7SpotBets[i] = GetPercRet(GetKSession(i)->l7SpotBetsCashWL,
                                     GetKSession(i)->l7SpotBetsCash);
  }
  Total_PercRet7SpotBets = GetPercRet(Total_TotalCashWL7SpotBets,Total_TotalCash7SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet8SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet8SpotBetsStr[i] = MakeString(PercRet8SpotBets[i]);
  }
  Total_PercRet8SpotBetsStr = MakeString(Total_PercRet8SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet8SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet8SpotBets[i] = GetPercRet(GetKSession(i)->l8SpotBetsCashWL,
                                     GetKSession(i)->l8SpotBetsCash);
  }
  Total_PercRet8SpotBets = GetPercRet(Total_TotalCashWL8SpotBets,Total_TotalCash8SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet9SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet9SpotBetsStr[i] = MakeString(PercRet9SpotBets[i]);
  }
  Total_PercRet9SpotBetsStr = MakeString(Total_PercRet9SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet9SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet9SpotBets[i] = GetPercRet(GetKSession(i)->l9SpotBetsCashWL,
                                     GetKSession(i)->l9SpotBetsCash);
  }
  Total_PercRet9SpotBets = GetPercRet(Total_TotalCashWL9SpotBets,Total_TotalCash9SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet10SpotBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet10SpotBetsStr[i] = MakeString(PercRet10SpotBets[i]);
  }
  Total_PercRet10SpotBetsStr = MakeString(Total_PercRet10SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet10SpotBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet10SpotBets[i] = GetPercRet(GetKSession(i)->l10SpotBetsCashWL,
                                      GetKSession(i)->l10SpotBetsCash);
  }
  Total_PercRet10SpotBets = GetPercRet(Total_TotalCashWL10SpotBets,Total_TotalCash10SpotBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet3SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet3SpotSpecBetsStr[i] = MakeString(PercRet3SpotSpecBets[i]);
  }
  Total_PercRet3SpotSpecBetsStr = MakeString(Total_PercRet3SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet3SpotSpecBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet3SpotSpecBets[i] = GetPercRet(GetKSession(i)->l3SpotSpecialBetsCashWL,
                                         GetKSession(i)->l3SpotSpecialBetsCash);
  }
  Total_PercRet3SpotSpecBets = GetPercRet(Total_TotalCashWL3SpotSpecBets,
                                          Total_TotalCash3SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet4SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet4SpotSpecBetsStr[i] = MakeString(PercRet4SpotSpecBets[i]);
  }
  Total_PercRet4SpotSpecBetsStr = MakeString(Total_PercRet4SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet4SpotSpecBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet4SpotSpecBets[i] = GetPercRet(GetKSession(i)->l4SpotSpecialBetsCashWL,
                                         GetKSession(i)->l4SpotSpecialBetsCash);
  }
  Total_PercRet4SpotSpecBets = GetPercRet(Total_TotalCashWL4SpotSpecBets,
                                          Total_TotalCash4SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Make_PercRet6SpotSpecBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet6SpotSpecBetsStr[i] = MakeString(PercRet6SpotSpecBets[i]);
  }
  Total_PercRet6SpotSpecBetsStr = MakeString(Total_PercRet6SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Calc_PercRet6SpotSpecBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercRet6SpotSpecBets[i] = GetPercRet(GetKSession(i)->l6SpotSpecialBetsCashWL,
                                         GetKSession(i)->l6SpotSpecialBetsCash);
  }
  Total_PercRet6SpotSpecBets = GetPercRet(Total_TotalCashWL6SpotSpecBets,
                                          Total_TotalCash6SpotSpecBets);
}
//*******************************************************************
void TStatCalcKeno::Make_TotalGamesStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalGamesStr[i] = MakeString(GetKSession(i)->nTotalGames);
  }
  Total_TotalGamesStr = MakeString(Total_TotalGames);
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalGames()
{
  Total_TotalGames = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
	 Total_TotalGames += GetKSession(i)->nTotalGames;
  }
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNum1Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
	 MostFreqNum1Str[i] = MakeString(MostFreqNum1Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNum1Str = NAStr;
  }
  else
  {
    Total_MostFreqNum1Str = MakeString(Total_MostFreqNum1Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_TotalMostFreqNums()
{
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    TotalMostFreqNums[nums] = 0;
  }

  for (int i=0;i<=LastSessionNum;i++)
  {
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      TotalMostFreqNums[j] += GetKSession(i)->nMostFrequentNums[j];
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNum1()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum1Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (GetKSession(i)->nMostFrequentNums[j] >
          GetKSession(i)->nMostFrequentNums[MostFreqNum1Index[i]])
      {
        MostFreqNum1Index[i] = j;
      }
    }
  }
  Total_MostFreqNum1Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (TotalMostFreqNums[nums] > TotalMostFreqNums[Total_MostFreqNum1Index])
    {
      Total_MostFreqNum1Index = nums;
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNum2Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum2Str[i] = MakeString(MostFreqNum2Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNum2Str = NAStr;
  }
  else
  {
    Total_MostFreqNum2Str = MakeString(Total_MostFreqNum2Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNum2()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum2Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (j != MostFreqNum1Index[i])
      {
        if (GetKSession(i)->nMostFrequentNums[j] >
            GetKSession(i)->nMostFrequentNums[MostFreqNum2Index[i]])
        {
          MostFreqNum2Index[i] = j;
        }
      }
    }
  }
  Total_MostFreqNum2Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (nums != Total_MostFreqNum1Index)
    {
      if (TotalMostFreqNums[nums] > TotalMostFreqNums[Total_MostFreqNum2Index])
      {
        Total_MostFreqNum2Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNum3Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum3Str[i] = MakeString(MostFreqNum3Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNum3Str = NAStr;
  }
  else
  {
    Total_MostFreqNum3Str = MakeString(Total_MostFreqNum3Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNum3()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum3Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (j != MostFreqNum1Index[i] && j != MostFreqNum2Index[i])
      {
        if (GetKSession(i)->nMostFrequentNums[j] >
            GetKSession(i)->nMostFrequentNums[MostFreqNum3Index[i]])
        {
          MostFreqNum3Index[i] = j;
        }
      }
    }
  }
  Total_MostFreqNum3Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (nums != Total_MostFreqNum1Index && nums != Total_MostFreqNum2Index)
    {
      if (TotalMostFreqNums[nums] > TotalMostFreqNums[Total_MostFreqNum3Index])
      {
        Total_MostFreqNum3Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNum4Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum4Str[i] = MakeString(MostFreqNum4Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNum4Str = NAStr;
  }
  else
  {
    Total_MostFreqNum4Str = MakeString(Total_MostFreqNum4Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNum4()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum4Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (j != MostFreqNum1Index[i] && j != MostFreqNum2Index[i] && j != MostFreqNum3Index[i])
      {
        if (GetKSession(i)->nMostFrequentNums[j] >
            GetKSession(i)->nMostFrequentNums[MostFreqNum4Index[i]])
        {
          MostFreqNum4Index[i] = j;
        }
      }
    }
  }
  Total_MostFreqNum4Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (nums != Total_MostFreqNum1Index && nums != Total_MostFreqNum2Index &&
        nums != Total_MostFreqNum3Index)
    {
      if (TotalMostFreqNums[nums] > TotalMostFreqNums[Total_MostFreqNum4Index])
      {
        Total_MostFreqNum4Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNum5Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
	 MostFreqNum5Str[i] = MakeString(MostFreqNum5Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNum5Str = NAStr;
  }
  else
  {
    Total_MostFreqNum5Str = MakeString(Total_MostFreqNum5Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNum5()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNum5Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (j != MostFreqNum1Index[i] && j != MostFreqNum2Index[i] &&
          j != MostFreqNum3Index[i] && j != MostFreqNum4Index[i])
      {
        if (GetKSession(i)->nMostFrequentNums[j] >
            GetKSession(i)->nMostFrequentNums[MostFreqNum5Index[i]])
        {
          MostFreqNum5Index[i] = j;
        }
      }
    }
  }
  Total_MostFreqNum5Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (nums != Total_MostFreqNum1Index && nums != Total_MostFreqNum2Index &&
        nums != Total_MostFreqNum3Index && nums != Total_MostFreqNum4Index)
    {
      if (TotalMostFreqNums[nums] > TotalMostFreqNums[Total_MostFreqNum5Index])
      {
        Total_MostFreqNum5Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNum1Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum1Str[i] = MakeString(LeastFreqNum1Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNum1Str = NAStr;
  }
  else
  {
    Total_LeastFreqNum1Str = MakeString(Total_LeastFreqNum1Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNum1()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum1Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (GetKSession(i)->nMostFrequentNums[j] <
          GetKSession(i)->nMostFrequentNums[LeastFreqNum1Index[i]])
      {
        LeastFreqNum1Index[i] = j;
      }
    }
  }
  Total_LeastFreqNum1Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (TotalMostFreqNums[nums] < TotalMostFreqNums[Total_LeastFreqNum1Index])
    {
      Total_LeastFreqNum1Index = nums;
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNum2Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum2Str[i] = MakeString(LeastFreqNum2Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNum2Str = NAStr;
  }
  else
  {
    Total_LeastFreqNum2Str = MakeString(Total_LeastFreqNum2Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNum2()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum2Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (j != LeastFreqNum1Index[i])
      {
        if (GetKSession(i)->nMostFrequentNums[j] <
            GetKSession(i)->nMostFrequentNums[LeastFreqNum2Index[i]])
        {
          LeastFreqNum2Index[i] = j;
        }
      }
    }
  }
  Total_LeastFreqNum2Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (nums != Total_LeastFreqNum1Index)
    {
      if (TotalMostFreqNums[nums] < TotalMostFreqNums[Total_LeastFreqNum2Index])
      {
        Total_LeastFreqNum2Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNum3Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum3Str[i] = MakeString(LeastFreqNum3Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNum3Str = NAStr;
  }
  else
  {
    Total_LeastFreqNum3Str = MakeString(Total_LeastFreqNum3Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNum3()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum3Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (j != LeastFreqNum1Index[i] && j != LeastFreqNum2Index[i])
      {
        if (GetKSession(i)->nMostFrequentNums[j] <
            GetKSession(i)->nMostFrequentNums[LeastFreqNum3Index[i]])
        {
          LeastFreqNum3Index[i] = j;
        }
      }
    }
  }
  Total_LeastFreqNum3Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (nums != Total_LeastFreqNum1Index && nums != Total_LeastFreqNum2Index)
    {
      if (TotalMostFreqNums[nums] < TotalMostFreqNums[Total_LeastFreqNum3Index])
      {
        Total_LeastFreqNum3Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNum4Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum4Str[i] = MakeString(LeastFreqNum4Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNum4Str = NAStr;
  }
  else
  {
    Total_LeastFreqNum4Str = MakeString(Total_LeastFreqNum4Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNum4()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum4Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (j != LeastFreqNum1Index[i] && j != LeastFreqNum2Index[i] && j != LeastFreqNum3Index[i])
      {
        if (GetKSession(i)->nMostFrequentNums[j] <
            GetKSession(i)->nMostFrequentNums[LeastFreqNum4Index[i]])
        {
          LeastFreqNum4Index[i] = j;
        }
      }
    }
  }
  Total_LeastFreqNum4Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (nums != Total_LeastFreqNum1Index && nums != Total_LeastFreqNum2Index &&
        nums != Total_LeastFreqNum3Index)
    {
      if (TotalMostFreqNums[nums] < TotalMostFreqNums[Total_LeastFreqNum4Index])
      {
        Total_LeastFreqNum4Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNum5Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
	 LeastFreqNum5Str[i] = MakeString(LeastFreqNum5Index[i]+1);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNum5Str = NAStr;
  }
  else
  {
    Total_LeastFreqNum5Str = MakeString(Total_LeastFreqNum5Index);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNum5()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNum5Index[i] = 0;
    for (int j=0;j<MAX_KENO_SPOTS;j++)
    {
      if (j != LeastFreqNum1Index[i] && j != LeastFreqNum2Index[i] &&
          j != LeastFreqNum3Index[i] && j != LeastFreqNum4Index[i])
      {
        if (GetKSession(i)->nMostFrequentNums[j] <
            GetKSession(i)->nMostFrequentNums[LeastFreqNum5Index[i]])
        {
          LeastFreqNum5Index[i] = j;
        }
      }
    }
  }
  Total_LeastFreqNum5Index = 0;
  for (int nums = 0;nums < MAX_KENO_SPOTS;nums++)
  {
    if (nums != Total_LeastFreqNum1Index && nums != Total_LeastFreqNum2Index &&
        nums != Total_LeastFreqNum3Index && nums != Total_LeastFreqNum4Index)
    {
      if (TotalMostFreqNums[nums] < TotalMostFreqNums[Total_LeastFreqNum5Index])
      {
        Total_LeastFreqNum5Index = nums;
      }
    }
  }
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNumOcc1Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNumOcc1Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[MostFreqNum1Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNumOcc1Str = NAStr;
  }
  else
  {
    Total_MostFreqNumOcc1Str = MakeString(TotalMostFreqNums[Total_MostFreqNum1Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNumOcc1()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNumOcc2Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNumOcc2Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[MostFreqNum2Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNumOcc2Str = NAStr;
  }
  else
  {
    Total_MostFreqNumOcc2Str = MakeString(TotalMostFreqNums[Total_MostFreqNum2Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNumOcc2()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNumOcc3Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNumOcc3Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[MostFreqNum3Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNumOcc3Str = NAStr;
  }
  else
  {
    Total_MostFreqNumOcc3Str = MakeString(TotalMostFreqNums[Total_MostFreqNum3Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNumOcc3()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNumOcc4Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNumOcc4Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[MostFreqNum4Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNumOcc4Str = NAStr;
  }
  else
  {
    Total_MostFreqNumOcc4Str = MakeString(TotalMostFreqNums[Total_MostFreqNum4Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNumOcc4()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_MostFreqNumOcc5Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    MostFreqNumOcc5Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[MostFreqNum5Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_MostFreqNumOcc5Str = NAStr;
  }
  else
  {
    Total_MostFreqNumOcc5Str = MakeString(TotalMostFreqNums[Total_MostFreqNum5Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_MostFreqNumOcc5()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNumOcc1Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNumOcc1Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[LeastFreqNum1Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNumOcc1Str = NAStr;
  }
  else
  {
    Total_LeastFreqNumOcc1Str = MakeString(TotalMostFreqNums[Total_LeastFreqNum1Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNumOcc1()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNumOcc2Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNumOcc2Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[LeastFreqNum2Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNumOcc2Str = NAStr;
  }
  else
  {
    Total_LeastFreqNumOcc2Str = MakeString(TotalMostFreqNums[Total_LeastFreqNum2Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNumOcc2()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNumOcc3Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNumOcc3Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[LeastFreqNum3Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNumOcc3Str = NAStr;
  }
  else
  {
    Total_LeastFreqNumOcc3Str = MakeString(TotalMostFreqNums[Total_LeastFreqNum3Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNumOcc3()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNumOcc4Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNumOcc4Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[LeastFreqNum4Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNumOcc4Str = NAStr;
  }
  else
  {
    Total_LeastFreqNumOcc4Str = MakeString(TotalMostFreqNums[Total_LeastFreqNum4Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNumOcc4()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::Make_LeastFreqNumOcc5Strs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LeastFreqNumOcc5Str[i] = MakeString(GetKSession(i)->nMostFrequentNums[LeastFreqNum5Index[i]]);
  }
  if (LastSessionNum < 0)
  {
    Total_LeastFreqNumOcc5Str = NAStr;
  }
  else
  {
    Total_LeastFreqNumOcc5Str = MakeString(TotalMostFreqNums[Total_LeastFreqNum5Index]);
  }
}
//*******************************************************************
void TStatCalcKeno::Calc_LeastFreqNumOcc5()
{
  // do nothing.
}
//*******************************************************************
void TStatCalcKeno::LocalCalculate()
{
  TCasinoStatCalc::LocalCalculate();

  if (PPlayerData == NULL) {return;}

  Calc_CumulativeCashWL();
  Calc_Num1SpotBets();
  Calc_Num2SpotBets();
  Calc_Num3SpotBets();
  Calc_Num4SpotBets();
  Calc_Num5SpotBets();
  Calc_Num6SpotBets();
  Calc_Num7SpotBets();
  Calc_Num8SpotBets();
  Calc_Num9SpotBets();
  Calc_Num10SpotBets();
  Calc_Num3SpotSpecBets();
  Calc_Num4SpotSpecBets();
  Calc_Num6SpotSpecBets();
  Calc_TotalCashWL1SpotBets();
  Calc_TotalCashWL2SpotBets();
  Calc_TotalCashWL3SpotBets();
  Calc_TotalCashWL4SpotBets();
  Calc_TotalCashWL5SpotBets();
  Calc_TotalCashWL6SpotBets();
  Calc_TotalCashWL7SpotBets();
  Calc_TotalCashWL8SpotBets();
  Calc_TotalCashWL9SpotBets();
  Calc_TotalCashWL10SpotBets();
  Calc_TotalCashWL3SpotSpecBets();
  Calc_TotalCashWL4SpotSpecBets();
  Calc_TotalCashWL6SpotSpecBets();
  Calc_TotalCash1SpotBets();
  Calc_TotalCash2SpotBets();
  Calc_TotalCash3SpotBets();
  Calc_TotalCash4SpotBets();
  Calc_TotalCash5SpotBets();
  Calc_TotalCash6SpotBets();
  Calc_TotalCash7SpotBets();
  Calc_TotalCash8SpotBets();
  Calc_TotalCash9SpotBets();
  Calc_TotalCash10SpotBets();
  Calc_TotalCash3SpotSpecBets();
  Calc_TotalCash4SpotSpecBets();
  Calc_TotalCash6SpotSpecBets();
  Calc_PercRet1SpotBets();
  Calc_PercRet2SpotBets();
  Calc_PercRet3SpotBets();
  Calc_PercRet4SpotBets();
  Calc_PercRet5SpotBets();
  Calc_PercRet6SpotBets();
  Calc_PercRet7SpotBets();
  Calc_PercRet8SpotBets();
  Calc_PercRet9SpotBets();
  Calc_PercRet10SpotBets();
  Calc_PercRet3SpotSpecBets();
  Calc_PercRet4SpotSpecBets();
  Calc_PercRet6SpotSpecBets();
  Calc_TotalGames();
  Calc_TotalMostFreqNums();
  Calc_MostFreqNum1();
  Calc_MostFreqNum2();
  Calc_MostFreqNum3();
  Calc_MostFreqNum4();
  Calc_MostFreqNum5();
  Calc_LeastFreqNum1();
  Calc_LeastFreqNum2();
  Calc_LeastFreqNum3();
  Calc_LeastFreqNum4();
  Calc_LeastFreqNum5();
  Calc_MostFreqNumOcc1();
  Calc_MostFreqNumOcc2();
  Calc_MostFreqNumOcc3();
  Calc_MostFreqNumOcc4();
  Calc_MostFreqNumOcc5();
  Calc_LeastFreqNumOcc1();
  Calc_LeastFreqNumOcc2();
  Calc_LeastFreqNumOcc3();
  Calc_LeastFreqNumOcc4();
  Calc_LeastFreqNumOcc5();
}
//*******************************************************************
void TStatCalcKeno::LocalMakeAllStrings()
{
  TCasinoStatCalc::LocalMakeAllStrings();

  if (PPlayerData == NULL) {return;}
  Make_CumulativeCashWLStrs();
  Make_Num1SpotBetsStrs();
  Make_Num2SpotBetsStrs();
  Make_Num3SpotBetsStrs();
  Make_Num4SpotBetsStrs();
  Make_Num5SpotBetsStrs();
  Make_Num6SpotBetsStrs();
  Make_Num7SpotBetsStrs();
  Make_Num8SpotBetsStrs();
  Make_Num9SpotBetsStrs();
  Make_Num10SpotBetsStrs();
  Make_Num3SpotSpecBetsStrs();
  Make_Num4SpotSpecBetsStrs();
  Make_Num6SpotSpecBetsStrs();
  Make_TotalCashWL1SpotBetsStrs();
  Make_TotalCashWL2SpotBetsStrs();
  Make_TotalCashWL3SpotBetsStrs();
  Make_TotalCashWL4SpotBetsStrs();
  Make_TotalCashWL5SpotBetsStrs();
  Make_TotalCashWL6SpotBetsStrs();
  Make_TotalCashWL7SpotBetsStrs();
  Make_TotalCashWL8SpotBetsStrs();
  Make_TotalCashWL9SpotBetsStrs();
  Make_TotalCashWL10SpotBetsStrs();
  Make_TotalCashWL3SpotSpecBetsStrs();
  Make_TotalCashWL4SpotSpecBetsStrs();
  Make_TotalCashWL6SpotSpecBetsStrs();
  Make_PercRet1SpotBetsStrs();
  Make_PercRet2SpotBetsStrs();
  Make_PercRet3SpotBetsStrs();
  Make_PercRet4SpotBetsStrs();
  Make_PercRet5SpotBetsStrs();
  Make_PercRet6SpotBetsStrs();
  Make_PercRet7SpotBetsStrs();
  Make_PercRet8SpotBetsStrs();
  Make_PercRet9SpotBetsStrs();
  Make_PercRet10SpotBetsStrs();
  Make_PercRet3SpotSpecBetsStrs();
  Make_PercRet4SpotSpecBetsStrs();
  Make_PercRet6SpotSpecBetsStrs();
  Make_TotalGamesStrs();
  Make_MostFreqNum1Strs();
  Make_MostFreqNum2Strs();
  Make_MostFreqNum3Strs();
  Make_MostFreqNum4Strs();
  Make_MostFreqNum5Strs();
  Make_LeastFreqNum1Strs();
  Make_LeastFreqNum2Strs();
  Make_LeastFreqNum3Strs();
  Make_LeastFreqNum4Strs();
  Make_LeastFreqNum5Strs();
  Make_MostFreqNumOcc1Strs();
  Make_MostFreqNumOcc2Strs();
  Make_MostFreqNumOcc3Strs();
  Make_MostFreqNumOcc4Strs();
  Make_MostFreqNumOcc5Strs();
  Make_LeastFreqNumOcc1Strs();
  Make_LeastFreqNumOcc2Strs();
  Make_LeastFreqNumOcc3Strs();
  Make_LeastFreqNumOcc4Strs();
  Make_LeastFreqNumOcc5Strs();
}
//*******************************************************************
void TStatCalcKeno::LocalFillGrid()
{
  TCasinoStatCalc::LocalFillGrid();

  if (PPlayerData == NULL) {return;}

  for(int col = 0;col <= LastSessionNum;col++)
  {
    int c = col + 1;
    PStatDlg->cell[1][c] = DateStr[col];
    PStatDlg->cell[2][c] = HoursPlayedStr[col];
    PStatDlg->cell[3][c] = TypeGamePlayedStr[col];
    PStatDlg->cell[6][c] = TotalNumBetsStr[col];
    PStatDlg->cell[7][c] = TotalCashBetsStr[col];
    PStatDlg->cell[8][c] = AvgCashBetsStr[col];
    PStatDlg->cell[9][c] = StartingBankrollStr[col];
    PStatDlg->cell[10][c] = EndingBankrollStr[col];
    PStatDlg->cell[11][c] = TotalCashWLStr[col];
    PStatDlg->cell[12][c] = ReturnPercStr[col];
    PStatDlg->cell[13][c] = AvgWLPerHourStr[col];
	 PStatDlg->cell[14][c] = TotalCashWLStr[col];// duh... TotalCashBetsStr[col];
    PStatDlg->cell[17][c] = TotalNumBetsStr[col];
    PStatDlg->cell[18][c] = HandsWonStr[col];
    PStatDlg->cell[19][c] = HandsLostStr[col];
    PStatDlg->cell[20][c] = PercHandsWonStr[col];
    PStatDlg->cell[21][c] = PercHandsLostStr[col];
    PStatDlg->cell[22][c] = LongestWinStreakStr[col];
    PStatDlg->cell[23][c] = LongestLosingStreakStr[col];
    PStatDlg->cell[24][c] = LargestCashWinStr[col];
    PStatDlg->cell[25][c] = LargestCashLostStr[col];
    PStatDlg->cell[26][c] = PeakCashWinLevelStr[col];
    PStatDlg->cell[27][c] = LowCashLostLevelStr[col];
    PStatDlg->cell[30][c] = Num1SpotBetsStr[col];
    PStatDlg->cell[31][c] = Num2SpotBetsStr[col];
    PStatDlg->cell[32][c] = Num3SpotBetsStr[col];
    PStatDlg->cell[33][c] = Num4SpotBetsStr[col];
    PStatDlg->cell[34][c] = Num5SpotBetsStr[col];
    PStatDlg->cell[35][c] = Num6SpotBetsStr[col];
    PStatDlg->cell[36][c] = Num7SpotBetsStr[col];
    PStatDlg->cell[37][c] = Num8SpotBetsStr[col];
    PStatDlg->cell[38][c] = Num9SpotBetsStr[col];
    PStatDlg->cell[39][c] = Num10SpotBetsStr[col];
    PStatDlg->cell[40][c] = Num3SpotSpecBetsStr[col];
    PStatDlg->cell[41][c] = Num4SpotSpecBetsStr[col];
    PStatDlg->cell[42][c] = Num6SpotSpecBetsStr[col];
    PStatDlg->cell[47][c] = TotalCashWL1SpotBetsStr[col];
    PStatDlg->cell[48][c] = TotalCashWL2SpotBetsStr[col];
    PStatDlg->cell[49][c] = TotalCashWL3SpotBetsStr[col];
    PStatDlg->cell[50][c] = TotalCashWL4SpotBetsStr[col];
    PStatDlg->cell[51][c] = TotalCashWL5SpotBetsStr[col];
    PStatDlg->cell[52][c] = TotalCashWL6SpotBetsStr[col];
    PStatDlg->cell[53][c] = TotalCashWL7SpotBetsStr[col];
    PStatDlg->cell[54][c] = TotalCashWL8SpotBetsStr[col];
    PStatDlg->cell[55][c] = TotalCashWL9SpotBetsStr[col];
    PStatDlg->cell[56][c] = TotalCashWL10SpotBetsStr[col];
    PStatDlg->cell[57][c] = TotalCashWL3SpotSpecBetsStr[col];
    PStatDlg->cell[58][c] = TotalCashWL4SpotSpecBetsStr[col];
    PStatDlg->cell[59][c] = TotalCashWL6SpotSpecBetsStr[col];
    PStatDlg->cell[64][c] = PercRet1SpotBetsStr[col];
    PStatDlg->cell[65][c] = PercRet2SpotBetsStr[col];
    PStatDlg->cell[66][c] = PercRet3SpotBetsStr[col];
    PStatDlg->cell[67][c] = PercRet4SpotBetsStr[col];
    PStatDlg->cell[68][c] = PercRet5SpotBetsStr[col];
    PStatDlg->cell[69][c] = PercRet6SpotBetsStr[col];
    PStatDlg->cell[70][c] = PercRet7SpotBetsStr[col];
    PStatDlg->cell[71][c] = PercRet8SpotBetsStr[col];
    PStatDlg->cell[72][c] = PercRet9SpotBetsStr[col];
    PStatDlg->cell[73][c] = PercRet10SpotBetsStr[col];
    PStatDlg->cell[74][c] = PercRet3SpotSpecBetsStr[col];
    PStatDlg->cell[75][c] = PercRet4SpotSpecBetsStr[col];
    PStatDlg->cell[76][c] = PercRet6SpotSpecBetsStr[col];
    PStatDlg->cell[81][c] = TotalGamesStr[col];
    PStatDlg->cell[82][c] = MostFreqNum1Str[col];
    PStatDlg->cell[83][c] = MostFreqNumOcc1Str[col];
    PStatDlg->cell[84][c] = MostFreqNum2Str[col];
    PStatDlg->cell[85][c] = MostFreqNumOcc2Str[col];
    PStatDlg->cell[86][c] = MostFreqNum3Str[col];
    PStatDlg->cell[87][c] = MostFreqNumOcc3Str[col];
    PStatDlg->cell[88][c] = MostFreqNum4Str[col];
    PStatDlg->cell[89][c] = MostFreqNumOcc4Str[col];
    PStatDlg->cell[90][c] = MostFreqNum5Str[col];
    PStatDlg->cell[91][c] = MostFreqNumOcc5Str[col];
    PStatDlg->cell[92][c] = LeastFreqNum1Str[col];
    PStatDlg->cell[93][c] = LeastFreqNumOcc1Str[col];
    PStatDlg->cell[94][c] = LeastFreqNum2Str[col];
    PStatDlg->cell[95][c] = LeastFreqNumOcc2Str[col];
    PStatDlg->cell[96][c] = LeastFreqNum3Str[col];
    PStatDlg->cell[97][c] = LeastFreqNumOcc3Str[col];
    PStatDlg->cell[98][c] = LeastFreqNum4Str[col];
    PStatDlg->cell[99][c] = LeastFreqNumOcc4Str[col];
    PStatDlg->cell[100][c] = LeastFreqNum5Str[col];
    PStatDlg->cell[101][c] = LeastFreqNumOcc5Str[col];
  }

  PStatDlg->totalcol[1] = Total_DateStr;
  PStatDlg->totalcol[2] = Total_HoursPlayedStr;
  PStatDlg->totalcol[3] = Total_TypeGamePlayedStr;
  PStatDlg->totalcol[6] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[7] = Total_TotalCashBetsStr;
  PStatDlg->totalcol[8] = Total_AvgCashBetsStr;
  PStatDlg->totalcol[9] = Total_StartingBankrollStr;
  PStatDlg->totalcol[10] = Total_EndingBankrollStr;
  PStatDlg->totalcol[11] = Total_TotalCashWLStr;
  PStatDlg->totalcol[12] = Total_ReturnPercStr;
  PStatDlg->totalcol[13] = Total_AvgWLPerHourStr;
  PStatDlg->totalcol[14] = Total_TotalCashWLStr;// duh... Total_TotalCashBetsStr;
  PStatDlg->totalcol[17] = Total_TotalNumBetsStr;
  PStatDlg->totalcol[18] = Total_HandsWonStr;
  PStatDlg->totalcol[19] = Total_HandsLostStr;
  PStatDlg->totalcol[20] = Total_PercHandsWonStr;
  PStatDlg->totalcol[21] = Total_PercHandsLostStr;
  PStatDlg->totalcol[22] = Total_LongestWinStreakStr;
  PStatDlg->totalcol[23] = Total_LongestLosingStreakStr;
  PStatDlg->totalcol[24] = Total_LargestCashWinStr;
  PStatDlg->totalcol[25] = Total_LargestCashLostStr;
  PStatDlg->totalcol[26] = Total_PeakCashWinLevelStr;
  PStatDlg->totalcol[27] = Total_LowCashLostLevelStr;
  PStatDlg->totalcol[30] = Total_Num1SpotBetsStr;
  PStatDlg->totalcol[31] = Total_Num2SpotBetsStr;
  PStatDlg->totalcol[32] = Total_Num3SpotBetsStr;
  PStatDlg->totalcol[33] = Total_Num4SpotBetsStr;
  PStatDlg->totalcol[34] = Total_Num5SpotBetsStr;
  PStatDlg->totalcol[35] = Total_Num6SpotBetsStr;
  PStatDlg->totalcol[36] = Total_Num7SpotBetsStr;
  PStatDlg->totalcol[37] = Total_Num8SpotBetsStr;
  PStatDlg->totalcol[38] = Total_Num9SpotBetsStr;
  PStatDlg->totalcol[39] = Total_Num10SpotBetsStr;
  PStatDlg->totalcol[40] = Total_Num3SpotSpecBetsStr;
  PStatDlg->totalcol[41] = Total_Num4SpotSpecBetsStr;
  PStatDlg->totalcol[42] = Total_Num6SpotSpecBetsStr;
  PStatDlg->totalcol[47] = Total_TotalCashWL1SpotBetsStr;
  PStatDlg->totalcol[48] = Total_TotalCashWL2SpotBetsStr;
  PStatDlg->totalcol[49] = Total_TotalCashWL3SpotBetsStr;
  PStatDlg->totalcol[50] = Total_TotalCashWL4SpotBetsStr;
  PStatDlg->totalcol[51] = Total_TotalCashWL5SpotBetsStr;
  PStatDlg->totalcol[52] = Total_TotalCashWL6SpotBetsStr;
  PStatDlg->totalcol[53] = Total_TotalCashWL7SpotBetsStr;
  PStatDlg->totalcol[54] = Total_TotalCashWL8SpotBetsStr;
  PStatDlg->totalcol[55] = Total_TotalCashWL9SpotBetsStr;
  PStatDlg->totalcol[56] = Total_TotalCashWL10SpotBetsStr;
  PStatDlg->totalcol[57] = Total_TotalCashWL3SpotSpecBetsStr;
  PStatDlg->totalcol[58] = Total_TotalCashWL4SpotSpecBetsStr;
  PStatDlg->totalcol[59] = Total_TotalCashWL6SpotSpecBetsStr;
  PStatDlg->totalcol[64] = Total_PercRet1SpotBetsStr;
  PStatDlg->totalcol[65] = Total_PercRet2SpotBetsStr;
  PStatDlg->totalcol[66] = Total_PercRet3SpotBetsStr;
  PStatDlg->totalcol[67] = Total_PercRet4SpotBetsStr;
  PStatDlg->totalcol[68] = Total_PercRet5SpotBetsStr;
  PStatDlg->totalcol[69] = Total_PercRet6SpotBetsStr;
  PStatDlg->totalcol[70] = Total_PercRet7SpotBetsStr;
  PStatDlg->totalcol[71] = Total_PercRet8SpotBetsStr;
  PStatDlg->totalcol[72] = Total_PercRet9SpotBetsStr;
  PStatDlg->totalcol[73] = Total_PercRet10SpotBetsStr;
  PStatDlg->totalcol[74] = Total_PercRet3SpotSpecBetsStr;
  PStatDlg->totalcol[75] = Total_PercRet4SpotSpecBetsStr;
  PStatDlg->totalcol[76] = Total_PercRet6SpotSpecBetsStr;
  PStatDlg->totalcol[81] = Total_TotalGamesStr;
  PStatDlg->totalcol[82] = Total_MostFreqNum1Str;
  PStatDlg->totalcol[83] = Total_MostFreqNumOcc1Str;
  PStatDlg->totalcol[84] = Total_MostFreqNum2Str;
  PStatDlg->totalcol[85] = Total_MostFreqNumOcc2Str;
  PStatDlg->totalcol[86] = Total_MostFreqNum3Str;
  PStatDlg->totalcol[87] = Total_MostFreqNumOcc3Str;
  PStatDlg->totalcol[88] = Total_MostFreqNum4Str;
  PStatDlg->totalcol[89] = Total_MostFreqNumOcc4Str;
  PStatDlg->totalcol[90] = Total_MostFreqNum5Str;
  PStatDlg->totalcol[91] = Total_MostFreqNumOcc5Str;
  PStatDlg->totalcol[92] = Total_LeastFreqNum1Str;
  PStatDlg->totalcol[93] = Total_LeastFreqNumOcc1Str;
  PStatDlg->totalcol[94] = Total_LeastFreqNum2Str;
  PStatDlg->totalcol[95] = Total_LeastFreqNumOcc2Str;
  PStatDlg->totalcol[96] = Total_LeastFreqNum3Str;
  PStatDlg->totalcol[97] = Total_LeastFreqNumOcc3Str;
  PStatDlg->totalcol[98] = Total_LeastFreqNum4Str;
  PStatDlg->totalcol[99] = Total_LeastFreqNumOcc4Str;
  PStatDlg->totalcol[100] = Total_LeastFreqNum5Str;
  PStatDlg->totalcol[101] = Total_LeastFreqNumOcc5Str;
}
//*******************************************************************
//*******************************************************************
TKenoGraph::TKenoGraph(TCasinoStatDlg* pstatdlg,TStatCalcKeno *pstatcalc)
:TCasinoGraphDlg(pstatdlg,pstatcalc)
{
  strName = "Keno";
  BaseStringID = KenoLabel;

  StatTT[0] = TStatCalcKeno::kTotalNumBets;
  StatTT[1] = TStatCalcKeno::kTotalCashBets;
  StatTT[2] = TStatCalcKeno::kAvgCashBet;
  StatTT[3] = TStatCalcKeno::kTotalCashWL;
  StatTT[4] = TStatCalcKeno::kCumCashWL;
  StatTT[5] = TStatCalcKeno::kBetsWon;
  StatTT[6] = TStatCalcKeno::kBetsLost;
  StatTT[7] = TStatCalcKeno::kPercBetsWon;
  StatTT[8] = TStatCalcKeno::kPercBetsLost;
  StatTT[9] = TStatCalcKeno::kLongestWinStreak;
  StatTT[10] = TStatCalcKeno::kLongestLosingStreak;
  StatTT[11] = TStatCalcKeno::kLargestCashWin;
  StatTT[12] = TStatCalcKeno::kLargestCashLoss;
  StatTT[13] = TStatCalcKeno::kPeakCashWinLevel;
  StatTT[14] = TStatCalcKeno::kLowCashLossLevel;

  LastStatIndex = 14;
}
//*******************************************************************
TKenoGraph::~TKenoGraph()
{
}
//*******************************************************************
