//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cstcalc.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/6/96     M.Perry    derived from Dave's code.
//********************************************************************
#include "cstcalc.h"
#include "cstatdlg.h"
#include <stdio.h>
//*******************************************************************
TCasinoStatCalc::TCasinoStatCalc(TCasinoStatDlg *pstatdlg):
  PStatDlg(pstatdlg),
  InPlayStr("in play"),
  NAStr("N/A"),
  EmptyStr(""),
  LastSessionNum(-1),
  PData(NULL),
  PPlayerData(NULL),
  bCalculated(false),
  bStringsMade(false),
  bDataStorageAlloc(false),
  bStringStorageAlloc(false),
  bShowCents(false)
{
  #if defined(PLAYERDATA_TRACKCENTS)
  bShowCents = true;
  #endif

  for (int i=0;i<NUMSESSIONS;i++)
  {
    PSessions[i] = NULL;
  }
}
//*******************************************************************
TCasinoStatCalc::~TCasinoStatCalc()
{
  DeAllocDataStorage();
  DeAllocStringStorage();
}
//*******************************************************************
void TCasinoStatCalc::GetSessionPtrs()
{
  PStatDlg->MessageBox("base GetSessionPtrs() called.\nAbout to blow up.","Error",MB_OK);
}
//*******************************************************************
#pragma argsused
bool TCasinoStatCalc::GetValues(int statindex,float *values,int maxvalueindex)
{
  return false;
}
//*******************************************************************
void TCasinoStatCalc::LocalAllocDataStorage()
{
}
//*******************************************************************
void TCasinoStatCalc::AllocDataStorage()
{
  if (IsDataStorageAlloc()) {DeAllocDataStorage();}
  LocalAllocDataStorage();
  SetDataStorageAlloc(true);
}
//*******************************************************************
void TCasinoStatCalc::LocalAllocStringStorage()
{
}
//*******************************************************************
void TCasinoStatCalc::AllocStringStorage()
{
  if (IsStringStorageAlloc()) {DeAllocStringStorage();}
  LocalAllocStringStorage();
  SetStringStorageAlloc(true);
}
//*******************************************************************
void TCasinoStatCalc::LocalDeAllocDataStorage()
{
}
//*******************************************************************
void TCasinoStatCalc::DeAllocDataStorage()
{
  LocalDeAllocDataStorage();
  SetDataStorageAlloc(false);
}
//*******************************************************************
void TCasinoStatCalc::LocalDeAllocStringStorage()
{
}
//*******************************************************************
void TCasinoStatCalc::DeAllocStringStorage()
{
  LocalDeAllocStringStorage();
  SetStringStorageAlloc(false);
}
//*******************************************************************
void TCasinoStatCalc::LocalCalculate()
{
  Calc_Date();
  Calc_HoursPlayed();
  Calc_TypeGamePlayed();
  Calc_TotalNumBets();
  Calc_TotalCashBets();
  Calc_AvgCashBets();
  Calc_StartingBankroll();
  Calc_EndingBankroll();
  Calc_TotalCashWL();
  Calc_ReturnPerc();
  Calc_AvgWLPerHour();
  Calc_HandsWon();
  Calc_HandsLost();
  Calc_PercHandsWon();
  Calc_PercHandsLost();
  Calc_LongestWinStreak();
  Calc_LongestLosingStreak();
  Calc_LargestCashWin();
  Calc_LargestCashLost();
  Calc_PeakCashWinLevel();
  Calc_LowCashLostLevel();
}
//*******************************************************************
void TCasinoStatCalc::Calculate()
{
  GetPlayerData();
  if (PPlayerData == NULL) {return;}

  if (IsCalculated()) {return;}
  if (!IsDataStorageAlloc()) {AllocDataStorage();}

  LocalCalculate();
  SetCalculated(true);
}
//*******************************************************************
void TCasinoStatCalc::LocalMakeAllStrings()
{
  Make_DateStrs();
  Make_HoursPlayedStrs();
  Make_TypeGamePlayedStrs();
  Make_TotalNumBetsStrs();
  Make_TotalCashBetsStrs();
  Make_AvgCashBetsStrs();
  Make_StartingBankrollStrs();
  Make_EndingBankrollStrs();
  Make_TotalCashWLStrs();
  Make_ReturnPercStrs();
  Make_AvgWLPerHourStrs();
  Make_HandsWonStrs();
  Make_HandsLostStrs();
  Make_PercHandsWonStrs();
  Make_PercHandsLostStrs();
  Make_LongestWinStreakStrs();
  Make_LongestLosingStreakStrs();
  Make_LargestCashWinStrs();
  Make_LargestCashLostStrs();
  Make_PeakCashWinLevelStrs();
  Make_LowCashLostLevelStrs();
}
//*******************************************************************
void TCasinoStatCalc::MakeAllStrings()
{
  if (PPlayerData == NULL) {return;}

  if (IsStringsMade()) {return;}
  if (!IsStringStorageAlloc()) {AllocStringStorage();}

  LocalMakeAllStrings();
  SetStringsMade(true);
}
//*******************************************************************
void TCasinoStatCalc::ClearOwnerGrid()
{
  string strEmpty = "";
  for (int totalrow = 0;totalrow < NUMCACHEROWS;totalrow++)
  {
    PStatDlg->totalcol[totalrow] = strEmpty;
  }
  for (int rows = 0;rows < NUMCACHEROWS;rows++)
  {
    for (int column = 1;column < NUMCACHECOLS;column++)
    {
      PStatDlg->cell[rows][column] = strEmpty;
    }
  }
}
//*******************************************************************
void TCasinoStatCalc::LocalFillGrid()
{
}
//*******************************************************************
void TCasinoStatCalc::FillGrid()
{
  GetPlayerData();
  Calculate();
  MakeAllStrings();
  if (PPlayerData == NULL) {return;}
  LocalFillGrid();
}
//*******************************************************************
void TCasinoStatCalc::GetPlayerData()
{
  TCasinoPlayerData *pplayerdata = PStatDlg->GetPlayerData();
//  if (PPlayerData == pplayerdata) {return;}
  DeAllocDataStorage();
  DeAllocStringStorage();
  PPlayerData = pplayerdata;
  SetCalculated(false);
  SetStringsMade(false);
  GetSessionPtrs();
}
//*******************************************************************
string TCasinoStatCalc::MakeDateStr(DWORD seconds)
{
  TTime timeInSects(seconds);
  TDate theDate(timeInSects);
  char str[25];
  int year = theDate.Year();
  if (year < 2000) {year -= 1900;}
  else             {year -= 2000;}
  sprintf(str,"%02d/%02d/%02d",theDate.Month(),theDate.DayOfMonth(),year);
  return string(str);
}
//*******************************************************************
string TCasinoStatCalc::MakeTimeStr(DWORD lengthseconds)
{
  int nHoursElapsed = (int) lengthseconds /3600;
  DWORD dwPartOfHour = lengthseconds % 3600;
  int nMinutes =(int) dwPartOfHour / 60;
  char szTimeElapsed[15] ;
  wsprintf(szTimeElapsed,"%02d:%02d",nHoursElapsed,nMinutes);

  return string(szTimeElapsed);
}
//*******************************************************************
string TCasinoStatCalc::MakeDenomStr(TMoney denom)
{
//  if (denom.GetCents() == 0)
//  {
//    return denom.GetDollarStr();
//  }
//  else
//  {
//    return denom.GetStr();
//  }

  return denom.GetStr(bShowCents);  // show cents unless zero.
}
//*******************************************************************
float TCasinoStatCalc::GetAvg(long upper,long lower,float def)
{
  float value = def;
  if (lower != 0)
  {
    value = (float)upper / lower;
  }
  return value;
}
//*******************************************************************
TMoney TCasinoStatCalc::GetMoneyAvg(TMoney upper,long lower,TMoney def)
{
  TMoney value = def;
  if (lower != 0)
  {
    // MRP - 06/08/99 - ver2.2 - ambiguity fix
    //value = upper / lower;
    value = double(upper / lower);
  }
  return value;
}
//*******************************************************************
float TCasinoStatCalc::GetPerc(long upper,long lower,float def)
{
  float value = def;
  if (lower != 0)
  {
    value = (float)upper/lower * 100;
  }
  return value;
}
//*******************************************************************
float TCasinoStatCalc::GetPercRet(long upper,long lower,float def)
{
  float value = def;
  if (lower != 0)
  {
    value = (float)(lower + upper)/lower * 100;
  }
  return value;
}
//*******************************************************************
float TCasinoStatCalc::GetAvgPerHour(long value,long seconds,float def)
{
  float avg = def;
  float total_hrs = (float)seconds / 3600;
  if (total_hrs > 0)
  {
    avg = (float)value / total_hrs;
  }
  return avg;
}
//*******************************************************************
string TCasinoStatCalc::MakeString(int value)
{
	char str[25];
	itoa(value,str,10);
	return string(str);
}
//*******************************************************************
string TCasinoStatCalc::MakeString(long value)
{
	char str[25];
	ltoa(value,str,10);
	return string(str);
}
//*******************************************************************
string TCasinoStatCalc::MakeString(TMoney money)
{
//  char str[25];
//  ltoa((long)money,str,10);
//  return string(str);
  return money.GetStr(bShowCents);
}
//*******************************************************************
string TCasinoStatCalc::MakeString(float value)
{
  char buffer[25];
  sprintf(buffer,"%.2f",value);
  return string(buffer);
}
//*******************************************************************
void TCasinoStatCalc::Calc_Date()
{
  Total_Date = 0;
  if (LastSessionNum < 0)
  {
    Total_Date = TTime().Seconds();
  }
  else
  {
    for (int i=0;i<=LastSessionNum;i++)
    {
      Total_Date = maxDW(GetSession(i)->StartingTime,Total_Date);
    }
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_DateStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    if (GetSession(i)->InPlay) {DateStr[i] = InPlayStr;}
    else                       {DateStr[i] = MakeDateStr(GetSession(i)->StartingTime);}
  }
  Total_DateStr = MakeDateStr(Total_Date);
}
//*******************************************************************
void TCasinoStatCalc::Make_HoursPlayedStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HoursPlayedStr[i] = MakeTimeStr(SecondsPlayed[i]);
  }
  Total_HoursPlayedStr = MakeTimeStr(Total_SecondsPlayed);
}
//*******************************************************************
void TCasinoStatCalc::Calc_HoursPlayed()
{
  Total_SecondsPlayed = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    if (GetSession(i)->InPlay)
    {
      SecondsPlayed[i] = TTime().Seconds() - GetSession(i)->StartingTime;
    }
    else
    {
      SecondsPlayed[i] = GetSession(i)->EndingTime - GetSession(i)->StartingTime;
      Total_SecondsPlayed += SecondsPlayed[i];
      if (SecondsPlayed[i] < 60) {SecondsPlayed[i] = 60;} // at least 1 minute.
    }
  }
}
//*******************************************************************
#pragma argsused
string TCasinoStatCalc::MakeSubGameTypeStr(int subgametype,TMoney denom)
{
  return string("unk game");
}
//*******************************************************************
void TCasinoStatCalc::Make_TypeGamePlayedStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TypeGamePlayedStr[i] = MakeSubGameTypeStr(GetSession(i)->GameSubtype,GetSession(i)->GameDenom);
  }
  Total_TypeGamePlayedStr = EmptyStr;
}
//*******************************************************************
void TCasinoStatCalc::Calc_TypeGamePlayed()
{
  // do nothing
}
//*******************************************************************
void TCasinoStatCalc::Make_TotalNumBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalNumBetsStr[i] = MakeString(TotalNumBets[i]);
  }
  Total_TotalNumBetsStr = MakeString(Total_TotalNumBets);
}
//*******************************************************************
void TCasinoStatCalc::Calc_TotalNumBets()
{
  Total_TotalNumBets = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    TotalNumBets[i] = GetSession(i)->HandsPlayed;
    Total_TotalNumBets += TotalNumBets[i];
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_TotalCashBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)                      
  {
    TotalCashBetsStr[i] = MakeString(GetSession(i)->TotalBetsCash);
  }
  Total_TotalCashBetsStr = MakeString(Total_TotalCashBets);
}
//*******************************************************************
void TCasinoStatCalc::Calc_TotalCashBets()
{
  Total_TotalCashBets = 0.0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_TotalCashBets += GetSession(i)->TotalBetsCash;
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_AvgCashBetsStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    AvgCashBetsStr[i] = MakeString(AvgCashBets[i]);
  }
  Total_AvgCashBetsStr = MakeString(Total_AvgCashBets);
}
//*******************************************************************
void TCasinoStatCalc::Calc_AvgCashBets()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    // MRP - 06/08/99 - ver2.2
    AvgCashBets[i] = (double)GetMoneyAvg(GetSession(i)->TotalBetsCash,TotalNumBets[i]);
  }
  // MRP - 06/08/99
  Total_AvgCashBets = (double)GetMoneyAvg(Total_TotalCashBets,Total_TotalNumBets);
}
//*******************************************************************
void TCasinoStatCalc::Make_StartingBankrollStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    StartingBankrollStr[i] = MakeString(GetSession(i)->StartingCash);
  }
  Total_StartingBankrollStr = MakeString(Total_StartingBankroll);
}
//*******************************************************************
void TCasinoStatCalc::Calc_StartingBankroll()
{
  if (LastSessionNum < 0)
  {
    Total_StartingBankroll = PPlayerData->Cash;
  }
  else
  {
    Total_StartingBankroll = GetSession(0)->StartingCash;
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_EndingBankrollStrs()
{
  bool sessioninplay = false;
  for (int i=0;i<=LastSessionNum;i++)
  {
    if (GetSession(i)->InPlay)
    {
      EndingBankrollStr[i] = InPlayStr;
      sessioninplay = true;
    }
    else
    {
      EndingBankrollStr[i] = MakeString(GetSession(i)->EndingCash);
    }
  }
  if (sessioninplay)
  {
    Total_EndingBankrollStr = InPlayStr;
  }
  else
  {
    Total_EndingBankrollStr = MakeString(Total_EndingBankroll);
  }
}
//*******************************************************************
void TCasinoStatCalc::Calc_EndingBankroll()
{
  if (LastSessionNum < 0)
  {
    Total_EndingBankroll = PPlayerData->Cash;
  }
  else
  {
    Total_EndingBankroll = GetSession(LastSessionNum)->EndingCash;
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_TotalCashWLStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    if (GetSession(i)->InPlay)
    {
      TotalCashWLStr[i] = InPlayStr;
    }
    else
    {
      TotalCashWLStr[i] = MakeString(TotalCashWL[i]);
    }
  }
  Total_TotalCashWLStr = MakeString(Total_TotalCashWL);
}
//*******************************************************************
void TCasinoStatCalc::Calc_TotalCashWL()
{
  Total_TotalCashWL = 0.0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    if (GetSession(i)->InPlay)
    {
      TotalCashWL[i] = 0.0;
    }
    else
    {
      // MRP - 06/08/99 - ver2.2
      TotalCashWL[i] = double(GetSession(i)->EndingCash - GetSession(i)->StartingCash);
      Total_TotalCashWL += TotalCashWL[i];
    }
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_ReturnPercStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    if (GetSession(i)->InPlay)
    {
      ReturnPercStr[i] = InPlayStr;
    }
    else
    {
      ReturnPercStr[i] = MakeString(ReturnPerc[i]);
    }
  }
  Total_ReturnPercStr = MakeString(Total_ReturnPerc);
}
//*******************************************************************
void TCasinoStatCalc::Calc_ReturnPerc()
{
  for (int i =0;i<=LastSessionNum;i++)
  {
    if (GetSession(i)->InPlay)
    {
      ReturnPerc[i] = 0;
    }
    else
    {
      ReturnPerc[i] = GetPercRet(TotalCashWL[i],GetSession(i)->TotalBetsCash);
    }
  }
  Total_ReturnPerc = GetPercRet(Total_TotalCashWL,Total_TotalCashBets);
}
//*******************************************************************
void TCasinoStatCalc::Make_AvgWLPerHourStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    if (GetSession(i)->InPlay)
    {
      AvgWLPerHourStr[i] = InPlayStr;
    }
    else
    {
      AvgWLPerHourStr[i] = MakeString(AvgWLPerHour[i]);
    }
  }
  Total_AvgWLPerHourStr = MakeString(Total_AvgWLPerHour);
}
//*******************************************************************
void TCasinoStatCalc::Calc_AvgWLPerHour()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    AvgWLPerHour[i] = GetAvgPerHour(TotalCashWL[i],SecondsPlayed[i]);
  }
  Total_AvgWLPerHour = GetAvgPerHour(Total_TotalCashWL,Total_SecondsPlayed);
}
//*******************************************************************
void TCasinoStatCalc::Make_HandsWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HandsWonStr[i] = MakeString(GetSession(i)->Wins);
  }
  Total_HandsWonStr = MakeString(Total_HandsWon);
}
//*******************************************************************
void TCasinoStatCalc::Calc_HandsWon()
{
  Total_HandsWon = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_HandsWon += GetSession(i)->Wins;
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_HandsLostStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    HandsLostStr[i] = MakeString(GetSession(i)->Losses);
  }
  Total_HandsLostStr = MakeString(Total_HandsLost);
}
//*******************************************************************
void TCasinoStatCalc::Calc_HandsLost()
{
  Total_HandsLost = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_HandsLost += GetSession(i)->Losses;
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_PercHandsWonStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercHandsWonStr[i] = MakeString(PercHandsWon[i]);
  }
  Total_PercHandsWonStr = MakeString(Total_PercHandsWon);
}
//*******************************************************************
void TCasinoStatCalc::Calc_PercHandsWon()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercHandsWon[i] = GetPerc(GetSession(i)->Wins,TotalNumBets[i]);
  }
  Total_PercHandsWon = GetPerc(Total_HandsWon,Total_TotalNumBets);
}
//*******************************************************************
void TCasinoStatCalc::Make_PercHandsLostStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercHandsLostStr[i] = MakeString(PercHandsLost[i]);
  }
  Total_PercHandsLostStr = MakeString(Total_PercHandsLost);
}
//*******************************************************************
void TCasinoStatCalc::Calc_PercHandsLost()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PercHandsLost[i] = GetPerc(GetSession(i)->Losses,TotalNumBets[i]);
  }
  Total_PercHandsLost = GetPerc(Total_HandsLost,Total_TotalNumBets);
}
//*******************************************************************
void TCasinoStatCalc::Make_LongestWinStreakStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LongestWinStreakStr[i] = MakeString(GetSession(i)->WinStreak);
  }
  Total_LongestWinStreakStr = MakeString(Total_LongestWinStreak);
}
//*******************************************************************
void TCasinoStatCalc::Calc_LongestWinStreak()
{
  Total_LongestWinStreak = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_LongestWinStreak = max(Total_LongestWinStreak,GetSession(i)->WinStreak);
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_LongestLosingStreakStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LongestLosingStreakStr[i] = MakeString(GetSession(i)->LosingStreak);
  }
  Total_LongestLosingStreakStr = MakeString(Total_LongestLosingStreak);
}
//*******************************************************************
void TCasinoStatCalc::Calc_LongestLosingStreak()
{
  Total_LongestLosingStreak = 0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_LongestLosingStreak = max(GetSession(i)->LosingStreak,Total_LongestLosingStreak);
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_LargestCashWinStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LargestCashWinStr[i] = MakeString(GetSession(i)->LargestWin);
  }
  Total_LargestCashWinStr = MakeString(Total_LargestCashWin);
}
//*******************************************************************
void TCasinoStatCalc::Calc_LargestCashWin()
{
  Total_LargestCashWin = 0.0;
  for (int i =0;i<=LastSessionNum;i++)
  {
    Total_LargestCashWin = max(GetSession(i)->LargestWin,Total_LargestCashWin);
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_LargestCashLostStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LargestCashLostStr[i] = MakeString(GetSession(i)->LargestLoss);
  }
  Total_LargestCashLostStr = MakeString(Total_LargestCashLost);
}
//*******************************************************************
void TCasinoStatCalc::Calc_LargestCashLost()
{
  Total_LargestCashLost = 0.0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_LargestCashLost = max(GetSession(i)->LargestLoss,Total_LargestCashLost);
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_PeakCashWinLevelStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    PeakCashWinLevelStr[i] = MakeString(GetSession(i)->PeakWin);
  }
  Total_PeakCashWinLevelStr = MakeString(Total_PeakWinLevel);
}
//*******************************************************************
void TCasinoStatCalc::Calc_PeakCashWinLevel()
{
  Total_PeakWinLevel = 0.0;
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_PeakWinLevel = max(GetSession(i)->PeakWin,Total_PeakWinLevel);
  }
}
//*******************************************************************
void TCasinoStatCalc::Make_LowCashLostLevelStrs()
{
  for (int i=0;i<=LastSessionNum;i++)
  {
    LowCashLostLevelStr[i] = MakeString(GetSession(i)->PeakLoss);
  }
  Total_LowCashLostLevelStr = MakeString(Total_LowCashLostLevel);
}
//*******************************************************************
void TCasinoStatCalc::Calc_LowCashLostLevel()
{
  if (LastSessionNum < 0)
  {
    Total_LowCashLostLevel = PPlayerData->Cash;
  }
  else
  {
    Total_LowCashLostLevel = GetSession(0)->StartingCash;
  }
  for (int i=0;i<=LastSessionNum;i++)
  {
    Total_LowCashLostLevel = min(GetSession(i)->PeakLoss,Total_LowCashLostLevel);
  }
}
//*******************************************************************
bool TCasinoStatCalc::GetTotalNumBets(float *values,int num)
{
  for(int i=0;i<num;i++)
  {
    values[i] = TotalNumBets[i];
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetTotalCashBets(float *values,int num)
{
  for(int i=0;i<num;i++)
  {
    values[i] = (double)GetSession(i)->TotalBetsCash;
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetAvgCashBets(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = (double)AvgCashBets[i];
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetTotalCashWL(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = (double)TotalCashWL[i];
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetWins(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetSession(i)->Wins;
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetLosses(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetSession(i)->Losses;
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetPercWins(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = PercHandsWon[i];
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetPercLosses(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = PercHandsLost[i];
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetLongestWinStreaks(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetSession(i)->WinStreak;
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetLongestLosingStreak(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = GetSession(i)->LosingStreak;
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetLargestCashWin(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = (long)GetSession(i)->LargestWin;
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetLargestCashLost(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = (double)GetSession(i)->LargestLoss;
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetPeakWinLevel(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = (double)GetSession(i)->PeakWin;
  }
  return true;
}
//*******************************************************************
bool TCasinoStatCalc::GetLowCashLostLevel(float *values,int num)
{
  for (int i=0;i<num;i++)
  {
    values[i] = (double)GetSession(i)->PeakLoss;
  }
  return true;
}
//*******************************************************************

