//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//                   
//       File Name:  casgfw.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//   1.0    9/26/95    M.Perry    original baseline
//   2.0   08/31/97    M.Perry    32-bit conversion.
//********************************************************************
#include "casgfw.h"
#include "cgametyp.h"
#include "cplaymgr.h"
#include "expert.h"
#include "casapp.h"
#include "obdlg.h"
#include "osndmgr.h"
#include "omodewin.h"
#include "ogame.h"
#include "cas_cm.rh"
#include "casmw.h"
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TCasinoGameFrameWindow,TOgleGameFrameWindow)
  EV_COMMAND(CM_FILEGAMBLERMANAGEMENT,CmFileGamblerManagement),
  EV_COMMAND_ENABLE(CM_FILEGAMBLERMANAGEMENT,CeFileGamblerManagement),
  EV_COMMAND(CM_FILEEXITGAME,CmFileExitGame),
  EV_COMMAND_ENABLE(CM_FILEEXITGAME,CeFileExitGame),
  EV_COMMAND(CM_SOUNDFX,CmSoundFX),
  EV_COMMAND_ENABLE(CM_SOUNDFX,CeSoundFX),
  EV_COMMAND(CM_SOUNDMUSIC,CmSoundMusic),
  EV_COMMAND_ENABLE(CM_SOUNDMUSIC,CeSoundMusic),
  EV_COMMAND(CM_HELPCONTENTS,CmHelpContents),
  EV_COMMAND_ENABLE(CM_HELPCONTENTS,CeHelpContents),
  EV_COMMAND(CM_HELPABOUT,CmHelpAbout),
  EV_COMMAND_ENABLE(CM_HELPABOUT,CeHelpAbout),
END_RESPONSE_TABLE;
//*******************************************************************
TCasinoMenuEnable::TCasinoMenuEnable(bool setvalues)
:TOgleMenuEnable(false)
{
  if (setvalues)
  {
    SetAll(true);
  }
  Lobby = false;
  Denom = 0.0;
}
//*******************************************************************
TCasinoMenuEnable::~TCasinoMenuEnable()
{
}
//*******************************************************************
void TCasinoMenuEnable::SetAll(bool value)
{
  GamblerManagement = value;
  Strategy = value;
  Statistics = value;
  Instruction = value;
  Recommendation = value;
  TOgleMenuEnable::SetAll(value);
}
//*******************************************************************
void TCasinoMenuEnable::SetModes(bool value)
{
  Lobby = value;
  TOgleMenuEnable::SetModes(value);
}
//*******************************************************************
//*******************************************************************
void TCasinoGameFrameWindow::CmShowIntro()
{
  ShowIntro = !ShowIntro;
  SetIniShowIntro();
}
//*******************************************************************
void TCasinoGameFrameWindow::CeShowIntro(TCommandEnabler &ce)
{
  if (ShowIntro)
  {
    ce.SetCheck(TCommandEnabler::Checked);
  }
  else
  {
    ce.SetCheck(TCommandEnabler::Unchecked);
  }
}
//*******************************************************************
void TCasinoGameFrameWindow::GetIniShowIntro()
{
  ShowIntro = GetOgleApp()->GetIniFile()->Readbool("Intro","Show",true);
}
//*******************************************************************
void TCasinoGameFrameWindow::SetIniShowIntro()
{
  GetOgleApp()->GetIniFile()->Writebool("Intro","Show",ShowIntro);
}
//*******************************************************************
TCasinoGameFrameWindow::TCasinoGameFrameWindow(TCasinoApp *pcasapp,const char *szTitle,bool shrinktofit,bool doresources)
:TOgleGameFrameWindow(pcasapp,szTitle,shrinktofit,doresources)
{
  FadeIn = false;
  GetIniShowIntro();
  if (doresources)
  {
    DoStyle();
    DoIcon();
    DoMenu();
  }
  StartupModeIndex = -1;
}
//*******************************************************************
TCasinoGameFrameWindow::~TCasinoGameFrameWindow()
{
}
//*******************************************************************
void TCasinoGameFrameWindow::CreateMenuEnable()
{
  PMenuEnable = new TCasinoMenuEnable(true);
}
//*******************************************************************
void TCasinoGameFrameWindow::CreatePlayerMgr()
{
  PlayerMgr = new TCasinoPlayerMgr(this,
                                   GetDriveDir(),
                                   PLAYERMGR_NUMACTIVEPLAYERS,
                                   PLAYERMGR_NUMPLAYERSALLOWED);
}
//*******************************************************************
void TCasinoGameFrameWindow::SwitchGameModes(int16 modeindex)
{
  if (!IsValidModeIndex(modeindex))
  {
	 MessageBox("Internal error, no switch attempted","Switch Mode",MB_OK);
  }
  else
  {
	 if (!SwitchModes(modeindex))
	 {
		MessageBox("Internal error, switching to Walkabout","Switch Mode",MB_OK);
		SwitchModes(StartupModeIndex);
	 }
  }
}
//*******************************************************************
void TCasinoGameFrameWindow::CmFileGamblerManagement()
{
  CmPlayerManagementDlg();
}
//*******************************************************************
void TCasinoGameFrameWindow::CeFileGamblerManagement(TCommandEnabler &enabler)
{
  enabler.Enable(GetCasMenuEnable()->GamblerManagement);
}
//*******************************************************************
void TCasinoGameFrameWindow::CmFileExitGame()
{
  CmExit();
}
//*******************************************************************
void TCasinoGameFrameWindow::CeFileExitGame(TCommandEnabler &enabler)
{
  enabler.Enable(GetCasMenuEnable()->Exit);
}
//*******************************************************************
void TCasinoGameFrameWindow::CmSoundFX()
{
  if (GetSoundMgr() != NULL) {GetSoundMgr()->ToggleAudioMute();}
}
//*******************************************************************
void TCasinoGameFrameWindow::CeSoundFX(TCommandEnabler &enabler)
{
  enabler.Enable(GetCasMenuEnable()->FX);

  if (GetSoundMgr() == NULL || GetSoundMgr()->GetAudioMgr() == NULL)
  {
    enabler.Enable(false);
  }

  if (GetSoundMgr()->IsAudioMuted())
  {
    enabler.SetCheck(TCommandEnabler::Unchecked);
  }
  else
  {
    enabler.SetCheck(TCommandEnabler::Checked);
  }
}
//*******************************************************************
void TCasinoGameFrameWindow::CmSoundMusic()
{
  MessageBox("not imp yet","Toggle Music",MB_OK);
}
//*******************************************************************
void TCasinoGameFrameWindow::CeSoundMusic(TCommandEnabler &enabler)
{
  enabler.Enable(GetCasMenuEnable()->Music);
}
//*******************************************************************
void TCasinoGameFrameWindow::CmHelpContents()
{
  GetOgleApp()->Help(HELP_INDEX);
}
//*******************************************************************
void TCasinoGameFrameWindow::CeHelpContents(TCommandEnabler &enabler)
{
  enabler.Enable(GetCasMenuEnable()->Contents);
}
//*******************************************************************
int TCasinoGameFrameWindow::GetAboutID()
{
  return 0;        
}
//*******************************************************************
void TCasinoGameFrameWindow::CmHelpAbout()
{
  bool waspaused = false;
  if (PModeWin != NULL)
  {
    waspaused = PModeWin->GetUpdatePause();
    PModeWin->SetUpdatePause();
    if (GetSoundMgr() != NULL)
    {
      GetSoundMgr()->Stop(true);
    }
  }
  TBitmap *pbitmap = new TBitmap(*GetOgleApp(),GetAboutID());
  TBitmapDlg *pbd = new TBitmapDlg(this,this,pbitmap);
  pbd->Execute();
  delete pbd;
  delete pbitmap;

  if (PModeWin != NULL && !waspaused)
  {
    PModeWin->ClearUpdatePause();
  }
}
//*******************************************************************
void TCasinoGameFrameWindow::CeHelpAbout(TCommandEnabler &enabler)
{
  enabler.Enable(GetCasMenuEnable()->About);
}
//*******************************************************************
int16 TCasinoGameFrameWindow::GetModeIndex(string &modename)
{
  return (GetGame()->GetModeIndex(modename));
}
//*******************************************************************
string TCasinoGameFrameWindow::GetModeName(int16 mi)
{
  return (GetGame()->GetModeName(mi));
}
//*******************************************************************
TWindow *TCasinoGameFrameWindow::CreateModeWindow(int16 ModeIndex)
// note: should probably be an empty function, now. MRP 4/13/97
{
  if (!IsValidModeIndex(ModeIndex)) {return PModeWin;}

  string modename = GetModeName(ModeIndex);

  ::MessageBox(NULL,"Default mode Win Created","Create Mode Window",MB_OK);
  PModeWin = new TCasinoModeWin(modename,GetOgleApp()->PMasterPalette,string("Default Casino ModeWin"));
  return PModeWin;
}
//*******************************************************************
