//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cplaydat.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     2/27/96    M.Perry    original baseline
//  1.3     5/19/97    M.Perry    added new define's
//  2.0    08/31/97    M.Perry    32-bit conversion.
//********************************************************************
#include "cplaydat.h"
#include <classlib\time.h>
//*******************************************************************
TCasinoPlayerData::TCasinoPlayerData()
{
  ClearData();
}
//*******************************************************************
void TCasinoPlayerData::ClearColumns()
// initially set to 2000.
{
  Columns[0] = 5;
  Columns[1] = 4;
  Columns[2] = 3;
  Columns[3] = 9;
  Columns[4] = 1;
  Columns[5] = 0;
}
//*******************************************************************
void TCasinoPlayerData::ClearData()
{
  TPlayerData::ClearData();
  GameType = gtNone;
  ClearColumns();
  CoachOn = false;

  #if defined(PLAYERDATA_SALARY)
  Salary = 0.0;
  SetCreationSec();
  LastPayedSec = CreationSec;
  #endif

  #if defined(PLAYERDATA_CREDIT)
  Debt = 0.0;
  AvailNow = 0.0;
  CreditLine = 10000.0;
  CashLimit = 0.0;
  MaxDebt = 0.0;
  MinDebt = 0.0;
  #endif

  #if defined(VP_COMPILE)
  VPData.ClearData();
  #endif

  #if defined(BJ_COMPILE)
  BJData.ClearData();
  #endif

  #if defined(ROULETTE_COMPILE)
  RData.ClearData();
  #endif

  #if defined(CRAPS_COMPILE)
  CData.ClearData();
  #endif

  #if defined(SLOTS_COMPILE)
  SData.ClearData();
  #endif

  #if defined(KENO_COMPILE)
  KData.ClearData();
  #endif
}
//*******************************************************************
int16 TCasinoPlayerData::GetColumn(int16 colnum)
{
  if ((colnum >= NUMCOLUMNS) || (colnum < 0)) {return 0;}
  return Columns[colnum];
}
//*******************************************************************
void TCasinoPlayerData::SetColumn(int16 colnum,int16 value)
{
  if ((colnum >= NUMCOLUMNS) || (colnum < 0)) {return;}
  Columns[colnum] = value;
}
//*******************************************************************
#if defined(PLAYERDATA_CREDIT)
void TCasinoPlayerData::CalcAvailNow()
{
  TMoney m = CashLimit - Cash;
  if (m <= 0.0)
  {
    AvailNow = 0.0;
  }
  else
  {
    bool crossover = false;
    int32 an = (int32)m;
    if (an % 1000) {crossover = true;}
    an /= 1000;
    an *= 1000;
    AvailNow = an;
    if (crossover) // basically, round up by a thousand.
    {
      AvailNow += 1000.0;
    }
  }

  // just in case.
  if (Debt >= CreditLine)
  {
    AvailNow = 0.0;
  }

  // also, just in case.
  TMoney diff = CreditLine - Debt;
  if (AvailNow > diff)
  {
    AvailNow = diff;
  }
}
//*******************************************************************
void TCasinoPlayerData::SetMaxMinDebt()
{
  if (Debt > MaxDebt)
  {
    MaxDebt = Debt;
    MinDebt = Debt;
  }
}
//*******************************************************************
void TCasinoPlayerData::SetCashLimit()
{
//  CashLimit = CreditLine / 10.0;
// MRP - 06/08/99 - ver2.2 - ambiguity fix.
//  CashLimit = CreditLine / 5.0;
  CashLimit = double(CreditLine / 5.0);

}
//*******************************************************************
void TCasinoPlayerData::CalcCreditLine()
{
  TMoney ncl = MaxDebt - MinDebt;
  ncl *= 3.0;
  if (ncl > CreditLine)
  {
    CreditLine = ncl;
    SetCashLimit();
    CalcAvailNow();
  }
}
#endif
//*******************************************************************
#if defined(BJ_COMPILE)
void TCasinoPlayerData::StartBJSession(int16 subgametype,TMoney denom)
{
  GameType = gtBlackJack;
  BJData.StartSession(Cash,subgametype,denom);
}
//*******************************************************************
void TCasinoPlayerData::EndBJSession()
{
  BJData.EndSession(Cash);
  GameType = gtNone;
}
#endif
//*******************************************************************
#if defined(SLOTS_COMPILE)
void TCasinoPlayerData::StartSSession(int16 subgametype,TMoney denom)
{
  //GameType = gtBJ;
  GameType = gtSlots;
  SData.StartSession(Cash,subgametype,denom);
}
//*******************************************************************
void TCasinoPlayerData::EndSSession()
{
  SData.EndSession(Cash);
  GameType = gtNone;
}
#endif
//*******************************************************************
#if defined(ROULETTE_COMPILE)
void TCasinoPlayerData::StartRSession(int16 subgametype,TMoney denom)
{
  GameType = gtRoulette;
  RData.StartSession(Cash,subgametype,denom);
}
//*******************************************************************
void TCasinoPlayerData::EndRSession()
{
  RData.EndSession(Cash);
  GameType = gtNone;
}
#endif
//*******************************************************************
#if defined(CRAPS_COMPILE)
void TCasinoPlayerData::StartCSession(int16 subgametype,TMoney denom)
{
  GameType = gtCraps;
  CData.StartSession(Cash,subgametype,denom);
}
//*******************************************************************
void TCasinoPlayerData::EndCSession()
{
  CData.EndSession(Cash);
  GameType = gtNone;
}
#endif
//*******************************************************************
#if defined(VP_COMPILE)
void TCasinoPlayerData::StartVPSession(int16 subgametype,TMoney denom)
{
  GameType = gtVideoPoker;
  VPData.StartSession(Cash,subgametype,denom);
}
//*******************************************************************
void TCasinoPlayerData::EndVPSession()
{
  VPData.EndSession(Cash);
  GameType = gtNone;
}
#endif
//*******************************************************************
#if defined(KENO_COMPILE)
void TCasinoPlayerData::StartKSession(int16 subgametype,TMoney denom)
{
  GameType = gtKeno;
  KData.StartSession(Cash,subgametype,denom);
}
//*******************************************************************
void TCasinoPlayerData::EndKSession()
{
  KData.EndSession(Cash);
  GameType = gtNone;
}
//*******************************************************************
bool TCasinoPlayerData::IsActiveKenoTicket()
{
  return KData.IsActiveTicket();
}
#endif
//*******************************************************************

