//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  cplaymgr.cpp
//    Project Name:  Casino
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     12/13/95   M.Perry    original baseline
//  2.0     08/31/97   M.Perry    32-bit conversion.
//********************************************************************
#include "cplaymgr.h"
#include "casgfw.h"
#include "cstatdlg.h"
#include "casapp.h"
#include "chelpid.h"

#if defined(PLAYERDATA_SALARY)
  #include "csalgdlg.h"
#else
  #include "ogambler.h"
#endif
#define NUMSECSINWEEK 604800l
#define NUMSECSINDAY  86400l
//*******************************************************************
bool TCasinoPlayerMgr::CreatePlayer(string name,bool male,TMoney cash,TMoney salary)
{
  if (!CanCreatePlayer()) {return false;}

  if (DoesNameExist(name)) {return false;}

  int16 playernum = GetOpenPlayerNum();
  if (!CheckPlayerNum(playernum)) {return false;}

  if (!ReadPlayerData(playernum)) {return false;}

  ClearPlayerData(playernum);

  TCasinoPlayerData *pcpd = GetCasinoPlayerDataPtr(playernum);
  if (pcpd == NULL) {return false;}

  pcpd->SetName(name);
  pcpd->SetMale(male);
  pcpd->SetCash(cash);
  pcpd->SetInitialCash(cash);

  #if defined(PLAYERDATA_SALARY)
  pcpd->SetSalary(salary);
  #endif

  pcpd->SetValid(true);
  if (!WritePlayerData(playernum)) {return false;}
  UpdateBasePlayerData(playernum);
  DeletePlayerData(playernum);
  return true;
}
//*******************************************************************
int32 TCasinoPlayerMgr::GetGameDataSize()
{
  return (sizeof(TCasinoGameData));
}
//*******************************************************************
int32 TCasinoPlayerMgr::GetPlayerDataSize()
{
  return (sizeof(TCasinoPlayerData));
}
//*******************************************************************
void TCasinoPlayerMgr::ClearPlayerData(int16 playernum)
{
  if (GetCasinoPlayerDataPtr(playernum) != NULL)
  {
    GetCasinoPlayerDataPtr(playernum)->ClearData();
  }
}
//*******************************************************************
string TCasinoPlayerMgr::GetSaveName()
{
  string s = Parent->GetOgleApp()->GetAppName();
  s += ".sav";
  return s;
}
//*******************************************************************
TCasinoPlayerMgr::TCasinoPlayerMgr(TCasinoGameFrameWindow *parent,string drivedir,
                                   int16 numactiveallowed,int16 numplayers,bool demo)
:TPlayerMgr(parent,drivedir,numactiveallowed,numplayers)
{
  #if defined(ACVP_COMPILE)
  PayDayWeekDay = -1; //everyday
  PayDayIntervalSecs = NUMSECSINDAY;
  #else
  PayDayWeekDay = 5; //friday
  PayDayIntervalSecs = NUMSECSINWEEK;
  #endif
  PayDayHour = 17;

  SetDefPlayerName("Joe Gambler");
  Init();
  if (demo)
  {
    RemovePlayer("Joe Gambler");
    CreateActivateDefPlayer();
  }
  else
  {
    ActivateSavedPlayers();
    if (GetNumActivePlayers() == 0) {CreateActivateDefPlayer();}
  }
}
//*******************************************************************
TCasinoPlayerMgr::~TCasinoPlayerMgr()
{
  Save();
}
//*******************************************************************
void TCasinoPlayerMgr::DoManagementDialog()
{
  UpdateAllActiveBasePlayerData();

  #if defined(PLAYERDATA_SALARY)
  TSalariedGamblerDlg *pgd = new TSalariedGamblerDlg(Parent,this,IDD_SALARIEDGAMBLER,IDH_CAS_GAMBLER);
  pgd->Execute(MB_TASKMODAL);
  #else
  TGamblerDialog *pgd = new TGamblerDialog(Parent,IDD_GAMBLER,this,IDH_CAS_GAMBLER);//,false);
  pgd->Execute();
  #endif
  delete pgd;
}
//*******************************************************************
void TCasinoPlayerMgr::DoStatisticsDialog(int16 playernum,TWindow *parent)
{
  TWindow *pwin = parent;
  if (pwin == NULL) {pwin = Parent;}
  TCasinoStatDlg *pstatdlg = new TCasinoStatDlg(pwin,this,playernum);
  pstatdlg->Execute();
  delete pstatdlg;
  ReleaseUnActivePlayerData(); // unloads any player data that was loaded by stats dlg.
}
//*******************************************************************
TGameData *TCasinoPlayerMgr::GetNewGameData()
{
  TCasinoGameData *pgd = new TCasinoGameData;
  return pgd;
}
//*******************************************************************
TPlayerData *TCasinoPlayerMgr::GetNewPlayerData()
{
  TCasinoPlayerData *ppd = new TCasinoPlayerData;
  return ppd;
}
//*******************************************************************
void TCasinoPlayerMgr::CreateDefPlayer()
{
  if (!IsPlayerValid(strDefPlayerName))
  {
    TMoney m(500.0);
    CreatePlayer(strDefPlayerName,true,m);
  }
}
//*******************************************************************
void TCasinoPlayerMgr::CreateActivateDefPlayer()
{
  CreateDefPlayer();
  ActivatePlayer(strDefPlayerName);
}
//*******************************************************************
void TCasinoPlayerMgr::ModeDeActivated()
{
  Save();
}
//*******************************************************************
#pragma argsused
void TCasinoPlayerMgr::PrePlayerActivation(int16 playernum,TWindow *pwin)
{
  #if defined(PLAYERDATA_SALARY)
  TWindow *dlgparent = pwin;
  if (dlgparent == NULL)
  {
    dlgparent = Parent;
  }
  if (Parent->IsSplashScreenActive())  // don't check if at startup.
  {
    dlgparent = NULL;
  }
    // check for paydays.
  CheckForPaydays(playernum,dlgparent,true);
  #endif
}
//*******************************************************************
#if defined(PLAYERDATA_SALARY)
void TCasinoPlayerMgr::CheckForPaydays(int16 playernum,TWindow *pwin,bool onactivation)
{
  TCasinoPlayerData *pdat = GetCasinoPlayerDataPtr(playernum);
  if (pdat == NULL) {return;}
  if (pdat->GetSalary() == 0.0) {return;} // don't bother if no salary.

  TTime lasttime(pdat->LastPayedSec);
  TTime now;
  int32 numpaydays = 0;

  TDate lastdate(lasttime);

  // start of set iter at next payday & paytime
  TTime timeiter(lastdate,GetPaydayHour()); // start at last day payed but at paytime
  TDate td(timeiter);

  // find a payday.
  int16 days = 0;
  if (GetPaydayWeekDay() == -1)
  {
    days = 0;
  }
  else if (td.WeekDay() <= GetPaydayWeekDay())
  {
    days = int16(GetPaydayWeekDay() - td.WeekDay());
  }
  else
  {
    days = int16(7 - td.WeekDay());
    days += GetPaydayWeekDay();
  }
  uint32 daysinsecs = days * NUMSECSINDAY;
  timeiter += daysinsecs;
  // end of set iter at next payday & paytime

  if (timeiter < lasttime)  // if payday & paytime is before last payed then jump a interval.
  {
    timeiter += PayDayIntervalSecs;//NUMSECSINWEEK;
  }
  if (timeiter > now) {return;}  // if payday & paytime is beyond current time then bye.

  while (timeiter <= now)        // count number of paydays up to now.
  {
    numpaydays++;
    timeiter += PayDayIntervalSecs;//NUMSECSINWEEK; // inc a week at a time.
  }
  if (numpaydays > 0)                // add it up, add it up, see what you got. "Violent Femmes"
  {
    TMoney salary = pdat->GetSalary();
    salary *= numpaydays;
    pdat->Cash += salary;
    pdat->LastPayedSec = now.Seconds();

    if (pwin == NULL) {return;}    // don't show any messages, splash screen showing.

    // start text
    string msg;
    if (onactivation)
    {
      msg = pdat->GetName();
      msg += " is being paid\n";
      msg += salary.GetStr();
      msg += "\nin income since";
      if (pdat->Male)
      {
        msg += " he last gambled";
      }
      else
      {
        msg += " she last gambled";
      }
    }
    else
    {
      msg = "It's 5:00 pm on PayDay!\n";//Friday!\n";
      msg += salary.GetStr();
      msg += "\nwill be added to ";
      msg += pdat->GetName();
      msg += "'s bankroll.";
    }
    // end text;

    // start title
    string strtitle;
    if (onactivation)
    {
      strtitle = "Income!!!";
    }
    else
    {
      strtitle = "Payday!!!";
    }
    // end title
    TSalariedGamblerMsgDlg *psgmd = new TSalariedGamblerMsgDlg(pwin,
                                                               IDD_SALARIEDGAMBLER_MSG,
                                                               msg,
                                                               strtitle);
    psgmd->Execute();
    delete psgmd;
  }
}
#endif
//*******************************************************************
#if defined(PLAYERDATA_SALARY)
void TCasinoPlayerMgr::CheckActivePaydays()
{
  for (int16 i=0;i<GetNumActiveAllowed();i++)
  {
    int16 playernum = GetActivePlayerNum(i);
    if (playernum != -1)
    {
      CheckForPaydays(playernum,Parent,false);
    }
  }
}
#endif
//*******************************************************************
