/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// gridfram.cpp                                                            //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.50 - 25 nov 1995                       //
// TGrid - Copyright  1994,1995 H. Picot, All Rights Reserved.            //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#include "griddll.h"
#pragma hdrstop
#include "grid.rh"


// Frame3d ------------------------------------------------------------------
void Frame3d (TDC &dc, TRect &rc, int w, TColor topleft, TColor bottomright)
   {
   Frame3d (dc, rc.left, rc.top, rc.right-1, rc.bottom-1, w, topleft, bottomright);
   }


void Frame3d (TDC &dc, int xorg, int y0, int x1, int y1, int w, TColor topleft, TColor bottomright)
   {
   for (int i = 0; i < w; i++)
      {
      dc.SelectObject (TPen (topleft));
      dc.MoveTo (x1, y0);
      dc.LineTo (xorg, y0);
      dc.LineTo (xorg, y1);
      dc.RestorePen ();

      dc.SelectObject (TPen (bottomright));
      dc.LineTo (x1, y1);
      dc.LineTo (x1, y0);
      dc.RestorePen ();
      if (i < w-1)
         {
         xorg++;
         y0++;
         x1--;
         y1--;
         }
      }
   }


// ------------------------------------------------------------------------//
// TGridClient                                                             //
// ------------------------------------------------------------------------//
DEFINE_RESPONSE_TABLE1 (TGridClient, TControl)
   EV_WM_CTLCOLOR,
   EV_WM_ERASEBKGND,
   EV_WM_KILLFOCUS,
   EV_WM_LBUTTONDOWN,
   EV_WM_LBUTTONUP,
   EV_WM_LBUTTONDBLCLK,
   EV_WM_RBUTTONDOWN,
   EV_WM_RBUTTONUP,
   EV_WM_RBUTTONDBLCLK,
   EV_WM_SETFOCUS,
END_RESPONSE_TABLE;

TGridClient::TGridClient (TGrid *g)
   : TControl (g, -1, 0, 0, 0, 0, 0, 0),
     grid (g)
   {
   Attr.Style &= (~WS_TABSTOP);
   Attr.Style |= WS_BORDER|WS_CLIPSIBLINGS;
   }


bool TGridClient::Create ()
   {
   bool rc = TControl::Create ();
   if (rc)
      grid->CalcPos ();
   return rc;
   }


char *TGridClient::GetClassName()
   {
   #ifdef __WIN32__
   return "TGridClient32";
   #else
   return "TGridClient16";
   #endif
   }


void TGridClient::Paint (TDC &dc, bool erase, TRect &clip)
   {
   if (grid->layout == aForm)
      dc.FillRect (clip, grid->formBkg);

   grid->PaintClient (dc, erase, clip);
   }


HBRUSH TGridClient::EvCtlColor(HDC hDC, HWND hWndChild, uint ctlType)
   {
   return grid->CtlColor (hDC, hWndChild, ctlType);
   }


void TGridClient::EvLButtonDown (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_LBUTTONDOWN, true);
   }


void TGridClient::EvLButtonUp (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_LBUTTONUP, true);
   }


void TGridClient::EvLButtonDblClk (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_LBUTTONDBLCLK, true);
   }


void TGridClient::EvRButtonDown (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_RBUTTONDOWN, true);
   }


void TGridClient::EvRButtonUp (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_RBUTTONUP, true);
   }
                

void TGridClient::EvRButtonDblClk (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_RBUTTONDBLCLK, true);
   }


void TGridClient::EvKillFocus (HWND hWndGetFocus)
   {
   grid->EvKillFocus (hWndGetFocus);
   }


void TGridClient::EvSetFocus (HWND hwndLooseFocus)
   {
   grid->EvSetFocus (hwndLooseFocus);
   }


LRESULT TGridClient::WindowProc(uint msg, WPARAM wParam, LPARAM lParam)
   {
   // handle WM_SETCURSOR here because BC chokes on the EV_WM_SETCURSOR line in the response table
   if (msg == WM_SETCURSOR)
      EvSetCursor (HWND(wParam), LOWORD(lParam), HIWORD(lParam));

   // Pass all commands through to TGrid itself, so that EV_COMMAND entries can be handled by TGrid directly
   else if (msg == WM_COMMAND)
      Parent->SendMessage (msg, wParam, lParam);

   // default processing
   return TControl::WindowProc (msg, wParam, lParam);
   }


bool TGridClient::EvSetCursor (HWND hwnd, uint hitTest, uint)
   {
   if (hwnd == HWindow && hitTest == HTCLIENT)
      {
      TPoint p;
      GetCursorPos (p);
      ScreenToClient (p);
      if (GetClientRect().Contains(p) && grid->SetCursorShape (p))
         return true;
      }

   SetCursor (0,0);  // mouse quitting the control
   return true; /*bool(DefaultProcessing());*/
   }


// ------------------------------------------------------------------------//
// TGridColumn                                                             //
// ------------------------------------------------------------------------//
TGridColumn::TGridColumn (TControl *c, TGridColumn *prev, TGrid *g, const char *t)
   : next (0),
     control (c),
     width(0),
     left(0),
     align (Left),
     grid (g),
     title (t),
     font (0),
     Hidden(0),                     // Charles R. Cordell
     label (0)
   {
   if (prev)
      prev->next = this;
   previous = prev;
   }


TGridColumn::~TGridColumn ()
   {
   if (font)
      delete font;
   if (next)
      delete next;
   }


void TGridColumn::HideControl ()
   {
   if (control && !Hidden && grid->layout == aGrid)          // Charles R. Cordell
      {
      if (type == gCombo)
         {
         TComboBox *cb = dynamic_cast<TComboBox *>(control);
         if (cb->GetDroppedState ())
            cb->HideList ();
         }

      control->SetWindowPos (0, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_HIDEWINDOW);
      if (dynamic_cast<TGComboBox *>(control))
         {
         int y = grid->lineHeight*grid->activeLine;
         TRect rc (TPoint (left, y-1), TSize (width, grid->lineHeight));
         grid->client->InvalidateRect (rc, false);
         }
      }
   }


void TGridColumn::ShowControl ()
   {
   if (control && !control->IsWindowVisible() && grid->nLines > 0 && !Hidden) // Charles R. Cordell
      {
      if (grid->layout == aGrid)
         {
         int x = left;
         int y = grid->lineHeight * grid->activeLine;
         int h = (type == gCombo) ? dynamic_cast<TGComboBox *>(control)->totalHeight : grid->lineHeight-1;
         int w = (type == gCombo) ? width-1 : width;

         // place check boxes in the center of the column if they have no text and a center alignment was required
         if (type == gCheck && align == Center)
            {
            int dx = (width-14)/2;       // 14 is the pixel width of a check box
            x += dx;
            w = 14;
            }
         control->SetWindowPos (0, 0,0,0,0, SWP_NOZORDER|SWP_SHOWWINDOW|SWP_NOSIZE|SWP_NOMOVE);
         control->SetWindowPos (HWND_TOP, x, y, w, h, 0);
         }
      else
         {
         int h = (type == gCombo) ? dynamic_cast<TGComboBox *>(control)->totalHeight : control->Attr.H;
         int id = control->GetId ();
         int line = grid->GetCurrentLine ();

         switch (grid->VisibleCell (line, id))
            {
            case gControl:
            case gFlat:
               control->SetWindowPos (0, control->Attr.X, control->Attr.Y, control->Attr.W, h, SWP_NOZORDER|SWP_SHOWWINDOW);
               break;

            case gLocked:
            case gEmpty:
               control->SetWindowPos (0, control->Attr.X, control->Attr.Y, control->Attr.W, h, SWP_NOZORDER|SWP_HIDEWINDOW);
               break;
            }

         if (!Hidden)
            {
            void *data = const_cast<void *>(grid->TransferItem (line, id));
            if (data)
               {
               #if __BORLANDC__ >= 0x500
               if (TGBitmap *bmp = dynamic_cast<TGBitmap *>(control))
                  {
                  TClientDC dc (*control);
                  TRect rc (control->GetClientRect ());
                  control->Transfer (data, tdSetData);
                  bmp->DrawCell (dc, rc, data, Left);
                  }
               else if (TGBitmapFile *bmp2 = dynamic_cast<TGBitmapFile *>(control))
                  {
                  TClientDC dc (*control);
                  TRect rc (control->GetClientRect ());
                  control->Transfer (data, tdSetData);
                  bmp2->DrawCell (dc, rc, data, Left);
                  }
               #else
               if (dynamic_cast<TGBitmap *>(control))
                  {
                  TClientDC dc (*control);
                  TRect rc (control->GetClientRect ());
                  control->Transfer (data, tdSetData);
                  dynamic_cast<TGBitmap *>(control)->DrawCell (dc, rc, data, Left);
                  }
               else if (dynamic_cast<TGBitmapFile *>(control))
                  {
                  TClientDC dc (*control);
                  TRect rc (control->GetClientRect ());
                  control->Transfer (data, tdSetData);
                  dynamic_cast<TGBitmapFile *>(control)->DrawCell (dc, rc, data, Left);
                  }
               #endif
               else if (type == gCombo)
                  dynamic_cast<TGComboBox *>(control)->SetData (data);
               else if (type == gButton)
                  dynamic_cast<TGButton *>(control)->SetCaption ((char *)data);
               else
                  control->Transfer (data, tdSetData);
               }
            }
         }
      }
   }


bool TGridColumn::SetFocus ()
   {
   if (!grid->IsWindowEnabled ())
      return false;

   if (control && type != gStatic && control->IsWindow () && control->IsWindowEnabled () && !Hidden) // Charles R. Cordell
      {
      bool isTabStop = (control->Attr.Style & WS_TABSTOP) ? true : false;
      bool self = (grid->current == this) ? true : false;
      int line = grid->GetCurrentLine ();
      TGVisible v = grid->VisibleCell (line, control->GetId());
      bool locked = (v == gEmpty || v == gLocked);
      if (!locked && isTabStop)
         {
         bool undo = false;
         if (grid->current && !self && grid->current->control->IsWindowVisible ())
            {
            grid->current->HideControl ();
            undo = true;
            }
         ShowControl ();
         if (control->SetFocus ())
            {
            ShowControl ();
            grid->current = this;
            if (grid->layout == aGrid)
               grid->client->UpdateWindow ();

            // check if control outside of clienta area: scroll

            return true;
            }
         else
            {
            if (grid->current && !self && undo)
               grid->current->ShowControl ();
            if (!self)
               HideControl ();
            if (grid->layout == aGrid)
               grid->client->UpdateWindow ();
            }
         }
      }
   return false;
   }


bool TGridColumn::CanKillFocus ()
   {
   if (grid->current == 0)
      return true;
   #if __BORLANDC__ >= 0x500
   if (TGEdit *c = dynamic_cast<TGEdit *>(grid->current->control))
   #else
   TGEdit *c = dynamic_cast<TGEdit *>(grid->current->control);
   if (c)
   #endif
      return c->IsValid(true); 
   else
      return true;
   }


// return the width of control owned by column --------------------------- //
int TGridColumn::CtrlWidth () const
   {
   if( Hidden )          // Charles R. Cordell
     return 0;           // Charles R. Cordell

   switch (type)
      {
      case gTimeEdit:
         return dynamic_cast<TGTimeEdit *>(control)->Width();
      case gDateEdit:
         return dynamic_cast<TGDateEdit *>(control)->Width();
      case gShortEdit:
      case gLongEdit:
      case gDoubleEdit:
      case gEdit:
         return dynamic_cast<TGEdit *>(control)->Width();
      case gStatic:
         return dynamic_cast<TGStatic *>(control)->Width();
      case gCheck:
         return dynamic_cast<TGCheckBox*>(control)->Width();
      case gCombo:
         return dynamic_cast<TGComboBox*>(control)->Width();
      case gButton:
         return dynamic_cast<TGButton*>(control)->Width();
      }
   return 0;
   }


// return the height of control owned by column -------------------------- //
int TGridColumn::CtrlHeight () const
   {
   switch (type)
      {
      case gTimeEdit:
         return dynamic_cast<TGTimeEdit *>(control)->Height();
      case gDateEdit:
         return dynamic_cast<TGDateEdit *>(control)->Height();
      case gShortEdit:
      case gLongEdit:
      case gDoubleEdit:
      case gEdit:
         return dynamic_cast<TGEdit *>(control)->Height();
      case gStatic:
         return dynamic_cast<TGStatic *>(control)->Height();
      case gCheck:
         return dynamic_cast<TGCheckBox*>(control)->Height();
      case gCombo:
         return dynamic_cast<TGComboBox*>(control)->Height();
      case gButton:
         return dynamic_cast<TGButton*>(control)->Height();
      }
   return 0;
   }


void TGridColumn::SetTitle (const char *newtitle)
   {
   title = newtitle;
   if (grid->title && left < grid->clientWidth && left+width >= 0)
      {
      TRect rc (left+1, 1, left+width, grid->titleHeight);
      grid->title->InvalidateRect (rc, false);
      }
   }


void TGridColumn::SetFont (const char *facename, int height, int weight, bool italic)
   {
   if (facename || height)
      {
      LOGFONT lf;
      if (grid->gridLogFont)
         memcpy (&lf, grid->gridLogFont, sizeof(lf));
      else
         memset (&lf, 0, sizeof(lf));
      strcpy (lf.lfFaceName, facename);
      if (height)
         lf.lfHeight = height;
      lf.lfWeight = weight;
      lf.lfItalic = BYTE(italic);
      font = new TFont (&lf);
      }
   }


void TGridColumn::SetFont ()
   {
   if (control && control->IsWindow ())
      control->SetWindowFont (HFONT(font ? *font : *grid->gridFont), true);
   }


char *TGridColumn::GetClassName()
   {
   #ifdef __WIN32__
   return "TGridColumn32";
   #else
   return "TGridColumn16";
   #endif
   }


// ------------------------------------------------------------------------//
// TGridVScroll                                                            //
// ------------------------------------------------------------------------//
DEFINE_RESPONSE_TABLE1 (TGridVScroll, TScrollBar)
   EV_WM_KEYDOWN,
END_RESPONSE_TABLE;

TGridVScroll::TGridVScroll (TGrid *g)
     : TScrollBar (g, -1, 0, 0, 0, 0, false),
       grid (g),
       minRange (0),
       maxRange (100),
       factor (1)
   {
   Attr.Style &= (~WS_TABSTOP);
   }


void TGridVScroll::EvKeyDown (uint key, uint repeat, uint flags)
   {
   switch (key)
      {
      case VK_SPACE:
         grid->SelectLine (grid->GetCurrentLine ());
         return;

      case VK_RETURN:
         if (grid->KeyDown (VK_RETURN, repeat))
            return;
         break;
      }

   TScrollBar::EvKeyDown (key, repeat, flags);
   }


void TGridVScroll::SetLRange (uint min, uint max, bool draw)
   {
   minRange = min;
   maxRange = max;
   if (maxRange >= 0x8000u)
      factor = (maxRange+0x8000u) / 0x8000u;
   if (factor < 1)
      factor = 1;
   ::SetScrollRange (*this, SB_CTL, min/factor, max/factor, draw);
   }


void TGridVScroll::SetLPosition (uint pos)
   {
   if (factor < 1)
      factor = 1;
   TScrollBar::SetPosition (pos/factor);
   }


void TGridVScroll::SBBottom ()
   {
   grid->Go (grid->nLines-1);
   }


void TGridVScroll::SBLineDown ()
   {
   grid->Move (1);
   }


void TGridVScroll::SBLineUp ()
   {
   grid->Move (-1);
   }


void TGridVScroll::SBPageDown ()
   {
   grid->Move (grid->visibleLines);
   }


void TGridVScroll::SBPageUp ()
   {
   grid->Move (-grid->visibleLines);
   }


void TGridVScroll::SBThumbPosition (int pos)
   {
   if (factor < 1)
      factor = 1;
   grid->Go (pos*factor);
   }


void TGridVScroll::SBThumbTrack (int pos)
   {
   if (factor < 1)
      factor = 1;
   grid->Go (pos*factor);
   }
               

void TGridVScroll::SBTop ()
   {
   grid->Go (0);
   }


// ------------------------------------------------------------------------//
// TGridHScroll                                                            //
// ------------------------------------------------------------------------//
TGridHScroll::TGridHScroll (TGrid *g)
     : TScrollBar (g, -1, 0, 0, 0, 0, true),
       grid (g)
   {
   Attr.Style &= (~WS_TABSTOP);
   }


void TGridHScroll::SBBottom ()
   {
   int min, max;
   GetRange (min, max);
   grid->SetHPos (max);
   }


void TGridHScroll::SBLineDown ()
   {
   grid->MoveHPos (LineMagnitude);
   }


void TGridHScroll::SBLineUp ()
   {
   grid->MoveHPos (-LineMagnitude);
   }


void TGridHScroll::SBPageDown ()
   {
   grid->MoveHPos (PageMagnitude);
   }


void TGridHScroll::SBPageUp ()
   {
   grid->MoveHPos (-PageMagnitude);
   }


void TGridHScroll::SBThumbPosition (int pos)
   {
   if (grid->hpos != pos)
      grid->SetHPos (pos);
   }


void TGridHScroll::SBThumbTrack (int pos)
   {
   if (grid->hpos != pos)
      grid->SetHPos (pos);
   }


void TGridHScroll::SBTop ()
   {
   if (grid->hpos != 0)
      grid->SetHPos (0);
   }


// ------------------------------------------------------------------------//
// TGridTitle                                                              //
// ------------------------------------------------------------------------//
DEFINE_RESPONSE_TABLE1 (TGridTitle, TControl)
   EV_WM_LBUTTONDOWN,
   EV_WM_LBUTTONUP,
   EV_WM_LBUTTONDBLCLK,
   EV_WM_RBUTTONDOWN,
   EV_WM_RBUTTONUP,
   EV_WM_RBUTTONDBLCLK,
   EV_WM_MOUSEMOVE,
END_RESPONSE_TABLE;

TGridTitle::TGridTitle (TGrid *g, bool show)
   : TControl (g, -1, 0, 0, 0, 0, 0, 0),
     grid (g),
     capture (false),
     visible (show)
   {
   Attr.Style &= (~WS_BORDER);  
   Attr.Style &= (~WS_TABSTOP);
   }


LRESULT TGridTitle::WindowProc(uint msg, WPARAM wParam, LPARAM lParam)
   {
   if (visible && msg == WM_SETCURSOR)
      EvSetCursor (HWND(wParam), LOWORD(lParam), HIWORD(lParam));

   return TControl::WindowProc (msg, wParam, lParam);
   }


bool TGridTitle::Create ()
   {
   bool rc = TControl::Create ();
   if (rc)
      grid->CalcPos ();
   return rc;
   }


char *TGridTitle::GetClassName()
   {
   #ifdef __WIN32__
   return "TGridTitle32";
   #else
   return "TGridTitle16";
   #endif
   }


void TGridTitle::GetWindowClass(WNDCLASS &wc)
   {
   TControl::GetWindowClass(wc);
   wc.style &= ~CS_GLOBALCLASS;
   wc.hCursor = 0;
   }


int TGridTitle::CalcHeight ()
   {
   if (!visible)
      return 0;

   int h;
   bool del = false;
   TFont *f = grid->titleFont;
   if (f == 0)
      {
      if (grid->titleLogFont)
         {
         f = new TFont (grid->titleLogFont);
         del = true;
         }
      else
         {
         f = grid->gridFont;
         if (f == 0)
            {
            f = new TFont (grid->gridLogFont);
            del = true;
            }
         }
      }
   TScreenDC dc;
   dc.SelectObject (*f);
   TEXTMETRIC tm;
   dc.GetTextMetrics (tm);
   dc.RestoreFont ();
   h = tm.tmHeight + 6;  // minimum
   if (del)
      delete f;
   if (h < 10)
      h = 10;
   return h;
   }


void TGridTitle::Paint (TDC &dc, bool, TRect &clip)
   {
   if (!visible)
      return;

   if (grid->titleFont)
      SetWindowFont (HFONT(*grid->titleFont), true);

   dc.FillRect (clip, TBrush (grid->GetTitleColor (TitleBackground)));
   dc.SelectClipRgn (TRegion (clip));
   TFont *font = grid->titleFont;
   if (font == 0)
      font = grid->gridFont;
   dc.SelectObject (*font);
   TPen pen (grid->GetTitleColor (TitleFrame));

   for (TGridColumn *ctrl = grid->firstColumn; ctrl; ctrl = ctrl->next)
      {
      if (ctrl->Hidden)    // Charles R. Cordell
         continue;         // Charles R. Cordell

      if (ctrl->previous)
         {
         dc.SelectObject (pen);
         dc.MoveTo (ctrl->left, 0);
         dc.LineTo (ctrl->left, Attr.H);
         dc.RestorePen ();
         }

      TGAlign align = grid->centerTitle ? Center : ctrl->align;
      DrawCell (dc, ctrl->left, ctrl->width, ctrl->title.c_str(), align);
      }

   dc.SelectObject (pen);
   dc.MoveTo (0, 0);
   dc.LineTo (0, Attr.H);
   dc.MoveTo (Attr.W-1, 0);
   dc.LineTo (Attr.W-1, Attr.H);
   dc.MoveTo (0,0);
   dc.LineTo (Attr.W, 0);
   dc.RestorePen ();

   dc.RestoreFont ();
   }


void TGridTitle::DrawCell (TDC &dc, int x, int w, const char *p, TGAlign a)
   {
   w = x+w-1;
   int h = Attr.H-1;
   Frame3d (dc, x+1, 1, w, h, 1, grid->GetTitleColor (TitleHilite), grid->GetTitleColor (TitleShadow));

   if (p)
      {
      TRect rc (x+2, 2, w-1, h-1);
      dc.SetTextColor (grid->GetTitleColor (TitleText));
      dc.SetBkColor (grid->GetTitleColor (TitleBackground));
      dc.DrawText (p, -1, rc, (WORD)(a|DT_SINGLELINE|DT_VCENTER));
      }
   }


bool TGridTitle::EvSetCursor (HWND hwnd, uint hitTest, uint)
   {
   if (visible && hwnd == HWindow && hitTest == HTCLIENT)
      {
      TPoint p;
      GetCursorPos (p);
      ScreenToClient (p);
      if (GetClientRect ().Contains (p))
         {
         if (MoveMouseCursor ())
            {
            #ifndef _GRIDLIB
               SetCursor (GridModule, HRESIZE);
            #else
               SetCursor (GetModule(), HRESIZE);
            #endif
            }
         else
            SetCursor (0, IDC_ARROW);
         return true;
         }
      }

   SetCursor (0,0);  // mouse quitting the control
   return true;
   }


bool TGridTitle::MoveMouseCursor ()
   {
   if (grid->sizeColumn == false)
      return false;

   TPoint p;
   GetCursorPos (p);
   ScreenToClient (p);
   int x = p.x+1;

   for (TGridColumn *e = grid->firstColumn; e; e = e->next)
      {
      if (e->Hidden)
         continue;
      int right = e->left+e->width;
      if (x >= right-4 && x <= right+4)
         {
         column = e;
         return true;
         }
      else if (x >= e->left && x < right)
         return false;
      }

   return false;
   }


void TGridTitle::EvLButtonDown (uint modKeys, TPoint &p)
   {
   TControl::EvLButtonDown (modKeys, p);
   if (visible)
      {
      if (capture == false && MoveMouseCursor () && column)
         {
         xorg = p.x;
         SetCapture ();
         capture = true;
         if (grid->current && grid->current->control->IsWindowVisible ())
            {
            showBack = true;
            grid->current->HideControl ();
            }
         else
            showBack = false;
         }
      else
         {
         MapWindowPoints (*grid, &p, 1);
         grid->ProcessMouse (modKeys, p, WM_LBUTTONDOWN, false);
         }
      }
   }


void TGridTitle::EvLButtonUp (uint modKeys, TPoint &p)
   {
   TControl::EvLButtonUp (modKeys, p);
   if (visible)
      {
      if (capture)
         {
         ReleaseCapture ();
         capture = false;
         grid->client->Invalidate ();
         if (showBack)
            grid->current->ShowControl ();
         }
      else
         {
         MapWindowPoints (*grid, &p, 1);
         grid->ProcessMouse (modKeys, p, WM_LBUTTONUP, false);
         }
      }
   }


void TGridTitle::EvLButtonDblClk (uint modKeys, TPoint &p)
   {
   MapWindowPoints (*grid, &p, 1);
   grid->ProcessMouse (modKeys, p, WM_LBUTTONDBLCLK, false);
   }


void TGridTitle::EvRButtonDblClk (uint modKeys, TPoint &p)
   {
   MapWindowPoints (*grid, &p, 1);
   grid->ProcessMouse (modKeys, p, WM_RBUTTONDBLCLK, false);
   }


void TGridTitle::EvRButtonDown   (uint modKeys, TPoint &p)
   {
   MapWindowPoints (*grid, &p, 1);
   grid->ProcessMouse (modKeys, p, WM_RBUTTONDOWN, false);
   }


void TGridTitle::EvRButtonUp     (uint modKeys, TPoint &p)
   {
   MapWindowPoints (*grid, &p, 1);
   grid->ProcessMouse (modKeys, p, WM_RBUTTONUP, false);
   }


void TGridTitle::EvMouseMove (uint modKeys, TPoint &p)
   {
   TControl::EvMouseMove (modKeys, p);
   if (visible && capture)
      {
      if (p.x < column->left+8)
         p.x = column->left+8;
      grid->ChangeColWidth (column, p.x-xorg);
      xorg = p.x;
      }
   }


// ------------------------------------------------------------------------//
// TGridVerticalBar                                                        //
// ------------------------------------------------------------------------//
DEFINE_RESPONSE_TABLE1 (TGridVerticalBar, TControl)
   EV_WM_LBUTTONDOWN,
   EV_WM_LBUTTONUP,
   EV_WM_LBUTTONDBLCLK,
   EV_WM_RBUTTONDOWN,
   EV_WM_RBUTTONUP,
   EV_WM_RBUTTONDBLCLK,
END_RESPONSE_TABLE;

TGridVerticalBar::TGridVerticalBar (TGrid *g)
   : TControl (g, -1, 0, 0, 0, 0, 0, 0),
     grid (g),
     lineNumber (true),
     charWidth (4)
   {
   Attr.Style |= WS_BORDER;
   Attr.Style &= (~WS_TABSTOP);
   }


bool TGridVerticalBar::Create ()
   {
   bool rc = TControl::Create ();
   if (rc)
      grid->CalcPos ();
   return rc;
   }


int TGridVerticalBar::CalcWidth ()
   {
   int w = 16;
   bool del = false;
   TFont *f = grid->titleFont;
   if (f == 0)
      f = grid->gridFont;
   if (f == 0)
      {
      f = new TFont (grid->gridLogFont);
      del = true;
      }
   if (f)
      {
      TScreenDC dc;
      dc.SelectObject (*f);
      TEXTMETRIC tm;
      dc.GetTextMetrics (tm);
      dc.RestoreFont ();
      w = tm.tmAveCharWidth * charWidth + 8;
      }
   if (del)
      delete f;
   if (w < 16)
      w = 16;   // default
   return w;
   }


void TGridVerticalBar::SetUserText (int cwidth)
   {
   if (!IsWindow ())
      {
      lineNumber = false;
      if (cwidth == 0)
         cwidth = 4;
      charWidth = cwidth;
      }
   }


void TGridVerticalBar::Paint (TDC &dc, bool, TRect &clip)
   {
   if (grid->titleFont)
      SetWindowFont (HFONT(*grid->titleFont), true);

   dc.SelectClipRgn (TRegion (clip));
   dc.FillRect (clip, TBrush (grid->GetTitleColor (TitleBackground)));

   int lh = (grid->layout == aGrid ? grid->lineHeight : grid->Attr.H);
   int x0 = 0;
   int x1 = Attr.W;
   int y0 = 0;
   int y1 = lh-1;
   TPen pen (grid->GetTitleColor (TitleFrame));
   for (int i = 0; i < grid->visibleLines; i++, y0 += lh, y1 += lh)
      {
      TRect rc (x0, y0, x1, y1);
      if (rc.Touches (clip))
         {
         DrawCell (dc, rc, i);
         dc.SelectObject (pen);
         dc.MoveTo (0, rc.bottom);
         dc.LineTo (Attr.W, rc.bottom);
         dc.RestorePen ();
         }
      }
   }


const char *TGridVerticalBar::Info (int line) const
   {
   static char string[10];
   const char *info;
   if (lineNumber)
      {
      sprintf (string, "%d", grid->firstLine+line+1);
      info = string;
      }
   else
      info = static_cast<const char *>(grid->TransferItem (grid->firstLine+line, 0));

   return info;
   }


void TGridVerticalBar::DrawCell (TDC &dc, TRect &rc, int line)
   {
   TRect clip = dc.GetClipBox ();
   dc.SelectClipRgn (TRegion (rc));
   Frame3d (dc, rc, 1, grid->GetTitleColor (TitleHilite), grid->GetTitleColor (TitleShadow));
   if (line == grid->activeLine)
      {
      if (grid->lineMarker != ArrowOnly)
         ShowText (line);
      ShowMarker (line);
      }
   else
      ShowText (line);
   dc.SelectClipRgn (TRegion (clip));
   }


void TGridVerticalBar::ShowText (int line) const
   {
   TClientDC dc (HWindow);
   TRect rc = FillRect (dc, line, true);
   if (grid->lineMarker == ReverseColor)
      {
      TRect rc1 (rc);
      rc1.Inflate (2, 2);
      Frame3d (dc, rc1, 1, grid->GetTitleColor (TitleHilite),
                           grid->GetTitleColor (TitleShadow));
      }

   const char *info = Info (line);
   if (info)
      {
      TFont *font = grid->titleFont;
      if (font == 0)
         font = grid->gridFont;
      dc.SelectObject (*font);
      dc.SetTextColor (grid->GetTitleColor (TitleText));
      dc.SetBkColor (grid->GetTitleColor (TitleBackground));
      dc.DrawText (info, -1, rc, (WORD)(DT_RIGHT|DT_VCENTER|DT_SINGLELINE));
      dc.RestoreFont ();
      }
   }


void TGridVerticalBar::ShowMarker (int line) const
   {
   TClientDC dc (HWindow);
   TRect rc = FillRect (dc, line, false);
   if (grid->lineMarker == ReverseColor)
      {
      if (grid->layout == aGrid)
         {
         TRect rc1 (rc);
         rc1.Inflate (2, 2);
         dc.InvertRect (rc1);
         }
      }
   else
      {
      dc.SelectObject (TPen (grid->GetTitleColor (TitleText)));
      int x = rc.left+4;
      int y0 = rc.top+2;
      int y1 = rc.bottom-2;
      while (y0 < y1 && x < rc.right)
         {
         dc.MoveTo (x, y0++);
         dc.LineTo (x++, y1--);
         }
      dc.RestorePen ();
      }
   }


TRect TGridVerticalBar::FillRect (TDC &dc, int line, bool erase) const
   {
   int lh = grid->lineHeight;
   int y = line*lh;
   TRect rc (1, y+1, Attr.W-3, y+lh-2);
   if (erase || grid->lineMarker == ArrowOnly)
      dc.FillRect (rc, TBrush(grid->GetTitleColor (TitleBackground)));
   return rc.InflatedBy(-1,-1);
   }

   
void TGridVerticalBar::EvLButtonDown (uint modKeys, TPoint &point)
   {
   if (grid->lineHeight > 0)
      {
      // calc cell where mouse was clicked
      switch (grid->lineSelection)
         {
         case Single:
         case Multiple:
         case SingleAutoSelect:
            grid->SelectLine ((point.y / grid->lineHeight) + grid->firstLine, modKeys | MK_TGRID);
            break;

         case None:
            break;
         }
      } 

   if (grid->current)
      grid->current->HideControl ();
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_LBUTTONDOWN, false);
   }


void TGridVerticalBar::EvLButtonUp (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_LBUTTONUP, false);
   }


void TGridVerticalBar::EvLButtonDblClk (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_LBUTTONDBLCLK, false);
   }


void TGridVerticalBar::EvRButtonDown (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_RBUTTONDOWN, false);
   }


void TGridVerticalBar::EvRButtonUp (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_RBUTTONUP, false);
   }


void TGridVerticalBar::EvRButtonDblClk (uint modKeys, TPoint &point)
   {
   MapWindowPoints (*grid, &point, 1);
   grid->ProcessMouse (modKeys, point, WM_RBUTTONDBLCLK, false);
   }


char *TGridVerticalBar::GetClassName()
   {
   #ifdef __WIN32__
   return "TGridVerticalBar32";
   #else
   return "TGridVerticalBar16";
   #endif
   }


// EOF
