#include "db.h"
//*******************************************************************
#define TIMEOUT 60  // one minute ???
//*******************************************************************
class TGateway
{
  TIPAddress;
  TIPPort;
  int16 Status;
};
//*******************************************************************
class TGWList
{
  TGateway *Get(TIPAddressPort addrport);
  bool Add(TGateway *pgateway);
};
//*******************************************************************
class TGS_CEHandler
{
  string XOut(string fi_accnum);

  bool Process(TCEMsg *pcemsg);

  bool Process_Core(TCEMsg *pcemsg);
  bool Process_Core_GS_Address(TCEMsg *pcemsg);
  bool Process_Core_GS_Status(TCEMsg *pcemsg);
  bool Process_Core_Default(TCEMsg *pcemsg);

  bool Process_User(TCEMsg *pcemsg);
  bool Process_User_Login(TCEMsg *pcemsg);
  bool Process_User_NewLogin(TCEMsg *pcemsg);
  bool Process_User_Logoff(TCEMsg *pcemsg);
  bool Process_User_SwitchGames(TCEMsg *pcemsg);
  bool Process_User_SetFreePlay(TCEMsg *pcemsg);
  bool Process_User_SetPassword(TCEMsg *pcemsg);
  bool Process_User_GetAvailGT(TCEMsg *pcemsg);
  bool Process_User_GetAvailMID(TCEMsg *pcemsg);
  bool Process_User_GetSysInfo(TCEMsg *pcemsg);
  bool Process_User_GetSysMsg(TCEMsg *pcemsg);
  bool Process_User_GetAccount(TCEMsg *pcemsg);
  bool Process_User_AddAccount(TCEMsg *pcemsg);
  bool Process_User_RemoveAccount(TCEMsg *pcemsg);
  bool Process_User_GetWallet(TCEMsg *pcemsg);
  bool Process_User_GetGameInfo(TCEMsg *pcemsg);
  bool Process_User_Default(TCEMsg *pcemsg);

  bool Process_TX(TCEMsg *pcemsg);
  bool Process_TX_TX(TCEMsg *pcemsg);
  bool Process_TX_GetAvail(TCEMsg *pcemsg);
  bool Process_TX_Get(TCEMsg *pcemsg);
  bool Process_TX_GetAllowed(TCEMsg *pcemsg);
  bool Process_TX_Default(TCEMsg *pcemsg);

  bool Process_Slots(TCEMsg *pcemsg);
  bool Process_Slots_Spin(TCEMsg *pcemsg);
  bool Process_Slots_BetOne(TCEMsg *pcemsg);
  bool Process_Slots_BetMax(TCEMsg *pcemsg);
  bool Process_Slots_CashOut(TCEMsg *pcemsg);
  bool Process_Slots_GetInfo(TCEMsg *pcemsg);
  bool Process_Slots_Default(TCEMsg *pcemsg);

  bool Process_BJ(TCEMsg *pcemsg);
  bool Process_VP(TCEMsg *pcemsg);
  bool Process_Roulette(TCEMsg *pcemsg);

  bool Process_Default(TCEMsg *pcemsg);
};
//*******************************************************************
class TGSApp
{
  TGSDB DB;
  int16 State;
  TIPAddr IPAddr;
  TIPPort IPPort;
  TDBUser GS_User;
  TPassword GS_PW;
  TDBID DBID;
  bool Startup();
  bool Shutdown();
};
//*******************************************************************
bool TGSApp::Startup()
{
  DB.Login(GS_User,GS_PW,DBID);
  DB.GS.Login(IPAddr,ec);
}
//*******************************************************************
bool TGSApp::Shutdown()
{
  DB.GS.Logoff(IPAddr,ec);
  DB.Logoff();
}
//*******************************************************************
void TPeriodicHandler::Process()
{
  DB.GS.GetGWArray();
  //check GWList for conflicts.
}
//*******************************************************************
bool TGS_CEHandler::Process(TCEMsg *pcemsg)
{
  bool rc = false;

  TGateway *pw = PApp->GWList.Get(pcemsg->GetConnectionID());
  if (pw && pw->GetAddress() == pcemsg->GetAddress())
  {
    switch (pcemsg->GetMT())
    {
      case MT_CORE     : rc = Process_Core(pcemsg);break;
      case MT_USER     : rc = Process_User(pcemsg);break;
      case MT_TX       : rc = Process_TX(pcemsg);break;
      case MT_SLOTS    : rc = Process_Slots(pcemsg);break;
      case MT_BJ       : rc = Process_BJ(pcemsg);break;
      case MT_VP       : rc = Process_VP(pcemsg);break;
      case MT_ROULETTE : rc = Process_Roulette(pcemsg);break;
      default          : rc = Process_Default(pcemsg);break;
    };
  }

  if (!rc)
  {
    delete pcemsg;
  }

  return rc;
}
//*******************************************************************
bool TGS_CEHandler::Process_Core(TCEMsg *pcemsg)
{
  switch (pcemsg->GetSMT())
  {
    case SMT_CORE_GS_ADDRESS : return Process_Core_GS_Address(pcemsg);break;
    case SMT_CORE_GS_STATUS  : return Process_Core_GS_Status(pcemsg);break;
    default                  : return Process_Core_Default(pcemsg);break;
  };
}
//*******************************************************************
bool TGS_CEHandler::Process_Core_GS_Address(TCEMsg *pcemsg)
{
  if (pcemsg->GetDMT() == DMT_REQUEST)
  {
    TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
    if (!pgw) {return false;}

    pcemsg->SetGSAddress(PApp->GetAddress());
    pcemsg->SetGSStatus(PApp->GetStatus());
    pcemsg->SetDMT(DMT_REPLY);
    pgw->Send(pcemsg);
    return true;
  }
  else
  {
    return false;
  }
}
//*******************************************************************
bool TGS_CEHandler::Process_Core_GS_Status(TCEMsg *pcemsg)
{
  if (pcemsg->GetDMT() == DMT_REQUEST)
  {
    TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
    if (!pgw) {return false;}

    pcemsg->SetGSStatus(PApp->GetStatus());
    pcemsg->SetDMT(DMT_REPLY);
    pgw->Send(pcemsg);
    return true;
  }
  else
  {
    return false;
  }
}
//*******************************************************************
bool TGS_CEHandler::Process_Core_Default(TCEMsg *pcemsg)
{
  if (pcemsg->GetDMT() == DMT_REQUEST)
  {
    TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
    if (!pgw) {return false;}

    pcemsg->SetDMT(DMT_REPLY);
    pcemsg->SetErrorCode(EC_SMT_UNKNOWN);
    pgw->Send(pcemsg);
    return true;
  }
  else
  {
    return false;
  }
}
//*******************************************************************
bool TGS_CEHandler::Process_User(TCEMsg *pcemsg)
{
  switch (pcemsg->GetSMT())
  {
    case SMT_USER_LOGIN         : return Process_User_Login(pcemsg);break;
    case SMT_USER_NEWLOGIN      : return Process_User_NewLogin(pcemsg);break;
    case SMT_USER_LOGOFF        : return Process_User_Logoff(pcemsg);break;
    case SMT_USER_SWITCHGAMES   : return Process_User_SwitchGames(pcemsg);break;
    case SMT_USER_SETFREEPLAY   : return Process_User_SetFreePlay(pcemsg);break;
    case SMT_USER_SETPASSWORD   : return Process_User_SetPassword(pcemsg);break;
    case SMT_USER_GETAVAILGT    : return Process_User_GetAvailGT(pcemsg);break;
    case SMT_USER_GETAVAILMID   : return Process_User_GetAvailMID(pcemsg);break;
    case SMT_USER_GETSYSINFO    : return Process_User_GetSysInfo(pcemsg);break;
    case SMT_USER_GETSYSMSG     : return Process_User_GetSysMsg(pcemsg);break;
    case SMT_USER_GETACCOUNT    : return Process_User_GetAccount(pcemsg);break;
    case SMT_USER_ADDACCOUNT    : return Process_User_AddAccount(pcemsg);break;
    case SMT_USER_REMOVEACCOUNT : return Process_User_RemoveAccount(pcemsg);break;
    case SMT_USER_GETWALLET     : return Process_User_GetWallet(pcemsg);break;
    case SMT_USER_GETGAMEINFO   : return Process_User_GetGameInfo(pcemsg):break;
    default                     : return Process_User_Default(pcemsg);break;
  };
}
//*******************************************************************
bool TGS_CEHandler::Process_User_Login(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS username,pw,gwid,mustchange,gt,mid,uan,ec;

  username = pcemsg->GetUsername();
  pw = pcemsg->GetPassword();
  gwid = pcemsg->GetGWID();

  PGSApp->DB.GS.User_Login(username,pw,gwid,mustchange,gt,mid,uan,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetMustChangePW(mustchange);
  pcemsg->SetGT(gt);
  pcemsg->SetMID(mid);
  pcemsg->SetUAN(uan);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_NewLogin(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS username,pw,gwid,fname,lname,street,city,state,country,zip,
       email,dayphone,eveningphone,dob,sex_male,mothersmaiden,hometown,
       uan,gt,mid,modun1,modun2,modun3,modun4,ec;

  username = pcemsg->GetUserName();
  pw = pcemsg->GetPassword();
  gwid = pcemsg->GetGWID();
  fname = pcemsg->GetFName();
  lname = pcemsg->GetLName();
  street = pcemsg->GetStreet();
  city = pcemsg->GetCity();
  state = pcemsg->GetState();
  country = pcemsg->GetCountry();
  zip = pcemsg->GetZip();
  email = pcemsg->GetEMail();
  dayphone = pcemsg->GetDayPhone();
  eveningphone = pcemsg->GetEveningPhone();
  dob = pcemsg->GetDOB();
  sex_male = pcemsg->GetSex_Male();
  mothersmaiden = pcemsg->GetMothersMaiden();
  hometown = pcemsg->GetHometown();

  PGSApp->DB.GS.User_NewLogin(username,pw,gwid,fname,lname,street,city,state,
                              country,zip,email,dayphone,eveningphone,dob,
                              sex_male,mothersmaiden,hometown,uan,gt,mid,modun1,
                              modun2,modun3,modun4,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetUAN(uan);
  pcemsg->SetGT(gt);
  pcemsg->SetMID(mid);
  pcemsg->SetModUN1(modun1);
  pcemsg->SetModUN2(modun2);
  pcemsg->SetModUN3(modun3);
  pcemsg->SetModUN4(modun4);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_Logoff(TCEMsg *pcemsg)
{
  TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,endcassession,ec;

  uan = pcemsg->GetUAN();
  endcassession = pcemsg->GetEndCasSession();

  PGSApp->DB.GS.User_Logoff(uan,endcassession,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_SwitchGames(TCEMsg *pcemsg)
{
  TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,endgamesession,gt,mid,ec;

  uan = pcemsg->GetUAN();
  endgamesession = pcemsg->GetEndGameSession();
  gt = pcemsg->GetGT();
  mid = pcemsg->GetMID();

  PGSApp->DB.GS.User_SwitchGames(uan,endgamesession,gt,mid,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetGT(gt);
  pcemsg->SetMID(mid);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_SetFreePlay(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,free,ec;

  uan = pcemsg->GetUAN();
  free = pcemsg->GetFreePlay();

  PGSApp->DB.GS.User_SetFree(uan,free,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetFreePlay(free);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_SetPassword(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,oldpw,newpw,ec;

  uan = pcemsg->GetUAN();
  oldpw = pcemsg->GetOldPW();
  newpw = pcemsg->GetNewPW();

  PGSApp->DB.GS.User_SetPassword(uan,oldpw,newpw,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetNewPW(newpw);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_GetAvailGT(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,gt_array,gt_num,ec,;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.User_GetAvailGT(uan,gt_array,gt_num,ec);

  pcemsg->SetDMT(DMT_REPLY);
  if (gt_num >= 1) {pcemsg->SetGT1(gt_array[0]);}
  if (gt_num >= 2) {pcemsg->SetGT2(gt_array[1]);}
  if (gt_num >= 3) {pcemsg->SetGT3(gt_array[2]);}
  if (gt_num >= 4) {pcemsg->SetGT4(gt_array[3]);}
  if (gt_num >= 5) {pcemsg->SetGT5(gt_array[4]);}

  delete [] gt_array;
  
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_GetAvailMID(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,gt,mid_array,mid_num,ec;

  uan = pcemsg->GetUAN();
  gt = pcemsg->GetGT();

  PGSApp->DB.GS.User_GetAvailMID(uan,gt,mid_array,mid_num,ec);

  pcemsg->SetDMT(DMT_REPLY);
  if (mid_num >= 1)  {pcemsg->SetMID1(mid_array[0]);}
  if (mid_num >= 2)  {pcemsg->SetMID2(mid_array[1]);}
  if (mid_num >= 3)  {pcemsg->SetMID3(mid_array[2]);}
  if (mid_num >= 4)  {pcemsg->SetMID4(mid_array[3]);}
  if (mid_num >= 5)  {pcemsg->SetMID5(mid_array[4]);}
  if (mid_num >= 6)  {pcemsg->SetMID6(mid_array[5]);}
  if (mid_num >= 7)  {pcemsg->SetMID7(mid_array[6]);}
  if (mid_num >= 8)  {pcemsg->SetMID8(mid_array[7]);}
  if (mid_num >= 9)  {pcemsg->SetMID9(mid_array[8]);}
  if (mid_num >= 10) {pcemsg->SetMID10(mid_array[9]);}
  if (mid_num >= 11) {pcemsg->SetMID11(mid_array[10]);}
  if (mid_num >= 12) {pcemsg->SetMID12(mid_array[11]);}
  if (mid_num >= 13) {pcemsg->SetMID13(mid_array[12]);}
  if (mid_num >= 14) {pcemsg->SetMID14(mid_array[13]);}
  if (mid_num >= 15) {pcemsg->SetMID15(mid_array[14]);}
  if (mid_num >= 16) {pcemsg->SetMID16(mid_array[15]);}
  if (mid_num >= 17) {pcemsg->SetMID17(mid_array[16]);}
  if (mid_num >= 18) {pcemsg->SetMID18(mid_array[17]);}
  if (mid_num >= 19) {pcemsg->SetMID19(mid_array[18]);}
  if (mid_num >= 20) {pcemsg->SetMID20(mid_array[19]);}
  if (mid_num >= 21) {pcemsg->SetMID21(mid_array[20]);}
  if (mid_num >= 22) {pcemsg->SetMID22(mid_array[21]);}
  if (mid_num >= 23) {pcemsg->SetMID23(mid_array[22]);}
  if (mid_num >= 24) {pcemsg->SetMID24(mid_array[23]);}
  if (mid_num >= 25) {pcemsg->SetMID25(mid_array[24]);}

  delete [] mid_array;

  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_GetSysInfo(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,phone,email,weburl,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.User_GetSysInfo(uan,phone,email,weburl,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetCasPhone(phone);
  pcemsg->SetCasEMail(email);
  pcemsg->SetCasWebUrl(weburl);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_GetSysMsg(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,gt,m1,m2,m3,m4,m5,ec;

  uan = pcemsg->GetUAN();
  gt = pcemsg->GetGT();

  PGSApp->DB.GS.User_GetSysMsg(uan,gt,m1,m2,m3,m4,m5,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetMsg1(m1);
  pcemsg->SetMsg2(m2);
  pcemsg->SetMsg3(m3);
  pcemsg->SetMsg4(m4);
  pcemsg->SetMsg5(m5);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
string TGS_CEHandler::XOut(string fi_accnum)
{
  string s fi_accnum;
  // x out all but the last 4 digits.
  for (int i=0;i < (s.length() - 4);i++)
  {
    s[i] = 'X';
  }
  return s;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_GetAccount(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,bid,ac,asc,fi_accnum,nameonacc,expdate,street,city,state,country,
       zip,descr,bankname,abanum,ec;
  uan = pcemsg->GetUAN();
  bid = pcemsg->GetBID();

  PGSApp->DB.GS.User_GetAccount(uan,???,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetBankName(bankname);
  pcemsg->SetExpDate(expdate);
  pcemsg->SetABANum(abanum);
  pcemsg->SetDescr(descr);
  pcemsg->SetAC(ac);
  pcemsg->SetASC(asc);
  fi_accnum = XOut(fi_accnum);
  pcemsg->SetFI_AccNum(fi_accnum);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_AddAccount(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,ac,asc,fi_accnum,nameonacc,expdate,street,city,state,country,
       zip,bankname,abanum,bid,ec;

  uan = pcemsg->GetUAN();
  ac = pcemsg->GetAC();
  asc = pcemsg->GetASC();
  fi_accnum = pcemsg->GetFI_AccNum();
  nameonacc = pcemsg->GetNameOnAcc();
  expdate = pcemsg->GetExpDate();
  street = pcemsg->GetStreet();
  city = pcemsg->GetCity();
  state = pcemsg->GetState();
  country = pcemsg->GetCountry();
  zip = pcemsg->GetZip();
  bankname = pcemsg->GetBankName();
  abanum = pcemsg->GetABANum();

  PGSApp->DB.GS.User_AddAccount(uan,???,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetBID(bid);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_RemoveAccount(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,bid,ec;

  uan = pcemsg->GetUAN();
  bid = pcemsg->GetBID();

  PGSApp->DB.GS.User_RemoveAccount(uan,bid,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_GetWallet(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,balance,chips,freechips,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.User_GetWallet(uan,balance,chips,freechips,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetBalance(balance);
  pcemsg->SetChips(chips);
  pcemsg->SetFreeChips(freechips);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_GetGameInfo(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,gt,mid,ec;

  uan = pcemsg->GetUAN();
  gt = pcemsg->GetGT();
  mid = pcemsg->GetMID();

  PGSApp->DB.GS.User_GetGameInfo(uan,gt,mid,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetGT(gt);
  pcemsg->SetMID(mid);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_User_Default(TCEMsg *pcemsg)
{
  PApp->Counters.Sys_Unk++;

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetErrorCode(EC_GS_SYS_SMT_UNK);

  TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}
  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_TX(TCEMsg *pcemsg)
{
  switch (pcemsg->GetSMT())
  {
    case SMT_TX_TX          : return Process_TX_TX(pcemsg);break;
    case SMT_TX_GETAVAIL    : return Process_TX_GetAvail(pcemsg);break;
    case SMT_TX_GET         : return Process_TX_Get(pcemsg);break;
    case SMT_TX_GETALLOWED  : return Process_TX_GetAllowed(pcemsg);break;
    default                 : return Process_TX_Default(pcemsg);break;
  };
}
//*******************************************************************
bool TGS_CEHandler::Process_TX_TX(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,bid,txt,txd,amount,balance,chips,ec;

  uan = pcemsg->GetUAN();
  bid = pcemsg->GetBID();
  txt = pcemsg->GetTXT();
  txd = pcemsg->GetTXD();
  amount = pcemsg->GetAmount();

  PGSApp->DB.GS.TX(uan,bid,txt,txd,amount,balance,chips,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetBalance(balance);
  pcemsg->SetChips(chips);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_TX_GetAvail(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,atx_array,atx_num,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.TX_GetAvail(uan,atx_array,atx_num,ec);

  pcemsg->SetDMT(DMT_REPLY);
  if (atx_num >= 1) {pcemsg->SetTXAvail1(atx_array[0]);}
  if (atx_num >= 2) {pcemsg->SetTXAvail2(atx_array[1]);}
  if (atx_num >= 3) {pcemsg->SetTXAvail3(atx_array[2]);}
  if (atx_num >= 4) {pcemsg->SetTXAvail4(atx_array[3]);}
  if (atx_num >= 5) {pcemsg->SetTXAvail5(atx_array[4]);}
  if (atx_num >= 6) {pcemsg->SetTXAvail6(atx_array[5]);}
  if (atx_num >= 7) {pcemsg->SetTXAvail7(atx_array[6]);}
  if (atx_num >= 8) {pcemsg->SetTXAvail8(atx_array[7]);}
  if (atx_num >= 9) {pcemsg->SetTXAvail9(atx_array[8]);}
  if (atx_num >= 10){pcemsg->SetTXAvail10(atx_array[9];}
  delete [] atx_array;

  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_TX_Get(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,tx_array,tx_num,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.TX_Get(uan,tx_array,tx_num,ec);

  pcemsg->SetDMT(DMT_REPLY);
  if (tx_num >= 1) {pcemsg->SetTX1(tx_array[0]);}
  if (tx_num >= 2) {pcemsg->SetTX2(tx_array[1]);}
  if (tx_num >= 3) {pcemsg->SetTX1(tx_array[2]);}
  if (tx_num >= 4) {pcemsg->SetTX1(tx_array[3]);}
  if (tx_num >= 5) {pcemsg->SetTX1(tx_array[4]);}
  if (tx_num >= 6) {pcemsg->SetTX1(tx_array[5]);}
  if (tx_num >= 7) {pcemsg->SetTX1(tx_array[6]);}
  if (tx_num >= 8) {pcemsg->SetTX1(tx_array[7]);}
  if (tx_num >= 9) {pcemsg->SetTX1(tx_array[8]);}
  if (tx_num >= 10) {pcemsg->SetTX1(tx_array[9]);}
  if (tx_num >= 11) {pcemsg->SetTX1(tx_array[10]);}
  if (tx_num >= 12) {pcemsg->SetTX1(tx_array[11]);}
  if (tx_num >= 13) {pcemsg->SetTX1(tx_array[12]);}
  if (tx_num >= 14) {pcemsg->SetTX1(tx_array[13]);}
  if (tx_num >= 15) {pcemsg->SetTX1(tx_array[14]);}
  if (tx_num >= 16) {pcemsg->SetTX1(tx_array[15]);}
  if (tx_num >= 17) {pcemsg->SetTX1(tx_array[16]);}
  if (tx_num >= 18) {pcemsg->SetTX1(tx_array[17]);}
  if (tx_num >= 19) {pcemsg->SetTX1(tx_array[18]);}
  if (tx_num >= 20) {pcemsg->SetTX1(tx_array[19]);}
  if (tx_num >= 21) {pcemsg->SetTX1(tx_array[20]);}
  if (tx_num >= 22) {pcemsg->SetTX1(tx_array[21]);}
  if (tx_num >= 23) {pcemsg->SetTX1(tx_array[22]);}
  if (tx_num >= 24) {pcemsg->SetTX1(tx_array[23]);}
  if (tx_num >= 25) {pcemsg->SetTX1(tx_array[24]);}
  if (tx_num >= 26) {pcemsg->SetTX1(tx_array[25]);}
  if (tx_num >= 27) {pcemsg->SetTX1(tx_array[26]);}
  if (tx_num >= 28) {pcemsg->SetTX1(tx_array[27]);}
  if (tx_num >= 29) {pcemsg->SetTX1(tx_array[28]);}
  if (tx_num >= 30) {pcemsg->SetTX1(tx_array[29]);}
  delete [] tx_array;

  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_TX_GetAllowed(TCEMsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,fac_array,fac_num,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.TX_GetAllowed(uan,fac_array,fac_num,ec);

  pcemsg->SetDMT(DMT_REPLY);
  if (fac_num >= 1) {pcemsg->SetFAC1(fac_array[0]);}
  if (fac_num >= 2) {pcemsg->SetFAC1(fac_array[1]);}
  if (fac_num >= 3) {pcemsg->SetFAC1(fac_array[2]);}
  if (fac_num >= 4) {pcemsg->SetFAC1(fac_array[3]);}
  if (fac_num >= 5) {pcemsg->SetFAC1(fac_array[4]);}
  if (fac_num >= 6) {pcemsg->SetFAC1(fac_array[5]);}
  if (fac_num >= 7) {pcemsg->SetFAC1(fac_array[6]);}
  if (fac_num >= 8) {pcemsg->SetFAC1(fac_array[7]);}
  if (fac_num >= 9) {pcemsg->SetFAC1(fac_array[8]);}
  if (fac_num >= 10) {pcemsg->SetFAC1(fac_array[9]);}
  if (fac_num >= 11) {pcemsg->SetFAC1(fac_array[10]);}
  if (fac_num >= 12) {pcemsg->SetFAC1(fac_array[11]);}
  if (fac_num >= 13) {pcemsg->SetFAC1(fac_array[12]);}
  if (fac_num >= 14) {pcemsg->SetFAC1(fac_array[13]);}
  if (fac_num >= 15) {pcemsg->SetFAC1(fac_array[14]);}
  delete [] fac_array;

  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_TX_Default(TCEMsg *pcemsg)
{
}
//*******************************************************************
bool TGS_CEHandler::Process_Slots(TCEMsg *pcemsg)
{
  bool rc = false;
  switch (pcemsg->GetSMT())
  {
    case SMT_SLOTS_SPIN    : rc = Process_Slots_Spin(pcemsg);break;
    case SMT_SLOTS_BETONE  : rc = Process_Slots_BetOne(pcemsg);break;
    case SMT_SLOTS_BETMAX  : rc = Process_Slots_BetMax(pcemsg);break;
    case SMT_SLOTS_CASHOUT : rc = Process_Slots_Cashout(pcemsg);break;
    case SMT_SLOTS_GETINFO : rc = Process_Slots_GetInfo(pcemsg);break;
    default                : rc = Process_Slots_Default(pcemsg);break;
  };
  return rc;
}
//*******************************************************************
bool TGS_CEHandler::Process_Slots_Spin(TCEMsg *pcemsg)
{
  TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,coinsplayed,credits,chips,winnerpaid,payoffhilitenum,rs1,rs2,rs3,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.Slots_Spin(uan,coinsplayed,credits,chips,winnerpaid,
                           payoffhilitenum,rs1,rs2,rs3,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetCoinsPlayed(coinsplayed);
  pcemsg->SetCredits(credits);
  pcemsg->SetChips(chips);
  pcemsg->SetWinnerPaid(winnerpaid);
  pcemsg->SetPayoffHiLiteNum(payoffhilitenum);
  pcemsg->SetReelStop1(rs1);
  pcemsg->SetReelStop2(rs2);
  pcemsg->SetReelStop3(rs3);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_Slots_BetOne(TCEMsg *pcemsg)
{
  TGateway *pgw = PApp->GwList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,coinsplayed,credits,chips,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.Slots_BetOne(uan,coinsplayed,credits,chips,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetCoinsPlayed(coinsplayed);
  pcemsg->SetCredits(credits);
  pcemsg->SetChips(chips);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_Slots_BetMax(TCEMsg *pcemsg)
{
  TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,coinsplayed,credits,chips,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.Slots_BetMax(uan,coinsplayed,credits,chips,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetCoinsPlayed(coinsplayed);
  pcemsg->SetCredits(credits);
  pcemsg->SetChips(chips);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_Slots_Cashout(TCEMsg *pcemsg)
{
  TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,coinsplayed,credits,chips,ec);

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.Slots_Cashout(uan,coinsplayed,credits,chips,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetCoinsPlayed(coinsplayed);
  pcemsg->SetCredits(credits);
  pcemsg->SetChips(chips);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_Slots_GetInfo(TCEmsg *pcemsg)
{
  TGateway *pgw = PGSApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  VARS uan,mid,coinsplayed,credits,chips,winnerpaid,
       payoffhilitenum,rs1,rs2,rs3,ec;

  uan = pcemsg->GetUAN();

  PGSApp->DB.GS.Slots_GetInfo(uan,mid,coinsplayed,credits,chips,winnerpaid,
                              payoffhilitenum,rs1,rs2,rs3,ec);

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetMID(mid);
  pcemsg->SetCoinsPlayed(coinsplayed);
  pcemsg->SetCredits(credits);
  pcemsg->SetChips(chips);
  pcemsg->SetWinnerPaid(winnerpaid);
  pcemsg->SetPayoffHiLiteNum(payoffhilitenum);
  pcemsg->SetReelStop1(rs1);
  pcemsg->SetReelStop2(rs2);
  pcemsg->SetReelStop3(rs3);
  pcemsg->SetErrorCode(ec);

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_Slots_Default(TCEMsg *pcemsg)
{
  PApp->Counters.Slots_Unk++;

  pcemsg->SetDMT(DMT_REPLY);
  pcemsg->SetErrorCode(EC_SLOTS_SMT_UNK);

  TGateway *pgw = PApp->GWList.Get(pcemsg->GetGWAddress());
  if (!pgw) {return false;}

  pgw->Send(pcemsg);
  return true;
}
//*******************************************************************
bool TGS_CEHandler::Process_BJ(TCEMsg *pcemsg)
{
}
//*******************************************************************
bool TGS_CEHandler::Process_VP(TCEMsg *pcemsg)
{
}
//*******************************************************************
bool TGS_CEHandler::Process_Roulette(TCEMsg *pcemsg)
{
}
//*******************************************************************
