#include "etmsg.h"
#include "cemsg.h"
//*******************************************************************
class TPublicKey
{
  // stuff
  string GetKey(); // ???
};
//*******************************************************************
class TPrivateKey
{
  // stuff
};
//*******************************************************************
class TIPAddressPort
{
  TIPAddress Address;
  TIPPort Port;
};
//*******************************************************************
class TClient
{
public:
  TIPAddressPort AddrPort;
  int16 ConnectionID;
  bool Connected;
  bool Handshaked;
  bool LoggedIn;
  int16 UAN;
  string Username;
  TPublicKey Client_PublicKey;
  TPublicKey GW_PublicKey;
  TPrivateKey GW_PrivateKey;
  TAddrPort Client_AddPort;
  uint8 Client_MajVer;
  uint8 Client_MinVer;
  int16 UnknownRequestCount;

public:
  void CreateGWKeys();
};
//*******************************************************************
class TClientList : public TList
{
  // list of TUser objects.
  TClient *GetFirst();
  TClient *GetNext();
  TClient *Get(int16 connectionid);
  TClient *Get(string username);
  TClient *Get(int16 UAN);
};
//*******************************************************************
class TGameServer
{
  int16 ConnectionID;
  bool Connected;
  TIPAddressPort AddrPort;
};
//*******************************************************************
class TGSList : public TList
{
  // list of TGameServer objects.
  TGameServer *GetFirst();
  TGameServer *GetNext();
};
//*******************************************************************
class TGatewayApp
{
  TGatewayID GatewayID;
  int16 Version;
  TClientList ClientList;
  TGSList GSList;
  bool Startup();
  bool Shutdown();
};
//*******************************************************************
class TGW_ETHandler
{
  bool Process(TETMsg *petmsg);
  bool Process_Handshake_Request(TETMsg *petmsg);
  bool Process_Encrypted_CE(petmsg);
};
//*******************************************************************
class TGW_CEHandler
{
  void Process(TCEMsg *pcemsg);
  bool Process_Core(TCEMsg *pcemsg);
  bool Process_Core_GS_Address(TCEMsg *pcemsg);
  bool Process_Core_GS_Status(TCEMsg *pcemsg);
  bool Process_Core_Default(TCEMsg *pcemsg);
  bool Process_User(TCEMsg *pcemsg);
  bool Process_User_Login(TCEMsg *pcemsg);
  bool Process_User_NewLogin(TCEMsg *pcemsg);
  bool Process_User_Logoff(TCEMsg *pcemsg);
  bool Process_User_Default(TCEMsg *pcemsg);
  bool Process_Default(TCEMsg *pcemsg);
};
//*******************************************************************
//*******************************************************************
bool TGatewayApp::Startup()
{
  // listen at inner net for GS msgs.
  // broadcast GS Address Request Msg at inner net.
  // listen at outer net for client msgs.
  return true;
}
//*******************************************************************
bool TGatewayApp::Shutdown()
{
  // disconnect all clients.
  // disconnect all GS's.
  return true;
}
//*******************************************************************
//*******************************************************************
bool TGW_ETHandler::Process(TETMsg *petmsg)
{
  bool rc = false;
  switch (petmsg->GetMT())
  {
    case ET_MT_HANDSHAKE_REQ : rc = Process_Handshake_Request(petmsg);break;
    case ET_MT_HEARBEAT_REQ  : rc = Process_Heartbeat_Request(petmsg);break;
    case ET_MT_HEARTBEAT_REP : rc = Process_Heartbeat_Reply(petmsg);break;
    case ET_MT_ENCRYPTED_CE  : rc = Process_Encrypted_CE(petmsg);break;
  }
  if (!rc)
  {
    delete petmsg;
  }
  return rc;
}
//*******************************************************************
bool TGW_ETHandler::Process_Handshake_Request(TETMsg *petmsg)
{
  if (petmsg->GetMT() != ET_MT_HANDSHAKE_REQ) {return false;}

  TClient *pclient = PApp->ClientList.Get(petmsg->GetConnectionID());
  if (!pclient) {return false;}

  if (!pclient->Handshaked)
  {
    pclient->Handshaked = true;
    pclient->Client_MajVer = petmsg->Get_HSReq_Client_MajVer;
    pclient->Client_MinVer = petmsg->Get_HSReq_Client_MinVer;
    pclient->Client_PublicKey = petmsg->Get_HSReq_PublicKey;
    pclient->CreateGWKeys();
    TETMsg *preply = new TETMsg(pclient->ConnectionID,pclient->GW_PublicKey,Version,EC_NONE);
    pclient->Send(preply);
  }
  else
  {
    TETMsg *preply2 = new TETMsg(pclient->ConnectionID,NULL,Version,EC_HANDSHAKE_ALREADY_DONE);
    pclient->Send(preply2);
  }
  return false;
}
//*******************************************************************
bool TGW_ETHandler::Process_Encrypted_CE(petmsg)
{
  if (petmsg->GetMT() != ET_MT_ENCRYPTED_CE) {return false;}

  TClient *pclient = PApp->ClientList.Get(petmsg->GetConnectionID());
  if (!pclient) {return false;}

  if (!pclient->Handshaked) {return false;}

  TCEMsg *pcemsg = new TCEMsg(petmsg); // handles decrypting msg.
  PApp->GetCEQueue()->Add(pcemsg);
  return false;
}
//*******************************************************************
//*******************************************************************
void TGW_CEHandler::Process(TCEMsg *pcemsg)
{
  bool rc = false;
  switch (pcemsg->GetMT())
  {
    case MT_CORE : rc = Process_Core(pcemsg);break;
    case MT_USER : rc = Process_User(pcemsg);break;
    default      : rc = Proces_Default(pcemsg);break;
  };

  if (!rc)
  {
    delete pcemsg;
  }
}
//*******************************************************************
bool TGW_CEHandler::Process_Core(TCEMsg *pcemsg)
{
  switch (pcemsg->GetSMT())
  {
    case SMT_CORE_GS_ADDRESS  : return Process_Core_GS_Address(pcemsg);break;
    case SMT_CORE_GS_STATUS   : return Process_Core_GS_Status(pcemsg);break;
    default                   : return Process_Core_Default(pcemsg);break;
  }
}
//*******************************************************************
bool TGW_CEHandler::Process_Core_GS_Address(TCEMsg *pcemsg)
{
  // should be the same for DMT_REPLY and DMT_INFO
  TGameServer *pgs = PApp->GSList.Get(pcemsg->GetGSAddress());
  if (!pgs)
  {
    PApp->GSList.Add(pcemsg->GetAddress(),
                     pcemsg->GetMajVer(),
                     pcemsg->GetMinVer(),
                     pcemsg->GetGSStatus());
  }
  return false;
}
//*******************************************************************
bool TGW_CEHandler::Process_Core_GS_Status(TCEMsg *pcemsg)
{
  // should be the same for DMT_REPLY and DMT_INFO
  TGameServer *pgs = PApp->GSList.Get(pcemsg->GetGSAddress());
  if (pgs)
  {
    pgs->Status = pcemsg->GetGSStatus();
  }
  return false;
}
//*******************************************************************
bool TGW_CEHandler::Process_Core_Default(TCEMsg *pcemsg)
{
}
//*******************************************************************
bool TGW_CEHandler::Process_User(TCEMsg *pcemsg)
{
  switch (pcemsg->GetSMT())
  {
    case SMT_USER_LOGIN    : return Process_User_Login(pcemsg);break;
    case SMT_USER_NEWLOGIN : return Process_User_NewLogin(pcemsg);break;
    case SMT_USER_LOGOFF   : return Process_User_Logoff(pcemsg);break;
    default                : return Process_User_Default(pcemsg);break;
  }
}
//*******************************************************************
bool TGW_CEHandler::Process_User_Login(TCEMsg *pcemsg)
{
  TClient *pclient = PApp->ClientList.Get(pcemsg->GetConnectionID());
  if (!pclient) {return false;}

  if (pcemsg->GetDMT() == DMT_REQUEST)
  {
    if (pclient->LoggedIn) {return false;}

    TGameServer *pgs = PApp->GSList.GetNext();
    if (!pgs) {return false;}

    pgs->Send(pcemsg);
    return true;
  }
  else if (pcemsg->GetDMT() == DMT_REPLY)
  {
    if (pcemsg->GetErrorCode() == EC_NONE)
    {
      pclient->UAN = pcemsg->GetUAN();
      pclient->LoggedIn = true;
    }
    TETMsg *petmsg = new TETMsg(pcemsg);
    pclient->Send(petmsg);
    return false;
  }
  else
  {
    // error - invalid DMT, drop ???
    return false;
  }
}
//*******************************************************************
bool TGW_CEHandler::Process_User_NewLogin(TCEMsg *pcemsg)
{
  // similiar to Login().
}
//*******************************************************************
bool TGW_CEHandler::Process_User_Logoff(TCEMsg *pcemsg)
{
  TClient *pclient = PApp->ClientList.Get(pcemsg->GetConnectionID());
  if (!pclient) {return false;}

  if (pcemsg->GetDMT() == DMT_REQUEST)
  {
    if (!pclient->LoggedIn) {return false;}

    TGameServer *pgs = PApp->GSList.GetNext();
    if (!pgs) {return false;}

    pgs->Send(pcemsg);
    return true;
  }
  else if (pcemsg->GetDMT() == DMT_REPLY)
  {
    if (pcemsg->GetErrorCode() == EC_NONE)
    {
      pclient->LoggedIn = false;
    }
    TETMsg *petmsg = new TETMsg(pcemsg);
    pclient->Send(petmsg);
    PApp->ClientList.Release(pclient);
    return false;
  }
  else
  {
    // error - invalid DMT, drop ???
    return false;
  }
}
//*******************************************************************
bool TGW_CEHandler::Process_User_Default(TCEMsg *pcemsg)
{
  return Process_Default(pcemsg);
}
//*******************************************************************
bool TGW_CEHandler::Process_Default(TCEMsg *pcemsg)
{
  if (pcemsg->GetMT() == MT_USER)
  {
    // should have been handled special.
    return false;
  }
  bool rc = false;
  switch (pcemsg->GetDMT())
  {
    case DMT_REQUEST : TGameServer *pgs = PApp->GSList.GetNext();
                       if (pgs)
                       {
                         pgs->Send(pcemsg);
                         rc = true;
                       }
                       break;
    case DMT_REPLY   : TETMsg *petmsg = new TETMsg(pcemsg);
                       TClient *pclient = PApp->ClientList.Get(pcemsg->GetUAN());
                       if (pclient)
                       {
                         pclient->Send(petmsg);
                         rc = false;
                       }
    default          : // error, inc a counter ???
                       break;
  }
  return rc;
}
//*******************************************************************
