create or replace package body PGameSession as
//*******************************************************************
procedure Create(uan IN PDef.TUAN,
                 cassessionid IN PDef.TCSID,
                 cassessioncounter IN PDef.TLgCount,
                 gamesessioncounter IN PDef.TLgCount,
                 gs IN OUT TGameSession,
                 ec OUT PDef.TErrorCode)
{
  PDef.TDate d = SYSDATE;
  gs = TGameSession(uan,
                    true,
                    cassessionid,
                    cassessioncounter,
                    Seq_GameSessionID.Next(),
                    gamesessioncounter,
                    GT_NONE,
                    0,
                    0,
                    0,
                    0,
                    d,
                    d,
                    0,
                    0,
                    d,
                    d,
                    0,
                    0,
                    0,
                    0,
                    false);
  ec = PDef.EC_NONE;
}
//*****************************************************************
procedure Add(gs IN OUT TGameSession,ec OUT PDef.TErrorCode)
{
  insert gs
  into Tbl_GameSession
   ec = PDef.EC_NONE;
}
//*****************************************************************
procedure CreateAdd(uan IN PDef.TUAN,
                    cassessionid IN PDef.TCSID,
                    cassessioncounter IN PDef.TLgCount,
                    gamesessioncounter IN PDef.TLgCount,
                    gs IN OUT TGameSession,
                    ec OUT PDef.TErrorCode)
{
  Create(uan,
         cassessionid,
         cassessioncounter,
         gamesessioncounter,
         gs,
         ec);
  if (ec == PDef.EC_NONE)
  {
    Add(gs,ec);
  }
}
//*******************************************************************
function IsStopSafe(gs IN OUT TGameSession) return PDef.TBool
{
  TBool rc = true;
  TErrorCode ec;
  if (gs.GameType == PDef.GT_SLOTS)
  {
    rc = PSlots.IsStopSafe(gs.InstanceID,gs.UAN);
  }
  else if (gs.GameType == PDef.GT_BJ)
  {
    rc = PBJ.IsStopSafe(gs.InstanceID,gs.UAN);
  }
  else if (gs.GameType == PDef.GT_VP)
  {
    rc = PVP.IsStopSafe(gs.InstanceID,gs.UAN);
  }
  else if (gs.GameType == PDef.GT_ROULETTE)
  {
    rc = PR.IsStopSafe(gs.InstanceID,gs.UAN);
  }
  return rc;
}
//*****************************************************************
procedure Start(user IN OUT TUser,
                gs IN OUT TGameSession,
                gametype IN PDef.TGame,
                mid IN PDef.TMID,
                ec OUT PDef.TErrorCode)
{
  gs.MachineID = mid;
  gs.GameType = gametype;
  TWallet wallet;
  PWallet.Lock(gs.UAN,wallet,ec);
  if (ec == PDef.EC_NONE)
  {
    gs.StartingChips = wallet.GetChips();
    gs.FreePlay = wallet.GetFreePlay();
    PWallet.UnLock(wallet,ec);
    gs.OverallStartDate = SYSDATE();
    gs.LastStartDate = gs.OverallStartDate;

    if (gs.GameType == PDef.GT_LOBBY)
    {
      // do nothing.
    }
    else if (gs.GameType == PDef.GT_SLOTS)
    {
      PSlots.Start(user,gs.MachineID,gs.FreePlay,gs.InstanceID,ec);
    }
    else if (gs.GameType == PDef.GT_BJ)
    {
      PBJ.Start(user,gs.MachineID,gs.FreePlay,gs.InstanceID,ec);
    }
    else if (gs.GameType == PDef.GT_VP)
    {
      PVP.Start(user,gs.MachineID,gs.FreePlay,gs.InstanceID,ec);
    }
    else if (gs.GameType == PDef.GT_ROULETTE)
    {
      PR.Start(user,gs.MachineID,gs.FreePlay,gs.InstanceID,ec);
    }
    else
    {
      ec = PDef.EC_GT_INVALID;
    }
  }
}
//*****************************************************************
procedure Pause(user IN OUT TUser,
                gs IN OUT TGameSession,
                ec OUT PDef.TErrorCode)
{
  gs.DisconnectCounter += 1;
  gs.LastEndDate = SYSDATE;
  gs.TimePlayedSec += gs.LastEndDate - gs.LastStartDate;
}
//*****************************************************************
procedure VerifyInstance(gs IN OUT TGameSession,ec OUT PDef.TErrorCode)
{
  if (gs.GameType == PDef.GT_LOBBY)
  {
    ec = PDef.EC_NONE;
  }
  else if (gs.GameType == PDef.GT_SLOTS)
  {
    PSlots.Verify(gs,ec);
  }
  else if (gs.GameType == PDef.GT_VP)
  {
    PVP.Verify(gs,ec);
  }
  else if (gs.GameType == PDef.GT_BJ)
  {
    PBJ.Verify(gs,ec);
  }
  else if (gs.GameType == PDef.GT_ROULETTE)
  {
    PR.Verify(gs,ec);
  }
  else
  {
    ec = PDef.EC_GT_INVALID;
  }
}
//*****************************************************************
procedure ReStart(user IN OUT TUser,
                  gs IN OUT TGameSession,
                  ec OUT PDef.TErrorCode)
{
  gs.LastStartDate = SYSDATE;
  VerifyInstance(gs,ec);
}
//*****************************************************************
procedure Stop(user IN OUT TUser,
               gs IN OUT TGameSession,
               ec OUT PDef.TErrorCode)
{
  if (gs.GameType == PDef.GT_LOBBY)
  {
    ec = PDef.EC_NONE;
  }
  else if (gs.GameType == PDef.GT_SLOTS)
  {
    PSlots.Stop(user,gs.InstanceID,gs.CasSessionCounter,gs.Wagered,gs.WonLost,ec);
  }
  else if (gs.GameType == PDef.GT_BJ)
  {
    PBJ.Stop(user,gs.InstanceID,gs.CasSessionCounter,gs.Wagered,gs.WonLost,ec);
  }
  else if (gs.GameType == PDef.GT_VP)
  {
    PVP.Stop(user,gs.InstanceID,gs.CasSessionCounter,gs.Wagered,gs.WonLost,ec);
  }
  else if (gs.GameType == PDef.GT_ROULETTE)
  {
    PR.Stop(user,gs.InstanceID,gs.CasSessionCounter,gs.Wagered,gs.WonLost,ec);
  }
  else
  {
    ec = PDef.EC_GT_INVALID;
  }


  // update stats.

  if (ec == PDef.EC_NONE)
  {
    gs.OverallEndDate = SYSDATE;
    gs.LastEndDate = gs.OverallEndDate;
    gs.TimePlayedSec += gs.LastEndDate - gs.LastStartDate;
    gs.InUse = false;

    stat TStat(gs.TimePlayedSec,gs.Wagered,gs.WonLost,
               gs.DisconnectCounter,gs.FreePlay);

    PMachine_Stats.UpdateAll(gs.UAN,
                             gs.GameType,
                             gs.MachineID,
                             gs.CasSessionCounter,
                             SYSDATE,
                             stat,
                             ec);
  }
}
//*******************************************************************
end PGameSession;