create or replace body PMachine_Stats as
//*******************************************************************
procedure Lock(uan IN PDef.TUAN,
               gt IN PDef.TGameType,
               machineid IN PDef.TMID,
               cscounter IN,
               date IN PDef.TDate,
               ms IN OUT TMachineStats,
               ec OUT PDef.TErrorCode)
{
  ec = PDef.EC_NONE;

  select ms
  from Tbl_Machine_Stats
  where (rec.UAN == uan &&
         rec.GameType == gt &&
         rec.MachineID == machineid &&
         rec.CasSessionCounter == cscounter &&
         rec.Date == date)
  for update;

  if (not found)
  {
    ec = PDef.EC_MACHINE_STAT_NOTFOUND;
  }
}
//*****************************************************************
procedure UnLock(ms IN OUT TMachineStats,ec OUT PDef.TErrorCode)
{
  if (ms.StatID > 0)
  {
    update Tbl_Machine_Stats
    rec = ms
    where (rec.StatID == ms.StatID);

    ec = PDef.EC_NONE;
  }
  else
  {
    ec = PDef.EC_MACHINE_STAT_NOTFOUND;
  }
}
//*****************************************************************
procedure Create(uan IN,
                 gt IN,
                 machineid IN PDef.TMID,
                 cscounter IN,
                 date IN PDef.TDate,
                 stat IN TStat,
                 ms IN OUT TMachine_Stats,
                 ec OUT PDef.TErrorCode);
{
  TStat stat2 = TStat(0,0,0,0,false);
  if (stat.FreePlay)
  {
    ms = TMachine_Stats(Seq_StatID.Next,
                        gt,
                        machineid,
                        date,
                        cscounter,
                        uan,
                        stat2,
                        stat);
  }
  else
  {
    ms = TMachine_Stats(Seq_StatID.Next(),
                        gt,
                        mid,
                        date,
                        cscounter,
                        uan,
                        stat,
                        stat2);
  }
  ec = PDef.EC_NONE;
}
//*****************************************************************
procedure Add(ms IN OUT TMachineStat,ec OUT PDef.TErrorCode)
{
  insert ms
  into Tbl_Machine_Stats

  if (DUP_VAL_ON_INDEX)
  {
    ec = PDef.EC_MACHINE_STAT_DUP_VAL;
  }
}
//*****************************************************************
procedure CreateAdd(uan IN,
                    gt IN,
                    machineid IN PDef.TMID,
                    cscounter IN,
                    date IN PDef.TDate,
                    stat IN TStat,
                    ec OUT PDef.TErrorCode)
{
  TMachine_Stats ms;
  Create(uan,
         gt,
         machineid,
         cscounter,
         date,
         stat,
         ms,
         ec);
  if (ec == PDef.EC_NONE)
  {
    Add(ms,ec);
  }
}
//*****************************************************************
procedure Update(uan IN,
                 gt IN,
                 machineid IN PDef.TMID,
                 cscounter IN,
                 date IN PDef.TDate,
                 stat IN TStat,
                 ec OUT PDef.TErrorCode)
{
  TMachineStats ms;

  Lock(uan,gt,MachineID,cscounter,date,ms,ec);
  if (ec == PDef.EC_NONE)
  {
    ms.Update(stat,free);
    UnLock(ms,ec);
  }
  else if (ec == PDef.EC_MACHINE_STAT_NOTFOUND)
  {
    CreateAdd(uan,
              gt,
              machineid,
              cscounter,
              date,
              stat,
              ec);
  }
}
//*******************************************************************
procedure UpdateAll(uan IN,
                    gt IN,
                    mid IN,
                    cscounter IN,
                    date IN,
                    cash IN TStat,
                    free IN TStat,
                    ec OUT PDef.TErrorCode)
{
// note: possible deadlock problem here.
//       might need to modify so that all obj's locked before mod's.

  int16 setting;
  PCasCfg.Machine_Stats__GetUpdate(setting,ec);
  if (setting & ALLUSERS_GT_MID_EACHDAY)
  {
    Update(0,gt,mid,0,trunc(date),cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & EACHUSER_GT)
  {
    Update(uan,gt,0,0,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & EACHUSER_GT_EACHCS)
  {
    Update(uan,gt,0,cscounter,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & EACHUSER_GT_MID)
  {
    Update(uan,gt,mid,0,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & EACHUSER_GT_MID_EACHCS)
  {
    Update(uan,gt,mid,cscounter,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & ALLUSERS)
  {
    Update(0,0,0,0,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & ALLUSERS_GT)
  {
    Update(0,gt,0,0,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & ALLUSERS_GT_MID)
  {
    Update(0,gt,mid,0,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & EACHUSER)
  {
    Update(uan,0,0,0,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
  if (setting & EACHUSER_EACHCS)
  {
    Update(uan,0,0,cscounter,0,cash,free,ec);
    if (ec != PDef.EC_NONE) {return;}
  }
}
//*******************************************************************
end PMachine_Stats;