//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oanm.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/5/95     M.Perry     original baseline
//          11/12/96   M.Perry     changed anm format to v2.0
//  2.0     8/21/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__oanm_h)
#define __oanm_h

#if !defined(__ogr_h)
#include "ogr.h"
#endif

#if !defined(__ogeo_h)
#include "ogeo.h"
#endif

#ifdef DEBUGREPORTS
  #include <iostream.h>
  #include <fstream.h>
#endif

#if defined(GED_COMPILE) && !defined(__oanmcvtd_h)
  #include "oanmcvtd.h"
#endif
//*******************************************************************
enum TDirection
{
  dReverse = -1,
  dStill = 0,
  dForward = 1
};
//*******************************************************************
// start of anmdata stat flags
#define anmdFIRST ((WORD)(grdLAST * 2))
#define anmdAUTOSTART (anmdFIRST)
#define anmdAUTOSTOP ((WORD)(anmdAUTOSTART * 2))
#define anmdAUTOCHANGE ((WORD)(anmdAUTOSTOP * 2))
#define anmdLAST (anmdAUTOCHANGE)
// end of anmdata stat flags
//*******************************************************************
class TAnmData:public TGrData
{
public:   // data
  int16 StartFrame,EndFrame,CurrentFrame;
  TDirection Direction;

public:
  void SetStartFrame(int16 sf);
  int16 GetStartFrame() const;
  void SetEndFrame(int16 ef);
  int16 GetEndFrame() const;
  void SetCurrentFrame(int16 cf);
  int16 GetCurrentFrame() const;
  void SetStartEndFrame(int16 sf,int16 ef);
  void SetDirection(TDirection direction);
  TDirection GetDirection() const;
  void SetDirFor();
  void SetDirStill();
  void SetDirRev();
  void SetAutoStart(bool autostart = true);
  bool GetAutoStart() const;
  void ClearAutoStart();
  void SetAutoStop(bool autostop = true);
  bool GetAutoStop() const;
  void ClearAutoStop();
  void SetAutoStartStop(bool autostart = true,bool autostop = true);
  void SetAutoChange(bool autochange = true);
  bool GetAutoChange() const;
  void ClearAutoChange();
  TAnmData &operator=(TAnmData &ad);
  TAnmData &SetData(TAnmData &ad);

  TAnmData(const TAnmData &ad);
  TAnmData();
  TAnmData(int16 x,int16 y,int16 z,
           int16 sf = 0,int16 ef = -1,int16 cf = 0,
           TDirection direction = dForward,
           bool autostart = true,
           bool autostop = true,
           bool autochange = true);
  virtual ~TAnmData();
};
//*******************************************************************
struct TAnmFrameLineSegment
// v2.0
{
  word LocalXOffset;
  word Len;
  byte Pixels[1];
  byte huge *GetPixelsPtr(int16 pixelnum)
  {
    return &(Pixels[pixelnum]);
  }
  #if defined(GED_COMPILE)
  dword GetSize();
  byte *PReadPixels;
  bool Write(TOgleBase *pbase);
  bool Convert(word framenum,word linenum,word segmentnum,byte huge *prawsegment,word localxoff,word lenleft);
  word ReadSize;
  word GetReadSize() {return ReadSize;}
  TAnmFrameLineSegment();
  ~TAnmFrameLineSegment();
  #endif
};
//*******************************************************************
struct TAnmFrameLine
// v2.0
{
  word NumSegments;
  dword SegmentOffsets[1];
  dword GetSegmentOffset(word segmentnum);
  bool CheckIndex(word segmentnum);
  TAnmFrameLineSegment huge *GetSegmentPtr(byte huge *p,word segmentnum);
  #if defined(GED_COMPILE)
  dword StartingOffset;
  dword GetSize();
  dword GetStructSize();
  TAnmFrameLineSegment *PReadSegments;
  dword *PReadSegmentOffsets;
  bool Write(TOgleBase *pbase);
  bool Convert(TAnmConvertDlg *pacd,word framenum,word linenum,int16 width,byte huge *prawline);
  void SetTableOffsets(dword offset);
  word ReadSize;
  word GetReadSize() {return ReadSize;}
  word CountSegments(byte huge *prawline,int16 width);
  TAnmFrameLine();
  ~TAnmFrameLine();
  #endif
};
//*******************************************************************
struct TAnmFrameHeader
// leave struct alone for v1.0
{
  int16 width;
  int16 height;
  TOglePoint ref[4];
};
//*******************************************************************
struct TAnmFrame:public TAnmFrameHeader
// added for v2.0
{
  dword LineOffsets[1];
  bool CheckIndex(word linenum);
  dword GetLineOffset(word linenum);
  TAnmFrameLine huge *GetLinePtr(byte huge *p,word linenum);
  #if defined(GED_COMPILE)
  dword StartingOffset;
  bool Convert(TAnmConvertDlg *pacd,word framenum,TAnmFrameHeader huge *pframeheader,TAnmFrameHeader &afh);
  void SetTableOffsets(dword offset);
  bool Write(TOgleBase *pbase);
  dword GetSize();
  dword GetStructSize();
  TAnmFrameLine *PReadLines;
  dword *PLineOffsets;
  TAnmFrame();
  ~TAnmFrame();
  #endif
};
//*******************************************************************
struct TAnmMainHeader
// leave struct alone for v1.0
{
  word NumFrames;
  TOglePoint MainRefPt;
  TOglePoint ScreenPt;
};
//*******************************************************************
struct TAnmMain:public TAnmMainHeader
// added for v2.0
{
  dword FrameOffsets[1];
  bool CheckIndex(word framenum);
  dword GetFrameOffset(word framenum);

  TAnmFrame huge *GetFramePtr(byte huge *p,word framenum);
  #if defined(GED_COMPILE)
  TAnmFrameHeader huge *GetFrameHeaderPtr(byte huge *p,word framenum);
  #endif
};
//*******************************************************************
#define aFIRST ((WORD)(grLAST * 2))
#define aREADFORMAT_1_0 (aFIRST)
#define aALNOTIFIEDSTARTSCHEDULED ((WORD)(aREADFORMAT_1_0 * 2))
#define aLAST (aALNOTIFIEDSTARTSCHEDULED)

#if (aLAST > fBit15)
#error StatFlag bit overflow in TAnm
#endif
class TAnmList;
//*******************************************************************
class TAnm:public TGr
{
protected:
  dword EndFUC; // MRP - v2.3 - used by still

public:
  TAnmData SAD,CAD;
  TAnmList *PAnmList; // set by anmlist when added to it.

public:
  TAnm(TOgleModeWindow *pmw,const TFileMode &filemode,TAnmData &sad,
       bool AutoRead = false,bool KeepBaseFile = true);

  virtual ~TAnm();

  // the following is workaround for TAnm (mostly GED_COMPILE), because most TOgleBase classes
  // convert the info during read and don't hold on to ramptr, but TAnm's are different, so,
  // aREADFORMAT_1_0 used by some access functions to know how to read the file because
  // MajVer is changed by writelocalheader.
  bool IsReadFormat_1_0();
  void SetReadFormat_1_0();
  void ClearReadFormat_1_0();
  void SetReadFormat();

  bool GetALNotifiedStartScheduled();
  void SetALNotifiedStartScheduled();
  void ClearALNotifiedStartScheduled();

  void ALNotifyStart();
  void ALNotifyStop();

  #ifdef DEBUGREPORTS
  void Report(ofstream &os);
  #endif
  #ifdef DEBUGSPRITES
  void RTReport(ofstream &os);
  #endif

  void SetCADRelative(TTriplet &trip);
  void SetStartingData(TAnmData &sad);
  void SetCurrentData(TAnmData &cad);
  void ResetCurrentData();
  TAnmData &GetCurrentData();
  TAnmData &GetStartingData();
  int16 GetDefaultScreenX();
  int16 GetDefaultScreenY();

public:
  TAnmList *GetAnmList() {return PAnmList;}
  void SetAnmList(TAnmList *panmlist) {PAnmList = panmlist;}
  const char *GetIDChars() const;
  virtual void SetMajVer();
  virtual void SetMinVer();
  virtual bool ReadLocalHeader();
  virtual bool WriteLocal();
  #if defined(GED_COMPILE)
  bool WriteLocal_From_1_0_To_2_0();
  #endif
  virtual bool ReadLocal();
  bool ReadLocal_1_0();
  bool ReadLocal_2_0();

  virtual string AddExt();
  virtual void Update(dword NewFUC);
  virtual void PostUpdate(dword NewFUC);
  virtual void Start();
  virtual void Start(dword StartFUC);
  virtual void Activate(dword StartFUC = 0);
  virtual void DeActivate();
  virtual void ManualStart();
  void AutoStart();
  virtual void ManualStop();
  virtual void Stop();
  virtual void Show();
  virtual void NoShow();
  virtual bool RequestChangeMode(TFileMode &NewFileMode);

  virtual bool BltTo(TOgleVideoDC &vdc,TOgleRect &clipvrect);

protected:
  void NextFrame();
  void NextForward();
  void NextReverse();
  void NextStill();
  virtual void NextManual();

public:
  int16 GetLastFrameNum();
  word GetFrameNum();
  bool IsValidFrame(int16 framenum);

protected:
  bool BltTo(TGr &gr,TOgleRect &clipvrect);

  TAnmFrame huge *GetFramePtr(word frameindex);
  void LoadCurrentAnmFrame();

  TAnmMain huge *GetMainPtr();

  #if defined(GED_COMPILE)
  TAnmFrameHeader huge *GetFrameHeaderPtr(word frameindex);
  byte huge *GetFrameOffsetTblPtr();
  bool CopyAnmFrameHeader(TAnmFrameHeader &afh,TAnmFrameHeader huge *pafh);
  #endif
};
//*******************************************************************
//********************  inlines *************************************
//*******************************************************************
inline void TAnmData::SetStartFrame(int16 sf)
{
  StartFrame = sf;
}
//*******************************************************************
inline int16 TAnmData::GetStartFrame() const
{
  return StartFrame;
}
//*******************************************************************
inline void TAnmData::SetEndFrame(int16 ef)
{
  EndFrame = ef;
}
//*******************************************************************
inline int16 TAnmData::GetEndFrame() const
{
  return EndFrame;
}
//*******************************************************************
inline void TAnmData::SetCurrentFrame(int16 cf)
{
  CurrentFrame = cf;
}
//*******************************************************************
inline int16 TAnmData::GetCurrentFrame() const
{
  return CurrentFrame;
}
//*******************************************************************
inline void TAnmData::SetStartEndFrame(int16 sf,int16 ef)
{
  SetStartFrame(sf);
  SetEndFrame(ef);
}
//*******************************************************************
inline void TAnmData::SetDirection(TDirection direction)
{
  Direction = direction;
}
//*******************************************************************
inline TDirection TAnmData::GetDirection() const
{
  return Direction;
}
//*******************************************************************
inline void TAnmData::SetDirFor()
{
  SetDirection(dForward);
}
//*******************************************************************
inline void TAnmData::SetDirStill()
{
  SetDirection(dStill);
}
//*******************************************************************
inline void TAnmData::SetDirRev()
{
  SetDirection(dReverse);
}
//*******************************************************************
inline void TAnmData::SetAutoStart(bool autostart)
{
  if (autostart) {StatFlags.Set(anmdAUTOSTART);}
  else           {StatFlags.Clear(anmdAUTOSTART);}
}
//*******************************************************************
inline bool TAnmData::GetAutoStart() const
{
  return StatFlags.IsSet(anmdAUTOSTART);
}
//*******************************************************************
inline void TAnmData::ClearAutoStart()
{
  StatFlags.Clear(anmdAUTOSTART);
}
//*******************************************************************
inline void TAnmData::SetAutoStop(bool autostop)
{
  if (autostop) {StatFlags.Set(anmdAUTOSTOP);}
  else          {StatFlags.Clear(anmdAUTOSTOP);}
}
//*******************************************************************
inline bool TAnmData::GetAutoStop() const
{
  return StatFlags.IsSet(anmdAUTOSTOP);
}
//*******************************************************************
inline void TAnmData::ClearAutoStop()
{
  StatFlags.Clear(anmdAUTOSTOP);
}
//*******************************************************************
inline void TAnmData::SetAutoStartStop(bool autostart,bool autostop)
{
  SetAutoStart(autostart);
  SetAutoStop(autostop);
}
//*******************************************************************
inline void TAnmData::SetAutoChange(bool autochange)
{
  if (autochange) {StatFlags.Set(anmdAUTOCHANGE);}
  else            {StatFlags.Clear(anmdAUTOCHANGE);}
}
//*******************************************************************
inline bool TAnmData::GetAutoChange() const
{
  return StatFlags.IsSet(anmdAUTOCHANGE);
}
//*******************************************************************
inline void TAnmData::ClearAutoChange()
{
  StatFlags.Clear(anmdAUTOCHANGE);
}
//*******************************************************************
inline TAnmData &TAnmData::operator=(TAnmData &ad)
{
  return SetData(ad);
}
//*******************************************************************
//*******************************************************************
inline dword TAnmFrameLine::GetSegmentOffset(word segmentnum)
{
  return SegmentOffsets[segmentnum];
}
//*******************************************************************
inline bool TAnmFrameLine::CheckIndex(word segmentnum)
{
  return (segmentnum < NumSegments);
}
//*******************************************************************
inline TAnmFrameLineSegment huge *TAnmFrameLine::GetSegmentPtr(byte huge *p,word segmentnum)
{
  if (!CheckIndex(segmentnum)) {return NULL;}
  return (TAnmFrameLineSegment huge *)(p + GetSegmentOffset(segmentnum));
}
//*******************************************************************
//*******************************************************************
inline bool TAnmFrame::CheckIndex(word linenum)
{
  return (linenum < height);
}
//*******************************************************************
inline dword TAnmFrame::GetLineOffset(word linenum)
{
  return LineOffsets[linenum];
}
//*******************************************************************
inline TAnmFrameLine huge *TAnmFrame::GetLinePtr(byte huge *p,word linenum)
{
  if (!CheckIndex(linenum)) {return NULL;}
  return (TAnmFrameLine huge *)(p + GetLineOffset(linenum));
}
//*******************************************************************
//*******************************************************************
inline bool TAnmMain::CheckIndex(word framenum)
{
  return (framenum < NumFrames);
}
//*******************************************************************
inline dword TAnmMain::GetFrameOffset(word framenum)
{
  return FrameOffsets[framenum];
}
//*******************************************************************
inline TAnmFrame huge *TAnmMain::GetFramePtr(byte huge *p,word framenum)
// v2.0
// p is ramptr
{
  if (!CheckIndex(framenum)) {return NULL;}
  return (TAnmFrame huge *)(p + GetFrameOffset(framenum));
}
//*******************************************************************
#if defined(GED_COMPILE)
inline TAnmFrameHeader huge *TAnmMain::GetFrameHeaderPtr(byte huge *p,word framenum)
{
  // v1.0
  // p is start of frame offset tbl
  if (!CheckIndex(framenum)) {return NULL;}
  return (TAnmFrameHeader huge *)(p + GetFrameOffset(framenum));
}
#endif
//*******************************************************************
//*******************************************************************
inline bool TAnm::IsReadFormat_1_0()
{
  return StatFlags.IsSet(aREADFORMAT_1_0);
}
//*******************************************************************
inline void TAnm::SetReadFormat_1_0()
{
  StatFlags.Set(aREADFORMAT_1_0);
}
//*******************************************************************
inline void TAnm::ClearReadFormat_1_0()
{
  StatFlags.Clear(aREADFORMAT_1_0);
}
//*******************************************************************
inline void TAnm::SetReadFormat()
{
  if (MajVer == 1) {SetReadFormat_1_0();}
  else {ClearReadFormat_1_0();}
}
//*******************************************************************
inline bool TAnm::GetALNotifiedStartScheduled()
{
  return StatFlags.IsSet(aALNOTIFIEDSTARTSCHEDULED);
}
//*******************************************************************
inline void TAnm::SetALNotifiedStartScheduled()
{
  StatFlags.Set(aALNOTIFIEDSTARTSCHEDULED);
}
//*******************************************************************
inline void TAnm::ClearALNotifiedStartScheduled()
{
  StatFlags.Clear(aALNOTIFIEDSTARTSCHEDULED);
}
//*******************************************************************
inline void TAnm::SetStartingData(TAnmData &sad)
{
  SAD = sad;
}
//*******************************************************************
inline void TAnm::SetCurrentData(TAnmData &cad)
{
  CAD = cad;
}
//*******************************************************************
inline void TAnm::ResetCurrentData()
{
  CAD = SAD;
}
//*******************************************************************
inline TAnmData &TAnm::GetCurrentData()
{
  return CAD;
}
//*******************************************************************
inline TAnmData &TAnm::GetStartingData()
{
  return SAD;
}
//*******************************************************************
inline void TAnm::NextFrame()
{
  if (!CAD.GetAutoChange())                {NextManual();}
  else if (CAD.GetDirection() == dForward) {NextForward();}
  else if (CAD.GetDirection() == dStill)   {NextStill();}
  else                                     {NextReverse();}
}
//*******************************************************************
inline void TAnm::NextForward()
{
  if ((CAD.GetCurrentFrame() + 1) <= CAD.GetEndFrame())
  {
    CAD.SetCurrentFrame(int16(CAD.GetCurrentFrame() + 1));
  }
  LoadCurrentAnmFrame();
}
//*******************************************************************
inline void TAnm::NextReverse()
{
  if ((CAD.GetCurrentFrame() - 1) >= CAD.GetEndFrame())
  {
    CAD.SetCurrentFrame(int16(CAD.GetCurrentFrame() - 1));
  }
  LoadCurrentAnmFrame();
}
//*******************************************************************
inline void TAnm::NextStill()
{
  LoadCurrentAnmFrame();
}
//*******************************************************************
inline int16 TAnm::GetDefaultScreenX()
{
  if (!GetRamLoaded()) {return 0;}
  int16 x = GetMainPtr()->ScreenPt.x;
  return x;
}
//*******************************************************************
inline int16 TAnm::GetDefaultScreenY()
{
  if (!GetRamLoaded()) {return 0;}
  int16 y = GetMainPtr()->ScreenPt.y;
  return y;
}
//*******************************************************************
inline TAnmMain huge *TAnm::GetMainPtr()
{
  #if defined(GED_COMPILE)
  dword offset = 0;
  if (!IsReadFormat_1_0()) {offset = FILEHEADERSIZE;}
  TAnmMain huge *pmain = (TAnmMain huge *)((byte huge *)GetRamPtr() + offset);
  return pmain;
  #else
  return (TAnmMain huge *)((byte huge *)GetRamPtr() + FILEHEADERSIZE);
  #endif
}
//*******************************************************************
#if defined(GED_COMPILE)
inline byte huge *TAnm::GetFrameOffsetTblPtr()
// v1.0
{
  dword offset = 0;
  if (IsReadFormat_1_0()) {offset = sizeof(TAnmMainHeader);}
  return (byte huge *)((byte huge *)GetRamPtr() + offset);
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
inline TAnmFrameHeader huge*TAnm::GetFrameHeaderPtr(word frameindex)
// v1.0
{
  if (!GetRamLoaded()) {return NULL;}
  return GetMainPtr()->GetFrameHeaderPtr(GetFrameOffsetTblPtr(),frameindex);
}
#endif
//*******************************************************************
inline TAnmFrame huge *TAnm::GetFramePtr(word frameindex)
// v2.0
{
  if (!GetRamLoaded()) {return NULL;}
  return GetMainPtr()->GetFramePtr((byte huge *)GetRamPtr(),frameindex);
}
//*******************************************************************
inline int16 TAnm::GetLastFrameNum()
{
  return int16(GetFrameNum() - 1);
}
//*******************************************************************
inline bool TAnm::IsValidFrame(int16 framenum)
{
  if (framenum > GetLastFrameNum()) {return false;}
  if (framenum < 0) {return false;}
  return true;
}
//*******************************************************************
#if defined(GED_COMPILE)
inline bool TAnm::CopyAnmFrameHeader(TAnmFrameHeader &afh,TAnmFrameHeader huge *pafh)
{
  // prevents 64k boundary straddling problem with structure.
  if (pafh == NULL) {return false;}
  hmemcpy(&afh,pafh,sizeof(TAnmFrameHeader));
  return true;
}
#endif
//*******************************************************************
inline word TAnm::GetFrameNum()
{
  if (!GetRamLoaded()) {return 0;}
  return GetMainPtr()->NumFrames;
}
//*******************************************************************
#endif

