//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oaudwmix.h
//    Project Name:  Ogle                 
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     10/18/96   M.Perry    original baseline
//  2.0     08/23/97   M.Perry    32-bit conversion.
//  2.2     06/01/99   M.Perry    fixed focus problem.
//********************************************************************
#if !defined(__oaudwmix_h)
#define __oaudwmix_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

//#include <owl\dialog.h>
#if !defined(__ohidewin_h)
#include "ohidewin.h"
#endif

#if !defined(_INC_MMSYSTEM_H)
#include <mmsystem.h>
#endif

#if !defined(__owmixdll_h)
#include "owmixdll.h"
#endif

#if !defined(__osliq_h)
#include "osliq.h"
#endif

#if !defined(__oobj_h)
#include "oobj.h"
#endif

#if !defined(__ofmode_h)
#include "ofmode.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__omemmgr_h)
#include "omemmgr.h"
#endif

#define NUM_AUDIO_CHANNELS 8

class TAudioWaveMix;
class TWaveMixControlData;
class TWav;
//*******************************************************************
class TAudioData
{
public:
  TFMStatus Status;
  word OwnerCount;
//  word StatusCounts[NUM_TFMSTATUS];
  word StatusCounts[TFileMode::fm_Num_Status];
  dword Size;
  string FileName;
  LPMIXWAVE PMixWave;
  TAudioWaveMix *PAudioMgr;

public:
  TAudioData(TAudioWaveMix *paudiomgr,string filename);
  ~TAudioData();
  void DecCount(TFMStatus status);
  void IncCount(TFMStatus status);
  void SetStatus();
  void IncOwnerCount();
  void DecOwnerCount();
  word GetOwnerCount();
  bool Load(TWav *pwav);
  void UnLoad();
  bool IsLoaded();
};
//*******************************************************************
class TAudioObj:public TOgleObj
{
public:
  TAudioObj();
  virtual ~TAudioObj();
  TFMStatus Status;
  TAudioData *PAudioData;
};
//*******************************************************************
class TAudioDataSliq:public TSliq
// this maintains list of TAudioData, not TAudioObj
{
public:
  TAudioDataSliq(TAudioWaveMix *paudiomgr);
  virtual ~TAudioDataSliq();
  bool Add(TAudioData *pad);
  TAudioData *PopAD();
//  TAudioData *GetAudioData(string name);
  TAudioData *GetAudioData(const string &name);
  TAudioData *GetFirst();
  TAudioData *GetNext();
  bool UnLoadBelow(TFMStatus status,dword sizeneeded,dword &sizesofar);
  void DestroyNoOwners();
private:
  TAudioWaveMix *PAudioMgr;
};
//*******************************************************************
#define NUMAUDIODATASLIQS 200
class TAudioDSArray
{
public:
  TAudioDataSliq *PDS[NUMAUDIODATASLIQS];
public:
  TAudioDSArray(TAudioWaveMix *paudiomgr);
  ~TAudioDSArray();
   bool Add(const string &name,TAudioData *pad);
//   TAudioData *GetAudioData(string &name);
   TAudioData *GetAudioData(const string &name);
   bool UnLoadBelow(TFMStatus status,dword sizeneeded,dword &sizesofar);
   void DestroyNoOwners();
private:
  int16 Hash(const string &name);
  TAudioWaveMix *PAudioMgr;
  int16 CurrentList;
};
//*******************************************************************
class OGLEOWLCLASS TChannelWindow:public TOgleHideWin//TDialog
{
private:
  bool Active;
  TWindow *PWin;
  TWaveMixControlData *PControlData;
public:
  TChannelWindow(TWaveMixControlData *pcontroldata,TWindow *pwin);
  virtual ~TChannelWindow();
  bool IsActive();
  void SetActive(bool active);
  LRESULT EvMMWomDone(WPARAM wParam,LPARAM lParam);
  DECLARE_RESPONSE_TABLE(TChannelWindow);
};
//*******************************************************************
class TWaveMixControlData
{
private:
  TAudioWaveMix *PAudioWaveMix;
  int16 Channel;

public:
  TWaveMixControlData(TAudioWaveMix *paudio,int16 channel);
  ~TWaveMixControlData();
  bool Valid;
  TWav *PWav;
  word LoopCount;
  bool WavWantsNotifyStop;
  MIXPLAYPARAMS MixPlayParams;
  bool Submitted;
  TChannelWindow *PChannelWindow;
  void Prep(TWav *pwav,word loopcount = 0);
  void Submit(dword flags);
  void NotifyStop(int16 channel,LPMIXWAVE pmixwave);
  void Stop();
  bool IsPlaying();
  bool IsPlaying(LPMIXWAVE pmixwave);
};
//*******************************************************************
class TPlayQData
{
public:
  TWav *PWav;
  word LoopCount;
  TPlayQData(TWav *pwav,word loopcount = 0);
  ~TPlayQData();
};
//*******************************************************************
class TPlayQ:public TSliq
{
public:
  TPlayQ();
  virtual ~TPlayQ();
  void Stop(bool notifystop = false);
  bool Add(TPlayQData *pdata);
  TPlayQData *Remove();
  bool Remove(TWav *pwav,bool allmatches = false);
  TPlayQData *GetFirstData();
  TPlayQData *GetNextData();
  bool IsPlaying(LPMIXWAVE lpmixwave);
};
//*******************************************************************
class TAudioWaveMix
{
protected:
  TOgleApp *POgleApp;
  int16 ErrorCount;
  dword LastTimeCount;
  bool CheckLoadStatus(TWav *pwav);
  bool CheckControlData();
  int16 UnStickControlData();

public:
  TOgleApp *GetOgleApp();
  dword SizeUsed;
  void IncSizeUsed(dword size);
  void DecSizeUsed(dword size);
  dword GetSizeUsed();
  void Register(TWav *pwav);
  void UnRegister(TWav *pwav);
  void ChangeStatus(TWav *pwav,TFMStatus status);
  bool UnLoadBelow(TFMStatus status,dword sizeneeded,dword &sizesofar);
  void ModeActivated();
  void ModeDeActivated();
public:
  HANDLE hMixSession;
  TWindow *PWin;
  bool Active;

  TWaveMixControlData *PControlData[NUM_AUDIO_CHANNELS];
  TPlayQ PlayQ;
  TAudioWaveMix(TWindow *pwin);
  virtual ~TAudioWaveMix();
  bool Initialized;
  bool IsInitialized();
  bool Initialize();
  bool DeInitialize();
  void Configure();
  void CreateControlData();
  void DestroyControlData();
  string GetMMErrorStr(uint err);
  void ErrorBox(string msg);
  void MMErrorBox(uint err);
  void Activate();
  void DeActivate();
  bool Add(TWav *pwav,word loopcount = 0);
  bool Remove(TWav *pwav);
  void ClearNotifyStop(TWav *pwav);
  void Stop(bool notifystop = false);
  int16 GetNumOpenChannels();
  int16 GetNextOpenChannel();
  bool PrepPlayQ();
  void SubmitPrepped();
  void Update(dword fuc);
  void Pump();
  bool IsPlaying();
  uint GetNumDevs();
  void ToggleMute();
  void SetMute(bool mute);
  bool IsMuted();
  void GetIniValues();
  void SetIniValues();
  bool IsPlaying(LPMIXWAVE pmixwave);

private:
  bool Muted;

protected:
  TAudioDSArray *PDSArray;
};
//*******************************************************************
//******************* TAudioData inlines ****************************
//*******************************************************************
inline bool TAudioData::IsLoaded()
{
  return bool(PMixWave != NULL);
}
//*******************************************************************
inline void TAudioData::DecCount(TFMStatus status)
{
  if (StatusCounts[status] > 0) {StatusCounts[status] -= int16(1);}
}
//*******************************************************************
inline void TAudioData::IncCount(TFMStatus status)
{
  StatusCounts[status] += int16(1);
}
//*******************************************************************
inline void TAudioData::IncOwnerCount()
{
  OwnerCount++;
}
//*******************************************************************
inline void TAudioData::DecOwnerCount()
{
  if (OwnerCount > 0) {OwnerCount--;}
}
//*******************************************************************
inline word TAudioData::GetOwnerCount()
{
  return OwnerCount;
}
//*******************************************************************
inline void TAudioData::SetStatus()
{
  if (StatusCounts[TFileMode::fm_Immed] > 0) {Status = TFileMode::fm_Immed;}
  else if (StatusCounts[TFileMode::fm_BG_Hi] > 0) {Status = TFileMode::fm_BG_Hi;}
  else if (StatusCounts[TFileMode::fm_BG_Med] > 0) {Status = TFileMode::fm_BG_Med;}
  else if (StatusCounts[TFileMode::fm_BG_Lo] > 0) {Status = TFileMode::fm_BG_Lo;}
  else {Status = TFileMode::fm_None;}
}
//*******************************************************************
//******************* TAudioDataSliq inlines  ***********************
//*******************************************************************
inline bool TAudioDataSliq::Add(TAudioData *pad)
{
  return Push(pad);
}
//*******************************************************************
inline TAudioData *TAudioDataSliq::PopAD()
{
  return (TAudioData *)Pop();
}
//*******************************************************************
inline TAudioData *TAudioDataSliq::GetFirst()
{
  return (TAudioData *)LTravFirst();
}
//*******************************************************************
inline TAudioData *TAudioDataSliq::GetNext()
{
  return (TAudioData *)LTravNext();
}
//*******************************************************************
//****************** TAudioDSArray inlines **************************
//*******************************************************************
inline bool TAudioDSArray::Add(const string &name,TAudioData *pad)
{
  int16 listnum = Hash(name);
  return PDS[listnum]->Add(pad);
}
//*******************************************************************
inline TAudioData *TAudioDSArray::GetAudioData(const string &name)
{
  int16 listnum = Hash(name);
  return PDS[listnum]->GetAudioData(name);
}
//*******************************************************************
//****************** TChannelWindow inlines *************************
//*******************************************************************
inline bool TChannelWindow::IsActive()
{
  return Active;
}
//*******************************************************************
inline void TChannelWindow::SetActive(bool active)
{
  Active = active;
}
//*******************************************************************
//******************** TWaveMixControlData inlines ******************
//*******************************************************************
inline bool TWaveMixControlData::IsPlaying()
{
  if (Valid && Submitted) {return true;}
  return false;
}
//*******************************************************************
inline bool TWaveMixControlData::IsPlaying(LPMIXWAVE pmixwave)
{
  if (Valid && MixPlayParams.lpMixWave == pmixwave) {return true;}
  return false;
}
//*******************************************************************
//**************** TPlayQData inlines *******************************
//*******************************************************************
inline TPlayQData::TPlayQData(TWav *pwav,word loopcount)
{
  PWav = pwav;
  LoopCount = loopcount;
}
//*******************************************************************
inline TPlayQData::~TPlayQData()
{
}
//*******************************************************************
//******************** TPlayQ inlines *******************************
//*******************************************************************
inline bool TPlayQ::Add(TPlayQData *pdata)
{
  return QAdd(pdata);
}
//*******************************************************************
inline TPlayQData *TPlayQ::Remove()
{
  return (TPlayQData *)QRemove();
}
//*******************************************************************
inline TPlayQData *TPlayQ::GetFirstData()
{
  return (TPlayQData *)LTravFirst();
}
//*******************************************************************
inline TPlayQData *TPlayQ::GetNextData()
{
  return (TPlayQData *)LTravNext();
}
//*******************************************************************
//********************** TAudioWaveMix inlines **********************
//*******************************************************************
inline TOgleApp *TAudioWaveMix::GetOgleApp()
{
  return POgleApp;
}
//*******************************************************************
inline void TAudioWaveMix::IncSizeUsed(dword size)
{
  SizeUsed += size;
  TOgleMemMgr::IncAudioSize(size);
}
//*******************************************************************
inline void TAudioWaveMix::DecSizeUsed(dword size)
{
  if (size > SizeUsed) {SizeUsed = 0;}
  else                 {SizeUsed -= size;}
  TOgleMemMgr::DecAudioSize(size);
}
//*******************************************************************
inline dword TAudioWaveMix::GetSizeUsed()
{
  return SizeUsed;
}
//*******************************************************************
inline bool TAudioWaveMix::UnLoadBelow(TFMStatus status,dword sizeneeded,dword &sizesofar)
{
  if (PDSArray == NULL) {return false;}
  return PDSArray->UnLoadBelow(status,sizeneeded,sizesofar);
}
//*******************************************************************
inline void TAudioWaveMix::ModeActivated()
{
}
//*******************************************************************
inline bool TAudioWaveMix::IsInitialized()
{
  return Initialized;
}
//*******************************************************************
inline void TAudioWaveMix::MMErrorBox(uint err)
{
  ErrorBox(GetMMErrorStr(err));
}
//*******************************************************************
inline void TAudioWaveMix::Pump()
{
  TWaveMix::Dll()->Pump();
}
//*******************************************************************
inline uint TAudioWaveMix::GetNumDevs()
{
  return waveOutGetNumDevs();
}
//*******************************************************************
inline void TAudioWaveMix::ToggleMute()
{
  SetMute(!Muted);
}
//*******************************************************************
inline bool TAudioWaveMix::IsMuted()
{
  return Muted;
}
//*******************************************************************
#endif
