//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ofile.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0    3/26/96     M.Perry    original baseline
//  2.0    8/26/97     M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__ofile_h )
#define __ofile_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__STDIO_H)
#include <stdio.h>
#endif

#if !defined(__ofilenam_h)
#include "ofilenam.h"
#endif

#if !defined(OGLE_PLAT_OS_WIN16) && !defined(OGLE_PLAT_OS_WIN32)
  #error Ogle - Unknown platform
#endif

//********************** enums and defines *****************

#if defined(OGLE_PLAT_OS_WIN16)
  #define OF_INVALID_HANDLE HFILE_ERROR
  #define OF_OS_INVALID_SEEK -1
#elif defined(OGLE_PLAT_OS_WIN32)
  #define OF_INVALID_HANDLE INVALID_HANDLE_VALUE
  #define OF_OS_INVALID_SEEK 0xFFFFFFFF
#endif

enum TCodes{OF_INVALID_SEEK = -1,
            OF_NONE = 0,
              #if defined(OGLE_PLAT_OS_WIN16)
//              OF_INVALID_HANDLE = HFILE_ERROR,
//              OF_OS_INVALID_SEEK = ???
              #elif defined(OGLE_PLAT_OS_WIN32)
//              OF_INVALID_HANDLE = INVALID_HANDLE_VALUE,
//              OF_OS_INVALID_SEEK = 0xFFFFFFFF
              #endif
             };

enum TSeekDir {
                 #if defined(OGLE_PLAT_OS_WIN16)
                 OFSD_BEGIN = 0,
                 OFSD_CURRENT = 1,
                 OFSD_END = 2
                 #elif defined(OGLE_PLAT_OS_WIN32)
                 OFSD_BEGIN = FILE_BEGIN,
                 OFSD_CURRENT = FILE_CURRENT,
                 OFSD_END = FILE_END
                 #endif
                };

  enum TAccess {OFA_NONE = 0,
                #if defined(OGLE_PLAT_OS_WIN16)
                OFA_READ = OF_READ,
                OFA_WRITE = OF_WRITE,
                OFA_READWRITE = OF_READ  | OF_WRITE
                #elif defined(OGLE_PLAT_OS_WIN32)
                OFA_READ = GENERIC_READ,
                OFA_WRITE = GENERIC_WRITE,
                OFA_READWRITE = GENERIC_READ | GENERIC_WRITE
                #endif
               };

  enum TShare {OFS_NONE = 0,
               #if defined(OGLE_PLAT_OS_WIN16)
               OFS_EXCLUSIVE = OF_SHARE_EXCLUSIVE,
               OFS_READ = OF_SHARE_DENY_WRITE,
               OFS_WRITE = OF_SHARE_DENY_READ,
               OFS_READWRITE = OF_SHARE_DENY_NONE,
               #elif defined(OGLE_PLAT_OS_WIN32)
               OFS_EXCLUSIVE = OFS_NONE,
               OFS_READ = FILE_SHARE_READ,
               OFS_WRITE = FILE_SHARE_WRITE,
               OFS_READWRITE = FILE_SHARE_READ | FILE_SHARE_WRITE,
               #endif
              };

  enum TCreation {OFC_NONE = 0,
                  #if defined(OGLE_PLAT_OS_WIN16)
                  OFC_CREATE_NEW = ,
                  OFC_CREATE_ALWAYS = OF_CREATE,
                  OFC_OPEN_EXISTING = ,
                  OFC_OPEN_ALWAYS = ,
                  OFC_TRUNCATE_EXISTING =
                  #elif defined(OGLE_PLAT_OS_WIN32)
                  OFC_CREATE_NEW = CREATE_NEW,
                  OFC_CREATE_ALWAYS = CREATE_ALWAYS,
                  OFC_OPEN_EXISTING = OPEN_EXISTING,
                  OFC_OPEN_ALWAYS = OPEN_ALWAYS,
                  OFC_TRUNCATE_EXISTING = TRUNCATE_EXISTING
                  #endif
                 };

  enum TAttributes {OFAT_NONE = 0,
                    #if defined(OGLE_PLAT_OS_WIN16)
                    // probably need dos.h defines here.
                    #elif defined(OGLE_PLAT_OS_WIN32)
                    OFAT_NORMAL = FILE_ATTRIBUTE_NORMAL,
                    OFAT_ARCHIVE = FILE_ATTRIBUTE_ARCHIVE,
                    OFAT_COMPRESSED = FILE_ATTRIBUTE_COMPRESSED,
                    OFAT_HIDDEN = FILE_ATTRIBUTE_HIDDEN,
                    OFAT_OFFLINE = FILE_ATTRIBUTE_OFFLINE,
                    OFAT_READONLY = FILE_ATTRIBUTE_READONLY,
                    OFAT_SYSTEM = FILE_ATTRIBUTE_SYSTEM,
                    OFAT_TEMPORARY = FILE_ATTRIBUTE_TEMPORARY
                    #endif
                   };

  enum TOSFlags {OFO_NONE = 0,
                 #if defined(OGLE_PLAT_OS_WIN16)
                 #elif defined(OGLE_PLAT_OS_WIN32)
                 OFO_WRITE_THROUGH = FILE_FLAG_WRITE_THROUGH,
                 OFO_OVERLAPPED = FILE_FLAG_OVERLAPPED,
                 OFO_NO_BUFFERING = FILE_FLAG_NO_BUFFERING,
                 OFO_RANDOM_ACCESS = FILE_FLAG_RANDOM_ACCESS,
                 OFO_SEQUENTIAL_SCAN = FILE_FLAG_SEQUENTIAL_SCAN,
                 OFO_DELETE_ON_CLOSE = FILE_FLAG_DELETE_ON_CLOSE
                 #endif
                };

//*******************************************************************
class TOgleFile
{
protected:
  int32 FileSize;
//  string FilePathName;
  TOgleFileName FileName;

  #if defined(OGLE_PLAT_OS_WIN16)
  typedef HFILE TFileHandle;
//  HFILE FileHandle;
  #elif defined(OGLE_PLAT_OS_WIN32)
  typedef HANDLE TFileHandle;
//  HANDLE FileHandle;
  #endif

  TFileHandle FileHandle;

//  uint OpenFlags;
  bool ShouldClose;

  dword AccessFlags;
  dword ShareFlags;
  dword CreationFlags;
  dword AttributeFlags;
  dword OSFlags;

public:
//  TOgleFile(const string &pathName,uint fuMode = OF_READ,
//            bool shouldOpen = true,bool shouldClose = true);

  TOgleFile(const TOgleFileName &ofn,
            dword access = OFA_READ,
            dword share = OFS_READ,
            dword creation = OFC_OPEN_EXISTING,
            dword os = OFO_NONE,
            dword attributes = OFAT_NORMAL,
            bool shouldOpen = true,
            bool shouldClose = true);

  virtual ~TOgleFile();

  bool IsOpen() const;
  virtual bool Open();
//  bool Open(const string &pathName,uint fuMode,bool shouldClose = true);
  virtual bool Open(const TOgleFileName &ofn,
                    dword access = OFA_READ,
                    dword share = OFS_READ,
                    dword creation = OFC_OPEN_EXISTING,
                    dword os = OFO_NONE,
                    dword attributes = OFAT_NORMAL,
                    bool shouldClose = true);

  virtual bool Close();

  //  HFILE GetFileHandle() const;
  TFileHandle GetFileHandle() const;

  //  void SetFileHandle(HFILE hfile);
  void SetFileHandle(TFileHandle filehandle);

  bool GetFileHandleValid() const {return bool(GetFileHandle() != OF_INVALID_HANDLE);}

  int32 GetFileSize();
  void SetFileSize(int32 size);
  int32 GetPosition();

  int32 Seek( int32 offset, TSeekDir origin = OFSD_BEGIN );
  int32 SeekToBegin();
  int32 SeekToEnd();
  int32 SeekFromCur(int32 offset);

  bool Read(void huge *buffer,int32 numBytes,int32 &bytesRead);
  bool Read(void huge *buffer,int32 numBytes);

  virtual bool Write(const void huge *buffer,int32 numBytes,int32 &bytesWritten);
  virtual bool Write(const void huge *buffer,int32 numBytes);

//  bool Remove(const string &name);
//  bool Remove();
  bool Delete();

//  bool Rename(const string &newname,const string &oldname);
  bool Rename(const TOgleFileName &ofn);
  bool Rename(const string &s);

  void SetShouldClose(bool close = true);
  bool GetShouldClose();
  
protected:
  void SetFileName(const string &name);
  void SetFileName(const TOgleFileName &ofn);

  TOgleFileName &GetFileName();
  const string &GetFileNameString();

//  void SetOpenFlags(uint flags);
//  uint GetOpenFlags();
};
//*******************************************************************
class TFileWriteBuf : public TOgleFile
{
protected:
//  string FileName;
//  HFILE FileHandle;
  void *PBuffer;
  int32 BufferSize;
  int32 BufferOffset;

public:
  TFileWriteBuf(const TOgleFileName &name,int32 bufsize = 32000l);
  virtual ~TFileWriteBuf();
  virtual bool Write(const void huge *ptr,int32 size);
  virtual bool Write(const void huge *ptr,int32 size,int32 &byteswritten)
  {
    return TOgleFile::Write(ptr,size,byteswritten);
  }
  bool Flush();
//  bool IsOpen() {return (FileHandle != HFILE_ERROR);}
protected:
  bool WriteToFile();
//  virtual bool Open();
  virtual bool Close();
};
//*******************************************************************
//********************* TOgleFile inlines ***************************
//*******************************************************************
inline bool TOgleFile::IsOpen() const
{
  return bool(GetFileHandle() != OF_INVALID_HANDLE);
//  #if defined(OGLE_PLAT_OS_WIN16)
//  return bool(GetFileHandle() != HFILE_ERROR);
//  #elif defined(OGLE_PLAT_OS_WIN32)
//  return bool(GetFileHandle() != INVALID_HANDLE_VALUE);
//  #endif
}
//*******************************************************************
//inline HFILE TOgleFile::GetFileHandle() const
inline TOgleFile::TFileHandle TOgleFile::GetFileHandle() const
{
  return FileHandle;
}
//*******************************************************************
//inline void TOgleFile::SetFileHandle(HFILE hfile)
inline void TOgleFile::SetFileHandle(TFileHandle filehandle)
{
//  FileHandle = hfile;
  FileHandle = filehandle;
}
//*******************************************************************
inline void TOgleFile::SetFileSize(int32 size)
{
  FileSize = size;
}
//*******************************************************************
inline int32 TOgleFile::GetPosition()
{
  return Seek(0,OFSD_CURRENT);
}
//*******************************************************************
inline int32 TOgleFile::SeekToBegin()
{
  return Seek(0,OFSD_BEGIN);
}
//*******************************************************************
inline int32 TOgleFile::SeekToEnd()
{
  return Seek(0,OFSD_END);
}
//*******************************************************************
inline int32 TOgleFile::SeekFromCur(int32 offset)
{
  return Seek(offset,OFSD_CURRENT);
}
//*******************************************************************
inline bool TOgleFile::Read(void huge *buffer,int32 numBytes)
{
  int32 bytesread = 0;
  return Read(buffer,numBytes,bytesread);
}
//*******************************************************************
inline bool TOgleFile::Write(const void huge *buffer,int32 numBytes)
{
  int32 byteswritten = 0;
  return Write(buffer,numBytes,byteswritten);
}
//*******************************************************************
inline bool TOgleFile::Delete()//const string &name)
{
  if (IsOpen()) {return false;}

  #if defined(OGLE_PLAT_OS_WIN16)
  int result = ::remove(FileName.GetStr());//name.c_str());
  return bool(result == 0);
  #elif defined(OGLE_PLAT_OS_WIN32)
  return (bool)DeleteFile(FileName.GetStr().c_str());
  #endif
}
//*******************************************************************
inline void TOgleFile::SetShouldClose(bool close)
{
  ShouldClose = close;
}
//*******************************************************************
inline bool TOgleFile::GetShouldClose()
{
  return ShouldClose;
}
//*******************************************************************
inline void TOgleFile::SetFileName(const string &name)
{
//  FilePathName = name;
  FileName = name;
}
//*******************************************************************
inline void TOgleFile::SetFileName(const TOgleFileName &ofn)
{
  FileName = ofn;
}
//*******************************************************************
inline const string &TOgleFile::GetFileNameString()
{
//  return FilePathName;
  return FileName.GetStr();
}
//*******************************************************************
inline TOgleFileName &TOgleFile::GetFileName()
{
  return FileName;
}
//*******************************************************************
//inline void TOgleFile::SetOpenFlags(uint flags)
//{
//  OpenFlags = flags;
//}
//*******************************************************************
//inline uint TOgleFile::GetOpenFlags()
//{
//  return OpenFlags;
//}
//*******************************************************************
#endif

