//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ofilemgr.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0      3/2/95    M.Perry     original baseline
//  2.0      8/25/97   M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__ofilemgr_h)
#define __ofilemgr_h                  

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__ofmode_h)
#include "ofmode.h"
#endif

#if !defined(__obasef_h)
#include "obasef.h"
#endif

#if !defined(__osliq_h)
#include "osliq.h"
#endif

#if !defined(__STDLIB_H)
#include <stdlib.h>
#endif

#if !defined(__IO_H)
#include <io.h>
#endif

#if !defined(__FCNTL_H)
#include <fcntl.h>
#endif

#if !defined(__CSTRING_H)
#include <cstring.h>
#endif

#if !defined(__omisc_h)
#include "omisc.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if defined(DEBUGUPDATES) && !defined(__osys_h)
#include "osys.h"
#endif

#define MAXDIRS (int8_max)
#define NUMFILELISTS 400
#define MAXLOADTRIES 3

class TOgleIniFile;
class TDet;
class TDetList;
class TCursorMgr;
class TOgleFileMgr;
class TOgleSoundMgr;
//*******************************************************************
#if defined(DEBUGUPDATES)
class TDebugFMUpdate
{
public:
  dword UpdateCount;
  dword PRStart;
  dword PREnd;
  dword PRTotals;

public:
  void ClearData();
  TDebugFMUpdate();
  ~TDebugFMUpdate();
  void StartPR();
  void EndPR();
  dword GetPRAvg();

protected:
  dword GetSysTime();
};
#endif
//*******************************************************************
class TFMData
{
private:
  string emptystr;

protected:
  dword LastLoadFUC;
  dword CurrentFUC;
  string DirNames[MAXDIRS];

public:
  TFMData();
  ~TFMData();

  // FUC data
  dword GetLastLoadFUC();
  void SetLastLoadFUC(dword fuc);
  dword GetCurrentFUC();
  void SetCurrentFUC(dword fuc);

  // start of dir index stuff that's going to go away.
  void SetDir(int8 dirindex,const string &dirnameonly);
  string &GetDirName(int8 dirindex);
  // end of stuff going away.
};
//*******************************************************************
class TFMNames
{
protected:
  string drive;
  string dir;
  string diradd;
  string fullpath;

public:
  TFMNames();
  ~TFMNames();
  string &GetDrive();
  void SetDrive(const string &s);
  string &GetDir();
  void SetDir(const string &s);
  string &GetDirAdd();
  void SetDirAdd(const string &s);
  void SetDirAdd(const char *pc);
  string GetFullDir();
  string GetFullPath();
  bool CheckFullPathExists();
  void MakeFullPath();
  bool IsDrive(const string &s);
};
//*******************************************************************
class TFMCDDataNames:public TFMNames
{
public:
  TFMCDDataNames();
  TFMCDDataNames(const string &drive,const string &dir);
  ~TFMCDDataNames();
};
//*******************************************************************
class TFMHDBufferNames:public TFMNames
{
public:
  dword UniqueFileNameCounter;
  string GetUniqueFileName();
  int16 UniqueFileNameCounterRollOver;

public:
  TFMHDBufferNames();
  TFMHDBufferNames(const string &drive,const string &dir);
  ~TFMHDBufferNames();
};
//*******************************************************************
class TFMHDDataNames:public TFMNames
{
public:
  TFMHDDataNames();
  TFMHDDataNames(const string &drive,const string &dir);
  ~TFMHDDataNames();
};
//*******************************************************************
class TFMDiskData
// like ram data but handles hd space.
{
public:
  dword TotalHDAllocated;

protected:
  dword LastAllocatedSize;
  dword LastAvailSize;
  int16 DriveNum;

public:
  TFMDiskData(int16 drivenum);
  dword GetActualDiskSpace();
  dword GetTotalAllocated();
  ~TFMDiskData();
  dword GetThresholdSize();
  void UpdateData();
  dword GetAvail();
  bool IsHDSpaceOk();
  bool WillFit(dword size);
  void PostUpdate(dword fuc);
  dword GetThresholdDelta();

private:
  bool GetDiskFreeData(dword &clusters,dword &clustersectors,dword &sectorbytes);
};
//*******************************************************************
TCompareResult CompareHiRamNeeded(void *FirstObj,void *SecObj);
TCompareResult CompareLoRamNeeded(void *FirstObj,void *SecObj);
//*******************************************************************
class TFileList:public TSliq
{
public:
  TFileList(bool VAllowEqObjPtr);
  ~TFileList();
  void SortOnRamNeededMost();
  void SortOnRamNeededLeast();
  int16 GetNumBaseFiles();
  TOgleBaseFile *GetFirst();
  TOgleBaseFile *GetNext();
  bool AddFile(TOgleBaseFile *PBaseFile);
  bool RemoveFile(TOgleBaseFile *PBaseFile);
  TOgleBaseFile *GetWithRamStatus(TFMStatus ramstatus);
  TOgleBaseFile *GetWithHDStatus(TFMStatus hdstatus);
  TOgleBaseFile *GetPBF(const string &name);
  TOgleBaseFile *GetWithOwnerCount(int16 count);
  bool DisposePBF(TOgleBaseFile *pbf);
  void DisposeAllWithOwnerCount(int16 count = 0);
  void DisposeNoOwners();
};
//*******************************************************************
class TFMFLData
{
protected:
  TOgleBaseFile *pfirstram;
  TOgleBaseFile *pfirsthd;
  TOgleFileMgr *PFMgr;

public:
  TFileList *PShareLists[NUMFILELISTS];
  TFileList *PNonShareList;
  int16 CurrentRamList;
  int16 CurrentHDList;

public:
  TFMFLData(TOgleFileMgr *pfmgr);
  ~TFMFLData();
  TOgleBaseFile *GetFirstWRamStatus(TFMStatus ramstatus);
  TOgleBaseFile *GetNextWRamStatus(TFMStatus ramstatus);
  TOgleBaseFile *GetFirstWHDStatus(TFMStatus hdstatus);
  TOgleBaseFile *GetNextWHDStatus(TFMStatus hdstatus);
  void DestroyNoOwners();
  bool UnLoadRamWithStatus(TFileList *plist,TFMStatus ramstatus,dword sizeneeded,dword &sizesofar);
  bool UnLoadHDWithStatus(TFileList *plist,TFMStatus hdstatus,dword sizeneeded,dword &sizesofar);
  bool UnLoadRamBelow(TFMStatus ramstatus,dword size,dword &sizesofar);
  bool UnLoadRamBelow(TFMStatus ramstatus,dword size);
  bool UnLoadHDBelow(TFMStatus hdstatus,dword size,dword &sizesofar);
  bool UnLoadHDBelow(TFMStatus hdstatus,dword size);
  bool CheckListNum(int16 listnum);
  int16 Hash(const string &name);
  int16 GetListNum(string &name);
  int16 GetListNum(TFileMode &fm);
  int16 GetListNum(TOgleBaseFile *pbf);
  TOgleBaseFile *GetPBF(TFileMode &FileMode);
  TFileList *SelectList(TFileMode &FileMode);
  TFileList *SelectList(TOgleBaseFile *PBaseFile);
  bool AddToList(TOgleBaseFile *PBaseFile,TFileMode &filemode);
  bool RemoveFromList(TFileList *whichList,TOgleBaseFile *PBaseFile);
  TOgleBaseFile *CheckListForFileName(TFileList *whichList,string BaseName);
  bool CreateLists();
  bool DestroyLists();
};
//*******************************************************************
class TOgleFileMgr
{
friend class TFMFLData;
private:
  TOgleApp *POgleApp;
  TFMDiskData *PDiskDat;
  TFMFLData *PFLDat;
  bool GlobalHDBuffering;
  bool InsideCore;

protected:
  bool Initialized;
  bool InitializedOk;

public:
  #if defined(DEBUGUPDATES)
  TDebugFMUpdate DebugFMUpdate;
  #endif
  TFMData *PDat;
  TFMHDBufferNames *PHDB;
  TFMCDDataNames *PCD;
  TFMHDDataNames *PHDD;
  int32 NumIdleHits;

public:
  TOgleSoundMgr *GetSoundMgr();
  bool IsGlobalHDBufferingOn();
  void SetGlobalHDBuffering(bool on = true);
  void ToggleHDBuffering();
  void IncTotalHDAllocated(dword size);
  void DecTotalHDAllocated(dword size);
  dword GetTotalHDAllocated();
  TOgleApp *GetOgleApp();
  string GetCDRomPath(int16 dirindex);
  TDet *CreateDet(string &detfname);
  bool RegisterDet(string &detfname);
  bool UnRegisterDet(string &detfname);
  void Activate();
  void DeActivate();
  void Idle();
  void IdleLoad();
  void BufferFiles(int16 num);
  void Update(dword NewFUC);
  void PostUpdate(dword NewFUC);
  void ModeActivated();
  void ModeDeActivated();
  void Info();
  void NotifyCompacting(uint compactRatio,bool now = false);
  string GetResString(uint id);
  TOgleIniFile *GetIniFile();
  TOgleFileMgr(TOgleApp *papp);
  ~TOgleFileMgr();

  // Start of Registering methods.
  TOgleBaseFile *IsRegistered(TFileMode &FileMode);
  bool Register(TOgleBaseFile *PBaseFile,TFileMode &FileMode);
  bool RegisterNewOwner(TOgleBaseFile *PBaseFile,TFileMode &FileMode);
  void UnRegister(TOgleBaseFile *PBaseFile,TFileMode &FileMode);
  bool ChangeRegisteredMode(TOgleBaseFile *PBaseFile,
                            TFileMode &OldFileMode,
                            const TFileMode &NewFileMode);
  // End of Registering methods.

protected:
  bool Initialize();
  void DeInitialize();
  string GetRamStr(string msg);
  void FatalError(string msg = "");
  bool PromptCDInsert(string filename);

  // Start of Register helper methods.
  void CheckRam();
  void CheckHD();
  bool Process(TOgleBaseFile *PBaseFile,int16 loadram,int16 loadhd);
  bool ProcessRegister(TOgleBaseFile *PBaseFile);
  bool ProcessBG(TOgleBaseFile *PBaseFile,int16 loadramreq,int16 loadhdreq);
  void IncModeCounters(TOgleBaseFile *PBaseFile,const TFileMode &FileMode);
  void DecModeCounters(TOgleBaseFile *PBaseFile,const TFileMode &FileMode);
  bool LoadFileNames(TOgleBaseFile *PBaseFile,TFileMode &FileMode);
  // End of Register helper methods.
                   
  // Start of Ram & HD caching methods.
  bool HandleCDReadFail(TOgleBaseFile *PBaseFile);
  bool LoadRamHD(TOgleBaseFile *PBaseFile,TFMStatus rampri,TFMStatus hdpri);
  bool LoadRam(TOgleBaseFile *pbf,TFMStatus rampri = TFileMode::fm_None);
  bool UnLoadRam(TOgleBaseFile *PBaseFile);
  bool HandleHDWriteFail(TOgleBaseFile *PBaseFile,TFMStatus hdpri);
  bool LoadHD(TOgleBaseFile *PBaseFile,TFMStatus hdpri = TFileMode::fm_None);
  bool UnLoadHD(TOgleBaseFile *PBaseFile);

private:
  TCursorMgr *GetCursorMgr();
  bool IsBGLoadOk();
  TDetList *PDetList;
  void CreateDetList();
  void DestroyDetList();
  void ReleaseUnUsedDets();
};
//*******************************************************************
//***************** TDebugFMUpdate inlines **************************
//*******************************************************************
#if defined(DEBUGUPDATES)
inline dword TDebugFMUpdate::GetSysTime()
{
  return TSys::GetMSec();
}
//*******************************************************************
inline void TDebugFMUpdate::ClearData()
{
  UpdateCount = 0;
  PRTotals = 0;
}
//*******************************************************************
inline TDebugFMUpdate::TDebugFMUpdate()
{
  ClearData();
}
//*******************************************************************
inline TDebugFMUpdate::~TDebugFMUpdate()
{
}
//*******************************************************************
inline void TDebugFMUpdate::StartPR()
{
  PRStart = GetSysTime();
}
//*******************************************************************
inline void TDebugFMUpdate::EndPR()
{
  PREnd = GetSysTime();
  UpdateCount++;
  PRTotals += (PREnd - PRStart);
}
//*******************************************************************
inline dword TDebugFMUpdate::GetPRAvg()
{
  return UpdateCount?(PRTotals / UpdateCount):0;
}
#endif
//*******************************************************************
//***************** TFMData inlines *********************************
//*******************************************************************
inline dword TFMData::GetLastLoadFUC()
{
  return LastLoadFUC;
}
//*******************************************************************
inline void TFMData::SetLastLoadFUC(dword fuc)
{
  LastLoadFUC = fuc;
}
//*******************************************************************
inline dword TFMData::GetCurrentFUC()
{
  return CurrentFUC;
}
//*******************************************************************
inline void TFMData::SetCurrentFUC(dword fuc)
{
  CurrentFUC = fuc;
}
//*******************************************************************
//***************** TFMNames inlines ********************************
//*******************************************************************
inline TFMNames::TFMNames()
{
}
//*******************************************************************
inline TFMNames::~TFMNames()
{
}
//*******************************************************************
inline string &TFMNames::GetDrive()
{
  return drive;
}
//*******************************************************************
inline void TFMNames::SetDrive(const string &s)
{
  drive = s;
  MakeFullPath();
}
//*******************************************************************
inline string &TFMNames::GetDir()
{
  return dir;
}
//*******************************************************************
inline void TFMNames::SetDir(const string &s)
{
  dir = s;
  MakeFullPath();
}
//*******************************************************************
inline string &TFMNames::GetDirAdd()
{
  return diradd;
}
//*******************************************************************
inline void TFMNames::SetDirAdd(const string &s)
{
  diradd = s;
  MakeFullPath();
}
//*******************************************************************
inline void TFMNames::SetDirAdd(const char *pc)
{
  diradd = pc;
  MakeFullPath();
}
//*******************************************************************
inline string TFMNames::GetFullPath()
{
  return fullpath;
}
//*******************************************************************
inline bool TFMNames::CheckFullPathExists()
{
  return TOgleFileName::DirExists(fullpath);
}
//*******************************************************************
inline void TFMNames::MakeFullPath()
{
  fullpath = drive + dir + diradd;
}
//*******************************************************************
//***************** TFMCDDataNames inlines **************************
//*******************************************************************
inline TFMCDDataNames::TFMCDDataNames()
{
}
//*******************************************************************
inline TFMCDDataNames::~TFMCDDataNames()
{
}
//*******************************************************************
//***************** TFMHDBufferNames inlines ************************
//*******************************************************************
inline TFMHDBufferNames::~TFMHDBufferNames()
{
}
//*******************************************************************
//***************** TFMHDDataNames inlines **************************
//*******************************************************************
inline TFMHDDataNames::~TFMHDDataNames()
{
}
//*******************************************************************
//***************** TFMDiskData inlines *****************************
//*******************************************************************
inline TFMDiskData::TFMDiskData(int16 drivenum)
:TotalHDAllocated(0),
 DriveNum(drivenum),
 LastAvailSize(GetActualDiskSpace()),
 LastAllocatedSize(0)
{
}
//*******************************************************************
inline dword TFMDiskData::GetTotalAllocated()
{
  return TotalHDAllocated;
}
//*******************************************************************
inline TFMDiskData::~TFMDiskData()
{
}
//*******************************************************************
inline dword TFMDiskData::GetThresholdSize()
{
  return (dword)5120000l;
}
//*******************************************************************
inline void TFMDiskData::UpdateData()
{
  LastAvailSize = GetActualDiskSpace();
  LastAllocatedSize = GetTotalAllocated();
}
//*******************************************************************
inline dword TFMDiskData::GetAvail()
{
  int32 delta = (int32)GetTotalAllocated() - (int32)LastAllocatedSize;
  int32 avail = (int32)LastAvailSize - delta;
  if (avail < 0) {avail = 0;}
  return (dword)avail;
}
//*******************************************************************
inline bool TFMDiskData::IsHDSpaceOk()
{
  // at least 5 meg.
  if (GetAvail() > GetThresholdSize()) {return true;}
  return false;
}
//*******************************************************************
inline bool TFMDiskData::WillFit(dword size)
{
  if (!IsHDSpaceOk()) {return false;}
  if (size <= GetAvail()) {return true;}
  return false;
}
//*******************************************************************
inline void TFMDiskData::PostUpdate(dword fuc)
{
  if ((fuc % 6) == 0) {UpdateData();}
}
//*******************************************************************
inline dword TFMDiskData::GetThresholdDelta()
{
  int32 temp = GetThresholdSize() - GetAvail();
  if (temp > 0) {return (dword)temp;}
  return (dword)0;
}
//*******************************************************************
//***************** TFileList inlines *******************************
//*******************************************************************
inline TFileList::TFileList(bool VAllowEqObjPtr)
:TSliq(VAllowEqObjPtr,true)
// ctor's tsliq 2nd param true will auto delete obj's it holding, ie. PBaseFile's.
{
}
//*******************************************************************
inline void TFileList::SortOnRamNeededMost()
{
  TCompareFunct OldCompareFunct = SetCompareFunct(CompareHiRamNeeded);
  SliqSort();
  SetCompareFunct(OldCompareFunct);
}
//*******************************************************************
inline void TFileList::SortOnRamNeededLeast()
{
  TCompareFunct OldCompareFunct = SetCompareFunct(CompareLoRamNeeded);
  SliqSort();
  SetCompareFunct(OldCompareFunct);
}
//*******************************************************************
inline int16 TFileList::GetNumBaseFiles()
{
  return RecCount;
}
//*******************************************************************
inline TOgleBaseFile *TFileList::GetFirst()
{
  return ((TOgleBaseFile *)LTravFirst());
}
//*******************************************************************
inline TOgleBaseFile *TFileList::GetNext()
{
  return ((TOgleBaseFile *)LTravNext());
}
//*******************************************************************
inline bool TFileList::AddFile(TOgleBaseFile *PBaseFile)
{
  return Push(PBaseFile);
}
//*******************************************************************
inline bool TFileList::RemoveFile(TOgleBaseFile *PBaseFile)
{
  return LMatchRemove(PBaseFile);
}
//*******************************************************************
inline void TFileList::DisposeNoOwners()
{
  DisposeAllWithOwnerCount(0);
}
//*******************************************************************
//***************** TFMFLData inlines *******************************
//*******************************************************************
inline bool TFMFLData::CheckListNum(int16 listnum)
{
  return ((listnum >= 0) && (listnum < NUMFILELISTS));
}
//*******************************************************************
inline int16 TFMFLData::GetListNum(string &name)
{
  return Hash(name);
}
//*******************************************************************
inline int16 TFMFLData::GetListNum(TFileMode &fm)
{
//  return Hash(fm.Name);
  return Hash(fm.Name.GetStr());
}
//*******************************************************************
inline bool TFMFLData::UnLoadRamBelow(TFMStatus ramstatus,dword size)
{
  dword sizesofar = 0;
  return UnLoadRamBelow(ramstatus,size,sizesofar);
}
//*******************************************************************
inline bool TFMFLData::UnLoadHDBelow(TFMStatus hdstatus,dword size)
{
  dword sizesofar = 0;
  return UnLoadHDBelow(hdstatus,size,sizesofar);
}
//*******************************************************************
inline int16 TFMFLData::GetListNum(TOgleBaseFile *pbf)
{
  if (pbf == NULL) {return -1;}
  else {return pbf->GetListNum();}
}
//*******************************************************************
inline TOgleBaseFile *TFMFLData::GetPBF(TFileMode &FileMode)
{
  TFileList *pfl = SelectList(FileMode);
  if (pfl == NULL) {return NULL;}
//  return pfl->GetPBF(FileMode.Name);
  return pfl->GetPBF(FileMode.Name.GetStr());
}
//*******************************************************************
inline TFileList *TFMFLData::SelectList(TFileMode &FileMode)
{
  if (!FileMode.IsShareable()) {return PNonShareList;}
  int16 listnum = GetListNum(FileMode);
  if (!CheckListNum(listnum)) {return NULL;}
  return PShareLists[listnum];
}
//*******************************************************************
inline TFileList *TFMFLData::SelectList(TOgleBaseFile *PBaseFile)
{
  if (PBaseFile == NULL) {return NULL;}
  if (!PBaseFile->IsShareable()) {return PNonShareList;}
  int16 listnum = PBaseFile->GetListNum();
  if (!CheckListNum(listnum)) {return NULL;}
  return PShareLists[listnum];
}
//*******************************************************************
inline bool TFMFLData::AddToList(TOgleBaseFile *PBaseFile,TFileMode &filemode)
{
  if (PBaseFile == NULL) {return false;}
  TFileList *pfl = SelectList(filemode);
  if (pfl == NULL) {return NULL;}
  if (pfl == PNonShareList)
  {
    PBaseFile->ClearShareable();
  }
  else
  {
    PBaseFile->SetShareable();
    PBaseFile->SetListNum(GetListNum(filemode));
  }
  if (!pfl->AddFile(PBaseFile)) {return false;}
  PBaseFile->SetOwnedByFileMgr();
  return true;
}
//*******************************************************************
inline bool TFMFLData::RemoveFromList(TFileList *whichList,TOgleBaseFile *PBaseFile)
{
  if (PBaseFile == NULL) {return false;}
  TFileList *pfl = SelectList(PBaseFile);
  if (pfl == NULL) {return false;}
  return(pfl->RemoveFile(PBaseFile));
}
//*******************************************************************
//***************** TOgleFileMgr inlines ****************************
//*******************************************************************
inline TOgleBaseFile *TOgleFileMgr::IsRegistered(TFileMode &FileMode)
{
  if (!FileMode.IsShareable()) {return NULL;}
  return PFLDat->GetPBF(FileMode);
}
//*******************************************************************
inline bool TOgleFileMgr::Register(TOgleBaseFile *PBaseFile,TFileMode &FileMode)
{
  if (PBaseFile == NULL) {return false;}
  if (!LoadFileNames(PBaseFile,FileMode)) {return false;}
  if (!PFLDat->AddToList(PBaseFile,FileMode)) {return false;}
  PBaseFile->SetOwnedByFileMgr();
  IncModeCounters(PBaseFile,FileMode);
  return ProcessRegister(PBaseFile);
}
//*******************************************************************
inline bool TOgleFileMgr::RegisterNewOwner(TOgleBaseFile *PBaseFile,TFileMode &FileMode)
{
  if (PBaseFile == NULL) {return false;}
  IncModeCounters(PBaseFile,FileMode);
  return ProcessRegister(PBaseFile);
}
//*******************************************************************
inline void TOgleFileMgr::UnRegister(TOgleBaseFile *PBaseFile,TFileMode &FileMode)
{
  if (PBaseFile == NULL) {return;}
  DecModeCounters(PBaseFile,FileMode);
  ProcessRegister(PBaseFile);
}
//*******************************************************************
inline bool TOgleFileMgr::ChangeRegisteredMode(TOgleBaseFile *PBaseFile,
                                               TFileMode &OldFileMode,
                                               const TFileMode &NewFileMode)
{
  if (PBaseFile == NULL) {return false;}
  DecModeCounters(PBaseFile,OldFileMode);
  IncModeCounters(PBaseFile,NewFileMode);
  OldFileMode = NewFileMode;
  return ProcessRegister(PBaseFile);
}
//*******************************************************************
inline void TOgleFileMgr::IncModeCounters(TOgleBaseFile *PBaseFile,const TFileMode &FileMode)
{
  if (PBaseFile == NULL) {return;}
  PBaseFile->IncOwnerCount();
  PBaseFile->IncStatus(FileMode.GetHDStatus(),FileMode.GetRamStatus());
}
//*******************************************************************
inline void TOgleFileMgr::DecModeCounters(TOgleBaseFile *PBaseFile,const TFileMode &FileMode)
{
  if (PBaseFile == NULL) {return;}
  PBaseFile->DecOwnerCount();
  PBaseFile->DecStatus(FileMode.GetHDStatus(),FileMode.GetRamStatus());
}
//*******************************************************************
inline bool TOgleFileMgr::UnLoadRam(TOgleBaseFile *PBaseFile)
{
  if (PBaseFile == NULL) {return true;}
  return PBaseFile->UnLoadRam();
}
//*******************************************************************
inline bool TOgleFileMgr::UnLoadHD(TOgleBaseFile *PBaseFile)
{
  if (PBaseFile == NULL) {return true;}
  return PBaseFile->UnLoadHD();
}
//*******************************************************************
inline TOgleSoundMgr *TOgleFileMgr::GetSoundMgr()
{
  return GetOgleApp()->GetSoundMgr();
}
//*******************************************************************
inline bool TOgleFileMgr::IsGlobalHDBufferingOn()
{
  return GlobalHDBuffering;
}
//*******************************************************************
inline void TOgleFileMgr::IncTotalHDAllocated(dword size)
{
  PDiskDat->TotalHDAllocated += size;
}
//*******************************************************************
inline void TOgleFileMgr::DecTotalHDAllocated(dword size)
{
  if (size > PDiskDat->TotalHDAllocated)
  {
    PDiskDat->TotalHDAllocated = 0;
  }
  else
  {
    PDiskDat->TotalHDAllocated -= size;
  }
}
//*******************************************************************
inline dword TOgleFileMgr::GetTotalHDAllocated()
{
  return PDiskDat->TotalHDAllocated;
}
//*******************************************************************
inline TOgleApp *TOgleFileMgr::GetOgleApp()
{
  return POgleApp;
}
//*******************************************************************
inline void TOgleFileMgr::Activate()
{
}
//*******************************************************************
inline void TOgleFileMgr::DeActivate()
{
}
//*******************************************************************
inline void TOgleFileMgr::Idle()
{
  IdleLoad();
}
//*******************************************************************
inline void TOgleFileMgr::BufferFiles(int16 num)
{
}
//*******************************************************************
inline void TOgleFileMgr::Update(dword NewFUC)
{
  PDat->SetCurrentFUC(NewFUC);
}
//*******************************************************************
inline void TOgleFileMgr::CheckRam()
{
  if (TOgleMemMgr::GetWorkingAreaOverflow())
  {
    PFLDat->UnLoadRamBelow(TFileMode::fm_Immed,TOgleMemMgr::GetWorkingAreaOverflowSize());
  }
}
//*******************************************************************
inline void TOgleFileMgr::CheckHD()
{
  if (!PDiskDat->IsHDSpaceOk())
  {
    PFLDat->UnLoadHDBelow(TFileMode::fm_Immed,PDiskDat->GetThresholdDelta());
  }
}
//*******************************************************************
inline TCursorMgr *TOgleFileMgr::GetCursorMgr()
{
  return GetOgleApp()->GetCursorMgr();
}
//*******************************************************************
#endif

