//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ofilenam.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  2.0     11/7/97    M.Perry    original baseline
//********************************************************************
#if !defined(__ofilenam_h)
#define __ofilenam_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif
//*******************************************************************
class TOgleFileName
{
public:
  enum {MAXFILEPARTSIZE = 256};

  enum {ofnDRIVE        = fBit0,
        ofnDIRPATH      = fBit1,
        ofnFNAME        = fBit2,
        ofnEXT          = fBit3};

protected:
  string strName;

public:
  void SetToUpperCase() {strName.to_upper();}
  void SetToLowerCase() {strName.to_lower();}
  TOgleFileName();
  TOgleFileName(string filename);
  TOgleFileName(const TOgleFileName &ofilename);
  ~TOgleFileName();
  void ClearParts(TOgleBYTEFlag parts);
  void ClearExt() {ClearParts(ofnEXT);}
  void ClearFName() {ClearParts(ofnFNAME);}
  void ClearFNameExt() {ClearParts(ofnFNAME | ofnEXT);}
  void ClearDirPath() {ClearParts(ofnDIRPATH);}
  void ClearDrive() {ClearParts(ofnDRIVE);}
  void Clear();
  string GetDrive() const;
  void SetDrive(const string &strdrive);
  string GetDirPath() const;
  void SetDirPath(const string &strdirpath);
  void AppendDirPath(const string &strdirpath);
  void PrePendDirPath(const string &strdirpath);
  string GetFName() const;
  void SetFName(const string &strfname);
  string GetExt(bool stripperiod = true) const;
  void SetExt(const string &strext);
  string GetFNameExt() const;
  void SetFNameExt(const string &strfnameext);
  string GetFullName() const;
  const string &GetStr() const {return strName;}
  string GetNameParts(TOgleBYTEFlag parts = ofnDRIVE | ofnDIRPATH | ofnFNAME | ofnEXT);
  void SetFullName(const string &strfullname);
  void SetLogicalRoot(const string &strlogicalroot);
  bool HasRoot(const string &strroot) const;
  bool HasRoot(const TOgleFileName &ofnroot) const;
  void Merge(const string &filename);
  void Merge(const TOgleFileName &ofilename);
  TOgleFileName &operator=(const TOgleFileName &ofilename);
  TOgleFileName &operator=(const string &filename);
  TOgleFileName &operator+=(const string &filename);
  void SplitUp(string &drive,string &dir,string &fname,string &ext) const;
  bool PullFirstDir(string &first,string &rest) const;
  string PullLastDir();
  bool DirExists() const;
  bool CreateDir();
  int GetDriveInt() const;
  char GetDriveChar() const;
  int32 GetFileSize();
  bool CopyFile(const string &dest);
  bool CopyFile(const TOgleFileName &ofn);
  bool DeleteFile();
  bool FileExists();
  bool LoadBuffer(void *p,int32 size,int32 fileoffset = 0) const;
  bool WriteBuffer(void *p,int32 size,int32 fileoffset = 0) const;
  bool RemoveRoot(const TOgleFileName &ofnroot);
  bool RemoveRoot(const string &strroot);
  void MassageDir() {SetDirPath(GetDirPath());}

  //  statics
  static bool RemoveRoot(const TOgleFileName &root,TOgleFileName &local);
  static bool RemoveRoot(const string &strroot,string &strlocal);
  static bool LoadBuffer(const string &s,void *p,int32 size,int32 fileoffset = 0);
  static bool LoadBuffer(const TOgleFileName &ofn,void *p,int32 size,int32 fileoffset = 0);
  static bool WriteBuffer(const TOgleFileName &ofn,void *p,int32 size,int32 fileoffset = 0);
  static bool WriteBuffer(const string &s,void *p,int32 size,int32 fileoffset = 0); 
  static void SplitUp(const string &fullpath,string &drive,string &dir,string &fname,string &ext);
  static string GetExt(const string &pathname,bool stripperiod = false);
  static bool PullFirstDir(const string &pathname,string &first,string &rest);
  static string PullLastDir(const string &pathname,bool keeplastslash = true);
  static bool DirExists(const string &fulldir);
  static bool CreateDir(const string &dir);
  static char GetDriveChar(const string &drive);
  static int ConvertDriveLetterToInt(const string &drive);
  static int ConvertDriveLetterToInt(char drive);
  static char ConvertIntToDriveLetter(int drive);
  static int32 GetFileSize(const string &fullpathname);
  static int32 GetFileSize(const char *filename);
  static bool CopyFile(const string &dest,const string &source);
  static bool DeleteFile(const string &fullpathname);
  static bool FileExists(const char *fullpathname);
  static bool FileExists(const string &fullpathname);
  static bool FileExists(const TOgleFileName &ofn);
  static string MassageDriveStr(const string &drive);
  static string MassageDirStr(const string &dir);

  //  friends
  friend TOgleFileName operator+(const TOgleFileName &ofilename1,const TOgleFileName &ofilename2);
};
//*******************************************************************
//******************** inlines **************************************
//*******************************************************************
inline void TOgleFileName::Clear()
{
  strName = TAppString::strEmpty;
}
//*******************************************************************
inline string TOgleFileName::GetFullName() const
{
  return strName;
}
//*******************************************************************
inline void TOgleFileName::SplitUp(string &drive,string &dir,string &fname,string &ext) const
{
  SplitUp(strName,drive,dir,fname,ext);
}
//*******************************************************************
inline bool TOgleFileName::DirExists() const
{
  return DirExists(strName);
}
//*******************************************************************
inline bool TOgleFileName::CreateDir()
{
  return CreateDir(strName);
}
//*******************************************************************
inline int TOgleFileName::GetDriveInt() const
{
  return ConvertDriveLetterToInt(strName);
}
//*******************************************************************
inline char TOgleFileName::GetDriveChar() const
{
  return GetDriveChar(strName);
}
//*******************************************************************
inline string TOgleFileName::PullLastDir()
{
  return PullLastDir(strName);
}
//*******************************************************************
inline int32 TOgleFileName::GetFileSize()
{
  return GetFileSize(strName);
}
//*******************************************************************
inline bool TOgleFileName::CopyFile(const TOgleFileName &ofn)
{
  return CopyFile(ofn.strName);
}
//*******************************************************************
inline bool TOgleFileName::DeleteFile()
{
  return DeleteFile(strName);
}
//*******************************************************************
inline bool TOgleFileName::FileExists()
{
  return FileExists(strName);
}
//*******************************************************************
#endif