//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name: ofmode.h
//    Project Name: Ogle
// Original Author: Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     3/6/95     M.Perry     original baseline
//  2.0     8/26/97    M.Perry     32-bit conversion.
//  2.1    11/19/97    M.Perry     changed shit around.
//********************************************************************
#if !defined(__ofmode_h) 
#define __ofmode_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__oflags_h)
#include "oflags.h"
#endif

#if !defined(__ofilenam_h)
#include "ofilenam.h"
#endif

typedef int8 TFMStatus;
//********************** Flags constants ****************************
#define fmSTART ((byte)(fBit0))
#define fmNOSHARE (fmSTART)
#define fmEND fmNOSHARE

#if (fmEND > byte_max)
  #error TFileMode byte flag overflow
#endif
//*******************************************************************
class TFileMode
{
public:
  enum {fm_None = 0,    // !! Important !!
        fm_BG_Lo = 1,   // !! Order important here !!
        fm_BG_Med = 2,  // !! Don't change order !!
        fm_BG_Hi = 3,
        fm_Immed = 4,
        fm_Num_Status = 5};

public:
  TOgleFileName Name;

protected:
  TFMStatus HDStatus;
  TFMStatus RamStatus;
  TOgleBYTEFlag Flags;
  int8 DirIndex; // shitty fucking thing.

public:
  TFileMode(const TFileMode& fm);
  TFileMode(string &name,
            TFMStatus hdstatus = fm_None,
            TFMStatus ramstatus = fm_None,
            int8 dirindex = -1,
            bool shareable = true);
  TFileMode(const string &name,
            TFMStatus hdstatus = fm_None,
            TFMStatus ramstatus = fm_None,
            int8 dirindex = -1,
            bool shareable = true);
  TFileMode(TOgleFileName &ofn,
            TFMStatus hdstatus = fm_None,
            TFMStatus ramstatus = fm_None,
            byte flags = 0,
            int8 dirindex = -1);

  TFileMode();
  ~TFileMode();

  void SetNoShare() {Flags.Set(fmNOSHARE);}
  void ClearNoShare() {Flags.Clear(fmNOSHARE);}
  bool GetNoShare() {return Flags.IsSet(fmNOSHARE);}

  void SetRamStatus(TFMStatus ramstatus);
  void SetHDStatus(TFMStatus hdstatus);
  void SetHDImmed();
  void SetRamImmed();
  void SetHDBGLo();
  void SetRamBGLo();
  void SetHDBGMed();
  void SetRamBGMed();
  void SetHDBGHi();
  void SetRamBGHi();
  void SetHDNone();
  void SetRamNone();
  void SetNone();
  void SetBGLo();
  void SetBGMed();
  void SetBGHi();
  void SetImmed();
  void SetShareable();
  void SetNonShareable();
  bool GetShareable();
  bool IsShareable();
  void SetDirIndex(int8 NewDirIndex = -1);
  int8 GetDirIndex();
  bool IsRamImmed();
  bool IsHDImmed();
  bool IsRamBGHi();
  bool IsHDBGHi();
  bool IsRamBGMed();
  bool IsHDBGMed();
  bool IsRamBGLo();
  bool IsHDBGLo();
  bool IsRamNone();
  bool IsHDNone();
  TFMStatus GetRamStatus() const;
  TFMStatus GetHDStatus() const;
  TFileMode &operator=(TFileMode &fm);
  TFileMode &operator=(const TFileMode &fm);
  bool operator==(const string &s);
};
//*******************************************************************
//************************ inlines **********************************
//*******************************************************************
inline TFileMode::TFileMode(const TFileMode& fm)
:Name(fm.Name),
 HDStatus(fm.HDStatus),
 RamStatus(fm.RamStatus),
 DirIndex(fm.DirIndex),
 Flags(fm.Flags)
{
}
//*******************************************************************
inline TFileMode::TFileMode(string &name,
                            TFMStatus hdstatus,
                            TFMStatus ramstatus,
                            int8 dirindex,
                            bool shareable)
:Name(name),
 HDStatus(hdstatus),
 RamStatus(ramstatus),
 DirIndex(dirindex)
{
  if (!shareable) {SetNoShare();}
}
//*******************************************************************
inline TFileMode::TFileMode(const string &name,
                            TFMStatus hdstatus,
                            TFMStatus ramstatus,
                            int8 dirindex,
                            bool shareable)
:Name(name),
 HDStatus(hdstatus),
 RamStatus(ramstatus),
 DirIndex(dirindex)
{
  if (!shareable) {SetNoShare();}
}
//*******************************************************************
inline TFileMode::TFileMode()
:HDStatus(fm_None),
 RamStatus(fm_None),
 DirIndex(-1)
{
}
//*******************************************************************
inline TFileMode::TFileMode(TOgleFileName &ofn,
                            TFMStatus hdstatus,
                            TFMStatus ramstatus,
                            byte flags,
                            int8 dirindex)
:Name(ofn),
 HDStatus(hdstatus),
 RamStatus(ramstatus),
 DirIndex(dirindex),
 Flags(flags)
{
}
//*******************************************************************
inline TFileMode::~TFileMode()
{
}
//*******************************************************************
inline void TFileMode::SetRamStatus(TFMStatus ramstatus)
{
  RamStatus = ramstatus;
}
//*******************************************************************
inline void TFileMode::SetHDStatus(TFMStatus hdstatus)
{
  HDStatus = hdstatus;
}
//*******************************************************************
inline void TFileMode::SetHDImmed()
{
  SetHDStatus(fm_Immed);
}
//*******************************************************************
inline void TFileMode::SetRamImmed()
{
  SetRamStatus(fm_Immed);
}
//*******************************************************************
inline void TFileMode::SetHDBGLo()
{
  SetHDStatus(fm_BG_Lo);
}
//*******************************************************************
inline void TFileMode::SetRamBGLo()
{
  SetRamStatus(fm_BG_Lo);
}
//*******************************************************************
inline void TFileMode::SetHDBGMed()
{
  SetHDStatus(fm_BG_Med);
}
//*******************************************************************
inline void TFileMode::SetRamBGMed()
{
  SetRamStatus(fm_BG_Med);
}
//*******************************************************************
inline void TFileMode::SetHDBGHi()
{
  SetHDStatus(fm_BG_Hi);
}
//*******************************************************************
inline void TFileMode::SetRamBGHi()
{
  SetRamStatus(fm_BG_Hi);
}
//*******************************************************************
inline void TFileMode::SetHDNone()
{
  SetHDStatus(fm_None);
}
//*******************************************************************
inline void TFileMode::SetRamNone()
{
  SetRamStatus(fm_None);
}
//*******************************************************************
inline void TFileMode::SetNone()
{
  SetHDNone();
  SetRamNone();
}
//*******************************************************************
inline void TFileMode::SetBGLo()
{
  SetHDBGLo();
  SetRamBGLo();
}
//*******************************************************************
inline void TFileMode::SetBGMed()
{
  SetHDBGMed();
  SetRamBGMed();
}
//*******************************************************************
inline void TFileMode::SetBGHi()
{
  SetHDBGHi();
  SetRamBGHi();
}
//*******************************************************************
inline void TFileMode::SetImmed()
{
  SetHDImmed();
  SetRamImmed();
}
//*******************************************************************
inline void TFileMode::SetShareable()
{
  ClearNoShare();
}
//*******************************************************************
inline void TFileMode::SetNonShareable()
{
  SetNoShare();
}
//*******************************************************************
inline bool TFileMode::GetShareable()
{
  return !GetNoShare();
}
//*******************************************************************
inline bool TFileMode::IsShareable()
{
  return !GetNoShare();
}
//*******************************************************************
inline void TFileMode::SetDirIndex(int8 dirindex)
{
  DirIndex = dirindex;
}
//*******************************************************************
inline int8 TFileMode::GetDirIndex()
{
  return DirIndex;
}
//*******************************************************************
inline bool TFileMode::IsRamImmed()
{
  return (RamStatus == fm_Immed);
}
//*******************************************************************
inline bool TFileMode::IsHDImmed()
{
  return (HDStatus == fm_Immed);
}
//*******************************************************************
inline bool TFileMode::IsRamBGHi()
{
  return (RamStatus == fm_BG_Hi);
}
//*******************************************************************
inline bool TFileMode::IsHDBGHi()
{
  return (HDStatus == fm_BG_Hi);
}
//*******************************************************************
inline bool TFileMode::IsRamBGMed()
{
  return (RamStatus == fm_BG_Med);
}
//*******************************************************************
inline bool TFileMode::IsHDBGMed()
{
  return (HDStatus == fm_BG_Med);
}
//*******************************************************************
inline bool TFileMode::IsRamBGLo()
{
  return (RamStatus == fm_BG_Lo);
}
//*******************************************************************
inline bool TFileMode::IsHDBGLo()
{
  return (RamStatus == fm_BG_Lo);
}
//*******************************************************************
inline bool TFileMode::IsRamNone()
{
  return (RamStatus == fm_None);
}
//*******************************************************************
inline bool TFileMode::IsHDNone()
{
  return (HDStatus == fm_None);
}
//*******************************************************************
inline TFMStatus TFileMode::GetRamStatus() const
{
  return RamStatus;
}
//*******************************************************************
inline TFMStatus TFileMode::GetHDStatus() const
{
  return HDStatus;
}
//*******************************************************************
inline TFileMode &TFileMode::operator=(TFileMode &fm)
{
  if (this == &fm) {return *this;}
  Name = fm.Name;
  HDStatus = fm.HDStatus;
  RamStatus = fm.RamStatus;
  DirIndex = fm.DirIndex;
  Flags = fm.Flags;
  return *this;
}
//*******************************************************************
inline TFileMode &TFileMode::operator=(const TFileMode &fm)
{
  if (this == &fm) {return *this;}
  Name = fm.Name;
  HDStatus = fm.HDStatus;
  RamStatus = fm.RamStatus;
  DirIndex = fm.DirIndex;
  Flags = fm.Flags;
  return *this;
}
//*******************************************************************
inline bool TFileMode::operator==(const string &s)
{
  return bool(s == Name.GetStr());
}
//*******************************************************************
#endif

