//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ogr.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/5/95     M.Perry     original baseline
//  2.0     8/21/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__ogr_h)
#define __ogr_h

#if !defined(__obase_h)
#include "obase.h"
#endif

#if !defined(__ogeo_h)
#include "ogeo.h"
#endif

// start of grdata stat flags
#define grdFIRST (fBit0)
#define grdUSEDEFAULTXY (grdFIRST)
#define grdLAST (grdUSEDEFAULTXY)
// end of grdata stat flags
//*******************************************************************
class TGrData
{
protected:
  // remember: OffsetVPt is just the offset from the owners pt
  // it's got nothing to do with creating the surrounding rectangle around
  // the bits, which uses the AdjVPt and internal offset pts.
  TTriplet VPt,OffsetVPt,AdjVPt;  // offsetvpt shifts vpt and makes adjvpt.

  TOgleWORDFlag StatFlags;

public:
  TGrData();
  TGrData(TTriplet &vpt,TTriplet &ovpt,TTriplet &avpt,bool usedefaultXY);
  TGrData(int16 x,int16 y,int16 z,bool usedefaultXY = true);
  TGrData(TGrData &grdata);
  virtual ~TGrData();
public:
  void SetUseDefaultXY(bool usedefaultxy = true);
  void ClearUseDefaultXY();
  bool GetUseDefaultXY();

  void SetAdjVPt();
  TTriplet &GetAdjVPt();
  void SetOffsetVPt(TTriplet &vpt);
  TTriplet &GetOffsetVPt();
  void SetX(int16 x);
  int16 GetX() const;
  void SetY(int16 y);
  int16 GetY() const;
  void SetZ(int16 z);
  int16 GetZ() const;
  void SetXY(int16 x,int16 y);
  void SetXYZ(int16 x,int16 y,int16 z);
  int16 GetAdjX() const;
  int16 GetAdjY() const;
  int16 GetAdjZ() const;
};
//*******************************************************************
// start of grdd stat flags
#define grddFIRST (fBit0)
#define grddOWNBITS (grddFIRST)
#define grddCHANGED ((WORD)(grddOWNBITS * 2))
#define grddTOPDOWN ((WORD)(grddCHANGED * 2))
#define grddLINEAR ((WORD)(grddTOPDOWN * 2))
#define grddLAST (grddLINEAR)
// end of grdd stat flags
//*******************************************************************
class TGrDrawData
{
protected:
  TOgleWORDFlag StatFlags;
  byte huge *PBits;
  TOgleRect OldVRect;
  TOgleRect VRect;
  int16 Z;
  int16 FrameNum;

public:
  TGrDrawData();
  TGrDrawData(TOgleRect &vrect,int16 z,byte *pbits,bool linear = false,bool topdown = false);
  TGrDrawData(TOglePoint &tl,TOglePoint &br,int16 z,byte *pbits,bool linear = false,bool topdown = false);
  TGrDrawData(int16 x1,int16 y1,int16 x2,int16 y2,int16 z,byte *pbits,bool linear = false,bool topdown = false);
  ~TGrDrawData();

  void SetFrameNum(int16 framenum);
  int16 GetFrameNum();
  byte huge*GetBits();
  void SetBits(byte huge *pbits);
  bool GetOwnBits() const;
  void SetOwnBits(bool ownbits = true);
  void ClearOwnBits();
  TOglePoint GetVTL();
  void SetVTL(TOglePoint &pt);
  TOglePoint GetVBR();
  void SetVBR(TOglePoint &pt);
  TOgleRect &GetVRect();
  void SetVRect(TOgleRect &vrect);
  TOgleRect &GetOldVRect();
  void SetOldVRect(TOgleRect &rect);
  void CalcVRect(TTriplet &adjpt,TOglePoint &vrectoffpt,int16 width,int16 height);
  void CalcVRect(TOglePoint &pt,int16 z,int16 width,int16 height);
  void CalcVRect(int16 width,int16 height,int16 z = 0);
  int16 GetZ();
  void SetZ(int16 z);
  bool GetChanged() const;
  bool IsChanged();
  void SetChanged(bool changed = true);
  void ClearChanged();
  bool GetTopDown() const;
  bool IsTopDown() const;
  void SetTopDown(bool topdown = true);
  void ClearTopDown();
  bool GetLinear() const;
  bool IsLinear() const;
  void SetLinear(bool linear = true);
  void ClearLinear();
};
//*******************************************************************
class TGrList;
class TOgleVideoDC;
// start of gr stat flags
#define grFIRST ((WORD)(obLAST * 2))
#define grADDEDTOSCRNMGR (grFIRST)
#define grLAST (grADDEDTOSCRNMGR)
// end of gr stat flags
//*******************************************************************
class TGr:public TOgleBase
{
protected:
  friend TGrList;
  int16 GetZ();
  void SetAddedToScrnMgr();
  void ClearAddedToScrnMgr();

  bool AddToScrnMgr();
  bool RemoveFromScrnMgr();

public:
  TGrDrawData GrDrawData;

  bool GetAddedToScrnMgr();

  TGr(TOgleModeWindow *pmw);
  TGr(TOgleModeWindow *pmw,const TFileMode &fm);
  virtual ~TGr();

  virtual bool BltTo(TOgleVideoDC &vdc,TOgleRect &clipvrect);

  static int16 NumScanBytes(int16 width, int16 bitcount);
  static int16 NumScanBytes(int16 width);

  static int32 GetBitsOffset(int16 x,int16 y,bool topdown,int16 width,int16 height);
  static int32 GetBitsOffset(TOglePoint const &lpt,bool topdown,int16 width,int16 height);
  static TOglePoint VptToLpt(TOglePoint &srcvpt,TOglePoint &destvpt);
  static TOglePoint VptToLpt(TOglePoint &srcvpt,int16 dx,int16 dy);
  static TOglePoint VptToLpt(int16 sx,int16 sy,int16 dx,int16 dy);
  static TOglePoint VptToLpt(int16 sx,int16 sy,TOglePoint &destvpt);
  static TOglePoint LptToVpt(int16 lleft,int16 ltop,int16 vleft,int16 vtop);
  static TOglePoint LptToVpt(TOglePoint &lpt,TOglePoint &vpt);
  static TOgleRect VrToLr(TOgleRect &rect);
  static byte huge *GetPixelAddr(byte huge *pbits,TOglePoint const &pt,bool topdown,int16 width,int16 height);
  static byte huge *GetPixelAddr(byte huge *pbits,int16 x,int16 y,bool topdown,int16 width,int16 height);
};
//*******************************************************************
//*********************  Inlines ************************************
//*******************************************************************
inline void TGrData::SetUseDefaultXY(bool usedefaultxy)
{
  if (usedefaultxy) {StatFlags.Set(grdUSEDEFAULTXY);}
  else              {StatFlags.Clear(grdUSEDEFAULTXY);}
}
//*******************************************************************
inline void TGrData::ClearUseDefaultXY()
{
  StatFlags.Clear(grdUSEDEFAULTXY);
}
//*******************************************************************
inline bool TGrData::GetUseDefaultXY()
{
  return StatFlags.IsSet(grdUSEDEFAULTXY);
}
//*******************************************************************
inline void TGrData::SetAdjVPt()
{
  AdjVPt = VPt + OffsetVPt;
}
//*******************************************************************
inline TTriplet &TGrData::GetAdjVPt()
{
  return AdjVPt;
}
//*******************************************************************
inline void TGrData::SetOffsetVPt(TTriplet &vpt)
{
  OffsetVPt = vpt;
  SetAdjVPt();
}
//*******************************************************************
inline TTriplet &TGrData::GetOffsetVPt()
{
  return OffsetVPt;
}
//*******************************************************************
inline void TGrData::SetX(int16 x)
{
  VPt.X = x;
  SetAdjVPt();
}
//*******************************************************************
inline int16 TGrData::GetX() const
{
  return VPt.X;
}
//*******************************************************************
inline void TGrData::SetY(int16 y)
{
  VPt.Y = y;
  SetAdjVPt();
}
//*******************************************************************
inline int16 TGrData::GetY() const
{
  return VPt.Y;
}
//*******************************************************************
inline void TGrData::SetZ(int16 z)
{
  VPt.Z = z;
  SetAdjVPt();
}
//*******************************************************************
inline int16 TGrData::GetZ() const
{
  return VPt.Z;
}
//*******************************************************************
inline void TGrData::SetXY(int16 x,int16 y)
{
  VPt.X = x;
  VPt.Y = y;
  SetAdjVPt();
}
//*******************************************************************
inline void TGrData::SetXYZ(int16 x,int16 y,int16 z)
{
  VPt.X = x;
  VPt.Y = y;
  VPt.Z = z;
  SetAdjVPt();
}
//*******************************************************************
inline int16 TGrData::GetAdjX() const
{
  return AdjVPt.X;
}
//*******************************************************************
inline int16 TGrData::GetAdjY() const
{
  return AdjVPt.Y;
}
//*******************************************************************
inline int16 TGrData::GetAdjZ() const
{
  return AdjVPt.Z;
}
//*******************************************************************
//*******************************************************************
inline void TGrDrawData::SetFrameNum(int16 framenum)
{
  if (FrameNum != framenum) {SetChanged(true);}
  FrameNum = framenum;
}
//*******************************************************************
inline int16 TGrDrawData::GetFrameNum()
{
  return FrameNum;
}
//*******************************************************************
inline byte huge *TGrDrawData::GetBits()
{
  return PBits;
}
//*******************************************************************
inline void TGrDrawData::SetBits(byte huge *pbits)
{
  PBits = pbits;
}
//*******************************************************************
inline bool TGrDrawData::GetOwnBits() const
{
  return StatFlags.IsSet(grddOWNBITS);
}
//*******************************************************************
inline void TGrDrawData::SetOwnBits(bool ownbits)
{
  if (ownbits) {StatFlags.Set(grddOWNBITS);}
  else         {StatFlags.Clear(grddOWNBITS);}
}
//*******************************************************************
inline void TGrDrawData::ClearOwnBits()
{
  StatFlags.Clear(grddOWNBITS);
}
//*******************************************************************
inline TOglePoint TGrDrawData::GetVTL()
{
  return VRect.GetTL();
}
//*******************************************************************
inline void TGrDrawData::SetVTL(TOglePoint &pt)
{
  VRect.SetTL(pt);
}
//*******************************************************************
inline TOglePoint TGrDrawData::GetVBR()
{
  return VRect.GetBR();
}
//*******************************************************************
inline void TGrDrawData::SetVBR(TOglePoint &pt)
{
  VRect.SetBR(pt);
}
//*******************************************************************
inline TOgleRect &TGrDrawData::GetVRect()
{
  return VRect;
}
//*******************************************************************
inline void TGrDrawData::SetVRect(TOgleRect &vrect)
{
  VRect = vrect;
}
//*******************************************************************
inline TOgleRect &TGrDrawData::GetOldVRect()
{
  return OldVRect;
}
//*******************************************************************
inline void TGrDrawData::SetOldVRect(TOgleRect &rect)
{
  OldVRect = rect;
}
//*******************************************************************
inline void TGrDrawData::CalcVRect(TTriplet &adjpt,TOglePoint &vrectoffpt,int16 width,int16 height)
{
  if (!IsChanged())
  {
    OldVRect = VRect;  // MRP 12/21/96 -> handles multiple calls before actual screen update.
  }
  VRect.left = int16(adjpt.X - vrectoffpt.x);
  VRect.top = int16(adjpt.Y - vrectoffpt.y);
  VRect.SetWidth(width);
  VRect.SetHeight(height);
  Z = adjpt.Z;
  if (OldVRect != VRect) {SetChanged(true);}
}
//*******************************************************************
inline void TGrDrawData::CalcVRect(TOglePoint &pt,int16 z,int16 width,int16 height)
{
  if (!IsChanged())
  {
    OldVRect = VRect; // MRP 12/21/96 -> handles multiple calls before actual screen update.
  }
  VRect.left = pt.x;
  VRect.top = pt.y;
  VRect.SetWidth(width);
  VRect.SetHeight(height);
  Z = z;
  if (OldVRect != VRect) {SetChanged(true);}
}
//*******************************************************************
inline void TGrDrawData::CalcVRect(int16 width,int16 height,int16 z)
{
  if (!IsChanged())
  {
    OldVRect = VRect;  // MRP 12/21/96 -> handles multiple calls before actual screen update.
  }
  VRect.SetWidth(width);
  VRect.SetHeight(height);
  Z = z;
  if (OldVRect != VRect) {SetChanged(true);}
}
//*******************************************************************
inline int16 TGrDrawData::GetZ()
{
  return Z;
}
//*******************************************************************
inline void TGrDrawData::SetZ(int16 z)
{
  Z = z;
}
//*******************************************************************
inline bool TGrDrawData::GetChanged() const
{
  return StatFlags.IsSet(grddCHANGED);
}
//*******************************************************************
inline bool TGrDrawData::IsChanged()
{
  return GetChanged();
}
//*******************************************************************
inline void TGrDrawData::SetChanged(bool changed)
{
  if (changed) {StatFlags.Set(grddCHANGED);}
  else         {StatFlags.Clear(grddCHANGED);}
}
//*******************************************************************
inline void TGrDrawData::ClearChanged()
{
  StatFlags.Clear(grddCHANGED);
}
//*******************************************************************
inline bool TGrDrawData::GetTopDown() const
{
  return StatFlags.IsSet(grddTOPDOWN);
}
//*******************************************************************
inline bool TGrDrawData::IsTopDown() const
{
  return StatFlags.IsSet(grddTOPDOWN);
}
//*******************************************************************
inline void TGrDrawData::SetTopDown(bool topdown)
{
  if (topdown) {StatFlags.Set(grddTOPDOWN);}
  else         {StatFlags.Clear(grddTOPDOWN);}
}
//*******************************************************************
inline void TGrDrawData::ClearTopDown()
{
  StatFlags.Clear(grddTOPDOWN);
}
//*******************************************************************
inline bool TGrDrawData::GetLinear() const
{
  return StatFlags.IsSet(grddLINEAR);
}
//*******************************************************************
inline bool TGrDrawData::IsLinear() const
{
  return StatFlags.IsSet(grddLINEAR);
}
//*******************************************************************
inline void TGrDrawData::SetLinear(bool linear)
{
  if (linear) {StatFlags.Set(grddLINEAR);}
  else        {StatFlags.Clear(grddLINEAR);}
}
//*******************************************************************
inline void TGrDrawData::ClearLinear()
{
  StatFlags.Clear(grddLINEAR);
}
//*******************************************************************
//*******************************************************************
inline int16 TGr::GetZ()
{
  return GrDrawData.GetZ();
}
//*******************************************************************
inline void TGr::SetAddedToScrnMgr()
{
  StatFlags.Set(grADDEDTOSCRNMGR);
}
//*******************************************************************
inline void TGr::ClearAddedToScrnMgr()
{
  StatFlags.Clear(grADDEDTOSCRNMGR);
}
//*******************************************************************
inline bool TGr::GetAddedToScrnMgr()
{
  return StatFlags.IsSet(grADDEDTOSCRNMGR);
}
//*******************************************************************
inline int16 TGr::NumScanBytes(int16 width, int16 bitcount)
{
  return int16((((int32)width*bitcount+31) / 32) * 4);
}
//*******************************************************************
inline int16 TGr::NumScanBytes(int16 width)
{
  return int16((((int32)width*8+31) / 32) * 4);
}
//*******************************************************************
inline int32 TGr::GetBitsOffset(int16 x,int16 y,bool topdown,int16 width,int16 height)
// width is including pad size on end of scanline.
{
  if (topdown) {return ((int32)y * width + x);}
  return ((int32)(height - 1 - y) * width + x);
}
//*******************************************************************
inline int32 TGr::GetBitsOffset(TOglePoint const &lpt,bool topdown,int16 width,int16 height)
{
  if (topdown) {return ((int32)lpt.y * width + lpt.x);}
  return ((int32)(height - 1 - lpt.y) * width + lpt.x);
}
//*******************************************************************
inline TOglePoint TGr::VptToLpt(TOglePoint &srcvpt,TOglePoint &destvpt)
// returns offset from TL of src, src local coord.
{
  return TOglePoint(int16(destvpt.x - srcvpt.x),int16(destvpt.y - srcvpt.y));
}
//*******************************************************************
inline TOglePoint TGr::VptToLpt(TOglePoint &srcvpt,int16 dx,int16 dy)
{
  return TOglePoint(int16(dx - srcvpt.x),int16(dy - srcvpt.y));
}
//*******************************************************************
inline TOglePoint TGr::VptToLpt(int16 sx,int16 sy,int16 dx,int16 dy)
{
  return TOglePoint(int16(dx - sx),int16(dy - sy));
}
//*******************************************************************
inline TOglePoint TGr::VptToLpt(int16 sx,int16 sy,TOglePoint &destvpt)
{
  return TOglePoint(int16(destvpt.x - sx),int16(destvpt.y - sy));
}
//*******************************************************************
inline TOglePoint TGr::LptToVpt(int16 lleft,int16 ltop,int16 vleft,int16 vtop)
{
  return TOglePoint(int16(vleft + lleft),int16(vtop + ltop));
}
//*******************************************************************
inline TOglePoint TGr::LptToVpt(TOglePoint &lpt,TOglePoint &vpt)
{
  return TOglePoint(int16(vpt.x + lpt.x),int16(vpt.y + lpt.y));
}
//*******************************************************************
inline TOgleRect TGr::VrToLr(TOgleRect &rect)
{
  return rect.OffsetBy(int16(-rect.left),int16(-rect.top));
}
//*******************************************************************
inline byte huge *TGr::GetPixelAddr(byte huge *pbits,TOglePoint const &pt,bool topdown,int16 width,int16 height)
{
  int32 offset = GetBitsOffset(pt,topdown,width,height);
  if (pbits == NULL || offset < 0) {return NULL;}
  return (pbits + offset);
}
//*******************************************************************
inline byte huge *TGr::GetPixelAddr(byte huge *pbits,int16 x,int16 y,bool topdown,int16 width,int16 height)
{
  int32 offset = GetBitsOffset(x,y,topdown,width,height);
  if (pbits == NULL || offset < 0) {return NULL;}
  return (pbits + offset);
}
//*******************************************************************
#endif

