//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  olist.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     6/12/95    M.Perry     original baseline
//  1.2     03/27/97   M.Perry     changed load dialog subsystem.
//  2.0     8/20/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__olist_h)
#define __olist_h

#if !defined(__obase_h)
#include "obase.h"
#endif

// start of statflags
#define olFIRST ((WORD)(obLAST * 2))
#define olOWNLISTPTRS (olFIRST)
#define olINDEXESSET ((WORD)(olOWNLISTPTRS * 2))
#define olSORTED ((WORD)(olINDEXESSET * 2))
#define olLAST (olSORTED)
// end of statflags
class TOgleGroupGaugeDlg;
//*******************************************************************
class TOgleList:public TOgleBase
{
protected:
  TOgleBase **List;
  int16 ArraySize;
  int16 LastAddedElementNum;

public:
  virtual void TimeChanged();
  void LoadRamAll(TFMStatus ramstatus1,TFMStatus ramstatus2);
  void SetOwnListPtrs();
  void ClearOwnListPtrs();
  bool GetOwnListPtrs();

  void SetIndexesSet();
  void ClearIndexesSet();
  bool GetIndexesSet();

  TOgleList();
  TOgleList(TOgleModeWindow *pmw);
  TOgleList(TOgleModeWindow *pmw,bool dummy,const TFileMode &fm);
  virtual ~TOgleList();
  void DeleteAll();

  int16 BinarySearch(const string &name);
  int16 LinearSearch(const string &name);
  int16 BSCompareFName(const string &name,const TOgleBase *pb);
  virtual void Sort();
  void SortByFName();
  bool IsSorted();
  bool GetSorted();
  void SetSorted(bool sorted = true);

  virtual TOgleGroupGaugeDlg *GetGaugeDlg(string title = "");
  virtual TOgleGroupGaugeDlg *CreateGaugeDlg(string title = "");
  virtual void ReleaseGaugeDlg(TOgleGroupGaugeDlg *pggd);

  int16 GetNumElements();
  int16 GetLAEN();

  void SetAllFUCOffsets(int16 offset);
  virtual bool AddElement(TOgleBase *PNewBase);
  bool GrowTo(int16 size);
  bool GrowBy(int16 size);
  virtual int16 GetGrowSize();
  virtual int16 AddPBase(TOgleBase *PNewBase);
  virtual bool RemoveElement(int16 ElementNum);
  virtual int16 IsElement(string *PName);
  virtual bool IsActiveElement(int16 ElementNum);
  virtual bool IsActiveElement(TOgleBase *PBase);
  virtual bool ActivateElement(int16 ElementNum,dword StartFUC);
  virtual bool ActivateElement(TOgleBase *PBase,dword StartFUC);
  virtual void DeActivateElement(int16 ElementNum);
  virtual void DeActivateElement(TOgleBase *PBase);
  virtual void Start();
  virtual void Start(dword StartFUC);
  virtual void Stop();
  virtual void Show();
  virtual void NoShow();
  virtual void Update(dword NewFUC);
  virtual void PostUpdate(dword NewFUC);
  virtual void Activate(dword StartFUC);
  virtual void DeActivate();
  virtual dword GetLastFUC();

  bool CheckElementNum(int16 ElementNum);
  int16 GetElementIndex(const string &name);
  int16 GetElementIndex(TOgleBase *pbase);
  TOgleBase *GetElementPtr(const string &name);
  TOgleBase *GetElementPtr(int16 ElementNum);
  TOgleBase *GetElementPtrF(int16 elementnum);

protected:
  #if defined(DEBUG_PMODEWIN_LIST)
  void CheckModeWinPtr(int ctornum);
  #endif
};
//*******************************************************************
//*****************  inlines  ***************************************
//*******************************************************************
inline bool TOgleList::IsSorted()
{
  return StatFlags.IsSet(olSORTED);
}
//*******************************************************************
inline bool TOgleList::GetSorted()
{
  return StatFlags.IsSet(olSORTED);
}
//*******************************************************************
inline void TOgleList::SetSorted(bool sorted)
{
  if (sorted) {StatFlags.Set(olSORTED);}
  else        {StatFlags.Clear(olSORTED);}
}
//*******************************************************************
inline void TOgleList::SetOwnListPtrs()
{
  StatFlags.Set(olOWNLISTPTRS);
}
//*******************************************************************
inline void TOgleList::ClearOwnListPtrs()
{
  StatFlags.Clear(olOWNLISTPTRS);
}
//*******************************************************************
inline bool TOgleList::GetOwnListPtrs()
{
  return StatFlags.IsSet(olOWNLISTPTRS);
}
//*******************************************************************
inline void TOgleList::SetIndexesSet()
{
  StatFlags.Set(olINDEXESSET);
}
//*******************************************************************
inline void TOgleList::ClearIndexesSet()
{
  StatFlags.Clear(olINDEXESSET);
}
//*******************************************************************
inline bool TOgleList::GetIndexesSet()
{
  return StatFlags.IsSet(olINDEXESSET);
}
//*******************************************************************
inline int16 TOgleList::GetNumElements()
{
  return int16(LastAddedElementNum + 1);
}
//*******************************************************************
inline int16 TOgleList::GetLAEN()
{
  return LastAddedElementNum;
}
//*******************************************************************
inline bool TOgleList::GrowBy(int16 size)
{
  return GrowTo(int16(ArraySize + size));
}
//*******************************************************************
inline bool TOgleList::CheckElementNum(int16 ElementNum)
{
  if ((ElementNum < 0) || (ElementNum > LastAddedElementNum)) {return false;}
  return true;
}
//*******************************************************************
inline int16 TOgleList::GetElementIndex(TOgleBase *pbase)
{
  if (pbase == NULL) {return -1;}
//  return GetElementIndex(pbase->GetFName());
  return GetElementIndex(pbase->FileMode.Name.GetStr());
}
//*******************************************************************
inline TOgleBase *TOgleList::GetElementPtr(int16 ElementNum)
{
  if (List == NULL) {return NULL;}
  if (!CheckElementNum(ElementNum)) {return NULL;}
  return (List[ElementNum]);
}
//*******************************************************************
inline TOgleBase *TOgleList::GetElementPtrF(int16 elementnum)
{
  return List[elementnum];
}
//*******************************************************************
#endif

