//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  olistwal.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     1/8/96     M.Perry    original baseline
//  2.0     8/20/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__olistwal_h)
#define __olistwal_h

#if !defined(__olist_h)
#include "olist.h"
#endif

#if !defined(__osliq_h)
#include "osliq.h"
#endif
//*******************************************************************
class TOgleActiveElement
{
public:
  bool Active;
  TOgleBase *PBase;
  dword StartFUC;

public:
  TOgleActiveElement();
  TOgleActiveElement(TOgleBase *pbase,bool active = true);
  TOgleActiveElement(TOgleBase *PNewBase,dword NewStartFUC);
  virtual ~TOgleActiveElement();
  void Stop();
  void Update(dword NewFUC);
  void PostUpdate(dword NewFUC);
  bool GetStarted();
  bool GetStopped();
  dword GetStartFUC();
  dword GetEndFUC();
};
//*******************************************************************
TCompareResult CompareHiStartFUC(void *FirstObj,void *SecObj);
//*******************************************************************
class TOgleActiveList:public TSliq
{
public:
  TOgleActiveList(TOgleModeWindow *pmw,const string &name);
  virtual ~TOgleActiveList();
  virtual TOgleActiveElement *CreateNewElement(TOgleBase *PNewBase,dword NewStartFUC);
  virtual TOgleActiveElement *CreateNewElement();
  bool AddElement(TOgleBase *PNewBase,dword NewStartFUC);
  TOgleActiveElement *GetFirst();
  TOgleActiveElement *GetNext();
  TOgleActiveElement *FindElement(TOgleBase *POldBase);
  bool RemoveElement(TOgleActiveElement *PActiveElement);
  bool RemoveElement(TOgleBase *PActiveElement);
  void RemoveNotActiveElements();
  void SortByStartFUC();
  virtual void Update(dword NewFUC);
  virtual void PostUpdate(dword NewFUC);
  bool IsActive(TOgleBase *POldBase);
  bool HasActiveElement();
  bool HasGoingElement();
  string &GetName();
protected:
  string Name;
  TOgleModeWindow *PModeWin;
};
//*******************************************************************
class TOgleListWAL:public TOgleList
{
protected:
  int16 NumActiveLists;
  TOgleActiveList **PActiveList;

public:
  TOgleListWAL(TOgleModeWindow *pmw);
  virtual ~TOgleListWAL();
  virtual bool CanActivateElement(int16 ElementNum,dword StartFUC);
  virtual bool CanActivateElement(TOgleBase *PBase,dword StartFUC);
  virtual bool IsActiveElement(int16 ElementNum);
  virtual bool IsActiveElement(TOgleBase *PBase);
  virtual bool ActivateElement(int16 ElementNum,dword StartFUC);
  virtual bool ActivateElement(TOgleBase *PBase,dword StartFUC);
  virtual void DeActivateElement(int16 ElementNum);
  virtual void DeActivateElement(TOgleBase *PBase);
  virtual void Update(dword NewFUC);
  virtual void PostUpdate(dword NewFUC);
  virtual void Activate(dword StartFUC = 0);
  virtual void DeActivate();
  TOgleActiveList *GetActiveList(int16 index);
  bool CheckActiveListIndex(int16 index);
  bool AddActiveList(const string &name);
  bool RemoveActiveList(int16 index);
  virtual int16 GetActiveListIndex(TOgleBase *PBase);

protected:
  virtual TOgleActiveList *CreateActiveList(const string &name);
  int16 GetNumActiveLists();
};
//*******************************************************************
//*******************  inlines  *************************************
//*******************************************************************
inline void TOgleActiveElement::Stop()
{
  if (PBase != NULL)
  {
    if (!PBase->GetStopped()) {PBase->Stop();}
    Active = false;
  }
}
//*******************************************************************
inline void TOgleActiveElement::Update(dword NewFUC)
{
  if (!Active || PBase == NULL) {return;}
  if (StartFUC == NewFUC)
  {
    PBase->Start(NewFUC);
  }
  else if (StartFUC < NewFUC)
  {
    PBase->Update(NewFUC);
  }
}
//*******************************************************************
inline bool TOgleActiveElement::GetStarted()
{
  if (PBase == NULL) {return false;}
  return PBase->GetStarted();
}
//*******************************************************************
inline bool TOgleActiveElement::GetStopped()
{
  if (PBase == NULL) {return false;}
  return PBase->GetStopped();
}
//*******************************************************************
inline dword TOgleActiveElement::GetStartFUC()
{
  return StartFUC;
}
//*******************************************************************
inline dword TOgleActiveElement::GetEndFUC()
{
  if (PBase == NULL) {return StartFUC;}
//mrp - v2.4 - 08/17/99 - possible fix for CanActivate failing.
//  return (PBase->Get_FUC_Length() + StartFUC);
  dword dw = PBase->Get_FUC_Length();
  if (dw > 0) {dw -= 1;}
  dw += StartFUC;
  return dw;
}
//*******************************************************************
//*******************************************************************
inline TOgleActiveElement *TOgleActiveList::GetFirst()
{
  return ((TOgleActiveElement *)LTravFirst());
}
//*******************************************************************
inline TOgleActiveElement *TOgleActiveList::GetNext()
{
  return ((TOgleActiveElement *)LTravNext());
}
//*******************************************************************
inline string &TOgleActiveList::GetName()
{
  return Name;
}
//*******************************************************************
//*******************************************************************
inline TOgleActiveList *TOgleListWAL::GetActiveList(int16 index)
{
  if (PActiveList == NULL) {return NULL;}
  if ((index >= NumActiveLists) || (index < 0)) {return NULL;}
  return (PActiveList[index]);
}
//*******************************************************************
inline bool TOgleListWAL::CheckActiveListIndex(int16 index)
{
  if ((index < 0) || (index >= NumActiveLists)) {return false;}
  return true;
}
//*******************************************************************
inline int16 TOgleListWAL::GetNumActiveLists()
{
  return NumActiveLists;
}
//*******************************************************************
#endif
