//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  orectlst.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     7/24/96    M.Perry     original baseline
//  2.0     8/22/97    M.Perry     32-bit conversion & added cached rect code.
// note: need to make some of the rect parameters 'const".                              
//********************************************************************
#if !defined(__orectlst_h)
#define __orectlst_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__ogeo_h)
#include "ogeo.h"
#endif

#if !defined(__osliq_h)
#include "osliq.h"
#endif
//*******************************************************************
class TOgleRectList:public TSliq
{
protected:
  bool AddRect(TOgleRect *prect);
  bool RemoveRect(TOgleRect *prect);
  TOgleRect ClipRect;
  bool Clipping;
  bool IsClipping();
  void SetClipping(bool clipping);
  void SetClipRect(TOgleRect &rect);
  bool CreateAddRect(TOgleRect &rect);
  bool DeleteRemoveRect(TOgleRect *prect);
  bool AddMergeRect(TOgleRect &rect);
  bool AddDisjointRects(TOgleRect &rect);
  TSliq *PCachedRectHolder;
  TOgleRect *GetCachedRect(TOgleRect &rect);
  void ReleaseCachedRect(TOgleRect *prect);

public:
  TOgleRectList();
  TOgleRectList(TOgleRect &cliprect);
  virtual ~TOgleRectList();
  TOgleRect *GetFirst();
  TOgleRect *GetNext();
  TOgleRect *PopRect();

  virtual bool Add(TOgleRect &rect);
  bool Remove(TOgleRect &rect);
  bool AddToList(TOgleRectList &rectlist);
  void Empty();
  TOgleRect GetEnclosingRect();
};
//*******************************************************************
class TRectMergeList:public TOgleRectList
{
public:
  TRectMergeList(TOgleRect &cliprect);
  TRectMergeList();
  virtual ~TRectMergeList();
  virtual bool Add(TOgleRect &rect);
};
//*******************************************************************
class TRectNoOverlapList:public TOgleRectList
{
public:
  TRectNoOverlapList(TOgleRect &cliprect);
  TRectNoOverlapList();
  virtual ~TRectNoOverlapList();
  virtual bool Add(TOgleRect &rect);
};
//*******************************************************************
//****************** TOgleRectList inlines **************************
//*******************************************************************
inline bool TOgleRectList::AddRect(TOgleRect *prect)
{
  return QFrontAdd(prect);
}
//*******************************************************************
inline bool TOgleRectList::RemoveRect(TOgleRect *prect)
{
  return LMatchRemove(prect);
}
//*******************************************************************
inline bool TOgleRectList::IsClipping()
{
  return Clipping;
}
//*******************************************************************
inline void TOgleRectList::SetClipping(bool clipping)
{
  Clipping = clipping;
}
//*******************************************************************
inline void TOgleRectList::SetClipRect(TOgleRect &rect)
{
  ClipRect = rect;
}
//*******************************************************************
inline bool TOgleRectList::CreateAddRect(TOgleRect &rect)
{
//  return AddRect(new TOgleRect(rect));
  return AddRect(GetCachedRect(rect));
}
//*******************************************************************
inline bool TOgleRectList::DeleteRemoveRect(TOgleRect *prect)
{
  RemoveRect(prect);
//  delete prect;
  ReleaseCachedRect(prect);
  return true;
}
//*******************************************************************
inline TOgleRect *TOgleRectList::GetFirst()
{
  return (TOgleRect *)LTravFirst();
}
//*******************************************************************
inline TOgleRect *TOgleRectList::GetNext()
{
  return (TOgleRect *)LTravNext();
}
//*******************************************************************
inline TOgleRect *TOgleRectList::PopRect()
{
  return (TOgleRect *)Pop();
}
//*******************************************************************
inline TOgleRect *TOgleRectList::GetCachedRect(TOgleRect &rect)
{
  TOgleRect *prect;
  if (PCachedRectHolder == NULL ||
      PCachedRectHolder->GetRecCount() == 0 ||
      ((prect = (TOgleRect *)PCachedRectHolder->Pop()) == NULL))
  {
    return new TOgleRect(rect);
  }
  else
  {
    *prect = rect;
    return prect;
  }
}
//*******************************************************************
inline void TOgleRectList::ReleaseCachedRect(TOgleRect *prect)
{
  if (PCachedRectHolder != NULL)
  {
    PCachedRectHolder->Push(prect);
  }
  else
  {
    delete prect;
  }
}
//*******************************************************************
#endif

