//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name: oscrnmgr.h
//    Project Name: Ogle
// Original Author: Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     02/27/95   M. Perry    original baseline
//  2.0     08/28/97   M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__oscrnmgr_h)
#define __oscrnmgr_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__oresplst_h)
#include "oresplst.h"
#endif

#if !defined(__ovideo_h)
#include "ovideo.h"
#endif

#ifdef DEBUGSPRITES
  #include <iostream.h>
  #include <fstream.h>
#endif

class OGLEOWLCLASS TOgleModeWindow;
class TBmp;
//*******************************************************************
class TOgleScreenMgr
{
protected:
  TOgleModeWindow *PModeWin;

public:
  TOgleScreenMgr(TOgleModeWindow *pmodewin,TBmp *pbg,TPal *ppal);
  virtual ~TOgleScreenMgr();

  TOgleVideo *PVideo;
  virtual bool CreateVideo(TOgleModeWindow *pmw,TBmp &bmp,TPal &pal);
  virtual bool DestroyVideo();

  TOgleModeWindow *GetModeWin();

  // graphics list stuff.
  bool AddGr(TGr *pgr);
  bool RemoveGr(TGr *pgr);

  // painting functions
  bool EvEraseBkgnd(HDC hDC);
  void Paint(TDC& dc,bool erase, TRect& rect);
  void Update();

  // handle system messages that affect the palette.
  void EvSetFocus(HWND hWndLostFocus);
  void EvKillFocus(HWND hWndGetFocus);
  bool EvQueryNewPalette();
  void EvPaletteChanged(HWND hWndPalChg);
  void Activate();
  void DeActivate();

  // keyboard & mouse handling.
  TResponderList *PResponderList;
  virtual void CreateResponderList();
  virtual void DestroyResponderList();
  void SetZChanged();
  bool AddResponder(TResponder *responder);
  bool RemoveResponder(TResponder *responder);
  bool SetPriority(TResponder *responder);
  void ClearPriority(TResponder *responder);
  void LDown(uint modkeys,TPoint &point);
  void LUp(uint modkeys,TPoint &point);
  void LDblClk(uint modkeys,TPoint &point);
  void RDown(uint modkeys,TPoint &point);
  void RUp(uint modkeys,TPoint &point);
  void RDblClk(uint modkeys,TPoint &point);
  void MouseMove(uint modkeys,TPoint &point);
  void Char(uint key,uint repeatCount,uint flags);
  void SetCursor(TPoint &point);
  void ReSendEnterArea();
  void SendToEnterAreaResponder(int msg);

  // GEd stuff
//  virtual void ChangeBG(TBmp &bmp);
  virtual bool ChangeBG(TBmp *pbmp);
  virtual void ChangePal(TPal &pal);

  #ifdef DEBUGSPRITES
  void RTReport(ofstream &os);
  #endif
};
//*******************************************************************
//************************* inlines *********************************
//*******************************************************************
inline TOgleModeWindow *TOgleScreenMgr::GetModeWin()
{
  return PModeWin;
}
//*******************************************************************
inline bool TOgleScreenMgr::AddGr(TGr *pgr)
{
  return PVideo->AddGr(pgr);
}
//*******************************************************************
inline bool TOgleScreenMgr::RemoveGr(TGr *pgr)
{
  return PVideo->RemoveGr(pgr);
}
//*******************************************************************
inline bool TOgleScreenMgr::EvEraseBkgnd(HDC hDC)
{
  return true;
}
//*******************************************************************
inline void TOgleScreenMgr::Paint(TDC& dc,bool erase, TRect& rect)
{
  if (PVideo != NULL)
  {
    PVideo->Paint(dc,erase,rect);
  }
}
//*******************************************************************
inline void TOgleScreenMgr::Update()
{
  if (PVideo != NULL)
  {
    PVideo->Update();
  }
}
//*******************************************************************
inline void TOgleScreenMgr::EvSetFocus(HWND hWndLostFocus)
{
  if (PVideo != NULL)
  {
    PVideo->Activate();
  }
}
//*******************************************************************
inline void TOgleScreenMgr::EvKillFocus(HWND hWndGetFocus)
{
}
//*******************************************************************
inline bool TOgleScreenMgr::EvQueryNewPalette()
{
  return ((PVideo == NULL) ? false : PVideo->Activate());
}
//*******************************************************************
inline void TOgleScreenMgr::EvPaletteChanged(HWND hWndPalChg)
{
  if (PVideo != NULL)
  {
    PVideo->DeActivate(true);
  }
}
//*******************************************************************
inline void TOgleScreenMgr::Activate()
{
  if (PVideo != NULL)
  {
    PVideo->Activate();
  }
}
//*******************************************************************
inline void TOgleScreenMgr::DeActivate()
{
  if (PVideo != NULL)
  {
    PVideo->DeActivate();
  }
}
//*******************************************************************
inline void TOgleScreenMgr::SetZChanged()
{
  PResponderList->SetZChanged();
}
//*******************************************************************
inline bool TOgleScreenMgr::AddResponder(TResponder *responder)
{
  return PResponderList->AddResponder(responder);
}
//*******************************************************************
inline bool TOgleScreenMgr::RemoveResponder(TResponder *responder)
{
  return PResponderList->RemoveResponder(responder);
}
//*******************************************************************
inline bool TOgleScreenMgr::SetPriority(TResponder *responder)
{
  return PResponderList->SetPriority(responder);
}
//*******************************************************************
inline void TOgleScreenMgr::ClearPriority(TResponder *responder)
{
  PResponderList->ClearPriority(responder);
}
//*******************************************************************
inline void TOgleScreenMgr::LDown(uint modkeys,TPoint &point)
{
  PResponderList->LDown(modkeys,point);
}
//*******************************************************************
inline void TOgleScreenMgr::LUp(uint modkeys,TPoint &point)
{
  PResponderList->LUp(modkeys,point);
}
//*******************************************************************
inline void TOgleScreenMgr::LDblClk(uint modkeys,TPoint &point)
{
  PResponderList->LDblClk(modkeys,point);
}
//*******************************************************************
inline void TOgleScreenMgr::RDown(uint modkeys,TPoint &point)
{
  PResponderList->RDown(modkeys,point);
}
//*******************************************************************
inline void TOgleScreenMgr::RUp(uint modkeys,TPoint &point)
{
  PResponderList->RUp(modkeys,point);
}
//*******************************************************************
inline void TOgleScreenMgr::RDblClk(uint modkeys,TPoint &point)
{
  PResponderList->RDblClk(modkeys,point);
}
//*******************************************************************
inline void TOgleScreenMgr::MouseMove(uint modkeys,TPoint &point)
{
  PResponderList->Move(modkeys,point);
}
//*******************************************************************
inline void TOgleScreenMgr::Char(uint key,uint repeatCount,uint flags)
{
  PResponderList->Char(key,repeatCount,flags);
}
//*******************************************************************
inline void TOgleScreenMgr::SetCursor(TPoint &point)
{
  PResponderList->SetCursor(point);
}
//*******************************************************************
inline void TOgleScreenMgr::ReSendEnterArea()
{
  PResponderList->ReSendEnterArea();
}
//*******************************************************************
inline void TOgleScreenMgr::SendToEnterAreaResponder(int msg)
{
  PResponderList->SendToEnterAreaResponder(msg);
}
//*******************************************************************
#endif

