//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oskitlst.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     8/27/95    M.Perry     original baseline
//  2.0     8/24/97    M.Perry     32-bit conversion.
//  2.1     12/6/97    M.Perry     filename mod's.
//********************************************************************
#if !defined(__oskitlst_h)                 
#define __oskitlst_h

#if !defined(__olist_h)
#include "olist.h"
#endif

#if !defined(__oskit_h)
#include "oskit.h"
#endif

#if !defined(__osktport_h)
#include "osktport.h"
#endif

class TGameObject;
//*******************************************************************
class TSkitList:public TOgleList
{
public:
  enum {FUC_DELTA = 96};
  
protected:
  TGObjSk *POwnerGObj;
  uint16 NumSkits;

public:
  TSkitPortfolio *PSkitPortfolio;

public:
  #ifdef GED_COMPILE
  void CriteriaReport(ofstream &os);
  void ReportSpriteUsage(ofstream &os,int16 gobjindex,int16 skitindex,int16 spriteindex);
  bool IsSpriteUsedBySkit(int16 gobjindex,int16 spriteindex,int16 &usedski);
  #endif

  virtual void Slenderize();

  bool StartSkit(int16 SkitIndex,dword StartFUC);
  bool ActivateSkit(int16 SkitIndex,dword StartFUC);
  bool CanActivateSkit(int16 SkitIndex,dword StartFUC);
  bool CriteriaCheck(int16 SkitIndex,dword StartFUC);
  bool StartCriteriaSkit(int16 skitindex,dword &StartFUC,
                         bool desperate = false,int16 fucdelta = FUC_DELTA);

  // Use to start up a random skit from portfolio lists.
  bool ChooseStartSkit(dword &StartFUC,dword &FUCLen,
                       int16 &skitindex,int16 fucdelta = FUC_DELTA);

  dword GetSkitLength(int16 SkitIndex);
  virtual void SetIndexes();
  virtual void SetIndexes(int16 skitlistindex);

  TSkitList(TOgleModeWindow *pmw,TGObjSk *PNewGObj);
  TSkitList(TOgleModeWindow *pmw,TFileMode NewFileMode,TGObjSk *PNewGObj,
            bool AutoRead = true,bool KeepBaseFile = false);
  virtual ~TSkitList();

  virtual bool Save();
  TSkit *CreateSkit(const string &SkitName);
  virtual string AddExt();
  int16 GetSkitIndex(const string &SkitFName);
//  string GetSkitFName(int16 SkitIndex);
  string GetSkitName(int16 skitindex);
  int16 GetLastNumSkits();
  int16 GetNumSkits();
  TSkit *GetSkitPtr(int16 SkitIndex);
  TSkit *GetSkitPtr(const string &SkitFName);
  TSkit *SwapSkits(int16 SkitIndex,TSkit *PSkit);
  bool DeleteSkit(string skitfname);

protected:
#if defined(SUPPORT_OLD_VER)
  bool ReadLocal1_0();
  bool ReadLocal1_1();
#endif
  bool ReadLocal2_0();
  virtual bool ReadLocal();
  virtual bool WriteLocal();
  virtual const char *GetIDChars() const;
//  virtual void SetMinVer();
  virtual void SetMajVer();
  bool SetSkitIndexTo(int16 SkitIndex,TSkit *PSkit);
  int16 GetNextPortfolioSkitIndex(int16 portfoliolist);
  int16 GetNextRandomPortfolioList();
  int16 GetPortfolioListCurrentPosition(int16 PortfolioList);
  bool LoopThruList(dword &StartFUC,dword &FUCLen,int16 &skitindex,int16 portfoliolist,
                    bool desperate = false,int16 fucdelta = FUC_DELTA);
  bool TryRandomList(dword &StartFUC,dword &FUCLen,int16 &skitindex,int16 &whichlist);
  bool TryLastList(dword &StartFUC,dword &FUCLen,int16 &skitindex,bool desperate = false,
                   int16 fucdelta = FUC_DELTA);
  const int16 GetNumPFLists() const;
  const int16 GetLastPFListNum() const;
};
//*******************************************************************
//*************** Inlines *******************************************
//*******************************************************************
inline int16 TSkitList::GetNextPortfolioSkitIndex(int16 portfoliolist)
{
  return PSkitPortfolio->GetNextPortfolioSkitIndex(portfoliolist);
}
//*******************************************************************
inline int16 TSkitList::GetNextRandomPortfolioList()
{
  int16 rpl = int16(random(GetNumPFLists()));
  return PSkitPortfolio->GetNextValidList(rpl);
}
//*******************************************************************
inline int16 TSkitList::GetPortfolioListCurrentPosition(int16 PortfolioList)
{
  return PSkitPortfolio->GetPortfolioListCurrentPosition(PortfolioList);
}
//*******************************************************************
inline bool TSkitList::TryRandomList(dword &StartFUC,dword &FUCLen,int16 &skitindex,
                                     int16 &whichlist)
{
  whichlist = GetNextRandomPortfolioList();
  return LoopThruList(StartFUC,FUCLen,skitindex,whichlist);
}
//*******************************************************************
inline bool TSkitList::TryLastList(dword &StartFUC,dword &FUCLen,int16 &skitindex,
                                   bool desperate,int16 fucdelta)
{
  int16 pl = GetLastPFListNum();
  return LoopThruList(StartFUC,FUCLen,skitindex,pl,desperate,fucdelta);
}
//*******************************************************************
inline bool TSkitList::StartSkit(int16 SkitIndex,dword StartFUC)
{
  if (!CanActivateSkit(SkitIndex,StartFUC)) {return false;}
  return ActivateSkit(SkitIndex,StartFUC);
}
//*******************************************************************
inline bool TSkitList::ActivateSkit(int16 SkitIndex,dword StartFUC)
{
  if (GetSkitPtr(SkitIndex) == NULL) {return false;}
  GetSkitPtr(SkitIndex)->Activate(StartFUC);
  return true;
}
//*******************************************************************
inline bool TSkitList::CanActivateSkit(int16 SkitIndex,dword StartFUC)
{
  if (GetSkitPtr(SkitIndex) == NULL) {return false;}
  return GetSkitPtr(SkitIndex)->CanActivate(StartFUC);
}
//*******************************************************************
inline bool TSkitList::CriteriaCheck(int16 SkitIndex,dword StartFUC)
{
  if (GetSkitPtr(SkitIndex) == NULL) {return false;}
  return GetSkitPtr(SkitIndex)->CriteriaCheck(StartFUC);
}
//*******************************************************************
inline int16 TSkitList::GetSkitIndex(const string &SkitFName)
{
  return GetElementIndex(SkitFName);
}
//*******************************************************************
//inline string TSkitList::GetSkitFName(int16 SkitIndex)
inline string TSkitList::GetSkitName(int16 skitindex)
{
//  return GetElementPtr(SkitIndex)->GetFName();
  return GetElementPtr(skitindex)->FileMode.Name.GetStr();
}
//*******************************************************************
inline int16 TSkitList::GetLastNumSkits()
{
  return LastAddedElementNum;
}                                     
//*******************************************************************
inline int16 TSkitList::GetNumSkits()
{
  return int16(LastAddedElementNum + 1);
}
//*******************************************************************
inline TSkit *TSkitList::GetSkitPtr(int16 SkitIndex)
{
  return ((TSkit *)GetElementPtr(SkitIndex));
}
//*******************************************************************
inline TSkit *TSkitList::GetSkitPtr(const string &SkitFName)
{
  return ((TSkit *)GetElementPtr(SkitFName));
}
//*******************************************************************
inline const int16 TSkitList::GetNumPFLists() const
{
  return NUMPORTFOLIOLISTS;
}
//*******************************************************************
inline const int16 TSkitList::GetLastPFListNum() const
{
  return int16(GetNumPFLists() - 1);
}
//*******************************************************************
#endif

