//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osktport.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/8/95     M.Perry    original baseline
//  2.0     8/24/97    M.Perry    32-bit conversion.
//  2.1     12/6/97    M.Perry    filename mod's
//********************************************************************
#if !defined(__osktport_h)
#define __osktport_h

#if !defined(__obase_h)
#include "obase.h"
#endif

#if !defined(__osliq_h)
#include "osliq.h"
#endif

#define NUMPORTFOLIOLISTS 10

class TGObjSk;
//*******************************************************************
class TSkitPortfolioElement
{
public:
//  string *PSkitName;
  string SkitName;
  int16 SkitIndex;

public:
  TSkitPortfolioElement(const string &skitname,int16 skitindex = -1);
  ~TSkitPortfolioElement();
  void ReleaseString();
  const string &GetSkitName();
};
//*******************************************************************
class TSkitPortfolioList:public TSliq
{
public:
  string SkitListName;
//  const string EmptyString;
  TGObjSk *POwnerGObj;
  int16 PortNum;
  int16 CurrentPosition;

public:
  int16 ReadCurPos();
  void WriteCurPos();
  void SetIndexes(int16 skitlistindex);
  void Slenderize();
  void ReleaseStrings();
  int16 GetCurrentPosition();
  void IncCurrentPosition();
  void DecCurrentPosition();
  TSkitPortfolioList(TGObjSk *PGObj,string &skitlistname,int16 portnum);
  TSkitPortfolioList(TSkitPortfolioList &spl);
  virtual ~TSkitPortfolioList();
  TSkitPortfolioElement *GetFirst();
  TSkitPortfolioElement *GetNext();
  const string &GetFirstName();
  const string &GetNextName();
  bool AddName(const string &Str,int16 skitindex);
  bool IndexAddName(int16 indexnum,const string &Str,int16 skitindex = -1);
  int16 GetNumNames() const;
  int16 GetNextSkitIndex();
};
//*******************************************************************
class TSkitPortfolio:public TOgleBase
{
public:
  TGObjSk *POwnerGObj;
  string SkitListName;
  TSkitPortfolioList *PortfolioLists[NUMPORTFOLIOLISTS];

public:
  virtual void SetIndexes();
  virtual void SetIndexes(int16 skitlistindex);
  void ReleaseStrings();
  virtual void SetMinVer();
  virtual void SetMajVer();
  virtual bool ReadLocal();
  bool ReadLocal1_0();
  bool ReadLocal1_1();
  bool ReadLocal2_0();
  bool ReadLocal2_4();
  virtual bool WriteLocal();
  virtual const char *GetIDChars() const;

  TSkitPortfolio(TGObjSk *PGObj);
  TSkitPortfolio(TGObjSk *PGObj,string const &skitlistname,void *PReadData,bool AutoRead = true);
  virtual ~TSkitPortfolio();

  bool AddToList(int16 listnum,const string &Str,int16 skitindex = -1);
  string GetCopyFromList(int16 listnum,int16 nameindex);
  int16 GetNextValidList(int16 StartingAtList);
  int16 GetPortfolioListCurrentPosition(int16 portfoliolist);
  int16 GetNextPortfolioSkitIndex(int16 portfoliolist);
};
//*******************************************************************
//******************* TSkitPortfolioElement inlines *****************
//*******************************************************************
inline TSkitPortfolioElement::TSkitPortfolioElement(const string &skitname,int16 skitindex)
:SkitName(skitname),
 SkitIndex(skitindex)
{
//  PSkitName = new string(skitname);
//  SkitIndex = skitindex;
}
//*******************************************************************
inline TSkitPortfolioElement::~TSkitPortfolioElement()
{
//  if (PSkitName != NULL)
//  {
//    delete PSkitName;
//    PSkitName = NULL;
//  }
//  SkitIndex = -1;
}
//*******************************************************************
inline void TSkitPortfolioElement::ReleaseString()
{
//  if (PSkitName != NULL)
//  {
//    delete PSkitName;
//    PSkitName = NULL;
//  }
  SkitName = TAppString::strEmpty;
}
//*******************************************************************
inline const string &TSkitPortfolioElement::GetSkitName()
{
//  if (PSkitName == NULL) {return TAppString::strEmpty;}
//  return *PSkitName;

  return SkitName;
}
//*******************************************************************
//**************** TSkitPortfolioList inlines ***********************
//*******************************************************************
inline void TSkitPortfolioList::Slenderize()
{
  ReleaseStrings();
}
//*******************************************************************
inline int16 TSkitPortfolioList::GetCurrentPosition()
{
  if (CurrentPosition >= GetNumNames()) {CurrentPosition = 0;}
  return CurrentPosition;
}
//*******************************************************************
inline void TSkitPortfolioList::IncCurrentPosition()
{
  CurrentPosition++;
  if (CurrentPosition >= GetNumNames()) {CurrentPosition = 0;}
}
//*******************************************************************
inline void TSkitPortfolioList::DecCurrentPosition()
{
  CurrentPosition--;
  if (CurrentPosition >= GetNumNames()) {CurrentPosition = int16(GetNumNames() - 1);}
  if (CurrentPosition < 0) {CurrentPosition = int16(GetNumNames() - 1);}
  if (CurrentPosition < 0) {CurrentPosition = 0;}
}
//*******************************************************************
inline TSkitPortfolioElement *TSkitPortfolioList::GetFirst()
{
  return (TSkitPortfolioElement *)LTravFirst();
}
//*******************************************************************
inline TSkitPortfolioElement *TSkitPortfolioList::GetNext()
{
  return (TSkitPortfolioElement *)LTravNext();
}
//*******************************************************************
inline const string &TSkitPortfolioList::GetFirstName()
{
  TSkitPortfolioElement *pspe = GetFirst();
  if (pspe != NULL) {return (pspe->GetSkitName());}
//  return TOgleBase::strGEmpty;
  return TAppString::strEmpty;
}
//*******************************************************************
inline const string &TSkitPortfolioList::GetNextName()
{
  TSkitPortfolioElement *pspe = GetNext();
  if (pspe != NULL) {return (pspe->GetSkitName());}
  //return TOgleBase::strGEmpty;
  return TAppString::strEmpty;
}
//*******************************************************************
inline int16 TSkitPortfolioList::GetNumNames() const
{
  return GetRecCount();
}
//*******************************************************************
inline int16 TSkitPortfolioList::GetNextSkitIndex()
{
  IncCurrentPosition();
  TSkitPortfolioElement *pspe = (TSkitPortfolioElement *)IndexGet(GetCurrentPosition());
  if (pspe != NULL) {return pspe->SkitIndex;}
  return 0;
}
//*******************************************************************
//******************** TSkitPortfolio inlines ***********************
//*******************************************************************
inline int16 TSkitPortfolio::GetPortfolioListCurrentPosition(int16 portfoliolist)
{
  if (portfoliolist < 0) {return 0;}
  if (portfoliolist >= NUMPORTFOLIOLISTS) {return 0;}
  return PortfolioLists[portfoliolist]->GetCurrentPosition();
}
//*******************************************************************
inline int16 TSkitPortfolio::GetNextPortfolioSkitIndex(int16 portfoliolist)
{
  if (portfoliolist < 0) {return 0;}
  if (portfoliolist >= NUMPORTFOLIOLISTS) {return 0;}
  return PortfolioLists[portfoliolist]->GetNextSkitIndex();
}
//*******************************************************************
#endif

