//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name: osliq.h
//    Project Name: Ogle
// Original Author: Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     3/13/95    M.Perry      original baseline
//  2.0     8/21/97    M.Perry      32-bit conversion.
//********************************************************************
#if !defined(__osliq_h)
#define __osliq_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

//*******************************************************************
typedef enum TagCompareResult
{
  LESS_THAN = -1,
  EQUAL_TO = 0,
  GREATER_THAN = 1
}TCompareResult;
//*******************************************************************
#define SORT_CUTOFF 20

typedef TCompareResult (*TCompareFunct)(void *,void *);
TCompareResult DefaultCompareFunct(void *,void *);
//*******************************************************************
class TRecHolder
{
public:
  void *ObjHolder;
  TRecHolder *Next;
  TRecHolder *Prev;

public:
  TRecHolder():ObjHolder(NULL),Next(NULL),Prev(NULL) {}
  ~TRecHolder() {}
};
//*******************************************************************
class TSliq
{
protected:
  TRecHolder *CachedRecHolders;
  bool RemovedEndNode;
  bool OwnObjs;
  bool CachingRecHolders;

public:
  TSliq(bool AllowEqObjPtr = false,bool ownobjs = false);
  virtual ~TSliq();

  TRecHolder *GetRecHolder();
  void ReleaseRecHolder(TRecHolder *precholder);

  TCompareFunct PCompareFunct;
  TCompareFunct SetCompareFunct(TCompareFunct PNewCompareFunct);
  bool Push(void *NewObj);
  bool QAdd(void *NewObj);
  bool LAdd(void *NewObj);
  bool QFrontAdd(void *NewObj);
  bool IndexAdd(int16 indexnum,void *NewObj);
  void *IndexRemove(int16 indexnum);
  void *IndexGet(int16 indexnum);

  void *Pop();
  void *QRemove();
  bool LMatchRemove(void *OldObj);

  bool SliqSort();
  void *LTravFirst();
  void *LTravNext();

  int16 GetRecCount() const;
  int16 GetLastNum() const;
  void IncRecCount();
  void DecRecCount();
  bool CheckObjPtr(void *Newobj);
  void SetLDirLToR();
  void SetLDirRToL();
  bool GetLDir() const;

protected:
  void Quick_Sort(void **a,int16 n);
  void Q_Sort(void **a,int16 left,int16 right);
  int16 Median3(void **a,int16 left, int16 right);
  void Swap(void **a,int16 i,int16 j);
  void Insertion_Sort(void **a,int16 n);

protected:
  bool AllowEq;
  TRecHolder *Left;
  TRecHolder *Right;
  TRecHolder *TravPtr;
  int16 RecCount;
  bool LDirLToR;
};
//*******************************************************************
//*************** inlines *******************************************
//*******************************************************************
inline void TSliq::SetLDirLToR()
{
  LDirLToR = true;
}
//*******************************************************************
inline void TSliq::SetLDirRToL()
{
  LDirLToR = false;
}
//*******************************************************************
inline bool TSliq::GetLDir() const
{
  return LDirLToR;
}
//*******************************************************************
inline void TSliq::IncRecCount()
{
//  RecCount += 1;
  RecCount++;
}
//*******************************************************************
inline void TSliq::DecRecCount()
{
//  RecCount -= 1;
  RecCount--;
  if (RecCount < 0) {RecCount = 0;}
}
//*******************************************************************
inline int16 TSliq::GetRecCount() const
{
  return RecCount;
}
//*******************************************************************
inline int16 TSliq::GetLastNum() const
{
  return int16(RecCount - 1);
}
//*******************************************************************
inline bool TSliq::QAdd(void *NewObj)
{
  return Push(NewObj);
}
//*******************************************************************
inline bool TSliq::LAdd(void *NewObj)
{
  return Push(NewObj);
}
//*******************************************************************
inline void TSliq::Quick_Sort(void **a,int16 n)
{
  Q_Sort(a,1,n);
  Insertion_Sort(a,n);
}
//*******************************************************************
inline int16 TSliq::Median3(void **a,int16 left,int16 right)
{
  int16 center;

  center = int16((left+right)/2);

  if(PCompareFunct(a[left],a[center])== GREATER_THAN)
  {
	 Swap(a,left,center);
  }

  if(PCompareFunct(a[left],a[right])== GREATER_THAN)
  {
    Swap(a,left,right);
  }

  if(PCompareFunct(a[center],a[right])== GREATER_THAN)
  {
    Swap(a,center,right);
  }

  Swap(a,center,int16(right-1));       // Hide pivot
  return int16(right-1);             // Return pivot
}
//*******************************************************************
inline void TSliq::Swap(void **a,int16 i,int16 j)
{
  void *pTmp;

  pTmp = a[i];
  a[i] = a[j];
  a[j] = pTmp;
}
//*******************************************************************
inline TCompareFunct TSliq::SetCompareFunct(TCompareFunct PNewCompareFunct)
{
  TCompareFunct OldCompareFunct = PCompareFunct;
  PCompareFunct = PNewCompareFunct;
  return OldCompareFunct;
}
//*******************************************************************
inline void *TSliq::LTravFirst()
{
   TRecHolder *ListHead = Left;
   TRecHolder *ListTail = Right;
   RemovedEndNode = false;

   if (LDirLToR)
   {
     if (ListHead == NULL) {TravPtr = NULL;return NULL;}
     else                  {TravPtr = ListHead;return (TravPtr->ObjHolder);}
   }
   else
   {
     if (ListTail == NULL) {TravPtr = NULL;return NULL;}
     else                  {TravPtr = ListTail;return (TravPtr->ObjHolder);}
   }
}
//*******************************************************************
inline void *TSliq::LTravNext()
{
  if ((TravPtr == NULL) && (!RemovedEndNode)) {return NULL;}

  if (LDirLToR)
  {
    if (RemovedEndNode) {TravPtr = Left;RemovedEndNode = false;}
    else                {TravPtr = TravPtr->Next;}
  }
  else
  {
    if (RemovedEndNode) {TravPtr = Right;RemovedEndNode = false;}
    else                {TravPtr = TravPtr->Prev;}
  }

  if (TravPtr == NULL) {return NULL;}
  else                 {return (TravPtr->ObjHolder);}
}
//*******************************************************************
inline TRecHolder *TSliq::GetRecHolder()
{
  TRecHolder *prh = NULL;
  if (CachingRecHolders && (CachedRecHolders != NULL))
  {
    prh = CachedRecHolders;
    CachedRecHolders = prh->Next;
  }
  else
  {
    prh = new TRecHolder;
  }
  return prh;
}
//*******************************************************************
inline void TSliq::ReleaseRecHolder(TRecHolder *precholder)
{
  if (CachingRecHolders)
  {
    precholder->Next = CachedRecHolders;
    precholder->Prev = NULL;
    CachedRecHolders = precholder;
  }
  else
  {
    delete precholder;
  }
}
//*******************************************************************
#endif

