//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osndmgr.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     3/2/95     M.Perry      original baseline
//          6/14/95    D.Anderson   implemented SoundMgr
//          6/14/95    M.Perry      merged with Ogle
//         10/21/96    M.Perry      modified audio & music mgrs
//  2.0    8/23/97     M.Perry      32-bit conversion.
//********************************************************************
#if !defined(__osndmgr_h)
#define __osndmgr_h

#if !defined(__osndmgr_rh)
#include "osndmgr.rh"
#endif

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__oaudwmix_h)
#include "oaudwmix.h"
#endif

#if !defined(__owav_h)
#include "owav.h"
#endif

class TMidi;
class TMusicMCI;
//*******************************************************************
class TOgleSoundMgr
{
public:
   TOgleSoundMgr(TWindow* pwin);
   virtual ~TOgleSoundMgr();

   TAudioWaveMix *GetAudioMgr() {return PAudioMgr;}
   bool IsWavPlaying();
   void ToggleAudioMute();
   void SetAudioMute(bool mute);
   bool IsAudioMuted();
   void Activate();
   void DeActivate();
   bool Active;
   void Pump();
   virtual void CreateAudioMgr();
	void Stop(bool notifystop = false);
	void Update(dword fuc);
	bool Play(TMidi* pmidi);
	bool DefPlay(TMidi* pmidi);
	bool Play(TWav* pwav,int16 loopcount = 0);
   void Stop(TWav *pwav);
   void Report();
   void Register(TWav *pwav);
   void UnRegister(TWav *pwav);
   void ChangeStatus(TWav *pwav,TFMStatus status);
   void ClearNotifyStop(TWav *pwav);
   bool UnLoadBelow(TFMStatus status,dword sizeneeded,dword &sizesofar);
   void ModeActivated();
   void ModeDeActivated();
   dword GetSizeUsed();

private:
   TWindow *PWin;
   TMusicMCI *PMusicMgr;
   TAudioWaveMix *PAudioMgr;
};
//*******************************************************************
//************************ inlines **********************************
//*******************************************************************
inline void TOgleSoundMgr::ModeActivated()
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->ModeActivated();}
}
//*******************************************************************
inline void TOgleSoundMgr::ModeDeActivated()
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->ModeDeActivated();}
}
//*******************************************************************
inline bool TOgleSoundMgr::IsWavPlaying()
{
  if (PAudioMgr == NULL) {return false;}
  return PAudioMgr->IsPlaying();
}
//*******************************************************************
inline void TOgleSoundMgr::ToggleAudioMute()
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->ToggleMute();}
}
//*******************************************************************
inline void TOgleSoundMgr::SetAudioMute(bool mute)
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->SetMute(mute);}
}
//*******************************************************************
inline bool TOgleSoundMgr::IsAudioMuted()
{
  if (GetAudioMgr() == NULL) {return true;}
  return GetAudioMgr()->IsMuted();
}
//*******************************************************************
inline void TOgleSoundMgr::Update(dword fuc)
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->Update(fuc);}
}
//*******************************************************************
inline bool TOgleSoundMgr::Play(TWav* pwav,int16 loopcount)
{
  if (GetAudioMgr() == NULL) {return false;}
  return GetAudioMgr()->Add(pwav,loopcount);
}
//*******************************************************************
inline void TOgleSoundMgr::Stop(TWav *pwav)
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->Remove(pwav);}
}
//*******************************************************************
inline void TOgleSoundMgr::ClearNotifyStop(TWav *pwav)
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->ClearNotifyStop(pwav);}
}
//*******************************************************************
inline void TOgleSoundMgr::Register(TWav *pwav)
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->Register(pwav);}
}
//*******************************************************************
inline void TOgleSoundMgr::UnRegister(TWav *pwav)
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->UnRegister(pwav);}
}
//*******************************************************************
inline void TOgleSoundMgr::ChangeStatus(TWav *pwav,TFMStatus status)
{
  if (GetAudioMgr() != NULL) {GetAudioMgr()->ChangeStatus(pwav,status);}
}
//*******************************************************************
inline bool TOgleSoundMgr::UnLoadBelow(TFMStatus status,dword sizeneeded,dword &sizesofar)
{
  if (GetAudioMgr() == NULL) {return false;}
  return GetAudioMgr()->UnLoadBelow(status,sizeneeded,sizesofar);
}
//*******************************************************************
inline dword TOgleSoundMgr::GetSizeUsed()
{
  if (GetAudioMgr() == NULL) {return 0;}
  return GetAudioMgr()->GetSizeUsed();
}
//*******************************************************************
#endif

