//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osprlist.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     6/7/95     M.Perry     original baseline
//  2.0     8/22/97    M.Perry     32-bit conversion.
//  2.1     12/6/97    M.Perry     filename mod's.
//********************************************************************
#if !defined(__osprlist_h)
#define __osprlist_h

#if !defined(__olistwal_h)
#include "olistwal.h"
#endif

#if !defined(__osprite_h)
#include "osprite.h"
#endif

#if !defined(__ogobj_h)
#include "ogobj.h"
#endif

#if defined(DEBUGREPORTS) || defined(DEBUGSPRITES)
  #include <iostream.h>
  #include <fstream.h>
#endif

//*******************************************************************
class TActiveSpriteElement:public TOgleActiveElement
{
  public:
  TActiveSpriteElement();
  TActiveSpriteElement(TOgleBase *PNewBase,dword NewStartFUC);
  virtual ~TActiveSpriteElement();
  #ifdef DEBUGSPRITES
  void RTReport(ofstream &os);
  #endif
};
//*******************************************************************
//*******************************************************************
class TSpriteSetIdle
{
  public:
  TSpriteSetIdle(TGObjSp *pownergobj,const string &spritefname);
  ~TSpriteSetIdle();
  string &GetName();
  void SetIndex();
  void Update(dword NewFUC);
  void PostUpdate(dword NewFUC);
  void Activate(dword StartFUC);
  void DeActivate();
  void Start(dword StartFUC);
  void Stop();
  bool GetStopped();
  void Show();
  void NoShow();

protected:
  TGObjSp *POwnerGObj;
  int16 SpriteListIndex;
  string SpriteName;
  TSprite *PSprite;
  int16 ActiveCount;
  bool Stopped;
};
//*******************************************************************
//*******************************************************************
class TSpriteSet
{
public:
  TGObjSp *POwnerGObj;

public:
  TSpriteSet(TGObjSp *pownergobj);
  ~TSpriteSet();
  bool AddIdle(const string &spritefname);
  bool RemoveIdle(int16 idleindex);
  TSpriteSetIdle *GetIdle(int16 idleindex);
  void SetIndexes();
  void Start(dword StartFUC);
  void Stop();
  void Show();
  void NoShow();
  void Activate(dword StartFUC);
  void DeActivate();
  void Update(dword NewFUC,bool stopidle);
  void PostUpdate(dword NewFUC);
  int16 GetNumIdles();
  string GetIdleName(int16 idleindex);
protected:
  void ChooseIdle();
  int16 CurrentIdleIndex;
  int16 NumIdles;
  TSpriteSetIdle **PPSpriteSetIdles;
};
//*******************************************************************
//*******************************************************************
class TSpriteActiveList:public TOgleActiveList
{
public:
  TGObjSp *POwnerGObj;

  #ifdef DEBUGSPRITES
  void RTReport(ofstream &os);
  #endif

  TSpriteActiveList(TOgleModeWindow *pmw,TGObjSp *PGObj,const string &spritetypename);
  virtual ~TSpriteActiveList();
  virtual TOgleActiveElement *CreateNewElement();
  virtual TOgleActiveElement *CreateNewElement(TOgleBase *PBase,dword StartFUC);

  virtual void Update(dword NewFUC);
  virtual void PostUpdate(dword NewFUC);
  bool IsRangeAvail(dword StartFUC,dword EndFUC);

  bool AddSet();
  bool RemoveSet(int16 setindex);
  bool AddSetIdle(int16 setindex,const string &spritefname);
  bool RemoveSetIdle(int16 setindex,int16 idleindex);
  void SetIndexes();

  void Activate(dword StartFUC);
  void DeActivate();
  void Show();
  void NoShow();
  bool SetCurrentSet(int16 setindex);
  int16 GetCurrentSetIndex();

  TSpriteSet *GetSet(int16 setindex);
  int16 GetNumSets();
  int16 GetSetNumIdles(int16 setindex);
  string GetSetIdleName(int16 setindex,int16 idleindex);

protected:
  int16 CurrentSetIndex;
  int16 NumSets;
  TSpriteSet **PPSpriteSets;
};
//*******************************************************************
//*******************************************************************
class TSpriteList:public TOgleListWAL
{
public:
  TGObjSp *POwnerGObj;

public:
  TGObjSp *GetGObjSp() {return POwnerGObj;}
  #ifdef DEBUGREPORTS
  void Report(ofstream &os);
  #endif
  #ifdef DEBUGSPRITES
  void RTReport(ofstream &os);
  #endif
  #if defined(GED_COMPILE)
  bool CheckSpriteIDConflicts(bool verbose = false);
  #endif
  #if defined(GED_COMPILE)
  bool CheckSpriteIDUsage(bool verbose = false);
  #endif
  virtual void Slenderize();
  TSpriteList(TOgleModeWindow *pmw,TGObjSp *PGObj);
  TSpriteList(TOgleModeWindow *pmw,const TFileMode &fm,TGObjSp *PGObj,
              bool AutoRead = true,bool KeepBaseFile = false);
  TSpriteList(TOgleModeWindow *pmw,bool Dummy,const TFileMode &fm,TGObjSp *PGObj);
  virtual TSprite *CreateNewElement();
  virtual ~TSpriteList();

protected:
//  bool ReadLocal1_0();
//  bool ReadLocal1_1();
#if defined(SUPPORT_OLD_VER)
  bool ReadLocal1_2();
  bool ReadLocal1_3();
#endif
  bool ReadLocal2_0();
  virtual bool ReadLocal();
  virtual bool WriteLocal();
  virtual const char *GetIDChars() const;

public:
  virtual bool Save();
  virtual TSprite *CreateSprite(string spritename,int16 id = 0,void *psource = NULL);
//  virtual TSprite *CreateSprite(const string &SpriteName,int16 id = 0);
//  virtual TSprite *CreateSprite(const string &SpriteName,void *psource,int16 id = 0);
  bool DeleteSprite(const string &SpriteName);
  virtual string AddExt();
  int16 GetSpriteIndex(const string &SpriteFName);
  int16 GetSpriteIndex(TSprite *psprite);
//  string GetSpriteFName(int16 SpriteIndex);
  string GetSpriteName(int16 spriteindex);
  TSprite *GetSpritePtr(int16 SpriteIndex);
  TSprite *GetSpritePtr(const string &SpriteName);
  int16 GetLastNumSprites();
  TSprite *SwapSprites(int16 SpriteIndex,TSprite *PSprite);
  bool CanActivateElement(int16 ElementNum,dword StartFUC);
  bool CanActivateElement(TOgleBase *PBase,dword StartFUC);
  bool CanActivateSprite(int16 SpriteIndex,dword StartFUC);
  dword GetSpriteLength(int16 SpriteIndex);
  bool SetCurrentSpriteSet(int16 typeindex,int16 setindex);
  int16 GetCurrentSpriteSetIndex(int16 typeindex);
  virtual void Activate(dword StartFUC);
  void DeActivate();
  virtual void Show();
  virtual void NoShow();

  int16 GetNumTypes();
  bool AddSpriteSet(int16 typeindex);
  bool RemoveSpriteSet(int16 typeindex,int16 setindex);
  bool AddSpriteSetIdle(int16 typeindex,int16 setindex,const string &spritefname);
  bool RemoveSpriteSetIdle(int16 typeindex,int16 setindex,int16 idleindex);
  int16 GetNumSets(int16 typeindex);
  string GetTypeName(int16 typeindex);
  int16 GetSpriteSetNumIdles(int16 typeindex,int16 setindex);
  string GetSpriteSetIdleName(int16 typeindex,int16 setindex,int16 idleindex);
  virtual int16 GetActiveListIndex(TOgleBase *PBase);
  void SetIndexes();

protected:
  virtual TOgleActiveList *CreateActiveList(const string &name);
  TSpriteActiveList *GetSpriteActiveList(int16 index);
  bool SetSpriteIndexTo(int16 SpriteIndex,TSprite *PSprite);

  virtual void SetMinVer();
  virtual void SetMajVer();
};
//*******************************************************************
//**********************  inlines  **********************************
//*******************************************************************
inline string &TSpriteSetIdle::GetName()
{
  return SpriteName;
}
//*******************************************************************
inline void TSpriteSetIdle::Update(dword NewFUC)
{
  if (PSprite != NULL) {PSprite->Update(NewFUC);}
}
//*******************************************************************
inline void TSpriteSetIdle::PostUpdate(dword NewFUC)
{
  if (PSprite != NULL)
  {
    PSprite->PostUpdate(NewFUC);
    if (PSprite->GetStopped()) {Stopped = true;}
  }
}
//*******************************************************************
inline void TSpriteSetIdle::Start(dword StartFUC)
{
  Stopped = false;
  if (PSprite != NULL) {PSprite->Start(StartFUC);}
}
//*******************************************************************
inline void TSpriteSetIdle::Stop()
{
  if (PSprite != NULL) {PSprite->Stop();}
  Stopped = true;
}
//*******************************************************************
inline bool TSpriteSetIdle::GetStopped()
{
  return Stopped;
}
//*******************************************************************
inline void TSpriteSetIdle::Show()
{
  if (PSprite != NULL) {PSprite->Show();}
}
//*******************************************************************
inline void TSpriteSetIdle::NoShow()
{
  if (PSprite != NULL) {PSprite->NoShow();}
}
//*******************************************************************
//*******************************************************************
inline TSpriteSetIdle *TSpriteSet::GetIdle(int16 idleindex)
{
  if (PPSpriteSetIdles == NULL) {return NULL;}
  if ((idleindex >= NumIdles) || (idleindex < 0)) {return NULL;}
  return (PPSpriteSetIdles[idleindex]);
}
//*******************************************************************
inline void TSpriteSet::Start(dword StartFUC)
{
  ChooseIdle();
  if (GetIdle(CurrentIdleIndex) != NULL)
  {
    GetIdle(CurrentIdleIndex)->Start(StartFUC);
  }
}
//*******************************************************************
inline void TSpriteSet::Stop()
{
  if (GetIdle(CurrentIdleIndex) != NULL) {GetIdle(CurrentIdleIndex)->Stop();}
  CurrentIdleIndex = -1;
}
//*******************************************************************
inline void TSpriteSet::PostUpdate(dword NewFUC)
{
  if (GetIdle(CurrentIdleIndex) != NULL)
  {
    GetIdle(CurrentIdleIndex)->PostUpdate(NewFUC);
  }
}
//*******************************************************************
inline int16 TSpriteSet::GetNumIdles()
{
  return NumIdles;
}
//*******************************************************************
inline string TSpriteSet::GetIdleName(int16 idleindex)
{
  if (GetIdle(idleindex) == NULL) {return string("");}
  return (GetIdle(idleindex)->GetName());
}
//*******************************************************************
inline void TSpriteSet::ChooseIdle()
{
  CurrentIdleIndex = int16(random(NumIdles));
}
//*******************************************************************
//*******************************************************************
inline int16 TSpriteActiveList::GetCurrentSetIndex()
{
  return CurrentSetIndex;
}
//*******************************************************************
inline TSpriteSet *TSpriteActiveList::GetSet(int16 setindex)
{
  if (PPSpriteSets == NULL) {return NULL;}
  if ((setindex < 0) || (setindex >= NumSets)) {return NULL;}
  return (PPSpriteSets[setindex]);
}
//*******************************************************************
inline int16 TSpriteActiveList::GetNumSets()
{
  return NumSets;
}
//*******************************************************************
//*******************************************************************
inline int16 TSpriteList::GetSpriteIndex(const string &SpriteFName)
{
  return GetElementIndex(SpriteFName);
}
//*******************************************************************
inline int16 TSpriteList::GetSpriteIndex(TSprite *psprite)
{
  return GetElementIndex(psprite);
}
//*******************************************************************
inline TSprite *TSpriteList::GetSpritePtr(int16 SpriteIndex)
{
  if (SpriteIndex > LastAddedElementNum) {return NULL;}
  if (SpriteIndex < 0) {return NULL;}
  return ((TSprite *)List[SpriteIndex]);
}
//*******************************************************************
inline TSprite *TSpriteList::GetSpritePtr(const string &SpriteName)
{
  return ((TSprite *)GetElementPtr(SpriteName));
}
//*******************************************************************
inline int16 TSpriteList::GetLastNumSprites()
{
  return LastAddedElementNum;
}
//*******************************************************************
inline bool TSpriteList::CanActivateElement(int16 ElementNum,dword StartFUC)
{
  return CanActivateElement(GetElementPtr(ElementNum),StartFUC);
}
//*******************************************************************
inline bool TSpriteList::CanActivateSprite(int16 SpriteIndex,dword StartFUC)
{
  return CanActivateElement(SpriteIndex,StartFUC);
}
//*******************************************************************
inline dword TSpriteList::GetSpriteLength(int16 SpriteIndex)
{
  if (GetSpritePtr(SpriteIndex) == NULL) {return 0;}
  return (GetSpritePtr(SpriteIndex)->Get_FUC_Length());
}
//*******************************************************************
inline int16 TSpriteList::GetNumTypes()
{
  return GetNumActiveLists();
}
//*******************************************************************
inline TSpriteActiveList *TSpriteList::GetSpriteActiveList(int16 index)
{
  return ((TSpriteActiveList *)GetActiveList(index));
}
//*******************************************************************
#endif

