//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ostr.h
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     5/3/96     M.Perry     original baseline
//  2.0     8/26/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__ostr_h)
#define __ostr_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

#if !defined(__STRING_H)
#include "string.h"
#endif
//*******************************************************************
class TStr
// always null terminated.
{
protected:
  char *PChars;
  #if defined(DEBUGTSTR)
  static int32 AllocSize;
  #endif
public:
  const char *GetP() const;
  void FreeP();
  void SetToNull();
  void AllocP(int16 size);
  TStr();
  ~TStr();
  TStr(const TStr &str);
  TStr(const string &s);
  TStr(const char *pchars);
  TStr(const char character);
  TStr &operator = (const TStr &str);
  TStr &operator = (const string &s);
  TStr &operator = (const char *pchars);
  TStr &operator = (const char character);
  TStr &operator += (const TStr &str);
  TStr &operator += (const string &s);
  TStr &operator += (const char *pchars);
  TStr &operator += (const char character);
  int16 Length() const;
  void To_Lower();
  void To_Upper();
  char *Alloc(int16 size);
  void DeAlloc(char *p);
  void Assign(const TStr &str);
  void Assign(const string &s);
  void Assign(const char *pchars);
  void Assign(const char character);
  void Append(const TStr &str);
  void Append(const string &s);
  void Append(const char *pchars);
  void Append(const char character);
  void Prepend(const TStr &str);
  void Prepend(const string &s);
  void Prepend(const char *pchars);
  void Prepend(const char character);
  int Compare(const TStr &str2);
  int Compare(const string &str);
  operator const char * () const;
  operator const string () const;
};
//*******************************************************************
//******************* Friend functions ******************************
ostream &operator << (ostream &os,const TStr &s);
//*******************************************************************
//**************** TStr inlines *************************************
//*******************************************************************
inline const char *TStr::GetP() const
{
  return PChars;
}
//*******************************************************************
inline void TStr::FreeP()
{
  DeAlloc(PChars);
  PChars = NULL;
}
//*******************************************************************
inline void TStr::SetToNull()
{
  if (PChars != NULL) {FreeP();}
  AllocP(1);
  PChars[0] = '\0';
}
//*******************************************************************
inline void TStr::AllocP(int16 size)
{
  PChars = Alloc(size);
}
//*******************************************************************
inline TStr::TStr()
{
  PChars = NULL;
  SetToNull();
}
//*******************************************************************
inline TStr::~TStr()
{
  FreeP();
}
//*******************************************************************
inline TStr::TStr(const TStr &str)
{
  PChars = NULL;
  Assign(str);
}
//*******************************************************************
inline TStr::TStr(const string &s)
{
  PChars = NULL;
  Assign(s);
}
//*******************************************************************
inline TStr::TStr(const char *pchars)
{
  PChars = NULL;
  Assign(pchars);
}
//*******************************************************************
inline TStr::TStr(const char character)
{
  PChars = NULL;
  Assign(character);
}
//*******************************************************************
inline TStr &TStr::operator = (const TStr &str)
{
  if (&str != this)
  {
    Assign(str);
  }
  return *this;
}
//*******************************************************************
inline TStr &TStr::operator = (const string &s)
{
  Assign(s);
  return *this;
}
//*******************************************************************
inline TStr &TStr::operator = (const char *pchars)
{
  if (pchars != PChars)
  {
    Assign(pchars);
  }
  return *this;
}
//*******************************************************************
inline TStr &TStr::operator = (const char character)
{
  Assign(character);
  return *this;
}
//*******************************************************************
inline TStr &TStr::operator += (const TStr &str)
{
  Append(str);
  return *this;
}
//*******************************************************************
inline TStr &TStr::operator += (const string &s)
{
  Append(s);
  return *this;
}
//*******************************************************************
inline TStr &TStr::operator += (const char *pchars)
{
  Append(pchars);
  return *this;
}
//*******************************************************************
inline TStr &TStr::operator += (const char character)
{
  Append(character);
  return *this;
}
//*******************************************************************
inline int16 TStr::Length() const
{
  return int16(strlen(PChars));
}
//*******************************************************************
inline void TStr::To_Lower()
{
  strlwr(PChars);
}
//*******************************************************************
inline void TStr::To_Upper()
{
  strupr(PChars);
}
//*******************************************************************
inline char *TStr::Alloc(int16 size)
{
  #ifdef DEBUGTSTR
  AllocSize += size;
  #endif
  return (new char[size]);
}
//*******************************************************************
inline void TStr::DeAlloc(char *p)
{
  if (p != NULL)
  {
    #ifdef DEBUGTSTR
    AllocSize -= (strlen(p) + 1);
    #endif
    delete [] p;
  }
}
//*******************************************************************
inline int TStr::Compare(const TStr &str2)
{
  return strcmp(GetP(),str2.GetP());
}
//*******************************************************************
inline int TStr::Compare(const string &str)
{
  return strcmp(GetP(),str.c_str());
}
//*******************************************************************
inline TStr::operator const char * () const
{
  return GetP();
}
//*******************************************************************
//**************** global functions *********************************
//*******************************************************************
TStr operator + (const TStr &str1,const TStr &str2);
TStr operator + (const TStr &str1,const string &s2);
TStr operator + (const TStr &str1,const char *pchars);
TStr operator + (const TStr &str1,const char character);

//************ inlines **********************************************
inline bool operator == (const TStr &str1,const TStr &str2)
{
  int result = strcmp(str1.GetP(),str2.GetP());
  return bool(result == 0);
}
//*******************************************************************
inline bool operator != (const TStr &str1,const TStr &str2)
{
  int result = strcmp(str1.GetP(),str2.GetP());
  return bool(result != 0);
}
//*******************************************************************
inline bool operator < (const TStr &str1,const TStr &str2)
{
  int result = strcmp(str1.GetP(),str2.GetP());
  return bool(result < 0);
}
//*******************************************************************
inline bool operator <= (const TStr &str1,const TStr &str2)
{
  int result = strcmp(str1.GetP(),str2.GetP());
  return bool(result <= 0);
}
//*******************************************************************
inline bool operator > (const TStr &str1,const TStr &str2)
{
  int result = strcmp(str1.GetP(),str2.GetP());
  return bool(result > 0);
}
//*******************************************************************
inline bool operator >= (const TStr &str1,const TStr &str2)
{
  int result = strcmp(str1.GetP(),str2.GetP());
  return bool(result >= 0);
}
//*******************************************************************
#endif

