//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ovideo
//    Project Name:  Ogle                  
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     3/2/95     M.Perry     original baseline
//********************************************************************
#if !defined(__ovideo_h)  
#define __ovideo_h

#if !defined(__ogle_h)
#include "ogle.h"
#endif

//#include "oframe.h"

#if !defined(__ovidbmp_h)
#include "ovidbmp.h"
#endif

#if !defined(__ovidpal_h)
#include "ovidpal.h"
#endif

#if !defined(__odc_h)
#include "odc.h"
#endif

#if !defined(__ogrlist_h)
#include "ogrlist.h"
#endif

#define TRANSPARENTCOLOR 0
#define SCREENWIDTH 640
#define SCREENHEIGHT 480
#define NUMBEROFCOLORS 256

class OGLEOWLCLASS TOgleModeWindow;
//*******************************************************************
enum TRenderMethod {rmSLOPPY,rmEXACT,rmREDUNDANT};
//*******************************************************************
class TOgleVideo
{
protected:
  TRenderMethod BGMethod,GrMethod;

  TSize WinSize;
  TGrList GrList;
  bool Active;

  TBmp *PBG;
  TPal *PMasterPal;
  TVideoBmp *PVBmp;
  TVideoPal *PVPal;
  TOgleVideoDC *PVDC;
  TOgleModeWindow *PModeWin;
  TOgleRect BgBigRect;
  bool PaintFrameDropsOk;
  void SetDropsOk(bool ok);

  TOgleRectList *PCurrentList;
  TOgleRectList *PRemovedList;
  TOgleRectList *PBGList;

  virtual bool CreateCurrentList();
  virtual void DestroyCurrentList();
  virtual bool CreateRemovedList();
  virtual void DestroyRemovedList();
  virtual bool CreateBGList();
  virtual void DestroyBGList();

  virtual bool CreateVideoBmp();
  virtual void DestroyVideoBmp();
  virtual bool CreateVideoPal();
  virtual void DestroyVideoPal();
  virtual bool CreateVideoDC();
  virtual void DestroyVideoDC();
  virtual void SetupVideoDC();
  virtual void CleanupVideoDC();
  virtual TOglePaintDC *CreatePaintDC(TDC &dc);
  virtual void DestroyPaintDC(TOglePaintDC *ppdc);
  bool Initialized;

public:
  bool IsDropsOk();
  TOgleVideo(TOgleModeWindow *pmw,TBmp &mbg,TPal &mpal,
             TRenderMethod bgmethod,TRenderMethod grmethod);
  virtual ~TOgleVideo();
  virtual bool Initialize();
  virtual bool DeInitialize();
  bool IsInitialized();

  // access functions.
  TGrList &GetGrList();
  TOgleRectList *GetCurrentList();
  bool AddGr(TGr *pgr);
  bool RemoveGr(TGr *pgr);
  TOgleModeWindow *GetModeWin();
  TVideoBmp *GetVBmp();
  TVideoPal *GetVPal();
  TOgleVideoDC *GetVDC();

  // update stuff.
  virtual void Paint(TDC& dc,bool erase, TRect& rect);
  virtual void Update();

  // palette stuff.
  bool Activate();
  void DeActivate(bool palchange = false);

  // GEd stuff.
//  virtual void ChangeBG(TBmp &bmp);
  virtual bool ChangeBG(TBmp *pbmp);
  virtual void ChangePal(TPal &pal);

  #if defined(ENB_FUC_STEPS)
  bool RectsShowing;
  void ShowRects();
  void UnShowRects() {ShowRects();}
  #endif

protected:
  // palette stuff.
  virtual bool RealizePaletteFG();
  virtual bool RealizePaletteBG();

  // update stuff.
  virtual void Paint(TOgleRect &rect,TOglePaintDC &pdc);
  virtual void ProcessRects();
  virtual void RenderBG();
  virtual void Render();
  virtual void Paint();
};
//*******************************************************************
//****************** inlines ****************************************
//*******************************************************************
inline void TOgleVideo::SetDropsOk(bool ok)
{
  PaintFrameDropsOk = ok;
}
//*******************************************************************
inline bool TOgleVideo::IsDropsOk()
{
  return PaintFrameDropsOk;
}
//*******************************************************************
inline bool TOgleVideo::IsInitialized()
{
  return Initialized;
}
//*******************************************************************
inline TGrList &TOgleVideo::GetGrList()
{
  return GrList;
}
//*******************************************************************
inline TOgleRectList *TOgleVideo::GetCurrentList()
{
  return PCurrentList;
}
//*******************************************************************
inline bool TOgleVideo::AddGr(TGr *pgr)
{
  return GrList.AddGr(pgr);
}
//*******************************************************************
inline bool TOgleVideo::RemoveGr(TGr *pgr)
{
  PRemovedList->Add(pgr->GrDrawData.GetOldVRect());
  PRemovedList->Add(pgr->GrDrawData.GetVRect());
  return GrList.RemoveGr(pgr);
}
//*******************************************************************
inline TOgleModeWindow *TOgleVideo::GetModeWin()
{
  return PModeWin;
}
//*******************************************************************
inline TVideoBmp *TOgleVideo::GetVBmp()
{
  return PVBmp;
}
//*******************************************************************
inline TVideoPal *TOgleVideo::GetVPal()
{
  return PVPal;
}
//*******************************************************************
inline TOgleVideoDC *TOgleVideo::GetVDC()
{
  return PVDC;
}
//*******************************************************************
#endif

