//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  odll.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//   1.0    7/7/96      M.Perry    original baseline
//   2.0    8/23/97     M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__odll_h)
#include "odll.h"
#endif

#if !defined(__odll_rh)
#include "odll.rh"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif
//*******************************************************************
bool TOgleDll::SetupProcs()
{
  return true;
}
//*******************************************************************
bool TOgleDll::CleanupProcs()
{
  return true;
}
//*******************************************************************
TOgleDll::TOgleDll(const char *dllName,bool loaderrorfatal)
  :Name(dllName)
{
  LoadErrorFatal = loaderrorfatal;
  #if defined(BI_PLAT_WIN16)
  hInst = 0;
  #else
  hInst = NULL;
  #endif
  RefCount = 0;
  Initialized = false;
}
//*******************************************************************
TOgleDll::~TOgleDll()
{
  DeInitialize();
}
//*******************************************************************
bool TOgleDll::Load()
{
  // check that class is initialized.
  if (!IsInitialized()) {Initialize();}
  if (!IsInitialized()) {return false;}
  // ok, either we just initialized or it was already initialized, now we RefCount++
  IncRefCount();
  return true;
}
//*******************************************************************
bool TOgleDll::UnLoad()
{
  // if we're not initialized then job is already done.
  if (!IsInitialized()) {return true;}
  DecRefCount();
  // Check if anybody else is using it, if not then unload it.
  if (GetRefCount() == 0) {DeInitialize();}
  return true;
}
//*******************************************************************
bool TOgleDll::Initialize()
{
  if (Initialized) {return true;}
  RefCount = 0;
  bool rc = LoadLibrary();

  if (rc) {rc = SetupProcs();}

  if (!rc) {FreeLibrary();}

  Initialized = rc;
  return Initialized;
}
//*******************************************************************
bool TOgleDll::DeInitialize()
{
  if (!Initialized) {return true;}
  if (CheckInstance()) {FreeLibrary();}
  CleanupProcs();
  Initialized = false;
  return true;
}
//*******************************************************************
bool TOgleDll::LoadLibrary()
{
  if (CheckInstance()) {return true;}
  hInst = ::LoadLibrary(Name.c_str());
  if (CheckInstance()) {return true;}
  return HandleInstanceError();
}
//*******************************************************************
bool TOgleDll::FreeLibrary()
{
  if (!CheckInstance()) {return true;}
  ::FreeLibrary(hInst);
  hInst = 0;
  return true;
}
//*******************************************************************
string TOgleDll::GetInstanceError()
{
  uint id;
  if (CheckInstance()) {id = IDS_LLE_OK;} // no error.
  else if (hInst < 0) {id = IDS_LLE_UNK;} // unknown error.
  else {id = (UINT)hInst + (UINT)IDS_LLE_00;} // index into string table.

  TOgleApp *papp = TOgleApp::GetOgleApp();
  string s;
  if (papp != NULL) {s = papp->LoadString(id);}
  else {s = "Couldn't find app ptr";}
  return s;
}
//*******************************************************************
bool TOgleDll::HandleInstanceError()
{
  TOgleApp *papp = TOgleApp::GetOgleApp();
  string msg;
  string title;
  if (papp != NULL)
  {
    if (CheckInstance()) {return true;}
    msg = papp->LoadString(IDS_LLE_BLURB);
    msg += Name;
    msg += "\n";
    msg += GetInstanceError();
    if (LoadErrorFatal)
    {
      msg += "\n";
      msg += papp->LoadString(IDS_LLE_BLURBFATAL);
    }
    title = papp->LoadString(IDS_LLE_ERRORTITLE);
  }
  else
  {
    msg = "During DLL load, couldn't find app ptr.";
    title = "Error - DLL Load";
  }
  ::MessageBox(NULL,msg.c_str(),title.c_str(),MB_OK);
  if (LoadErrorFatal) {exit(EXIT_FAILURE);}
  return false;
}
//*******************************************************************
bool TOgleDll::SysReLoad()
{
  int16 refcount = RefCount;
  HINSTANCE oldhinst = hInst;
  DeInitialize();
  int16 after = GetSysUsage(oldhinst);
  for (int16 i=0;i<after;i++)
  {
    bool rc = (bool)::FreeModule(oldhinst);
    if (!rc) {break;}
  }
  for (int16 j=0;j<refcount;j++) // bring our own internal count back to our original.
  {
    if (!Load())
    {
      DeInitialize();
      return false;
    }
  }
  return true;
}
//*******************************************************************

