//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ofilemgr.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     3/2/95     M.Perry      original baseline
//********************************************************************
#if !defined(__ofilemgr_h)
#include "ofilemgr.h"
#endif

#if !defined(__STDIO_H)
#include <stdio.h> //disk
#endif

#if !defined(__DOS_H)
#include <dos.h>   //disk
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__DIR_H)
#include <dir.h>
#endif

#include <assert.h>

#if !defined(__NEW_H)
#include <new.h>
#endif

#if !defined(__ocurmgr_h)
#include "ocurmgr.h"
#endif

#if !defined(__odirent_h)
#include "odirent.h"
#endif

#if !defined(__odet_h)
#include "odet.h"
#endif

#if !defined(__odetlist_h)
#include "odetlist.h"
#endif

#if !defined(__osndmgr_h)
#include "osndmgr.h"
#endif

#if !defined(__oinifile_h)
#include "oinifile.h"
#endif

#if defined(BI_PLAT_WIN16)
#if !defined(__TOOLHELP_H)
  #include <toolhelp.h>
#endif
#endif

DIAG_DEFINE_GROUP(FileManager,1,2);
//*******************************************************************
//************ start data classes ***********************************
//*******************************************************************
TFMData::TFMData()
{
  emptystr = "\0";
  SetLastLoadFUC(0);
  SetCurrentFUC(0);
  for(int8 i=0;i<MAXDIRS;i++)
  {
    DirNames[i] = "\\";
  }
}
//*******************************************************************
TFMData::~TFMData()
{
}
//*******************************************************************
void TFMData::SetDir(int8 dirindex,const string &dirnameonly)
// make sure ending backslash is there.
{
  if (dirindex < MAXDIRS)
  {
    string str = dirnameonly;
    if (str.length() > 0)
    {
      if (str.substr(str.length() -1) != "\\") {str += "\\";}
    }
    DirNames[dirindex] = str;
  }
}
//*******************************************************************
string &TFMData::GetDirName(int8 dirindex)
{
  if ((dirindex < 0) || (dirindex >= MAXDIRS))
  {
    return emptystr;
  }
  else
  {
    return DirNames[dirindex];
  }
}
//*******************************************************************
//*******************************************************************
string TFMNames::GetFullDir()
{
  string s = GetDir() + GetDirAdd();
  return s;
}
//*******************************************************************
bool TFMNames::IsDrive(const string &s)
{
  char tempDrive[_MAX_DRIVE];
  _splitpath(s.c_str(),tempDrive,NULL,NULL,NULL);

  if (strcmp(tempDrive,GetDrive().c_str()) == 0) {return true;}
  return false;
}
//*******************************************************************
//*******************************************************************
TFMCDDataNames::TFMCDDataNames(const string &drive,const string &dir)
{
  SetDrive(drive);
  SetDir(dir);
}
//*******************************************************************
//*******************************************************************
TFMHDBufferNames::TFMHDBufferNames()
{
  SetDirAdd("buffer\\");
}
//*******************************************************************
TFMHDBufferNames::TFMHDBufferNames(const string &drive,const string &dir)
{
  SetDrive(drive);
  SetDir(dir);
  SetDirAdd("buffer\\");
  UniqueFileNameCounterRollOver = 0;
  UniqueFileNameCounter = 0;
}
//*******************************************************************
string TFMHDBufferNames::GetUniqueFileName()
// creates a unique filename to be used in buffer dir.
{
  // setup fname.
  char fn[33];
  char fnamebuffer[33];
  char extbuffer[17];
  ultoa(UniqueFileNameCounter,fnamebuffer,10);
  int16 fnamelen = int16(strlen(fnamebuffer));
  strcpy(fn,"fm000000.tmp");
  // copy over part of fname with number.
  int16 fnstart = int16(8 - fnamelen);
  for(int16 i=0;i<fnamelen;i++)
  {
    fn[fnstart + i] = fnamebuffer[i];
  }
  // setup extension.
  if (UniqueFileNameCounterRollOver > 0)
  {
    fn[9] = '0';
    fn[10] = '0';
    fn[11] = '0';
    itoa(UniqueFileNameCounterRollOver,extbuffer,10);
    int16 extlen = int16(strlen(extbuffer));
    // copy over part of ext with number.
    int16 estart = int16(12 - extlen);
    for (int16 j=0;j<extlen;j++)
    {
      fn[estart + j] = extbuffer[j];
    }
  }
  // return string.
  string filename = fn;

  // now setup counters for next time.
  if (UniqueFileNameCounter >= 999999l)
  {
    UniqueFileNameCounterRollOver++;
    UniqueFileNameCounter = 0;
  }
  else
  {
    UniqueFileNameCounter++;
  }
  return filename;
}
//*******************************************************************
//*******************************************************************
TFMHDDataNames::TFMHDDataNames()
{
  SetDirAdd("data\\");
}
//*******************************************************************
TFMHDDataNames::TFMHDDataNames(const string &drive,const string &dir)
{
  SetDrive(drive);
  SetDir(dir);
  SetDirAdd("data\\");
}
//*******************************************************************
//*******************************************************************
bool TFMDiskData::GetDiskFreeData(dword &clusters,
                                  dword &clustersectors,
                                  dword &sectorbytes)
{
  struct diskfree_t free;
  unsigned result = _dos_getdiskfree(DriveNum,&free);
  if (result == 0)
  {
    clusters = (dword)free.avail_clusters;
    clustersectors = (dword)free.sectors_per_cluster;
    sectorbytes = (dword)free.bytes_per_sector;
    return true;
  }
  else
  {
    clusters = 0;
    clustersectors = 0;
    sectorbytes = 0;
    return false;
  }
}
//*******************************************************************
dword TFMDiskData::GetActualDiskSpace()
{
  dword clusters;
  dword clustersectors;
  dword sectorbytes;
  dword availbytes = 0;
  if (GetDiskFreeData(clusters,clustersectors,sectorbytes))
  {
    availbytes = clusters * clustersectors * sectorbytes;
  }
  return availbytes;
}
//*******************************************************************
//*******************************************************************
TCompareResult CompareHiRamNeeded(void *FirstObj,void *SecObj)
{
  if ((FirstObj == NULL) || (SecObj == NULL)) {return EQUAL_TO;}

  TFMStatus First = ((TOgleBaseFile *)FirstObj)->GetRamStatus();
  TFMStatus Second = ((TOgleBaseFile *)SecObj)->GetRamStatus();

  if (First > Second) {return GREATER_THAN;}
  else if (First < Second) {return LESS_THAN;}
  return EQUAL_TO;
}
//*******************************************************************
TCompareResult CompareLoRamNeeded(void *FirstObj,void *SecObj)
{
  if ((FirstObj == NULL) || (SecObj == NULL)) {return EQUAL_TO;}

  TFMStatus First = ((TOgleBaseFile *)FirstObj)->GetRamStatus();
  TFMStatus Second = ((TOgleBaseFile *)SecObj)->GetRamStatus();

  if (First > Second) {return LESS_THAN;}
  else if (First < Second) {return GREATER_THAN;}
  return EQUAL_TO;
}
//*******************************************************************
TFileList::~TFileList()
{
  if (OwnObjs)
  {
    TOgleBaseFile *pbf;
    while ((pbf = (TOgleBaseFile *)Pop()) != NULL)
    {
      delete pbf;
    }
  }
  else
  {
    do {} while (Pop() != NULL);
  }
}
//*******************************************************************
TOgleBaseFile *TFileList::GetWithRamStatus(TFMStatus ramstatus)
// loop thru list, looking for a matching ramstatus.
{
  if (GetNumBaseFiles() == 0) {return NULL;}

  // get next pbf, if at end then start at beginning.
  TOgleBaseFile *pbf = GetNext();
  if (pbf == NULL) {pbf = GetFirst();} // if at end, then start at beginning.
  if (pbf == NULL) {return NULL;} // if shit then bail.

  int16 counter = GetNumBaseFiles();
  bool Out = false;
  while (!Out)
  {
    if (pbf->GetRamStatus() == ramstatus) {return pbf;}
    pbf = GetNext();
    if (pbf == NULL) {pbf = GetFirst();}
    counter--;
    if (counter == 0) {Out = true;} // sanity check! don't want to loop forever.
  }
  return NULL;
}
//*******************************************************************
TOgleBaseFile *TFileList::GetWithHDStatus(TFMStatus hdstatus)
// loop thru list looking for a matching hdstatus.
{
  if (GetNumBaseFiles() == 0) {return NULL;}

  // get next pbf, if at end then start at beginning.
  TOgleBaseFile *pbf = GetNext();
  if (pbf == NULL) {pbf = GetFirst();} // if at end, then start at beginning.
  if (pbf == NULL) {return NULL;} // if shit then bail.

  int16 counter = GetNumBaseFiles();
  bool Out = false;
  while (!Out)
  {
    if (pbf->GetHDStatus() == hdstatus) {return pbf;}
    pbf = GetNext();
    if (pbf == NULL) {pbf = GetFirst();}
    counter--;
    if (counter == 0) {Out = true;} // only loop thru list once.
  }
  return NULL; // none like that here, bail.
}
//*******************************************************************
TOgleBaseFile *TFileList::GetPBF(const string &name)
{
  for (TOgleBaseFile *pbf = GetFirst();pbf != NULL;pbf = GetNext())
  {
    if ((pbf->BaseFileName) == name) {return pbf;}
  }
  return NULL;
}
//*******************************************************************
TOgleBaseFile *TFileList::GetWithOwnerCount(int16 count)
// loop thru list looking for a matching owner count.
{
  for (TOgleBaseFile *pbf = GetFirst();
       pbf != NULL;
       pbf = GetNext())
  {
    if (pbf->GetOwnerCount() == count) {return pbf;}
  }
  return NULL;
}
//*******************************************************************
bool TFileList::DisposePBF(TOgleBaseFile *pbf)
{
  if (pbf == NULL) {return true;}
  if (!LMatchRemove(pbf)) {return false;}
  delete pbf;
  return true;
}
//*******************************************************************
void TFileList::DisposeAllWithOwnerCount(int16 count)
{
  TOgleBaseFile *pbf;
  while ((pbf = GetWithOwnerCount(count)) != NULL)
  {
    DisposePBF(pbf);
  }
}
//*******************************************************************
//*******************************************************************
TFMFLData::TFMFLData(TOgleFileMgr *pfmgr):PFMgr(pfmgr)
{
  pfirstram = NULL;
  pfirsthd = NULL;
  CurrentRamList = 0;
  CurrentHDList = 0;
  for (int16 i=0;i<NUMFILELISTS;i++)
  {
    PShareLists[i] = NULL;
  }
  PNonShareList = NULL;
  CreateLists();
}
//*******************************************************************
TFMFLData::~TFMFLData()
{
  DestroyLists();
}
//*******************************************************************
bool TFMFLData::CreateLists()
{
  for (int16 i=0;i<NUMFILELISTS;i++)
  {
    PShareLists[i] = new TFileList(false);
  }
  PNonShareList = new TFileList(true);
  return true;
}
//*******************************************************************
bool TFMFLData::DestroyLists()
{
  for (int16 i=0;i<NUMFILELISTS;i++)
  {
    delete PShareLists[i];
    PShareLists[i] = NULL;
  }
  delete PNonShareList;
  PNonShareList = NULL;

  return true;
}
//*******************************************************************
int16 TFMFLData::Hash(const string &name)
{
  uint32 n = 0;
  for (const char *ptr = name.c_str();*ptr;++ptr)
  {
    n <<= 1;
    n += *ptr;
  }
  return (int16)(n % uint32(NUMFILELISTS));
}
//*******************************************************************
TOgleBaseFile *TFMFLData::GetFirstWRamStatus(TFMStatus ramstatus)
{
  TOgleBaseFile *pbf = NULL;
  int16 counter = NUMFILELISTS;
  while (counter >= 0)
  {
    if (CurrentRamList >= NUMFILELISTS) {CurrentRamList = 0;}
    pbf = PShareLists[CurrentRamList]->GetWithRamStatus(ramstatus);
    if (pbf != NULL)
    {
      pfirstram = pbf;
      return pbf;
    }
    CurrentRamList++;
    counter--;
  }
  pfirstram = NULL;
  return NULL;
}
//*******************************************************************
TOgleBaseFile *TFMFLData::GetNextWRamStatus(TFMStatus ramstatus)
{
  if (pfirstram == NULL) {return NULL;}
  TOgleBaseFile *pbf = NULL;
  int16 counter = NUMFILELISTS;
  while (counter >= 0)
  {
    if (CurrentRamList >= NUMFILELISTS) {CurrentRamList = 0;}
    pbf = PShareLists[CurrentRamList]->GetWithRamStatus(ramstatus);
    if ((pbf != NULL) && (pbf != pfirstram))
    {
      return pbf;
    }
    CurrentRamList++;
    counter--;
  }
  pfirstram = NULL;
  return NULL;
}
//*******************************************************************
TOgleBaseFile *TFMFLData::GetFirstWHDStatus(TFMStatus hdstatus)
{
  TOgleBaseFile *pbf = NULL;
  int16 counter = NUMFILELISTS;
  while (counter >= 0)
  {
    if (CurrentHDList >= NUMFILELISTS) {CurrentHDList = 0;}
    pbf = PShareLists[CurrentHDList]->GetWithHDStatus(hdstatus);
    if (pbf != NULL)
    {
      pfirsthd = pbf;
      return pbf;
    }
    CurrentHDList++;
    counter--;
  }
  pfirsthd = NULL;
  return NULL;
}
//*******************************************************************
TOgleBaseFile *TFMFLData::GetNextWHDStatus(TFMStatus hdstatus)
{
  if (pfirsthd == NULL) {return NULL;}
  TOgleBaseFile *pbf = NULL;
  int16 counter = NUMFILELISTS;
  while (counter >= 0)
  {
    if (CurrentHDList >= NUMFILELISTS) {CurrentHDList = 0;}
    pbf = PShareLists[CurrentHDList]->GetWithHDStatus(hdstatus);
    if ((pbf != NULL) && (pbf != pfirsthd))
    {
      return pbf;
    }
    CurrentHDList++;
    counter--;
  }
  pfirsthd = NULL;
  return NULL;
}
//*******************************************************************
bool TFMFLData::UnLoadRamBelow(TFMStatus ramstatus,dword sizeneeded,dword &sizesofar)
{
  TRACEX(FileManager,1,"UL Ram -> status: "<<ramstatus<<" Size:"<<sizeneeded<<" ssf: "<<sizesofar);
  WARNX(FileManager,(sizeneeded == ULONG_MAX),0,"Max Unload");

  TOgleSoundMgr *psoundmgr = PFMgr->GetSoundMgr();
  TFMStatus  CurRamStatus = TFileMode::fm_None; // start at lowest priority.

  // starting at fm_None progress up to ramstatus level looking for ram to dealloc.
  while (CurRamStatus < ramstatus)  // CurRamStatus gets inc'd at end of while loop.
  {
    if (CurRamStatus == TFileMode::fm_Immed) {return false;}  // making sure we don't unload fm_Immed's.

    if (UnLoadRamWithStatus(PNonShareList,CurRamStatus,sizeneeded,sizesofar)) {return true;}

    int16 counter = NUMFILELISTS;
    while(counter >= 0)
    {
      if (CurrentRamList >= NUMFILELISTS) {CurrentRamList = 0;}
      if (UnLoadRamWithStatus(PShareLists[CurrentRamList],CurRamStatus,sizeneeded,sizesofar))
      {
        return true;
      }
      CurrentRamList++;
      counter--;
    }// end while counter.

    // ask sound mgr to unload any sound's not currently playing.
    if (psoundmgr != NULL && psoundmgr->UnLoadBelow(CurRamStatus,sizeneeded,sizesofar))
    {
      return true;
    }
    CurRamStatus++;
  } // end while CurRamStatus.
  return false;  //couldn't find enough ram to dealloc.
}
//*******************************************************************
bool TFMFLData::UnLoadHDBelow(TFMStatus hdstatus,dword sizeneeded,dword &sizesofar)
{
  TRACEX(FileManager,1,PFMgr->GetRamStr("UnLoadHDBelow In Action").c_str());

  TFMStatus  CurHDStatus = TFileMode::fm_None; // start at lowest priority.

  // starting at fm_None progress up to hdstatus level looking for hd files to delete.
  while (CurHDStatus < hdstatus)  // CurHDStatus gets inc'd at end of while loop.
  {
    if (UnLoadHDWithStatus(PNonShareList,CurHDStatus,sizeneeded,sizesofar)) {return true;}
    int16 counter = NUMFILELISTS;
    while(counter >= 0)
    {
      if (CurrentHDList >= NUMFILELISTS) {CurrentHDList = 0;}
      if (UnLoadHDWithStatus(PShareLists[CurrentHDList],CurHDStatus,sizeneeded,sizesofar))
      {
        return true;
      }
      CurrentHDList++;
      counter--;
    }// end for
    CurHDStatus++;
  } // end while
  return false;  //couldn't find enough ram to dealloc.
}
//*******************************************************************
void TFMFLData::DestroyNoOwners()
{
  TRACEX(FileManager,0,"DestroyNoOwners");
  PNonShareList->DisposeNoOwners();
  for (int16 i = 0;i<NUMFILELISTS;i++)
  {
    PShareLists[i]->DisposeNoOwners();
  }
}
//*******************************************************************
bool TFMFLData::UnLoadRamWithStatus(TFileList *plist,TFMStatus ramstatus,dword sizeneeded,dword &sizesofar)
{
  if (plist == NULL) {return false;}
  if (plist->GetNumBaseFiles() == 0) {return false;}

  // get next pbf, if at end then start at beginning.
  TOgleBaseFile *pbf = plist->GetNext();
  if (pbf == NULL) {pbf = plist->GetFirst();} // if at end, then start at beginning.
  if (pbf == NULL) {return false;} // if shit then bail.

  int16 counter = plist->GetNumBaseFiles();
  bool Out = false;
  while (!Out)
  {
    if (pbf->GetRamStatus() == ramstatus)
    {
      if (pbf->IsRamAvail())
      {
        dword size = pbf->GetSize();
        PFMgr->ProcessBG(pbf,-1,0);  // ram -> unload, hd -> don't care.
        if (!pbf->IsRamAvail()) {sizesofar += size;}
        if (sizesofar >= sizeneeded) {return true;}
      }
    }
    pbf = plist->GetNext();
    if (pbf == NULL) {pbf = plist->GetFirst();}
    counter--;
    if (counter == 0) {Out = true;} // sanity check! don't want to loop forever.
  }
  return false;
}
//*******************************************************************
bool TFMFLData::UnLoadHDWithStatus(TFileList *plist,TFMStatus hdstatus,dword sizeneeded,dword &sizesofar)
{
  if (plist == NULL) {return false;}
  if (plist->GetNumBaseFiles() == 0) {return false;}

  // get next pbf, if at end then start at beginning.
  TOgleBaseFile *pbf = plist->GetNext();
  if (pbf == NULL) {pbf = plist->GetFirst();} // if at end, then start at beginning.
  if (pbf == NULL) {return false;} // if shit then bail.

  int16 counter = plist->GetNumBaseFiles();
  bool Out = false;
  while (!Out)
  {
    if (pbf->GetHDStatus() == hdstatus)
    {
      if (pbf->IsHDAvail())
      {
        dword size = pbf->GetSize();
        PFMgr->ProcessBG(pbf,0,-1);
        if (!pbf->IsHDAvail()) {sizesofar += size;}
        if (sizesofar >= sizeneeded) {return true;}
      }
    }
    pbf = plist->GetNext();
    if (pbf == NULL) {pbf = plist->GetFirst();}
    counter--;
    if (counter == 0) {Out = true;} // sanity check! don't want to loop forever.
  }
  return false;
}
//*******************************************************************
//************ end data classes *************************************
//*******************************************************************
string TOgleFileMgr::GetCDRomPath(int16 dirindex)
{
  string s = PCD->GetFullPath();
  s += PDat->GetDirName(dirindex);
  return s;
}
//*******************************************************************
inline bool TOgleFileMgr::IsBGLoadOk()
{
  if (GetSoundMgr() == NULL) {return true;}
  return !GetSoundMgr()->IsWavPlaying();
}
//*******************************************************************
void TOgleFileMgr::IdleLoad()
{
  NumIdleHits++;
  if ((PDat->GetLastLoadFUC() < (PDat->GetCurrentFUC() + 30)) && IsBGLoadOk())
  {
    BufferFiles(1);
  }
}
//*******************************************************************
void TOgleFileMgr::CreateDetList()
{
  PDetList = new TDetList;
}
//*******************************************************************
void TOgleFileMgr::DestroyDetList()
{
  delete PDetList;
  PDetList = NULL;
}
//*******************************************************************
TDet *TOgleFileMgr::CreateDet(string &detfname)
{
  TDet *pdet = new TDet(TFileMode(detfname,TFileMode::fm_None,TFileMode::fm_Immed));
  return pdet;
}
//*******************************************************************
bool TOgleFileMgr::RegisterDet(string &detfname)
{
  if (PDetList == NULL) {return false;}
  if (detfname == "") {return false;}

  if (PDetList->IsDetAdded(detfname))
  {
    return PDetList->IncDetUsage(detfname);
  }
  else
  {
    TDet *pdet = CreateDet(detfname);
    if (!PDetList->AddDet(pdet))
    {
      delete pdet;
      return false;
    }
    return PDetList->IncDetUsage(detfname);
  }
}
//*******************************************************************
bool TOgleFileMgr::UnRegisterDet(string &detfname)
{
  if (PDetList == NULL) {return false;}
  return PDetList->DecDetUsage(detfname);
}
//*******************************************************************
void TOgleFileMgr::ReleaseUnUsedDets()
{
  if (PDetList != NULL) {PDetList->ReleaseUnUsed();}
}
//*******************************************************************
string TOgleFileMgr::GetResString(uint id)
{
  return (POgleApp->LoadString(id));
}
//*******************************************************************
TOgleIniFile *TOgleFileMgr::GetIniFile()
{
  return POgleApp->GetIniFile();
}
//*******************************************************************
bool TOgleFileMgr::PromptCDInsert(string filename)
// need cd version & dev version.
// also need to pauseupdates() ??
{
  string str = "Couldn't find CDRom File: ";
  str += filename;
  str += "\n";
  str += " - Please Insert Disk";
  ::MessageBox(NULL,str.c_str(),"CD Rom Error",MB_OK | MB_SYSTEMMODAL | MB_ICONHAND);
  return true;
}
//*******************************************************************
void TOgleFileMgr::SetGlobalHDBuffering(bool on)
{
  GlobalHDBuffering = on;
  if (on)
  {
    GetIniFile()->SetHDBuffering(1);
  }
  else
  {
    GetIniFile()->SetHDBuffering(0);
  }
}
//*******************************************************************
void TOgleFileMgr::ToggleHDBuffering()
{
  bool on = IsGlobalHDBufferingOn();
  SetGlobalHDBuffering(!on);
}
//*******************************************************************
bool TOgleFileMgr::Initialize()
{
  int16 hdbuffering = int16(GetIniFile()->GetHDBuffering());
  GlobalHDBuffering = hdbuffering?true:false;

  PDat = new TFMData;
  PHDB = new TFMHDBufferNames(GetIniFile()->GetHDDrive(),GetIniFile()->GetHDDir());
  PHDD = new TFMHDDataNames(GetIniFile()->GetHDDrive(),GetIniFile()->GetHDDir());
  PCD = new TFMCDDataNames(GetIniFile()->GetCDDrive(),GetIniFile()->GetCDDir());
  PDiskDat = new TFMDiskData(int16(TOgleFileName::ConvertDriveLetterToInt(PHDB->GetDrive())));
  PFLDat = new TFMFLData(this);
  Initialized = true;
  return true;
}
//*******************************************************************
void TOgleFileMgr::DeInitialize()
{
  if (!Initialized) {return;}

  delete PFLDat;
  PFLDat = NULL;
  delete PHDB;
  PHDB = NULL;
  delete PHDD;
  PHDD = NULL;
  delete PCD;
  PCD = NULL;
  delete PDiskDat;
  PDiskDat = NULL;
  delete PDat;
  PDat = NULL;

  Initialized = false;
}
//*******************************************************************
TOgleFileMgr::TOgleFileMgr(TOgleApp *papp)
{
  POgleApp = papp;
  GlobalHDBuffering = true;
  Initialized = false;
  PDat = NULL;
  PHDD = NULL;
  PFLDat = NULL;
  PCD = NULL;
  PHDB = NULL;
  PDiskDat = NULL;
  NumIdleHits = 0;
  InitializedOk = Initialize();

  CreateDetList();
}
//*******************************************************************
TOgleFileMgr::~TOgleFileMgr()
{
  DestroyDetList();

  GetIniFile()->SetCDDrive(PCD->GetDrive());
  GetIniFile()->SetCDDir(PCD->GetDir());
  DeInitialize();
}
//*******************************************************************
bool TOgleFileMgr::LoadFileNames(TOgleBaseFile *PBaseFile,TFileMode &NewFileMode)
// sets up cd and hd filenames, cd file size, and file offset (for dews).
{
//  string cdfullpathname;
  TOgleFileName cdfullpathname;

  if (PBaseFile == NULL) {return false;}

//  PBaseFile->SetBaseName(NewFileMode.Name);
  PBaseFile->SetBaseName(NewFileMode.Name.GetStr());
//  string str = PDat->GetDirName(NewFileMode.GetDirIndex());
//  str += NewFileMode.Name;
  string str = NewFileMode.Name.GetStr();
  TDirEntry *pdirentry = PDetList->GetDirEntry(str);
  if (pdirentry != NULL)       
  {
    // first check hd drive data dir for dew file.
    cdfullpathname = PHDD->GetFullPath();
//    cdfullpathname += pdirentry->GetDewName();
    cdfullpathname.Merge(pdirentry->GetDewName());
//    if (TOgleFileName::FileExists(cdfullpathname))
    if (cdfullpathname.FileExists())
    {
      PBaseFile->SetHDBuffering(false);
//      PBaseFile->SetCDRomName(cdfullpathname);
      PBaseFile->SetCDRomName(cdfullpathname.GetStr());
      PBaseFile->SetSize(pdirentry->GetSize());
      PBaseFile->SetOffset(pdirentry->GetOffset());
    }
    else
    {
      // next check cd drive data dir for dew file.
      cdfullpathname = PCD->GetFullPath();
//      cdfullpathname += pdirentry->GetDewName();
      cdfullpathname.Merge(pdirentry->GetDewName());
//      if (TOgleFileName::FileExists(cdfullpathname))
      if (cdfullpathname.FileExists())
      {
//        PBaseFile->SetCDRomName(cdfullpathname);
        PBaseFile->SetCDRomName(cdfullpathname.GetStr());
        PBaseFile->SetSize(pdirentry->GetSize());
        PBaseFile->SetOffset(pdirentry->GetOffset());
      }
      else
      {
        // next check cd drive data dir for regular file.
        cdfullpathname = PCD->GetFullPath();
//        cdfullpathname += PDat->GetDirName(NewFileMode.GetDirIndex());
//        cdfullpathname += NewFileMode.Name.GetStr();
        cdfullpathname.Merge(NewFileMode.Name);
//        PBaseFile->SetCDRomName(cdfullpathname);
        PBaseFile->SetCDRomName(cdfullpathname.GetStr());
//        PBaseFile->SetSize(TOgleFileName::GetFileSize(cdfullpathname));
        PBaseFile->SetSize(cdfullpathname.GetFileSize());
        PBaseFile->SetOffset(0);
      }
    }
  }
  else  // no pdirentry.
  {
    // check hd drive data dir for regular file.
    cdfullpathname = PHDD->GetFullPath();
//    cdfullpathname += PDat->GetDirName(NewFileMode.GetDirIndex());
//    cdfullpathname += NewFileMode.Name.GetStr();
    cdfullpathname.Merge(NewFileMode.Name);
//    if (TOgleFileName::FileExists(cdfullpathname))
    if (cdfullpathname.FileExists())
    {
//      PBaseFile->SetCDRomName(cdfullpathname);
      PBaseFile->SetCDRomName(cdfullpathname.GetStr());
//      PBaseFile->SetSize(TOgleFileName::GetFileSize(cdfullpathname));
      PBaseFile->SetSize(cdfullpathname.GetFileSize());
      PBaseFile->SetOffset(0);
    }
    else
    {
      // check cd drive data dir for regular file.
      cdfullpathname = PCD->GetFullPath();
//      cdfullpathname += PDat->GetDirName(NewFileMode.GetDirIndex());
//      cdfullpathname += NewFileMode.Name.GetStr();
      cdfullpathname.Merge(NewFileMode.Name);
//      PBaseFile->SetCDRomName(cdfullpathname);
      PBaseFile->SetCDRomName(cdfullpathname.GetStr());
//      PBaseFile->SetSize(TOgleFileName::GetFileSize(cdfullpathname));
      PBaseFile->SetSize(cdfullpathname.GetFileSize());
      PBaseFile->SetOffset(0);
    }
  }
  string hdbufferfullpathname = PHDB->GetFullPath();
  hdbufferfullpathname += PHDB->GetUniqueFileName();
  PBaseFile->SetHDName(hdbufferfullpathname);
  PBaseFile->ClearHDAvail();

  return true;
}
//*******************************************************************
void TOgleFileMgr::FatalError(string msg)
{
  TRACEX(FileManager,0,"Fatal Error");
  if (msg == "") {::MessageBox(NULL,"Unknown fatal error","Error - File Mgr",MB_OK);}
  else           {::MessageBox(NULL,msg.c_str(),"Error - File Mgr",MB_OK);}
  abort();
}
//*******************************************************************
string TOgleFileMgr::GetRamStr(string msg)
{
  char buffer[100];
  memset(buffer,0,100);
  dword freespace = GetFreeSpace(0);
  dword maxblock = GlobalCompact(0);
  wsprintf(buffer," Fsp: %lu, gc: %lu",freespace,maxblock);
  string s = msg;
  s += buffer;
  return s;
}
//*******************************************************************
bool TOgleFileMgr::Process(TOgleBaseFile *PBaseFile,int16 loadram,int16 loadhd)
// note: this function makes sure order of loads and unloads are efficient.
{
  if (PBaseFile == NULL) {return false;}

  TCursorMgr &cursormgr = *GetCursorMgr();

  bool rc = true;

  if (loadram > 0)
  {
    if (loadhd > 0)
    {
      cursormgr.SetCursor(ctFileMgr,0,IDC_WAIT,true);
      PDat->SetLastLoadFUC(PDat->GetCurrentFUC());
      rc = LoadRamHD(PBaseFile,TFileMode::fm_Immed,TFileMode::fm_Immed);
    }
    else if (loadhd == 0)
    {
      cursormgr.SetCursor(ctFileMgr,0,IDC_WAIT,true);
      PDat->SetLastLoadFUC(PDat->GetCurrentFUC());
      rc = LoadRam(PBaseFile,TFileMode::fm_Immed);
    }
    else
    {
      cursormgr.SetCursor(ctFileMgr,0,IDC_WAIT,true);
      PDat->SetLastLoadFUC(PDat->GetCurrentFUC());
      bool r = LoadRam(PBaseFile,TFileMode::fm_Immed);
      bool h = UnLoadHD(PBaseFile);
      rc = (r && h);
    }
  }
  else if (loadram == 0)
  {
    if (loadhd > 0)
    {
      cursormgr.SetCursor(ctFileMgr,0,IDC_WAIT,true);
      PDat->SetLastLoadFUC(PDat->GetCurrentFUC());

      // added this part to load ram if not loaded, before trying to load hd.
      if (PBaseFile->IsRamAvail())
      {
        rc = LoadHD(PBaseFile,TFileMode::fm_Immed);
      }
      else
      {
        LoadRam(PBaseFile,TFileMode::fm_Immed);
        rc = LoadHD(PBaseFile,TFileMode::fm_Immed);
        if (PBaseFile->GetHDStatus() < TFileMode::fm_Immed) // just in case.
        {
          UnLoadRam(PBaseFile);
        }
      }
    }
    else if (loadhd == 0)
    {
      // do nothing.
    }
    else
    {
      cursormgr.SetCursor(ctFileMgr,0,IDC_WAIT,true);
      rc = UnLoadHD(PBaseFile);
    }
  }
  else
  {
    if (loadhd > 0)
    {
      cursormgr.SetCursor(ctFileMgr,0,IDC_WAIT,true);
      PDat->SetLastLoadFUC(PDat->GetCurrentFUC());
      bool h = LoadHD(PBaseFile,TFileMode::fm_Immed);
      bool r = UnLoadRam(PBaseFile);
      rc = (h && r);
    }
    else if (loadhd == 0)
    {
      cursormgr.SetCursor(ctFileMgr,0,IDC_WAIT,true);
      rc = UnLoadRam(PBaseFile);
    }
    else
    {
      cursormgr.SetCursor(ctFileMgr,0,IDC_WAIT,true);
      bool r = UnLoadRam(PBaseFile);
      bool h = UnLoadHD(PBaseFile);
      rc = (r && h);
    }
  }
  cursormgr.SetCursor(ctFileMgr,0,0,true);
  return rc;
}
//*******************************************************************
bool TOgleFileMgr::ProcessRegister(TOgleBaseFile *PBaseFile)
// we only want to handle fm_None and fm_Immed here.
// we look for any excuse not to call "process".
{
  if (PBaseFile == NULL) {return false;}

  #ifdef DEBUGUPDATES
  DebugFMUpdate.StartPR();
  #endif

  InsideCore = true;

  TFMStatus rs = PBaseFile->GetRamStatus();
  TFMStatus hs = PBaseFile->GetHDStatus();

  int16 loadram = 0;
  if ((rs == TFileMode::fm_Immed) && !PBaseFile->IsRamAvail())
  {
    loadram = 1;
  }
  else if ((rs == TFileMode::fm_None) && PBaseFile->IsRamAvail())
  {
    loadram = -1;
  }

  int16 loadhd = 0;
  if ((hs == TFileMode::fm_Immed) &&
      !PBaseFile->IsHDAvail() &&
      PBaseFile->GetHDBuffering())
  {
    loadhd = 1; // if immed and buffering ok then load to buffer.
  }
  else if ((hs != TFileMode::fm_None) && PBaseFile->IsRamAvail() &&
           (loadram == -1) && !PBaseFile->IsHDAvail() &&
           IsBGLoadOk() && PBaseFile->GetHDBuffering() &&
           IsGlobalHDBufferingOn())
  {
    loadhd = 1; // if !none and unloading ram and buffering ok then buffer hd before unloading ram.
  }
  else if ((hs == TFileMode::fm_None) && PBaseFile->IsHDAvail())
  {
    loadhd = -1; // if none and it is loaded to the hd buffer then unload from hd buffer.
  }

  bool rc = true;

  if ((loadhd != 0) || (loadram != 0))
  {
    rc = Process(PBaseFile,loadram,loadhd);
  }

  WARNX(FileManager,(loadram == 1 && !PBaseFile->IsRamAvail()),0,"Ram Immed Failure");
  WARNX(FileManager,(loadhd == 1 && !PBaseFile->IsHDAvail()),0,"HD Immed Failure");

  if (rs == TFileMode::fm_Immed && !PBaseFile->IsRamAvail()) {FatalError("Ram Immed Failure");}

  CheckRam();

  InsideCore = false;

  #ifdef DEBUGUPDATES
  DebugFMUpdate.EndPR();
  #endif

  return rc;
}
//*******************************************************************
bool TOgleFileMgr::ProcessBG(TOgleBaseFile *PBaseFile,int16 loadramreq,int16 loadhdreq)
{
  if (PBaseFile == NULL) {return false;}

  TFMStatus rs = PBaseFile->GetRamStatus();
  TFMStatus hs = PBaseFile->GetHDStatus();
  bool ramavail = PBaseFile->IsRamAvail();
  bool hdavail = PBaseFile->IsHDAvail();
  bool bgloadok = IsBGLoadOk();
  bool hdbufok = PBaseFile->GetHDBuffering() && IsGlobalHDBufferingOn();

  int16 loadram = 0;
  if      ((rs == TFileMode::fm_Immed) && !ramavail) {loadram = 1;}
  else if ((loadramreq == 1) && (rs != TFileMode::fm_None) && bgloadok && !ramavail) {loadram = 1;}
  else if ((loadramreq == -1) && (rs != TFileMode::fm_Immed)) {loadram = -1;}

  int16 loadhd = 0;

  bool willfit = PDiskDat->WillFit(PBaseFile->GetSize());

  if ((loadhdreq == 1) && !hdavail && (hs != TFileMode::fm_None) && bgloadok && hdbufok && willfit)
  {
    loadhd = 1;
  }
  else if ((loadhdreq == 0) && (loadram == -1) && bgloadok && hdbufok && willfit)
  {
    loadhd = 1;
  }
  else if ((loadhdreq == -1) && hs != TFileMode::fm_Immed)
  {
    loadhd = -1;
  }

  bool rc = true;

  if ((loadhd != 0) || (loadram != 0))
  {
    rc = Process(PBaseFile,loadram,loadhd);
  }

  return rc;
}
//*******************************************************************
#pragma argsused
void TOgleFileMgr::NotifyCompacting(uint compactRatio,bool now)
// this is a posted msg from kernel.
{
  TRACEX(FileManager,0,GetRamStr("Notify compacting ").c_str());

  if (now)
  {
    PFLDat->UnLoadRamBelow(TFileMode::fm_Immed,ULONG_MAX);
    TOgleMemMgr::SetCompactPending(false);
  }
  else
  {
    TOgleMemMgr::SetCompactPending(true);
  }

//  dword Per = ((dword)compactRatio * 100) / (dword)UINT_MAX;
}
//*******************************************************************
bool TOgleFileMgr::HandleCDReadFail(TOgleBaseFile *PBaseFile)
{
  for (int16 i=0;i<3;i++)
  {
    TLoadResult result = PBaseFile->LoadRam();
    if (result == lrOk) {return true;}
    else if (result == lrMemFail) {return false;}
    else if (result == lrCDReadFail) {PromptCDInsert(PBaseFile->CDRomFullPathName);}
    else {FatalError();}
  }
  string str = "Couldn't find file: ";
  str += PBaseFile->CDRomFullPathName;
  FatalError(str);
  return lrCDReadFail;  // shouldn't hit this, FatalError() should abort !!
}
//*******************************************************************
bool TOgleFileMgr::LoadRam(TOgleBaseFile *pbf,TFMStatus rampri)
{
  if (pbf == NULL) {return false;}
  TFMStatus ramstatus = pbf->GetRamStatus();
  if (ramstatus < rampri) {ramstatus = rampri;} // use higher of 2 values.

  dword size = pbf->GetSize();

  #ifdef DEBUGFILEMGR
  if (size == 0)
  {
    string s = "File name: ";
    s += pbf->GetCDRomName();
    s += "reports 0 file size";
    ::MessageBox(NULL,s.c_str(),"Warning ! Load Ram",MB_OK);
  }
  #endif

  // if immed and no unloading needed, then go ahead and try.
  if ((ramstatus == TFileMode::fm_Immed) && TOgleMemMgr::WillFit(size))
  {
    if (HandleCDReadFail(pbf)) {return true;}
  }

  // try unloading at each status less than ours.
  dword sizesofar = 0;
  for (TFMStatus stat = TFileMode::fm_BG_Lo;stat < ramstatus;stat++)
  {
    // try to unload exactly enough.
    if (PFLDat->UnLoadRamBelow(stat,size,sizesofar))
    {
      if (HandleCDReadFail(pbf)) {return true;}
    }
  }

  // hit the swap file if we have to but load the damn thing.
  if (ramstatus == TFileMode::fm_Immed) {return HandleCDReadFail(pbf);}

  return false;
}
//*******************************************************************
bool TOgleFileMgr::HandleHDWriteFail(TOgleBaseFile *PBaseFile,TFMStatus hdpri)
{
  if (PBaseFile == NULL) {return false;}

  TLoadResult result;
  dword size = PBaseFile->GetSize();
  dword sizesofar = 0;
  for (TFMStatus stat = TFileMode::fm_BG_Lo;stat < hdpri;stat++)
  {
    PFLDat->UnLoadHDBelow(stat,size,sizesofar);
    if ((result = PBaseFile->LoadHD()) == lrOk) {return true;}
    if (result != lrHDWriteFail) {return false;}

  }
  return false;
}
//*******************************************************************
bool TOgleFileMgr::LoadHD(TOgleBaseFile *PBaseFile,TFMStatus hdpri)
// ram must already be loaded.
{
  if (PBaseFile == NULL) {return false;}
  if (PBaseFile->IsHDAvail()) {return true;}
  if (!PBaseFile->IsRamAvail()) {return false;}

  TFMStatus hdstatus = PBaseFile->GetHDStatus();
  if (hdpri > hdstatus) {hdstatus = hdpri;}

  // check if load needed.
  if (hdstatus == TFileMode::fm_None) {return false;}

  // if not ok to do non-immed loading then bail.
  if (!IsBGLoadOk() && (hdstatus != TFileMode::fm_Immed)) {return false;}
  if (!PDiskDat->WillFit(PBaseFile->GetSize())) {return false;}

  TLoadResult result = PBaseFile->LoadHD();
  if (result == lrOk) {return true;}
  if (result == lrHDWriteFail) {return HandleHDWriteFail(PBaseFile,hdstatus);}
  return false;
}
//*******************************************************************
bool TOgleFileMgr::LoadRamHD(TOgleBaseFile *PBaseFile,TFMStatus rampri,TFMStatus hdpri)
{
  if (rampri < hdpri)
  {
    if (!LoadRam(PBaseFile,hdpri)) {return false;}
    return LoadHD(PBaseFile,hdpri);
  }
  else
  {
    if (!LoadRam(PBaseFile,rampri)) {return false;}
    return LoadHD(PBaseFile,hdpri);
  }
}
//*******************************************************************
#pragma argsused
void TOgleFileMgr::PostUpdate(dword NewFUC)
{
  if (((NewFUC % 30) == 0) || TOgleMemMgr::IsCompactPending())
  {
    CheckRam();
    TOgleMemMgr::SetCompactPending(false);
    TRACEX(FileManager,1,"postupdate: tra = " << TOgleMemMgr::GetBytesAllocated());
  }

  PDiskDat->PostUpdate(NewFUC);
  if (!PDiskDat->IsHDSpaceOk()) {CheckHD();}
}
//*******************************************************************
void TOgleFileMgr::ModeActivated()
{
  TRACEX(FileManager,0,"Mode Created");
  PFLDat->DestroyNoOwners();
  PFLDat->UnLoadRamBelow(TFileMode::fm_BG_Med,ULONG_MAX);
  ReleaseUnUsedDets();
}
//*******************************************************************
void TOgleFileMgr::ModeDeActivated()
{
  TRACEX(FileManager,0,"Mode Destroyed");
  PFLDat->DestroyNoOwners();
  PFLDat->UnLoadRamBelow(TFileMode::fm_Immed,ULONG_MAX);
}
//*******************************************************************
void TOgleFileMgr::Info()
{
  #if defined(BI_PLAT_WIN16)
  char buffer[100];
  memset(buffer,0,100);
  GLOBALINFO globalinfo;
  globalinfo.dwSize = sizeof(GLOBALINFO);
  GlobalInfo(&globalinfo);
  wsprintf(buffer,"Total Items: %u ,Items Free: %u ,Items LRU: %u \n",
           globalinfo.wcItems,globalinfo.wcItemsFree,globalinfo.wcItemsLRU);
  string str = buffer;
  MessageBox(NULL,str.c_str(),"Global Info",MB_OK|MB_APPLMODAL);

  memset(buffer,0,100);
  MEMMANINFO mmi;
  mmi.dwSize = sizeof(MEMMANINFO);
  MemManInfo(&mmi);
  wsprintf(buffer,"lfb: %lu, mpa: %lu, mpl: %lu,\n tls: %lu, tulp: %lu, fp: %lu,\n tp: %lu fls: %lu, sfp: %lu, ps: %u\n",
           mmi.dwLargestFreeBlock,mmi.dwMaxPagesAvailable,mmi.dwMaxPagesLockable,mmi.dwTotalLinearSpace,
           mmi.dwTotalUnlockedPages,mmi.dwFreePages,mmi.dwTotalPages,mmi.dwFreeLinearSpace,mmi.dwSwapFilePages,mmi.wPageSize);
  str = buffer;
  MessageBox(NULL,str.c_str(),"Mem Man Info",MB_OK|MB_APPLMODAL);
  memset(buffer,0,100);
  wsprintf(buffer,"Free Space: %lu \n",GetFreeSpace(0));
  str = buffer;
  memset(buffer,0,100);
  wsprintf(buffer,"Global compact space: %lu",GlobalCompact(0));
  str += buffer;
  MessageBox(NULL,str.c_str(),"f&g Info",MB_OK|MB_APPLMODAL);
  #endif
}
//*******************************************************************

