//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  otable.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     6/20/95    M.Perry    original baseline
//  2.0     8/20/97    M.Perry    32-bit conversion.
//********************************************************************
#include "otable.h"           

//*******************************************************************
TOgleRow *TOgleTable::CreateNewRow()
{
  return (new TOgleRow(PModeWin));
}
//*******************************************************************
TOgleRow *TOgleTable::CreateNewRow(void *PSource)
{
  return (new TOgleRow(PModeWin,PSource));
}
//*******************************************************************
const char *TOgleTable::GetIDChars() const
{
  return "TBL ";
}
//*******************************************************************
void TOgleTable::SetIndexes()
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TOgleRow *prow = GetRowPtr(i);
    if (prow != NULL) {prow->SetIndexes();}
  }
}
//*******************************************************************
void TOgleTable::CreateLists()
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TOgleRow *prow = GetRowPtr(i);
    if (prow != NULL) {prow->CreateLists(i);}
  }
}
//*******************************************************************
TOgleTable::TOgleTable(TOgleModeWindow *pmw,void *PSource,bool AutoRead)
:TOgleList(pmw)
// NumRowsInFile(0)
{
  Init(PSource,AutoRead);
}
//*******************************************************************
TOgleTable::TOgleTable(TOgleModeWindow *pmw)
:TOgleList(pmw)
// NumRowsInFile(0)
{
}
//*******************************************************************
TOgleTable::TOgleTable(TOgleModeWindow *pmw,bool dummy,const TFileMode &fm)
:TOgleList(pmw,dummy,fm)
// NumRowsInFile(0)
{
}
//*******************************************************************
TOgleTable::~TOgleTable()
{
}
//*******************************************************************
bool TOgleTable::ReadLocal()
{
  int16 NumRowsInFile;
  if (!(Readint16(&NumRowsInFile))) {return false;}

  for (int16 i=0;i<NumRowsInFile;i++)
  {
    TOgleRow *PRow = CreateNewRow(GetPtr());
    FileOffset += PRow->ReadSize;
    AddElement(PRow);
  }
  return true;
}
//*******************************************************************
bool TOgleTable::WriteLocal()
{
  int16 NumRowsInFile = int16(LastAddedElementNum + 1);
  if (!Writeint16(&NumRowsInFile)) {return false;}

  for (int16 i=0;i<NumRowsInFile;i++)
  {
    TOgleRow *prow = GetRowPtr(i);
    if (!prow)
    {
      #ifdef DEBUGTABLE
      string s = "Row # ";
      char buffer[17];
      s += itoa(i,buffer,10);
      s += " was NULL";
      ::MessageBox(NULL,s.c_str(),"Error - Table::WriteLocal",MB_OK);
      #endif
      return false;
    }
    if (!prow->Write(PWriteBuf)) {return false;}
  }
  return true;
}
//*******************************************************************
void TOgleTable::AddRow()
{
  AddElement(CreateNewRow());
}
//*******************************************************************
void TOgleTable::RemoveTopRow()
{
  RemoveElement(LastAddedElementNum);
}
//*******************************************************************
void TOgleTable::CreateRows(int16 NewNumRows)
{
  for (int16 i=0;i<NewNumRows;i++)
  {
    AddRow();
  }
}
//*******************************************************************
void TOgleTable::Activate(dword StartFUC)
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TOgleRow *prow = GetRowPtr(i);
    if (prow != NULL) {prow->Activate(StartFUC + i);}
  }
}
//*******************************************************************

