//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  obdlg.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     12/19/96   M.Perry    original baseline
//  1.2     03/27/97   M.Perry    made into base class for dialog hierarchy.
//  2.0     08/26/97   M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__obdlg_h)
#include "obdlg.h"
#endif

#if !defined(OWL_GDIOBJEC_H)
#include <owl\gdiobjec.h>
#endif
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TBitmapDlg,TDialog)
  EV_WM_PAINT,
  EV_WM_ERASEBKGND,
  EV_WM_LBUTTONUP,
  EV_WM_RBUTTONUP,
END_RESPONSE_TABLE;
//*******************************************************************
TBitmapDlg::TBitmapDlg(void *powner,TWindow *pwin,TBitmap *pbitmap,
                       word options,
                       int dlgid,
                       const char *title)
:TDialog(pwin,dlgid),
 PBitmap(pbitmap),
 Options(options),
 POwner(powner),
 POkBtn(NULL)
{
  if (dlgid == IDD_BITMAP)
  {
    POkBtn = new TButton(this,IDOK);
  }
  if (title != NULL) {SetCaption(title);}
}
//*******************************************************************
TBitmapDlg::~TBitmapDlg()
{
  if (Options.IsSet(bdOWNBITMAP) && PBitmap != NULL)
  {
    delete PBitmap;
    PBitmap = NULL;
  }
}
//*******************************************************************
void TBitmapDlg::EvPaint()
// do our thing if we have a bg bitmap.
{
  if (PBitmap != NULL)
  {
    TPaintDC dc(*this);
    TRect &rect = *(TRect *)&dc.Ps.rcPaint;
    Paint(dc,dc.Ps.fErase,rect);
  }
  else
  {
    DefaultProcessing();
  }
}
//*******************************************************************
void TBitmapDlg::Paint(TDC &dc,bool /*erase*/,TRect &/*rect*/)
{
  if (PBitmap == NULL) {return;}

  TMemoryDC memdc(dc);
  memdc.SelectObject(*PBitmap);
  TRect wrect;
  GetClientRect(wrect);
  dc.StretchBlt(0,0,wrect.Width()+1,wrect.Height()+1,
               memdc,
               0,0,PBitmap->Width(),PBitmap->Height(),
               SRCCOPY);
  memdc.RestoreBitmap();
}
//*******************************************************************
bool TBitmapDlg::EvEraseBkgnd(HDC hDC)
// do nothing if we have a bitmap.
{
  if (PBitmap != NULL)
  {
    return true;
  }
  else
  {
    return TDialog::EvEraseBkgnd(hDC);
  }
}
//*******************************************************************
void TBitmapDlg::SetupWindow()
{
  TDialog::SetupWindow();

  TRect rect = GetWindowRect();
  if (Options.IsSet(bdRESIZE) && PBitmap != NULL)
  {
    MoveWindow(rect.left,
               rect.top,
               PBitmap->Width(),
               PBitmap->Height());
    rect = GetWindowRect();
  }
  if (Parent == NULL)
  {
    int xres = ::GetSystemMetrics(SM_CXSCREEN);
    int yres = ::GetSystemMetrics(SM_CYSCREEN);
    MoveWindow((xres - rect.Width())/2,
               (yres - rect.Height())/2,
                rect.Width(),
                rect.Height());
  }
  else
  {
    TRect prect = Parent->GetWindowRect();
    int pcx = prect.left + (prect.Width()/2);
    int pcy = prect.top + (prect.Height()/2);
    MoveWindow(pcx - (rect.Width()/2),
               pcy - (rect.Height()/2),
               rect.Width(),
               rect.Height());
  }
  if (Options.IsClear(bdSHOWOK) && POkBtn != NULL)
  {
    POkBtn->ShowWindow(SW_HIDE);
  }
  if (Options.IsSet(bdIGNOREINPUT) && POkBtn != NULL)
  {
    POkBtn->EnableWindow(false);
  }
}
//*******************************************************************
#pragma argsused
void TBitmapDlg::EvRButtonUp(uint modKeys,TPoint &point)
{
  if (Options.IsClear(bdIGNOREINPUT))
  {
    CmOk();
  }
}
//*******************************************************************
#pragma argsused
void TBitmapDlg::EvLButtonUp(uint modKeys,TPoint &point)
{
  if (Options.IsClear(bdIGNOREINPUT))
  {
    CmOk();
  }
}
//*******************************************************************

