//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ogobjsp.cpp                     
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     2/5/96     M.Perry     original baseline
//  2.0     8/23/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__ogobjsp_h)
#include "ogobjsp.h"
#endif

#if !defined(__osprlist_h)
#include "osprlist.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif
#include <assert.h>
//*******************************************************************
#if defined(GED_COMPILE)
bool TGObjSp::CheckSpriteIDConflict()
{
  if (GetSpriteList() != NULL)
  {
    return GetSpriteList()->CheckSpriteIDConflicts();
  }
  return true;
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
bool TGObjSp::CheckSpriteIDUsage()
{
  if (GetSpriteList() != NULL)
  {
    return GetSpriteList()->CheckSpriteIDUsage();
  }
  return true;
}
#endif
//*******************************************************************
#ifdef DEBUGSPRITES
void TGObjSp::RTReport(ofstream &os)
{
//  os<<"Start of Run Time GObj Report for "<<GetFName()<<"\n";
  os<<"Start of Run Time GObj Report for "<<FileMode.Name.GetStr()<<"\n";
  if (GetSpriteList() != NULL) {GetSpriteList()->RTReport(os);}
  else {os<<"No Sprite List"<<"\n";}
}
#endif                                
//*******************************************************************
TGObjSp::TGObjSp(TOgleModeWindow *pmw)
:TGObjBase(pmw),
 PSpriteList(NULL),
 UniqueIDStringsInitialized(false)
{
}
//*******************************************************************
TGObjSp::TGObjSp(TOgleModeWindow *pmw,const TFileMode &fm)
:TGObjBase(pmw,fm),
 PSpriteList(NULL),
 UniqueIDStringsInitialized(false)
{
}
//*******************************************************************
#if defined(SUPPORT_OLD_VER)
void TGObjSp::SetUIDS()
{
  // need to override
}
//*******************************************************************
void TGObjSp::SetUIDSStrings()
{
  // need to override
}
//*******************************************************************
void TGObjSp::SetupUniqueIDStrings()
{
  // quick out if called more than once.
  if (UniqueIDStringsInitialized) {return;}
  UniqueIDStringsInitialized = true;

  SetUIDS(); // can be overridden up to denom level,usually gametype level
  SetUIDSStrings(); // override at gametype level
}
#endif
//*******************************************************************
TGObjSp::~TGObjSp()
{
  if (PSpriteList != NULL)
  {
    delete PSpriteList;
    PSpriteList = NULL;
  }
}
//*******************************************************************
void TGObjSp::Update(dword NewFUC)
{
  if (GetSpriteList() != NULL) {GetSpriteList()->Update(NewFUC);}
}
//*******************************************************************
void TGObjSp::PostUpdate(dword NewFUC)
{
  if (GetSpriteList() != NULL) {GetSpriteList()->PostUpdate(NewFUC);}
}
//*******************************************************************
void TGObjSp::Activate(dword StartFUC)
{
  TOgleBase::Activate(StartFUC);
  if (GetSpriteList() != NULL) {GetSpriteList()->Activate(StartFUC);}
}
//*******************************************************************
void TGObjSp::DeActivate()
{
  if (GetSpriteList() != NULL) {GetSpriteList()->DeActivate();}
  TOgleBase::DeActivate();
}
//*******************************************************************
void TGObjSp::SetIndexes()
{
  if (GetSpriteList() != NULL) {GetSpriteList()->SetIndexes();}
}
//*******************************************************************
void TGObjSp::Slenderize()
{
  if (GetSpriteList() != NULL) {GetSpriteList()->Slenderize();}
}
//*******************************************************************
void TGObjSp::SpriteError(string SpriteDescr,string SpriteFName)
{
  // when this is called the POwnerWindow is not a client of the frame window yet.
  string str = "Error setting up ";
  str += SpriteDescr;
  str += "\n Sprite Name: ";
  str += SpriteFName;
  str += "\n Exiting program";
  FatalError(str,string("Sprite Setup"));
//  ::MessageBox(NULL,str.c_str(),"Sprite Setup",MB_OK);
//  exit(EXIT_FAILURE);
}
//*******************************************************************
bool TGObjSp::CreateAddSpriteList(TFileMode splfm)
{
  PSpriteList = CreateSpriteList(splfm);
  return true;
}
//*******************************************************************
TSpriteList *TGObjSp::CreateSpriteList(TFileMode splfm)
{
  return (new TSpriteList(PModeWin,splfm,this));
}
//*******************************************************************
bool TGObjSp::AddSprite(TFileMode NewFileMode)
{
  TSprite *PTempSprite = new TSprite(PModeWin,NewFileMode);
  if (AddSprite(PTempSprite)) {return true;}
  delete PTempSprite;
  return false;
}
//*******************************************************************
bool TGObjSp::AddSprite(TSprite *PNewSprite)
{
  if (GetSpriteList() == NULL) {return false;}
  return GetSpriteList()->AddElement(PNewSprite);
}
//*******************************************************************
bool TGObjSp::SetCurrentSpriteSet(int16 typeindex,int16 setindex)
{
  if (GetSpriteList() == NULL) {return false;}
  return GetSpriteList()->SetCurrentSpriteSet(typeindex,setindex);
}
//*******************************************************************
int16 TGObjSp::GetSpriteIndex(string SpriteName)
{
  return (GetSpriteList()->GetSpriteIndex(SpriteName));
}
//*******************************************************************
int16 TGObjSp::GetSpriteIndex(TSprite *psprite)
{
  return (GetSpriteList()->GetSpriteIndex(psprite));
}
//*******************************************************************
TSprite *TGObjSp::GetSpritePtr(int16 SpriteIndex)
{
  return (GetSpriteList()->GetSpritePtr(SpriteIndex));
}
//*******************************************************************
TSprite *TGObjSp::GetSpritePtr(string SpriteFName)
{
  return (GetSpritePtr(GetSpriteIndex(SpriteFName)));
}
//*******************************************************************
int16 TGObjSp::GetSpriteIndex(int16 GObjIndex,string SpriteName)
{
  return PModeWin->GetSpriteIndex(GObjIndex,SpriteName);
}
//*******************************************************************
int16 TGObjSp::GetLastNumSprites()
{
  if (GetSpriteList() == NULL) {return -1;}
  return GetSpriteList()->GetLastNumSprites();
}
//*******************************************************************
int16 TGObjSp::CreateAddSprite(string SpriteFName)
{
  TSprite *PSprite = new TSprite(PModeWin);
//  PSprite->SetBaseName(SpriteFName);
  PSprite->FileMode.Name = SpriteFName;
  PSprite->FileMode.Name.SetExt(PSprite->AddExt());
  PSprite->FileMode.SetDirIndex(FileMode.GetDirIndex());

  if (!AddSprite(PSprite))
  {
    assert(false); // would be a problem !!!
  }
//  return (GetSpriteList()->GetSpriteIndex(SpriteFName));
  return GetSpriteList()->GetSpriteIndex(PSprite->FileMode.Name.GetStr());
}
//*******************************************************************
void TGObjSp::StartSpriteNow(int16 ListNum)
{
  ActivateSprite(ListNum,GetAdjFUC());
}
//*******************************************************************
dword TGObjSp::GetSpriteLength(int16 SpriteIndex)
{
  if (GetSpriteList() == NULL) {return 0;}
  else {return (GetSpriteList()->GetSpriteLength(SpriteIndex));}
}
//*******************************************************************
dword TGObjSp::GetSpriteLength(int16 GObjIndex,int16 SpriteIndex)
{
  if (PModeWin == NULL) {return 0;}
  return PModeWin->GetSpriteLength(GObjIndex,SpriteIndex);
}
//*******************************************************************
bool TGObjSp::CanActivateSprite(int16 GObjIndex,int16 ListNum,dword StartFUC)
{
  if (PModeWin == NULL) {return false;}
  return PModeWin->CanActivateSprite(GObjIndex,ListNum,StartFUC);
}
//*******************************************************************
bool TGObjSp::CanActivateSprite(int16 ListNum,dword StartFUC)
{
  if (GetSpriteList() == NULL) {return false;}
  else {return (GetSpriteList()->CanActivateSprite(ListNum,StartFUC));}
}
//*******************************************************************
bool TGObjSp::ActivateSprite(int16 GObjIndex,int16 ListNum,dword StartFUC)
{
  if (PModeWin == NULL) {return false;}
  return PModeWin->ActivateSprite(GObjIndex,ListNum,StartFUC);
}
//*******************************************************************
bool TGObjSp::ActivateSprite(int16 ListNum,dword StartFUC)
{
  if (GetSpriteList() == NULL) {return false;}
  else {return (GetSpriteList()->ActivateElement(ListNum,StartFUC));}
}
//*******************************************************************
bool TGObjSp::ActivateSprite(TSprite *psprite,dword StartFUC)
{
  return ActivateSprite(GetSpriteIndex(psprite),StartFUC);
}
//*******************************************************************
bool TGObjSp::ActivateSprite(int16 ListNum,dword &StartFUC,dword &FUCLength)
{
  if (ListNum < 0) {return false;}

  dword RequestedStartFUC = StartFUC;
  FUCLength = GetSpriteLength(ListNum);
  for (dword i = RequestedStartFUC;i < (RequestedStartFUC + 20);i++)
  {
    if (CanActivateSprite(ListNum,i))
    {
      StartFUC = i;
      if (ActivateSprite(ListNum,StartFUC)) {return true;}
    }
  }
  return false;
}
//*******************************************************************
bool TGObjSp::IsActiveSprite(int16 ListNum)
{
  if (GetSpriteList() == NULL) {return false;}
  return GetSpriteList()->IsActiveElement(ListNum);
}
//*******************************************************************
#pragma argsused
TSprite *TGObjSp::CreateSprite(string spritename,int16 id,void *psource)
{
  TFileMode FM = FileMode;
//  FM.SetName(spritename);
  FM.Name = spritename;
  if (psource != NULL)
  {
    FM.SetNone();
    return new TSprite(PModeWin,FM,psource);
  }
  else
  {
    FM.SetRamImmed();
    FM.SetHDNone();
    return new TSprite(PModeWin,FM);
  }
}
//*******************************************************************

