//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  orectlst.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     7/24/96    M.Perry    original bsaeline
//  2.0     8/22/97    M.Perry    32-bit conversion & added Cached rect code.
//********************************************************************
#include "orectlst.h"
//*******************************************************************
TOgleRectList::TOgleRectList()
:TSliq(true,true),
 Clipping(false)
{
  PCachedRectHolder = NULL;
}
//*******************************************************************
TOgleRectList::TOgleRectList(TOgleRect &cliprect)
:TSliq(true,true),
 ClipRect(cliprect),
 Clipping(true)
{
  PCachedRectHolder = NULL;
}
//*******************************************************************
TOgleRectList::~TOgleRectList()
{
  Empty();

  if (PCachedRectHolder != NULL)
  {
    TOgleRect *prect = (TOgleRect *)PCachedRectHolder->Pop();
    while (prect != NULL)
    {
      delete prect;
      prect = (TOgleRect *)PCachedRectHolder->Pop();
    }
  }
}
//*******************************************************************
bool TOgleRectList::Add(TOgleRect &rect)
{
  return CreateAddRect(rect);
}
//*******************************************************************
bool TOgleRectList::Remove(TOgleRect &rect)
{
  TOgleRect *prect = GetFirst();
  if (*prect == rect) {return DeleteRemoveRect(prect);}
  while (prect != NULL)
  {
    prect = GetNext();
    if (*prect == rect) {return DeleteRemoveRect(prect);}
  }
  return false;
}
//*******************************************************************
bool TOgleRectList::AddMergeRect(TOgleRect &rect)
{
//  TOgleRect Rect = rect; // don't change original.
  if (rect.IsEmpty()) {return true;}

  TOgleRect *prect = GetFirst();
  while (prect != NULL)
  {
    if (rect.Contains(*prect))
    {
      if (rect == *prect) {return true;} //exact match so we know we can bail early.
      rect += *prect;
      DeleteRemoveRect(prect);
      prect = GetFirst();
    }
    else
    {
      prect = GetNext();
    }
  }
  CreateAddRect(rect);
  return true;
}
//*******************************************************************
bool TOgleRectList::AddDisjointRects(TOgleRect &rect)
// recursive function !!!!
{
  if (rect.IsEmpty()) {return true;}

  TOgleRect r1,r2,r3,r4;
  bool vr1,vr2,vr3,vr4;
  TOgleRect *prect = GetFirst();
  while (prect != NULL)
  {
    if (rect.Contains(*prect))
    {
      if (rect.Disjoint(*prect,r1,r2,r3,r4,vr1,vr2,vr3,vr4))
      {
        // try adding the parts of the rect that didn't overlap.
        if (vr1) {AddDisjointRects(r1);}
        if (vr2) {AddDisjointRects(r2);}
        if (vr3) {AddDisjointRects(r3);}
        if (vr4) {AddDisjointRects(r4);}
      }
      return true; // either we added parts, or were completely enveloped, but our job is done.
    }
    prect = GetNext();
  }
  CreateAddRect(rect);  // rect never overlapped another in the list, so add the whole rect.
  return true;
}
//*******************************************************************
bool TOgleRectList::AddToList(TOgleRectList &rectlist)
{
  TOgleRect *prect = GetFirst();
  while (prect != NULL)
  {
    rectlist.Add(*prect);
    prect = GetNext();
  }
  return true;
}
//*******************************************************************
void TOgleRectList::Empty()
{
  TOgleRect *prect = PopRect();
  while (prect != NULL)
  {
    delete prect;
    prect = PopRect();
  }
}
//*******************************************************************
TOgleRect TOgleRectList::GetEnclosingRect()
{
  TOgleRect rect;
  TOgleRect *prect = GetFirst();
  if (prect != NULL) {rect = *prect;}  // this keeps rect's tl from starting at (0,0).
  while (prect != NULL)
  {
    rect += *prect;
    prect = GetNext();
  }
  return rect;
}
//*******************************************************************
//*******************************************************************
TRectMergeList::TRectMergeList(TOgleRect &cliprect)
:TOgleRectList(cliprect)
{
}
//*******************************************************************
TRectMergeList::TRectMergeList()
{
}
//*******************************************************************
TRectMergeList::~TRectMergeList()
{
}
//*******************************************************************
bool TRectMergeList::Add(TOgleRect &rect)
{
  TOgleRect Rect(rect);
  if (IsClipping())
  {
    Rect = Rect.Intersect(ClipRect);
    if (Rect.IsEmpty()) {return true;}
  }
  return AddMergeRect(Rect);
}
//*******************************************************************
//*******************************************************************
TRectNoOverlapList::TRectNoOverlapList(TOgleRect &cliprect)
:TOgleRectList(cliprect)
{
}
//*******************************************************************
TRectNoOverlapList::TRectNoOverlapList()
{
}
//*******************************************************************
TRectNoOverlapList::~TRectNoOverlapList()
{
}
//*******************************************************************
bool TRectNoOverlapList::Add(TOgleRect &rect)
{
  TOgleRect Rect(rect);
  if (IsClipping())
  {
    Rect = Rect.Intersect(ClipRect);
    if (Rect.IsEmpty()) {return true;}
  }
  return AddDisjointRects(rect);
}
//*******************************************************************

