//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oanm.cpp           
//    Project Name:  Ogle
// Original Author:  Michael R. Perry        
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/5/95     M.Perry    original baseline
//  2.0     8/22/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__oanm_h)
#include "oanm.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#include <assert.h>

#if !defined(__obmp_h)
#include "obmp.h"
#endif

#if !defined(__omisc_h)
#include "omisc.h"
#endif

#if !defined(__oanmlst_h)
#include "oanmlst.h"
#endif

#if !defined(__odc_h)
#include "odc.h"
#endif

// note: MRP -> I might have been off by 1 here. word_max == 65535.
// look into it when I work on GEd and integrating cutter.
// It shouldn't affect reading any old files if changed, I think :)
#define MAX_WORD ((DWORD)(65536l))

DIAG_DEFINE_GROUP(ANM,1,0);

//*******************************************************************
TAnmData &TAnmData::SetData(TAnmData &ad)
{
  SetXYZ(ad.GetX(),ad.GetY(),ad.GetZ());
  SetStartEndFrame(ad.GetStartFrame(),ad.GetEndFrame());
  SetAutoStart(ad.GetAutoStart());
  SetAutoStop(ad.GetAutoStop());
  SetAutoChange(ad.GetAutoChange());
  SetDirection(ad.GetDirection());
  SetCurrentFrame(ad.GetCurrentFrame());
  OffsetVPt = ad.OffsetVPt;
  SetUseDefaultXY(ad.GetUseDefaultXY());
  return *this;
}
//*******************************************************************
TAnmData::TAnmData(const TAnmData &ad)
:TGrData((TGrData &)ad),
 StartFrame(ad.StartFrame),EndFrame(ad.EndFrame),CurrentFrame(ad.CurrentFrame),
 Direction(ad.Direction)
{
  SetAutoStart(ad.GetAutoStart());
  SetAutoStop(ad.GetAutoStop());
  SetAutoChange(ad.GetAutoChange());
}
//*******************************************************************
TAnmData::TAnmData()
:StartFrame(0),EndFrame(-1),CurrentFrame(0),
 Direction(dForward)
{
  SetAutoStart(true);
  SetAutoStop(true);
  SetAutoChange(true);
}
//*******************************************************************
TAnmData::TAnmData(int16 x,int16 y,int16 z,
                   int16 sf,int16 ef,int16 cf,
                   TDirection direction,
                   bool autostart,bool autostop,bool autochange)
:TGrData(x,y,z),
 StartFrame(sf),EndFrame(ef),CurrentFrame(cf),
 Direction(direction)
{
  SetAutoStart(autostart);
  SetAutoStop(autostop);
  SetAutoChange(autochange);
}
//*******************************************************************
TAnmData::~TAnmData()
{
}
//*******************************************************************
//*******************************************************************
#if defined(GED_COMPILE)
TAnmFrameLineSegment::TAnmFrameLineSegment()
{
  PReadPixels = NULL;
}
//*******************************************************************
TAnmFrameLineSegment::~TAnmFrameLineSegment()
{
  delete [] PReadPixels;
  PReadPixels = NULL;
}
//*******************************************************************
dword TAnmFrameLineSegment::GetSize()
{
  dword size = sizeof(word) + sizeof(word) + Len;
  return size;
}
//*******************************************************************
bool TAnmFrameLineSegment::Write(TOgleBase *pbase)
{
  pbase->Writeword(&LocalXOffset);
  pbase->Writeword(&Len);
  pbase->WriteData(PReadPixels,Len);
  return true;
}
//*******************************************************************
#pragma argsused
bool TAnmFrameLineSegment::Convert(word framenum,word linenum,word segmentnum,
                                   byte huge *prawsegment,word localxoff,word lenleft)
{
  byte huge *psrc = prawsegment;
  Len = 0;
  while (Len < lenleft && *psrc != 0)
  {
    Len++;
    psrc++;
  }
  assert(Len > 0);

  ReadSize = Len;
  PReadPixels = new byte[Len];
  hmemcpy(PReadPixels,prawsegment,Len);
  LocalXOffset = localxoff;
  return true;
}
//*******************************************************************
//*******************************************************************
TAnmFrameLine::TAnmFrameLine()
{
  PReadSegmentOffsets = NULL;
  PReadSegments = NULL;
}
//*******************************************************************
TAnmFrameLine::~TAnmFrameLine()
{
  delete [] PReadSegmentOffsets;
  PReadSegmentOffsets = NULL;
  delete [] PReadSegments;
  PReadSegments = NULL;
}
//*******************************************************************
dword TAnmFrameLine::GetSize()
{
  dword size = GetStructSize(); // if zero segments then size is just sizeof(word)
  if (NumSegments > 0) // this takes into account padding between table and first segment.
  {
    size = PReadSegmentOffsets[NumSegments] - StartingOffset;
  }
  return size;
}
//*******************************************************************
dword TAnmFrameLine::GetStructSize()
{
  dword size = sizeof(word);
  if (NumSegments > 0)
  {
    size += (NumSegments * sizeof(dword));
  }
  return size;
}
//*******************************************************************
bool TAnmFrameLine::Write(TOgleBase *pbase)
{
  // we know that our struct will fit because frame made sure for us.
  pbase->Writeword(&NumSegments);
  if (NumSegments == 0) {return true;} // don't write table if zero segments.
  pbase->WriteData(PReadSegmentOffsets,(NumSegments * sizeof(dword)));

  // but after our struct we fend for ourselves.

  // start -> pad between table and first segement.
  int32 begpad = PReadSegmentOffsets[0] - (StartingOffset + GetStructSize());
  assert(begpad >= 0);
  if (begpad > 0)
  {
    byte *pb2 = new byte[(int)begpad];
    memset(pb2,'\0',(int)begpad);
    pbase->WriteData(pb2,begpad);
    delete [] pb2;
  }
  // end

  for (word i=0;i<NumSegments;i++)
  {
    PReadSegments[i].Write(pbase);
    // pad out til 64k boundary.
    if (i < (NumSegments - 1)) // don't pad after last one.
    {
      int32 diff = PReadSegmentOffsets[i + 1] - PReadSegmentOffsets[i];
      diff -= PReadSegments[i].GetSize();
      assert(diff >= 0); // just in case of a fuck-up.
      if (diff > 0) // if diff > 0 then we're near 64k boundary.
      {
        byte *pb = new byte[(int)diff];
        memset(pb,'\0',(int)diff);
        pbase->WriteData(pb,diff);
        delete [] pb;
      }
    }
    // end of padding.
  }
  return true;
}
//*******************************************************************
word TAnmFrameLine::CountSegments(byte huge *prawline,int16 width)
// only count non transparent segments.
{
  word count = 0;
  byte huge *psrc = prawline;
  int16 x = 0;
  bool tranny = true;
  while (x < width)
  {
    if (*psrc == 0)
    {
      int16 translen;
      hmemcpy(&translen,psrc + 1,sizeof(int16));
      psrc += 3;
      x += translen;
      tranny = true;
    }
    else
    {
      x++;
      psrc++;
      if (tranny)
      {
        tranny = false;
        count++;
      }
    }
  }
  return count;
}
//*******************************************************************
bool TAnmFrameLine::Convert(TAnmConvertDlg *pacd,word framenum,word linenum,int16 width,byte huge *prawline)
{
  NumSegments = CountSegments(prawline,width); // loop thru line counting non-tran segments.

  pacd->SetSegmentRange(0,NumSegments);
  pacd->SetSegmentValue(0);

  PReadSegments = new TAnmFrameLineSegment[NumSegments];
  PReadSegmentOffsets = new dword[NumSegments + 1];

  word segcount = 0;

  byte huge *psrc = prawline;
  int16 x = 0;
  ReadSize = 0;
  while (x < width)
  {
    if (*psrc == 0)
    {
      int16 translen;
      hmemcpy(&translen,psrc + 1,sizeof(int16));  // get next int16 after byte, it's length of tran seg.
      psrc += 3;
      ReadSize += word(3);
      x += translen;
    }
    else
    {
      assert(segcount < NumSegments);
      PReadSegments[segcount].Convert(framenum,linenum,segcount,psrc,x,width - x);
      word readsize = PReadSegments[segcount].GetReadSize();
      ReadSize += readsize;
      x += readsize;
      psrc += readsize;
      segcount++;
      pacd->IncSegmentValue();
    }
  }
  return true;
}
//*******************************************************************
void TAnmFrameLine::SetTableOffsets(dword offset)
{
  StartingOffset = offset;
  PReadSegmentOffsets[0] = offset + GetStructSize();
  for (word i=0;i<NumSegments;i++)
  {
    // start -> increase offsets past 64k boundary.
    dword offset2 = PReadSegmentOffsets[i];
    offset2 = offset2 % MAX_WORD;
    dword structsize = PReadSegments[i].GetSize(); // the whole thing because of the
                                                   // Pixel array.
    if ((offset2 + structsize) > MAX_WORD)
    {
      PReadSegmentOffsets[i] += (MAX_WORD - offset2);
    }
    // end.
    PReadSegmentOffsets[i + 1] = PReadSegmentOffsets[i] + PReadSegments[i].GetSize();
  }
}
//*******************************************************************
//*******************************************************************
TAnmFrame::TAnmFrame()
{
  PReadLines = NULL;
  PLineOffsets = NULL;
}
//*******************************************************************
TAnmFrame::~TAnmFrame()
{
  delete [] PLineOffsets;
  PLineOffsets = NULL;
  delete [] PReadLines;
  PReadLines = NULL;
}
//*******************************************************************
dword TAnmFrame::GetSize()
{
  dword size = GetStructSize();  // if zero lines then equal to sizeof(word)
  if (height > 0) // this takes into account padding between table and first line.
  {
    size = PLineOffsets[height] - StartingOffset;
  }
  return size;
}
//*******************************************************************
dword TAnmFrame::GetStructSize()
{
  dword size = sizeof(TAnmFrameHeader) + (height * sizeof(dword));
  return size;
}
//*******************************************************************
bool TAnmFrame::Convert(TAnmConvertDlg *pacd,word framenum,TAnmFrameHeader huge *pframeheader,TAnmFrameHeader &afh)
{
  height = afh.height;

  width = afh.width;

  if (height < 0 || width < 0)
  {
    // put off screen and ignore rest of v1.0 frame.
    height = 0;
    width = 0;
    ref[0].x = -5;
    ref[0].y = -5;
    return true;
  }

  ref[0] = afh.ref[0];
  ref[1] = afh.ref[1];
  ref[2] = afh.ref[2];
  ref[3] = afh.ref[3];

  pacd->SetLineRange(0,height);
  pacd->SetLineValue(0);

  PReadLines = new TAnmFrameLine[height];
  PLineOffsets = new dword[height + 1];
  byte huge *prawdata = (byte huge *)((byte huge *)pframeheader + sizeof(TAnmFrameHeader));
  for (int16 i = int16(height - 1);i >= 0;i--)
  {
    PReadLines[i].Convert(pacd,framenum,i,width,prawdata);
    prawdata += PReadLines[i].GetReadSize();
    pacd->IncLineValue();
  }
  return true;
}
//*******************************************************************
void TAnmFrame::SetTableOffsets(dword offset)
// sets up offsets, and indirectly determines final size.
{
  StartingOffset = offset;
  if (height <= 0) {return;}

  PLineOffsets[0] = offset + GetStructSize();
  for (word i=0;i<height;i++)
  {
    // start -> increases offsets past 64k boundaries.
    dword offset2 = PLineOffsets[i];
    offset2 = offset2 % MAX_WORD;
    dword structsize = PReadLines[i].GetStructSize();
    if ((offset2 + structsize) > MAX_WORD)
    {
      PLineOffsets[i] += (MAX_WORD - offset2);
    }
    // end
    PReadLines[i].SetTableOffsets(PLineOffsets[i]);
    PLineOffsets[i + 1] = PLineOffsets[i] + PReadLines[i].GetSize();
  }
}
//*******************************************************************
bool TAnmFrame::Write(TOgleBase *pbase)
{
  // we know that our struct will fit because anm made sure for us.
  pbase->WriteData(this,sizeof(TAnmFrameHeader));

  if (height <= 0) {return true;}

  pbase->WriteData(PLineOffsets,(height * sizeof(dword)));

  // but after our struct, we fend for ourselves.

  // start -> pad between table and first line, if necessary.
  int32 begpad = PLineOffsets[0] - (StartingOffset + GetStructSize());
  assert(begpad >= 0);

  if (begpad > 0)
  {
    byte *pb2 = new byte[(int)begpad];
    memset(pb2,'\0',(int)begpad);
    pbase->WriteData(pb2,begpad);
    delete [] pb2;
  }
  // end

  for (word i=0;i<height;i++)
  {
    PReadLines[i].Write(pbase);
    // pad out til 64k boundary.
    if (i < (height - 1))  // don't pad after last one.
    {
      int32 diff = PLineOffsets[i + 1] - PLineOffsets[i];
      diff -= PReadLines[i].GetSize();
      assert(diff >= 0); // just in case of a fuck-up.
      if (diff > 0) // if diff > 0 then we're near 64k boundary.
      {
        byte *pb = new byte[(int)diff];
        memset(pb,'\0',(int)diff);
        pbase->WriteData(pb,diff);
        delete [] pb;
      }
    }
    // end of padding.
  }
  return true;
}
#endif
//*******************************************************************
//*******************************************************************
void TAnm::SetMajVer()
{
  MajVer = 2;
}
//*******************************************************************
void TAnm::SetMinVer()
{
  MinVer = 0;
}
//*******************************************************************
const char *TAnm::GetIDChars() const
{
  return "ANM ";
}
//*******************************************************************
bool TAnm::ReadLocalHeader()
{
  bool oldanmfile = false;
  if (!ReadIDChars(IDChars)) {return false;}
  if (!IDOk()) {oldanmfile = true;} // ?? should jump over rest of header reads
                                    //    this way just makes sure the file is
                                    //    at least (header size) filelength.
  if (!Readword(&MajVer)) {return false;}
  if (!Readword(&MinVer)) {return false;}
  dword tempsize;
  if (!Readdword(&tempsize)) {return false;}

  if (oldanmfile)
  {
    MajVer = 1;
    MinVer = 0;
//    FileOffset = 0; // if old anm file then getptr() needs to start at beginning of buffer.
  }
  SetReadFormat();
  return true;
}
//*******************************************************************
// MRP - v2.3 - note: re-look at the whole BeenRead thing, probably need
//                    to let requestmodechange do the read, or whenever locked.
bool TAnm::ReadLocal_1_0()
// if GED_COMPILE we write new format,dump ram,reload file, then do read again, this time
// using ReadLocal_2_0
// if not GED_COMPILE then we're playing the game so we want to abort because this
// is an old anm format that hasn't been converted yet
{
  #if defined(GED_COMPILE)
  Write();
  SetBeenRead(); // to keep requestmodechange from doing the read.

  // *** special situation ***
  PBaseFile->UnLoadRamHD();
//  LoadNone();

  string s = PBaseFile->GetCDRomName();
  PBaseFile->SetSize(TOgleFileName::GetFileSize(s));

  // ** special situation
  PBaseFile->LoadRam();
//  LoadRamImmed();

  Read();
  return true;
  #else
  string msgstr = "Invalid file version read\nBAD THINGS, MAN.\n";
  msgstr += FileMode.Name.GetStr();
  TRACEX(ANM,0,msgstr.c_str());
  PModeWin->MessageBox(msgstr.c_str(),"Error - TAnm",MB_OK);
  abort();
  return false;
  #endif
}
//*******************************************************************
bool TAnm::ReadLocal_2_0()
{
  if (CAD.GetUseDefaultXY())
  {
    int16 x = GetDefaultScreenX();
    int16 y = GetDefaultScreenY();
    SAD.SetXY(int16(x - 1),int16(y - 1));
    CAD.SetXY(int16(x - 1),int16(y - 1));
  }
  else
  {
    SAD.SetXY(int16(SAD.GetX() + 1),int16(SAD.GetY() + 1));
    CAD.SetXY(int16(CAD.GetX() + 1),int16(CAD.GetY() + 1));
  }
  return true;
}
//*******************************************************************
bool TAnm::ReadLocal()
{
  switch (MajVer)
  {
    case 1:return ReadLocal_1_0();
    case 2:return ReadLocal_2_0();
    default:
    {
      PModeWin->MessageBox(NULL,"Unknown anm file version","Error - TAnm",MB_OK);
      return false;
    }
  }
}
//*******************************************************************
#if defined(GED_COMPILE)
bool TAnm::WriteLocal_From_1_0_To_2_0()
{
  TAnmConvertDlg *pacd = new TAnmConvertDlg(PModeWin,FileMode.Name.GetStr());
  pacd->Create();

  // convert all frames.
  word numframes = GetFrameNum();
  pacd->SetFrameMax(numframes);

  TAnmFrame *pframes = new TAnmFrame[numframes];
  dword *pframeoffsets = new dword[numframes + 1];
  for (word i=0;i<numframes;i++)
  {
    TAnmFrameHeader huge *pafh = GetFrameHeaderPtr(i);
    TAnmFrameHeader afh;
    CopyAnmFrameHeader(afh,pafh);
    pframes[i].Convert(pacd,i,pafh,afh);
    pacd->IncFrameValue();
  }

  pacd->StartOffsets();

  dword StartingOffset = FILEHEADERSIZE + sizeof(TAnmMainHeader) + (numframes * sizeof(dword));
  pframeoffsets[0] = StartingOffset;
  for (word i = 0;i<numframes;i++)
  {
    dword offset = pframeoffsets[i]; // get offset.
    offset = offset % MAX_WORD;   // get offset into last 64k boundary.
    dword structsize = pframes[i].GetStructSize(); // get size of run-time frame struct.
    if ((offset + structsize) > MAX_WORD) // will struct overlap 64k boundary?
    {
      pframeoffsets[i] += (MAX_WORD - offset);  // increase offset to 0 of next boundary.
    }
    pframes[i].SetTableOffsets(pframeoffsets[i]);
    pframeoffsets[i + 1] = pframeoffsets[i] + pframes[i].GetSize();
  }

  pacd->StartWriting();
  // file header already written before this is called.

  // write anm's main header.
  WriteData(GetMainPtr(),sizeof(TAnmMainHeader));

  // write out frame offset table
  WriteData(pframeoffsets,(numframes * sizeof(dword)));

  // don't need to bother padding after table and before first frame, because we're
  // assuming that it can't possibly be past the first 64k boundary already....

  // write each frame.
  for (word i=0;i<numframes;i++)
  {
    pframes[i].Write(this);
    // pad out til end of 64k boundary.
    int32 diff = pframeoffsets[i+1] - pframeoffsets[i];
    diff -= pframes[i].GetSize();
    assert(diff >= 0); // just in case of a fuck-up.
    if (diff > 0)  // if diff > 0 then we're near 64k boundary.
    {
      byte *pb = new byte[(int)diff];
      memset(pb,'\0',(int)diff);
      WriteData(pb,diff);
      delete [] pb;
    }
    // end of padding.
  }

  // cleanup temp frame offset table.
  delete [] pframeoffsets;

  // cleanup temp frame list
  delete [] pframes;

  delete pacd;

  return true;
}
#endif
//*******************************************************************
bool TAnm::WriteLocal()
{
  #if defined(GED_COMPILE)
  if (IsReadFormat_1_0())
  {
    return WriteLocal_From_1_0_To_2_0();
  }
  #endif
  return true;
}
//*******************************************************************
void TAnm::Show()
{
  SetShowing();
  if (GetStarted() && !GetStopped())
  {
    AddToScrnMgr();
  }
}
//*******************************************************************
void TAnm::NoShow()
{
  RemoveFromScrnMgr();
  ClearShowing();
}
//*******************************************************************
string TAnm::AddExt()
{
  return string(".anm");
}
//*******************************************************************
#ifdef DEBUGSPRITES
void TAnm::RTReport(ofstream &os)
{
//  os<<"Anm Name: "<<GetFName()<<"\n";
  os<<"Anm Name: "<<FileMode.Name.GetStr()<<"\n";
  os<<"Ram loaded: "<<GetRamLoaded()<<"\n";
  os<<"Added: "<<GetAddedToScrnMgr()<<"\n";
  os<<"Showing: "<<GetShowing()<<"\n";
  os<<"BeginFUC: "<<BeginFUC<<"\n";
  os<<"Started: "<<GetStarted()<<"\n";
  os<<"Stopped: "<<GetStopped()<<"\n";
}
#endif
//*******************************************************************
#ifdef DEBUGREPORTS
void TAnm::Report(ofstream &os)
{
//  os<<"Anm Name: "<<GetFName()<<"\n";
  os<<"Anm Name: "<<FileMode.Name.GetStr()<<"\n";
}
#endif
//*******************************************************************
TAnm::TAnm(TOgleModeWindow *pmw,
           const TFileMode &filemode,
           TAnmData &sad,
           bool AutoRead,
           bool KeepBaseFile)
:TGr(pmw,filemode),
 SAD(sad),
 CAD(sad),
 PAnmList(NULL)
{
  #ifdef DEBUG_PMODEWIN_ANM
  if (PModeWin == NULL)
  {
    string s = "PModeWin == NULL for anm ";
    s += FileMode.Name;
    ::MessageBox(NULL,s.c_str(),"Warning !!",MB_OK);
  }
  #endif

  #if defined(GED_COMPILE)
  // we're forcing an auto read to convert file if needed.
  TFileMode fm(filemode);
  fm.SetRamImmed();
  AutoRead = true;
  Init(fm,AutoRead,KeepBaseFile);
  Load(filemode.GetRamStatus(),filemode.GetHDStatus());  // put into originally requested status.
  #else
  Init(filemode,AutoRead,KeepBaseFile);
  #endif

  #ifdef DEBUGANM
  if (PBaseFile == NULL)
  {
    ::MessageBox(NULL,"PBaseFile is NULL","Anm Init Error",MB_APPLMODAL|MB_OK);
    assert(PBaseFile);
  }
  #endif
}
//*******************************************************************
TAnm::~TAnm()
{
}
//*******************************************************************
void TAnm::Activate(dword StartFUC)
{
  TGr::Activate(StartFUC);
  if (!LoadAhead(StartFUC)) {ACLoad();}
}
//*******************************************************************
void TAnm::DeActivate()
{
  TGr::DeActivate();
  ACLoad();
}
//*******************************************************************
void TAnm::SetCADRelative(TTriplet &trip)
{
  CAD.SetOffsetVPt(trip);
}
//*******************************************************************
// MRP - v2.3 - note: need to re-think BeenRead logic, should probably
//                    re-read after every pbf->Lock()
bool TAnm::RequestChangeMode(TFileMode &NewFileMode)
{
  if (GetAddedToScrnMgr() && !NewFileMode.IsRamImmed()) {NewFileMode.SetRamImmed();}
  bool rc = TGr::RequestChangeMode(NewFileMode);
  if (rc && GetRamLoaded() && !GetBeenRead())
  {
    ReadSize = Read(false); // false -> don't use fileoffset.
  }
  return rc;
}
//*******************************************************************
void TAnm::LoadCurrentAnmFrame()
// with new v2.0 format we no longer have to worry about straddling 64k boundary
// so we don't bother hmemcpy'ing structure to new structure buffer.
{
  if (!GetRamLoaded())
  {
    TRACEX(ANM,0,"LoadCurrentAnmFrame->Ram not loaded");
    return;
  }

  int16 currentframe = CAD.GetCurrentFrame();
  TAnmFrame huge *paf = GetFramePtr(currentframe);
  if (paf == NULL) {return;}

  GrDrawData.CalcVRect(CAD.GetAdjVPt(),paf->ref[0],paf->width,paf->height);
  GrDrawData.SetFrameNum(currentframe);
}
//*******************************************************************
void TAnm::NextManual()
{
  LoadCurrentAnmFrame();
}
//*******************************************************************
void TAnm::Update(dword NewFUC)
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartAnmUpdate();
  #endif
  if (BeginFUC == NewFUC) // get her going.
  {
    Start();
  }
  else if (GetStarted() && !GetStopped())
  {
    if (GetShowing() && !GetAddedToScrnMgr())
    {
      AddToScrnMgr();
    }
    NextFrame();
  }
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndAnmUpdate();
  #endif
}
//*******************************************************************
#pragma argsused
void TAnm::PostUpdate(dword NewFUC)
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartAnmPUpdate();
  #endif

  if (!GetStarted() || GetStopped())
  {
    #if defined(DEBUGUPDATES)
    PModeWin->DebugUpdate.EndAnmPUpdate();
    #endif
    return;
  }

  if (CAD.GetDirection() == dForward)
  {
    if ((CAD.GetCurrentFrame() == CAD.GetEndFrame()) ||
        (Get_FUC_Length() > 0 && NewFUC >= EndFUC) ||
        (!IsValidFrame(int16(CAD.GetCurrentFrame() + 1))))
    {
      Stop();
    }
  }
  else if (CAD.GetDirection() == dReverse)
  {
    if ((CAD.GetCurrentFrame() == CAD.GetEndFrame()) ||
        (Get_FUC_Length() > 0 && NewFUC >= EndFUC) ||
        (!IsValidFrame(int16(CAD.GetCurrentFrame() - 1))))
    {
      Stop();
    }
  }
  else if (CAD.GetDirection() == dStill) // MRP - v2.3
  {
    // if not set to forever then check endfuc for stop check
    if (NewFUC >= EndFUC && Get_FUC_Length() > 0 && CAD.GetAutoStop())
    {                                               // added for reels.
      Stop();                                       // need to add autostop logic everywhere.
    }
    #if defined(GED_COMPILE)
    if (Get_FUC_Length() == 0)
    {
      if (GetAnmList() == NULL)
      {
        Stop();
      }
      else if (GetAnmList()->GetNumElements() == 1)
      {
        Stop();
      }
      else if (GetAnmList()->GetNotifyRef() == 1)
      {
        Stop();
      }
    }
    #endif
  }
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndAnmPUpdate();
  #endif
}
//*******************************************************************
void TAnm::ManualStart()
{
}
//*******************************************************************
void TAnm::AutoStart()
{
  if (CAD.GetDirection() == dForward)
  {
    if (Get_FUC_Length() <= 0)
    {
//      CAD.SetStartFrame(0);
      CAD.SetEndFrame(GetLastFrameNum());
    }
    else
    {
      int16 ef = CAD.GetStartFrame() + Get_FUC_Length() - 1;
      if (ef > GetLastFrameNum())
      {
        ef = GetLastFrameNum();
      }
      CAD.SetEndFrame(ef);
    }
  }
  else if (CAD.GetDirection() == dReverse)
  {
    if (CAD.GetStartFrame() == 0 || Get_FUC_Length() == 0)
    {
      CAD.SetStartFrame(GetLastFrameNum());
      CAD.SetEndFrame(0);
    }
    else
    {
      int16 ef = CAD.GetStartFrame() - Get_FUC_Length();
      if (ef < 0) {ef = 0;}
      CAD.SetEndFrame(ef);
    }
  }
  else if (CAD.GetDirection() == dStill)
  {
    //do something for still ?
  }
  CAD.SetCurrentFrame(CAD.GetStartFrame());
}
//*******************************************************************
void TAnm::Start()
{
  LoadRamImmed();
  if (GetShowing() && !GetAddedToScrnMgr())
  {
    AddToScrnMgr();
  }
  TGr::Start();

  if (CAD.GetAutoStart()) {AutoStart();}
  else {ManualStart();}

  LoadCurrentAnmFrame();
}
//*******************************************************************
void TAnm::Start(dword StartFUC)
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartAnmStart();
  #endif
  // schedule starting time, used by Update().
  BeginFUC = StartFUC + FUC_Offset;
  EndFUC = BeginFUC + Get_FUC_Length() - 1; // mrp - v2.3
  if (EndFUC < BeginFUC) {EndFUC = BeginFUC;} // mrp - v2.3

  ClearStarted();
  ClearStopped();
  dword curFUC = AskCurrentFUC();
  if (BeginFUC == curFUC) {Start();}
  ALNotifyStart();
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndAnmStart();
  #endif
}
//*******************************************************************
void TAnm::ManualStop()
{
  // ?? ALNotifyStop(); ??
}
//*******************************************************************
void TAnm::Stop()
{
  if (!CAD.GetAutoStop()) {ManualStop();}
  else
  {
    RemoveFromScrnMgr();
    SetStopped();
    ACLoad();
//    ALNotifyStop();
  }
  ALNotifyStop();
}
//*******************************************************************
void TAnm::ALNotifyStart()
{
  if (PAnmList == NULL) {return;}
  if (!GetALNotifiedStartScheduled())
  {
    SetALNotifiedStartScheduled();
    PAnmList->NotifyStart(this);
  }
}
//*******************************************************************
void TAnm::ALNotifyStop()
{
  if (PAnmList == NULL) {return;}
  if (GetALNotifiedStartScheduled())
  {
    PAnmList->NotifyStop(this);
    ClearALNotifiedStartScheduled();
  }
}
//*******************************************************************
bool TAnm::BltTo(TOgleVideoDC &vdc,TOgleRect &clipvrect)
// GED_COMPILE - we check just in case we didn't convert file
// otherwise we assume that we caught bad format before getting to this point.
{
  #if defined(GED_COMPILE)
  if (IsReadFormat_1_0())
  {
    string msg = "Anm: ";
    msg += FileMode.Name.GetStr();
    msg += " is v1.0\nWill abort";
    PModeWin->MessageBox(msg.c_str(),"Fatal Error - TAnm",MB_OK | MB_SYSTEMMODAL);
    abort();
    return false;
  }
  #endif
  TBmp *pbmp = vdc.GetBmp();
  if (pbmp == NULL) {return false;}
  if (!pbmp->GrDrawData.IsLinear()) {return false;}

  return BltTo(*pbmp,clipvrect);
}
//*******************************************************************
bool TAnm::BltTo(TGr &gr,TOgleRect &clipvrect)
// This function assumes a linear destination pointer, in the same format as bmp's.

// Note!! This function accounts for the majority of run-time cpu cycles.
// every line has been broken down into continuous non-transparent pixel runs called segments.
// There are no transparent pixels in a segment. This is the single biggest reason of the
// speed-up. We no longer have to check every pixel for transparency. We can now just
// do a memory copy of the pixels we are interested in, after, of course, CLIPPING.
// Clipping is a necessary evil, especially to prevent writing into non-existent memory areas.
{
  // bail as early as possible.
  TOgleRect vrect(GrDrawData.GetVRect());
  TOgleRect isvrect(vrect.Intersect(clipvrect));
  if (isvrect.IsEmpty()) {return true;} // if empty frame or offscreen then we'll bail here.

  byte huge *pram = (byte huge *)GetRamPtr();
  if (pram == NULL) {return false;}

  TAnmFrame huge *pframe = GetFramePtr(GrDrawData.GetFrameNum());
  if (pframe == NULL) {return false;}

  byte huge *pdest = NULL;
  byte huge *pdestbits = gr.GrDrawData.GetBits();
  if (pdestbits == NULL) {return false;}

  TOgleRect DestVRect(gr.GrDrawData.GetVRect());

  int16 bmpwidth = DestVRect.GetWidth();
  int16 bmpheight = DestVRect.GetHeight();

  int16 bmpwidth4 = NumScanBytes(bmpwidth);

  bool topdown = gr.GrDrawData.IsTopDown();

  // var's used in loop, to avoid any possible compiler induced slowdown every iteration.
  TAnmFrameLine huge *pline;
  TAnmFrameLineSegment huge *psegment;
  int16 vleft,vright,copyleft,copyright,copylen,diff;

  for (int16 vy = isvrect.top,ly = int16(isvrect.top - vrect.top);
       vy < isvrect.bottom;
       ly++,vy++)
  {
    pline = pframe->GetLinePtr(pram,ly);
    if (pline == NULL) {return false;}

    for (int16 seg = 0;seg < pline->NumSegments;seg++)
    {
      psegment = pline->GetSegmentPtr(pram,seg);
      vleft = int16(vrect.left + psegment->LocalXOffset);
      vright = int16(vleft + psegment->Len);

      // start -> clip the local rect.
      if (vright < isvrect.left) // completely to left of isvrect.
      {
        continue;
      }

      copyleft = vleft;
      if (vleft < isvrect.left)
      {
        copyleft = isvrect.left;
      }

      copyright = vright;
      if (vright > isvrect.right)
      {
        copyright = isvrect.right;
      }

      copylen = int16(copyright - copyleft);
      if (copylen <= 0) // completely to right of isvrect.
      {
        continue;
      }

      diff = int16(copyleft - vleft); // offset into Pixel array.
      // end -> clip the local rect.

      pdest = GetPixelAddr(pdestbits,copyleft,vy,topdown,bmpwidth4,bmpheight);
      hmemcpy(pdest,&(psegment->Pixels[diff]),copylen);
//      PModeWin->UpdateWindow();
    }
  }
  return true;
}
//*******************************************************************

