//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:   oanmlst.cpp
//    Project Name:   Ogle
// Original Author:   Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     6/20/95    M.Perry    original baseline
//  2.0     8/22/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__oanmlst_h)
#include "oanmlst.h"
#endif

#if !defined(__oanm_h)
#include "oanm.h"
#endif

#if !defined(__osprite_h)
#include "osprite.h"
#endif

#include <assert.h>
//*******************************************************************
#ifdef DEBUGSPRITES
void TAnmList::RTReport(ofstream &os)
{
  os<<"********************************************\n";
  os<<"Start of Anm List\n";
  os<<"Started: "<<GetStarted()<<"\n";
  os<<"Stopped: "<<GetStopped()<<"\n";
  os<<"Last Anm Index #: "<<LastAddedElementNum<<"\n";
  for(int16 i=0;i<=LastAddedElementNum;i++)
  {
    os<<"**********************************\n";
    os<<"Index #"<<i<<"\n";
    TAnm *panm = GetAnmPtr(i);
    if (panm == NULL) {os<<"Anm Ptr was NULL\n";}
    else              {panm->RTReport(os);}
  }
}
#endif
//*******************************************************************
#ifdef DEBUGREPORTS
void TAnmList::Report(ofstream &os)
{
  os<<"Start of Anm List"<<"\n";
  os<<"Last Anm Index #: "<<LastAddedElementNum<<"\n";
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    os<<"Index #"<<i<<"\n";
    TAnm *panm = GetAnmPtr(i);
    if (panm != NULL) {panm->Report(os);}
    else              {os<<"Anm Ptr was NULL"<<"\n";}
  }
}
#endif
//*******************************************************************
TAnmList::TAnmList(TOgleModeWindow *pmw,TSprite *psprite)
:TOgleList(pmw),
 NotifyRef(0),
 PSprite(psprite)
{
}
//*******************************************************************
TAnmList::~TAnmList()
{
}
//*******************************************************************
void TAnmList::SetAllCADRelative(TTriplet &trip)
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TAnm *panm = GetAnmPtr(i);
    if (panm != NULL)
    {
      panm->SetCADRelative(trip);
    }
  }
}
//*******************************************************************
bool TAnmList::AddAnm(TAnm *PNewAnm)
{
  if (PNewAnm == NULL) {return false;}
  PNewAnm->PAnmList = this;
  return (AddElement(PNewAnm));
}
//*******************************************************************
bool TAnmList::AddAnm(TFileMode NewFileMode,int16 NewScreenZ)
{
  TAnmData ad(0,0,NewScreenZ);
  TAnm *PTempAnm = new TAnm(PModeWin,NewFileMode,ad);
  PTempAnm->PAnmList = this;
  if (AddElement(PTempAnm))
  {
    return true;
  }
  else
  {
    delete PTempAnm;
    return false;
  }
}
//*******************************************************************
bool TAnmList::AddAnm(TFileMode NewFileMode,int16 NewScreenX,
                                            int16 NewScreenY,
                                            int16 NewScreenZ)
{
  TAnmData ad(NewScreenX,NewScreenY,NewScreenZ);

  TAnm *PTempAnm = new TAnm(PModeWin,NewFileMode,ad);
  PTempAnm->PAnmList = this;
  if (AddElement(PTempAnm))
  {
    return true;
  }
  else
  {
    delete PTempAnm;
    return false;
  }
}
//*******************************************************************
//dword TAnmList::GetLastFUC()
//{
//  dword EndingFUC = 0;
//  dword sf;
//  dword ef;
//
//  for (int16 i=0;i<=LastAddedElementNum;i++)
//  {
//    TAnm *panm = GetAnmPtr(i);
//    if (panm != NULL)
//    {
//      sf = panm->GetFUCOffset();
//      ef = sf + panm->Get_FUC_Length() - 1;
//      if (ef > EndingFUC)
//      {
//        EndingFUC = ef;
//      }
//    }
//  }
//  return EndingFUC;
//}
//*******************************************************************
void TAnmList::Start(dword StartFUC)
{
  Start();
  TOgleList::Start(StartFUC);

  #if defined(DEBUGANM)
  if (NotifyRef != (LastAddedElementNum + 1))
  {
    string s = "Studies show that this msg is caused by a sprite being started more than once.\n";
    s += "better check your GameLord's SetupSprites() and Init_GameType() functions.\n";
    s += "Sprite: ";
    s += PSprite->FileMode.Name.GetStr();
    s += "NotifyRef Problem.\n";
    s += "NotifyRef: ";
    char buffer[50];
    itoa(NotifyRef,buffer,10);
    s += buffer;
    s += "\nNum Anms: ";
    itoa(LastAddedElementNum,buffer,10);
    s += buffer;
    ::MessageBox(NULL,s.c_str(),"Warning! AnmList::NotifyRef",MB_OK|MB_TASKMODAL);
    for (int16 i=0;i<=LastAddedElementNum;i++)
    {
      TAnm *panm = GetAnmPtr(i);
      string msg = "Anm #";
      msg += itoa(i,buffer,10);
      msg += " is ";
      if (panm == NULL) {msg += "NULL";}
      else if (panm->GetStarted()) {msg += "Started";}
      else {msg += "not Started";}
      ::MessageBox(NULL,msg.c_str(),"Info - NotifyRef",MB_OK | MB_SYSTEMMODAL);
    }
  }
  assert(NotifyRef == (LastAddedElementNum + 1));
  #endif

  if (NotifyRef == 0) // we are either empty or didn't get any notifies,
                      // either way we don't have anything to do.
  {
    SetStopped();
    SpriteNotifyStop();
  }
}
//*******************************************************************
void TAnmList::Start()
{
  NotifyRef = 0; // just in case ?
  TOgleList::Start();
}
//*******************************************************************
void TAnmList::SpriteNotifyStop()
{
  if (PSprite != NULL) {PSprite->NotifyStop(this);}
}
//*******************************************************************
