//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  obmp.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/28/95    M.Perry     original baseline
//  2.0     8/21/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__obmp_h)
#include "obmp.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if !defined(OWL_COLOR_H)
#include <owl\color.h>
#endif

#if !defined(__odc_h)
#include "odc.h"
#endif

#include <assert.h>
//*******************************************************************
TBitmapFileHeader::TBitmapFileHeader()
{
}
//*******************************************************************
TBitmapFileHeader::~TBitmapFileHeader()
{
}
//*******************************************************************
BITMAPFILEHEADER &TBitmapFileHeader::operator=(BITMAPFILEHEADER &bitmapfileheader)
{
  if (&bitmapfileheader == &BitMapFileHeader) {return BitMapFileHeader;}
  BitMapFileHeader = bitmapfileheader;
  return BitMapFileHeader;
}
//*******************************************************************
//*******************************************************************
TBitmapInfo::TBitmapInfo()
:PBitMapInfo(NULL)
{
}
//*******************************************************************
TBitmapInfo::TBitmapInfo(int16 numquads)
:PBitMapInfo(NULL)
{
  CreateBITMAPINFO(numquads);
}
//*******************************************************************
TBitmapInfo::TBitmapInfo(BITMAPINFO *pbitmapinfo)
:PBitMapInfo(NULL)
{
  if (pbitmapinfo == NULL) {return;}

  int16 numquads = int16(pbitmapinfo->bmiHeader.biClrUsed);
  if (numquads == 0) {numquads = LookupNumQuads(pbitmapinfo->bmiHeader.biBitCount);}
  CreateBITMAPINFO(numquads);
  *GetHeader() = pbitmapinfo->bmiHeader;
  GetHeader()->biClrUsed = numquads;
  RGBQUAD *prgbquads = GetQuads();
  RGBQUAD *prgbquads2 = (RGBQUAD *)((byte *)(pbitmapinfo) + (word)pbitmapinfo->bmiHeader.biSize);
  for (int16 i=0;i<numquads;i++)
  {
    prgbquads[i] = prgbquads2[i];
  }
}
//*******************************************************************
TBitmapInfo::~TBitmapInfo()
{
  DestroyBITMAPINFO();
}
//*******************************************************************
bool TBitmapInfo::SetupBitmapInfoHeader(int16 numquads)
{
  BITMAPINFOHEADER *pbmih = GetHeader();
  if (pbmih == NULL) {return false;}
  pbmih->biSize = sizeof(BITMAPINFOHEADER);
  pbmih->biWidth = 0;
  pbmih->biHeight = 0;
  pbmih->biPlanes = 1;
  pbmih->biBitCount = 8;
  pbmih->biCompression = BI_RGB;
  pbmih->biSizeImage = 0;
  pbmih->biXPelsPerMeter = 0;
  pbmih->biYPelsPerMeter = 0;
  pbmih->biClrUsed = numquads;
  pbmih->biClrImportant = numquads;
  return true;
}
//*******************************************************************
bool TBitmapInfo::SetAllQuads(RGBQUAD rgbquad)
{
  if (GetHeader() == NULL) {return false;}
  int16 numquads = int16(GetHeader()->biClrUsed);
  RGBQUAD *prgbquads = GetQuads();
  for (int16 i=0;i<numquads;i++)
  {
    prgbquads[i] = rgbquad;
  }
  return true;
}
//*******************************************************************
bool TBitmapInfo::CreateBITMAPINFO(int16 numquads)
{
  if (PBitMapInfo != NULL) {DestroyBITMAPINFO();}
  PBitMapInfo = (BITMAPINFO *) new uint8[sizeof(BITMAPINFO) + (numquads - 1)*sizeof(RGBQUAD)];
  SetupBitmapInfoHeader(numquads);
  SetAllQuads(TRgbQuad(0,0,0));
  return true;
}
//*******************************************************************
bool TBitmapInfo::DestroyBITMAPINFO()
{
  delete [] PBitMapInfo;
  PBitMapInfo = NULL;
  return true;
}
//*******************************************************************
TBitmapInfo &TBitmapInfo::operator=(TPal &pal)
{
  if (pal.GetRGBQuadPalette() != NULL)
  {
    int16 numquads = pal.GetRGBQuadPalette()->GetNumQuads();

    if (GetHeader() != NULL)
    {
      BITMAPINFOHEADER bmih = *GetHeader(); // hold onto header info.
      CreateBITMAPINFO(numquads);
      RGBQUAD *prgbquads = GetQuads();
      for(int16 i=0;i<numquads;i++)
      {
        prgbquads[i] = pal.GetRGBQuadPalette()->GetQuad(i);
      }
      *GetHeader() = bmih;
      GetHeader()->biClrUsed = numquads;
      GetHeader()->biClrImportant = numquads;
    }
    else
    {
      CreateBITMAPINFO(numquads);
      RGBQUAD *prgbquads = GetQuads();
      for(int16 i=0;i<numquads;i++)
      {
        prgbquads[i] = pal.GetRGBQuadPalette()->GetQuad(i);
      }
    }
  }
  else
  {
    assert(false);
    // pal didn't have a rgbquadpalette, what do we do?
  }
  return *this;
}
//*******************************************************************
int16 TBitmapInfo::LookupNumQuads(int16 bitcount)
{
  int16 numquads = 0;
  switch (bitcount)
  {
    case 1: numquads = 2;break;
    case 4: numquads = 16;break;
    case 8: numquads = 256;break;
    case 24: numquads = 256;break;
  }
  return numquads;
}
//*******************************************************************
TBitmapInfo &TBitmapInfo::operator=(BITMAPINFO &bitmapinfo)
{
  if (&bitmapinfo == PBitMapInfo) {return *this;}

  int16 numquads = int16(bitmapinfo.bmiHeader.biClrUsed);
  if (numquads == 0) {numquads = LookupNumQuads(bitmapinfo.bmiHeader.biBitCount);}
  CreateBITMAPINFO(numquads);
  *GetHeader() = bitmapinfo.bmiHeader;
  GetHeader()->biClrUsed = numquads;
  RGBQUAD *prgbquads = GetQuads();
  RGBQUAD *prgbquads2 = (RGBQUAD *)((byte *)(&bitmapinfo) + (word)bitmapinfo.bmiHeader.biSize);
  for (int16 i=0;i<numquads;i++)
  {
    prgbquads[i] = prgbquads2[i];
  }
  return *this;
}
//*******************************************************************
//*******************************************************************
bool TBmp::WriteLocalHeader()
{
  return true;
}
//*******************************************************************
bool TBmp::WriteLocal()
{
  return true;
}
//*******************************************************************
string TBmp::AddExt()
{
  return string(".bmp");
}
//*******************************************************************
TBmp::TBmp()
:TGr(NULL),
 hBitmap(NULL),
 PBitmapInfo(NULL),
 PBitmapFileHeader(NULL)
{
}
//*******************************************************************
TBmp::TBmp(TOgleModeWindow *pmw,const TFileMode &fm,bool AutoRead,bool KeepBaseFile)
:TGr(pmw,fm),
 hBitmap(NULL),
 PBitmapInfo(NULL),
 PBitmapFileHeader(NULL)
{
  Init(fm,AutoRead,KeepBaseFile);
}
//*******************************************************************
TBmp::TBmp(TOgleModeWindow *pmw)
:TGr(pmw),
 hBitmap(NULL),
 PBitmapInfo(NULL),
 PBitmapFileHeader(NULL)
{
}
//*******************************************************************
TBmp::~TBmp()
{
  DeleteHBITMAP();
  delete PBitmapInfo;
  PBitmapInfo = NULL;
  delete PBitmapFileHeader;
  PBitmapFileHeader = NULL;
}
//*******************************************************************
bool TBmp::CreateHBITMAP()
{
  hBitmap = ::CreateBitmap(int(GetWidth()),int(GetHeight()),GetNumColorPlanes(),GetBitCount(),GrDrawData.GetBits());
  return (hBitmap != NULL);
}
//*******************************************************************
bool TBmp::DeleteHBITMAP()
{
  if (!IsHBITMAPValid()) {return true;}
  return ::DeleteObject(hBitmap);
}
//*******************************************************************
byte *TBmp::FindBits()
{
  if (PBitmapInfo == NULL) {return NULL;}
  byte huge *pb = ((byte huge *)GetRamPtr() + sizeof(BITMAPFILEHEADER) + PBitmapInfo->GetSizeStruct());
  return (byte *)pb;
}
//*******************************************************************
bool TBmp::ReadLocalHeader()
// reads BITMAPFILEHEADER
{
  BITMAPFILEHEADER *pbitmapfileheader = (BITMAPFILEHEADER *)GetPtr();
  assert(pbitmapfileheader);

  PBitmapFileHeader = new TBitmapFileHeader;
  *PBitmapFileHeader = *pbitmapfileheader;
  MajVer = 0;
  MinVer = 0;
  return true;
}
//*******************************************************************
bool TBmp::ReadLocal()
// reads BITMAPINFO (which includes: BITMAPINFOHEADER & RGBQUAD array.
{
  BITMAPINFO *pbitmapinfo = (BITMAPINFO *)((byte *)GetPtr() + sizeof(BITMAPFILEHEADER));
  PBitmapInfo = new TBitmapInfo(pbitmapinfo);
  GrDrawData.SetBits(FindBits());
  GrDrawData.CalcVRect(int16(PBitmapInfo->GetWidth()),int16(PBitmapInfo->GetHeight()));
//  GrDrawData.CalcVRect(int16(PBitmapInfo->GetWidth()-1),int16(PBitmapInfo->GetHeight()-1));
  GrDrawData.SetLinear(!PBitmapInfo->IsCompressed());
  return true;
}
//*******************************************************************
bool TBmp::BltTo(TOgleVideoDC &vdc,TOgleRect &clipvrect)
{
  TBmp *pbmp = vdc.GetBmp();
  if (pbmp == NULL) {return false;}
  if (!pbmp->GrDrawData.IsLinear()) {return false;}
  return BltTo(*pbmp,clipvrect);
}
//*******************************************************************
//#pragma intrinsic memcpy
bool TBmp::BltTo(TGr &gr,TOgleRect &clipvrect)
{
  byte huge *PSrcBits = GrDrawData.GetBits();
  byte huge *PDestBits = gr.GrDrawData.GetBits();
  if (PSrcBits == NULL || PDestBits == NULL) {return false;}
  bool topdown = gr.GrDrawData.IsTopDown();
  byte huge *PSrc = NULL;
  byte huge *PDest = NULL;

  TOgleRect VRect = GrDrawData.GetVRect();
  TOgleRect DestVRect = gr.GrDrawData.GetVRect();
  TOgleRect isvrect = VRect.Intersect(clipvrect);

  if (isvrect.IsEmpty()) {return true;} // no intersection.

  int16 width4 = NumScanBytes(VRect.GetWidth());
  int16 destwidth4 = NumScanBytes(DestVRect.GetWidth());
  TOglePoint vrecttl = VRect.GetTL();
  int16 vrecth = VRect.GetHeight();

  for (int16 y = int16(isvrect.bottom - 1);
       y >= isvrect.top;
       y--)
  {
    int16 x = isvrect.left;
    PSrc = GetPixelAddr(PSrcBits,VptToLpt(vrecttl,/*VRect.GetTL(),*/x,y),false,width4,vrecth);//VRect.GetHeight());
    PDest = GetPixelAddr(PDestBits,x,y,topdown,destwidth4,DestVRect.GetHeight());
    int16 xlen = isvrect.GetWidth();
    hmemcpy(PDest,PSrc,xlen);
  }
  return true;
}
//*******************************************************************

