//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ovideo.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     3/2/95     M.Perry      original baseline
//  2.0     8/22/97    M.Perry      32-bit conversion.
//********************************************************************
#if !defined(__ovideo_h)
#include "ovideo.h"
#endif

#if !defined(__oscrnmgr_h)
#include "oscrnmgr.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if !defined(__mstate_h)
#include "mstate.h"
#endif
DIAG_DEFINE_GROUP(VIDEO,1,0);
//*******************************************************************
TOgleVideo::TOgleVideo(TOgleModeWindow *pmw,TBmp &mbg,TPal &mpal,
                       TRenderMethod bgmethod,TRenderMethod grmethod)
:PVBmp(NULL),
 PVPal(NULL),
 PVDC(NULL),
 PBG(&mbg),
 PMasterPal(&mpal),
 PModeWin(pmw),
 Active(true),
 Initialized(false),
 PaintFrameDropsOk(false),//true),
 BGMethod(bgmethod),
 GrMethod(grmethod),
 #if defined(ENB_FUC_STEPS)
 RectsShowing(false),
 #endif
 WinSize(640,480)
{
}
//*******************************************************************
TOgleVideo::~TOgleVideo()
{
  DeInitialize();
}
//*******************************************************************
bool TOgleVideo::ChangeBG(TBmp *pbmp)
{
  PBG = pbmp;
  SetupVideoDC();
  return true;
}
//*******************************************************************
#pragma argsused
void TOgleVideo::ChangePal(TPal &pal)
{
  PMasterPal = &pal;
  DestroyVideoPal();
  CreateVideoPal();
}
//*******************************************************************
bool TOgleVideo::CreateCurrentList()
{
  if (GrMethod == rmSLOPPY)
  {
    PCurrentList = new TRectMergeList;
  }
  else if (GrMethod == rmEXACT)
  {
    PCurrentList = new TRectNoOverlapList;
  }
  else
  {
    PCurrentList = new TOgleRectList;
  }
  return true;
}
//*******************************************************************
void TOgleVideo::DestroyCurrentList()
{
  delete PCurrentList;
  PCurrentList = NULL;
}
//*******************************************************************
bool TOgleVideo::CreateRemovedList()
{
  PRemovedList = new TOgleRectList;
  return true;
}
//*******************************************************************
void TOgleVideo::DestroyRemovedList()
{
  delete PRemovedList;
  PRemovedList = NULL;
}
//*******************************************************************
bool TOgleVideo::CreateBGList()
{
  if (BGMethod == rmSLOPPY)
  {
    PBGList = new TRectMergeList;
  }
  else if (BGMethod == rmEXACT)
  {
    PBGList = new TRectNoOverlapList;
  }
  else
  {
    PBGList = new TOgleRectList;
  }
  return true;
}
//*******************************************************************
void TOgleVideo::DestroyBGList()
{
  delete PBGList;
  PBGList = NULL;
}
//*******************************************************************
bool TOgleVideo::CreateVideoBmp()
{
  return false;
}
//*******************************************************************
void TOgleVideo::DestroyVideoBmp()
{
  delete PVBmp;
  PVBmp = NULL;
}
//*******************************************************************
bool TOgleVideo::CreateVideoPal()
{
  return false;
}
//*******************************************************************
void TOgleVideo::DestroyVideoPal()
{
  delete PVPal;
  PVPal = NULL;
}
//*******************************************************************
bool TOgleVideo::CreateVideoDC()
{
  return false;
}
//*******************************************************************
void TOgleVideo::DestroyVideoDC()
{
  delete PVDC;
  PVDC = NULL;
}
//*******************************************************************
void TOgleVideo::SetupVideoDC()
{
  PBGList->Add(TOgleRect(TPoint(0,0),WinSize,true));
  RenderBG();
  GetModeWin()->Invalidate(false);
}
//*******************************************************************
void TOgleVideo::CleanupVideoDC()
{
}
//*******************************************************************
bool TOgleVideo::Initialize()
{
  if (IsInitialized()) {return true;}
  CreateCurrentList();
  CreateRemovedList();
  CreateBGList();
  CreateVideoDC();
  CreateVideoPal();
  CreateVideoBmp();
  SetupVideoDC();
  Initialized = true;
  return Initialized;
}
//*******************************************************************
bool TOgleVideo::DeInitialize()
{
  if (!IsInitialized()) {return true;}
  CleanupVideoDC();
  DestroyVideoBmp();
  DestroyVideoPal();
  DestroyVideoDC();
  DestroyBGList();
  DestroyRemovedList();
  DestroyCurrentList();
  Initialized = false;
  return true;
}
//*******************************************************************
TOglePaintDC *TOgleVideo::CreatePaintDC(TDC &dc)
{
  return (new TOglePaintDC(dc));
}
//*******************************************************************
void TOgleVideo::DestroyPaintDC(TOglePaintDC *ppdc)
{
  delete ppdc;
}
//*******************************************************************
#pragma argsused
void TOgleVideo::Paint(TDC& dc,bool erase, TRect& rect)
{
  TOglePaintDC *ppaintdc = CreatePaintDC(dc);

  // clip to TOgleRect(0,0,640,480)
  TOgleRect r(rect);
  Paint(r,*ppaintdc);
//  Paint(TOgleRect(rect),*ppaintdc);

  DestroyPaintDC(ppaintdc);
}
//*******************************************************************
void TOgleVideo::Paint()
{
  if (BgBigRect.IsEmpty()) {return;}

  #if defined(ENB_FUC_STEPS) // if we're single-stepping then force immediate paint.
  bool wasdropsok = IsDropsOk();
  if (PModeWin->GetState()->Is_FUC_Stepping())
  {
    SetDropsOk(false);
  }
  #endif
  PModeWin->InvalidateRect(BgBigRect);
  if (!IsDropsOk())
  {
    PModeWin->UpdateWindow();
  }
  #if defined(ENB_FUC_STEPS)
  SetDropsOk(wasdropsok);
  #endif
}
//*******************************************************************
void TOgleVideo::Update()
{
  #if defined (ENB_FUC_STEPS)
  if (RectsShowing) {UnShowRects();}
  #endif

  #ifdef DEBUGUPDATES
  PModeWin->DebugUpdate.StartProcessRects();
  #endif
  ProcessRects();
  #ifdef DEBUGUPDATES
  PModeWin->DebugUpdate.EndProcessRects();
  PModeWin->DebugUpdate.StartRenderBG();
  #endif
  RenderBG();
  #ifdef DEBUGUPDATES
  PModeWin->DebugUpdate.EndRenderBG();
  PModeWin->DebugUpdate.StartRender();
  #endif
  Render();
  #ifdef DEBUGUPDATES
  PModeWin->DebugUpdate.EndRender();
  #endif
  Paint();

  #if defined(ENB_FUC_STEPS)
//  if (PModeWin->PState->Is_FUC_Stepping())
  if (PModeWin->GetState() != NULL && PModeWin->GetState()->Is_FUC_Stepping())
  {
    ShowRects();
  }
  #endif
}
//*******************************************************************
#if defined(ENB_FUC_STEPS)
void TOgleVideo::ShowRects()
{
  TOgleWindowDC wdc(*PModeWin);
  TOgleRect *prect = PCurrentList->GetFirst();
  while (prect != NULL)
  {
    wdc.DrawFocusRect(*prect);
    prect = PCurrentList->GetNext();
  }
  RectsShowing = !RectsShowing;
}
#endif
//*******************************************************************
bool TOgleVideo::Activate()
{
  if (Active) {return false;}
  Active = true;
  return RealizePaletteFG();
}
//*******************************************************************
void TOgleVideo::DeActivate(bool palchange)
{
  if (Active || palchange) {RealizePaletteBG();}
  Active = false;
}
//*******************************************************************
bool TOgleVideo::RealizePaletteFG()
{
  TOgleWindowDC *pwdc = new TOgleWindowDC(*PModeWin);
  pwdc->SelectPal(*PVPal,false);
  pwdc->RealizePalette();
  pwdc->RestorePalette();
  delete pwdc;
  ::SendMessage(HWND_BROADCAST,WM_PALETTECHANGED,(WPARAM)PModeWin->HWindow,(LPARAM)0);
  GetModeWin()->Invalidate(); // don't think we need for fg.
  return true;
}
//*******************************************************************
bool TOgleVideo::RealizePaletteBG()
{
  TOgleWindowDC *pwdc = new TOgleWindowDC(*GetModeWin());
  pwdc->SelectPal(*PVPal,true);
  pwdc->RealizePalette();
  pwdc->RestorePalette();
  delete pwdc;
  GetModeWin()->Invalidate();
  return true;
}
//*******************************************************************
void TOgleVideo::Paint(TOgleRect &rect,TOglePaintDC &pdc)
{
  pdc.SelectPal(*PVPal,false);
  pdc.RealizePalette();
  #if defined(DEBUGVIDEO)
  bool rc =
  #endif
  pdc.BitBlt(rect,*PVDC);

}
//*******************************************************************
void TOgleVideo::ProcessRects()
{
  PBGList->Empty();
  PCurrentList->Empty();
  PRemovedList->AddToList(*PCurrentList);
  PRemovedList->AddToList(*PBGList);
  PRemovedList->Empty();
  GrList.AddChanged(*PCurrentList);
  GrList.AddChanged(*PBGList);
  GrList.ClearChanged();
  BgBigRect = PBGList->GetEnclosingRect();
  if (BgBigRect.IsEmpty()) {return;}
  TOgleClientDC(*PModeWin).AlignRect(BgBigRect); // align to dword boundaries.
  TOgleRect r(PModeWin->GetClientRect());
  BgBigRect.Intersect(r);
//  BgBigRect.Intersect((TOgleRect)PModeWin->GetClientRect()); // clip back to client window size if aligning went off window.
}
//*******************************************************************
void TOgleVideo::RenderBG()
{
  TOgleRect *prect = PBGList->GetFirst();
  while (prect != NULL)
  {                                                 
    PBG->BltTo(*PVDC,*prect);
    prect = PBGList->GetNext();
  }
}
//*******************************************************************
void TOgleVideo::Render()
{
  TOgleRect cliprect;
  TOgleRect vrect = PVBmp->GrDrawData.GetVRect();
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartGrListSort();
  #endif
  GrList.SortByZ(false);
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndGrListSort();
  int16 count = 0;
  #endif
  TOgleRect *prect = PCurrentList->GetFirst();
  while (prect != NULL)
  {
    TGr *pgr = GrList.GetFirst();
    while (pgr != NULL)
    {
      cliprect = prect->Intersect(vrect);    // clip dirty rect to video rect.
      if (pgr->GrDrawData.GetVRect().Contains(cliprect)) // make sure gr contains cliprect.
      {
        #if defined(DEBUGUPDATES)
        PModeWin->DebugUpdate.StartBltTo();
        PModeWin->DebugUpdate.StartBltToSingle(pgr->FileMode.Name.GetStr(),pgr->GrDrawData.GetFrameNum(),count);
        #endif
        pgr->BltTo(*PVDC,cliprect);
        #if defined(DEBUGUPDATES)
        PModeWin->DebugUpdate.EndBltToSingle(count);
        count++;
        PModeWin->DebugUpdate.EndBltTo();
        #endif
      }
      pgr = GrList.GetNext();
    }
    prect = PCurrentList->GetNext();
  }
}
//*******************************************************************
