
//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oskit.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     8/27/95    M.Perry    original baseline
//  2.0     8/24/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__oskit_h)
#include "oskit.h"
#endif

#if !defined(__ogobjsk_h)
#include "ogobjsk.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#include <assert.h>
//*******************************************************************
#ifdef GED_COMPILE
void TSkit::CriteriaReport(ofstream &os)
{
  os<<"\n";
  os<<"*********** Skit Name: "<<FileMode.Name.GetStr()<<" ********\n";

  if (PSkitCriteria != NULL)
  {
    PSkitCriteria->CriteriaReport(os);
  }
  else
  {                                        
    os<<"Skit was missing criteria\n";
  }
  os<<"End of criteria report for skit: "<<FileMode.Name.GetStr()<<"\n";
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TSkit::ReportSpriteUsage(ofstream &os,int16 gobjindex,int16 spriteindex)
{
  if (GetTable() == NULL)
  {
    os<<"Skit "<<FileMode.Name.GetStr()<<" was missing it's skittable\n";
  }
  else
  {
    GetTable()->ReportSpriteUsage(os,string(FileMode.Name.GetStr()),gobjindex,spriteindex);
  }
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
bool TSkit::IsSpriteUsedBySkit(int16 gobjindex,int16 spriteindex)
{
  if (GetTable() == NULL) {return false;}
  return GetTable()->IsSpriteUsedBySkit(gobjindex,spriteindex);
}
#endif
//*******************************************************************
#if defined(DEBUGSKIT)
void TSkit::SkitError(string msg)
{
  if (GetModeWin() != NULL)
  {
    PModeWin->MessageBox(msg.c_str(),"Skit Error",MB_OK);
  }
}
#endif
//*******************************************************************
void TSkit::SetMinVer()
{
  MinVer = 1;
}
//*******************************************************************
void TSkit::SetIndexes()
{
  if (GetTable() != NULL)
  {
    GetTable()->SetIndexes();
    Set_FUC_Length(GetCalcFUCLength());
  }
  #if defined(OGLE_DEBUG_SKITS)
  else
  {
    string title = "Warning - TSkit";
    string pstrmsg[2];
    pstrmsg[0] = "TSkit::SetIndexes()";
    pstrmsg[1] = "GetTable() == NULL";

    if (GetModeWin() != NULL)
    {
      GetModeWin()->ScrollMsg(title,pstrmsg,2,true);
    }
    else
    {
      string s;
      for (int16 i=0;i<2;i++)
      {
        s += pstrmsg[i] + "\n";
        ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
      }
    }
  }
  #endif
}
//*******************************************************************
void TSkit::Slenderize()
{
  if (GetTable() == NULL) {return;}
  int16 num = GetTable()->GetNumValidCells();
  PSkitDataList = new TSkitDataList(this,POwnerGObj,num);
  GetTable()->AddToDataList(PSkitDataList);
  delete PSkitTable;
  PSkitTable = NULL;
  SetSlenderized();
}
//*******************************************************************
void TSkit::Start()
{
  TOgleBase::Start();
}
//*******************************************************************
void TSkit::Activate(dword StartFUC)
{
  Start(StartFUC);
}
//*******************************************************************
const char *TSkit::GetIDChars() const
{
  return "SKIT";
}
//*******************************************************************
string TSkit::AddExt()
{
  return string(".sk");
}
//*******************************************************************
bool TSkit::ReadLocal1_0()
{
  if (!Readint16(&WhatType)) {return false;}
  PSkitTable = new TSkitTable(PModeWin,FileMode,POwnerGObj,GetPtr());
  FileOffset += PSkitTable->ReadSize;

  #ifdef GED_COMPILE
  if (GetTable()->GetFrameCount() <= 0)
  {
    string msg = "Reading skit '" + FileMode.Name.GetStr() + "' with zero rows in table.";
    ::MessageBox(NULL,msg.c_str(),"Warning !!  Skit Read",MB_OK|MB_TASKMODAL);
  }
  #endif

  PSkitCriteria = new TSkitCriteria(GetPtr());
  FileOffset += PSkitCriteria->ReadSize;
  return true;
}
//*******************************************************************
bool TSkit::ReadLocal1_1()
{
  if (!Readint16(&WhatType)) {return false;}
  bool KnownEmptyTable;
  if (!Readbool(&KnownEmptyTable)) {return false;}
  SetKnownEmptyTable(KnownEmptyTable);

  PSkitTable = new TSkitTable(PModeWin,FileMode,POwnerGObj,GetPtr());
  FileOffset += PSkitTable->ReadSize;

  #ifdef GED_COMPILE
  if (GetTable()->GetFrameCount() <= 0 && !GetKnownEmptyTable())
  {
    string msg = "Reading skit '" + FileMode.Name.GetStr() + "' with zero rows in table.";
    ::MessageBox(NULL,msg.c_str(),"Warning !!  Skit Read",MB_OK|MB_TASKMODAL);
  }
  #endif

  PSkitCriteria = new TSkitCriteria(GetPtr());
  FileOffset += PSkitCriteria->ReadSize;
  return true;
}
//*******************************************************************
bool TSkit::WriteLocal()
{
  if (!Writeint16(&WhatType)) {return false;}
  bool KnownEmptyTable = GetKnownEmptyTable();
  if (!Writebool(&KnownEmptyTable)) {return false;}
  if (GetTable()->GetFrameCount() <= 0 && !GetKnownEmptyTable())
  {
    string msg = "Saving skit '" + FileMode.Name.GetStr() + "' with zero rows in table.";
    ::MessageBox(NULL,msg.c_str(),"Warning !! Skit Write",MB_OK|MB_TASKMODAL);
  }
  if (!GetTable()->Write(PWriteBuf)) {return false;}

  if (!GetSkitCriteria()->Write(PWriteBuf)) {return false;}
  return true;
}
//*******************************************************************
bool TSkit::ReadLocal()
{
  switch (MinVer)
  {
    case 1: return ReadLocal1_1();
    default:return ReadLocal1_0();
  }
}
//*******************************************************************
dword TSkit::GetCalcFUCLength()
{
  if (GetTable() == NULL) {return 0;}
  return GetTable()->GetCalcFUCLength();
}
//*******************************************************************
TSkit::TSkit(TOgleModeWindow *pmw,TGObjSk *PNewGObj)
:TOgleBase(pmw),
 POwnerGObj(PNewGObj),
 PSkitDataList(NULL)
{
  PSkitTable = new TSkitTable(PModeWin,POwnerGObj);
  PSkitCriteria = new TSkitCriteria;
}
//*******************************************************************
TSkit::TSkit(TOgleModeWindow *pmw,TFileMode NewFileMode,TGObjSk *PNewGObj,
             bool AutoRead,bool KeepBaseFile)
:TOgleBase(pmw),
 POwnerGObj(PNewGObj),
 PSkitTable(NULL),
 PSkitCriteria(NULL),
 PSkitDataList(NULL)
// this ctor reads data from file.
{
  Init(NewFileMode,AutoRead,KeepBaseFile);

  #if !defined(MULTIGOBJSPRITES) && !defined(GED_COMPILE)
  Set_FUC_Length(GetCalcFUCLength());
  Slenderize();
  #endif
}
//*******************************************************************
TSkit::TSkit(TOgleModeWindow *pmw,string &skitname,TGObjSk *PNewGObj,
             void *PReadData,bool AutoRead)
:TOgleBase(pmw),
 POwnerGObj(PNewGObj),
 PSkitTable(NULL),
 PSkitCriteria(NULL),
 PSkitDataList(NULL)
// this ctor reads data from ptr.
{

  // set up name, because we'll need for searches.
  FileMode = POwnerGObj->FileMode;
  FileMode.Name = skitname;
//  FileMode.Name += AddExt();
  FileMode.Name.SetExt(AddExt());
  FileMode.SetNone();

  Init(PReadData,AutoRead);

  #if !defined(MULTIGOBJSPRITES) && !defined(GED_COMPILE)
  Set_FUC_Length(GetCalcFUCLength());
  Slenderize();
  #endif
}
//*******************************************************************
TSkit::~TSkit()
{
  if (PSkitCriteria != NULL)
  {
    delete PSkitCriteria;
    PSkitCriteria = NULL;
  }
  if (PSkitTable != NULL)
  {
    delete PSkitTable;
    PSkitTable = NULL;
  }
  if (PSkitDataList != NULL)
  {
    delete PSkitDataList;
    PSkitDataList = NULL;
  }
}
//*******************************************************************
bool TSkit::Save()
{
  if (!Write())
  {
    assert(false);
    return false;
  }
  return true;
}
//*******************************************************************
void TSkit::Start(dword StartFUC)
{
  Start();

  #if defined(DEBUGSKIT)
  string s = "Skit: ";
  s += FileMode.Name.GetStr();
  s += "\n";
  bool slender = IsSlenderized();
  if (slender && GetTable() != NULL)
  {
    s += "Was Slenderized but still has valid skit table";
    SkitError(s);
  }
  if (slender && GetDataList() == NULL)
  {
    s += "Was Slenderized but doesn't have valid data list";
    SkitError(s);
  }
  #endif

  if (IsSlenderized())
  {
    GetDataList()->Start(StartFUC);
  }
  else
  {
    GetTable()->Start(StartFUC);
  }
}
//*******************************************************************
bool TSkit::CanActivate(dword StartFUC)
{
  #if defined(OGLE_DEBUG_SKITS)
  string s = "Skit: ";
  s += FileMode.Name.GetStr();
  s += "\n";
  bool slender = IsSlenderized();
  if (slender && GetTable() != NULL)
  {
    s += "Was Slenderized but still has valid skit table";
    SkitError(s);
  }
  if (slender && GetDataList() == NULL)
  {
    s += "Was Slenderized but doesn't have valid data list";
    SkitError(s);
  }
  #endif

  if (IsSlenderized())
  {
    return GetDataList()->CanActivate(StartFUC);
  }
  else
  {
    return (GetTable()->CanActivate(StartFUC));
  }
}
//*******************************************************************
#pragma argsused
bool TSkit::CriteriaCheck(dword StartFUC)
{
  int16 NumQ = GetSkitCriteria()->GetNumQuestions();
  int16 Ans = 0;
  int16 QAns = 0;
  for (int16 i=0;i<NumQ;i++)
  {
    Ans = GetSkitCriteria()->GetNeededAnswer(i);
    if (Ans != 0)
    {
      QAns = POwnerGObj->AskCriteriaQuestion(i);
      if (QAns != 0 && QAns != Ans) {return false;}
    }
  }
  return true;
}
//*******************************************************************
void TSkit::SetCriteriaAns(int16 QNum,int16 QAns)
{
  GetSkitCriteria()->SetNeededAnswer(QNum,QAns);
}
//*******************************************************************
int16 TSkit::GetCriteriaAns(int16 QNum)
{
  return GetSkitCriteria()->GetNeededAnswer(QNum);
}
//*******************************************************************

