//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  oskitlst.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     8/27/95    M.Perry    original baseline
//  2.0     8/24/97    M.Perry    32-bit conversion.
//  2.1     12/6/97    M.Perry    filename mod's
//  2.2     06/08/99   M.Perry    added GED_COMPILE define.
//********************************************************************
#if !defined(__oskitlst_h)
#include "oskitlst.h"
#endif

#if !defined(__ofilemgr_h)
#include "ofilemgr.h"
#endif

#if defined(OGLE_DEBUG_SKITS) && !defined(__omodewin_h)
#include "omodewin.h"
#endif

#include <assert.h>
//*******************************************************************
#ifdef GED_COMPILE
void TSkitList::CriteriaReport(ofstream &os)
{
  os<<"******************************************************\n";
  os<<"Skit list name: "<<FileMode.Name.GetStr()<<"\n";
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSkit *pskit = GetSkitPtr(i);
    if (pskit != NULL) {pskit->CriteriaReport(os);}
    else {os<<"Skit was not there for index #"<<i<<"\n";}
  }
  os<<"******************************************************\n";
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TSkitList::ReportSpriteUsage(ofstream &os,int16 gobjindex,int16 skitindex,int16 spriteindex)
{
  if (skitindex == -1)
  {
    int16 numskits = GetLastNumSkits();
    for (int16 i=0;i<=numskits;i++)
    {
      TSkit *pskit = GetSkitPtr(i);
      if (pskit == NULL)
      {
        os<<"Skit was not there for index #"<<i<<"\n";
      }
      else
      {
        pskit->ReportSpriteUsage(os,gobjindex,spriteindex);
      }
    }
  }
  else
  {
    TSkit *pskit = GetSkitPtr(skitindex);
    if (pskit == NULL)
    {
      os<<"Skit was not there for index #"<<skitindex<<"\n";
    }
    else
    {
      pskit->ReportSpriteUsage(os,gobjindex,spriteindex);
    }
  }
}
#endif
//*******************************************************************
#ifdef GED_COMPILE
bool TSkitList::IsSpriteUsedBySkit(int16 gobjindex,int16 spriteindex,int16 &usedski)
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSkit *pskit = GetSkitPtr(i);
    if (pskit != NULL)
    {
      if (pskit->IsSpriteUsedBySkit(gobjindex,spriteindex))
      {
        usedski = i;
        return true;
      }
    }
  }
  return false;
}
#endif
//*******************************************************************
const char *TSkitList::GetIDChars() const
{
  return "KLST";
}
//*******************************************************************
//void TSkitList::SetMinVer()
//{
//  MinVer = 1;
//}
//*******************************************************************
void TSkitList::SetMajVer()
{
  MajVer = 2;
}
//*******************************************************************
string TSkitList::AddExt()
{
  return string(".skl");
}
//*******************************************************************
void TSkitList::SetIndexes()
{
  TOgleList::SetIndexes();
}
//*******************************************************************
void TSkitList::SetIndexes(int16 skitlistindex)
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSkit *ps = GetSkitPtr(i);
    if (ps != NULL)
    {
      ps->SetIndexes();
    }
    #if defined(OGLE_DEBUG_SKITS)
    else
    {
      char buffer[50];
      string title = "Warning - Skits";
      string pstrmsg[2];
      pstrmsg[0] = "TSkitList::SetIndexes(int16 skitlistindex";
      itoa(skitlistindex,buffer,10);
      pstrmsg[1] = "GetSkitPtr(";
      pstrmsg[1] += buffer;
      pstrmsg[1] += ") == NULL";
      if (GetModeWin() != NULL)
      {
        GetModeWin()->ScrollMsg(title,pstrmsg,2,true);
      }
      else
      {
        string s;
        for (int16 i = 0;i<2;i++)
        {
          s += pstrmsg[i];
          s += "\n";
        }
        ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
      }
    }
    #endif
  }

  PSkitPortfolio->SetIndexes(skitlistindex);
}
//*******************************************************************
TSkitList::TSkitList(TOgleModeWindow *pmw,TGObjSk *PNewGObj)
:TOgleList(pmw),
 POwnerGObj(PNewGObj)
{
  PSkitPortfolio = new TSkitPortfolio(POwnerGObj);
}
//*******************************************************************
TSkitList::TSkitList(TOgleModeWindow *pmw,TFileMode NewFileMode,TGObjSk *PNewGObj,
                     bool AutoRead,bool KeepBaseFile)
:TOgleList(pmw),
 POwnerGObj(PNewGObj)
{
  Init(NewFileMode,AutoRead,KeepBaseFile);
}
//*******************************************************************
TSkitList::~TSkitList()
{
  if (PSkitPortfolio != NULL)
  {
    delete PSkitPortfolio;
    PSkitPortfolio = NULL;
  }
}
//*******************************************************************
bool TSkitList::DeleteSkit(string skitfname)
{
  int16 si = GetSkitIndex(skitfname);
  if (si < 0) {return false;}
  return RemoveElement(si);
}
//*******************************************************************
bool TSkitList::StartCriteriaSkit(int16 skitindex,dword &StartFUC,bool desperate,int16 fucdelta)
{
  if (!CriteriaCheck(skitindex,StartFUC)) {return false;}  // criteria bad answer.

  // start if possible, but if not then no sweat.
  if (!desperate) {return StartSkit(skitindex,StartFUC);}

  // sweating !! Trying to find a starting time for a good criteria match.
  dword ef = dword(StartFUC + fucdelta);
  for (dword fuc = StartFUC;fuc < ef;fuc++)
  {
    if (StartSkit(skitindex,fuc))
    {
      StartFUC = fuc;
      return true;
    }
  }

  // damn !! Couldn't find a starting time up to fucdelta in the future for this
  // criteria matching skit.
  return false;
}
//*******************************************************************
bool TSkitList::LoopThruList(dword &StartFUC,dword &FUCLen,int16 &skitindex,
                             int16 portfoliolist,bool desperate,int16 fucdelta)
{
  bool oneitemflag = true;
  skitindex = GetNextPortfolioSkitIndex(portfoliolist);
  for (int16 startposition = GetPortfolioListCurrentPosition(portfoliolist);
       ((startposition != GetPortfolioListCurrentPosition(portfoliolist)) ||
        oneitemflag);
       skitindex = GetNextPortfolioSkitIndex(portfoliolist))
  {
    oneitemflag = false;
    if (skitindex >= 0)
    {
      if (StartCriteriaSkit(skitindex,StartFUC,desperate,fucdelta))
      {
        TSkit *ps = GetSkitPtr(skitindex);
        if (ps == NULL)
        {
          #if defined(OGLE_DEBUG_SKITS)
          string title = "Warning - Skits";
          string pstrmsg[2];
          pstrmsg[0] = "TSkitList::LoopThruList()";
          pstrmsg[1] = "GetSkitPtr(skitindex) == NULL";
          if (GetModeWin() != NULL)
          {
            GetModeWin()->ScrollMsg(title,pstrmsg,2,true);
          }
          else
          {
            string s;
            for (int16 i=0;i<2;i++)
            {
              s += pstrmsg[i];
              s += "\n";
            }
            ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
          }
          #endif
          return false;
        }
        FUCLen = ps->Get_FUC_Length();
        return true;
      }
    }
//    #if defined(OGLE_DEBUG_SKITS)
//    else
//    {
//      string title = "Warning - Skits";
//      string pstrmsg[2];
//      pstrmsg[0] = "TSkitList::LoopThruList()";
//      pstrmsg[1] = "SkitIndex < 0";
//      if (GetModeWin() != NULL)
//      {
//        GetModeWin()->ScrollMsg(title,pstrmsg,2,true);
//      }
//      else
//      {
//        string s;
//        for (int16 i = 0;i< 2;i++)
//        {
//          s += pstrmsg[i] + "\n";
//        }
//        ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
//      }
//    }
//    #endif
  }
  return false;
}
//*******************************************************************
bool TSkitList::ChooseStartSkit(dword &StartFUC,dword &FUCLen,int16 &skitindex,int16 fucdelta)
{
  int16 whichlist;
  if (TryRandomList(StartFUC,FUCLen,skitindex,whichlist)) {return true;}

  if (whichlist != GetLastPFListNum())
  {
    if (TryLastList(StartFUC,FUCLen,skitindex)) {return true;}
  }
  bool rc = TryLastList(StartFUC,FUCLen,skitindex,true,fucdelta);
  #if defined(OGLE_DEBUG_SKITS)
  if (!rc)
  {
    string title = "Warning - Possible Skit problem";
    string pstrmsg[3];
    pstrmsg[0] = "TSkitList::ChooseStartSkit()";
    pstrmsg[1] = "rc == false";
    pstrmsg[2] = "SkitList Name: " + FileMode.Name.GetStr();
    if (GetModeWin() != NULL)
    {
      GetModeWin()->ScrollMsg(title,pstrmsg,3,true);
    }
    else
    {
      string s;
      for (int16 i=0;i<3;i++)
      {
        s += pstrmsg[i];
        s += "\n";
      }
      ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
    }
  }
  #endif
  return rc;
}
//*******************************************************************
dword TSkitList::GetSkitLength(int16 SkitIndex)
{
  TSkit *ps = GetSkitPtr(SkitIndex);
  if (ps == NULL) {return 0;}
  return ps->Get_FUC_Length();
}
//*******************************************************************
#if defined(SUPPORT_OLD_VER)
bool TSkitList::ReadLocal1_0()
{
  // start filename mod's - MRP v2.1
  TOgleFileName ofn;
  string dirpath = GetFileMgr()->PDat->GetDirName(FileMode.GetDirIndex());
  ofn.SetDirPath(dirpath);
  //end

  if (!(Readuint16(&NumSkits))) {return false;}

  string str;
  for (int16 i=0;i<NumSkits;i++)
  {
    if (!ReadFNameString(str)) {return false;}
    // start filename mod's - MRP v2.1
    ofn.SetFName(str);
    ofn.SetExt(string(".sk"));
    str = ofn.GetStr();
    // end
    AddElement(CreateSkit(str));
  }
//  PSkitPortfolio = new TSkitPortfolio(POwnerGObj,GetFName(),GetPtr());
  PSkitPortfolio = new TSkitPortfolio(POwnerGObj,FileMode.Name.GetStr(),GetPtr());
  FileOffset += PSkitPortfolio->ReadSize;

  return true;
}
#endif
//*******************************************************************
#if defined(SUPPORT_OLD_VER)
bool TSkitList::ReadLocal1_1()
{
  // start filename mod's - MRP v2.1
  TOgleFileName ofn;
  string dirpath = GetFileMgr()->PDat->GetDirName(FileMode.GetDirIndex());
  ofn.SetDirPath(dirpath);
  // end

  #ifndef GED_COMPILE
  string *names;
  #endif

  if (!(Readuint16(&NumSkits))) {return false;}
  GrowBy(NumSkits); // MRP 12/12/96
  #ifndef GED_COMPILE
  names = new string [NumSkits];
  #endif

  string str;
  for (int16 i=0;i<NumSkits;i++)
  {
    if (!ReadFNameString(str)) {return false;}

    // start filename mod's - MRP v2.1
    ofn.SetFName(str);
    ofn.SetExt(string(".sk"));
    str = ofn.GetStr();
    // end

    #ifdef GED_COMPILE
    AddElement(CreateSkit(str));
    #else
    names[i] = str;
    #endif
  }
//  PSkitPortfolio = new TSkitPortfolio(POwnerGObj,GetFName(),GetPtr());
  PSkitPortfolio = new TSkitPortfolio(POwnerGObj,FileMode.Name.GetStr(),GetPtr());
  FileOffset += PSkitPortfolio->ReadSize;

  // the following reads all skits out of single skitlist file.
  #ifndef GED_COMPILE
  for (int16 j=0;j<NumSkits;j++)
  {
    TSkit *PSkit = new TSkit(PModeWin,names[j],POwnerGObj,GetPtr());
    FileOffset += PSkit->ReadSize;
    AddElement(PSkit);
  }
  delete [] names;
  #endif

  Sort();

  return true;
}
#endif
//*******************************************************************
bool TSkitList::ReadLocal2_0()
{
  #ifndef GED_COMPILE
  string *names;
  #endif

  if (!(Readuint16(&NumSkits))) {return false;}
  GrowBy(NumSkits); // MRP 12/12/96
  #ifndef GED_COMPILE
  names = new string [NumSkits];
  #endif

  string str;
  for (int16 i=0;i<NumSkits;i++)
  {
    if (!ReadFileName(str)) {return false;}

    #ifdef GED_COMPILE
    AddElement(CreateSkit(str));
    #else
    names[i] = str;
    #endif
  }
//  PSkitPortfolio = new TSkitPortfolio(POwnerGObj,GetFName(),GetPtr());
  PSkitPortfolio = new TSkitPortfolio(POwnerGObj,FileMode.Name.GetStr(),GetPtr());
  FileOffset += PSkitPortfolio->ReadSize;

  // the following reads all skits out of single skitlist file.
  #ifndef GED_COMPILE
  for (int16 j=0;j<NumSkits;j++)
  {
    TSkit *PSkit = new TSkit(PModeWin,names[j],POwnerGObj,GetPtr());
    FileOffset += PSkit->ReadSize;
    AddElement(PSkit);
  }
  delete [] names;
  #endif

  Sort();

  return true;
}
//*******************************************************************
bool TSkitList::ReadLocal()
{
  switch (MajVer)
  {
    case 1:
           switch (MinVer)
           {
             // MRP - 06/08/99 - ver2.2
             #if defined(SUPPORT_OLD_VER)
             case 0:return ReadLocal1_0();
             case 1:return ReadLocal1_1();
             #endif
             default: return false;
           }
    case 2: return ReadLocal2_0();
    default: return false;
  }
}
//*******************************************************************
bool TSkitList::WriteLocal()
{
//  NumSkits = int16(LastAddedElementNum + 1);
//  if (!Writeuint16(&NumSkits)) {return false;}
//  for (int16 i = 0;i<=LastAddedElementNum;i++)
//  {
//    string tempStr = List[i]->GetFName();
//    if (!WriteFNameString(tempStr)) {return false;}
//  }
//  if (!PSkitPortfolio->Write(PWriteBuf)) {return false;}
//
//  // will write skits to end of skitlist file.
//  for (int16 j=0;j<=LastAddedElementNum;j++)
//  {
//    if (GetSkitPtr(j) == NULL) {return false;}
//    if (!GetSkitPtr(j)->Write(PWriteBuf)) {return false;}
//  }
//  return true;

  NumSkits = int16(LastAddedElementNum + 1);
  if (!Writeuint16(&NumSkits)) {return false;}
  for (int16 i = 0;i<=LastAddedElementNum;i++)
  {
//    string tempStr = List[i]->GetFName();
    string tempStr = List[i]->FileMode.Name.GetStr();
    if (!WriteFileName(tempStr)) {return false;}
  }
  if (!PSkitPortfolio->Write(PWriteBuf)) {return false;}

  // will write skits to end of skitlist file.
  for (int16 j=0;j<=LastAddedElementNum;j++)
  {
    if (GetSkitPtr(j) == NULL) {return false;}
    if (!GetSkitPtr(j)->Write(PWriteBuf)) {return false;}
  }
  return true;
}
//*******************************************************************
bool TSkitList::Save()
{
  for (int16 i = 0;i<=LastAddedElementNum;i++)
  {
    TSkit *ps = GetSkitPtr(i);
    if (ps != NULL) {ps->Save();}
  }
  Write();
  return true;
}
//*******************************************************************
TSkit *TSkitList::CreateSkit(const string &SkitName)
{
  TFileMode FM(SkitName,
               TFileMode::fm_None,
               TFileMode::fm_Immed,
               FileMode.GetDirIndex());
  TSkit *PSkit = new TSkit(PModeWin,FM,POwnerGObj);
  return PSkit;
}
//*******************************************************************
bool TSkitList::SetSkitIndexTo(int16 SkitIndex,TSkit *PSkit)
{
  if ((SkitIndex > LastAddedElementNum) || (SkitIndex < 0)) {return false;}
  List[SkitIndex] = PSkit;
  return true;
}
//*******************************************************************
TSkit *TSkitList::SwapSkits(int16 SkitIndex,TSkit *PSkit)
{
  TSkit *PTempSkit = GetSkitPtr(SkitIndex);
  if (PTempSkit == NULL) {return NULL;}
  if (PSkit == NULL) {return NULL;}
  if (!SetSkitIndexTo(SkitIndex,PSkit)) {return NULL;}
  return PTempSkit;
}
//*******************************************************************
void TSkitList::Slenderize()
{
  PSkitPortfolio->Slenderize();
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSkit *pskit = GetSkitPtr(i);
    if (pskit != NULL) {pskit->Slenderize();}
  }
}
//*******************************************************************

