//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osktcrit.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     9/7/95     M.Perry    original baseline
//  2.0     8/24/97    M.Perry    32-bit conversion.
//********************************************************************
#if !defined(__osktcrit_h)
#include "osktcrit.h"
#endif

#if !defined(__oapp_h)
#include "oapp.h"
#endif

#if !defined(__ogamefw_h)
#include "ogamefw.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if defined(GED_COMPILE)
#if !defined(__omode_h)
  #include "omode.h"
#endif
#endif
//*******************************************************************
#ifdef GED_COMPILE
void TSkitCriteria::CriteriaReport(ofstream &os)
{
  int16 numnotdontcare = 0;
//  int16 idnum = GetFW()->GetModeWin()->PMode->GetStringTableID();
  int16 idnum = GetFW()->GetModeWin()->GetMode()->GetStringTableID();
  for (int16 i = 0;i< GetNumQuestions();i++)
  {
    int16 ans = GetNeededAnswer(i);
    if (ans != 0)
    {
      numnotdontcare++;
      string qstr = GetResString(idnum + i);
      os<<"Question # "<<i<<" :"<<qstr.c_str()<<" --> ";
      if (ans < 0) {os<<"No \n";}
      else {os<<"Yes\n";}
    }
  }
  if (numnotdontcare == 0)
  {
    os<<"All questions are Don't care\n";
  }
}
#endif
//*******************************************************************
const char *TSkitCriteria::GetIDChars() const
{
  return "CRIT";
}
//*******************************************************************
bool TSkitCriteria::ReadLocal()
{
  int16 NumUINTS;
  if (!Readint16(&NumUINTS)) {return false;}
  uint16 temp;
  for (int16 i = 0;i<NumUINTS;i++)
  {
    if (!Readuint16(&temp)) {return false;}
    SetData(i,temp);
  }
  return true;
}
//*******************************************************************
bool TSkitCriteria::WriteLocal()
{
  int16 NumUINTS = NUMQUESTIONUINTS;
  if (!Writeint16(&NumUINTS)) {return false;}
  uint16 temp;
  for (int16 i=0;i<NumUINTS;i++)
  {
    temp = GetData(i);
    if (!Writeuint16(&temp)) {return false;}
  }
  return true;
}
//*******************************************************************
TSkitCriteria::TSkitCriteria()
:TOgleBase(NULL)
{
  ClearData();
}
//*******************************************************************
TSkitCriteria::TSkitCriteria(void *PReadData,bool AutoRead)
:TOgleBase(NULL)
{
  ClearData();
  Init(PReadData,AutoRead);
}
//*******************************************************************
TSkitCriteria::TSkitCriteria(const TSkitCriteria &sc)
:TOgleBase(NULL)
{
  int16 NumQ = sc.GetNumQuestions();
  for (int16 i=0;i<NumQ;i++)
  {
    SetData(i,sc.GetData(i));
  }
}
//*******************************************************************
TSkitCriteria::~TSkitCriteria()
{
}
//*******************************************************************
void TSkitCriteria::SetData(int16 dataindex,uint16 NewData)
{
  if (dataindex > NUMQUESTIONUINTS) {return;}
  if (dataindex < 0) {return;}

  Data[dataindex].eight_twobits = NewData;
}
//*******************************************************************
void TSkitCriteria::ClearData()
{
  for (int16 i= 0;i<NUMQUESTIONUINTS;i++)
  {
    Data[i].eight_twobits = 0;
  }
}
//*******************************************************************
int16 TSkitCriteria::Get2BitInt(int16 dataindex,int16 ModNum)
{
  switch (ModNum)
  {
    case 0: return int16(Data[dataindex].cbf.Mod0);
    case 1: return int16(Data[dataindex].cbf.Mod1);
    case 2: return int16(Data[dataindex].cbf.Mod2);
    case 3: return int16(Data[dataindex].cbf.Mod3);
    case 4: return int16(Data[dataindex].cbf.Mod4);
    case 5: return int16(Data[dataindex].cbf.Mod5);
    case 6: return int16(Data[dataindex].cbf.Mod6);
    case 7: return int16(Data[dataindex].cbf.Mod7);
    default: return 0;
  }
}
//*******************************************************************
void TSkitCriteria::Set2BitInt(int16 dataindex,int16 ModNum,int16 QuestionAns)
{
  switch (ModNum)
  {
    case 0: Data[dataindex].cbf.Mod0 = QuestionAns;return;
    case 1: Data[dataindex].cbf.Mod1 = QuestionAns;return;
    case 2: Data[dataindex].cbf.Mod2 = QuestionAns;return;
    case 3: Data[dataindex].cbf.Mod3 = QuestionAns;return;
    case 4: Data[dataindex].cbf.Mod4 = QuestionAns;return;
    case 5: Data[dataindex].cbf.Mod5 = QuestionAns;return;
    case 6: Data[dataindex].cbf.Mod6 = QuestionAns;return;
    case 7: Data[dataindex].cbf.Mod7 = QuestionAns;return;
    default: return;
  }
}
//*******************************************************************
int16 TSkitCriteria::GetNeededAnswer(int16 QuestionNum)
{
  // -1 -> Needs to be false
  //  0 -> Don't care
  //  1 -> Needs to be true

  if (QuestionNum >= NUMQUESTIONS) {return 0;}
  int16 dataIndex = int16(QuestionNum / 8);
  if (dataIndex >= NUMQUESTIONUINTS) {return 0;}
  int16 TwoBitIndex = int16(QuestionNum % 8);

  return Get2BitInt(dataIndex,TwoBitIndex);
}
//*******************************************************************
void TSkitCriteria::SetNeededAnswer(int16 QuestionNum,int16 QuestionAns)
{
  if (QuestionNum >= NUMQUESTIONS) {return;}
  int16 dataIndex = int16(QuestionNum / 8);
  if (dataIndex >= NUMQUESTIONUINTS) {return;}
  int16 TwoBitIndex = int16(QuestionNum % 8);

  Set2BitInt(dataIndex,TwoBitIndex,QuestionAns);
}
//*******************************************************************

