//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:   omusmci.cpp
//    Project Name:   Ogle
// Original Author:   Dave Anderson
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0    06/14/95   D.Anderson     original baseline
//         10/21/96   M.Perry        under modification.
//  2.0    08/23/97   M.Perry        32-bit conversion.
//  2.2    06/01/99   M.Perry        fixed focus problem.
//********************************************************************
#if !defined(__omusmci_h)
#include "omusmci.h"
#endif

#if !defined(__midi_h)
#include "omidi.h"
#endif

#if !defined(__osndmgr_h)
#include "osndmgr.h"
#endif
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TChildMidi,TOgleHideWin)//TDialog)
	EV_MESSAGE(MM_MCINOTIFY,EvMMMCINotify),
END_RESPONSE_TABLE;
//*******************************************************************
TChildMidi::TChildMidi(TWindow *pparent)
:TOgleHideWin(pparent,string("Hidden Midi Window"))
{
}
//*******************************************************************
TChildMidi::~TChildMidi()
{
}
//*******************************************************************
LRESULT TChildMidi::EvMMMCINotify(WPARAM,LPARAM )
{
	return 0;
}
//*******************************************************************
//*******************************************************************
DEFINE_RESPONSE_TABLE1(TChildMidiChannel,TChildMidi)
	EV_MESSAGE(MM_MCINOTIFY,EvMMMCINotify),
END_RESPONSE_TABLE;
//*******************************************************************
TChildMidiChannel::TChildMidiChannel(TWindow *pparent)
:TChildMidi(pparent)
{
}
//*******************************************************************
TChildMidiChannel::~TChildMidiChannel()
{
}
//*******************************************************************
LRESULT TChildMidiChannel::EvMMMCINotify(WPARAM notify_msg,LPARAM)
{ //have not seen a combo of flags,
  // or anything other than abort or successful

	if (notify_msg & MCI_NOTIFY_ABORTED)
   {
     PMusicMgr->Close(MCI_NOTIFY_ABORTED);
   }
	else if (notify_msg & MCI_NOTIFY_SUCCESSFUL)
   {
     PMusicMgr->Close(MCI_NOTIFY_SUCCESSFUL);
   }
	else if (notify_msg & MCI_NOTIFY_SUPERSEDED)
   {
     PMusicMgr->Close(MCI_NOTIFY_SUPERSEDED);
   }
	else if (notify_msg &  MCI_NOTIFY_FAILURE)
   {
     PMusicMgr->Close(MCI_NOTIFY_FAILURE);
   }
	else
   {
     PMusicMgr->Close(-1);
   }
	return 0;
}
//*******************************************************************
//*******************************************************************
TMusicMCI::TMusicMCI(TWindow* parentWnd)
{
	SetNotBusy();
	m_parentWindow = parentWnd;

	invisibleWindowMidi = new TChildMidiChannel(parentWnd);//,SND_DIALOG_1);
	invisibleWindowMidi->PMusicMgr = this;
   invisibleWindowMidi->Create();

	m_deviceID = 0;
	defaultTMidi = NULL;
	currentTMidi = NULL;
	midi_waiting = NULL;
	bDefaultMidiActive = false;
}
//*******************************************************************
TMusicMCI::~TMusicMCI()
{
	if (Busy())
		Stop();
	delete invisibleWindowMidi;
}
//*******************************************************************
bool TMusicMCI::Open(const char* strFileName)
{
  MCI_OPEN_PARMS mciOpen;
  dword dwResult;
  UINT midiOut;
  midiOut = midiOutGetNumDevs();
  if (midiOut == 0) //no midi output capacity
	  return false;

  //open midi device
  mciOpen.wDeviceID= NULL;
  mciOpen.lpstrDeviceType = "sequencer";
  mciOpen.lpstrElementName = strFileName;     //
  dwResult = mciSendCommand(NULL,MCI_OPEN ,MCI_OPEN_TYPE | MCI_OPEN_ELEMENT,
									  (dword)(LPVOID)&mciOpen);
  if (dwResult != 0)
  {
	  MCIError(dwResult);
	  return false;
  }
  m_deviceID = mciOpen.wDeviceID;//save as part of class the deviceID


 // IsOutputDeviceMidiMapper();
  return true;
}
//*******************************************************************
bool TMusicMCI::Play(TMidi* p)
{
	if (Busy())
	{
		Stop();
		midi_waiting = p;//will play when closed
		return true;
	}
   if (!(p->LoadHDImmed())) {return false;} // Mike - ??

	if (Play(p->GetHDName().c_str()))
	{
		p->SetStarted();
		currentTMidi = p;
		return true;
	}
	else
		return false;
}
//*******************************************************************
bool TMusicMCI::Play(const char* strFileName)
{
  dword dwResult=0;
  MCI_PLAY_PARMS mciPlay;

  if (!Open(strFileName))
	  return false;
  SetBusy();
  //play midi device
  mciPlay.dwCallback = (dword)invisibleWindowMidi->HWindow;
  dwResult = mciSendCommand(m_deviceID,MCI_PLAY,MCI_NOTIFY,
									  (dword)(LPVOID)&mciPlay);
  if (dwResult != 0)
  {
		mciSendCommand(m_deviceID,MCI_CLOSE,0,NULL);
		MCIError(dwResult);
		return false;
  }
  return true;
}
//*******************************************************************
bool TMusicMCI::Close(int16 notify_msg)
{ //called by child notification window when midi is done

	currentTMidi->SetStopped();
	SetNotBusy();

	switch (notify_msg)
	{
		case MCI_NOTIFY_ABORTED://0x0004 called when a midi file is stopped
										 //usually when a new midi file is to be played
										  //and the current one is stopped
			if (midi_waiting)
			{
				Play(midi_waiting);
				midi_waiting = NULL;
			}
			break;
		case MCI_NOTIFY_SUCCESSFUL://0x0001
			//called when a midi file has played to completion OK
			Stop();
			if (bDefaultMidiActive)
			{
				if(Play(defaultTMidi))
					return true;
			}
			break;
		case MCI_NOTIFY_SUPERSEDED://0x0002
			break;
		case MCI_NOTIFY_FAILURE://0x0008
			break;
		default:
			break;
	}

	 return true;
}
//*******************************************************************
bool TMusicMCI::Stop(void)
{
	dword dwResult = 0;

	if (m_deviceID)
		 dwResult = mciSendCommand(m_deviceID,MCI_CLOSE,MCI_WAIT,NULL);
	else return false;
	if (dwResult != 0)
	{
		 MCIError(dwResult);
		 return false;
	}
	m_deviceID =0;

	return true;
}
//*******************************************************************
inline bool TMusicMCI::Busy()
{
	return m_bBusy;
}
//*******************************************************************
inline void TMusicMCI::SetBusy()
{
	m_bBusy = true;
}
//*******************************************************************
inline void TMusicMCI::SetNotBusy()
{
	m_bBusy = false;
}
//*******************************************************************
bool TMusicMCI::IsOutputDeviceMidiMapper(void)
{//if we want to know if midi mapper is used
	dword dwReturn;
	MCI_STATUS_PARMS mciStatusParms;

	mciStatusParms.dwItem = MCI_SEQ_STATUS_PORT;
	dwReturn = mciSendCommand(m_deviceID,MCI_STATUS, MCI_STATUS_ITEM,
											(dword)(LPVOID) &mciStatusParms);
	if (dwReturn != 0)
		return false;//had problem with MCI_STATUS command
	if (LOWORD(mciStatusParms.dwReturn) == (WORD)MIDIMAPPER)
		return true;
	//else
	return false;
}
//*******************************************************************
bool TMusicMCI::GetMidiCapabilities()
{
	UINT rtrn;
	MIDIOUTCAPS midiOutCapabilities;
	rtrn = midiOutGetDevCaps(m_deviceID,&midiOutCapabilities,sizeof(MIDIOUTCAPS));
	if (rtrn == 0) //successful capture of Device Capabilites above
	{
		WORD wTechnology;
		wTechnology = midiOutCapabilities.wTechnology;
		switch (wTechnology)
		{
			case MOD_MIDIPORT:
				break;
			case MOD_SQSYNTH:
				break;
			case MOD_FMSYNTH:
				break;
			case MOD_SYNTH:
				break;
			case MOD_MAPPER:
				break;

		}//end switch
	}

	if (rtrn != 0)
	{
		//did not get device capabilities for MIDI
		switch(rtrn)
		{
			case  MMSYSERR_BADDEVICEID:
				// m_deviceID is out of range
				break;
			case  MMSYSERR_NODRIVER:
				//driver was not installed
				break;
			case  MMSYSERR_NOMEM:
				//system unable to load the mapper string description
				break;
		}//end switch
	return false;
	}//end if failed
	//I'm interested in volume control capabilities
	if (midiOutCapabilities.dwSupport & MIDICAPS_VOLUME)
	{  //driver supports volume control
		MessageBox(NULL,"Supports midi volume","",MB_OK);
	}
	if (midiOutCapabilities.dwSupport & MIDICAPS_LRVOLUME)
	{ //driver supports individual volume control
	  //  for left and right channels
		MessageBox(NULL,"Supports L & R volume midi","",MB_OK);
	}
	MessageBox(NULL,midiOutCapabilities.szPname,"",MB_OK);
	return true;//
}//end GetMidiCapabilities()
//*******************************************************************
void TMusicMCI::MCIError(dword dwErr)
{
	 char buf[256];

	 buf[0] = '\0';
	 mciGetErrorString(dwErr, buf, sizeof(buf));
	 if (!strlen(buf)) {
		  strcpy(buf, "Unknown error");
	 }
	 TRACE(buf);
}
//*******************************************************************
bool TMusicMCI::SetDefaultMidi(TMidi* makedefaultTMidi)
{
	defaultTMidi = makedefaultTMidi;
	return true;
}
//*******************************************************************
bool TMusicMCI::SetDefaultMidiActive(bool bActive)
{
	if (bActive)
		bDefaultMidiActive = true;
	else
		bDefaultMidiActive = false;
	return true;
}
//*******************************************************************
bool TMusicMCI::SetDefaultMidiInactive(bool bActive)
{
	if (bActive)
		bDefaultMidiActive = false;
	else
		bDefaultMidiActive = true;
   return true;
}
//*******************************************************************
bool TMusicMCI::PlayMidiDefault(TMidi* p)
{
	if (Play(p))
	{
		SetDefaultMidi(p);
		SetDefaultMidiActive(true);
	}
  return true;
}
//*******************************************************************

