//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  obtnsp.cpp
//    Project Name:  ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  2.5    11/04/99    M.Perry     original baseline
//********************************************************************
#include "obtnsp.h"
#include "omodewin.h"
#include "orespond.h"
#include "oanm.h"
#include "owav.h"
//*******************************************************************
void TOgleButton::Enable(bool enable)
{
  if (PButtonResponder != NULL)
  {
    if (enable)
    {
      PButtonResponder->Add();
    }
    else
    {
      PButtonResponder->Remove();
    }
  }
}
//*******************************************************************
void TOgleButton::SetupWavs()
{
	 PDownWav = GetWav(0);
	 PUpWav = GetWav(1);
	 PChimeWav = GetWav(2);
}
//*******************************************************************
void TOgleButton::SetupButton()
{
  //set to initial state
  switch (CurrentButtonStatus)
  {
    case bsUp    :ShowUp();break;
    case bsDown  :ShowDown();break;
    case bsGrayed:ShowGray();break;
    case bsLite  :ShowLite();break;
    default      :PButton->NoShow();break;
  }
}
//*******************************************************************
void TOgleButton::SetupAnms()
{
	 PButton = GetAnm(0);

	 if (PButton == NULL) {AnmError("button sprite","button",0);}
	 PButton->CAD.ClearAutoStart();
	 PButton->CAD.SetDirStill();
	 PButton->CAD.ClearAutoStop();
}
//*******************************************************************
TOgleButton::TOgleButton(TOgleModeWindow *pmw,
                       TFileMode NewFileMode,
                       TOgleBase *pbase,
							  int16 ZOrder,
                       const TRect &rect,
                       int16 idnum,
                       int16 upframe,
                       int16 downframe,
                       int16 grayframe,
                       int16 startstate,
                       char Ch,
                       const TCursorData &cd,
                       const TCursorData &dac)
:TSprite(pmw,NewFileMode),
 CurrentUpFrame(upframe),
 CurrentDownFrame(downframe),
 CurrentGrayFrame(grayframe),
 CurrentLiteFrame(-1),
 DefaultActionCursor(dac),
 POwnerBase(pbase),//GameLord),
 LeftDown(false),
 RightDown(false),
 CurrentButtonStatus(startstate),
 HandleLeftClicks(true),
 HandleRightClicks(false),
 HandleGrayClicks(false),
 HandleLiteClicks(false),
 UpWavNDX(bwUpNDX),
 DownWavNDX(bwDownNDX),
 ChimeWavNDX(bwChimeNDX),
 UpFUC(0),
 IDNum(idnum),
 UpInfo(-1)
{
	 SetToManual();
//	 NoShow();
	 SetupAnms();
	 SetupWavs();
	 PButtonResponder = new TResponder(PModeWin,ZOrder,rect,idnum,this,true,Ch,cd);
    SetupButton();
}
//*******************************************************************
TOgleButton::TOgleButton(TOgleModeWindow *pmw,
                       TFileMode fm,
                       void *psource,
                       TOgleBase *pbase,
							  int16 ZOrder,
                       const TRect &rect,
                       int16 idnum,
                       int16 upframe,
                       int16 downframe,
                       int16 grayframe,
                       int16 startstate,
                       char Ch,
                       const TCursorData &cd,
                       const TCursorData &dac)
:TSprite(pmw,fm,psource),
 CurrentButtonStatus(startstate),
 LeftDown(false),
 RightDown(false),
 CurrentUpFrame(upframe),
 CurrentDownFrame(downframe),
 CurrentGrayFrame(grayframe),
 CurrentLiteFrame(-1),
 DefaultActionCursor(dac),
 HandleLeftClicks(true),
 HandleRightClicks(false),
 HandleGrayClicks(false),
 HandleLiteClicks(false),
 POwnerBase(pbase),
 UpFUC(0),
 IDNum(idnum),
 UpInfo(-1),
 UpWavNDX(bwUpNDX),
 DownWavNDX(bwDownNDX),
 ChimeWavNDX(bwChimeNDX)
{
	 SetToManual();
//	 NoShow();
	 SetupAnms();
	 SetupWavs();
	 PButtonResponder = new TResponder(PModeWin,ZOrder,rect,idnum,this,true,Ch,cd);
    SetupButton();
}
//*******************************************************************
TOgleButton::TOgleButton(TOgleModeWindow *pmw,TFileMode NewFileMode)
:TSprite(pmw,NewFileMode),
 CurrentUpFrame(-1),
 CurrentDownFrame(-1),
 CurrentGrayFrame(-1),
 CurrentLiteFrame(-1),
 POwnerBase(NULL),//GameLord),
 CurrentButtonStatus(bsNone),
 LeftDown(false),
 RightDown(false),
 PButtonResponder(NULL),
 HandleLeftClicks(true),
 HandleRightClicks(false),
 HandleGrayClicks(false),
 HandleLiteClicks(false),
 UpFUC(0),
 UpInfo(-1),
 IDNum(0),
 UpWavNDX(bwUpNDX),
 DownWavNDX(bwDownNDX),
 ChimeWavNDX(bwChimeNDX)
{
	 SetToManual();
//	 NoShow();
	 SetupAnms();
	 SetupWavs();
    SetupButton();
}
//*******************************************************************
TOgleButton::TOgleButton(TOgleModeWindow *pmw,TFileMode fm,void *psource)
:TSprite(pmw,fm,psource),
  CurrentUpFrame(-1),
 CurrentDownFrame(-1),
 CurrentGrayFrame(-1),
 CurrentLiteFrame(-1),
 POwnerBase(NULL),//GameLord),
 CurrentButtonStatus(bsNone),
 LeftDown(false),
 RightDown(false),
 PButtonResponder(NULL),
 HandleLeftClicks(true),
 HandleRightClicks(false),
 HandleGrayClicks(false),
 HandleLiteClicks(false),
 UpFUC(0),
 UpInfo(-1),
 IDNum(0),
 UpWavNDX(bwUpNDX),
 DownWavNDX(bwDownNDX),
 ChimeWavNDX(bwChimeNDX)
{
	 SetToManual();
//	 NoShow();
	 SetupAnms();
	 SetupWavs();
    SetupButton();
}
//*******************************************************************
TOgleButton::~TOgleButton()
{
  if(PButtonResponder != NULL){delete PButtonResponder;}
}
//*******************************************************************
void TOgleButton::ShowGray()
{
  CurrentButtonStatus = bsGrayed;

  if(CurrentGrayFrame < 0)
  {
//	 PButton->NoShow();
    NoShow();
  }
  else
  {
	 PButton->CAD.SetCurrentFrame(CurrentGrayFrame);
//	 PButton->Show();
    Show();
  }
}
//*******************************************************************
void TOgleButton::ShowLite()
{
  CurrentButtonStatus = bsLite;

  if (CurrentLiteFrame < 0)
  {
//    PButton->NoShow();
    NoShow();
  }
  else
  {
	 PButton->CAD.SetCurrentFrame(CurrentLiteFrame);
//	 PButton->Show();
    Show();
  }
  if(PChimeWav != NULL)
  {
    PChimeWav->Start();
  }
}
//*******************************************************************
void TOgleButton::ShowUp(bool actioncursor)
{
  CurrentButtonStatus = bsUp;

  if (actioncursor)
  {
     GetOgleApp()->GetCursorMgr()->SetCursor(ctAction,0,0); //clear action cursor
  }

  if(CurrentUpFrame < 0)
  {
//	 PButton->NoShow();
    NoShow();
  }
  else
  {
    PButton->CAD.SetCurrentFrame(CurrentUpFrame);
//	 PButton->Show();
    Show();
  }
  if(PUpWav != NULL)
  {
	 PUpWav->Start();
  }
}
//*******************************************************************
void TOgleButton::ShowDown(bool actioncursor)
{
  CurrentButtonStatus = bsDown;

  if (actioncursor) //set action cursor
  {
    GetOgleApp()->GetCursorMgr()->SetCursor(ctAction,DefaultActionCursor);
  }

  if (CurrentDownFrame < 0)
  {
//    PButton->NoShow();
    NoShow();
  }
  else
  {
    PButton->CAD.SetCurrentFrame(CurrentDownFrame);
//	 PButton->Show();
    Show();
  }
  if(PDownWav != NULL)
  {
    PDownWav->Start();
  }
}
//*******************************************************************
bool TOgleButton::Respond(TResponder &responder)
{
  int16 re = int16(responder.GetResponseEvent());

  switch (re)
  {
    case reLUp:if (!HandleLeftClicks || !LeftDown) {return true;}
               if (!responder.Contains(responder.GetPoint()))
               {
                 UpInfo = bcNone;
               }
               else if (CurrentButtonStatus == bsDown)
               {
                 UpInfo = bcLeft;
               }
               else if (CurrentButtonStatus == bsUp)
               {
                 return true;
               }
               else if (CurrentButtonStatus == bsGrayed && HandleGrayClicks)
               {
                 UpInfo = bcLeftGray;
               }
               else if (CurrentButtonStatus == bsLite && HandleLiteClicks)
               {
                 UpInfo = bcLeftLite;
               }

               if (UpInfo >= 0)
               {
                 UpFUC = GetAdjFUC() + 2; //schedule showup()
                 LeftDown = false;
                 responder.ClearPriority();
               }
               return true;
    case reRUp:if (!HandleRightClicks || !RightDown) {return true;}
               if (!responder.Contains(responder.GetPoint()))
               {
                 UpInfo = bcNone;
               }
               else if (CurrentButtonStatus == bsDown)
               {
                 UpInfo = bcRight;
               }
               else if (CurrentButtonStatus == bsUp)
               {
                 return true;
               }
               else if (CurrentButtonStatus == bsGrayed && HandleGrayClicks)
               {
                 UpInfo = bcRightGray;
               }
               else if (CurrentButtonStatus == bsLite && HandleLiteClicks)
               {
                 UpInfo = bcRightLite;
               }
               if (UpInfo >= 0)
               {
                 UpFUC = GetAdjFUC() + 2; //schedule ShowUp()
                 RightDown = false;
                 responder.ClearPriority();
               }
               return true;
    case reLDown:if (!HandleLeftClicks ||
                     LeftDown ||
                     RightDown ||
                     CurrentButtonStatus == bsDown ||
                     (CurrentButtonStatus == bsGrayed && !HandleGrayClicks) ||
                     !GetShowing()) //NoShow'd button shouldn't respond.
                 {
                   return true;
                 }
                 LeftDown = true;
                 responder.SetPriority();
                 ShowDown(true);
                 return true;
    case reRDown:if (!HandleRightClicks ||
                     RightDown ||
                     LeftDown ||
                     CurrentButtonStatus == bsDown ||
                     (CurrentButtonStatus == bsGrayed && !HandleGrayClicks) ||
                     !GetShowing())
                 {
                   return true;
                 }
                 RightDown = true;
                 responder.SetPriority();
                 ShowDown(true);
                 return true;
    case reChar: if (CurrentButtonStatus == bsGrayed && !HandleGrayClicks)
                 {
                   return true;
                 }
                 ShowDown();
                 UpFUC = GetAdjFUC() + 2;
                 responder.ClearPriority();
                 UpInfo = bcLeft;
                 return true;
  }
  return false;
}
//*******************************************************************
void TOgleButton::Update(dword NewFUC)
{
  // check for scheduled ShowUp()'s
  if (UpFUC != 0)
  {
    if (NewFUC == (UpFUC - 1)) //do up
    {
      ShowUp(true);
    }
    else if (NewFUC == UpFUC) //wait until after button comes back up to
    {                         //actually call owners respond(int16,int16)
      if (UpInfo >= 0)
      {
        POwnerBase->Respond(IDNum,UpInfo);
      }
      UpFUC = 0;
    }
  }
  TSprite::Update(NewFUC);
}
//*******************************************************************
void TOgleButton::PostUpdate(dword NewFUC)
{
	TSprite::PostUpdate(NewFUC);
}
//*******************************************************************

