//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  osprlist.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     6/7/95     M.Perry     original baseline
//  2.0                M.Perry     32-bit conversion.
//  2.1     12/6/97    M.Perry     filename mod's.
//********************************************************************
#if !defined(__osprlist_h)
#include "osprlist.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if !defined(__omode_h)
#include "omode.h"
#endif

#if !defined(__ofilemgr_h)
#include "ofilemgr.h"
#endif

#ifdef GED_COMPILE
  #undef SPL_READWASP
  #define SPL_WRITEWASP
#else
  #ifdef SPL_READSLOW
    #undef SPL_READWASP
  #else
    #define SPL_READWASP
  #endif
  #define SPL_WRITEWASP
#endif

#if !defined(__ogrpgdlg_h)
#include "ogrpgdlg.h"
#endif

#if !defined(__osmsgdlg_h)
#include "osmsgdlg.h"
#endif

//#if !defined(__inifile_h)
//#include "inifile.h"
//#endif

#if !defined(__ospidini_h) && defined(GED_COMPILE)
#include "ospidini.h"
#endif

#if !defined(__ogamefw_h)
#include "ogamefw.h"
#endif

#include <assert.h>
DIAG_DEFINE_GROUP(OSPRLIST,1,0); //name,enabled,level
//*******************************************************************
TActiveSpriteElement::TActiveSpriteElement()
{
}
//*******************************************************************
TActiveSpriteElement::TActiveSpriteElement(TOgleBase *PNewBase,dword NewStartFUC)
:TOgleActiveElement(PNewBase,true)
{
  if (PNewBase == NULL) {return;}

// mrp - v2.4 - 08/17/99 - ??? the following doesn't need to subtract 1 from fuc
// because fucoffset's have already implicitly subtracted 1.
  StartFUC = PBase->GetFUCOffset() + NewStartFUC;
  PBase->Activate(StartFUC);
}
//*******************************************************************
TActiveSpriteElement::~TActiveSpriteElement()
{
}
//*******************************************************************
#ifdef DEBUGSPRITES
void TActiveSpriteElement::RTReport(ofstream &os)
{
  TSprite *psprite = (TSprite *)PBase;
  os<<"Sprite name: "<<psprite->FileMode.Name.GetStr()<<"\n";
  os<<"StartFUC: "<<StartFUC<<"\n";
  os<<"Active: "<<Active<<"\n";
  psprite->RTReport(os);
}                                             
#endif
//*******************************************************************
//*******************************************************************
TSpriteSetIdle::TSpriteSetIdle(TGObjSp *pownergobj,const string &spritefname)
:POwnerGObj(pownergobj),
 SpriteName(spritefname),
 SpriteListIndex(-1),
 PSprite(NULL),
 ActiveCount(0),
 Stopped(true)
{
}
//*******************************************************************
TSpriteSetIdle::~TSpriteSetIdle()
{
  if ((ActiveCount > 0) && (PSprite != NULL))
  {
    PSprite->DeActivate();
  }
}
//*******************************************************************
void TSpriteSetIdle::SetIndex()
{
  SpriteListIndex = POwnerGObj->GetSpriteIndex(SpriteName);
  PSprite = POwnerGObj->GetSpritePtr(SpriteListIndex);
  assert(PSprite);
  if (PSprite != NULL)
  {
//    PSprite->Priority(); // prevents loading/unloading of idles by raising bg level
    if (ActiveCount > 0) {PSprite->Activate(POwnerGObj->AskCurrentFUC());} // so does this.
  }
}
//*******************************************************************
void TSpriteSetIdle::Activate(dword StartFUC)
{
  if (SpriteListIndex < 0) {SetIndex();}
  ActiveCount++;
  if (PSprite != NULL) {PSprite->Activate(StartFUC);}
}
//*******************************************************************
void TSpriteSetIdle::DeActivate()
{
  ActiveCount--;
  if (ActiveCount < 0) {ActiveCount = 0;}
  if (PSprite != NULL) {PSprite->DeActivate();}
}
//*******************************************************************
//*******************************************************************
TSpriteSet::TSpriteSet(TGObjSp *pownergobj)
{
  POwnerGObj = pownergobj;
  NumIdles = 0;
  PPSpriteSetIdles = NULL;
  CurrentIdleIndex = -1;
}
//*******************************************************************
TSpriteSet::~TSpriteSet()
{
//  while (NumIdles > 0)
//  {
//    RemoveIdle(0);
//  }

  // added by MRP 8/22/97
  if (NumIdles > 0 && PPSpriteSetIdles != NULL)
  {
    for (int16 i = 0;i<NumIdles;i++)
    {
      delete PPSpriteSetIdles[i];
      PPSpriteSetIdles[i] = NULL;
    }
    delete [] PPSpriteSetIdles;
    PPSpriteSetIdles = NULL;
  }
  NumIdles = 0;
}
//*******************************************************************
bool TSpriteSet::AddIdle(const string &spritefname)
{
  if ((NumIdles == 0) || (PPSpriteSetIdles == NULL))
  {
    PPSpriteSetIdles = new TSpriteSetIdle *[1];
    NumIdles = 1;
    PPSpriteSetIdles[0] = new TSpriteSetIdle(POwnerGObj,spritefname);
    return true;
  }
  else
  {
    TSpriteSetIdle **Holder = PPSpriteSetIdles;
    int16 OldNumIdles = NumIdles;
//    NumIdles += 1;
    NumIdles++;
    PPSpriteSetIdles = new TSpriteSetIdle *[NumIdles];

    for (int16 i = 0;i < OldNumIdles;i++)
    {
      PPSpriteSetIdles[i] = Holder[i];
    }
    PPSpriteSetIdles[NumIdles - 1] = new TSpriteSetIdle(POwnerGObj,spritefname);
    delete [] Holder;
    return true;
  }
}
//*******************************************************************
bool TSpriteSet::RemoveIdle(int16 idleindex)
{
  if ((idleindex >= NumIdles) || (idleindex < 0)) {return false;}
  if (PPSpriteSetIdles == NULL) {return false;}

  if (NumIdles == 1)
  {
    delete PPSpriteSetIdles[0];
    delete [] PPSpriteSetIdles;
    PPSpriteSetIdles = NULL;
    NumIdles = 0;
    return true;
  }
  else
  {
    TSpriteSetIdle **Holder = PPSpriteSetIdles;
    int16 OldNumIdles = NumIdles;
//    NumIdles -= 1;
    NumIdles--;
    PPSpriteSetIdles = new TSpriteSetIdle *[NumIdles];
    int16 CurrentIndex = 0;
    for (int16 i=0;i<OldNumIdles;i++)
    {
      if (i != idleindex)
      {
        PPSpriteSetIdles[CurrentIndex] = Holder[i];
        CurrentIndex++;
      }
    }
    delete Holder[idleindex];
    delete [] Holder;
    return true;
  }
}
//*******************************************************************
void TSpriteSet::Update(dword NewFUC,bool stopidle)
{
  if (stopidle)
  {
    if (GetIdle(CurrentIdleIndex) != NULL) {GetIdle(CurrentIdleIndex)->Stop();}
    CurrentIdleIndex = -1;
  }
  else if (CurrentIdleIndex < 0) {Start(NewFUC);}
  else if (GetIdle(CurrentIdleIndex) == NULL) {Start(NewFUC);}
  else if (GetIdle(CurrentIdleIndex)->GetStopped()) {Start(NewFUC);}
  else {GetIdle(CurrentIdleIndex)->Update(NewFUC);}
}
//*******************************************************************
void TSpriteSet::SetIndexes()
{
  for (int16 i=0;i<NumIdles;i++)
  {
    TSpriteSetIdle *pssi = GetIdle(i);
    if (pssi != NULL) {pssi->SetIndex();}
  }
}
//*******************************************************************
void TSpriteSet::Show()
{
  int16 ni = GetNumIdles();
  for (int16 i=0;i<ni;i++)
  {
    TSpriteSetIdle *pssi = GetIdle(i);
    if (pssi != NULL) {pssi->Show();}
  }
}
//*******************************************************************
void TSpriteSet::NoShow()
{
  int16 ni = GetNumIdles();
  for (int16 i=0;i<ni;i++)
  {
    TSpriteSetIdle *pssi = GetIdle(i);
    if (pssi != NULL) {pssi->NoShow();}
  }
}
//*******************************************************************
void TSpriteSet::Activate(dword StartFUC)
{
  int16 ni = GetNumIdles();
  for (int16 i=0;i<ni;i++)
  {
    TSpriteSetIdle *pssi = GetIdle(i);
    if (pssi != NULL) {pssi->Activate(StartFUC);}
  }
}
//*******************************************************************
void TSpriteSet::DeActivate()
{
  int16 ni = GetNumIdles();
  for (int16 i=0;i<ni;i++)
  {
    TSpriteSetIdle *pssi = GetIdle(i);
    if (pssi != NULL) {pssi->DeActivate();}
  }
}
//*******************************************************************
//*******************************************************************
#ifdef DEBUGSPRITES
void TSpriteActiveList::RTReport(ofstream &os)
{
  int16 counter = 0;
  TOgleActiveElement *PElementHolder = NULL;
  for (PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    os<<"***********************************************\n";
    os<<"Active element counter is: "<<counter<<"\n";
    ((TActiveSpriteElement *)PElementHolder)->RTReport(os);
    counter++;
  }
  if (counter == 0)
  {
    os<<"No active elements in list\n";
  }
}
#endif
//*******************************************************************
bool TSpriteActiveList::AddSet()
{
  if ((NumSets == 0) || (PPSpriteSets == NULL))
  {
    PPSpriteSets = new TSpriteSet *[1];
    NumSets = 1;
    PPSpriteSets[0] = new TSpriteSet(POwnerGObj);
    return true;
  }
  else
  {
    TSpriteSet **Holder = PPSpriteSets;
    int16 OldNumSets = NumSets;
//    NumSets += 1;
    NumSets++;
    PPSpriteSets = new TSpriteSet *[NumSets];

    for (int16 i = 0;i < OldNumSets;i++)
    {
      PPSpriteSets[i] = Holder[i];
    }
    PPSpriteSets[NumSets - 1] = new TSpriteSet(POwnerGObj);
    delete [] Holder;
    return true;
  }
}
//*******************************************************************
bool TSpriteActiveList::RemoveSet(int16 setindex)
{
  if ((setindex >= NumSets) || (setindex < 0)) {return false;}
  if (PPSpriteSets == NULL) {return false;}

  if (NumSets == 1)
  {
    delete PPSpriteSets[0];
    delete [] PPSpriteSets;
    PPSpriteSets = NULL;
    NumSets = 0;
    return true;
  }
  else
  {
    TSpriteSet **Holder = PPSpriteSets;
    int16 OldNumSets = NumSets;
//    NumSets -= 1;
    NumSets--;
    PPSpriteSets = new TSpriteSet *[NumSets];
    int16 CurrentIndex = 0;
    for (int16 i=0;i<OldNumSets;i++)
    {
      if (i != setindex)
      {
        PPSpriteSets[CurrentIndex] = Holder[i];
        CurrentIndex++;
      }
    }
    delete Holder[setindex];
    delete [] Holder;
    return true;
  }
}
//*******************************************************************
bool TSpriteActiveList::AddSetIdle(int16 setindex,const string &spritefname)
{
  TSpriteSet *pss = GetSet(setindex);
  if (pss == NULL) {return false;}
  return (pss->AddIdle(spritefname));
}
//*******************************************************************
bool TSpriteActiveList::RemoveSetIdle(int16 setindex,int16 idleindex)
{
  TSpriteSet *pss = GetSet(setindex);
  if (pss == NULL) {return false;}
  return (pss->RemoveIdle(idleindex));
}
//*******************************************************************
void TSpriteActiveList::SetIndexes()
{
  for (int16 i = 0;i<NumSets;i++)
  {
    TSpriteSet *pss = GetSet(i);
    if (pss != NULL) {pss->SetIndexes();}
  }
}
//*******************************************************************
void TSpriteActiveList::Activate(dword StartFUC)
{
  int16 ns = GetNumSets();
  for (int16 i=0;i<ns;i++)
  {
    TSpriteSet *pss = GetSet(i);
    if (pss != NULL) {pss->Activate(StartFUC);}
  }
}
//*******************************************************************
void TSpriteActiveList::DeActivate()
{
  int16 ns = GetNumSets();
  for (int16 i=0;i<ns;i++)
  {
    TSpriteSet *pss = GetSet(i);
    if (pss != NULL) {pss->DeActivate();}
  }
}
//*******************************************************************
void TSpriteActiveList::Show()
{
  int16 ns = GetNumSets();
  for (int16 i=0;i<ns;i++)
  {
    TSpriteSet *pss = GetSet(i);
    if (pss != NULL) {pss->Show();}
  }
}
//*******************************************************************
void TSpriteActiveList::NoShow()
{
  int16 ns = GetNumSets();
  for (int16 i=0;i<ns;i++)
  {
    TSpriteSet *pss = GetSet(i);
    if (pss != NULL) {pss->NoShow();}
  }
}
//*******************************************************************
int16 TSpriteActiveList::GetSetNumIdles(int16 setindex)
{
  TSpriteSet *pss = GetSet(setindex);
  if (pss == NULL) {return 0;}
  return (pss->GetNumIdles());
}
//*******************************************************************
string TSpriteActiveList::GetSetIdleName(int16 setindex,int16 idleindex)
{
  TSpriteSet *pss = GetSet(setindex);
  if (pss == NULL) {return string("");}
  return (pss->GetIdleName(idleindex));
}
//*******************************************************************
TSpriteActiveList::TSpriteActiveList(TOgleModeWindow *pmw,
                                     TGObjSp *PGObj,
                                     const string &spritetypename)
:TOgleActiveList(pmw,spritetypename)
{
  POwnerGObj = PGObj;
  NumSets = 0;
  PPSpriteSets = NULL;
  CurrentSetIndex = -1;
}
//*******************************************************************
TSpriteActiveList::~TSpriteActiveList()
{
  while (NumSets > 0)
  {
    RemoveSet(0);
  }
}
//*******************************************************************
TOgleActiveElement *TSpriteActiveList::CreateNewElement()
{
  return (new TActiveSpriteElement);
}
//*******************************************************************
TOgleActiveElement *TSpriteActiveList::CreateNewElement(TOgleBase *PBase,dword StartFUC)
{
  return (new TActiveSpriteElement(PBase,StartFUC));
}
//*******************************************************************
void TSpriteActiveList::Update(dword NewFUC)
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartUActiveList();
  PModeWin->DebugUpdate.SetUActiveListNumElements(GetRecCount());
  #endif
  TOgleActiveList::Update(NewFUC);
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndUActiveList();
  PModeWin->DebugUpdate.StartUActiveListIdles();
  #endif
  TSpriteSet *pss = GetSet(CurrentSetIndex);
  if (pss != NULL)
  {
    bool going = HasGoingElement();
    #if defined(DEBUGUPDATES)
    PModeWin->DebugUpdate.SetUActiveListHasGoing(going);
    #endif
    pss->Update(NewFUC,going);
  }
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndUActiveListIdles();
  #endif
}
//*******************************************************************
void TSpriteActiveList::PostUpdate(dword NewFUC)
{
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.StartPUActiveList();
  #endif
  TOgleActiveList::PostUpdate(NewFUC);
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.EndPUActiveList();
  PModeWin->DebugUpdate.StartPUActiveListIdles();
  #endif
  TSpriteSet *pss = GetSet(CurrentSetIndex);
  if (pss != NULL) {pss->PostUpdate(NewFUC);}
  #if defined(DEBUGUPDATES)
  PModeWin->DebugUpdate.SetPUActiveListNumElements(GetRecCount());
  PModeWin->DebugUpdate.EndPUActiveListIdles();
  #endif
}
//*******************************************************************
bool TSpriteActiveList::IsRangeAvail(dword StartFUC,dword EndFUC)
{
  SortByStartFUC();
  dword StartFUC2,EndFUC2;
  TOgleActiveElement *PElementHolder = NULL;
  for (PElementHolder = GetFirst();
       PElementHolder != NULL;
       PElementHolder = GetNext())
  {
    StartFUC2 = PElementHolder->GetStartFUC();
    EndFUC2 = PElementHolder->GetEndFUC();
    if (EndFUC < StartFUC2) {return true;}
//    if (StartFUC < EndFUC2)
    if (StartFUC <= EndFUC2)
    {
      #if defined(OGLE_DEBUG_SKITS_DATA)
      char buffer[50];
      string title = "Warning - Skits && Sprites";
      string pstrmsg[7];
      pstrmsg[0] = "TSpriteActiveList::IsRangeAvail() failed";
      pstrmsg[1] = "StartFUC == ";
      ultoa(StartFUC,buffer,10);
      pstrmsg[1] += buffer;
      pstrmsg[2] = "EndFUC == ";
      ultoa(EndFUC,buffer,10);
      pstrmsg[2] += buffer;
      pstrmsg[3] = "Element StartFUC == ";
      ltoa(StartFUC2,buffer,10);
      pstrmsg[3] += buffer;
      pstrmsg[4] = "Element EndFUC == ";
      ltoa(EndFUC2,buffer,10);
      pstrmsg[4] += buffer;
      pstrmsg[5] = "Sprite Name: ";
      if (PElementHolder->PBase != NULL)
      {
        pstrmsg[5] += PElementHolder->PBase->FileMode.Name.GetStr();
      }
      else
      {
        pstrmsg[5] += "unknown";
      }
      pstrmsg[6] = "Active: ";
      if (PElementHolder->Active)
      {
        pstrmsg[6] += "true";
      }
      else
      {
        pstrmsg[6] += "false";
      }
      if (POwnerGObj != NULL && POwnerGObj->GetModeWin() != NULL)
      {
        POwnerGObj->GetModeWin()->ScrollMsg(title,pstrmsg,7,true);
      }
      else
      {
        string s;
        for (int16 i=0;i<7;i++)
        {
          s += pstrmsg[i];
          s += "\n";
        }
        ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
      }
      #endif
      return false;
    }
  }
  return true;
}
//*******************************************************************
bool TSpriteActiveList::SetCurrentSet(int16 setindex)
{
  if (setindex != CurrentSetIndex)
  {
     TSpriteSet *pss = GetSet(CurrentSetIndex);
     if (pss != NULL) {pss->Stop();}
     CurrentSetIndex = setindex;
     pss = GetSet(CurrentSetIndex);
     if ((pss != NULL) && (!HasGoingElement()))
     {
       pss->Start(POwnerGObj->AskCurrentFUC());
     }
  }
  return true;
}
//*******************************************************************
//*******************************************************************
const char *TSpriteList::GetIDChars() const
{
  return "PLST";
}
//*******************************************************************
string TSpriteList::AddExt()
{
  return string(".spl");
}
//*******************************************************************
int16 TSpriteList::GetActiveListIndex(TOgleBase *PBase)
{
  if (PBase == NULL) {return 0;}

  int16 sprtype = PBase->GetType();
  if ((sprtype >= NumActiveLists) || (sprtype < 0))
  {
    sprtype = 0;
  }
  return sprtype;
}
//*******************************************************************
void TSpriteList::SetMinVer()
{
  MinVer = 3;
}
//*******************************************************************
void TSpriteList::SetMajVer()
{
  MajVer = 2;
}
//*******************************************************************
#ifdef DEBUGREPORTS
void TSpriteList::Report(ofstream &os)
{
  os<<"Start of report for " << FileMode.Name.GetStr() << "\n";
  int16 num = GetLastNumSprites();
  os<<"Last Sprite Index #: "<<num<<"\n\n";
  for (int16 i=0;i<=num;i++)
  {
    os<<"****************************************************"<<"\n";
    os<<"index #"<<i<<"\n";
    if (GetSpritePtr(i) != NULL)
    {
      GetSpritePtr(i)->Report(os);
    }
    else
    {
      os<<"Sprite was NULL"<<"\n";
    }
  }
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
bool TSpriteList::CheckSpriteIDConflicts(bool verbose)
{
  int16 num = GetLastNumSprites();
  for (int16 i=0;i<=num;i++)
  {
    TSprite *psprite1 = GetSpritePtr(i);
    if (psprite1 != NULL)
    {
      for (int16 j=0;j<=num;j++)
      {
        TSprite *psprite2 = GetSpritePtr(j);
        if (psprite2 != NULL &&
            i != j &&
            psprite1 != psprite2)
        {
          int16 id1 = psprite1->GetIDCode();
          int16 id2 = psprite2->GetIDCode();

          if (id1 > 0 &&
              id2 > 0 &&
              id1 == id2)
          {
            string title = "Error - SpriteID conflict";
            string pstrmsg[5];
            pstrmsg[0] = "TSpriteList::CheckSpriteIDConflicts()";
            pstrmsg[1] = "Sprite1 Name: ";
            pstrmsg[1] += psprite1->FileMode.Name.GetStr();
            pstrmsg[2] = "Sprite1 ID: ";
            char buffer[50];
            itoa(id1,buffer,10);
            pstrmsg[2] += buffer;
            pstrmsg[3] = "Sprite2 Name: ";
            pstrmsg[3] += psprite2->FileMode.Name.GetStr();
            pstrmsg[4] = "Sprite2 ID: ";
            itoa(id2,buffer,10);
            pstrmsg[4] += buffer;
            string s;
            for (int16 i=0;i<5;i++)
            {
              s += pstrmsg[i];
              s += "\n";
            }
            ::MessageBox(NULL,s.c_str(),title.c_str(),MB_OK | MB_TASKMODAL);
            return false;
          }
        }
      }//for j
    }
  }//for i
  
  if (verbose)
  {
    ::MessageBox(NULL,
                 "no conflicts found",
                 "Info - sprite id conflicts",
                 MB_OK | MB_TASKMODAL);
  }
  return true;
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
bool TSpriteList::CheckSpriteIDUsage(bool verbose)
{
  if (GetGObjSp() == NULL)
  {
    ::MessageBox(NULL,
                 "POwnerGObj == NULL",
                 "Warning - Sprite ID Usage",
                 MB_OK | MB_TASKMODAL);
    return false;
  }

  // check "root\gobjdir\gobjname.idc" for existence and get num of id codes.
//  string strInfoSection = "Info";
//  string strIDSection = "ID";
//  string strNumIDKey = "NumID";

  TOgleFileName ofn = GetFileMgr()->GetCDRomPath(-1);
  ofn.AppendDirPath(GetGObjSp()->FileMode.Name.GetDirPath());
  ofn.SetFName(GetGObjSp()->FileMode.Name.GetFName());
  ofn.SetExt(string(".idc"));

  // only show msgbox when something in list.
  if (GetLAEN() < 0) {return true;}

  if (!ofn.FileExists())
  {
    string s = "IDCode file: ";
    s += ofn.GetStr();
    s += " doesn't exist";
    ::MessageBox(NULL,
                 s.c_str(),
                 "Warning - Sprite ID Usage",
                 MB_OK | MB_TASKMODAL);
    return false;
  }
//  TIniFile inifile(ofn.GetStr());
  TSpriteIDIniFile spriteidinifile(ofn);
  spriteidinifile.Squeeze();
//  int16 numids = (int16)inifile.ReadInt(strInfoSection,strNumIDKey,0);
  int16 numids = spriteidinifile.GetNumID();
  if (numids <= 0)
  {
    ::MessageBox(NULL,
                 "numids <= 0",
                 "Warning - Sprite ID Usage",
                 MB_OK | MB_TASKMODAL);
    return false;
  }

  numids++; //add 1 because what's saved is actually lastaddedelementnum.
  bool *pidfound = new bool[numids];

  TSprite **psprites = new TSprite *[numids];

  for (int16 i = 0;i<numids;i++)
  {
    pidfound[i] = false;
    psprites[i] = NULL;
  }
  for (int16 i = 0;i<=GetLAEN();i++)
  {
    TSprite *psprite = GetSpritePtr(i);
    if (psprite != NULL)
    {
      int16 id = psprite->GetIDCode();
      if (id >= 0 && id < numids)
      {
        pidfound[id] = true;
        psprites[id] = psprite;
      }
      else
      {
        string s = psprite->FileMode.Name.GetStr();
        s += "\nidcode: ";
        char buffer[50];
        itoa(id,buffer,10);
        s += buffer;
        s += " is invalid";
        if (id >= numids)
        {
          s += "\nid >= ";
          itoa(numids,buffer,10);
          s += buffer;
        }
        else
        {
          s += "\nid < 0";
        }
        ::MessageBox(NULL,
                     s.c_str(),
                     "Warning - Sprite ID Usage",
                     MB_OK | MB_TASKMODAL);
      }
    }
  }
  int16 count = 0;
  // don't count idcode == 0's
  for (int16 i = 1;i<numids;i++)
  {
    if (!pidfound[i])
    {
      count++;
    }
  }
  if (count <= 0)
  {
    if (verbose)
    {
      ::MessageBox(NULL,
                   "all sprite ids assigned",
                   "Info - Sprite ID Usage",
                   MB_OK | MB_TASKMODAL);
    }
  }
  else
  {
    string title = "Warning - Sprite ID Usage";
    string *pstrmsg = new string[count + 2];
    pstrmsg[0] = "Mode Name: ";
    pstrmsg[0] += GetModeWin()->GetMode()->FileMode.Name.GetStr();
    pstrmsg[1] = "Sprite List Name: ";
    pstrmsg[1] += FileMode.Name.GetStr();
//    string pstrmsg[count];
    char buffer[50];
    int16 iter = 2;
    // don't check idcode == 0
    for (int16 i = 1;i<numids;i++)
    {
      if (!pidfound[i])
      {
        if (iter >= (count + 2))
        {
          pstrmsg[0] += "\niterated past pstrmsg size";
        }
        else
        {
          pstrmsg[iter] = "Missing ID Code: ";
          itoa(i,buffer,10);
          pstrmsg[iter] += buffer;
          if (psprites[i] != NULL)
          {
            pstrmsg[iter] += "psprite[id] != NULL";
            pstrmsg[iter] += psprites[i]->FileMode.Name.GetStr();
          }
          iter++;
        }
      }
    }
    TScrollMsgDlg smd(GetOgleApp()->GetFW(),title,pstrmsg,count + 2);
    smd.Execute();
    delete [] pstrmsg;
    pstrmsg = NULL;
  }
  delete [] psprites;
  psprites = NULL;
  delete [] pidfound;
  pidfound = NULL;
  
  return true;
}
#endif
//*******************************************************************
#ifdef DEBUGSPRITES
void TSpriteList::RTReport(ofstream &os)
{
  os<<"******************************************************\n";
  os<<"Start of Run Time SpriteList Report for " << FileMode.Name.GetStr() << "\n";
  int16 num = GetLastNumSprites();
  os<<"Last Sprite Index #: "<<num<<"\n\n";
  for (int16 i=0;i<=num;i++)
  {
    if (GetSpritePtr(i) != NULL)
    {
      if (GetSpritePtr(i)->GetStarted() && !GetSpritePtr(i)->GetStopped())
      {
        os << GetSpritePtr(i)->FileMode.Name.GetStr()
           << " ,index #"
           << i
           << " ,is going, Type: "
           << GetSpritePtr(i)->GetType()
           << "\n";
      }
    }
    else
    {
      os<<"Index #"<<i<<" was NULL"<<"\n";
    }
  }
  os<<"*******************************************************\n";
  //start active lists report.
  int16 nal = GetNumActiveLists();
  for (int16 j=0;j<nal;j++)
  {
    os<<"***********************************************\n";
    os<<"Active List # "<<j<<"\n";
    if (GetSpriteActiveList(j) != NULL) {GetSpriteActiveList(j)->RTReport(os);}
    else {os<<"List is NULL\n";}
  }
  os<<"**********************************************************\n";
}
#endif
//*******************************************************************
bool TSpriteList::SetCurrentSpriteSet(int16 typeindex,int16 setindex)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return false;}
  return (psal->SetCurrentSet(setindex));
}
//*******************************************************************
int16 TSpriteList::GetCurrentSpriteSetIndex(int16 typeindex)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return -1;}
  return psal->GetCurrentSetIndex();
}
//*******************************************************************
void TSpriteList::Activate(dword StartFUC)
{
  TOgleBase::Activate(StartFUC); // don't call TOgleList::Activate(StartFUC)
                                 // because it will go thru list and activate.
                                 // which will call TSpriteList::ActivateElement()
                                 // and start all sprites.

  int16 nal = GetNumActiveLists();
  for (int16 i=0;i<nal;i++)
  {
    TSpriteActiveList *psal = GetSpriteActiveList(i);
    if (psal != NULL) {psal->Activate(StartFUC);}
  }
}
//*******************************************************************
void TSpriteList::DeActivate()
{
  int16 nal = GetNumActiveLists();
  for (int16 i=0;i<nal;i++)
  {
    TSpriteActiveList *psal = GetSpriteActiveList(i);
    if (psal != NULL) {psal->DeActivate();}
  }
  TOgleBase::DeActivate();  // see Activate msg above.
}
//*******************************************************************
void TSpriteList::Show()
{
  TOgleListWAL::Show();
  int16 nal = GetNumActiveLists();
  for (int16 i=0;i<nal;i++)
  {
    TSpriteActiveList *psal = GetSpriteActiveList(i);
    if (psal != NULL) {psal->Show();}
  }
}
//*******************************************************************
void TSpriteList::NoShow()
{
  int16 nal = GetNumActiveLists();
  for (int16 i=0;i<nal;i++)
  {
    TSpriteActiveList *psal = GetSpriteActiveList(i);
    if (psal != NULL) {psal->NoShow();}
  }
  TOgleListWAL::NoShow();
}
//*******************************************************************
bool TSpriteList::AddSpriteSet(int16 typeindex)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return false;}
  return (psal->AddSet());
}
//*******************************************************************
bool TSpriteList::RemoveSpriteSet(int16 typeindex,int16 setindex)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return false;}
  return (psal->RemoveSet(setindex));
}
//*******************************************************************
bool TSpriteList::AddSpriteSetIdle(int16 typeindex,int16 setindex,const string &spritefname)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return false;}
  return (psal->AddSetIdle(setindex,spritefname));
}
//*******************************************************************
bool TSpriteList::RemoveSpriteSetIdle(int16 typeindex,int16 setindex,int16 idleindex)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return false;}
  return (psal->RemoveSetIdle(setindex,idleindex));
}
//*******************************************************************
int16 TSpriteList::GetNumSets(int16 typeindex)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return 0;}
  return (psal->GetNumSets());
}
//*******************************************************************
string TSpriteList::GetTypeName(int16 typeindex)
{
  if (GetActiveList(typeindex) == NULL) {return string("");}
  return (GetActiveList(typeindex)->GetName());
}
//*******************************************************************
int16 TSpriteList::GetSpriteSetNumIdles(int16 typeindex,int16 setindex)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return 0;}
  return (psal->GetSetNumIdles(setindex));
}
//*******************************************************************
string TSpriteList::GetSpriteSetIdleName(int16 typeindex,int16 setindex,int16 idleindex)
{
  TSpriteActiveList *psal = GetSpriteActiveList(typeindex);
  if (psal == NULL) {return string("");}
  return (psal->GetSetIdleName(setindex,idleindex));
}
//*******************************************************************
void TSpriteList::Slenderize()
{
  for (int16 i=0;i<=LastAddedElementNum;i++)
  {
    TSprite *ps = GetSpritePtr(i);
    if (ps != NULL) {ps->Slenderize();}
  }
}
//*******************************************************************
//string TSpriteList::GetSpriteFName(int16 SpriteIndex)
string TSpriteList::GetSpriteName(int16 spriteindex)
{
  TSprite *ps = GetSpritePtr(spriteindex);
  if (ps == NULL) {return TAppString::strEmpty;}
//  return (ps->GetFName());
  return ps->FileMode.Name.GetStr();
}
//*******************************************************************
TSprite *TSpriteList::CreateSprite(string spritename,int16 id,void *psource)
{
  return GetGObjSp()->CreateSprite(spritename,id,psource);
}
//*******************************************************************
bool TSpriteList::DeleteSprite(const string &SpriteName)
{
  int16 si = GetSpriteIndex(SpriteName);
  if (si >= 0)
  {
    RemoveElement(si);
    // make sure skit's indexes are still correct.
    PModeWin->SetIndexes();
    return true;
  }
  else
  {
    return false;
  }
}
//*******************************************************************
#if defined(SUPPORT_OLD_VER)
bool TSpriteList::ReadLocal1_2()
{
  TOgleGroupGaugeDlg *pggd = NULL;

  // start - MRP v2.1
  TOgleFileName ofn;
  string dirpath = GetFileMgr()->PDat->GetDirName(FileMode.GetDirIndex());
  ofn.SetDirPath(dirpath);
  // end

  uint16 NumSprites;
  #ifdef SPL_READWASP
  string *names;
  #endif
  if (!(Readuint16(&NumSprites))) {return false;}
  GrowBy(NumSprites); // MRP 12/12/96
  #ifdef SPL_READWASP
  names = new string [NumSprites];
  #endif

  string str;
  if (NumSprites > 0)
  {
    // MRP - 07/16/99 - v2.3 - fake spriteid's on older data files.
    if (GetGObjSp() != NULL)
    {
      GetGObjSp()->SetupUniqueIDStrings();
    }

    #ifndef SPL_READWASP
    pggd = GetGaugeDlg("Loading Sprites");
    pggd->SetActiveGauge(1);
    pggd->SetGaugeRange(0,NumSprites - 1);
    #endif

    for (int16 i=0;i<NumSprites;i++)
    {
      if (!(ReadFNameString(str))) {return false;}
      // start filename mod's - MRP v2.1
      ofn.SetFName(str);
      ofn.SetExt(string(".sp"));
      str = ofn.GetStr();
      // end
      #ifndef SPL_READWASP
      pggd->SetGaugeText(str);

      // MRP - 07/16/99 - v2.3 - fake spriteid's
      int16 id = 0;
      if (GetGObjSp() != NULL)
      {
        id = GetGObjSp()->GetUniqueID(str);
      }
      AddElement(CreateSprite(str,id));
      // end changes.
      pggd->IncGaugeValue();
      #else
      names[i] = str;
      #endif
    }
    #ifndef SPL_READWASP
    ReleaseGaugeDlg(pggd);
    pggd = NULL;
    #endif
  }

  int16 tempnumtypes;
  if (!Readint16(&tempnumtypes)) {return false;}
  string spritetypename;
  int16 tempnumsets;
  for (int16 j = 1;j<tempnumtypes;j++) // don't read sprite type 0 ("none").
  {
    if (!ReadTypeNameString(spritetypename)) {return false;}
    AddActiveList(spritetypename);
    if (!Readint16(&tempnumsets)) {return false;}

    int16 tempnumidles;
    for (int16 k = 0;k<tempnumsets;k++)
    {
      AddSpriteSet(j);
      if (!Readint16(&tempnumidles)) {return false;}
      string tempspritename;
      for (int16 l = 0;l<tempnumidles;l++)
      {
        if (!ReadFNameString(tempspritename)) {return false;}
        // start filename mod's - MRP v2.1
        ofn.SetFName(tempspritename);
        tempspritename = ofn.GetStr();
        // end
        AddSpriteSetIdle(j,k,tempspritename);
      }
    }
  }

  // read sprite data from end of spritelist file.
  #ifdef SPL_READWASP
  if (NumSprites > 0)
  {
    pggd = GetGaugeDlg("Loading Sprites");
    pggd->SetActiveGauge(1);
    pggd->SetGaugeRange(0,NumSprites - 1);
    for (int16 j=0;j<NumSprites;j++)
    {
      pggd->SetGaugeText(names[j]);

      // MRP - 07/16/99 - v2.3
      int16 id = 0;
      if (GetGObjSp() != NULL)
      {
        id = GetGObjSp()->GetUniqueID(str);
      }
      TSprite *psprite = CreateSprite(names[j],id,GetPtr());
      FileOffset += psprite->ReadSize;
      AddElement(psprite);
      pggd->IncGaugeValue();
    }
    ReleaseGaugeDlg(pggd);
    pggd = NULL;
  }
  delete [] names;
  #endif

  Sort();

  return true;
}
#endif
//*******************************************************************
#if defined(SUPPORT_OLD_VER)
bool TSpriteList::ReadLocal1_3()
{
  TOgleGroupGaugeDlg *pggd = NULL;

  // start filenname mod's - MRP v2.1
  TOgleFileName ofn;
  string dirpath = GetFileMgr()->PDat->GetDirName(FileMode.GetDirIndex());
  ofn.SetDirPath(dirpath);
  // end

  uint16 NumSprites;
  #ifdef SPL_READWASP
  string *names;
  int16 *idcodes;
  #endif
  if (!(Readuint16(&NumSprites))) {return false;}
  GrowBy(NumSprites); // MRP 12/12/96
  #ifdef SPL_READWASP
  names = new string [NumSprites];
  idcodes = new int16[NumSprites];
  #endif

  string str;
  if (NumSprites > 0)
  {
    // MRP - v2.3 - 07/16/99
    // work around for internal in-between wad builds
    // before ID's actually set correctly.
    #if defined(CASINO_COMPILE)
    if (GetGObjSp() != NULL)
    {
      GetGObjSp()->SetupUniqueIDStrings();
    }
    #endif

    #ifndef SPL_READWASP
    pggd = GetGaugeDlg("Loading Sprites");
    pggd->SetActiveGauge(1);
    pggd->SetGaugeRange(0,NumSprites - 1);
    #endif
    int16 id = 0;
    for (int16 i=0;i<NumSprites;i++)
    {
      if (!(ReadFNameString(str))) {return false;}

      // start filename mod's - MRP v2.1
      ofn.SetFName(str);
      ofn.SetExt(string(".sp"));
      str = ofn.GetStr();
      // end

      if (!Readint16(&id)) {return false;}

      // MRP - v2.3 - 07/19/99
      #if defined(CASINO_COMPILE)
      if (GetGObjSp() != NULL)
      {
        id = GetGObjSp()->GetUniqueID(str);
      }
      #endif

      #ifndef SPL_READWASP
      pggd->SetGaugeText(str);
      AddElement(CreateSprite(str,id));
      pggd->IncGaugeValue();
      #else
      names[i] = str;
      idcodes[i] = id;
      #endif
    }
    #ifndef SPL_READWASP
    ReleaseGaugeDlg(pggd);
    pggd = NULL;
    #endif
  }

  int16 tempnumtypes;
  if (!Readint16(&tempnumtypes)) {return false;}
  string spritetypename;
  int16 tempnumsets;
  for (int16 j = 1;j<tempnumtypes;j++) // don't read sprite type 0 ("none").
  {
    if (!ReadTypeNameString(spritetypename)) {return false;}
    AddActiveList(spritetypename);
    if (!Readint16(&tempnumsets)) {return false;}

    int16 tempnumidles;
    for (int16 k = 0;k<tempnumsets;k++)
    {
      AddSpriteSet(j);
      if (!Readint16(&tempnumidles)) {return false;}
      string tempspritename;
      for (int16 l = 0;l<tempnumidles;l++)
      {
        if (!ReadFNameString(tempspritename)) {return false;}

        // start filename mod's - MRP v2.1
        ofn.SetFName(tempspritename);
        tempspritename = ofn.GetStr();
        // end

        AddSpriteSetIdle(j,k,tempspritename);
      }
    }
  }

  // read sprite data from end of spritelist file.
  #ifdef SPL_READWASP
  if (NumSprites > 0)
  {
    pggd = GetGaugeDlg("Loading Sprites");
    pggd->SetActiveGauge(1);
    pggd->SetGaugeRange(0,NumSprites - 1);
    for (int16 j=0;j<NumSprites;j++)
    {
      pggd->SetGaugeText(names[j]);
      TSprite *psprite = CreateSprite(names[j],idcodes[j],GetPtr());
      FileOffset += psprite->ReadSize;
      AddElement(psprite);
      pggd->IncGaugeValue();
    }
    ReleaseGaugeDlg(pggd);
    pggd = NULL;
  }
  delete [] names;
  delete [] idcodes;
  #endif

  Sort();

  return true;
}
#endif
//*******************************************************************
bool TSpriteList::ReadLocal2_0()
// also used for ver 2.3
{
  TOgleGroupGaugeDlg *pggd = NULL;


  uint16 NumSprites;
  #ifdef SPL_READWASP
  string *names;
  int16 *idcodes;
  #endif
  if (!(Readuint16(&NumSprites))) {return false;}
  GrowBy(NumSprites); // MRP 12/12/96
  #ifdef SPL_READWASP
  names = new string [NumSprites];
  idcodes = new int16[NumSprites];
  #endif

  string str;
  if (NumSprites > 0)
  {
    // MRP - v2.3 - 07/19/99
    #if defined(CASINO_COMPILE) && defined(SUPPORT_OLD_VER)
    if (GetGObjSp() != NULL)
    {
      GetGObjSp()->SetupUniqueIDStrings();
    }
    #endif

    #ifndef SPL_READWASP
    pggd = GetGaugeDlg("Loading Sprites");
    pggd->SetActiveGauge(1);
    pggd->SetGaugeRange(0,NumSprites - 1);
    #endif
    int16 id = 0;
    for (int16 i=0;i<NumSprites;i++)
    {
      if (!ReadFileName(str)) {return false;}

      //try to remove spaces from sprite file names.
      #if defined(GED_COMPILE)
      TOgleFileName ofn(str);
      string s = ofn.GetFName();
      s = s.strip(string::Both);
      ofn.SetFName(s);
      str = ofn.GetStr();
      #endif

      if (!Readint16(&id)) {return false;}
      // MRP - v2.3 - 07/19/99
      #if defined(CASINO_COMPILE) && defined(SUPPORT_OLD_VER)
      if (id <= 0 && GetGObjSp() != NULL)
      {
        id = GetGObjSp()->GetUniqueID(str);
      }
      #endif

      #ifndef SPL_READWASP
      pggd->SetGaugeText(str);
      AddElement(CreateSprite(str,id));
      pggd->IncGaugeValue();
      #else
      names[i] = str;
      idcodes[i] = id;
      #endif
    }
    #ifndef SPL_READWASP
    ReleaseGaugeDlg(pggd);
    pggd = NULL;
    #endif
  }

  int16 tempnumtypes;
  if (!Readint16(&tempnumtypes)) {return false;}
  string spritetypename;
  int16 tempnumsets;
  for (int16 j = 1;j<tempnumtypes;j++) // don't read sprite type 0 ("none").
  {
    if (!ReadTypeNameString(spritetypename)) {return false;}
    AddActiveList(spritetypename);
    if (!Readint16(&tempnumsets)) {return false;}

    int16 tempnumidles;
    for (int16 k = 0;k<tempnumsets;k++)
    {
      AddSpriteSet(j);
      if (!Readint16(&tempnumidles)) {return false;}
      string tempspritename;
      for (int16 l = 0;l<tempnumidles;l++)
      {
        if (!ReadFileName(tempspritename)) {return false;}
        AddSpriteSetIdle(j,k,tempspritename);
      }
    }
  }

  // read sprite data from end of spritelist file.
  #ifdef SPL_READWASP
  if (NumSprites > 0)
  {
    pggd = GetGaugeDlg("Loading Sprites");
    pggd->SetActiveGauge(1);
    pggd->SetGaugeRange(0,NumSprites - 1);
    for (int16 j=0;j<NumSprites;j++)
    {
      pggd->SetGaugeText(names[j]);
      TSprite *psprite = CreateSprite(names[j],idcodes[j],GetPtr());
      FileOffset += psprite->ReadSize;
      AddElement(psprite);
      pggd->IncGaugeValue();
    }
    ReleaseGaugeDlg(pggd);
    pggd = NULL;
  }
  delete [] names;
  delete [] idcodes;
  #endif

  Sort();

  return true;
}
//*******************************************************************
bool TSpriteList::ReadLocal()
{
  switch (MajVer)
  {
    case 1:
           switch (MinVer)
           {
             case 2:return ReadLocal1_2();
             case 3:return ReadLocal1_3();
             default: return false;
           }
    case 2:
           switch (MinVer)
           {
             case 0:return ReadLocal2_0();
             case 3:return ReadLocal2_0(); // just inc'd for re-releases.
             default: return false;
           }
    default: return false;
  }
}
//*******************************************************************
bool TSpriteList::WriteLocal()
// MRP 5/29/97 - added idcode stuff.
{
  int16 idcode = 0;
  string tempStr;
  uint16 NumSprites = uint16(LastAddedElementNum + 1);
  if (!Writeuint16(&NumSprites)) {return false;}
  for (int16 i = 0;i<=LastAddedElementNum;i++)
  {
    TSprite *psprite = GetSpritePtr(i);
    assert(psprite != NULL);
//    tempStr = psprite->GetBaseName();
    tempStr = psprite->FileMode.Name.GetStr();
    if (!WriteFileName(tempStr)) {return false;}
    idcode = psprite->GetIDCode();
    if (!Writeint16(&idcode)) {return false;}
  }

  int16 NumTypes = GetNumActiveLists();
  if (!Writeint16(&NumTypes)) {return false;}
  for (int16 j = 1;j<NumTypes;j++)  // don't write type 0 ("none").
  {
    string temptypename = GetTypeName(j);
    if (!WriteTypeNameString(temptypename)) {return false;}

    int16 numsets = GetNumSets(j);
    if (!Writeint16(&numsets)) {return false;}

    for (int16 k=0;k<numsets;k++)
    {
      int16 numidles = GetSpriteSetNumIdles(j,k);
      if (!Writeint16(&numidles)) {return false;}

      for (int16 l=0;l<numidles;l++)
      {
        string spritename = GetSpriteSetIdleName(j,k,l);
        if (!WriteFileName(spritename)) {return false;}
      }
    }
  }

  // will write sprites to end of spritelist file.
  for (int16 k = 0;k<=LastAddedElementNum;k++)
  {
    if (GetSpritePtr(k) == NULL) {return false;}
    if (!GetSpritePtr(k)->Write(PWriteBuf)) {return false;}
  }

  return true;
}
//*******************************************************************
bool TSpriteList::Save()
{
  for (int16 i = 0;i <= LastAddedElementNum;i++)
  {
    TSprite *psprite = GetSpritePtr(i);
    if (psprite != NULL) {psprite->Save();}
  }
  Write();
  return true;
}
//*******************************************************************
TSprite *TSpriteList::CreateNewElement()
{
  return (new TSprite(PModeWin));
}
//*******************************************************************
TSpriteList::TSpriteList(TOgleModeWindow *pmw,TGObjSp *PGObj)
  :TOgleListWAL(pmw)
{
  #ifdef DEBUG_PMODEWIN_SPL
  if (PModeWin == NULL)
  {
    string s = "PModeWin == NULL in ";
    s += FileMode.Name;
    s += "'s ctor";
    ::MessageBox(NULL,s.c_str(),"Warning - TSpriteList 1",MB_OK);
  }
  #endif

  POwnerGObj = PGObj;
  string nonetypename = "none";
  AddActiveList(nonetypename);
}
//*******************************************************************
TSpriteList::TSpriteList(TOgleModeWindow *pmw,const TFileMode &fm,TGObjSp *PGObj,
                         bool AutoRead,bool KeepBaseFile)
  :TOgleListWAL(pmw)
{
  #ifdef DEBUG_PMODEWIN_SPL
  if (PModeWin == NULL)
  {
    string s = "PModeWin == NULL in ";
    s += FileMode.Name;
    s += "'s ctor";
    ::MessageBox(NULL,s.c_str(),"Warning - TSpriteList 2",MB_OK);
  }
  #endif

  POwnerGObj = PGObj;
  string nonetypename = "none";
  AddActiveList(nonetypename);
  Init(fm,AutoRead,KeepBaseFile);
}
//*******************************************************************
#pragma argsused
TSpriteList::TSpriteList(TOgleModeWindow *pmw,bool Dummy,const TFileMode &fm,TGObjSp *PGObj)
  :TOgleListWAL(pmw)
{
  #ifdef DEBUG_PMODEWIN_SPL
  if (PModeWin == NULL)
  {
    string s = "PModeWin == NULL in ";
    s += FileMode.Name;
    s += "'s ctor";
    ::MessageBox(NULL,s.c_str(),"Warning - TSpriteList 3",MB_OK);
  }
  #endif

  POwnerGObj = PGObj;
  string nonetypename = "none";
  AddActiveList(nonetypename);
}
//*******************************************************************
TSpriteList::~TSpriteList()
{
}
//*******************************************************************
TOgleActiveList *TSpriteList::CreateActiveList(const string &name)
{
  return (new TSpriteActiveList(PModeWin,POwnerGObj,name));
}
//*******************************************************************
void TSpriteList::SetIndexes()
{
  for (int16 i=0;i<GetNumActiveLists();i++)
  {
    TSpriteActiveList *psal = GetSpriteActiveList(i);
    if (psal != NULL) {psal->SetIndexes();}
  }
}
//*******************************************************************
bool TSpriteList::SetSpriteIndexTo(int16 SpriteIndex,TSprite *PSprite)
{
  if (SpriteIndex > LastAddedElementNum) {return false;}
  if (SpriteIndex < 0) {return false;}
  List[SpriteIndex] = PSprite;
  return true;
}
//*******************************************************************
TSprite *TSpriteList::SwapSprites(int16 SpriteIndex,TSprite *PSprite)
{
  TSprite *PTempSprite = GetSpritePtr(SpriteIndex);
  if (PTempSprite == NULL) {return NULL;}

  if (PSprite == NULL) {return NULL;}

  if (!SetSpriteIndexTo(SpriteIndex,PSprite)) {return NULL;}
  SetIndexes();  // Make sure idles have a correct "tsprite *".
  return (PTempSprite);
}
//*******************************************************************
bool TSpriteList::CanActivateElement(TOgleBase *PBase,dword StartFUC)
{
  if (PBase == NULL) {return false;}
  dword EndFUC = StartFUC + PBase->Get_FUC_Length();
  int16 sprtype = GetActiveListIndex(PBase);
  TSpriteActiveList *psal = GetSpriteActiveList(sprtype);
  if (psal == NULL) {return false;}

  if (sprtype == 0) {return true;} // 0 is none, so anything goes.
  return (psal->IsRangeAvail(StartFUC,EndFUC));
}
//*******************************************************************

