//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  mstate.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     5/23/95    M.Perry     original baseline
//  2.0    08/27/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__mstate_h)
#include "mstate.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if !defined(__omode_h)
#include "omode.h"
#endif

#if !defined(__oscrnmgr_h)
#include "oscrnmgr.h"
#endif

#if defined(DEBUGSPRITES)
#if !defined(__IOSTREAM_H)
  #include <iostream.h>
#endif
#if !defined(__FSTREAM_H)
  #include <fstream.h>
#endif
#endif
//*******************************************************************
TModeState::TModeState(TOgleModeWindow *PNewModeWindow)
{
 #if defined(ENB_FUC_STEPS)
 FUC_Stepping = false;
 Got_FUC_Key = false;
 #endif

 POwnerWindow = PNewModeWindow;

 #ifdef STATEDEBUG
 strcpy(ClassName,"TModeState");
 #endif
}
//*******************************************************************
TModeState::~TModeState()
{
}
//*******************************************************************
TOgleScreenMgr *TModeState::GetScreenMgr()
{
  if (POwnerWindow == NULL) {return NULL;}
  TOgleMode *pm = POwnerWindow->GetMode();
  if (pm == NULL) {return NULL;}
  return pm->GetScreenMgr();
}
//*******************************************************************
#if defined(ENB_FUC_STEPS)
bool TModeState::Is_FUC_Stepping()
{
  if (PActiveSubState != NULL)
  {
    return ((TModeState *)PActiveSubState)->Is_FUC_Stepping();
  }
  else
  {
    return FUC_Stepping;
  }
}
#endif
//*******************************************************************
void TModeState::Timer()
{
  if (Status != ss_Running) {return;}

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->Timer();
  }
  else if (!HandleTimer())
  {
    POwnerWindow->Update();
  }
}
//*******************************************************************
bool TModeState::HandleTimer()
{
  #if defined(ENB_FUC_STEPS)
  if (FUC_Stepping)
  {
    if (Got_FUC_Key)
    {
      // wait until actually updates before clearing flag.
      if (POwnerWindow->Update())
      {
        Got_FUC_Key = false;
      }
    }
    return true;
  }
  #endif

  return false;
}
//*******************************************************************
void TModeState::LButtonDown(uint modKeys, TPoint &point)
{
  if (Status != ss_Running)
  {
    return;
  }

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->LButtonDown(modKeys,point);
  }
  else if (!HandleLDown(modKeys,point))
  {
//    if (POwnerWindow != NULL && POwnerWindow->PScreenMgr != NULL)
//    {
//      POwnerWindow->PScreenMgr->LDown(modKeys,point);
//    }
    TOgleScreenMgr *psm = GetScreenMgr();
    if (psm != NULL)
    {
      psm->LDown(modKeys,point);
    }
  }
}
//*******************************************************************
void TModeState::LButtonUp(uint modKeys,TPoint &point)
{
  if (Status != ss_Running) {return;}

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->LButtonUp(modKeys,point);
  }
  else if (!HandleLUp(modKeys,point))
  {
//    if (POwnerWindow != NULL && POwnerWindow->PScreenMgr != NULL)
//    {
//      POwnerWindow->PScreenMgr->LUp(modKeys,point);
//    }
    TOgleScreenMgr *psm = GetScreenMgr();
    if (psm != NULL)
    {
      psm->LUp(modKeys,point);
    }
  }
}
//*******************************************************************
void TModeState::LButtonDblClk(uint modKeys, TPoint &point)
{
  if (Status != ss_Running) {return;}

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->LButtonDblClk(modKeys,point);
  }
  else if (!HandleLDblClk(modKeys,point))
  {
//    if (POwnerWindow != NULL && POwnerWindow->PScreenMgr != NULL)
//    {
//      POwnerWindow->PScreenMgr->LDblClk(modKeys,point);
//    }
    TOgleScreenMgr *psm = GetScreenMgr();
    if (psm != NULL)
    {
      psm->LDblClk(modKeys,point);
    }
  }
}
//*******************************************************************
void TModeState::RButtonDown(uint modKeys, TPoint &point)
{
  if (Status != ss_Running) {return;}

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->RButtonDown(modKeys,point);
  }
  else if (!HandleRDown(modKeys,point))
  {
//    if (POwnerWindow != NULL && POwnerWindow->PScreenMgr != NULL)
//    {
//      POwnerWindow->PScreenMgr->RDown(modKeys,point);
//    }
    TOgleScreenMgr *psm = GetScreenMgr();
    if (psm != NULL)
    {
      psm->RDown(modKeys,point);
    }
  }
}
//*******************************************************************
void TModeState::RButtonUp(uint modKeys,TPoint &point)
{
  if (Status != ss_Running) {return;}

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->RButtonUp(modKeys,point);
  }
  else if (!HandleRUp(modKeys,point))
  {
//    if (POwnerWindow != NULL && POwnerWindow->PScreenMgr != NULL)
//    {
//      POwnerWindow->PScreenMgr->RUp(modKeys,point);
//    }
    TOgleScreenMgr *psm = GetScreenMgr();
    if (psm != NULL)
    {
      psm->RUp(modKeys,point);
    }
  }
}
//*******************************************************************
void TModeState::RButtonDblClk(uint modKeys,TPoint &point)
{
  if (Status != ss_Running) {return;}

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->RButtonDblClk(modKeys,point);
  }
  else if (!HandleRDblClk(modKeys,point))
  {
//    if (POwnerWindow != NULL && POwnerWindow->PScreenMgr != NULL)
//    {
//      POwnerWindow->PScreenMgr->RDblClk(modKeys,point);
//    }
    TOgleScreenMgr *psm = GetScreenMgr();
    if (psm != NULL)
    {
      psm->RDblClk(modKeys,point);
    }
  }
}
//*******************************************************************
void TModeState::MouseMove(uint modKeys,TPoint &point)
{
  if (Status != ss_Running) {return;}

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->MouseMove(modKeys,point);
  }
  else if (!HandleMouseMove(modKeys,point))
  {
//    if (POwnerWindow != NULL && POwnerWindow->PScreenMgr != NULL)
//    {
//      POwnerWindow->PScreenMgr->MouseMove(modKeys,point);
//    }
    TOgleScreenMgr *psm = GetScreenMgr();
    if (psm != NULL)
    {
      psm->MouseMove(modKeys,point);
    }
  }
}
//*******************************************************************
void TModeState::Char(uint key,uint repeatCount,uint flags)
{
  if (Status != ss_Running) {return;}

  if (PActiveSubState != NULL)
  {
    ((TModeState *)PActiveSubState)->Char(key,repeatCount,flags);
  }
  else if (!HandleChar(key,repeatCount,flags))
  {
//    if (POwnerWindow != NULL && POwnerWindow->PScreenMgr != NULL)
//    {
//      POwnerWindow->PScreenMgr->Char(key,repeatCount,flags);
//    }
    TOgleScreenMgr *psm = GetScreenMgr();
    if (psm != NULL)
    {
      psm->Char(key,repeatCount,flags);
    }
  }
}
//*******************************************************************
#pragma argsused
bool TModeState::HandleChar(uint key,uint repeatCount,uint flags)
{
  #if defined(DEBUGSPRITES)
  if ((unsigned char)key == '?')
  {
    POwnerWindow->RTReport();
    return false; // don't filter out.
  }
  #endif

  #ifdef DEBUGUPDATES
  if ((unsigned char)key == 'T')
  {
    POwnerWindow->ShowTime();
    return false;  // don't filter out.
  }
  #endif

  #if defined(ENB_FUC_STEPS)
  if (FUC_Stepping && key == VK_SPACE)
  {
    Got_FUC_Key = true;
    return false; // don't filter out.
  }
  if ((unsigned char)key == 'F')
  {
    if (FUC_Stepping)
    {
      FUC_Stepping = false;
      Got_FUC_Key = false;
    }
    else
    {
      FUC_Stepping = true;
      Got_FUC_Key = false;
    }
    return false;
  }
  #endif

  return false;
}
//*******************************************************************

