//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  ofile.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/23/96    M.Perry     original baseline
//  2.0     8/26/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__ofile_h)
#include "ofile.h"
#endif

#if !defined(__omemmgr_h)
#include "omemmgr.h"
#endif
//*******************************************************************
//TOgleFile::TOgleFile(const string &pathName,uint fuMode,bool shouldOpen,bool shouldClose)
TOgleFile::TOgleFile(const TOgleFileName &ofn,dword access,dword share,
                     dword creation,dword os,dword attributes,bool shouldOpen,
                     bool shouldClose)
:FileName(ofn),
 AccessFlags(access),
 ShareFlags(share),
 CreationFlags(creation),
 OSFlags(os),
 AttributeFlags(attributes),
 ShouldClose(shouldClose),
 FileHandle(OF_INVALID_HANDLE),
 FileSize(0)
{
//  SetFileHandle(HFILE_ERROR);
//  SetFileHandle(OF_INVALID_HANDLE);
//  SetFileSize(0);
//  SetFileName(ofn);//pathName);
//  SetOpenFlags(fuMode);
//  SetShouldClose(shouldClose);
  if (shouldOpen) {Open();}
}
//*******************************************************************
TOgleFile::~TOgleFile()
{
  if (IsOpen() && GetShouldClose()) {Close();}
}
//*******************************************************************
bool TOgleFile::Open()
{
  if (IsOpen()) {return true;}
  SetFileSize(0);

  #if defined(OGLE_PLAT_OS_WIN16)
  OFSTRUCT ofstruct;
  ofstruct.cBytes = sizeof(OFSTRUCT);
  SetFileHandle(OpenFile(GetFileName().c_str(),
                         &ofstruct,
                         GetOpenFlags()));
  #elif defined(OGLE_PLAT_OS_WIN32)
  SetFileHandle(CreateFile(GetFileName().GetStr().c_str(),
                           AccessFlags,
                           ShareFlags,
                           NULL,
                           CreationFlags,
                           OSFlags | AttributeFlags,
                           NULL));
  #endif
  return IsOpen();
}
//*******************************************************************
//bool TOgleFile::Open(const string &pathName,uint fuMode,bool shouldClose)
//{
//  SetShouldClose(shouldClose);
//  if (pathName != GetFileName() || fuMode != GetOpenFlags())
//  {
//    if (IsOpen()) {Close();}
//    SetFileName(pathName);
//    SetOpenFlags(fuMode);
//    return Open();
//  }
//  else
//  {
//    if (!IsOpen()) {return Open();}
//    else {return true;}
//  }
//}
//*******************************************************************
bool TOgleFile::Open(const TOgleFileName &ofn,
                     dword access,
                     dword share,
                     dword creation,
                     dword os,
                     dword attributes,
                     bool shouldClose)
{
  if (IsOpen()) {return false;} // already open, so probably a user error.
  
  SetShouldClose(shouldClose);
  FileName = ofn;
  AccessFlags = access;
  ShareFlags = share;
  CreationFlags = creation;
  OSFlags = os;
  AttributeFlags = attributes;
  return Open();
}
//*******************************************************************
int32 TOgleFile::Seek(int32 offset, TSeekDir origin)
{
  if (!IsOpen()) {return OF_INVALID_SEEK;}

  #if defined(OGLE_PLAT_OS_WIN16)

    return _llseek(GetFileHandle(),offset,origin);

  #elif defined(OGLE_PLAT_OS_WIN32)

    int32 rc = OF_INVALID_SEEK;
    dword dw = SetFilePointer(GetFileHandle(),offset,NULL,origin);
    if (dw != OF_OS_INVALID_SEEK)
    {
      rc = int32(dw);
    }
    return rc;

  #endif
}
//*******************************************************************
bool TOgleFile::Read(void huge *buffer,int32 numBytes,int32 &bytesRead)
{
  #if defined(OGLE_PLAT_OS_WIN16)
  bytesRead =  _hread(GetFileHandle(),buffer,numBytes);
  #elif defined(OGLE_PLAT_OS_WIN32)
  uint32 br = uint32(bytesRead);
  bool rc = (bool)ReadFile(GetFileHandle(),buffer,numBytes,&br,NULL);
  bytesRead += int32(br);
  #endif
  return bool(bytesRead == numBytes);
}
//*******************************************************************
bool TOgleFile::Write(const void huge *buffer,int32 numBytes,int32 &bytesWritten)
{
//  #if defined(BI_PLAT_WIN16)
  #if defined(OGLE_PLAT_OS_WIN16)
  bytesWritten = _hwrite(GetFileHandle(),buffer,numBytes);
  return bool(bytesWritten == numBytes);
//  #else  // if BI_PLAT_WIN32
  #elif defined(OGLE_PLAT_OS_WIN32)
//  bytesWritten = _hwrite(GetFileHandle(),(const char *)buffer,numBytes);
  uint32 bw = uint32(bytesWritten);
  bool rc = (bool)WriteFile(GetFileHandle(),buffer,numBytes,&bw,NULL);
  bytesWritten += int32(bw);
  return bool(rc && numBytes == bytesWritten);
  #endif
}
//*******************************************************************
bool TOgleFile::Close()
{
  bool result = true;
  if (IsOpen())
  {
    #if defined(OGLE_PLAT_OS_WIN16)
    TFileHandle hfile = _lclose(GetFileHandle());
    result = bool(hfile == 0);
    #elif defined(OGLE_PLAT_OS_WIN32)
    result = (bool)CloseHandle(GetFileHandle());
    #endif
  }
//  SetFileHandle(HFILE_ERROR);
  SetFileHandle(OF_INVALID_HANDLE);
  return result;
}
//*******************************************************************
int32 TOgleFile::GetFileSize()
{
  if (FileSize > 0) {return FileSize;}
  if (!IsOpen())
  {
    if (!Open()) {return FileSize;}
    int32 curpos = GetPosition();
    SetFileSize(SeekToEnd());
    Seek(curpos,OFSD_BEGIN);            // put file pointer back.
    Close();
  }
  else
  {
    int32 curpos = GetPosition();
    SetFileSize(SeekToEnd());
    Seek(curpos,OFSD_BEGIN);
  }
  return FileSize;
}
//*******************************************************************
//bool TOgleFile::Rename(const string &newname,const string &oldname)
//{
//  if (IsOpen()) {return false;}
//  if (::rename(oldname.c_str(),newname.c_str()) != 0) {return false;}
//  SetFileName(newname);
//  return true;
//}
//*******************************************************************
bool TOgleFile::Rename(const TOgleFileName &ofn)
{
  return Rename(ofn.GetStr());
}
//*******************************************************************
bool TOgleFile::Rename(const string &s)
{
  bool result = false;
  if (IsOpen()) {return false;}
  #if defined(OGLE_PLAT_OS_WIN16)
  result = bool(::rename(FileName.GetStr().c_str(),s.c_str()) == 0);
  #elif defined(OGLE_PLAT_OS_WIN32)
  result = (bool)MoveFile(FileName.GetStr().c_str(),s.c_str());
  #endif
  if (result) {SetFileName(s);}
  return result;
}
//*******************************************************************
//*******************************************************************
TFileWriteBuf::TFileWriteBuf(const TOgleFileName &name,int32 bufsize)
:TOgleFile(name,
           OFA_WRITE,
           OFS_EXCLUSIVE,
           OFC_CREATE_ALWAYS,
           OFO_NONE,
           OFAT_NORMAL,
           false), // don't let base auto open file.
//:FileName(name),
// FileHandle(HFILE_ERROR),
 BufferSize(bufsize),
 PBuffer(NULL),
 BufferOffset(0)
{
  if (BufferSize > 0)
  {
//    PBuffer = GlobalAllocPtr(GMEM_MOVEABLE,BufferSize);
    PBuffer = TOgleMemMgr::AllocPtr(GMEM_MOVEABLE,BufferSize);
  }
  Open();
}
//*******************************************************************
TFileWriteBuf::~TFileWriteBuf()
{
  Close();
  if (PBuffer != NULL)
  {
//    GlobalFreePtr(PBuffer);
    TOgleMemMgr::DeAllocPtr(PBuffer);
  }
  PBuffer = NULL;
}
//*******************************************************************
bool TFileWriteBuf::Write(const void huge *ptr,int32 size)
{
  if (PBuffer != NULL)
  {
    int32 leftover = BufferSize - BufferOffset;
    if (size >= leftover)
    {
      if (!WriteToFile()) {return false;}
    }

    if (size >= BufferSize) // too big for buffer, write directly to disk.
    {
      if (!GetFileHandleValid()) {return false;}
//      if (FileHandle == HFILE_ERROR) {return false;}
//      #if defined(BI_PLAT_WIN16)
//      int32 numwritten = _hwrite(FileHandle,ptr,size);
//      return bool(numwritten == size);
//      #else // if BI_PLAT_WIN32
//      int32 numwritten = _hwrite(FileHandle,(const char *)ptr,size);
      return TOgleFile::Write(ptr,size);
//      #endif
//      return (numwritten == size);
    }
    else   // fits in buffer.
    {
      hmemcpy(((byte huge *)PBuffer) + BufferOffset,ptr,size);
      BufferOffset += size;
      return true;
    }
  }
  else // no buffer
  {
//    if (FileHandle == HFILE_ERROR) {return false;}
    if (!GetFileHandleValid()) {return false;}
//    #if defined(BI_PLAT_WIN16)
//    int32 numwritten = _hwrite(FileHandle,ptr,size);
//    #else // BI_PLAT_WIN32
//    int32 numwritten = _hwrite(FileHandle,(const char *)ptr,size);
//    #endif
//    return (numwritten == size);
    return TOgleFile::Write(ptr,size);
  }
}
//*******************************************************************
bool TFileWriteBuf::WriteToFile()
{
//  if (FileHandle == HFILE_ERROR) {return false;}
  if  (!GetFileHandleValid()) {return false;}
  if (BufferOffset == 0) {return true;}

//  #if defined(BI_PLAT_WIN16)
//  int32 numwritten = _hwrite(FileHandle,PBuffer,BufferOffset);
//  #else // BI_PLAT_WIN32
//  int32 numwritten = _hwrite(FileHandle,(const char *)PBuffer,BufferOffset);
//  #endif
//  bool rc = (numwritten == BufferOffset);
  bool rc = TOgleFile::Write(PBuffer,BufferOffset);
  BufferOffset = 0;
  return rc;
}
//*******************************************************************
bool TFileWriteBuf::Flush()
{
  return WriteToFile();
}
//*******************************************************************
//bool TFileWriteBuf::Open()
//{
//  OFSTRUCT ofstruct;
//  ofstruct.cBytes = sizeof(OFSTRUCT);
//  FileHandle = OpenFile(FileName.c_str(),&ofstruct,OF_CREATE | OF_WRITE);
//  if (FileHandle == HFILE_ERROR) {return false;}
//  return true;
//}
//*******************************************************************
bool TFileWriteBuf::Close()
{
  Flush();
  return TOgleFile::Close();
//  if (_lclose(FileHandle) == HFILE_ERROR) {return false;}
//  FileHandle = HFILE_ERROR;
//  return true;
}
//*******************************************************************
