//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  odirent.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//  1.0     4/30/96    M.Perry     original baseline
//  2.0     8/26/97    M.Perry     32-bit conversion.
//********************************************************************
#if !defined(__odirent_h)
#include "odirent.h"
#endif

#if !defined(__odet_h)
#include "odet.h"
#endif

#if defined(GED_COMPILE)
#if !defined(__IOSTREAM_H)
  #include <iostream.h>
#endif
#if !defined(__FSTREAM_H)
  #include <fstream.h>
#endif
#if !defined(OWL_WINDOW_H)
  #include <owl\window.h>
#endif
#endif

#if !defined(__omisc_h)
#include "omisc.h"
#endif

#if defined(GED_COMPILE)
int32 TDirEntry::refcount = 0;
#endif
//*******************************************************************
const string &TDirEntry::GetDewName()
{
  return PDet->GetDewName();
}
//*******************************************************************
TDirEntry *TDirEntry::Find(const string &name)
// binary search list for name.
{
  int16 low = 0;
  int16 high = int16(NumEntries - 1);
  while (low <= high)
  {
    int16 mid = int16((low + high) / 2);
    TDirEntry *pdirentry = GetDirEntry(mid);
    int16 compareresult = CompareNames(name,pdirentry);
    if (compareresult < 0) {high = int16(mid - 1);}
    else if (compareresult > 0) {low = int16(mid + 1);}
    else {return pdirentry;}
  }
  return NULL;
}
//*******************************************************************
TDirEntry *TDirEntry::GetDirEntry(string &pathname)
{
  string first;
  string rest;
  TOgleFileName::PullFirstDir(pathname,first,rest);
  if (first == "")
  {                          
    return Find(rest);
  }
  else
  {
    TDirEntry *pdirentry = Find(first);
    if (pdirentry == NULL) {return NULL;}
    return pdirentry->GetDirEntry(rest);
  }
}
//*******************************************************************
#if defined(GED_COMPILE)
void TDirEntry::Report(ofstream &os)
{
  if (IsDir)
  {
    os << "**** Start of dir: " << Name << "\n";
    os << "Num Entries = " << NumEntries << "\n";

    for (int16 i=0;i<NumEntries;i++)
    {
      if (PPDirEntries[i] != NULL) {PPDirEntries[i]->Report(os);}
    }
    os << "**** End of dir: " << Name << "\n";
  }
  else
  {
    os << "FileName: " << Name << "\n";
  }
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
int QSCompareNames(const void *a,const void *b)
{
  if (a == NULL || b == NULL) {return 1;}
  TDirEntry **ppde1 = (TDirEntry **)a;
  TDirEntry **ppde2 = (TDirEntry **)b;
  TDirEntry *pde1 = (TDirEntry *)(*ppde1);
  TDirEntry *pde2 = (TDirEntry *)(*ppde2);
  return strcmp(pde1->Name,pde2->Name);
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
void TDirEntry::Sort()
{
  if (PPDirEntries == NULL || NumEntries == 0) {return;}

  qsort((void *)PPDirEntries,NumEntries,sizeof(void *),QSCompareNames);
  for (int16 i=0;i<NumEntries;i++)
  {
    if (PPDirEntries[i] != NULL) {PPDirEntries[i]->Sort();}
  }
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
bool TDirEntry::CreateDewFile()
{
  if (IsDir)
  {
    for (int16 i=0;i<NumEntries;i++)
    {
      if (PPDirEntries[i] != NULL)
      {
        if (!PPDirEntries[i]->CreateDewFile()) {return false;}
      }
    }
  }                                
  else
  {
    Offset = PDet->PDewFile->GetPosition();
    string pathname = PDet->GetRoot();
    pathname += (const char *)Dir;
    pathname += (const char *)Name;
    TOgleFile sourcefile(pathname,OFA_READ,OFS_READ,OFC_OPEN_EXISTING); //v2.3
    if (!sourcefile.GetFileHandleValid()) {return false;}

    void huge *p = GlobalAllocPtr(GMEM_MOVEABLE,Size);
    if (p == 0)
    {
      char buffer[33];
      ltoa(Size,buffer,10);
      string s = "Name: ";
      s += pathname;
      s += "\nmemory alloc failure,size: ";
      s += buffer;
      PDet->PWin->MessageBox(s.c_str(),"Create Dew File",MB_OK);
      return false;
    }
    if (!sourcefile.Read(p,Size))
    {
      PDet->PWin->MessageBox("read failure","Create Dew File",MB_OK);
      GlobalFreePtr(p);
      return false;
    }
    if (!PDet->PDewFile->Write(p,Size))
    {
      PDet->PWin->MessageBox("write failure","Create Dew File",MB_OK);
      GlobalFreePtr(p);
      return false;
    }
    GlobalFreePtr(p);
  }
  return true;
}
#endif
//*******************************************************************
bool TDirEntry::Write()
{
  string name = Name;
  if (!PDet->Writestring(name)) {return false;}
  if (!PDet->Writeint32(&Size)) {return false;}
  if (!PDet->Writeint32(&Offset)) {return false;}
  if (!PDet->Writebool(&IsDir)) {return false;}
  if (!PDet->Writeint16(&NumEntries)) {return false;}  
  for (int16 i=0;i<NumEntries;i++)
  {
    if (!PPDirEntries[i]->Write()) {return false;}
  }
  return true;
}
//*******************************************************************
bool TDirEntry::Read()
{
  string name;
  if (!PDet->Readstring(name)) {return false;}
  Name = name;
  if (!PDet->Readint32(&Size)) {return false;}
  if (!PDet->Readint32(&Offset)) {return false;}
  if (!PDet->Readbool(&IsDir)) {return false;}
  int16 num;
  if (!PDet->Readint16(&num)) {return false;}
  for (int16 i=0;i<num;i++)
  {
    TDirEntry *pde = new TDirEntry(PDet);
    if (!pde->Read())
    {
      delete pde;
      return false;
    }
    AddDirEntry(pde);
  }
  return true;
}
//*******************************************************************
#if defined(GED_COMPILE)
string &TDirEntry::GetRootDriveDir()
{
  return PDet->GetRoot();
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
TDirEntry::TDirEntry(TDet *pdet,const string &dir,const string &filename,int32 size)
// ctor for file
{
  refcount++;
  NumEntries = 0;
  PPDirEntries = NULL;
  Dir = dir;
  PDet = pdet;
  IsDir = false;
  Name = filename;
  Size = size;
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
TDirEntry::TDirEntry(TDet *pdet,const string &dir,const string &dirname)
// ctor for dir
{
  refcount++;
  PPDirEntries = NULL;
  NumEntries = 0;
  Dir = dir;
  IsDir = true;
  PDet = pdet;
  Name = dirname;
  Size = 0;
  CreateDir();
}
#endif
//*******************************************************************
TDirEntry::TDirEntry(TDet *pdet)
{
  NumEntries = 0;
  Size = 0;
  PDet = pdet;

  #if defined(GED_COMPILE)
  refcount++;
  #endif
}
//*******************************************************************
#if defined(GED_COMPILE)
TDirEntry *TDirEntry::CreateDirEntry(const string &dir,const string &filename,int32 size)
{
  return (new TDirEntry(PDet,dir,filename,size));
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
TDirEntry *TDirEntry::CreateDirEntry(const string &dir,const string &dirname)
{
  return (new TDirEntry(PDet,dir,dirname));
}
#endif
//*******************************************************************
#if defined(GED_COMPILE)
void TDirEntry::CreateDir()
// Basically recursive with ctor's wrapped around calls.
{
  struct ffblk ffblkdata;
  string fulldir = Dir;
  fulldir += (const char *)Name;
  string workstr = GetRootDriveDir();
  workstr += fulldir + "*.*";
  int result = findfirst(workstr.c_str(),&ffblkdata,FA_DIREC);
  while (result == 0)
  {
    string namefound = ffblkdata.ff_name;
    namefound.to_lower();
    int attrib = ffblkdata.ff_attrib;
    bool isdir = ((attrib & FA_DIREC) == FA_DIREC);
    if (isdir)
    {
      string newdirname = namefound + "\\";
      if (PDet->IsDirOk(fulldir,newdirname))
      {
        AddDirEntry(CreateDirEntry(fulldir,newdirname));
      }
    }
    else
    {
      if (PDet->IsExtOk(fulldir,namefound) && PDet->IsNameOk(fulldir,namefound))
      {
        int32 filesize = ffblkdata.ff_fsize;
        AddDirEntry(CreateDirEntry(fulldir,namefound,filesize));
      }
    }
    result = findnext(&ffblkdata);
  }
}
#endif
//*******************************************************************
TDirEntry::~TDirEntry()
{
  #if defined(GED_COMPILE)
  refcount--;
  #endif
  while (RemoveDirEntry(0)) {}
}
//*******************************************************************
bool TDirEntry::AddDirEntry(TDirEntry *pdirentry)
{
  if ((NumEntries == 0) || (PPDirEntries == NULL))
  {
    PPDirEntries = new TDirEntry *[1];
    NumEntries = 1;
    PPDirEntries[0] = pdirentry;
    return true;
  }
  else
  {
    TDirEntry **Holder = PPDirEntries;
    int oldnumentries = NumEntries;
    NumEntries += int16(1);
    PPDirEntries = new TDirEntry *[NumEntries];

    for (int16 i = 0;i < oldnumentries;i++)
    {
      PPDirEntries[i] = Holder[i];
    }
    PPDirEntries[NumEntries - 1] = pdirentry;
    delete [] Holder;
    return true;
  }
}
//*******************************************************************
bool TDirEntry::RemoveDirEntry(int16 direntryindex)
{
  if ((direntryindex >= NumEntries) || (direntryindex < 0)) {return false;}
  if (PPDirEntries == NULL) {return false;}

  if (NumEntries == 1)
  {
    delete PPDirEntries[0];
    delete [] PPDirEntries;
    PPDirEntries = NULL;
    NumEntries = 0;
    return true;
  }
  else
  {
    TDirEntry **Holder = PPDirEntries;
    int16 oldnumentries = NumEntries;
    NumEntries -= int16(1);
    PPDirEntries = new TDirEntry *[NumEntries];
    int16 CurrentIndex = 0;
    for (int16 i=0;i<oldnumentries;i++)
    {
      if (i != direntryindex)
      {
        PPDirEntries[CurrentIndex] = Holder[i];
        CurrentIndex++;
      }
    }
    delete Holder[direntryindex];
    delete [] Holder;
    return true;
  }
}
//*******************************************************************

