//********************************************************************
//                    Confidential Material.
//
//  Copyright (c) 1995 CARDOZA ENTERTAINMENT. All Rights Reserved.
//********************************************************************
//
//       File Name:  omode.cpp
//    Project Name:  Ogle
// Original Author:  Michael R. Perry
//
// (Revision History)
// Version   Date      Author       Changes and Notes
// ------- --------   ---------  ------------------------------
//   1.0    7/19/95    M.Perry    original baseline
//   2.0    8/24/97    M.Perry    32-bit conversion.
//   2.1    12/6/97    M.Perry    filename mod's.
//********************************************************************
#if !defined(__omode_h)
#include "omode.h"
#endif

#if !defined(__omodewin_h)
#include "omodewin.h"
#endif

#if !defined(__ogobjlst_h)
#include "ogobjlst.h"
#endif

#if !defined(__obmpmono_h)
#include "obmpmono.h"
#endif

#if !defined(__oscrnmgr_h)
#include "oscrnmgr.h"
#endif

#if !defined(__ogamefw_h)
#include "ogamefw.h"
#endif
//*******************************************************************
void TOgleMode::SetupGObjs()
{
  if (GetModeWin() != NULL)
  {
    GetModeWin()->SetupGObjs();
  }
}
//*******************************************************************
bool TOgleMode::NotifyMainPalChanged()
{
  if (GetScreenMgr() == NULL) {return false;}  
  DestroyPal();
  CreatePal();
  if (PPal == NULL) {return false;}
  GetScreenMgr()->ChangePal(*PPal);
  //??? should get result from screenmgr call
  return true;
}
//*******************************************************************
void TOgleMode::NotifyNumGObjs(int16 numgobjs)
{
  if (numgobjs > 0 && PModeWin != NULL)
  {
    PModeWin->CreatePlayerList(int16(numgobjs - 1));
  }
}
//*******************************************************************
void TOgleMode::SetMajVer()
{
  MajVer = 2;
}
//*******************************************************************
void TOgleMode::SetMinVer()
{
  MinVer = 1;
}
//*******************************************************************
string TOgleMode::AddExt()
{
  return string(".m");
}
//*******************************************************************
const char *TOgleMode::GetIDChars() const
{
  return "MODE";
}
//*******************************************************************
bool TOgleMode::ChangeBG(TOgleFileName ofn)
{
  DestroyBG();
  BGFileName = ofn.GetStr();

  if (!CreateBG()) {return false;}

  if (GetScreenMgr() != NULL)
  {
    return GetScreenMgr()->ChangeBG(PBG);
  }
  return false;
}
//*******************************************************************
TOgleMode::TOgleMode(TOgleModeWindow *pmw,string ModeName,
                     bool usebmpmono,int16 monocolorndx)
:TOgleBase(pmw),
 GameId(0),
 PGObjList(NULL),
 PBG(NULL),
 PPal(NULL),
 PScreenMgr(NULL),
 UseBmpMono(usebmpmono),
 MonoColorNDX(monocolorndx)
{
  PModeWin->SetMode(this);  // hack.
  FileMode = TFileMode(ModeName,TFileMode::fm_None,TFileMode::fm_Immed);
  Init(FileMode,true,false);
  // read creates all necessary sub-objects.
}
//*******************************************************************
TOgleMode::TOgleMode(TOgleModeWindow *pmw,bool usebmpmono,int16 monocolorndx)
:TOgleBase(pmw),
 GameId(0),
 PGObjList(NULL),
 PBG(NULL),
 PPal(NULL),
 PScreenMgr(NULL),
 UseBmpMono(usebmpmono),
 MonoColorNDX(monocolorndx)
{
  PModeWin->SetMode(this); // hack.
  BGFileName = "mbg";
  CreatePal();
  CreateBG();
  CreateScreenMgr();
  CreateGObjList();
}
//*******************************************************************
TOgleMode::~TOgleMode()
{
  DestroyGObjList();
  DestroyScreenMgr();
  DestroyBG();
  DestroyPal();
}
//*******************************************************************
void TOgleMode::CreateGObjList()
{
  if (PGObjList != NULL) {return;} // if called by old readlocal's.

  if (GObjListName == "")
  {
    PGObjList = new TGObjList(PModeWin,this);
    PGObjList->FileMode = FileMode;
    PGObjList->FileMode.Name.SetExt(string(".gol"));
    GObjListName = PGObjList->FileMode.Name.GetStr();
  }
  else
  {
    TFileMode fm(GObjListName,TFileMode::fm_None,TFileMode::fm_Immed);
    PGObjList = new TGObjList(PModeWin,this,fm);
  }
}
//*******************************************************************
void TOgleMode::DestroyGObjList()
{
  delete PGObjList;
  PGObjList = NULL;
}
//*******************************************************************
void TOgleMode::SetGameId(string gamename)
{
  if (gamename == "slots") {SetGameId(IDG_SLOTS);}
  else if (gamename == "keno") {SetGameId(IDG_KENO);}
  else if (gamename == "craps") {SetGameId(IDG_CRAPS);}
  else if (gamename == "roulette") {SetGameId(IDG_ROULETTE);}
  else if (gamename == "vidpoker") {SetGameId(IDG_VIDPOKER);}
  else if (gamename == "bj") {SetGameId(IDG_BJ);}
  else if (gamename == "poker") {SetGameId(IDG_POKER);}
  else {SetGameId(0);}
}
//*******************************************************************
int16 TOgleMode::GetStringTableID()
{
  switch (GameId)
  {
    // Note: I didn't want to include the strings into ogle
    case IDG_SLOTS:    return IDS_SLOTSQ0;
    case IDG_KENO:     return IDS_KENOQ0;
    case IDG_CRAPS:    return IDS_CRAPSQ0;
    case IDG_ROULETTE: return IDS_ROULETTEQ0;
    case IDG_VIDPOKER: return IDS_VIDPOKERQ0;
    case IDG_BJ:       return IDS_BJQ0;
    case IDG_POKER:    return IDS_POKERQ0;
    default:           return 0;
  }
}
//*******************************************************************
string TOgleMode::GetGameStr(int16 NewGameId)
{
  switch (NewGameId)
  {
    case IDG_SLOTS:    return string("slots");
    case IDG_KENO:     return string("keno");
    case IDG_CRAPS:    return string("craps");
    case IDG_ROULETTE: return string("roulette");
    case IDG_VIDPOKER: return string("vidpoker");
    case IDG_BJ:       return string("bj");
    case IDG_POKER:    return string("poker");
    default:           return string("");
  }
}
//*******************************************************************
#if defined(SUPPORT_OLD_VER)
bool TOgleMode::ReadLocal1_0()
{
  if (!Readint16(&GameId)) {return false;}

  if (!ReadFNameString(BGFileName)) {return false;}
  // don't need to add dir to it because it was in logroot.
  BGFileName += ".bmp";

  CreatePal();
  CreateBG();
  CreateScreenMgr();
  CreateGObjList();

  uint16 NumGObjs;
  if (!Readuint16(&NumGObjs)) {return false;}

  PGObjList->CreateNameArray(NumGObjs);

  for (int16 i = 0;i<NumGObjs;i++)
  {
    string s;
    if (!ReadFNameString(s)) {return false;}

    // start - prepend dir to name.
    TOgleFileName ofn;
    ofn.SetFName(s);
    ofn.SetDirPath(s);
    ofn.SetExt(string(".go"));
    s = ofn.GetStr();
    // end

    PGObjList->PNames[i] = s;
  }
//  SortByName();
  PGObjList->CreateGObjs();
//  PGObjList->SetIDs();
  return true;
}
#endif
//*******************************************************************
#if defined(SUPPORT_OLD_VER)
bool TOgleMode::ReadLocal2_0()
{
  if (!Readint16(&GameId)) {return false;}

  if (!ReadFileName(BGFileName)) {return false;}
  CreatePal();
  CreateBG();
  CreateScreenMgr();
  CreateGObjList();

  uint16 numgobjs;
  if (!Readuint16(&numgobjs)) {return false;}

  PGObjList->CreateNameArray(numgobjs);

  for (int16 i=0;i<numgobjs;i++)
  {
    string s;
    if (!ReadFileName(s)) {return false;}
    PGObjList->PNames[i] = s;
  }
//  SortByName();
  PGObjList->CreateGObjs();
  return true;
}
#endif
//*******************************************************************
bool TOgleMode::ReadLocal2_1()
{
  if (!Readint16(&GameId)) {return false;}
  if (!ReadFileName(BGFileName)) {return false;}
  CreatePal();
  CreateBG();
  CreateScreenMgr();
  if (!ReadFileName(GObjListName)) {return false;}
  CreateGObjList();
  return true;
}
//*******************************************************************
bool TOgleMode::ReadLocal()
{
  switch (MajVer)
  {
    case 1:return ReadLocal1_0();
    case 2:
           switch (MinVer)
           {
             case 0:return ReadLocal2_0();
             case 1:return ReadLocal2_1();
             default:return false;
           }
    default:return false;
  }
}
//*******************************************************************
bool TOgleMode::WriteLocal()
{
//  if (!Writeint16(&GameId)) {return false;}
//
//  uint16 NumGObjs = GObjs.GetRecCount();
//  if (!WriteFNameString(BGFileName)) {return false;}
//
//  if (!Writeuint16(&NumGObjs)) {return false;}
//
//  for (int16 i = 0;i<NumGObjs;i++)
//  {
//    if (!WriteFNameString(GetGObjName(i))) {return false;}
//  }
//  return true;

  if (!Writeint16(&GameId)) {return false;}

//  uint16 numgobjs = GObjs.GetRecCount();
  if (!WriteFileName(BGFileName)) {return false;}

//  if (!Writeuint16(&numgobjs)) {return false;}

//  string gobjname;
//  for (int16 i=0;i<numgobjs;i++)
//  {
//    gobjname = GetGObjName(i);
//    if (!WriteFileName(gobjname)) {return false;}
//  }
  if (!WriteFileName(GObjListName)) {return false;}

//  return true;

  return GetGObjList()->Write();
}
//*******************************************************************
bool TOgleMode::Save()
{
  return Write();
//  GetGObjList()->Save();
}
//*******************************************************************
//TCompareResult CompareGObjStrings(void *firstpstr,void *secpstr)
//{
//  if ((firstpstr == NULL) || (secpstr == NULL)) {return EQUAL_TO;}
//  string *pstr1 = (string *)firstpstr;
//  string *pstr2 = (string *)secpstr;
//  if (*pstr1 > *pstr2) {return GREATER_THAN;}
//  else if (*pstr1 < *pstr2) {return LESS_THAN;}
//  else {return EQUAL_TO;}
//}
//*******************************************************************
//void TOgleMode::SortByName()
//{
//  TCompareFunct OldCompareFunct = GObjs.SetCompareFunct(CompareGObjStrings);
//  GObjs.SliqSort();
//  GObjs.SetCompareFunct(OldCompareFunct);
//}
//*******************************************************************
bool TOgleMode::CreateBG()
{
  if (PBG != NULL) {return false;}
  if (UseBmpMono || BGFileName == TAppString::strEmpty)
  {
    int16 x,y,w,h;
    if (PModeWin == NULL)
    {
      x = 0;
      y = 0;
      w = 640;
      h = 480;
    }
    else
    {
      x = int16(PModeWin->Attr.X);
      y = int16(PModeWin->Attr.Y);
      w = int16(PModeWin->Attr.W);
      h = int16(PModeWin->Attr.H);
    }
    TOgleRect rect(x,y,w,h);
    PBG = new TBmpMono(PModeWin,rect,MonoColorNDX);
  }
  else
  {
    PBG = new TBmp(PModeWin,TFileMode(BGFileName,TFileMode::fm_None,TFileMode::fm_Immed));
    // MRP - 06/27/99 - v2.3 - start
    if (PModeWin != NULL &&
        GetFW() != NULL &&
        GetFW()->GetFadeIn() &&
        PModeWin->AllowFadeIn)
    {
      GetFW()->SetFadeIn(false);
      PModeWin->HideGauge = true;
      PModeWin->FadeIn(PBG);
    }
    // end
    if (PBG->IsFileNotFound()) {return false;}
  }
  return true;
}
//*******************************************************************
void TOgleMode::DestroyBG()
{
  delete PBG;
  PBG = NULL;
}
//*******************************************************************
void TOgleMode::CreatePal()
{
  if (PPal != NULL)
  {
    DestroyPal();
  }
  TFileMode fm = GetOgleApp()->GetPal()->FileMode;
  PPal = new TPal(PModeWin,fm);
}
//*******************************************************************
void TOgleMode::DestroyPal()
{
  delete PPal;
  PPal = NULL;
}
//*******************************************************************
void TOgleMode::CreateScreenMgr()
{
  PScreenMgr = new TOgleScreenMgr(PModeWin,PBG,PPal);
}
//*******************************************************************
void TOgleMode::DestroyScreenMgr()
{
  delete PScreenMgr;
  PScreenMgr = NULL;
}
//*******************************************************************
TGameObject *TOgleMode::CreateGObj(TFileMode &fm,int16 id,void *psource)
{
  if (PModeWin == NULL) {return NULL;}
  return PModeWin->CreateGObj(fm,id,psource);
}
//*******************************************************************
bool TOgleMode::DestroyGObj(TGameObject *pgobj)
{
  if (PModeWin == NULL) {return false;}
  return PModeWin->DestroyGObj(pgobj);
}
//*******************************************************************
